@@ -12,14 +12,23 @@
 #include "ash/system/toast/toast_manager.h"
 #include "ash/touch/touch_devices_controller.h"
 #include "ash/wallpaper/wallpaper_controller.h"
+#include "ash/wm/focus_rules.h"
 #include "ash/wm/tablet_mode/tablet_mode_controller.h"
 #include "ash/wm/widget_finder.h"
-#include "ash/wm/window_properties.h"
+#include "ash/wm/window_state.h"
 #include "ash/wm/window_util.h"
+#include "ash/ws/window_lookup.h"
+#include "ash/ws/window_service_owner.h"
 #include "base/command_line.h"
 #include "base/metrics/user_metrics.h"
 #include "base/metrics/user_metrics_action.h"
 #include "base/strings/utf_string_conversions.h"
+#include "services/service_manager/public/cpp/connector.h"
+#include "services/ws/window_service.h"
+#include "ui/accessibility/ax_tree_id.h"
+#include "ui/accessibility/platform/aura_window_properties.h"
+#include "ui/aura/mus/window_tree_client.h"
+#include "ui/base/ui_base_features.h"
 #include "ui/compositor/debug_utils.h"
 #include "ui/compositor/layer.h"
 #include "ui/display/manager/display_manager.h"
@@ -27,7 +36,9 @@
 #include "ui/gfx/image/image_skia.h"
 #include "ui/gfx/image/image_skia_rep.h"
 #include "ui/views/debug_utils.h"
+#include "ui/views/mus/mus_client.h"
 #include "ui/views/widget/widget.h"
+#include "ui/wm/core/window_properties.h"
 
 namespace ash {
 namespace debug {
@@ -41,6 +52,16 @@ void HandlePrintLayerHierarchy() {
           layer,
           RootWindowController::ForWindow(root)->GetLastMouseLocationInRoot());
   }
+
+  if (!features::IsSingleProcessMash())
+    return;
+
+  for (aura::Window* mus_root :
+       views::MusClient::Get()->window_tree_client()->GetRoots()) {
+    ui::Layer* layer = mus_root->layer();
+    if (layer)
+      ui::PrintLayerHierarchy(layer, gfx::Point());
+  }
 }
 
 void HandlePrintViewHierarchy() {
@@ -53,7 +74,9 @@ void HandlePrintViewHierarchy() {
   views::PrintViewHierarchy(widget->GetRootView());
 }
 
-void PrintWindowHierarchy(const aura::Window* active_window,
+void PrintWindowHierarchy(ws::WindowService* window_service,
+                          const aura::Window* active_window,
+                          const aura::Window* focused_window,
                           aura::Window* window,
                           int indent,
                           std::ostringstream* out) {
@@ -61,27 +84,63 @@ void PrintWindowHierarchy(const aura::Window* active_window,
   std::string name(window->GetName());
   if (name.empty())
     name = "\"\"";
-  *out << indent_str << name << " (" << window << ")"
-       << " type=" << window->type()
-       << ((window == active_window) ? " [active] " : " ")
-       << (window->IsVisible() ? " visible " : " ")
-       << window->bounds().ToString()
-       << (window->GetProperty(kSnapChildrenToPixelBoundary) ? " [snapped] "
-                                                             : "")
-       << ", subpixel offset="
-       << window->layer()->subpixel_position_offset().ToString() << '\n';
+  const gfx::Vector2dF& subpixel_position_offset =
+      window->layer()->subpixel_position_offset();
+  *out << indent_str;
+  if (window_service && ws::WindowService::IsProxyWindow(window))
+    *out << " [proxy] id=" << window_service->GetIdForDebugging(window) << " ";
+  *out << name << " (" << window << ")"
+       << " type=" << window->type();
+  if (ash::IsToplevelWindow(window))
+    *out << " " << wm::GetWindowState(window)->GetStateType();
+  *out << ((window == active_window) ? " [active]" : "")
+       << ((window == focused_window) ? " [focused]" : "")
+       << (window->IsVisible() ? " visible" : "") << " "
+       << window->bounds().ToString();
+  if (window->GetProperty(::wm::kSnapChildrenToPixelBoundary))
+    *out << " [snapped]";
+  if (!subpixel_position_offset.IsZero())
+    *out << " subpixel offset=" + subpixel_position_offset.ToString();
+  if (features::IsSingleProcessMash()) {
+    aura::Window* proxy_window =
+        window_lookup::GetProxyWindowForClientWindow(window);
+    if (proxy_window)
+      *out << " id=" << window_service->GetIdForDebugging(proxy_window);
+  }
+  std::string* tree_id = window->GetProperty(ui::kChildAXTreeID);
+  if (tree_id)
+    *out << " ax_tree_id=" << *tree_id;
+  *out << '\n';
 
-  for (aura::Window* child : window->children())
-    PrintWindowHierarchy(active_window, child, indent + 3, out);
+  for (aura::Window* child : window->children()) {
+    PrintWindowHierarchy(window_service, active_window, focused_window, child,
+                         indent + 3, out);
+  }
 }
 
 void HandlePrintWindowHierarchy() {
   aura::Window* active_window = wm::GetActiveWindow();
+  aura::Window* focused_window = wm::GetFocusedWindow();
   aura::Window::Windows roots = Shell::Get()->GetAllRootWindows();
+  ws::WindowService* window_service =
+      Shell::Get()->window_service_owner()->window_service();
   for (size_t i = 0; i < roots.size(); ++i) {
     std::ostringstream out;
     out << "RootWindow " << i << ":\n";
-    PrintWindowHierarchy(active_window, roots[i], 0, &out);
+    PrintWindowHierarchy(window_service, active_window, focused_window,
+                         roots[i], 0, &out);
+    // Error so logs can be collected from end-users.
+    LOG(ERROR) << out.str();
+  }
+
+  if (!features::IsSingleProcessMash())
+    return;
+
+  for (aura::Window* mus_root :
+       views::MusClient::Get()->window_tree_client()->GetRoots()) {
+    std::ostringstream out;
+    out << "WindowService Client RootWindow\n";
+    PrintWindowHierarchy(nullptr, nullptr, nullptr, mus_root, 0, &out);
     // Error so logs can be collected from end-users.
     LOG(ERROR) << out.str();
   }
@@ -109,26 +168,28 @@ void HandleToggleWallpaperMode() {
   static int index = 0;
   WallpaperController* wallpaper_controller =
       Shell::Get()->wallpaper_controller();
-  wallpaper::WallpaperInfo info("", wallpaper::WALLPAPER_LAYOUT_STRETCH,
-                                wallpaper::DEFAULT,
-                                base::Time::Now().LocalMidnight());
+  WallpaperInfo info("", WALLPAPER_LAYOUT_STRETCH, DEFAULT,
+                     base::Time::Now().LocalMidnight());
   switch (++index % 4) {
     case 0:
       wallpaper_controller->ShowDefaultWallpaperForTesting();
       break;
     case 1:
-      wallpaper_controller->SetWallpaperImage(
-          CreateWallpaperImage(SK_ColorRED, SK_ColorBLUE), info);
+      wallpaper_controller->ShowWallpaperImage(
+          CreateWallpaperImage(SK_ColorRED, SK_ColorBLUE), info,
+          /*preview_mode=*/false, /*always_on_top=*/false);
       break;
     case 2:
-      info.layout = wallpaper::WALLPAPER_LAYOUT_CENTER;
-      wallpaper_controller->SetWallpaperImage(
-          CreateWallpaperImage(SK_ColorBLUE, SK_ColorGREEN), info);
+      info.layout = WALLPAPER_LAYOUT_CENTER;
+      wallpaper_controller->ShowWallpaperImage(
+          CreateWallpaperImage(SK_ColorBLUE, SK_ColorGREEN), info,
+          /*preview_mode=*/false, /*always_on_top=*/false);
       break;
     case 3:
-      info.layout = wallpaper::WALLPAPER_LAYOUT_CENTER_CROPPED;
-      wallpaper_controller->SetWallpaperImage(
-          CreateWallpaperImage(SK_ColorGREEN, SK_ColorRED), info);
+      info.layout = WALLPAPER_LAYOUT_CENTER_CROPPED;
+      wallpaper_controller->ShowWallpaperImage(
+          CreateWallpaperImage(SK_ColorGREEN, SK_ColorRED), info,
+          /*preview_mode=*/false, /*always_on_top=*/false);
       break;
   }
 }
@@ -142,8 +203,8 @@ void HandleToggleTouchscreen() {
   base::RecordAction(base::UserMetricsAction("Accel_Toggle_Touchscreen"));
   TouchDevicesController* controller = Shell::Get()->touch_devices_controller();
   controller->SetTouchscreenEnabled(
-      !controller->GetTouchscreenEnabled(TouchscreenEnabledSource::USER_PREF),
-      TouchscreenEnabledSource::USER_PREF);
+      !controller->GetTouchscreenEnabled(TouchDeviceEnabledSource::USER_PREF),
+      TouchDeviceEnabledSource::USER_PREF);
 }
 
 void HandleToggleTabletMode() {
@@ -153,7 +214,7 @@ void HandleToggleTabletMode() {
 }
 
 void HandleTriggerCrash() {
-  CHECK(false) << "Intentional crash via debug accelerator.";
+  LOG(FATAL) << "Intentional crash via debug accelerator.";
 }
 
 }  // namespace
