@@ -9,7 +9,7 @@
 
 #include "ash/app_list/model/app_list_item.h"
 #include "ash/app_list/model/app_list_item_list.h"
-#include "ash/public/cpp/app_list/app_list_constants.h"
+#include "ash/public/cpp/app_list/app_list_config.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "ui/base/l10n/l10n_util.h"
@@ -26,14 +26,14 @@ namespace app_list {
 
 namespace {
 
-constexpr int kItemIconDimension = 16;
-constexpr float kFolderBubbleRadius = 23;
-constexpr float kFolderBubbleOffsetY = 1;
+// The margin of item icon in folder icon.
+constexpr int kItemIconMargin = 2;
 
-// Gets the size of a small app icon inside the folder icon.
-gfx::Size ItemIconSize() {
-  return gfx::Size(kItemIconDimension, kItemIconDimension);
-}
+// The shadow blur of icon.
+constexpr int kIconShadowBlur = 5;
+
+// The shadow color of icon.
+constexpr SkColor kIconShadowColor = SkColorSetA(SK_ColorBLACK, 31);
 
 // Generates the folder icon with the top 4 child item icons laid in 2x2 tile.
 class FolderImageSource : public gfx::CanvasImageSource {
@@ -74,34 +74,48 @@ void FolderImageSource::DrawIcon(gfx::Canvas* canvas,
   if (icon.isNull())
     return;
 
-  gfx::ImageSkia resized(gfx::ImageSkiaOperations::CreateResizedImage(
+  const gfx::ImageSkia resized(gfx::ImageSkiaOperations::CreateResizedImage(
       icon, skia::ImageOperations::RESIZE_BEST, icon_size));
-  canvas->DrawImageInt(resized, 0, 0, resized.width(), resized.height(), x, y,
-                       resized.width(), resized.height(), true);
+
+  // Draw a shadowed icon on the specified location.
+  const gfx::ImageSkia shadowed(
+      gfx::ImageSkiaOperations::CreateImageWithDropShadow(
+          resized, gfx::ShadowValues(
+                       1, gfx::ShadowValue(gfx::Vector2d(), kIconShadowBlur,
+                                           kIconShadowColor))));
+  const gfx::Size shadow_size = shadowed.size();
+  x -= (shadow_size.width() - icon_size.width()) / 2;
+  y -= (shadow_size.height() - icon_size.height()) / 2;
+  canvas->DrawImageInt(shadowed, 0, 0, shadow_size.width(),
+                       shadow_size.height(), x, y, shadow_size.width(),
+                       shadow_size.height(), true);
 }
 
 void FolderImageSource::Draw(gfx::Canvas* canvas) {
-  cc::PaintFlags flags;
-  // Draw circle for folder bubble.
   gfx::PointF bubble_center(size().width() / 2, size().height() / 2);
-  bubble_center.Offset(0, -kFolderBubbleOffsetY);
+  bubble_center.Offset(0, -AppListConfig::instance().folder_bubble_y_offset());
+
+  // Draw circle for folder bubble.
+  cc::PaintFlags flags;
   flags.setStyle(cc::PaintFlags::kFill_Style);
   flags.setAntiAlias(true);
-  flags.setColor(FolderImage::kFolderBubbleColor);
-  canvas->DrawCircle(bubble_center, kFolderBubbleRadius, flags);
+  flags.setColor(AppListConfig::instance().folder_bubble_color());
+  canvas->DrawCircle(bubble_center,
+                     AppListConfig::instance().folder_bubble_radius(), flags);
 
   if (icons_.size() == 0)
     return;
 
   // Draw top items' icons.
-  const gfx::Size item_icon_size(ItemIconSize());
+  const size_t num_items =
+      std::min(FolderImage::kNumFolderTopItems, icons_.size());
   std::vector<gfx::Rect> top_icon_bounds =
-      FolderImage::GetTopIconsBounds(gfx::Rect(size()));
+      FolderImage::GetTopIconsBounds(gfx::Rect(size()), num_items);
 
-  for (size_t i = 0; i < FolderImage::kNumFolderTopItems && i < icons_.size();
-       ++i) {
-    DrawIcon(canvas, icons_[i], item_icon_size, top_icon_bounds[i].x(),
-             top_icon_bounds[i].y());
+  for (size_t i = 0; i < num_items; ++i) {
+    DrawIcon(canvas, icons_[i],
+             AppListConfig::instance().item_icon_in_folder_icon_size(),
+             top_icon_bounds[i].x(), top_icon_bounds[i].y());
   }
 }
 
@@ -109,8 +123,6 @@ void FolderImageSource::Draw(gfx::Canvas* canvas) {
 
 // static
 const size_t FolderImage::kNumFolderTopItems = 4;
-const SkColor FolderImage::kFolderBubbleColor =
-    SkColorSetARGB(0x1F, 0xFF, 0xFF, 0xFF);
 
 FolderImage::FolderImage(AppListItemList* item_list) : item_list_(item_list) {
   item_list_->AddObserver(this);
@@ -127,44 +139,89 @@ void FolderImage::UpdateIcon() {
     item->RemoveObserver(this);
   top_items_.clear();
 
-  for (size_t i = 0; i < kNumFolderTopItems && i < item_list_->item_count();
+  for (size_t i = 0;
+       i < item_list_->item_count() && top_items_.size() < kNumFolderTopItems;
        ++i) {
     AppListItem* item = item_list_->item_at(i);
+    // If this item is currently being dragged, pretend it has already left our
+    // folder
+    if (item == dragged_item_)
+      continue;
     item->AddObserver(this);
     top_items_.push_back(item);
   }
   RedrawIconAndNotify();
 }
 
+void FolderImage::UpdateDraggedItem(const AppListItem* dragged_item) {
+  DCHECK(dragged_item_ != dragged_item);
+  dragged_item_ = dragged_item;
+  UpdateIcon();
+}
+
 // static
 std::vector<gfx::Rect> FolderImage::GetTopIconsBounds(
-    const gfx::Rect& folder_icon_bounds) {
-  const int delta_to_center = 1;
+    const gfx::Rect& folder_icon_bounds,
+    size_t num_items) {
+  DCHECK_LE(num_items, kNumFolderTopItems);
+  const int item_icon_dimension =
+      AppListConfig::instance().item_icon_in_folder_icon_dimension();
   gfx::Point icon_center = folder_icon_bounds.CenterPoint();
   std::vector<gfx::Rect> top_icon_bounds;
 
-  // Get the top left icon bounds.
-  int left_x = icon_center.x() - kItemIconDimension - delta_to_center;
-  int top_y = icon_center.y() - kItemIconDimension - delta_to_center;
-  gfx::Rect top_left(left_x, top_y, kItemIconDimension, kItemIconDimension);
-  top_icon_bounds.push_back(top_left);
+  const gfx::Rect center_rect(icon_center.x() - item_icon_dimension / 2,
+                              icon_center.y() - item_icon_dimension / 2,
+                              item_icon_dimension, item_icon_dimension);
+  const int origin_offset = (AppListConfig::instance().folder_icon_dimension() -
+                             item_icon_dimension) /
+                                2 -
+                            kItemIconMargin;
+  if (num_items == 1) {
+    // Center icon bounds.
+    top_icon_bounds.emplace_back(center_rect);
+    return top_icon_bounds;
+  }
 
-  // Get the top right icon bounds.
-  int right_x = icon_center.x() + delta_to_center;
-  gfx::Rect top_right(right_x, top_y, kItemIconDimension, kItemIconDimension);
-  top_icon_bounds.push_back(top_right);
+  if (num_items == 2) {
+    // Left icon bounds.
+    gfx::Rect left_rect = center_rect;
+    left_rect.Offset(-origin_offset, 0);
+    top_icon_bounds.emplace_back(left_rect);
 
-  // Get the bottom left icon bounds.
-  int bottom_y = icon_center.y() + delta_to_center;
-  gfx::Rect bottom_left(left_x, bottom_y, kItemIconDimension,
-                        kItemIconDimension);
-  top_icon_bounds.push_back(bottom_left);
+    // Right icon bounds.
+    gfx::Rect right_rect = center_rect;
+    right_rect.Offset(origin_offset, 0);
+    top_icon_bounds.emplace_back(right_rect);
+    return top_icon_bounds;
+  }
 
-  // Get the bottom right icon bounds.
-  gfx::Rect bottom_right(right_x, bottom_y, kItemIconDimension,
-                         kItemIconDimension);
-  top_icon_bounds.push_back(bottom_right);
+  // Top left icon bounds.
+  gfx::Rect top_left_rect = center_rect;
+  top_left_rect.Offset(-origin_offset, -origin_offset);
+  top_icon_bounds.emplace_back(top_left_rect);
 
+  // Top right icon bounds.
+  gfx::Rect top_right_rect = center_rect;
+  top_right_rect.Offset(origin_offset, -origin_offset);
+  top_icon_bounds.emplace_back(top_right_rect);
+
+  if (num_items == 3) {
+    // Bottom icon bounds.
+    gfx::Rect bottom_rect = center_rect;
+    bottom_rect.Offset(0, origin_offset);
+    top_icon_bounds.emplace_back(bottom_rect);
+    return top_icon_bounds;
+  }
+
+  // Bottom left icon bounds.
+  gfx::Rect bottom_left_rect = center_rect;
+  bottom_left_rect.Offset(-origin_offset, origin_offset);
+  top_icon_bounds.emplace_back(bottom_left_rect);
+
+  // Bottom right icon bounds.
+  gfx::Rect bottom_right_rect = center_rect;
+  bottom_right_rect.Offset(origin_offset, origin_offset);
+  top_icon_bounds.emplace_back(bottom_right_rect);
   return top_icon_bounds;
 }
 
@@ -173,13 +230,15 @@ gfx::Rect FolderImage::GetTargetIconRectInFolderForItem(
     const gfx::Rect& folder_icon_bounds) const {
   for (size_t i = 0; i < top_items_.size(); ++i) {
     if (item->id() == top_items_[i]->id()) {
-      std::vector<gfx::Rect> rects = GetTopIconsBounds(folder_icon_bounds);
+      std::vector<gfx::Rect> rects =
+          GetTopIconsBounds(folder_icon_bounds, top_items_.size());
       return rects[i];
     }
   }
 
   gfx::Rect target_rect(folder_icon_bounds);
-  target_rect.ClampToCenteredSize(ItemIconSize());
+  target_rect.ClampToCenteredSize(
+      AppListConfig::instance().item_icon_in_folder_icon_size());
   return target_rect;
 }
 
@@ -220,8 +279,8 @@ void FolderImage::RedrawIconAndNotify() {
   FolderImageSource::Icons top_icons;
   for (const auto* item : top_items_)
     top_icons.push_back(item->icon());
-
-  const gfx::Size icon_size = gfx::Size(kGridIconDimension, kGridIconDimension);
+  const gfx::Size icon_size =
+      AppListConfig::instance().folder_unclipped_icon_size();
   icon_ = gfx::ImageSkia(
       std::make_unique<FolderImageSource>(top_icons, icon_size), icon_size);
 
