@@ -27,9 +27,9 @@
 #include "ash/app_list/views/folder_background_view.h"
 #include "ash/app_list/views/search_box_view.h"
 #include "ash/app_list/views/search_result_tile_item_view.h"
-#include "ash/app_list/views/suggestions_container_view.h"
+#include "ash/app_list/views/suggestion_chip_container_view.h"
 #include "ash/app_list/views/test/apps_grid_view_test_api.h"
-#include "ash/public/cpp/app_list/app_list_constants.h"
+#include "ash/public/cpp/app_list/app_list_config.h"
 #include "ash/public/cpp/app_list/app_list_features.h"
 #include "ash/public/cpp/app_list/app_list_switches.h"
 #include "base/command_line.h"
@@ -47,6 +47,7 @@
 #include "ui/events/event_utils.h"
 #include "ui/keyboard/keyboard_controller.h"
 #include "ui/views/controls/label.h"
+#include "ui/views/controls/textfield/textfield.h"
 #include "ui/views/test/views_test_base.h"
 
 namespace app_list {
@@ -68,7 +69,7 @@ class PageFlipWaiter : public PaginationModelObserver {
     DCHECK(!wait_);
     wait_ = true;
 
-    ui_run_loop_.reset(new base::RunLoop);
+    ui_run_loop_ = std::make_unique<base::RunLoop>();
     ui_run_loop_->Run();
     wait_ = false;
   }
@@ -83,7 +84,7 @@ class PageFlipWaiter : public PaginationModelObserver {
   void SelectedPageChanged(int old_selected, int new_selected) override {
     if (!selected_pages_.empty())
       selected_pages_ += ',';
-    selected_pages_ += base::IntToString(new_selected);
+    selected_pages_ += base::NumberToString(new_selected);
 
     if (wait_)
       ui_run_loop_->QuitWhenIdle();
@@ -140,50 +141,28 @@ class TestSuggestedSearchResult : public TestSearchResult {
   DISALLOW_COPY_AND_ASSIGN(TestSuggestedSearchResult);
 };
 
-struct TestParams {
-  bool is_rtl_enabled;
-  bool is_apps_grid_gap_enabled;
-};
-
-const TestParams kAppsGridViewTestParams[] = {
-    {false /* is_rtl_enabled */, false /* is_apps_grid_gap_enabled */},
-    {true, false},
-};
-
-const TestParams kAppsGridViewDragTestParams[] = {
-    {false /* is_rtl_enabled */, false /* is_apps_grid_gap_enabled */},
-    {true, false},
-    {false, true},
-    {true, true},
-};
-
-const TestParams kAppsGridGapTestParams[] = {
-    {false /* is_rtl_enabled */, true /* is_apps_grid_gap_enabled */},
-    {true, true},
-};
-
 }  // namespace
 
 class AppsGridViewTest : public views::ViewsTestBase,
-                         public testing::WithParamInterface<TestParams> {
+                         public testing::WithParamInterface<bool> {
  public:
   AppsGridViewTest() = default;
   ~AppsGridViewTest() override = default;
 
   // testing::Test overrides:
   void SetUp() override {
-    views::ViewsTestBase::SetUp();
+    AppListView::SetShortAnimationForTesting(true);
     if (testing::UnitTest::GetInstance()->current_test_info()->value_param()) {
-      is_rtl_ = GetParam().is_rtl_enabled;
+      is_rtl_ = GetParam();
       if (is_rtl_)
         base::i18n::SetICUDefaultLocale("he");
     }
+    views::ViewsTestBase::SetUp();
     gfx::NativeView parent = GetContext();
     // Ensure that parent is big enough to show the full AppListView.
     parent->SetBounds(gfx::Rect(gfx::Point(0, 0), gfx::Size(1024, 768)));
-    delegate_.reset(new AppListTestViewDelegate);
+    delegate_ = std::make_unique<AppListTestViewDelegate>();
     app_list_view_ = new AppListView(delegate_.get());
-    app_list_view_->set_short_animation_for_testing();
     AppListView::InitParams params;
     params.parent = parent;
     app_list_view_->Initialize(params);
@@ -193,22 +172,24 @@ class AppsGridViewTest : public views::ViewsTestBase,
 
     model_ = delegate_->GetTestModel();
     search_model_ = delegate_->GetSearchModel();
-    suggestions_container_ = apps_grid_view_->suggestions_container_for_test();
-    expand_arrow_view_ = apps_grid_view_->expand_arrow_view_for_test();
+    suggestions_container_ = contents_view_->GetAppsContainerView()
+                                 ->suggestion_chip_container_view_for_test();
+    expand_arrow_view_ = contents_view_->expand_arrow_view();
     for (size_t i = 0; i < kNumOfSuggestedApps; ++i) {
       search_model_->results()->Add(
           std::make_unique<TestSuggestedSearchResult>());
     }
     // Needed to update suggestions from |model_|.
-    apps_grid_view_->ResetForShowApps();
+    suggestions_container_->Update();
     app_list_view_->SetState(AppListViewState::FULLSCREEN_ALL_APPS);
     app_list_view_->Layout();
 
-    test_api_.reset(new AppsGridViewTestApi(apps_grid_view_));
+    test_api_ = std::make_unique<AppsGridViewTestApi>(apps_grid_view_);
   }
   void TearDown() override {
     app_list_view_->GetWidget()->Close();
     views::ViewsTestBase::TearDown();
+    AppListView::SetShortAnimationForTesting(false);
   }
 
  protected:
@@ -295,7 +276,7 @@ class AppsGridViewTest : public views::ViewsTestBase,
   AppListView* app_list_view_ = nullptr;    // Owned by native widget.
   AppsGridView* apps_grid_view_ = nullptr;  // Owned by |app_list_view_|.
   ContentsView* contents_view_ = nullptr;   // Owned by |app_list_view_|.
-  SuggestionsContainerView* suggestions_container_ =
+  SearchResultContainerView* suggestions_container_ =
       nullptr;                                    // Owned by |apps_grid_view_|.
   ExpandArrowView* expand_arrow_view_ = nullptr;  // Owned by |apps_grid_view_|.
   std::unique_ptr<AppListTestViewDelegate> delegate_;
@@ -315,11 +296,7 @@ class AppsGridViewTest : public views::ViewsTestBase,
   DISALLOW_COPY_AND_ASSIGN(AppsGridViewTest);
 };
 
-// Instantiate the Boolean which is used to toggle RTL in
-// the parameterized tests.
-INSTANTIATE_TEST_CASE_P(,
-                        AppsGridViewTest,
-                        testing::ValuesIn(kAppsGridViewTestParams));
+INSTANTIATE_TEST_SUITE_P(, AppsGridViewTest, testing::Bool());
 
 class TestAppsGridViewFolderDelegate : public AppsGridViewFolderDelegate {
  public:
@@ -349,13 +326,14 @@ class TestAppsGridViewFolderDelegate : public AppsGridViewFolderDelegate {
   DISALLOW_COPY_AND_ASSIGN(TestAppsGridViewFolderDelegate);
 };
 
-TEST_F(AppsGridViewTest, CreatePage) {
+TEST_P(AppsGridViewTest, CreatePage) {
   // Fully populates a page.
   const int kPages = 1;
 
   EXPECT_EQ(kNumOfSuggestedApps, suggestions_container_->num_results());
-  int kExpectedTilesOnFirstPage =
-      apps_grid_view_->cols() * (apps_grid_view_->rows_per_page() - 1);
+  // For new style launcher, each page has the same number of rows.
+  const int kExpectedTilesOnFirstPage =
+      apps_grid_view_->cols() * (apps_grid_view_->rows_per_page());
   EXPECT_EQ(kExpectedTilesOnFirstPage, GetTilesPerPage(kPages - 1));
 
   model_->PopulateApps(kPages * GetTilesPerPage(kPages - 1));
@@ -406,63 +384,6 @@ TEST_F(AppsGridViewTest, RemoveSelectedLastApp) {
   EXPECT_TRUE(apps_grid_view_->IsSelectedView(view));
 }
 
-// TODO(crbug.com/766807): Remove the test once the new focus model is stable.
-TEST_P(AppsGridViewTest, DISABLED_MoveSelectedOnAllAppsTiles) {
-  const int kItemsOnSecondPage = 3;
-  const int kAllAppsItems = GetTilesPerPage(0) + kItemsOnSecondPage;
-  const int kLastIndexOfFirstPage = GetTilesPerPage(0) - 1;
-  const int kFirstIndexOfLastRowFirstPage =
-      GetTilesPerPage(0) - apps_grid_view_->cols();
-  model_->PopulateApps(kAllAppsItems);
-
-  // Tests moving left from the first tile on the first page.
-  apps_grid_view_->SetSelectedView(GetItemViewAt(0));
-  SimulateKeyPress(is_rtl_ ? ui::VKEY_RIGHT : ui::VKEY_LEFT);
-  EXPECT_FALSE(apps_grid_view_->has_selected_view());
-  EXPECT_EQ(suggestions_container_->num_results() - 1,
-            suggestions_container_->selected_index());
-  suggestions_container_->ClearSelectedIndex();
-
-  // Tests moving left from the first tile on the second page.
-  apps_grid_view_->SetSelectedView(GetItemViewAt(GetTilesPerPage(0)));
-  SimulateKeyPress(is_rtl_ ? ui::VKEY_RIGHT : ui::VKEY_LEFT);
-  EXPECT_TRUE(
-      apps_grid_view_->IsSelectedView(GetItemViewAt(kLastIndexOfFirstPage)));
-
-  // Tests moving right from the last slot on the first page.
-  apps_grid_view_->SetSelectedView(GetItemViewAt(kLastIndexOfFirstPage));
-  SimulateKeyPress(is_rtl_ ? ui::VKEY_LEFT : ui::VKEY_RIGHT);
-  EXPECT_TRUE(
-      apps_grid_view_->IsSelectedView(GetItemViewAt(GetTilesPerPage(0))));
-
-  // Tests moving up from the first row on the first page.
-  apps_grid_view_->SetSelectedView(GetItemViewAt(1));
-  SimulateKeyPress(ui::VKEY_UP);
-  EXPECT_FALSE(apps_grid_view_->has_selected_view());
-  EXPECT_EQ(1, suggestions_container_->selected_index());
-  suggestions_container_->ClearSelectedIndex();
-
-  // Tests moving up from the first row on the second page.
-  apps_grid_view_->SetSelectedView(GetItemViewAt(kAllAppsItems - 1));
-  SimulateKeyPress(ui::VKEY_UP);
-  int expected_index =
-      GetTilesPerPage(0) - 1 - (apps_grid_view_->cols() - kItemsOnSecondPage);
-  EXPECT_TRUE(apps_grid_view_->IsSelectedView(GetItemViewAt(expected_index)));
-
-  // Tests moving down from the last row on the first page.
-  apps_grid_view_->SetSelectedView(
-      GetItemViewAt(kFirstIndexOfLastRowFirstPage));
-  SimulateKeyPress(ui::VKEY_DOWN);
-  EXPECT_TRUE(
-      apps_grid_view_->IsSelectedView(GetItemViewAt(GetTilesPerPage(0))));
-
-  // Tests moving down if no direct tile exists, clamp to the last item.
-  apps_grid_view_->SetSelectedView(GetItemViewAt(kLastIndexOfFirstPage));
-  SimulateKeyPress(ui::VKEY_DOWN);
-  EXPECT_TRUE(
-      apps_grid_view_->IsSelectedView(GetItemViewAt(kAllAppsItems - 1)));
-}
-
 // Tests that UMA is properly collected when either a suggested or normal app is
 // launched.
 TEST_F(AppsGridViewTest, UMATestForLaunchingApps) {
@@ -584,7 +505,8 @@ TEST_F(AppsGridViewTest, CloseFolderByClickingBackground) {
   AppsContainerView* apps_container_view =
       contents_view_->GetAppsContainerView();
 
-  const size_t kTotalItems = kMaxFolderItemsPerPage;
+  const size_t kTotalItems =
+      AppListConfig::instance().max_folder_items_per_page();
   model_->CreateAndPopulateFolderWithApps(kTotalItems);
   EXPECT_EQ(1u, model_->top_level_item_list()->item_count());
   EXPECT_EQ(AppListFolderItem::kItemType,
@@ -602,8 +524,33 @@ TEST_F(AppsGridViewTest, CloseFolderByClickingBackground) {
   EXPECT_FALSE(apps_container_view->IsInFolderView());
 }
 
+TEST_F(AppsGridViewTest, TapsBetweenAppsWontCloseAppList) {
+  model_->PopulateApps(2);
+  gfx::Point between_apps = GetItemRectOnCurrentPageAt(0, 0).right_center();
+  gfx::Point empty_space = GetItemRectOnCurrentPageAt(0, 2).CenterPoint();
+
+  ui::GestureEvent tap_between(between_apps.x(), between_apps.y(), 0,
+                               base::TimeTicks(),
+                               ui::GestureEventDetails(ui::ET_GESTURE_TAP));
+  ui::GestureEvent tap_outside(empty_space.x(), empty_space.y(), 0,
+                               base::TimeTicks(),
+                               ui::GestureEventDetails(ui::ET_GESTURE_TAP));
+
+  // Taps between apps should be handled to prevent them from going into
+  // app_list
+  apps_grid_view_->OnGestureEvent(&tap_between);
+  EXPECT_TRUE(tap_between.handled());
+
+  // Taps outside of occupied tiles should not be handled, that they may close
+  // the app_list
+  apps_grid_view_->OnGestureEvent(&tap_outside);
+  EXPECT_FALSE(tap_outside.handled());
+}
+
 TEST_F(AppsGridViewTest, PageResetAfterOpenFolder) {
-  const size_t kTotalItems = kMaxFolderPages * kMaxFolderItemsPerPage;
+  const size_t kTotalItems =
+      AppListConfig::instance().max_folder_pages() *
+      AppListConfig::instance().max_folder_items_per_page();
   model_->CreateAndPopulateFolderWithApps(kTotalItems);
   EXPECT_EQ(1u, model_->top_level_item_list()->item_count());
   EXPECT_EQ(AppListFolderItem::kItemType,
@@ -669,7 +616,8 @@ TEST_F(AppsGridViewTest, FolderColsAndRows) {
 }
 
 TEST_P(AppsGridViewTest, ScrollDownShouldNotExitFolder) {
-  const size_t kTotalItems = kMaxFolderItemsPerPage;
+  const size_t kTotalItems =
+      AppListConfig::instance().max_folder_items_per_page();
   model_->CreateAndPopulateFolderWithApps(kTotalItems);
   EXPECT_EQ(1u, model_->top_level_item_list()->item_count());
   EXPECT_EQ(AppListFolderItem::kItemType,
@@ -726,38 +674,7 @@ TEST_F(AppsGridViewTest, AppIconSelectedWhenMenuIsShown) {
   EXPECT_FALSE(apps_grid_view_->IsSelectedView(app));
 }
 
-// Tests various dragging behaviors.
-class AppsGridViewDragTest : public AppsGridViewTest {
- public:
-  AppsGridViewDragTest() = default;
-  ~AppsGridViewDragTest() override = default;
-
-  // testing::Test overrides:
-  void SetUp() override {
-    if (testing::UnitTest::GetInstance()->current_test_info()->value_param()) {
-      is_apps_grid_gap_enabled_ = GetParam().is_apps_grid_gap_enabled;
-      if (is_apps_grid_gap_enabled_) {
-        scoped_feature_list_.InitWithFeatures(
-            {app_list::features::kEnableAppsGridGapFeature}, {});
-      }
-    }
-    AppsGridViewTest::SetUp();
-  }
-
- protected:
-  bool is_apps_grid_gap_enabled_ = false;
-
- private:
-  base::test::ScopedFeatureList scoped_feature_list_;
-
-  DISALLOW_COPY_AND_ASSIGN(AppsGridViewDragTest);
-};
-
-INSTANTIATE_TEST_CASE_P(,
-                        AppsGridViewDragTest,
-                        testing::ValuesIn(kAppsGridViewDragTestParams));
-
-TEST_P(AppsGridViewDragTest, MouseDragItemIntoFolder) {
+TEST_P(AppsGridViewTest, MouseDragItemIntoFolder) {
   size_t kTotalItems = 3;
   model_->PopulateApps(kTotalItems);
   EXPECT_EQ(model_->top_level_item_list()->item_count(), kTotalItems);
@@ -804,9 +721,11 @@ TEST_P(AppsGridViewDragTest, MouseDragItemIntoFolder) {
   test_api_->LayoutToIdealBounds();
 }
 
-TEST_P(AppsGridViewDragTest, MouseDragMaxItemsInFolder) {
+TEST_P(AppsGridViewTest, MouseDragMaxItemsInFolder) {
   // Create and add a folder with |kMaxFolderItemsFullscreen - 1| items.
-  const size_t kMaxItems = kMaxFolderItemsPerPage * kMaxFolderPages;
+  const size_t kMaxItems =
+      AppListConfig::instance().max_folder_items_per_page() *
+      AppListConfig::instance().max_folder_pages();
   const size_t kTotalItems = kMaxItems - 1;
   AppListFolderItem* folder_item =
       model_->CreateAndPopulateFolderWithApps(kTotalItems);
@@ -849,9 +768,11 @@ TEST_P(AppsGridViewDragTest, MouseDragMaxItemsInFolder) {
 
 // Check that moving items around doesn't allow a drop to happen into a full
 // folder.
-TEST_P(AppsGridViewDragTest, MouseDragMaxItemsInFolderWithMovement) {
+TEST_P(AppsGridViewTest, MouseDragMaxItemsInFolderWithMovement) {
   // Create and add a folder with |kMaxFolderItemsFullscreen| in it.
-  const size_t kMaxItems = kMaxFolderItemsPerPage * kMaxFolderPages;
+  const size_t kMaxItems =
+      AppListConfig::instance().max_folder_items_per_page() *
+      AppListConfig::instance().max_folder_pages();
   size_t kTotalItems = kMaxItems;
   model_->CreateAndPopulateFolderWithApps(kMaxItems);
   EXPECT_EQ(1u, model_->top_level_item_list()->item_count());
@@ -900,13 +821,13 @@ TEST_P(AppsGridViewDragTest, MouseDragMaxItemsInFolderWithMovement) {
 }
 
 // Test reordering items via dragging.
-TEST_P(AppsGridViewDragTest, MouseDragItemReorder) {
-  // Using a simulated 2x3 layout for the test. The first row is occupied by
-  // suggested apps, so just populate 4 apps.
-  apps_grid_view_->SetLayout(2, 3);
-  model_->PopulateApps(4);
-  EXPECT_EQ(4u, model_->top_level_item_list()->item_count());
-  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3"),
+TEST_P(AppsGridViewTest, MouseDragItemReorder) {
+  // The default layout is 5x4, populate 7 apps so that we have second row to
+  // test dragging item to second row.
+  model_->PopulateApps(7);
+  contents_view_->GetAppsContainerView()->Layout();
+  EXPECT_EQ(7u, model_->top_level_item_list()->item_count());
+  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3,Item 4,Item 5,Item 6"),
             model_->GetModelContent());
 
   // Dragging an item towards its neighbours should not reorder until the drag
@@ -923,21 +844,23 @@ TEST_P(AppsGridViewDragTest, MouseDragItemReorder) {
   drag_vector.set_x(-half_tile_width - 4);
   SimulateDrag(AppsGridView::MOUSE, top_right, top_right + drag_vector);
   apps_grid_view_->EndDrag(false);
-  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3"),
+  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3,Item 4,Item 5,Item 6"),
             model_->GetModelContent());
   TestAppListItemViewIndice();
 
   // Drag left, past the folder dropping circle.
   gfx::Vector2d last_drag_vector(drag_vector);
-  drag_vector.set_x(-3 * half_tile_width + 4);
+  drag_vector.set_x(-2 * half_tile_width -
+                    AppListConfig::instance().folder_dropping_circle_radius() -
+                    4);
   SimulateDrag(AppsGridView::MOUSE, top_right + last_drag_vector,
                top_right + drag_vector);
   apps_grid_view_->EndDrag(false);
-  EXPECT_EQ(std::string("Item 1,Item 0,Item 2,Item 3"),
+  EXPECT_EQ(std::string("Item 1,Item 0,Item 2,Item 3,Item 4,Item 5,Item 6"),
             model_->GetModelContent());
   TestAppListItemViewIndice();
 
-  // Drag down, between apps 2 and 3. The gap should open up, making space for
+  // Drag down, between apps 5 and 6. The gap should open up, making space for
   // app 1 in the bottom left.
   last_drag_vector = drag_vector;
   drag_vector.set_x(-half_tile_width);
@@ -945,7 +868,7 @@ TEST_P(AppsGridViewDragTest, MouseDragItemReorder) {
   SimulateDrag(AppsGridView::MOUSE, top_right + last_drag_vector,
                top_right + drag_vector);
   apps_grid_view_->EndDrag(false);
-  EXPECT_EQ(std::string("Item 0,Item 2,Item 1,Item 3"),
+  EXPECT_EQ(std::string("Item 0,Item 2,Item 3,Item 4,Item 5,Item 1,Item 6"),
             model_->GetModelContent());
   TestAppListItemViewIndice();
 
@@ -957,7 +880,7 @@ TEST_P(AppsGridViewDragTest, MouseDragItemReorder) {
   SimulateDrag(AppsGridView::MOUSE, top_right + last_drag_vector,
                top_right + drag_vector);
   apps_grid_view_->EndDrag(false);
-  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3"),
+  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3,Item 4,Item 5,Item 6"),
             model_->GetModelContent());
   TestAppListItemViewIndice();
 
@@ -968,12 +891,12 @@ TEST_P(AppsGridViewDragTest, MouseDragItemReorder) {
   SimulateDrag(AppsGridView::MOUSE, top_right + last_drag_vector,
                top_right + drag_vector);
   apps_grid_view_->EndDrag(false);
-  EXPECT_EQ(std::string("Item 0,Item 2,Item 3,Item 1"),
+  EXPECT_EQ(std::string("Item 0,Item 2,Item 3,Item 4,Item 5,Item 6,Item 1"),
             model_->GetModelContent());
   TestAppListItemViewIndice();
 }
 
-TEST_P(AppsGridViewDragTest, MouseDragFolderReorder) {
+TEST_P(AppsGridViewTest, MouseDragFolderReorder) {
   size_t kTotalItems = 2;
   model_->CreateAndPopulateFolderWithApps(kTotalItems);
   model_->PopulateAppWithId(kTotalItems);
@@ -998,7 +921,7 @@ TEST_P(AppsGridViewDragTest, MouseDragFolderReorder) {
   TestAppListItemViewIndice();
 }
 
-TEST_P(AppsGridViewDragTest, MouseDragWithCancelDeleteAddItem) {
+TEST_P(AppsGridViewTest, MouseDragWithCancelDeleteAddItem) {
   size_t kTotalItems = 4;
   model_->PopulateApps(kTotalItems);
   EXPECT_EQ(model_->top_level_item_list()->item_count(), kTotalItems);
@@ -1031,7 +954,7 @@ TEST_P(AppsGridViewDragTest, MouseDragWithCancelDeleteAddItem) {
   test_api_->LayoutToIdealBounds();
 }
 
-TEST_P(AppsGridViewDragTest, MouseDragFlipPage) {
+TEST_P(AppsGridViewTest, MouseDragFlipPage) {
   apps_grid_view_->set_page_flip_delay_in_ms_for_testing(10);
   GetPaginationModel()->SetTransitionDurations(10, 10);
 
@@ -1045,7 +968,7 @@ TEST_P(AppsGridViewDragTest, MouseDragFlipPage) {
   gfx::Point from = GetItemRectOnCurrentPageAt(0, 0).CenterPoint();
   gfx::Point to;
   const gfx::Rect apps_grid_bounds = apps_grid_view_->GetLocalBounds();
-  to = gfx::Point(apps_grid_bounds.width() / 2, apps_grid_bounds.bottom());
+  to = gfx::Point(apps_grid_bounds.width() / 2, apps_grid_bounds.bottom() + 1);
 
   // For fullscreen/bubble launcher, drag to the bottom/right of bounds.
   page_flip_waiter.Reset();
@@ -1056,15 +979,10 @@ TEST_P(AppsGridViewDragTest, MouseDragFlipPage) {
     page_flip_waiter.Wait();
   }
 
-  if (is_apps_grid_gap_enabled_) {
-    // When apps grid gap is enabled, the user can drag an item to an extra page
-    // created at the end.
-    EXPECT_EQ("1,2,3", page_flip_waiter.selected_pages());
-    EXPECT_EQ(3, GetPaginationModel()->selected_page());
-  } else {
-    EXPECT_EQ("1,2", page_flip_waiter.selected_pages());
-    EXPECT_EQ(2, GetPaginationModel()->selected_page());
-  }
+  // When apps grid gap is enabled, the user can drag an item to an extra page
+  // created at the end.
+  EXPECT_EQ("1,2,3", page_flip_waiter.selected_pages());
+  EXPECT_EQ(3, GetPaginationModel()->selected_page());
 
   // Cancel drag and put the dragged view back to its ideal position so that
   // the next drag would pick it up.
@@ -1087,7 +1005,7 @@ TEST_P(AppsGridViewDragTest, MouseDragFlipPage) {
   apps_grid_view_->EndDrag(true);
 }
 
-TEST_F(AppsGridViewDragTest, UpdateFolderBackgroundOnCancelDrag) {
+TEST_F(AppsGridViewTest, UpdateFolderBackgroundOnCancelDrag) {
   const int kTotalItems = 4;
   TestAppsGridViewFolderDelegate folder_delegate;
   apps_grid_view_->set_folder_delegate(&folder_delegate);
@@ -1105,6 +1023,28 @@ TEST_F(AppsGridViewDragTest, UpdateFolderBackgroundOnCancelDrag) {
             model_->GetModelContent());
 }
 
+// Test focus change before and after dragging an item. (See
+// https://crbug.com/834682)
+TEST_F(AppsGridViewTest, FocusOfDraggedView) {
+  model_->PopulateApps(1);
+  contents_view_->GetAppsContainerView()->Layout();
+  auto* search_box = contents_view_->GetSearchBoxView()->search_box();
+  auto* item_view = apps_grid_view_->view_model()->view_at(0);
+  EXPECT_TRUE(search_box->HasFocus());
+  EXPECT_FALSE(item_view->HasFocus());
+
+  // Dragging the item towards its right.
+  const gfx::Point from = GetItemRectOnCurrentPageAt(0, 0).CenterPoint();
+  const gfx::Point to = GetItemRectOnCurrentPageAt(0, 1).CenterPoint();
+  SimulateDrag(AppsGridView::MOUSE, from, to);
+  EXPECT_FALSE(search_box->HasFocus());
+  EXPECT_TRUE(item_view->HasFocus());
+
+  apps_grid_view_->EndDrag(false);
+  EXPECT_FALSE(search_box->HasFocus());
+  EXPECT_TRUE(item_view->HasFocus());
+}
+
 // Test various dragging behaviors only allowed when apps grid gap (part of
 // home launcher feature) is enabled.
 class AppsGridGapTest : public AppsGridViewTest {
@@ -1114,8 +1054,6 @@ class AppsGridGapTest : public AppsGridViewTest {
 
   // testing::Test overrides:
   void SetUp() override {
-    scoped_feature_list_.InitWithFeatures(
-        {app_list::features::kEnableAppsGridGapFeature}, {});
     AppsGridViewTest::SetUp();
     apps_grid_view_->set_page_flip_delay_in_ms_for_testing(10);
     GetPaginationModel()->SetTransitionDurations(10, 10);
@@ -1142,7 +1080,7 @@ class AppsGridGapTest : public AppsGridViewTest {
     const gfx::Rect apps_grid_bounds = apps_grid_view_->GetLocalBounds();
     gfx::Point point_in_page_flip_buffer =
         gfx::Point(apps_grid_bounds.width() / 2,
-                   next_page ? apps_grid_bounds.bottom() : 0);
+                   next_page ? apps_grid_bounds.bottom() + 1 : 0);
 
     // Build the drag event which will be triggered after page flip.
     gfx::Point root_to(to);
@@ -1169,15 +1107,9 @@ class AppsGridGapTest : public AppsGridViewTest {
   std::unique_ptr<PageFlipWaiter> page_flip_waiter_;
 
  private:
-  base::test::ScopedFeatureList scoped_feature_list_;
-
   DISALLOW_COPY_AND_ASSIGN(AppsGridGapTest);
 };
 
-INSTANTIATE_TEST_CASE_P(,
-                        AppsGridGapTest,
-                        testing::ValuesIn(kAppsGridGapTestParams));
-
 TEST_P(AppsGridGapTest, MoveAnItemToNewEmptyPage) {
   const int kApps = 2;
   model_->PopulateApps(kApps);
@@ -1348,8 +1280,8 @@ TEST_P(AppsGridGapTest, MoveLastItemToNewEmptyPage) {
   EXPECT_EQ(std::string("Item 0"), model_->GetModelContent());
 }
 
-TEST_P(AppsGridGapTest, MoveItemToPreviousFullPageNotAllowed) {
-  const int kApps = 1 + GetTilesPerPage(0);
+TEST_P(AppsGridGapTest, MoveItemToPreviousFullPage) {
+  const int kApps = 2 + GetTilesPerPage(0);
   model_->PopulateApps(kApps);
 
   // There are two pages and last item is on second page.
@@ -1359,56 +1291,78 @@ TEST_P(AppsGridGapTest, MoveItemToPreviousFullPageNotAllowed) {
   const views::ViewModelT<AppListItemView>* view_model =
       apps_grid_view_->view_model();
   EXPECT_EQ(kApps, view_model->view_size());
-  for (int i = 0; i < kApps - 1; ++i) {
-    EXPECT_EQ(view_model->view_at(i),
-              test_api_->GetViewAtVisualIndex(0 /* page */, i /* slot */));
-    EXPECT_EQ("Item " + std::to_string(i),
+  for (int i = 0; i < kApps; ++i) {
+    EXPECT_EQ(view_model->view_at(i), test_api_->GetViewAtVisualIndex(
+                                          i / GetTilesPerPage(0) /* page */,
+                                          i % GetTilesPerPage(0) /* slot */));
+    EXPECT_EQ("Item " + base::NumberToString(i),
               view_model->view_at(i)->item()->id());
   }
-  EXPECT_EQ(view_model->view_at(kApps - 1),
-            test_api_->GetViewAtVisualIndex(1 /* page */, 0 /* slot */));
-  EXPECT_EQ("Item " + std::to_string(kApps - 1),
-            view_model->view_at(kApps - 1)->item()->id());
 
-  // There's no "page break" item between Item 19 and 20, although there are two
-  // pages. It will only be added after user operations.
-  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3,Item 4,Item 5,Item 6,Item "
-                        "7,Item 8,Item 9,Item 10,Item 11,Item 12,Item 13,Item "
-                        "14,Item 15,Item 16,Item 17,Item 18,Item 19,Item 20"),
-            model_->GetModelContent());
+  // There's no "page break" item at the end of first page, although there are
+  // two pages. It will only be added after user operations.
+  std::string model_content = "Item 0";
+  for (int i = 1; i < kApps; ++i)
+    model_content.append(",Item " + base::NumberToString(i));
+  EXPECT_EQ(model_content, model_->GetModelContent());
 
   // Drag the last item to the first item's left position in previous page.
-  gfx::Point from = test_api_->GetItemTileRectAtVisualIndex(1, 0).CenterPoint();
+  gfx::Point from = test_api_->GetItemTileRectAtVisualIndex(1, 1).CenterPoint();
   gfx::Rect tile_rect = test_api_->GetItemTileRectAtVisualIndex(0, 0);
   gfx::Point to_in_previous_page = tile_rect.CenterPoint();
   to_in_previous_page.set_x(tile_rect.x());
   GetPaginationModel()->SelectPage(1, false);
   SimulateDragToNeighborPage(false /* next_page */, from, to_in_previous_page);
 
-  // The dragging is not successfull, so nothing changes visually.
+  // The dragging is successful, the last item becomes the first item.
   EXPECT_EQ("0", page_flip_waiter_->selected_pages());
   EXPECT_EQ(0, GetPaginationModel()->selected_page());
   TestAppListItemViewIndice();
   EXPECT_EQ(kApps, view_model->view_size());
-  EXPECT_EQ(view_model->view_at(0),
-            test_api_->GetViewAtVisualIndex(0 /* page */, 0 /* slot */));
-  for (int i = 0; i < kApps - 1; ++i) {
-    EXPECT_EQ(view_model->view_at(i),
-              test_api_->GetViewAtVisualIndex(0 /* page */, i /* slot */));
-    EXPECT_EQ("Item " + std::to_string(i),
+  for (int i = 0; i < kApps; ++i) {
+    EXPECT_EQ(view_model->view_at(i), test_api_->GetViewAtVisualIndex(
+                                          i / GetTilesPerPage(0) /* page */,
+                                          i % GetTilesPerPage(0) /* slot */));
+    EXPECT_EQ("Item " + base::NumberToString((i + kApps - 1) % kApps),
               view_model->view_at(i)->item()->id());
   }
-  EXPECT_EQ(view_model->view_at(kApps - 1),
-            test_api_->GetViewAtVisualIndex(1 /* page */, 0 /* slot */));
-  EXPECT_EQ("Item " + std::to_string(kApps - 1),
-            view_model->view_at(kApps - 1)->item()->id());
 
   // A "page break" item is added to split the pages.
-  EXPECT_EQ(std::string("Item 0,Item 1,Item 2,Item 3,Item 4,Item 5,Item 6,Item "
-                        "7,Item 8,Item 9,Item 10,Item 11,Item 12,Item 13,Item "
-                        "14,Item 15,Item 16,Item 17,Item 18,Item "
-                        "19,PageBreakItem,Item 20"),
-            model_->GetModelContent());
+  model_content = "Item " + base::NumberToString(kApps - 1);
+  for (int i = 1; i < kApps; ++i) {
+    model_content.append(",Item " + base::NumberToString(i - 1));
+    if (i == GetTilesPerPage(0) - 1)
+      model_content.append(",PageBreakItem");
+  }
+  EXPECT_EQ(model_content, model_->GetModelContent());
+
+  // Again drag the last item to the first item's left position in previous
+  // page.
+  GetPaginationModel()->SelectPage(1, false);
+  SimulateDragToNeighborPage(false /* next_page */, from, to_in_previous_page);
+
+  // The dragging is successful, the last item becomes the first item again.
+  EXPECT_EQ("0", page_flip_waiter_->selected_pages());
+  EXPECT_EQ(0, GetPaginationModel()->selected_page());
+  TestAppListItemViewIndice();
+  EXPECT_EQ(kApps, view_model->view_size());
+  for (int i = 0; i < kApps; ++i) {
+    EXPECT_EQ(view_model->view_at(i), test_api_->GetViewAtVisualIndex(
+                                          i / GetTilesPerPage(0) /* page */,
+                                          i % GetTilesPerPage(0) /* slot */));
+    EXPECT_EQ("Item " + base::NumberToString((i + kApps - 2) % kApps),
+              view_model->view_at(i)->item()->id());
+  }
+
+  // A "page break" item still exists.
+  model_content = "Item " + base::NumberToString(kApps - 2) + ",Item " +
+                  base::NumberToString(kApps - 1);
+  for (int i = 2; i < kApps; ++i) {
+    model_content.append(",Item " + base::NumberToString(i - 2));
+    if (i == GetTilesPerPage(0) - 1)
+      model_content.append(",PageBreakItem");
+  }
+  EXPECT_EQ(model_content, model_->GetModelContent());
 }
 
 }  // namespace test
