@@ -13,13 +13,15 @@
 #include "ash/app_list/views/app_list_view.h"
 #include "ash/app_list/views/apps_container_view.h"
 #include "ash/app_list/views/apps_grid_view.h"
+#include "ash/app_list/views/assistant/assistant_page_view.h"
+#include "ash/app_list/views/expand_arrow_view.h"
 #include "ash/app_list/views/horizontal_page_container.h"
 #include "ash/app_list/views/search_box_view.h"
 #include "ash/app_list/views/search_result_answer_card_view.h"
 #include "ash/app_list/views/search_result_list_view.h"
 #include "ash/app_list/views/search_result_page_view.h"
 #include "ash/app_list/views/search_result_tile_item_list_view.h"
-#include "ash/public/cpp/app_list/app_list_constants.h"
+#include "ash/public/cpp/app_list/app_list_config.h"
 #include "ash/public/cpp/app_list/app_list_features.h"
 #include "ash/public/cpp/app_list/app_list_switches.h"
 #include "base/logging.h"
@@ -27,6 +29,8 @@
 #include "ui/display/display.h"
 #include "ui/display/screen.h"
 #include "ui/events/event.h"
+#include "ui/keyboard/keyboard_controller.h"
+#include "ui/views/accessibility/view_accessibility.h"
 #include "ui/views/view_model.h"
 #include "ui/views/widget/widget.h"
 
@@ -34,12 +38,17 @@ namespace app_list {
 
 namespace {
 
+// The range of app list transition progress in which the expand arrow'
+// opacity changes from 0 to 1.
+constexpr float kExpandArrowOpacityStartProgress = 0.61;
+constexpr float kExpandArrowOpacityEndProgress = 1;
+
 void DoAnimation(base::TimeDelta animation_duration,
                  ui::Layer* layer,
                  float target_opacity) {
   ui::ScopedLayerAnimationSettings animation(layer->GetAnimator());
   animation.SetTransitionDuration(animation_duration);
-  animation.SetTweenType(gfx::Tween::EASE_OUT);
+  animation.SetTweenType(gfx::Tween::EASE_IN);
   animation.SetPreemptionStrategy(
       ui::LayerAnimator::IMMEDIATELY_ANIMATE_TO_NEW_TARGET);
   layer->SetOpacity(target_opacity);
@@ -49,8 +58,9 @@ void DoAnimation(base::TimeDelta animation_duration,
 
 ContentsView::ContentsView(AppListView* app_list_view)
     : app_list_view_(app_list_view) {
-  pagination_model_.SetTransitionDurations(kPageTransitionDurationInMs,
-                                           kOverscrollPageTransitionDurationMs);
+  pagination_model_.SetTransitionDurations(
+      AppListConfig::instance().page_transition_duration_ms(),
+      AppListConfig::instance().overscroll_page_transition_duration_ms());
   pagination_model_.AddObserver(this);
 }
 
@@ -77,13 +87,16 @@ void ContentsView::Init(AppListModel* model) {
   SearchModel::SearchResults* results =
       view_delegate->GetSearchModel()->results();
 
-  if (features::IsAnswerCardEnabled()) {
+  if (app_list_features::IsAnswerCardEnabled()) {
     search_result_answer_card_view_ =
         new SearchResultAnswerCardView(view_delegate);
     search_results_page_view_->AddSearchResultContainerView(
         results, search_result_answer_card_view_);
   }
 
+  expand_arrow_view_ = new ExpandArrowView(this, app_list_view_);
+  AddChildView(expand_arrow_view_);
+
   search_result_tile_item_list_view_ = new SearchResultTileItemListView(
       search_results_page_view_, GetSearchBoxView()->search_box(),
       view_delegate);
@@ -98,6 +111,14 @@ void ContentsView::Init(AppListModel* model) {
   AddLauncherPage(search_results_page_view_,
                   ash::AppListState::kStateSearchResults);
 
+  if (app_list_features::IsEmbeddedAssistantUIEnabled()) {
+    assistant_page_view_ =
+        new AssistantPageView(this, view_delegate->GetAssistantViewDelegate());
+    assistant_page_view_->SetVisible(false);
+    AddLauncherPage(assistant_page_view_,
+                    ash::AppListState::kStateEmbeddedAssistant);
+  }
+
   AddLauncherPage(horizontal_page_container_, ash::AppListState::kStateApps);
 
   int initial_page_index = GetPageIndexForState(ash::AppListState::kStateStart);
@@ -136,13 +157,18 @@ void ContentsView::SetDragAndDropHostOfCurrentAppList(
 }
 
 void ContentsView::SetActiveState(ash::AppListState state) {
-  SetActiveState(state, true);
+  SetActiveState(state, !AppListView::ShortAnimationsForTesting());
 }
 
 void ContentsView::SetActiveState(ash::AppListState state, bool animate) {
   if (IsStateActive(state))
     return;
 
+  // The primary way to set the state to search or Assistant results should be
+  // via |ShowSearchResults| or |ShowEmbeddedAssistantUI|.
+  DCHECK(state != ash::AppListState::kStateSearchResults &&
+         state != ash::AppListState::kStateEmbeddedAssistant);
+
   SetActiveStateInternal(GetPageIndexForState(state), false, animate);
 }
 
@@ -189,12 +215,12 @@ AppsContainerView* ContentsView::GetAppsContainerView() {
 }
 
 void ContentsView::SetActiveStateInternal(int page_index,
-                                          bool show_search_results,
+                                          bool show_search_or_assistant_results,
                                           bool animate) {
   if (!GetPageView(page_index)->visible())
     return;
 
-  if (!show_search_results)
+  if (!show_search_or_assistant_results)
     page_before_search_ = page_index;
 
   app_list_pages_[GetActivePageIndex()]->OnWillBeHidden();
@@ -218,6 +244,10 @@ void ContentsView::ActivePageChanged() {
   app_list_pages_[GetActivePageIndex()]->OnWillBeShown();
 
   GetAppListMainView()->model()->SetState(state);
+
+  UpdateExpandArrowFocusBehavior(state);
+
+  UpdateSearchBoxVisibility(state);
 }
 
 void ContentsView::ShowSearchResults(bool show) {
@@ -225,13 +255,42 @@ void ContentsView::ShowSearchResults(bool show) {
       GetPageIndexForState(ash::AppListState::kStateSearchResults);
   DCHECK_GE(search_page, 0);
 
-  SetActiveStateInternal(show ? search_page : page_before_search_, show, true);
+  // Hide or Show results
+  GetPageView(search_page)->SetVisible(show);
+
+  SetActiveStateInternal(show ? search_page : page_before_search_, show,
+                         !AppListView::ShortAnimationsForTesting());
 }
 
 bool ContentsView::IsShowingSearchResults() const {
   return IsStateActive(ash::AppListState::kStateSearchResults);
 }
 
+void ContentsView::ShowEmbeddedAssistantUI(bool show) {
+  const int assistant_page =
+      GetPageIndexForState(ash::AppListState::kStateEmbeddedAssistant);
+  DCHECK_GE(assistant_page, 0);
+
+  // Hide or Show results.
+  GetPageView(assistant_page)->SetVisible(show);
+  if (show)
+    GetPageView(assistant_page)->RequestFocus();
+
+  // Embedded Assistant UI can only be transitioned from/to
+  // |search_result_page_view_|.
+  const int search_results_page =
+      GetPageIndexForState(ash::AppListState::kStateSearchResults);
+  DCHECK_GE(search_results_page, 0);
+  GetPageView(search_results_page)->SetVisible(!show);
+  SetActiveStateInternal(show ? assistant_page : search_results_page,
+                         /*show_search_or_assistant_results=*/true,
+                         !AppListView::ShortAnimationsForTesting());
+}
+
+bool ContentsView::IsShowingEmbeddedAssistantUI() const {
+  return IsStateActive(ash::AppListState::kStateEmbeddedAssistant);
+}
+
 void ContentsView::UpdatePageBounds() {
   // The bounds calculations will potentially be mid-transition (depending on
   // the state of the PaginationModel).
@@ -249,35 +308,38 @@ void ContentsView::UpdatePageBounds() {
 
   ash::AppListState current_state = GetStateForPageIndex(current_page);
   ash::AppListState target_state = GetStateForPageIndex(target_page);
-
   // Update app list pages.
   for (AppListPage* page : app_list_pages_) {
-    page->OnAnimationUpdated(progress, current_state, target_state);
-
     gfx::Rect to_rect = page->GetPageBoundsForState(target_state);
     gfx::Rect from_rect = page->GetPageBoundsForState(current_state);
-    if (from_rect == to_rect)
-      continue;
 
     // Animate linearly (the PaginationModel handles easing).
     gfx::Rect bounds(
         gfx::Tween::RectValueBetween(progress, from_rect, to_rect));
 
     page->SetBoundsRect(bounds);
+
+    if (ShouldLayoutPage(page, current_state, target_state))
+      page->OnAnimationUpdated(progress, current_state, target_state);
   }
 
   // Update the search box.
   UpdateSearchBox(progress, current_state, target_state);
+
+  // Update the expand arrow view's opacity.
+  UpdateExpandArrowOpacity(progress, current_state, target_state);
 }
 
 void ContentsView::UpdateSearchBox(double progress,
                                    ash::AppListState current_state,
                                    ash::AppListState target_state) {
+  SearchBoxView* search_box = GetSearchBoxView();
+  if (!search_box->GetWidget())
+    return;
+
   AppListPage* from_page = GetPageView(GetPageIndexForState(current_state));
   AppListPage* to_page = GetPageView(GetPageIndexForState(target_state));
 
-  SearchBoxView* search_box = GetSearchBoxView();
-
   gfx::Rect search_box_from(from_page->GetSearchBoxBounds());
   gfx::Rect search_box_to(to_page->GetSearchBoxBounds());
   gfx::Rect search_box_rect =
@@ -285,9 +347,65 @@ void ContentsView::UpdateSearchBox(double progress,
 
   search_box->UpdateLayout(progress, current_state, target_state);
   search_box->UpdateBackground(progress, current_state, target_state);
-  search_box->GetWidget()->SetBounds(
-      search_box->GetViewBoundsForSearchBoxContentsBounds(
-          ConvertRectToWidget(search_box_rect)));
+  search_box_rect = search_box->GetViewBoundsForSearchBoxContentsBounds(
+      ConvertRectToWidgetWithoutTransform(search_box_rect));
+
+  // Search box is in a different widget with AppListMainView, so we need to
+  // manually transform the search box using the same scale based on the same
+  // origin.
+  const float scale = GetAppListMainViewScale();
+  search_box_rect.set_origin(
+      gfx::Point(search_box_rect.x() * scale, search_box_rect.y() * scale));
+  search_box->GetWidget()->SetBounds(search_box_rect);
+  gfx::Transform transform;
+  transform.Scale(scale, scale);
+  search_box->GetWidget()->GetNativeView()->SetTransform(transform);
+}
+
+void ContentsView::UpdateExpandArrowOpacity(double progress,
+                                            ash::AppListState current_state,
+                                            ash::AppListState target_state) {
+  if (current_state == ash::AppListState::kStateSearchResults &&
+      (target_state == ash::AppListState::kStateStart ||
+       target_state == ash::AppListState::kStateApps)) {
+    // Fade in the expand arrow when search results page is opened.
+    expand_arrow_view_->layer()->SetOpacity(
+        gfx::Tween::FloatValueBetween(progress, 0, 1));
+  } else if (target_state == ash::AppListState::kStateSearchResults &&
+             (current_state == ash::AppListState::kStateStart ||
+              current_state == ash::AppListState::kStateApps)) {
+    // Fade out the expand arrow when search results page is closed.
+    expand_arrow_view_->layer()->SetOpacity(
+        gfx::Tween::FloatValueBetween(progress, 1, 0));
+  }
+}
+
+void ContentsView::UpdateExpandArrowFocusBehavior(
+    ash::AppListState current_state) {
+  bool state_start = current_state == ash::AppListState::kStateStart;
+
+  // The expand arrow is only focusable and has InkDropMode on in peeking
+  // state.
+  expand_arrow_view_->SetFocusBehavior(state_start ? FocusBehavior::ALWAYS
+                                                   : FocusBehavior::NEVER);
+  expand_arrow_view_->SetInkDropMode(
+      state_start ? views::InkDropHostView::InkDropMode::ON
+                  : views::InkDropHostView::InkDropMode::OFF);
+
+  // Allow ChromeVox to focus the expand arrow only when peeking launcher.
+  expand_arrow_view_->GetViewAccessibility().OverrideIsIgnored(!state_start);
+  expand_arrow_view_->GetViewAccessibility().NotifyAccessibilityEvent(
+      ax::mojom::Event::kTreeChanged);
+}
+
+void ContentsView::UpdateSearchBoxVisibility(ash::AppListState current_state) {
+  auto* search_box_widget = GetSearchBoxView()->GetWidget();
+  if (search_box_widget) {
+    // Hide search box widget in order to click on the embedded Assistant UI.
+    const bool show_search_box =
+        current_state != ash::AppListState::kStateEmbeddedAssistant;
+    show_search_box ? search_box_widget->Show() : search_box_widget->Hide();
+  }
 }
 
 PaginationModel* ContentsView::GetAppsPaginationModel() {
@@ -335,7 +453,8 @@ gfx::Rect ContentsView::GetDefaultSearchBoxBounds() const {
   search_box_bounds.set_size(GetSearchBoxView()->GetPreferredSize());
   search_box_bounds.Offset((bounds().width() - search_box_bounds.width()) / 2,
                            0);
-  search_box_bounds.set_y(kSearchBoxTopPadding);
+  search_box_bounds.set_y(
+      AppListConfig::instance().search_box_fullscreen_top_padding());
   return search_box_bounds;
 }
 
@@ -349,37 +468,42 @@ gfx::Rect ContentsView::GetDefaultContentsBounds() const {
   return GetContentsBounds();
 }
 
-gfx::Size ContentsView::GetMaximumContentsSize() const {
-  int max_width = 0;
-  int max_height = 0;
-  for (AppListPage* page : app_list_pages_) {
-    const gfx::Size size(page->GetPreferredSize());
-    max_width = std::max(size.width(), max_width);
-    max_height = std::max(size.height(), max_height);
-  }
-  return gfx::Size(max_width, max_height);
-}
-
 bool ContentsView::Back() {
+  // If the virtual keyboard is visible, dismiss the keyboard and return early
+  auto* const keyboard_controller = keyboard::KeyboardController::Get();
+  if (keyboard_controller->IsKeyboardVisible()) {
+    keyboard_controller->HideKeyboardByUser();
+    return true;
+  }
   ash::AppListState state = view_to_state_[GetActivePageIndex()];
   switch (state) {
     case ash::AppListState::kStateStart:
       // Close the app list when Back() is called from the start page.
       return false;
-    case ash::AppListState::kStateApps:
+    case ash::AppListState::kStateApps: {
+      PaginationModel* pagination_model =
+          GetAppsContainerView()->apps_grid_view()->pagination_model();
       if (GetAppsContainerView()->IsInFolderView()) {
         GetAppsContainerView()->app_list_folder_view()->CloseFolderPage();
+      } else if (app_list_view_->is_tablet_mode() &&
+                 pagination_model->total_pages() > 0 &&
+                 pagination_model->selected_page() > 0) {
+        pagination_model->SelectPage(
+            0, !app_list_view_->ShortAnimationsForTesting());
       } else {
         // Close the app list when Back() is called from the apps page.
         return false;
       }
       break;
+    }
     case ash::AppListState::kStateSearchResults:
       GetSearchBoxView()->ClearSearch();
-      GetSearchBoxView()->SetSearchBoxActive(false);
+      GetSearchBoxView()->SetSearchBoxActive(false, ui::ET_UNKNOWN);
       ShowSearchResults(false);
       break;
-    case ash::AppListState::kStateCustomLauncherPageDeprecated:
+    case ash::AppListState::kStateEmbeddedAssistant:
+      ShowEmbeddedAssistantUI(false);
+      break;
     case ash::AppListState::kInvalidState:  // Falls through.
       NOTREACHED();
       break;
@@ -391,34 +515,20 @@ gfx::Size ContentsView::GetDefaultContentsSize() const {
   return horizontal_page_container_->GetPreferredSize();
 }
 
-gfx::Size ContentsView::CalculatePreferredSize() const {
-  return gfx::Size(GetDisplayWidth(), GetDisplayHeight());
-}
-
 void ContentsView::Layout() {
-  // Immediately finish all current animations.
-  pagination_model_.FinishAnimation();
-
-  if (GetContentsBounds().IsEmpty())
+  const gfx::Rect rect = GetContentsBounds();
+  if (rect.IsEmpty())
     return;
 
-  for (AppListPage* page : app_list_pages_) {
-    // Ensures re-layout happens even when the page bounds does not change. So
-    // that |horizontal_page_container_| can layout its children in response to
-    // user dragging.
-    page->InvalidateLayout();
-    page->SetBoundsRect(page->GetPageBoundsForState(GetActiveState()));
-  }
+  // Layout expand arrow.
+  gfx::Rect arrow_rect(GetContentsBounds());
+  const gfx::Size arrow_size(expand_arrow_view_->GetPreferredSize());
+  arrow_rect.set_height(arrow_size.height());
+  arrow_rect.ClampToCenteredSize(arrow_size);
+  expand_arrow_view_->SetBoundsRect(arrow_rect);
+  expand_arrow_view_->SchedulePaint();
 
-  // The search box is contained in a widget so set the bounds of the widget
-  // rather than the SearchBoxView.
-  views::Widget* search_box_widget = GetSearchBoxView()->GetWidget();
-  if (search_box_widget && search_box_widget != GetWidget()) {
-    gfx::Rect search_box_bounds = GetSearchBoxBoundsForState(GetActiveState());
-    search_box_widget->SetBounds(ConvertRectToWidget(
-        GetSearchBoxView()->GetViewBoundsForSearchBoxContentsBounds(
-            search_box_bounds)));
-  }
+  UpdatePageBounds();
 }
 
 const char* ContentsView::GetClassName() const {
@@ -443,22 +553,6 @@ void ContentsView::TransitionChanged() {
 
 void ContentsView::TransitionEnded() {}
 
-int ContentsView::GetDisplayHeight() const {
-  return display::Screen::GetScreen()
-      ->GetDisplayNearestView(GetWidget()->GetNativeView())
-      .work_area()
-      .size()
-      .height();
-}
-
-int ContentsView::GetDisplayWidth() const {
-  return display::Screen::GetScreen()
-      ->GetDisplayNearestView(GetWidget()->GetNativeView())
-      .work_area()
-      .size()
-      .width();
-}
-
 void ContentsView::FadeOutOnClose(base::TimeDelta animation_duration) {
   DoAnimation(animation_duration, layer(), 0.0f);
   DoAnimation(animation_duration, GetSearchBoxView()->layer(), 0.0f);
@@ -475,4 +569,104 @@ views::View* ContentsView::GetSelectedView() const {
   return app_list_pages_[GetActivePageIndex()]->GetSelectedView();
 }
 
+void ContentsView::UpdateYPositionAndOpacity() {
+  AppListViewState state = app_list_view_->app_list_state();
+  if (state == AppListViewState::CLOSED ||
+      state == AppListViewState::FULLSCREEN_SEARCH ||
+      state == AppListViewState::HALF) {
+    return;
+  }
+
+  const bool should_restore_opacity =
+      !app_list_view_->is_in_drag() &&
+      (app_list_view_->app_list_state() != AppListViewState::CLOSED);
+
+  // Changes the opacity of expand arrow between 0 and 1 when app list
+  // transition progress changes between |kExpandArrowOpacityStartProgress|
+  // and |kExpandArrowOpacityEndProgress|.
+  expand_arrow_view_->layer()->SetOpacity(
+      should_restore_opacity
+          ? 1.0f
+          : std::min(std::max((app_list_view_->GetAppListTransitionProgress() -
+                               kExpandArrowOpacityStartProgress) /
+                                  (kExpandArrowOpacityEndProgress -
+                                   kExpandArrowOpacityStartProgress),
+                              0.f),
+                     1.0f));
+
+  expand_arrow_view_->SchedulePaint();
+
+  AppsContainerView* apps_container_view = GetAppsContainerView();
+  SearchBoxView* search_box = GetSearchBoxView();
+  gfx::Rect search_rect = search_box->GetViewBoundsForSearchBoxContentsBounds(
+      ConvertRectToWidgetWithoutTransform(
+          apps_container_view->GetSearchBoxExpectedBounds()));
+
+  // Search box is in a different widget with AppListMainView, so we need to
+  // manually transform the search box position using the same scale based on
+  // the same origin.
+  const float scale = GetAppListMainViewScale();
+  search_rect.set_origin(
+      gfx::Point(search_rect.x() * scale, search_rect.y() * scale));
+  search_box->GetWidget()->SetBounds(search_rect);
+
+  search_results_page_view()->SetBoundsRect(
+      search_results_page_view()->AddShadowBorderToBounds(
+          apps_container_view->GetSearchBoxExpectedBounds()));
+
+  apps_container_view->UpdateYPositionAndOpacity();
+}
+
+float ContentsView::GetAppListMainViewScale() const {
+  // The x and y scale are the same.
+  return app_list_view_->app_list_main_view()->GetTransform().Scale2d().x();
+}
+
+void ContentsView::SetExpandArrowViewVisibility(bool show) {
+  if (expand_arrow_view_->visible() == show)
+    return;
+
+  expand_arrow_view_->SetVisible(show);
+}
+
+bool ContentsView::ShouldLayoutPage(AppListPage* page,
+                                    ash::AppListState current_state,
+                                    ash::AppListState target_state) const {
+  if (page == horizontal_page_container_) {
+    return (current_state == ash::AppListState::kStateStart &&
+            target_state == ash::AppListState::kStateApps) ||
+           (current_state == ash::AppListState::kStateApps &&
+            target_state == ash::AppListState::kStateStart) ||
+           (current_state == ash::AppListState::kStateSearchResults &&
+            target_state == ash::AppListState::kStateStart);
+  }
+
+  if (page == search_results_page_view_) {
+    return ((current_state == ash::AppListState::kStateSearchResults &&
+             target_state == ash::AppListState::kStateStart) ||
+            (current_state == ash::AppListState::kStateStart &&
+             target_state == ash::AppListState::kStateSearchResults)) ||
+           ((current_state == ash::AppListState::kStateSearchResults &&
+             target_state == ash::AppListState::kStateApps) ||
+            (current_state == ash::AppListState::kStateApps &&
+             target_state == ash::AppListState::kStateSearchResults));
+  }
+
+  if (page == assistant_page_view_) {
+    return current_state == ash::AppListState::kStateEmbeddedAssistant ||
+           target_state == ash::AppListState::kStateEmbeddedAssistant;
+  }
+
+  return false;
+}
+
+gfx::Rect ContentsView::ConvertRectToWidgetWithoutTransform(
+    const gfx::Rect& rect) {
+  gfx::Rect widget_rect = rect;
+  for (const views::View* v = this; v; v = v->parent()) {
+    widget_rect.Offset(v->GetMirroredPosition().OffsetFromOrigin());
+  }
+  return widget_rect;
+}
+
 }  // namespace app_list
