@@ -8,11 +8,12 @@
 
 #include "ash/app_list/app_list_metrics.h"
 #include "ash/app_list/app_list_view_delegate.h"
+#include "ash/app_list/model/app_list_model.h"
 #include "ash/app_list/model/app_list_view_state.h"
 #include "ash/app_list/model/search/search_result.h"
 #include "ash/app_list/pagination_model.h"
-#include "ash/app_list/views/search_result_container_view.h"
-#include "ash/public/cpp/app_list/app_list_constants.h"
+#include "ash/app_list/views/app_list_item_view.h"
+#include "ash/public/cpp/app_list/app_list_config.h"
 #include "ash/public/cpp/app_list/app_list_features.h"
 #include "ash/public/cpp/app_list/vector_icons/vector_icons.h"
 #include "base/bind.h"
@@ -22,9 +23,8 @@
 #include "base/strings/utf_string_conversions.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/base/l10n/l10n_util.h"
-#include "ui/base/ui_base_features.h"
 #include "ui/gfx/canvas.h"
-#include "ui/gfx/image/canvas_image_source.h"
+#include "ui/gfx/color_palette.h"
 #include "ui/gfx/image/image_skia_operations.h"
 #include "ui/gfx/paint_vector_icon.h"
 #include "ui/strings/grit/ui_strings.h"
@@ -40,64 +40,46 @@ namespace {
 
 constexpr int kSearchTileWidth = 80;
 constexpr int kSearchTileTopPadding = 4;
-constexpr int kSearchTitleSpacing = 5;
+constexpr int kSearchTitleSpacing = 7;
 constexpr int kSearchPriceSize = 37;
 constexpr int kSearchRatingSize = 26;
 constexpr int kSearchRatingStarSize = 12;
 constexpr int kSearchRatingStarHorizontalSpacing = 1;
 constexpr int kSearchRatingStarVerticalSpacing = 2;
+// Text line height in the search result tile.
+constexpr int kTileTextLineHeight = 16;
+
+// Delta applied to the font size of SearchResultTile title.
+constexpr int kSearchResultTileTitleTextSizeDelta = 1;
 
 constexpr int kIconSelectedSize = 56;
 constexpr int kIconSelectedCornerRadius = 4;
-// Icon selected color, #000 8%.
-constexpr int kIconSelectedColor = SkColorSetARGB(0x14, 0x00, 0x00, 0x00);
+// Icon selected color, Google Grey 900 8%.
+constexpr int kIconSelectedColor = SkColorSetA(gfx::kGoogleGrey900, 0x14);
 
-constexpr SkColor kSearchTitleColor = SkColorSetARGB(0xDF, 0x00, 0x00, 0x00);
-constexpr SkColor kSearchAppRatingColor =
-    SkColorSetARGB(0x8F, 0x00, 0x00, 0x00);
-constexpr SkColor kSearchAppPriceColor = SkColorSetARGB(0xFF, 0x0F, 0x9D, 0x58);
-constexpr SkColor kSearchRatingStarColor =
-    SkColorSetARGB(0x8F, 0x00, 0x00, 0x00);
-
-// The background image source for badge.
-class BadgeBackgroundImageSource : public gfx::CanvasImageSource {
- public:
-  explicit BadgeBackgroundImageSource(int size)
-      : CanvasImageSource(gfx::Size(size, size), false),
-        radius_(static_cast<float>(size / 2)) {}
-  ~BadgeBackgroundImageSource() override = default;
-
- private:
-  // gfx::CanvasImageSource overrides:
-  void Draw(gfx::Canvas* canvas) override {
-    cc::PaintFlags flags;
-    flags.setColor(SK_ColorWHITE);
-    flags.setAntiAlias(true);
-    flags.setStyle(cc::PaintFlags::kFill_Style);
-    canvas->DrawCircle(gfx::PointF(radius_, radius_), radius_, flags);
-  }
-
-  const float radius_;
-
-  DISALLOW_COPY_AND_ASSIGN(BadgeBackgroundImageSource);
-};
+constexpr SkColor kSearchTitleColor = gfx::kGoogleGrey900;
+constexpr SkColor kSearchAppRatingColor = gfx::kGoogleGrey700;
+constexpr SkColor kSearchAppPriceColor = gfx::kGoogleGreen600;
+constexpr SkColor kSearchRatingStarColor = gfx::kGoogleGrey700;
 
 }  // namespace
 
 SearchResultTileItemView::SearchResultTileItemView(
-    SearchResultContainerView* result_container,
     AppListViewDelegate* view_delegate,
-    PaginationModel* pagination_model)
-    : result_container_(result_container),
-      view_delegate_(view_delegate),
+    PaginationModel* pagination_model,
+    bool show_in_apps_page)
+    : view_delegate_(view_delegate),
       pagination_model_(pagination_model),
       is_play_store_app_search_enabled_(
-          features::IsPlayStoreAppSearchEnabled()),
+          app_list_features::IsPlayStoreAppSearchEnabled()),
+      is_app_reinstall_recommendation_enabled_(
+          app_list_features::IsAppReinstallZeroStateEnabled()),
+      show_in_apps_page_(show_in_apps_page),
       weak_ptr_factory_(this) {
   SetFocusBehavior(FocusBehavior::ALWAYS);
 
-  // When |item_| is null, the tile is invisible. Calling SetSearchResult with a
-  // non-null item makes the tile visible.
+  // When |result_| is null, the tile is invisible. Calling SetSearchResult with
+  // a non-null item makes the tile visible.
   SetVisible(false);
 
   // Prevent the icon view from interfering with our mouse events.
@@ -106,7 +88,9 @@ SearchResultTileItemView::SearchResultTileItemView(
   icon_->SetVerticalAlignment(views::ImageView::LEADING);
   AddChildView(icon_);
 
-  if (is_play_store_app_search_enabled_) {
+  if (is_play_store_app_search_enabled_ ||
+      app_list_features::IsAppShortcutSearchEnabled() ||
+      is_app_reinstall_recommendation_enabled_) {
     badge_ = new views::ImageView;
     badge_->set_can_process_events_within_subtree(false);
     badge_->SetVerticalAlignment(views::ImageView::LEADING);
@@ -114,21 +98,20 @@ SearchResultTileItemView::SearchResultTileItemView(
     AddChildView(badge_);
   }
 
-  ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
   title_ = new views::Label;
   title_->SetAutoColorReadabilityEnabled(false);
-  title_->SetEnabledColor(kGridTitleColor);
-  title_->SetFontList(rb.GetFontList(kItemTextFontStyle));
+  title_->SetEnabledColor(AppListConfig::instance().grid_title_color());
+  title_->SetLineHeight(kTileTextLineHeight);
   title_->SetHorizontalAlignment(gfx::ALIGN_CENTER);
   title_->SetHandlesTooltips(false);
+  title_->SetAllowCharacterBreak(true);
   AddChildView(title_);
 
-  if (is_play_store_app_search_enabled_) {
-    const gfx::FontList& font = AppListAppTitleFont();
+  if (is_play_store_app_search_enabled_ ||
+      is_app_reinstall_recommendation_enabled_) {
     rating_ = new views::Label;
     rating_->SetEnabledColor(kSearchAppRatingColor);
-    rating_->SetFontList(font);
-    rating_->SetLineHeight(font.GetHeight());
+    rating_->SetLineHeight(kTileTextLineHeight);
     rating_->SetHorizontalAlignment(gfx::ALIGN_RIGHT);
     rating_->SetVisible(false);
     AddChildView(rating_);
@@ -137,14 +120,13 @@ SearchResultTileItemView::SearchResultTileItemView(
     rating_star_->set_can_process_events_within_subtree(false);
     rating_star_->SetVerticalAlignment(views::ImageView::LEADING);
     rating_star_->SetImage(gfx::CreateVectorIcon(
-        kIcBadgeRatingIcon, kSearchRatingStarSize, kSearchRatingStarColor));
+        kBadgeRatingIcon, kSearchRatingStarSize, kSearchRatingStarColor));
     rating_star_->SetVisible(false);
     AddChildView(rating_star_);
 
     price_ = new views::Label;
     price_->SetEnabledColor(kSearchAppPriceColor);
-    price_->SetFontList(font);
-    price_->SetLineHeight(font.GetHeight());
+    price_->SetLineHeight(kTileTextLineHeight);
     price_->SetHorizontalAlignment(gfx::ALIGN_LEFT);
     price_->SetVisible(false);
     AddChildView(price_);
@@ -154,64 +136,84 @@ SearchResultTileItemView::SearchResultTileItemView(
 }
 
 SearchResultTileItemView::~SearchResultTileItemView() {
-  if (item_)
-    item_->RemoveObserver(this);
+  ClearResult();
 }
 
-void SearchResultTileItemView::SetSearchResult(SearchResult* item) {
+void SearchResultTileItemView::OnResultChanged() {
   // Handle the case where this may be called from a nested run loop while its
   // context menu is showing. This cancels the menu (it's for the old item).
   context_menu_.reset();
 
-  SetVisible(!!item);
+  SetVisible(!!result());
 
-  SearchResult* old_item = item_;
-  if (old_item)
-    old_item->RemoveObserver(this);
-
-  item_ = item;
-
-  if (!item)
+  if (!result())
     return;
 
-  item_->AddObserver(this);
+  SetTitle(result()->title());
+  SetRating(result()->rating());
+  SetPrice(result()->formatted_price());
 
-  SetTitle(item_->title());
-  SetRating(item_->rating());
-  SetPrice(item_->formatted_price());
-
-  const gfx::FontList& font = AppListAppTitleFont();
-  if (IsSuggestedAppTile()) {
+  const gfx::FontList& font = AppListConfig::instance().app_title_font();
+  if (IsSuggestedAppTileShownInAppPage()) {
     title_->SetFontList(font);
-    title_->SetLineHeight(font.GetHeight());
-    title_->SetEnabledColor(kGridTitleColor);
+    title_->SetEnabledColor(AppListConfig::instance().grid_title_color());
   } else {
-    DCHECK_EQ(ash::SearchResultDisplayType::kTile, item_->display_type());
     // Set solid color background to avoid broken text. See crbug.com/746563.
     if (rating_) {
-      rating_->SetBackground(
-          views::CreateSolidBackground(kCardBackgroundColor));
+      rating_->SetBackground(views::CreateSolidBackground(
+          AppListConfig::instance().card_background_color()));
+      if (!IsSuggestedAppTile()) {
+        // App search results use different fonts than AppList apps.
+        rating_->SetFontList(
+            ui::ResourceBundle::GetSharedInstance().GetFontList(
+                AppListConfig::instance().search_result_title_font_style()));
+      } else {
+        rating_->SetFontList(font);
+      }
     }
     if (price_) {
-      price_->SetBackground(views::CreateSolidBackground(kCardBackgroundColor));
+      price_->SetBackground(views::CreateSolidBackground(
+          AppListConfig::instance().card_background_color()));
+      if (!IsSuggestedAppTile()) {
+        // App search results use different fonts than AppList apps.
+        price_->SetFontList(ui::ResourceBundle::GetSharedInstance().GetFontList(
+            AppListConfig::instance().search_result_title_font_style()));
+      } else {
+        price_->SetFontList(font);
+      }
+    }
+    title_->SetBackground(views::CreateSolidBackground(
+        AppListConfig::instance().card_background_color()));
+    if (!IsSuggestedAppTile()) {
+      // App search results use different fonts than AppList apps.
+      title_->SetFontList(
+          ui::ResourceBundle::GetSharedInstance()
+              .GetFontList(
+                  AppListConfig::instance().search_result_title_font_style())
+              .DeriveWithSizeDelta(kSearchResultTileTitleTextSizeDelta));
+    } else {
+      title_->SetFontList(font);
     }
-    title_->SetBackground(views::CreateSolidBackground(kCardBackgroundColor));
-    title_->SetFontList(font);
-    title_->SetLineHeight(font.GetHeight());
     title_->SetEnabledColor(kSearchTitleColor);
   }
 
   title_->SetMaxLines(2);
   title_->SetMultiLine(
-      item_->display_type() == ash::SearchResultDisplayType::kTile &&
-      item_->result_type() == ash::SearchResultType::kInstalledApp);
+      (result()->display_type() == ash::SearchResultDisplayType::kTile ||
+       (IsSuggestedAppTile() && !show_in_apps_page_)) &&
+      result()->result_type() == ash::SearchResultType::kInstalledApp);
 
   // If the new icon is null, it's being decoded asynchronously. Not updating it
   // now to prevent flickering from showing an empty icon while decoding.
-  if (!item->icon().isNull())
+  if (!result()->icon().isNull())
     OnMetadataChanged();
 
-  base::string16 accessible_name = title_->text();
+  base::string16 accessible_name;
+  if (!result()->accessible_name().empty())
+    accessible_name = result()->accessible_name();
+  else
+    accessible_name = title_->text();
+
   if (rating_ && rating_->visible()) {
     accessible_name +=
         base::UTF8ToUTF16(", ") +
@@ -233,9 +235,9 @@ void SearchResultTileItemView::ButtonPressed(views::Button* sender,
   if (IsSuggestedAppTile())
     LogAppLaunch();
 
-  RecordSearchResultOpenSource(item_, view_delegate_->GetModel(),
+  RecordSearchResultOpenSource(result(), view_delegate_->GetModel(),
                                view_delegate_->GetSearchModel());
-  view_delegate_->OpenSearchResult(item_->id(), event.flags());
+  view_delegate_->OpenSearchResult(result()->id(), event.flags());
 }
 
 void SearchResultTileItemView::GetAccessibleNodeData(
@@ -257,18 +259,17 @@ void SearchResultTileItemView::GetAccessibleNodeData(
 }
 
 bool SearchResultTileItemView::OnKeyPressed(const ui::KeyEvent& event) {
-  // Return early if |item_| was deleted due to the search result list changing.
-  // see crbug.com/801142
-  if (!item_)
+  // Return early if |result()| was deleted due to the search result list
+  // changing. see crbug.com/801142
+  if (!result())
     return true;
 
   if (event.key_code() == ui::VKEY_RETURN) {
     if (IsSuggestedAppTile())
       LogAppLaunch();
-
-    RecordSearchResultOpenSource(item_, view_delegate_->GetModel(),
+    RecordSearchResultOpenSource(result(), view_delegate_->GetModel(),
                                  view_delegate_->GetSearchModel());
-    view_delegate_->OpenSearchResult(item_->id(), event.flags());
+    view_delegate_->OpenSearchResult(result()->id(), event.flags());
     return true;
   }
 
@@ -280,12 +281,11 @@ void SearchResultTileItemView::OnFocus() {
       view_delegate_->GetModel()->state() == ash::AppListState::kStateApps) {
     // Go back to first page when app in suggestions container is focused.
     pagination_model_->SelectPage(0, false);
-  } else if (!IsSuggestedAppTile()) {
+  } else {
     ScrollRectToVisible(GetLocalBounds());
   }
   SetBackgroundHighlighted(true);
   UpdateBackgroundColor();
-  NotifyAccessibilityEvent(ax::mojom::Event::kSelection, true);
 }
 
 void SearchResultTileItemView::OnBlur() {
@@ -298,19 +298,20 @@ void SearchResultTileItemView::StateChanged(ButtonState old_state) {
 }
 
 void SearchResultTileItemView::PaintButtonContents(gfx::Canvas* canvas) {
-  if (!item_ || !background_highlighted())
+  if (!result() || !background_highlighted())
     return;
 
   gfx::Rect rect(GetContentsBounds());
   cc::PaintFlags flags;
   flags.setAntiAlias(true);
   flags.setStyle(cc::PaintFlags::kFill_Style);
-  if (IsSuggestedAppTile()) {
-    rect.ClampToCenteredSize(gfx::Size(kGridSelectedSize, kGridSelectedSize));
-    flags.setColor(kGridSelectedColor);
-    canvas->DrawRoundRect(gfx::RectF(rect), kGridSelectedCornerRadius, flags);
+  if (IsSuggestedAppTileShownInAppPage()) {
+    rect.ClampToCenteredSize(AppListConfig::instance().grid_focus_size());
+    flags.setColor(AppListConfig::instance().grid_selected_color());
+    canvas->DrawRoundRect(gfx::RectF(rect),
+                          AppListConfig::instance().grid_focus_corner_radius(),
+                          flags);
   } else {
-    DCHECK(item_->display_type() == ash::SearchResultDisplayType::kTile);
     const int kLeftRightPadding = (rect.width() - kIconSelectedSize) / 2;
     rect.Inset(kLeftRightPadding, 0);
     rect.set_height(kIconSelectedSize);
@@ -320,33 +321,24 @@ void SearchResultTileItemView::PaintButtonContents(gfx::Canvas* canvas) {
 }
 
 void SearchResultTileItemView::OnMetadataChanged() {
-  SetIcon(item_->icon());
-  SetBadgeIcon(item_->badge_icon());
-  SetRating(item_->rating());
-  SetPrice(item_->formatted_price());
+  SetIcon(result()->icon());
+  SetTitle(result()->title());
+  SetBadgeIcon(result()->badge_icon());
+  SetRating(result()->rating());
+  SetPrice(result()->formatted_price());
   Layout();
 }
 
-void SearchResultTileItemView::OnResultDestroying() {
-  // The menu comes from |item_|. If we're showing a menu we need to cancel it.
-  context_menu_.reset();
-
-  if (item_)
-    item_->RemoveObserver(this);
-
-  SetSearchResult(nullptr);
-}
-
 void SearchResultTileItemView::ShowContextMenuForView(
     views::View* source,
     const gfx::Point& point,
     ui::MenuSourceType source_type) {
-  // |item_| could be null when result list is changing.
-  if (!item_)
+  // |result()| could be null when result list is changing.
+  if (!result())
     return;
 
   view_delegate_->GetSearchResultContextMenuModel(
-      item_->id(),
+      result()->id(),
       base::BindOnce(&SearchResultTileItemView::OnGetContextMenuModel,
                      weak_ptr_factory_.GetWeakPtr(), source, point,
                      source_type));
@@ -360,31 +352,20 @@ void SearchResultTileItemView::OnGetContextMenuModel(
   if (menu.empty() || (context_menu_ && context_menu_->IsShowingMenu()))
     return;
 
-  // TODO(warx): This is broken (https://crbug.com/795994).
-  if (!HasFocus())
-    result_container_->ClearSelectedIndex();
-
-  int run_types = views::MenuRunner::HAS_MNEMONICS;
-  views::MenuAnchorPosition anchor_position = views::MENU_ANCHOR_TOPLEFT;
-  gfx::Rect anchor_rect = gfx::Rect(point, gfx::Size());
-
-  if (::features::IsTouchableAppContextMenuEnabled()) {
-    anchor_position = views::MENU_ANCHOR_BUBBLE_TOUCHABLE_RIGHT;
-    run_types |= views::MenuRunner::USE_TOUCHABLE_LAYOUT |
-                 views::MenuRunner::CONTEXT_MENU |
-                 views::MenuRunner::FIXED_ANCHOR;
-    anchor_rect = source->GetBoundsInScreen();
-    // Anchor the menu to the same rect that is used for selection highlight.
-    anchor_rect.ClampToCenteredSize(
-        gfx::Size(kGridSelectedSize, kGridSelectedSize));
-  }
+  gfx::Rect anchor_rect = source->GetBoundsInScreen();
+  // Anchor the menu to the same rect that is used for selection highlight.
+  anchor_rect.ClampToCenteredSize(AppListConfig::instance().grid_focus_size());
 
   context_menu_ = std::make_unique<AppListMenuModelAdapter>(
-      item_->id(), this, source_type, this, GetAppType(),
+      result()->id(), this, source_type, this, GetAppType(),
       base::BindOnce(&SearchResultTileItemView::OnMenuClosed,
                      weak_ptr_factory_.GetWeakPtr()));
   context_menu_->Build(std::move(menu));
-  context_menu_->Run(anchor_rect, anchor_position, run_types);
+  context_menu_->Run(anchor_rect, views::MENU_ANCHOR_BUBBLE_TOUCHABLE_RIGHT,
+                     views::MenuRunner::HAS_MNEMONICS |
+                         views::MenuRunner::USE_TOUCHABLE_LAYOUT |
+                         views::MenuRunner::CONTEXT_MENU |
+                         views::MenuRunner::FIXED_ANCHOR);
   source->RequestFocus();
 }
 
@@ -393,16 +374,17 @@ void SearchResultTileItemView::OnMenuClosed() {
 }
 
 void SearchResultTileItemView::ExecuteCommand(int command_id, int event_flags) {
-  if (item_) {
-    view_delegate_->SearchResultContextMenuItemSelected(item_->id(), command_id,
-                                                        event_flags);
+  if (result()) {
+    view_delegate_->SearchResultContextMenuItemSelected(
+        result()->id(), command_id, event_flags);
   }
 }
 
 void SearchResultTileItemView::SetIcon(const gfx::ImageSkia& icon) {
+  const int icon_size = AppListConfig::instance().search_tile_icon_dimension();
   gfx::ImageSkia resized(gfx::ImageSkiaOperations::CreateResizedImage(
       icon, skia::ImageOperations::RESIZE_BEST,
-      gfx::Size(kTileIconSize, kTileIconSize)));
+      gfx::Size(icon_size, icon_size)));
   icon_->SetImage(resized);
 }
 
@@ -415,11 +397,10 @@ void SearchResultTileItemView::SetBadgeIcon(const gfx::ImageSkia& badge_icon) {
     return;
   }
 
-  const int size = kBadgeBackgroundRadius * 2;
-  gfx::ImageSkia background(std::make_unique<BadgeBackgroundImageSource>(size),
-                            gfx::Size(size, size));
-  gfx::ImageSkia icon_with_background =
-      gfx::ImageSkiaOperations::CreateSuperimposedImage(background, badge_icon);
+  gfx::ImageSkia resized_badge_icon(
+      gfx::ImageSkiaOperations::CreateResizedImage(
+          badge_icon, skia::ImageOperations::RESIZE_BEST,
+          AppListConfig::instance().search_tile_badge_icon_size()));
 
   gfx::ShadowValues shadow_values;
   shadow_values.push_back(
@@ -427,12 +408,13 @@ void SearchResultTileItemView::SetBadgeIcon(const gfx::ImageSkia& badge_icon) {
   shadow_values.push_back(
       gfx::ShadowValue(gfx::Vector2d(0, 1), 2, SkColorSetARGB(0x33, 0, 0, 0)));
   badge_->SetImage(gfx::ImageSkiaOperations::CreateImageWithDropShadow(
-      icon_with_background, shadow_values));
+      resized_badge_icon, shadow_values));
   badge_->SetVisible(true);
 }
 
 void SearchResultTileItemView::SetTitle(const base::string16& title) {
   title_->SetText(title);
+  title_->NotifyAccessibilityEvent(ax::mojom::Event::kTextChanged, true);
 }
 
 void SearchResultTileItemView::SetRating(float rating) {
@@ -486,8 +468,12 @@ SearchResultTileItemView::GetAppType() const {
 }
 
 bool SearchResultTileItemView::IsSuggestedAppTile() const {
-  return item_ &&
-         item_->display_type() == ash::SearchResultDisplayType::kRecommendation;
+  return result() && result()->display_type() ==
+                         ash::SearchResultDisplayType::kRecommendation;
+}
+
+bool SearchResultTileItemView::IsSuggestedAppTileShownInAppPage() const {
+  return IsSuggestedAppTile() && show_in_apps_page_;
 }
 
 void SearchResultTileItemView::LogAppLaunch() const {
@@ -510,33 +496,33 @@ void SearchResultTileItemView::UpdateBackgroundColor() {
 
 void SearchResultTileItemView::Layout() {
   gfx::Rect rect(GetContentsBounds());
-  if (rect.IsEmpty() || !item_)
+  if (rect.IsEmpty() || !result())
     return;
 
-  if (IsSuggestedAppTile()) {
-    rect.Inset(0, kGridIconTopPadding, 0, 0);
-    icon_->SetBoundsRect(rect);
-
-    rect.Inset(kGridTitleHorizontalPadding,
-               kGridIconDimension + kGridTitleSpacing,
-               kGridTitleHorizontalPadding, 0);
-    rect.set_height(title_->GetPreferredSize().height());
-    title_->SetBoundsRect(rect);
+  if (IsSuggestedAppTileShownInAppPage()) {
+    icon_->SetBoundsRect(AppListItemView::GetIconBoundsForTargetViewBounds(
+        rect, icon_->GetImage().size()));
+    title_->SetBoundsRect(AppListItemView::GetTitleBoundsForTargetViewBounds(
+        rect, title_->GetPreferredSize()));
   } else {
-    DCHECK(item_->display_type() == ash::SearchResultDisplayType::kTile);
-    rect.Inset(0, kSearchTileTopPadding, 0, 0);
-    icon_->SetBoundsRect(rect);
+    gfx::Rect icon_rect(rect);
+    icon_rect.ClampToCenteredSize(icon_->GetImage().size());
+    icon_rect.set_y(kSearchTileTopPadding);
+    icon_->SetBoundsRect(icon_rect);
 
     if (badge_) {
-      gfx::Rect badge_rect(rect);
-      gfx::Size icon_size = icon_->GetImage().size();
-      badge_rect.Offset(
-          (icon_size.width() - kAppBadgeIconSize) / 2,
-          icon_size.height() - kBadgeBackgroundRadius - kAppBadgeIconSize / 2);
+      const int badge_icon_dimension =
+          AppListConfig::instance().search_tile_badge_icon_dimension();
+      const int badge_icon_offset =
+          AppListConfig::instance().search_tile_badge_icon_offset();
+      const gfx::Rect badge_rect(
+          icon_rect.right() - badge_icon_dimension + badge_icon_offset,
+          icon_rect.bottom() - badge_icon_dimension + badge_icon_offset,
+          badge_icon_dimension, badge_icon_dimension);
       badge_->SetBoundsRect(badge_rect);
     }
 
-    rect.Inset(0, kGridIconDimension + kSearchTitleSpacing, 0, 0);
+    rect.set_y(icon_rect.bottom() + kSearchTitleSpacing);
     rect.set_height(title_->GetPreferredSize().height());
     title_->SetBoundsRect(rect);
 
@@ -574,14 +560,16 @@ const char* SearchResultTileItemView::GetClassName() const {
 }
 
 gfx::Size SearchResultTileItemView::CalculatePreferredSize() const {
-  if (!item_)
+  if (!result())
     return gfx::Size();
 
-  if (IsSuggestedAppTile())
-    return gfx::Size(kGridTileWidth, kGridTileHeight);
+  if (IsSuggestedAppTileShownInAppPage()) {
+    return gfx::Size(AppListConfig::instance().grid_tile_width(),
+                     AppListConfig::instance().grid_tile_height());
+  }
 
-  DCHECK(item_->display_type() == ash::SearchResultDisplayType::kTile);
-  return gfx::Size(kSearchTileWidth, kSearchTileHeight);
+  return gfx::Size(kSearchTileWidth,
+                   AppListConfig::instance().search_tile_height());
 }
 
 bool SearchResultTileItemView::GetTooltipText(const gfx::Point& p,
