@@ -4,16 +4,23 @@
 
 #include "ash/display/cursor_window_controller.h"
 
-#include "ash/ash_constants.h"
+#include "ash/components/cursor/cursor_view.h"
+#include "ash/display/display_color_manager.h"
 #include "ash/display/mirror_window_controller.h"
 #include "ash/display/window_tree_host_manager.h"
+#include "ash/magnifier/magnification_controller.h"
+#include "ash/public/cpp/ash_constants.h"
 #include "ash/public/cpp/ash_pref_names.h"
+#include "ash/public/cpp/ash_switches.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/root_window_controller.h"
 #include "ash/session/session_controller.h"
 #include "ash/shell.h"
+#include "ash/window_factory.h"
+#include "base/command_line.h"
+#include "base/metrics/histogram_macros.h"
 #include "components/prefs/pref_service.h"
-#include "services/ui/public/interfaces/window_tree_constants.mojom.h"
+#include "services/ws/public/mojom/window_tree_constants.mojom.h"
 #include "ui/aura/env.h"
 #include "ui/aura/window_delegate.h"
 #include "ui/aura/window_event_dispatcher.h"
@@ -29,6 +36,7 @@
 #include "ui/gfx/geometry/dip_util.h"
 #include "ui/gfx/image/image_skia.h"
 #include "ui/gfx/image/image_skia_operations.h"
+#include "ui/views/widget/widget.h"
 
 namespace ash {
 namespace {
@@ -72,7 +80,7 @@ class CursorWindowDelegate : public aura::WindowDelegate {
   void OnWindowDestroyed(aura::Window* window) override {}
   void OnWindowTargetVisibilityChanged(bool visible) override {}
   bool HasHitTestMask() const override { return false; }
-  void GetHitTestMask(gfx::Path* mask) const override {}
+  void GetHitTestMask(SkPath* mask) const override {}
 
   // Sets the cursor image for the |display|'s scale factor.
   void SetCursorImage(const gfx::Size& size, const gfx::ImageSkia& image) {
@@ -91,7 +99,10 @@ class CursorWindowDelegate : public aura::WindowDelegate {
 };
 
 CursorWindowController::CursorWindowController()
-    : delegate_(new CursorWindowDelegate()) {}
+    : delegate_(new CursorWindowDelegate()),
+      is_cursor_motion_blur_enabled_(
+          base::CommandLine::ForCurrentProcess()->HasSwitch(
+              switches::kAshEnableCursorMotionBlur)) {}
 
 CursorWindowController::~CursorWindowController() {
   SetContainer(NULL);
@@ -114,28 +125,48 @@ void CursorWindowController::SetLargeCursorSizeInDip(
 }
 
 bool CursorWindowController::ShouldEnableCursorCompositing() {
+  if (is_cursor_motion_blur_enabled_)
+    return true;
+
   // During startup, we may not have a preference service yet. We need to check
   // display manager state first so that we don't accidentally ignore it while
   // early outing when there isn't a PrefService yet.
-  display::DisplayManager* display_manager = Shell::Get()->display_manager();
-  if ((display_manager->is_multi_mirroring_enabled() &&
-       display_manager->IsInSoftwareMirrorMode()) ||
+  Shell* shell = Shell::Get();
+  display::DisplayManager* display_manager = shell->display_manager();
+  if ((display_manager->IsInSoftwareMirrorMode()) ||
       display_manager->IsInUnifiedMode() ||
       display_manager->screen_capture_is_active()) {
     return true;
   }
 
-  PrefService* prefs =
-      Shell::Get()->session_controller()->GetActivePrefService();
+  if (shell->magnification_controller()->IsEnabled())
+    return true;
+
+  PrefService* prefs = shell->session_controller()->GetActivePrefService();
   if (!prefs) {
     // The active pref service can be null early in startup.
     return false;
   }
+
+  if (prefs->GetBoolean(prefs::kNightLightEnabled)) {
+    // All or some displays don't support setting a CRTC matrix, which means
+    // Night Light is using the composited color matrix, and hence software
+    // cursor should be used.
+    // TODO(afakhry): Instead of switching to the composited cursor on all
+    // displays if any of them don't support a CRTC matrix, we should provide
+    // the functionality to turn on the composited cursor on a per-display basis
+    // (i.e. use it only on the displays that don't support CRTC matrices).
+    const DisplayColorManager::DisplayCtmSupport displays_ctm_support =
+        shell->display_color_manager()->displays_ctm_support();
+    UMA_HISTOGRAM_ENUMERATION("Ash.NightLight.DisplayCrtcCtmSupport",
+                              displays_ctm_support);
+    if (displays_ctm_support != DisplayColorManager::DisplayCtmSupport::kAll)
+      return true;
+  }
+
   return prefs->GetBoolean(prefs::kAccessibilityLargeCursorEnabled) ||
          prefs->GetBoolean(prefs::kAccessibilityHighContrastEnabled) ||
-         prefs->GetBoolean(prefs::kAccessibilityScreenMagnifierEnabled) ||
-         prefs->GetBoolean(prefs::kDockedMagnifierEnabled) ||
-         prefs->GetBoolean(prefs::kNightLightEnabled);
+         prefs->GetBoolean(prefs::kDockedMagnifierEnabled);
 }
 
 void CursorWindowController::SetCursorCompositingEnabled(bool enabled) {
@@ -156,16 +187,7 @@ void CursorWindowController::UpdateContainer() {
     if (display.is_valid())
       SetDisplay(display);
   } else {
-    aura::Window::Windows mirror_windows = Shell::Get()
-                                               ->window_tree_host_manager()
-                                               ->mirror_window_controller()
-                                               ->GetAllRootWindows();
-    if (mirror_windows.empty()) {
-      SetContainer(nullptr);
-      return;
-    }
-    display_ = display::Screen::GetScreen()->GetPrimaryDisplay();
-    SetContainer(mirror_windows[0]);
+    SetContainer(nullptr);
   }
   // Updates the hot point based on the current display.
   UpdateCursorImage();
@@ -189,7 +211,7 @@ void CursorWindowController::SetDisplay(const display::Display& display) {
 
   SetContainer(RootWindowController::ForWindow(root_window)
                    ->GetContainer(kShellWindowId_MouseCursorContainer));
-  SetBoundsInScreen(display.bounds());
+  SetBoundsInScreenAndRotation(display.bounds(), display.rotation());
   // Updates the hot point based on the current display.
   UpdateCursorImage();
 }
@@ -197,12 +219,8 @@ void CursorWindowController::SetDisplay(const display::Display& display) {
 void CursorWindowController::UpdateLocation() {
   if (!cursor_window_)
     return;
-  gfx::Point point = aura::Env::GetInstance()->last_mouse_location();
-  if (!is_cursor_compositing_enabled_) {
-    Shell::GetPrimaryRootWindow()->GetHost()->ConvertDIPToPixels(&point);
-  } else {
-    point.Offset(-bounds_in_screen_.x(), -bounds_in_screen_.y());
-  }
+  gfx::Point point = Shell::Get()->aura_env()->last_mouse_location();
+  point.Offset(-bounds_in_screen_.x(), -bounds_in_screen_.y());
   point.Offset(-hot_point_.x(), -hot_point_.y());
   gfx::Rect bounds = cursor_window_->bounds();
   bounds.set_origin(point);
@@ -210,9 +228,9 @@ void CursorWindowController::UpdateLocation() {
 }
 
 void CursorWindowController::SetCursor(gfx::NativeCursor cursor) {
-  if (cursor_type_ == cursor.native_type())
+  if (cursor_ == cursor)
     return;
-  cursor_type_ = cursor.native_type();
+  cursor_ = cursor;
   UpdateCursorImage();
   UpdateCursorVisibility();
 }
@@ -233,123 +251,129 @@ void CursorWindowController::SetContainer(aura::Window* container) {
   container_ = container;
   if (!container) {
     cursor_window_.reset();
+    cursor_view_.reset();
     return;
   }
 
-  // Reusing the window does not work when the display is disconnected.
-  // Just creates a new one instead. crbug.com/384218.
-  cursor_window_.reset(new aura::Window(delegate_.get()));
-  cursor_window_->SetTransparent(true);
-  cursor_window_->Init(ui::LAYER_TEXTURED);
-  cursor_window_->SetEventTargetingPolicy(
-      ui::mojom::EventTargetingPolicy::NONE);
-  cursor_window_->set_owned_by_parent(false);
-  // Call UpdateCursorImage() to figure out |cursor_window_|'s desired size.
-  UpdateCursorImage();
+  bounds_in_screen_ = display_.bounds();
+  rotation_ = display_.rotation();
 
-  container->AddChild(cursor_window_.get());
+  if (is_cursor_motion_blur_enabled_) {
+    UpdateCursorView();
+  } else {
+    // Reusing the window does not work when the display is disconnected.
+    // Just creates a new one instead. crbug.com/384218.
+    cursor_window_ = window_factory::NewWindow(delegate_.get());
+    cursor_window_->SetTransparent(true);
+    cursor_window_->Init(ui::LAYER_TEXTURED);
+    cursor_window_->SetEventTargetingPolicy(
+        ws::mojom::EventTargetingPolicy::NONE);
+    cursor_window_->set_owned_by_parent(false);
+    // Call UpdateCursorImage() to figure out |cursor_window_|'s desired size.
+    UpdateCursorImage();
+    container->AddChild(cursor_window_.get());
+  }
   UpdateCursorVisibility();
-  SetBoundsInScreen(container->bounds());
+  UpdateLocation();
 }
 
-void CursorWindowController::SetBoundsInScreen(const gfx::Rect& bounds) {
+void CursorWindowController::SetBoundsInScreenAndRotation(
+    const gfx::Rect& bounds,
+    display::Display::Rotation rotation) {
+  if (bounds == bounds_in_screen_ && rotation == rotation_)
+    return;
   bounds_in_screen_ = bounds;
+  rotation_ = rotation;
+  if (cursor_view_)
+    UpdateCursorView();
   UpdateLocation();
 }
 
 void CursorWindowController::UpdateCursorImage() {
-  float cursor_scale;
-  if (!is_cursor_compositing_enabled_) {
-    cursor_scale = display_.device_scale_factor();
-  } else {
-    // Use the original device scale factor instead of the display's, which
-    // might have been adjusted for the UI scale.
-    const float original_scale = Shell::Get()
-                                     ->display_manager()
-                                     ->GetDisplayInfo(display_.id())
-                                     .device_scale_factor();
-    // And use the nearest resource scale factor.
-    cursor_scale =
-        ui::GetScaleForScaleFactor(ui::GetSupportedScaleFactor(original_scale));
-  }
-  int resource_id;
-  // TODO(hshi): support custom cursor set.
-  if (!ui::GetCursorDataFor(cursor_size_, cursor_type_, cursor_scale,
-                            &resource_id, &hot_point_)) {
+  if (!is_cursor_compositing_enabled_)
     return;
-  }
-  const gfx::ImageSkia* image =
-      ui::ResourceBundle::GetSharedInstance().GetImageSkiaNamed(resource_id);
-  if (!is_cursor_compositing_enabled_) {
-    gfx::ImageSkia rotated = *image;
-    switch (display_.rotation()) {
-      case display::Display::ROTATE_0:
-        break;
-      case display::Display::ROTATE_90:
-        rotated = gfx::ImageSkiaOperations::CreateRotatedImage(
-            *image, SkBitmapOperations::ROTATION_90_CW);
-        hot_point_.SetPoint(rotated.width() - hot_point_.y(), hot_point_.x());
-        break;
-      case display::Display::ROTATE_180:
-        rotated = gfx::ImageSkiaOperations::CreateRotatedImage(
-            *image, SkBitmapOperations::ROTATION_180_CW);
-        hot_point_.SetPoint(rotated.height() - hot_point_.x(),
-                            rotated.width() - hot_point_.y());
-        break;
-      case display::Display::ROTATE_270:
-        rotated = gfx::ImageSkiaOperations::CreateRotatedImage(
-            *image, SkBitmapOperations::ROTATION_270_CW);
-        hot_point_.SetPoint(hot_point_.y(), rotated.height() - hot_point_.x());
-        break;
-    }
-    // Note that mirror window's scale factor is always 1.0f, therefore we
-    // need to take 2x's image and paint as if it's 1x image.
-    const gfx::ImageSkiaRep& image_rep =
-        rotated.GetRepresentation(cursor_scale);
-    delegate_->SetCursorImage(
-        image_rep.pixel_size(),
-        gfx::ImageSkia::CreateFrom1xBitmap(image_rep.sk_bitmap()));
+
+  // Use the original device scale factor instead of the display's, which
+  // might have been adjusted for the UI scale.
+  const float original_scale = Shell::Get()
+                                   ->display_manager()
+                                   ->GetDisplayInfo(display_.id())
+                                   .device_scale_factor();
+  // And use the nearest resource scale factor.
+  float cursor_scale =
+      ui::GetScaleForScaleFactor(ui::GetSupportedScaleFactor(original_scale));
+
+  gfx::ImageSkia image;
+  if (cursor_.native_type() == ui::CursorType::kCustom) {
+    SkBitmap bitmap = cursor_.GetBitmap();
+    if (bitmap.isNull())
+      return;
+    image = gfx::ImageSkia::CreateFrom1xBitmap(bitmap);
+    hot_point_ = cursor_.GetHotspot();
   } else {
-    gfx::ImageSkia resized = *image;
-
-    // Rescale cursor size. This is used with the combination of accessibility
-    // large cursor. We don't need to care about the case where cursor
-    // compositing is disabled as we always use cursor compositing if
-    // accessibility large cursor is enabled.
-    if (cursor_size_ == ui::CursorSize::kLarge &&
-        large_cursor_size_in_dip_ != image->size().width()) {
-      float rescale = static_cast<float>(large_cursor_size_in_dip_) /
-                      static_cast<float>(image->size().width());
-      resized = gfx::ImageSkiaOperations::CreateResizedImage(
-          *image, skia::ImageOperations::ResizeMethod::RESIZE_GOOD,
-          gfx::ScaleToCeiledSize(image->size(), rescale));
-      hot_point_ = gfx::ScaleToCeiledPoint(hot_point_, rescale);
+    int resource_id;
+    if (!ui::GetCursorDataFor(cursor_size_, cursor_.native_type(), cursor_scale,
+                              &resource_id, &hot_point_)) {
+      return;
     }
-
-    const gfx::ImageSkiaRep& image_rep =
-        resized.GetRepresentation(cursor_scale);
-    delegate_->SetCursorImage(
-        resized.size(),
-        gfx::ImageSkia(gfx::ImageSkiaRep(image_rep.sk_bitmap(), cursor_scale)));
-    hot_point_ = gfx::ConvertPointToDIP(cursor_scale, hot_point_);
+    image =
+        *ui::ResourceBundle::GetSharedInstance().GetImageSkiaNamed(resource_id);
   }
 
+  gfx::ImageSkia resized = image;
+
+  // Rescale cursor size. This is used with the combination of accessibility
+  // large cursor. We don't need to care about the case where cursor
+  // compositing is disabled as we always use cursor compositing if
+  // accessibility large cursor is enabled.
+  if (cursor_size_ == ui::CursorSize::kLarge &&
+      large_cursor_size_in_dip_ != image.size().width()) {
+    float rescale = static_cast<float>(large_cursor_size_in_dip_) /
+                    static_cast<float>(image.size().width());
+    resized = gfx::ImageSkiaOperations::CreateResizedImage(
+        image, skia::ImageOperations::ResizeMethod::RESIZE_GOOD,
+        gfx::ScaleToCeiledSize(image.size(), rescale));
+    hot_point_ = gfx::ScaleToCeiledPoint(hot_point_, rescale);
+  }
+
+  const gfx::ImageSkiaRep& image_rep = resized.GetRepresentation(cursor_scale);
+  delegate_->SetCursorImage(
+      resized.size(),
+      gfx::ImageSkia(gfx::ImageSkiaRep(image_rep.GetBitmap(), cursor_scale)));
+  hot_point_ = gfx::ConvertPointToDIP(cursor_scale, hot_point_);
+
+  if (cursor_view_) {
+    cursor_view_->SetCursorImage(delegate_->cursor_image(), delegate_->size(),
+                                 hot_point_);
+  }
   if (cursor_window_) {
     cursor_window_->SetBounds(gfx::Rect(delegate_->size()));
     cursor_window_->SchedulePaintInRect(
         gfx::Rect(cursor_window_->bounds().size()));
-    UpdateLocation();
   }
+  UpdateLocation();
 }
 
 void CursorWindowController::UpdateCursorVisibility() {
-  if (!cursor_window_)
-    return;
-  bool visible = (visible_ && cursor_type_ != ui::CursorType::kNone);
-  if (visible)
-    cursor_window_->Show();
-  else
-    cursor_window_->Hide();
+  bool visible = (visible_ && cursor_.native_type() != ui::CursorType::kNone);
+  if (visible) {
+    if (cursor_view_)
+      cursor_view_->GetWidget()->Show();
+    if (cursor_window_)
+      cursor_window_->Show();
+  } else {
+    if (cursor_view_)
+      cursor_view_->GetWidget()->Hide();
+    if (cursor_window_)
+      cursor_window_->Hide();
+  }
+}
+
+void CursorWindowController::UpdateCursorView() {
+  cursor_view_.reset(new cursor::CursorView(
+      container_, Shell::Get()->aura_env()->last_mouse_location(),
+      is_cursor_motion_blur_enabled_));
+  UpdateCursorImage();
 }
 
 const gfx::ImageSkia& CursorWindowController::GetCursorImageForTest() const {
