@@ -13,8 +13,12 @@
 #include "ash/public/cpp/scale_utility.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/shell.h"
+#include "ash/shell_state.h"
 #include "ash/system/palette/palette_utils.h"
+#include "base/bind.h"
 #include "base/metrics/histogram_macros.h"
+#include "base/timer/timer.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_tree_host.h"
 #include "ui/events/base_event_utils.h"
@@ -91,6 +95,11 @@ void HighlighterController::UpdateEnabledState(
     observer.OnHighlighterEnabledChanged(enabled_state);
 }
 
+void HighlighterController::AbortSession() {
+  if (enabled_state_ == HighlighterEnabledState::kEnabled)
+    UpdateEnabledState(HighlighterEnabledState::kDisabledBySessionAbort);
+}
+
 void HighlighterController::BindRequest(
     mojom::HighlighterControllerRequest request) {
   binding_.Bind(std::move(request));
@@ -217,8 +226,21 @@ void HighlighterController::RecognizeGesture() {
 
   if (!box.IsEmpty() &&
       gesture_type != HighlighterGestureType::kNotRecognized) {
-    const gfx::Rect selection_rect = gfx::ToEnclosingRect(
-        gfx::ScaleRect(box, GetScreenshotScale(current_window)));
+    // The window for selection should be the root window to show assistant.
+    Shell::Get()->shell_state()->SetRootWindowForNewWindows(
+        current_window->GetRootWindow());
+
+    // TODO(muyuanli): Delete the check when native assistant is default on.
+    // This is a temporary workaround to support both ARC-based assistant
+    // and native assistant. In ARC-based assistant, we send the rect in pixels
+    // to ARC side, where the app will crop the screenshot. In native assistant,
+    // we pass the rect directly to UI snapshot API, which assumes coordinates
+    // in DP.
+    const gfx::Rect selection_rect =
+        chromeos::switches::IsAssistantEnabled()
+            ? gfx::ToEnclosingRect(box)
+            : gfx::ToEnclosingRect(
+                  gfx::ScaleRect(box, GetScreenshotScale(current_window)));
     if (client_)
       client_->HandleSelection(selection_rect);
 
