@@ -10,31 +10,30 @@
 #include "ash/accessibility/accessibility_controller.h"
 #include "ash/ime/ime_controller.h"
 #include "ash/ime/test_ime_controller_client.h"
-#include "ash/public/cpp/config.h"
+#include "ash/keyboard/ash_keyboard_controller.h"
 #include "ash/shell.h"
 #include "ash/system/tray/system_tray_notifier.h"
 #include "ash/system/virtual_keyboard/virtual_keyboard_observer.h"
 #include "ash/test/ash_test_base.h"
-#include "ash/wm/tablet_mode/scoped_disable_internal_mouse_and_keyboard.h"
+#include "ash/wm/tablet_mode/internal_input_devices_event_blocker.h"
 #include "ash/wm/tablet_mode/tablet_mode_controller.h"
+#include "ash/wm/tablet_mode/tablet_mode_controller_test_api.h"
 #include "base/command_line.h"
-#include "services/ui/public/cpp/input_devices/input_device_client_test_api.h"
+#include "services/ws/public/cpp/input_devices/input_device_client_test_api.h"
 #include "ui/display/test/display_manager_test_api.h"
 #include "ui/events/devices/input_device.h"
 #include "ui/events/devices/touchscreen_device.h"
-#include "ui/keyboard/keyboard_switches.h"
-#include "ui/keyboard/keyboard_util.h"
+#include "ui/keyboard/public/keyboard_switches.h"
+#include "ui/keyboard/test/keyboard_test_util.h"
+
+using keyboard::mojom::KeyboardEnableFlag;
 
 namespace ash {
 
 namespace {
 
 VirtualKeyboardController* GetVirtualKeyboardController() {
-  return Shell::Get()->virtual_keyboard_controller();
-}
-
-keyboard::KeyboardController* GetKeyboardController() {
-  return keyboard::KeyboardController::Get();
+  return Shell::Get()->ash_keyboard_controller()->virtual_keyboard_controller();
 }
 
 }  // namespace
@@ -44,16 +43,19 @@ class VirtualKeyboardControllerTest : public AshTestBase {
   VirtualKeyboardControllerTest() = default;
   ~VirtualKeyboardControllerTest() override = default;
 
-  // Sets the event blocker on the maximized window controller.
-  void SetEventBlocker(
-      std::unique_ptr<ScopedDisableInternalMouseAndKeyboard> blocker) {
-    Shell::Get()->tablet_mode_controller()->event_blocker_ = std::move(blocker);
-  }
-
   void SetUp() override {
     AshTestBase::SetUp();
-    ui::InputDeviceClientTestApi().SetKeyboardDevices({});
-    ui::InputDeviceClientTestApi().SetTouchscreenDevices({});
+    ws::InputDeviceClientTestApi().SetKeyboardDevices({});
+    ws::InputDeviceClientTestApi().SetTouchscreenDevices({});
+    keyboard_controller_ = keyboard::KeyboardController::Get();
+  }
+
+  void TearDown() override {
+    // Ensure inputs devices are reset for the next test.
+    ws::InputDeviceClientTestApi().SetKeyboardDevices({});
+    ws::InputDeviceClientTestApi().SetTouchscreenDevices({});
+
+    AshTestBase::TearDown();
   }
 
   display::Display GetPrimaryDisplay() {
@@ -79,20 +81,22 @@ class VirtualKeyboardControllerTest : public AshTestBase {
     focusable_window->Focus();
   }
 
+  keyboard::KeyboardController* keyboard_controller_ = nullptr;
+
  private:
   DISALLOW_COPY_AND_ASSIGN(VirtualKeyboardControllerTest);
 };
 
 // Mock event blocker that enables the internal keyboard when it's destructor
 // is called.
-class MockEventBlocker : public ScopedDisableInternalMouseAndKeyboard {
+class MockEventBlocker : public InternalInputDevicesEventBlocker {
  public:
   MockEventBlocker() = default;
   ~MockEventBlocker() override {
     std::vector<ui::InputDevice> keyboard_devices;
     keyboard_devices.push_back(ui::InputDevice(
         1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL, "keyboard"));
-    ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+    ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
   }
 
  private:
@@ -104,22 +108,17 @@ class MockEventBlocker : public ScopedDisableInternalMouseAndKeyboard {
 TEST_F(VirtualKeyboardControllerTest, RestoreKeyboardDevices) {
   // Toggle tablet mode on.
   Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(true);
-  std::unique_ptr<ScopedDisableInternalMouseAndKeyboard> blocker(
+  std::unique_ptr<InternalInputDevicesEventBlocker> blocker(
       new MockEventBlocker);
-  SetEventBlocker(std::move(blocker));
+  TabletModeControllerTestApi().set_event_blocker(std::move(blocker));
 }
 
 TEST_F(VirtualKeyboardControllerTest,
        ForceToShowKeyboardWithKeysetWhenAccessibilityKeyboardIsEnabled) {
-  // TODO(mash): Turning on accessibility keyboard does not create a valid
-  // KeyboardController under MASH. See https://crbug.com/646565.
-  if (Shell::GetAshConfig() == Config::MASH)
-    return;
-
   AccessibilityController* accessibility_controller =
       Shell::Get()->accessibility_controller();
   accessibility_controller->SetVirtualKeyboardEnabled(true);
-  ASSERT_TRUE(accessibility_controller->IsVirtualKeyboardEnabled());
+  ASSERT_TRUE(accessibility_controller->virtual_keyboard_enabled());
 
   // Set up a mock ImeControllerClient to test keyset changes.
   TestImeControllerClient client;
@@ -129,7 +128,7 @@ TEST_F(VirtualKeyboardControllerTest,
   GetVirtualKeyboardController()->ForceShowKeyboardWithKeyset(
       chromeos::input_method::mojom::ImeKeyset::kEmoji);
   Shell::Get()->ime_controller()->FlushMojoForTesting();
-  EXPECT_TRUE(accessibility_controller->IsVirtualKeyboardEnabled());
+  EXPECT_TRUE(accessibility_controller->virtual_keyboard_enabled());
 
   // Keyset should be emoji.
   Shell::Get()->ime_controller()->FlushMojoForTesting();
@@ -137,13 +136,14 @@ TEST_F(VirtualKeyboardControllerTest,
             client.last_keyset_);
 
   // Simulate the keyboard hiding.
-  if (GetKeyboardController()->HasObserver(GetVirtualKeyboardController())) {
-    GetVirtualKeyboardController()->OnKeyboardHidden();
+  if (keyboard_controller_->HasObserver(GetVirtualKeyboardController())) {
+    GetVirtualKeyboardController()->OnKeyboardHidden(
+        false /* is_temporary_hide */);
   }
   base::RunLoop().RunUntilIdle();
 
   // The keyboard should still be enabled.
-  EXPECT_TRUE(accessibility_controller->IsVirtualKeyboardEnabled());
+  EXPECT_TRUE(accessibility_controller->virtual_keyboard_enabled());
 
   // Reset the accessibility prefs.
   accessibility_controller->SetVirtualKeyboardEnabled(false);
@@ -155,39 +155,39 @@ TEST_F(VirtualKeyboardControllerTest,
 }
 
 TEST_F(VirtualKeyboardControllerTest,
-       ForceToShowKeyboardWithKeysetWhenAccessibilityKeyboardIsDisabled) {
-  // TODO(mash): Turning on accessibility keyboard does not create a valid
-  // KeyboardController under MASH. See https://crbug.com/646565.
-  if (Shell::GetAshConfig() == Config::MASH)
-    return;
-
-  AccessibilityController* accessibility_controller =
-      Shell::Get()->accessibility_controller();
-  accessibility_controller->SetVirtualKeyboardEnabled(false);
-  ASSERT_FALSE(accessibility_controller->IsVirtualKeyboardEnabled());
-
+       ForceToShowKeyboardWithKeysetWhenKeyboardIsDisabled) {
   // Set up a mock ImeControllerClient to test keyset changes.
   TestImeControllerClient client;
   Shell::Get()->ime_controller()->SetClient(client.CreateInterfacePtr());
 
-  // Should show the keyboard by turning on the accesibility keyboard.
+  // Should show the keyboard by enabling it temporarily.
+  EXPECT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
+  EXPECT_FALSE(
+      keyboard_controller_->IsEnableFlagSet(KeyboardEnableFlag::kShelfEnabled));
+
   GetVirtualKeyboardController()->ForceShowKeyboardWithKeyset(
       chromeos::input_method::mojom::ImeKeyset::kEmoji);
   Shell::Get()->ime_controller()->FlushMojoForTesting();
-  EXPECT_TRUE(accessibility_controller->IsVirtualKeyboardEnabled());
+
+  EXPECT_TRUE(
+      keyboard_controller_->IsEnableFlagSet(KeyboardEnableFlag::kShelfEnabled));
+  EXPECT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
 
   // Keyset should be emoji.
   EXPECT_EQ(chromeos::input_method::mojom::ImeKeyset::kEmoji,
             client.last_keyset_);
 
   // Simulate the keyboard hiding.
-  if (GetKeyboardController()->HasObserver(GetVirtualKeyboardController())) {
-    GetVirtualKeyboardController()->OnKeyboardHidden();
+  if (keyboard_controller_->HasObserver(GetVirtualKeyboardController())) {
+    GetVirtualKeyboardController()->OnKeyboardHidden(
+        false /* is_temporary_hide */);
   }
   base::RunLoop().RunUntilIdle();
 
   // The keyboard should still be disabled again.
-  EXPECT_FALSE(accessibility_controller->IsVirtualKeyboardEnabled());
+  EXPECT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
+  EXPECT_FALSE(
+      keyboard_controller_->IsEnableFlagSet(KeyboardEnableFlag::kShelfEnabled));
 
   // Keyset should be reset to none.
   Shell::Get()->ime_controller()->FlushMojoForTesting();
@@ -195,6 +195,42 @@ TEST_F(VirtualKeyboardControllerTest,
             client.last_keyset_);
 }
 
+TEST_F(VirtualKeyboardControllerTest,
+       ForceToShowKeyboardWithKeysetTemporaryHide) {
+  // Set up a mock ImeControllerClient to test keyset changes.
+  TestImeControllerClient client;
+  Shell::Get()->ime_controller()->SetClient(client.CreateInterfacePtr());
+
+  // Should show the keyboard by enabling it temporarily.
+  GetVirtualKeyboardController()->ForceShowKeyboardWithKeyset(
+      chromeos::input_method::mojom::ImeKeyset::kEmoji);
+  Shell::Get()->ime_controller()->FlushMojoForTesting();
+
+  EXPECT_TRUE(
+      keyboard_controller_->IsEnableFlagSet(KeyboardEnableFlag::kShelfEnabled));
+  EXPECT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
+
+  // Keyset should be emoji.
+  EXPECT_EQ(chromeos::input_method::mojom::ImeKeyset::kEmoji,
+            client.last_keyset_);
+
+  // Simulate the keyboard hiding temporarily.
+  if (keyboard_controller_->HasObserver(GetVirtualKeyboardController())) {
+    GetVirtualKeyboardController()->OnKeyboardHidden(
+        true /* is_temporary_hide */);
+  }
+  base::RunLoop().RunUntilIdle();
+
+  // The keyboard should still be enabled.
+  EXPECT_TRUE(
+      keyboard_controller_->IsEnableFlagSet(KeyboardEnableFlag::kShelfEnabled));
+  EXPECT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
+
+  // Keyset should still be emoji.
+  EXPECT_EQ(chromeos::input_method::mojom::ImeKeyset::kEmoji,
+            client.last_keyset_);
+}
+
 class VirtualKeyboardControllerAutoTest : public VirtualKeyboardControllerTest,
                                           public VirtualKeyboardObserver {
  public:
@@ -202,17 +238,13 @@ class VirtualKeyboardControllerAutoTest : public VirtualKeyboardControllerTest,
   ~VirtualKeyboardControllerAutoTest() override = default;
 
   void SetUp() override {
-    AshTestBase::SetUp();
-    // Set the current list of devices to empty so that they don't interfere
-    // with the test.
-    ui::InputDeviceClientTestApi().SetKeyboardDevices({});
-    ui::InputDeviceClientTestApi().SetTouchscreenDevices({});
+    VirtualKeyboardControllerTest::SetUp();
     Shell::Get()->system_tray_notifier()->AddVirtualKeyboardObserver(this);
   }
 
   void TearDown() override {
     Shell::Get()->system_tray_notifier()->RemoveVirtualKeyboardObserver(this);
-    AshTestBase::TearDown();
+    VirtualKeyboardControllerTest::TearDown();
   }
 
   void OnKeyboardSuppressionChanged(bool suppressed) override {
@@ -246,31 +278,31 @@ TEST_F(VirtualKeyboardControllerAutoTest, DisabledIfInternalKeyboardPresent) {
   screens.push_back(
       ui::TouchscreenDevice(1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL,
                             "Touchscreen", gfx::Size(1024, 768), 0));
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
   std::vector<ui::InputDevice> keyboard_devices;
   keyboard_devices.push_back(ui::InputDevice(
       1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL, "keyboard"));
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
   // Remove the internal keyboard. Virtual keyboard should now show.
-  ui::InputDeviceClientTestApi().SetKeyboardDevices({});
-  EXPECT_TRUE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetKeyboardDevices({});
+  EXPECT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   // Replug in the internal keyboard. Virtual keyboard should hide.
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
-  EXPECT_FALSE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  EXPECT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
 }
 
 TEST_F(VirtualKeyboardControllerAutoTest, DisabledIfNoTouchScreen) {
   std::vector<ui::TouchscreenDevice> devices;
   // Add a touchscreen. Keyboard should deploy.
   devices.push_back(
-      ui::TouchscreenDevice(1, ui::InputDeviceType::INPUT_DEVICE_EXTERNAL,
+      ui::TouchscreenDevice(1, ui::InputDeviceType::INPUT_DEVICE_USB,
                             "Touchscreen", gfx::Size(800, 600), 0));
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices(devices);
-  EXPECT_TRUE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices(devices);
+  EXPECT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   // Remove touchscreen. Keyboard should hide.
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices({});
-  EXPECT_FALSE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices({});
+  EXPECT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
 }
 
 TEST_F(VirtualKeyboardControllerAutoTest, SuppressedIfExternalKeyboardPresent) {
@@ -278,31 +310,31 @@ TEST_F(VirtualKeyboardControllerAutoTest, SuppressedIfExternalKeyboardPresent) {
   screens.push_back(ui::TouchscreenDevice(
       1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL, "Touchscreen",
       gfx::Size(1024, 768), 0, false /* has_stylus */));
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
   std::vector<ui::InputDevice> keyboard_devices;
-  keyboard_devices.push_back(ui::InputDevice(
-      1, ui::InputDeviceType::INPUT_DEVICE_EXTERNAL, "keyboard"));
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  keyboard_devices.push_back(
+      ui::InputDevice(1, ui::InputDeviceType::INPUT_DEVICE_USB, "keyboard"));
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_TRUE(IsVirtualKeyboardSuppressed());
   // Toggle show keyboard. Keyboard should be visible.
   ResetObserver();
   GetVirtualKeyboardController()->ToggleIgnoreExternalKeyboard();
-  ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+  ASSERT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_TRUE(IsVirtualKeyboardSuppressed());
   // Toggle show keyboard. Keyboard should be hidden.
   ResetObserver();
   GetVirtualKeyboardController()->ToggleIgnoreExternalKeyboard();
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_TRUE(IsVirtualKeyboardSuppressed());
   // Remove external keyboard. Should be notified that the keyboard is not
   // suppressed.
   ResetObserver();
-  ui::InputDeviceClientTestApi().SetKeyboardDevices({});
-  ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetKeyboardDevices({});
+  ASSERT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_FALSE(IsVirtualKeyboardSuppressed());
 }
@@ -312,12 +344,12 @@ TEST_F(VirtualKeyboardControllerAutoTest, HandleMultipleKeyboardsPresent) {
   std::vector<ui::InputDevice> keyboards;
   keyboards.push_back(ui::InputDevice(
       1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL, "keyboard"));
-  keyboards.push_back(ui::InputDevice(
-      2, ui::InputDeviceType::INPUT_DEVICE_EXTERNAL, "keyboard"));
-  keyboards.push_back(ui::InputDevice(
-      3, ui::InputDeviceType::INPUT_DEVICE_EXTERNAL, "keyboard"));
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboards);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  keyboards.push_back(
+      ui::InputDevice(2, ui::InputDeviceType::INPUT_DEVICE_USB, "keyboard"));
+  keyboards.push_back(
+      ui::InputDevice(3, ui::InputDeviceType::INPUT_DEVICE_USB, "keyboard"));
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboards);
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
 }
 
 // Tests tablet mode interaction without disabling the internal keyboard.
@@ -326,61 +358,61 @@ TEST_F(VirtualKeyboardControllerAutoTest, EnabledDuringTabletMode) {
   screens.push_back(
       ui::TouchscreenDevice(1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL,
                             "Touchscreen", gfx::Size(1024, 768), 0));
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
   std::vector<ui::InputDevice> keyboard_devices;
   keyboard_devices.push_back(ui::InputDevice(
       1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL, "Keyboard"));
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
   // Toggle tablet mode on.
-  Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(true);
-  ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+  TabletModeControllerTestApi().EnterTabletMode();
+  ASSERT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   // Toggle tablet mode off.
-  Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(false);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  TabletModeControllerTestApi().LeaveTabletMode();
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
 }
 
 // Tests that keyboard gets suppressed in tablet mode.
-TEST_F(VirtualKeyboardControllerAutoTest, SuppressedInMaximizedMode) {
+TEST_F(VirtualKeyboardControllerAutoTest, SuppressedInTabletMode) {
   std::vector<ui::TouchscreenDevice> screens;
   screens.push_back(
       ui::TouchscreenDevice(1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL,
                             "Touchscreen", gfx::Size(1024, 768), 0));
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
   std::vector<ui::InputDevice> keyboard_devices;
   keyboard_devices.push_back(ui::InputDevice(
       1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL, "Keyboard"));
-  keyboard_devices.push_back(ui::InputDevice(
-      2, ui::InputDeviceType::INPUT_DEVICE_EXTERNAL, "Keyboard"));
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  keyboard_devices.push_back(
+      ui::InputDevice(2, ui::InputDeviceType::INPUT_DEVICE_USB, "Keyboard"));
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
   // Toggle tablet mode on.
-  Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(true);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  TabletModeControllerTestApi().EnterTabletMode();
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_TRUE(IsVirtualKeyboardSuppressed());
   // Toggle show keyboard. Keyboard should be visible.
   ResetObserver();
   GetVirtualKeyboardController()->ToggleIgnoreExternalKeyboard();
-  ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+  ASSERT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_TRUE(IsVirtualKeyboardSuppressed());
   // Toggle show keyboard. Keyboard should be hidden.
   ResetObserver();
   GetVirtualKeyboardController()->ToggleIgnoreExternalKeyboard();
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_TRUE(IsVirtualKeyboardSuppressed());
   // Remove external keyboard. Should be notified that the keyboard is not
   // suppressed.
   ResetObserver();
   keyboard_devices.pop_back();
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
-  ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  ASSERT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
   ASSERT_TRUE(notified());
   ASSERT_FALSE(IsVirtualKeyboardSuppressed());
   // Toggle tablet mode oFF.
-  Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(false);
-  ASSERT_FALSE(keyboard::IsKeyboardEnabled());
+  TabletModeControllerTestApi().LeaveTabletMode();
+  ASSERT_FALSE(keyboard_controller_->IsKeyboardEnableRequested());
 }
 
 class VirtualKeyboardControllerAlwaysEnabledTest
@@ -407,18 +439,19 @@ TEST_F(VirtualKeyboardControllerAlwaysEnabledTest, DoesNotSuppressKeyboard) {
   screens.push_back(
       ui::TouchscreenDevice(1, ui::InputDeviceType::INPUT_DEVICE_INTERNAL,
                             "Touchscreen", gfx::Size(1024, 768), 0));
-  ui::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
+  ws::InputDeviceClientTestApi().SetTouchscreenDevices(screens);
   std::vector<ui::InputDevice> keyboard_devices;
-  keyboard_devices.push_back(ui::InputDevice(
-      1, ui::InputDeviceType::INPUT_DEVICE_EXTERNAL, "keyboard"));
-  ui::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
-  ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+  keyboard_devices.push_back(
+      ui::InputDevice(1, ui::InputDeviceType::INPUT_DEVICE_USB, "keyboard"));
+  ws::InputDeviceClientTestApi().SetKeyboardDevices(keyboard_devices);
+  ASSERT_TRUE(keyboard_controller_->IsKeyboardEnableRequested());
 }
 
-// Test for http://crbug.com/297858. |MoveKeyboardToTouchableDisplay| should
-// move keyboard to primary display if no display has touch capability.
+// Test for http://crbug.com/297858. |GetContainerForDefaultDisplay| should
+// return the primary display if no display has touch capability and
+// no window is focused.
 TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
-       MovesKeyboardToPrimaryDisplayWhenNoDisplayHasTouch) {
+       DefaultContainerInPrimaryDisplayWhenNoDisplayHasTouch) {
   UpdateDisplay("500x500,500x500");
 
   EXPECT_NE(display::Display::TouchSupport::AVAILABLE,
@@ -426,15 +459,32 @@ TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
   EXPECT_NE(display::Display::TouchSupport::AVAILABLE,
             GetSecondaryDisplay().touch_support());
 
-  GetVirtualKeyboardController()->MoveKeyboardToTouchableDisplay();
-
-  EXPECT_EQ(GetPrimaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetPrimaryRootWindow(), GetVirtualKeyboardController()
+                                        ->GetContainerForDefaultDisplay()
+                                        ->GetRootWindow());
 }
 
-// Test for http://crbug.com/303429. |MoveKeyboardToTouchableDisplay| should
+// Test for http://crbug.com/297858. |GetContainerForDefaultDisplay| should
+// move keyboard to focused display if no display has touch capability.
+TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
+       DefaultContainerIsInFocusedDisplayWhenNoDisplayHasTouch) {
+  UpdateDisplay("500x500,500x500");
+
+  EXPECT_NE(display::Display::TouchSupport::AVAILABLE,
+            GetPrimaryDisplay().touch_support());
+  EXPECT_NE(display::Display::TouchSupport::AVAILABLE,
+            GetSecondaryDisplay().touch_support());
+
+  CreateFocusedTestWindowInRootWindow(GetSecondaryRootWindow());
+  EXPECT_EQ(GetSecondaryRootWindow(), GetVirtualKeyboardController()
+                                          ->GetContainerForDefaultDisplay()
+                                          ->GetRootWindow());
+}
+
+// Test for http://crbug.com/303429. |GetContainerForDefaultDisplay| should
 // move keyboard to first touchable display when there is one.
 TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
-       MovesKeyboardToFirstTouchableDisplay) {
+       DefaultContainerIsInFirstTouchableDisplay) {
   UpdateDisplay("500x500,500x500");
 
   // Make secondary display touchable.
@@ -447,16 +497,17 @@ TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
   EXPECT_EQ(display::Display::TouchSupport::AVAILABLE,
             GetSecondaryDisplay().touch_support());
 
-  GetVirtualKeyboardController()->MoveKeyboardToTouchableDisplay();
-
-  EXPECT_EQ(GetSecondaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetSecondaryRootWindow(), GetVirtualKeyboardController()
+                                          ->GetContainerForDefaultDisplay()
+                                          ->GetRootWindow());
 }
 
-// Test for http://crbug.com/303429. |MoveKeyboardToTouchableDisplay| should
+// Test for http://crbug.com/303429. |GetContainerForDefaultDisplay| should
 // move keyboard to first touchable display when the focused display is not
 // touchable.
-TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
-       MovesKeyboardToFirstTouchableDisplayIfFocusedDisplayIsNotTouchable) {
+TEST_F(
+    VirtualKeyboardControllerAlwaysEnabledTest,
+    DefaultContainerIsInFirstTouchableDisplayIfFocusedDisplayIsNotTouchable) {
   UpdateDisplay("500x500,500x500");
 
   // Make secondary display touchable.
@@ -472,14 +523,15 @@ TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
   // Focus on primary display.
   CreateFocusedTestWindowInRootWindow(GetPrimaryRootWindow());
 
-  GetVirtualKeyboardController()->MoveKeyboardToTouchableDisplay();
-  EXPECT_EQ(GetSecondaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetSecondaryRootWindow(), GetVirtualKeyboardController()
+                                          ->GetContainerForDefaultDisplay()
+                                          ->GetRootWindow());
 }
 
-// Test for http://crbug.com/303429. |MoveKeyboardToTouchableDisplay| should
+// Test for http://crbug.com/303429. |GetContainerForDefaultDisplay| should
 // move keyborad to first touchable display when there is one.
 TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
-       MovesKeyboardToFocusedDisplayIfTouchable) {
+       DefaultContainerIsInFocusedDisplayIfTouchable) {
   UpdateDisplay("500x500,500x500");
 
   // Make both displays touchable.
@@ -497,19 +549,20 @@ TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
 
   // Focus on secondary display.
   CreateFocusedTestWindowInRootWindow(GetSecondaryRootWindow());
-  GetVirtualKeyboardController()->MoveKeyboardToTouchableDisplay();
-  EXPECT_EQ(GetSecondaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetSecondaryRootWindow(), GetVirtualKeyboardController()
+                                          ->GetContainerForDefaultDisplay()
+                                          ->GetRootWindow());
 
   // Focus on primary display.
   CreateFocusedTestWindowInRootWindow(GetPrimaryRootWindow());
-  GetVirtualKeyboardController()->MoveKeyboardToTouchableDisplay();
-  EXPECT_EQ(GetPrimaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetPrimaryRootWindow(), GetVirtualKeyboardController()
+                                        ->GetContainerForDefaultDisplay()
+                                        ->GetRootWindow());
 }
 
-// Test for http://crbug.com/303429. |MoveKeyboardToDisplay| should move
+// Test for http://crbug.com/303429. |GetContainerForDisplay| should move
 // keyboard to specified display even when it's not touchable.
-TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
-       MovesKeyboardToSpecifiedDisplay) {
+TEST_F(VirtualKeyboardControllerAlwaysEnabledTest, GetContainerForDisplay) {
   UpdateDisplay("500x500,500x500");
 
   // Make primary display touchable.
@@ -523,12 +576,28 @@ TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
             GetSecondaryDisplay().touch_support());
 
   // Move to primary display.
-  GetVirtualKeyboardController()->MoveKeyboardToDisplay(GetPrimaryDisplay());
-  EXPECT_EQ(GetPrimaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetPrimaryRootWindow(),
+            GetVirtualKeyboardController()
+                ->GetContainerForDisplay(GetPrimaryDisplay())
+                ->GetRootWindow());
 
   // Move to secondary display.
-  GetVirtualKeyboardController()->MoveKeyboardToDisplay(GetSecondaryDisplay());
-  EXPECT_EQ(GetSecondaryRootWindow(), GetKeyboardController()->GetRootWindow());
+  EXPECT_EQ(GetSecondaryRootWindow(),
+            GetVirtualKeyboardController()
+                ->GetContainerForDisplay(GetSecondaryDisplay())
+                ->GetRootWindow());
+}
+
+// Test for https://crbug.com/897007.
+TEST_F(VirtualKeyboardControllerAlwaysEnabledTest,
+       ShowKeyboardInSecondaryDisplay) {
+  UpdateDisplay("500x500,500x500");
+
+  // Show in secondary display.
+  keyboard_controller_->ShowKeyboardInDisplay(GetSecondaryDisplay());
+  EXPECT_EQ(GetSecondaryRootWindow(), keyboard_controller_->GetRootWindow());
+  ASSERT_TRUE(keyboard::WaitUntilShown());
+  EXPECT_TRUE(!keyboard_controller_->GetKeyboardWindow()->bounds().IsEmpty());
 }
 
 }  // namespace ash
