@@ -5,6 +5,7 @@
 #include "ash/laser/laser_pointer_view.h"
 
 #include "ash/laser/laser_segment_utils.h"
+#include "base/bind.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "third_party/skia/include/core/SkColor.h"
 #include "third_party/skia/include/core/SkTypes.h"
@@ -128,15 +129,15 @@ class LaserSegment {
     DCHECK_EQ(4u, ordered_points.size());
     path_.moveTo(ordered_points[0].x(), ordered_points[0].y());
     if (!is_first_segment) {
-      path_.arcTo(start_radius, start_radius, 180.0f, gfx::Path::kSmall_ArcSize,
-                  gfx::Path::kCW_Direction, ordered_points[1].x(),
+      path_.arcTo(start_radius, start_radius, 180.0f, SkPath::kSmall_ArcSize,
+                  SkPath::kCW_Direction, ordered_points[1].x(),
                   ordered_points[1].y());
     }
 
     path_.lineTo(ordered_points[2].x(), ordered_points[2].y());
     path_.arcTo(
-        end_radius, end_radius, 180.0f, gfx::Path::kSmall_ArcSize,
-        is_last_segment ? gfx::Path::kCW_Direction : gfx::Path::kCCW_Direction,
+        end_radius, end_radius, 180.0f, SkPath::kSmall_ArcSize,
+        is_last_segment ? SkPath::kCW_Direction : SkPath::kCCW_Direction,
         ordered_points[3].x(), ordered_points[3].y());
     path_.lineTo(ordered_points[0].x(), ordered_points[0].y());
 
@@ -164,12 +165,10 @@ LaserPointerView::LaserPointerView(base::TimeDelta life_duration,
       laser_points_(life_duration),
       predicted_laser_points_(life_duration),
       presentation_delay_(presentation_delay),
-      stationary_timer_(
-          new base::Timer(FROM_HERE,
-                          stationary_point_delay,
-                          base::BindRepeating(&LaserPointerView::UpdateTime,
-                                              base::Unretained(this)),
-                          /*is_repeating=*/true)),
+      stationary_timer_(FROM_HERE,
+                        stationary_point_delay,
+                        base::BindRepeating(&LaserPointerView::UpdateTime,
+                                            base::Unretained(this))),
       weak_ptr_factory_(this) {}
 
 LaserPointerView::~LaserPointerView() = default;
@@ -186,7 +185,7 @@ void LaserPointerView::AddNewPoint(const gfx::PointF& new_point,
                      : 0);
   AddPoint(new_point, new_time);
   stationary_point_location_ = new_point;
-  stationary_timer_->Reset();
+  stationary_timer_.Reset();
 }
 
 void LaserPointerView::FadeOut(base::OnceClosure done) {
