@@ -12,17 +12,21 @@
 #include "ash/detachable_base/detachable_base_pairing_status.h"
 #include "ash/ime/ime_controller.h"
 #include "ash/login/login_screen_controller.h"
-#include "ash/login/ui/layout_util.h"
 #include "ash/login/ui/lock_contents_view.h"
 #include "ash/login/ui/lock_screen.h"
 #include "ash/login/ui/login_data_dispatcher.h"
 #include "ash/login/ui/login_detachable_base_model.h"
 #include "ash/login/ui/non_accessible_view.h"
+#include "ash/login/ui/views_utils.h"
 #include "ash/public/interfaces/kiosk_app_info.mojom.h"
+#include "ash/shelf/login_shelf_view.h"
 #include "ash/shelf/shelf.h"
 #include "ash/shelf/shelf_widget.h"
 #include "ash/shell.h"
+#include "ash/wallpaper/wallpaper_controller.h"
+#include "base/bind.h"
 #include "base/optional.h"
+#include "base/stl_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "ui/base/ime/chromeos/ime_keyboard.h"
 #include "ui/views/controls/button/md_text_button.h"
@@ -37,23 +41,33 @@ namespace {
 namespace ButtonId {
 enum {
   kGlobalAddUser = 1,
+  kGlobalAddTenUsers,
   kGlobalRemoveUser,
   kGlobalToggleBlur,
   kGlobalToggleNoteAction,
   kGlobalToggleCapsLock,
-  kGlobalAddDevChannelInfo,
+  kGlobalAddSystemInfo,
   kGlobalToggleAuth,
   kGlobalAddKioskApp,
   kGlobalRemoveKioskApp,
+  kGlobalShowKioskError,
   kGlobalToggleDebugDetachableBase,
   kGlobalCycleDetachableBaseStatus,
   kGlobalCycleDetachableBaseId,
-
+  kGlobalCycleAuthErrorMessage,
+  kGlobalToggleWarningBanner,
+  kGlobalToggleManagedSessionDisclosure,
+  kGlobalToggleParentAccess,
   kPerUserTogglePin,
+  kPerUserToggleTap,
   kPerUserCycleEasyUnlockState,
+  kPerUserCycleFingerprintState,
+  kPerUserAuthFingerprintSuccessState,
+  kPerUserAuthFingerprintFailState,
   kPerUserForceOnlineSignIn,
   kPerUserToggleAuthEnabled,
   kPerUserUseDetachableBase,
+  kPerUserTogglePublicAccount,
 };
 }  // namespace ButtonId
 
@@ -62,6 +76,11 @@ constexpr const char* kDebugUserNames[] = {
     "Debbie Craig",     "Stella Wong",  "Stephanie Wade",
 };
 
+constexpr const char* kDebugPublicAccountNames[] = {
+    "Seattle Public Library", "San Jose Public Library",
+    "Sunnyvale Public Library", "Mountain View Public Library",
+};
+
 constexpr const char* kDebugDetachableBases[] = {"Base A", "Base B", "Base C"};
 
 constexpr const char kDebugOsVersion[] =
@@ -72,18 +91,26 @@ constexpr const char kDebugBluetoothName[] = "Bluetooth adapter";
 constexpr const char kDebugKioskAppId[] = "asdf1234";
 constexpr const char kDebugKioskAppName[] = "Test App Name";
 
+constexpr const char kDebugDefaultLocaleCode[] = "en-GB";
+constexpr const char kDebugDefaultLocaleTitle[] = "English";
+constexpr const char kDebugEnterpriseDomain[] = "library.com";
+
 // Additional state for a user that the debug UI needs to reference.
 struct UserMetadata {
   explicit UserMetadata(const mojom::UserInfoPtr& user_info)
       : account_id(user_info->account_id),
-        display_name(user_info->display_name) {}
+        display_name(user_info->display_name),
+        type(user_info->type) {}
 
   AccountId account_id;
   std::string display_name;
   bool enable_pin = false;
-  bool enable_click_to_unlock = false;
+  bool enable_tap_to_unlock = false;
   bool enable_auth = true;
+  user_manager::UserType type = user_manager::USER_TYPE_REGULAR;
   mojom::EasyUnlockIconId easy_unlock_id = mojom::EasyUnlockIconId::NONE;
+  mojom::FingerprintState fingerprint_state =
+      mojom::FingerprintState::UNAVAILABLE;
 };
 
 std::string DetachableBasePairingStatusToString(
@@ -101,6 +128,49 @@ std::string DetachableBasePairingStatusToString(
   return "Unknown";
 }
 
+// Update the user data based on |type| and |user_index|.
+mojom::LoginUserInfoPtr PopulateUserData(const mojom::LoginUserInfoPtr& user,
+                                         user_manager::UserType type,
+                                         int user_index) {
+  mojom::LoginUserInfoPtr result = user->Clone();
+  result->basic_user_info->type = type;
+
+  bool is_public_account = type == user_manager::USER_TYPE_PUBLIC_ACCOUNT;
+  // Set debug user names and email. Useful for the stub user, which does not
+  // have a name  and email set.
+  result->basic_user_info->display_name =
+      is_public_account
+          ? kDebugPublicAccountNames[user_index %
+                                     base::size(kDebugPublicAccountNames)]
+          : kDebugUserNames[user_index % base::size(kDebugUserNames)];
+  result->basic_user_info->display_email =
+      result->basic_user_info->account_id.GetUserEmail();
+
+  if (is_public_account) {
+    result->public_account_info = mojom::PublicAccountInfo::New();
+    result->public_account_info->enterprise_domain = kDebugEnterpriseDomain;
+    result->public_account_info->default_locale = kDebugDefaultLocaleCode;
+    result->public_account_info->show_expanded_view = true;
+
+    std::vector<mojom::LocaleItemPtr> locales;
+    mojom::LocaleItemPtr locale_item = mojom::LocaleItem::New();
+    locale_item->language_code = kDebugDefaultLocaleCode;
+    locale_item->title = kDebugDefaultLocaleTitle;
+    locales.push_back(std::move(locale_item));
+    result->public_account_info->available_locales = std::move(locales);
+
+    // Request keyboard layouts for the default locale.
+    Shell::Get()
+        ->login_screen_controller()
+        ->RequestPublicSessionKeyboardLayouts(
+            result->basic_user_info->account_id, kDebugDefaultLocaleCode);
+  } else {
+    result->public_account_info.reset();
+  }
+
+  return result;
+}
+
 }  // namespace
 
 // Applies a series of user-defined transformations to a |LoginDataDispatcher|
@@ -125,7 +195,10 @@ class LockDebugView::DebugDataDispatcherTransformer
   LoginDataDispatcher* debug_dispatcher() { return &debug_dispatcher_; }
 
   // Changes the number of displayed users to |count|.
-  void SetUserCount(int count) {
+  void SetUserCount(int count) { NotifyUsers(BuildUserList(count)); }
+
+  // Create user list.
+  std::vector<mojom::LoginUserInfoPtr> BuildUserList(int count) {
     DCHECK(!root_users_.empty());
 
     count = std::max(count, 0);
@@ -146,15 +219,20 @@ class LockDebugView::DebugDataDispatcherTransformer
                 std::to_string(i));
       }
 
-      // Set debug user names. Useful for the stub user, which does not have a
-      // name set.
-      users[i]->basic_user_info->display_name =
-          kDebugUserNames[i % base::size(kDebugUserNames)];
+      // Setup user data based on the user type in debug_users_.
+      user_manager::UserType type = (i < debug_users_.size())
+                                        ? debug_users_[i].type
+                                        : users[i]->basic_user_info->type;
+      users[i] = PopulateUserData(users[i], type, i);
 
       if (i >= debug_users_.size())
         debug_users_.push_back(UserMetadata(users[i]->basic_user_info));
     }
 
+    return users;
+  }
+
+  void NotifyUsers(std::vector<mojom::LoginUserInfoPtr> users) {
     // User notification resets PIN state.
     for (UserMetadata& user : debug_users_)
       user.enable_pin = false;
@@ -184,6 +262,15 @@ class LockDebugView::DebugDataDispatcherTransformer
                                            debug_user->enable_pin);
   }
 
+  // Activates or deactivates tap unlock for the user at |user_index|.
+  void ToggleTapStateForUserIndex(size_t user_index) {
+    DCHECK(user_index >= 0 && user_index < debug_users_.size());
+    UserMetadata* debug_user = &debug_users_[user_index];
+    debug_user->enable_tap_to_unlock = !debug_user->enable_tap_to_unlock;
+    debug_dispatcher_.SetTapToUnlockEnabledForUser(
+        debug_user->account_id, debug_user->enable_tap_to_unlock);
+  }
+
   // Enables click to auth for the user at |user_index|.
   void CycleEasyUnlockForUserIndex(size_t user_index) {
     DCHECK(user_index >= 0 && user_index < debug_users_.size());
@@ -212,7 +299,7 @@ class LockDebugView::DebugDataDispatcherTransformer
     debug_user->easy_unlock_id = get_next_id(debug_user->easy_unlock_id);
 
     // Enable/disable click to unlock.
-    debug_user->enable_click_to_unlock =
+    debug_user->enable_tap_to_unlock =
         debug_user->easy_unlock_id == mojom::EasyUnlockIconId::UNLOCKED;
 
     // Prepare icon that we will show.
@@ -234,8 +321,26 @@ class LockDebugView::DebugDataDispatcherTransformer
 
     // Show icon and enable/disable click to unlock.
     debug_dispatcher_.ShowEasyUnlockIcon(debug_user->account_id, icon);
-    debug_dispatcher_.SetClickToUnlockEnabledForUser(
-        debug_user->account_id, debug_user->enable_click_to_unlock);
+    debug_dispatcher_.SetTapToUnlockEnabledForUser(
+        debug_user->account_id, debug_user->enable_tap_to_unlock);
+  }
+
+  // Enables fingerprint auth for the user at |user_index|.
+  void CycleFingerprintStateForUserIndex(size_t user_index) {
+    DCHECK(user_index >= 0 && user_index < debug_users_.size());
+    UserMetadata* debug_user = &debug_users_[user_index];
+
+    debug_user->fingerprint_state = static_cast<mojom::FingerprintState>(
+        (static_cast<int>(debug_user->fingerprint_state) + 1) %
+        (static_cast<int>(mojom::FingerprintState::kMaxValue) + 1));
+    debug_dispatcher_.SetFingerprintState(debug_user->account_id,
+                                          debug_user->fingerprint_state);
+  }
+  void AuthenticateFingerprintForUserIndex(size_t user_index, bool success) {
+    DCHECK(user_index >= 0 && user_index < debug_users_.size());
+    UserMetadata* debug_user = &debug_users_[user_index];
+    debug_dispatcher_.NotifyFingerprintAuthResult(debug_user->account_id,
+                                                  success);
   }
 
   // Force online sign-in for the user at |user_index|.
@@ -256,6 +361,23 @@ class LockDebugView::DebugDataDispatcherTransformer
             base::TimeDelta::FromHours(8));
   }
 
+  // Convert user type to regular user or public account for the user at
+  // |user_index|.
+  void TogglePublicAccountForUserIndex(size_t user_index) {
+    DCHECK(user_index >= 0 && user_index < debug_users_.size());
+    UserMetadata& user = debug_users_[user_index];
+    // Swap the type between regular and public account.
+    user.type = user.type == user_manager::USER_TYPE_REGULAR
+                    ? user_manager::USER_TYPE_PUBLIC_ACCOUNT
+                    : user_manager::USER_TYPE_REGULAR;
+
+    std::vector<mojom::LoginUserInfoPtr> users =
+        BuildUserList(debug_users_.size());
+    // Update display name and email in debug users.
+    debug_users_[user_index] = UserMetadata(users[user_index]->basic_user_info);
+    NotifyUsers(std::move(users));
+  }
+
   void ToggleLockScreenNoteButton() {
     if (lock_screen_note_state_ == mojom::TrayActionState::kAvailable) {
       lock_screen_note_state_ = mojom::TrayActionState::kNotAvailable;
@@ -268,26 +390,33 @@ class LockDebugView::DebugDataDispatcherTransformer
 
   void AddKioskApp(ShelfWidget* shelf_widget) {
     mojom::KioskAppInfoPtr app_info = mojom::KioskAppInfo::New();
-    app_info->app_id = kDebugKioskAppId;
+    app_info->identifier = mojom::KioskAppIdentifier::New();
+    app_info->identifier->set_app_id(kDebugKioskAppId);
     app_info->name = base::UTF8ToUTF16(kDebugKioskAppName);
     kiosk_apps_.push_back(std::move(app_info));
-    shelf_widget->SetLoginKioskApps(mojo::Clone(kiosk_apps_));
+    shelf_widget->login_shelf_view()->SetKioskApps(mojo::Clone(kiosk_apps_));
   }
 
   void RemoveKioskApp(ShelfWidget* shelf_widget) {
     if (kiosk_apps_.empty())
       return;
     kiosk_apps_.pop_back();
-    shelf_widget->SetLoginKioskApps(mojo::Clone(kiosk_apps_));
+    shelf_widget->login_shelf_view()->SetKioskApps(mojo::Clone(kiosk_apps_));
   }
 
-  void AddLockScreenDevChannelInfo(const std::string& os_version,
-                                   const std::string& enterprise_info,
-                                   const std::string& bluetooth_name) {
-    debug_dispatcher_.SetDevChannelInfo(os_version, enterprise_info,
-                                        bluetooth_name);
+  void AddSystemInfo(const std::string& os_version,
+                     const std::string& enterprise_info,
+                     const std::string& bluetooth_name) {
+    debug_dispatcher_.SetSystemInfo(true /*show_if_hidden*/, os_version,
+                                    enterprise_info, bluetooth_name);
   }
 
+  void ShowWarningBanner(const base::string16& message) {
+    debug_dispatcher_.ShowWarningBanner(message);
+  }
+
+  void HideWarningBanner() { debug_dispatcher_.HideWarningBanner(); }
+
   // LoginDataDispatcher::Observer:
   void OnUsersChanged(
       const std::vector<mojom::LoginUserInfoPtr>& users) override {
@@ -312,13 +441,13 @@ class LockDebugView::DebugDataDispatcherTransformer
       }
     }
   }
-  void OnClickToUnlockEnabledForUserChanged(const AccountId& user,
-                                            bool enabled) override {
+  void OnTapToUnlockEnabledForUserChanged(const AccountId& user,
+                                          bool enabled) override {
     // Forward notification only if the user is currently being shown.
     for (size_t i = 0u; i < debug_users_.size(); ++i) {
       if (debug_users_[i].account_id == user) {
-        debug_users_[i].enable_click_to_unlock = enabled;
-        debug_dispatcher_.SetClickToUnlockEnabledForUser(user, enabled);
+        debug_users_[i].enable_tap_to_unlock = enabled;
+        debug_dispatcher_.SetTapToUnlockEnabledForUser(user, enabled);
         break;
       }
     }
@@ -345,6 +474,12 @@ class LockDebugView::DebugDataDispatcherTransformer
                                                       keyboard_layouts);
   }
 
+  void OnPublicSessionShowFullManagementDisclosureChanged(
+      bool show_full_management_disclosure) override {
+    debug_dispatcher_.SetPublicSessionShowFullManagementDisclosure(
+        show_full_management_disclosure);
+  }
+
  private:
   // The debug overlay UI takes ground-truth data from |root_dispatcher_|,
   // applies a series of transformations to it, and exposes it to the UI via
@@ -407,7 +542,7 @@ class LockDebugView::DebugLoginDetachableBaseModel
   // Calculates the debugging detachable base ID that should become the paired
   // base in the model when the button for cycling paired bases is clicked.
   int NextBaseId() const {
-    return (base_id_ + 1) % arraysize(kDebugDetachableBases);
+    return (base_id_ + 1) % base::size(kDebugDetachableBases);
   }
 
   // Gets the descripting text for currently paired base, if any.
@@ -426,7 +561,7 @@ class LockDebugView::DebugLoginDetachableBaseModel
     pairing_status_ = pairing_status;
     if (pairing_status == DetachableBasePairingStatus::kAuthenticated) {
       CHECK_GE(base_id, 0);
-      CHECK_LT(base_id, static_cast<int>(arraysize(kDebugDetachableBases)));
+      CHECK_LT(base_id, static_cast<int>(base::size(kDebugDetachableBases)));
       base_id_ = base_id;
     } else {
       base_id_ = kNullBaseId;
@@ -501,7 +636,8 @@ LockDebugView::LockDebugView(mojom::TrayActionState initial_note_action_state,
           data_dispatcher,
           base::BindRepeating(
               &LockDebugView::UpdatePerUserActionContainerAndLayout,
-              base::Unretained(this)))) {
+              base::Unretained(this)))),
+      next_auth_error_type_(AuthErrorType::kFirstUnlockFailed) {
   SetLayoutManager(
       std::make_unique<views::BoxLayout>(views::BoxLayout::kHorizontal));
 
@@ -537,6 +673,8 @@ LockDebugView::LockDebugView(mojom::TrayActionState initial_note_action_state,
 
   auto* change_users_container = add_horizontal_container();
   AddButton("Add user", ButtonId::kGlobalAddUser, change_users_container);
+  AddButton("Add 10 users", ButtonId::kGlobalAddTenUsers,
+            change_users_container);
   AddButton("Remove user", ButtonId::kGlobalRemoveUser, change_users_container);
 
   auto* toggle_container = add_horizontal_container();
@@ -545,15 +683,28 @@ LockDebugView::LockDebugView(mojom::TrayActionState initial_note_action_state,
             toggle_container);
   AddButton("Toggle caps lock", ButtonId::kGlobalToggleCapsLock,
             toggle_container);
-  AddButton("Add dev channel info", ButtonId::kGlobalAddDevChannelInfo,
+  AddButton("Add system info", ButtonId::kGlobalAddSystemInfo,
             toggle_container);
   global_action_toggle_auth_ = AddButton(
       "Auth (allowed)", ButtonId::kGlobalToggleAuth, toggle_container);
+  AddButton("Cycle auth error", ButtonId::kGlobalCycleAuthErrorMessage,
+            toggle_container);
+  AddButton("Toggle warning banner", ButtonId::kGlobalToggleWarningBanner,
+            toggle_container);
+  AddButton("Toggle parent access", ButtonId::kGlobalToggleParentAccess,
+            toggle_container);
 
   auto* kiosk_container = add_horizontal_container();
   AddButton("Add kiosk app", ButtonId::kGlobalAddKioskApp, kiosk_container);
   AddButton("Remove kiosk app", ButtonId::kGlobalRemoveKioskApp,
             kiosk_container);
+  AddButton("Show kiosk error", ButtonId::kGlobalShowKioskError,
+            kiosk_container);
+
+  auto* managed_sessions_container = add_horizontal_container();
+  AddButton("Toggle managed session disclosure",
+            ButtonId::kGlobalToggleManagedSessionDisclosure,
+            managed_sessions_container);
 
   global_action_detachable_base_group_ = add_horizontal_container();
   UpdateDetachableBaseColumn();
@@ -595,15 +746,58 @@ void LockDebugView::Layout() {
   container_->SizeToPreferredSize();
 }
 
+void LockDebugView::CycleAuthErrorMessage() {
+  switch (next_auth_error_type_) {
+    case AuthErrorType::kFirstUnlockFailed:
+      next_auth_error_type_ = AuthErrorType::kFirstUnlockFailedCapsLockOn;
+      Shell::Get()->ime_controller()->UpdateCapsLockState(
+          false /*caps_enabled*/);
+      debug_detachable_base_model_->SetPairingState(
+          DetachableBasePairingStatus::kNone,
+          DebugLoginDetachableBaseModel::kNullBaseId);
+      lock_->ShowAuthErrorMessageForDebug(1 /*unlock_attempt*/);
+      return;
+    case AuthErrorType::kFirstUnlockFailedCapsLockOn:
+      next_auth_error_type_ = AuthErrorType::kSecondUnlockFailed;
+      Shell::Get()->ime_controller()->UpdateCapsLockState(
+          true /*caps_enabled*/);
+      lock_->ShowAuthErrorMessageForDebug(1 /*unlock_attempt*/);
+      return;
+    case AuthErrorType::kSecondUnlockFailed:
+      next_auth_error_type_ = AuthErrorType::kSecondUnlockFailedCapsLockOn;
+      Shell::Get()->ime_controller()->UpdateCapsLockState(
+          false /*caps_enabled*/);
+      lock_->ShowAuthErrorMessageForDebug(2 /*unlock_attempt*/);
+      return;
+    case AuthErrorType::kSecondUnlockFailedCapsLockOn:
+      next_auth_error_type_ = AuthErrorType::kDetachableBaseFailed;
+      Shell::Get()->ime_controller()->UpdateCapsLockState(
+          true /*caps_enabled*/);
+      lock_->ShowAuthErrorMessageForDebug(2 /*unlock_attempt*/);
+      return;
+    case AuthErrorType::kDetachableBaseFailed:
+      next_auth_error_type_ = AuthErrorType::kFirstUnlockFailed;
+      debug_detachable_base_model_->SetPairingState(
+          DetachableBasePairingStatus::kNotAuthenticated,
+          DebugLoginDetachableBaseModel::kNullBaseId);
+      return;
+    default:
+      NOTREACHED();
+  }
+}
+
 void LockDebugView::ButtonPressed(views::Button* sender,
                                   const ui::Event& event) {
   // Add or remove a user.
   bool is_add_user = sender->id() == ButtonId::kGlobalAddUser;
+  bool is_add_many_users = sender->id() == ButtonId::kGlobalAddTenUsers;
   bool is_remove_user = sender->id() == ButtonId::kGlobalRemoveUser;
-  if (is_add_user || is_remove_user) {
+  if (is_add_user || is_add_many_users || is_remove_user) {
     int num_users = debug_data_dispatcher_->GetUserCount();
     if (is_add_user)
       ++num_users;
+    else if (is_add_many_users)
+      num_users += 10;
     else if (is_remove_user)
       --num_users;
     if (num_users < 0)
@@ -616,7 +810,8 @@ void LockDebugView::ButtonPressed(views::Button* sender,
 
   // Enable or disable wallpaper blur.
   if (sender->id() == ButtonId::kGlobalToggleBlur) {
-    LockScreen::Get()->ToggleBlurForDebug();
+    Shell::Get()->wallpaper_controller()->UpdateWallpaperBlur(
+        !Shell::Get()->wallpaper_controller()->IsWallpaperBlurred());
     return;
   }
 
@@ -633,22 +828,21 @@ void LockDebugView::ButtonPressed(views::Button* sender,
     return;
   }
 
-  // Iteratively adds more info to the dev channel labels to test 7 permutations
+  // Iteratively adds more info to the system info labels to test 7 permutations
   // and then disables the button.
-  if (sender->id() == ButtonId::kGlobalAddDevChannelInfo) {
-    DCHECK_LT(num_dev_channel_info_clicks_, 7u);
-    ++num_dev_channel_info_clicks_;
-    if (num_dev_channel_info_clicks_ == 7u)
+  if (sender->id() == ButtonId::kGlobalAddSystemInfo) {
+    DCHECK_LT(num_system_info_clicks_, 7u);
+    ++num_system_info_clicks_;
+    if (num_system_info_clicks_ == 7u)
       sender->SetEnabled(false);
 
-    std::string os_version =
-        num_dev_channel_info_clicks_ / 4 ? kDebugOsVersion : "";
+    std::string os_version = num_system_info_clicks_ / 4 ? kDebugOsVersion : "";
     std::string enterprise_info =
-        (num_dev_channel_info_clicks_ % 4) / 2 ? kDebugEnterpriseInfo : "";
+        (num_system_info_clicks_ % 4) / 2 ? kDebugEnterpriseInfo : "";
     std::string bluetooth_name =
-        num_dev_channel_info_clicks_ % 2 ? kDebugBluetoothName : "";
-    debug_data_dispatcher_->AddLockScreenDevChannelInfo(
-        os_version, enterprise_info, bluetooth_name);
+        num_system_info_clicks_ % 2 ? kDebugBluetoothName : "";
+    debug_data_dispatcher_->AddSystemInfo(os_version, enterprise_info,
+                                          bluetooth_name);
     return;
   }
 
@@ -700,6 +894,11 @@ void LockDebugView::ButtonPressed(views::Button* sender,
         Shelf::ForWindow(GetWidget()->GetNativeWindow())->shelf_widget());
   }
 
+  if (sender->id() == ButtonId::kGlobalShowKioskError) {
+    Shell::Get()->login_screen_controller()->ShowKioskAppError(
+        "Test error message.");
+  }
+
   if (sender->id() == ButtonId::kGlobalToggleDebugDetachableBase) {
     if (debug_detachable_base_model_->debugging_pairing_state()) {
       debug_detachable_base_model_->ClearDebugPairingState();
@@ -736,14 +935,53 @@ void LockDebugView::ButtonPressed(views::Button* sender,
     return;
   }
 
+  if (sender->id() == ButtonId::kGlobalCycleAuthErrorMessage) {
+    CycleAuthErrorMessage();
+    return;
+  }
+
+  // Show or hide warning banner.
+  if (sender->id() == ButtonId::kGlobalToggleWarningBanner) {
+    if (is_warning_banner_shown_) {
+      debug_data_dispatcher_->HideWarningBanner();
+    } else {
+      debug_data_dispatcher_->ShowWarningBanner(base::ASCIIToUTF16(
+          "A critical update is ready to install. Sign in to get started."));
+    }
+    is_warning_banner_shown_ = !is_warning_banner_shown_;
+  }
+
   // Enable or disable PIN.
   if (sender->id() == ButtonId::kPerUserTogglePin)
     debug_data_dispatcher_->TogglePinStateForUserIndex(sender->tag());
 
+  // Enable or disable tap.
+  if (sender->id() == ButtonId::kPerUserToggleTap)
+    debug_data_dispatcher_->ToggleTapStateForUserIndex(sender->tag());
+
   // Cycle easy unlock.
   if (sender->id() == ButtonId::kPerUserCycleEasyUnlockState)
     debug_data_dispatcher_->CycleEasyUnlockForUserIndex(sender->tag());
 
+  // Cycle fingerprint unlock state.
+  if (sender->id() == ButtonId::kPerUserCycleFingerprintState)
+    debug_data_dispatcher_->CycleFingerprintStateForUserIndex(sender->tag());
+  if (sender->id() == ButtonId::kPerUserAuthFingerprintSuccessState) {
+    debug_data_dispatcher_->AuthenticateFingerprintForUserIndex(sender->tag(),
+                                                                true);
+  }
+  if (sender->id() == ButtonId::kPerUserAuthFingerprintFailState) {
+    debug_data_dispatcher_->AuthenticateFingerprintForUserIndex(sender->tag(),
+                                                                false);
+  }
+
+  if (sender->id() == ButtonId::kGlobalToggleManagedSessionDisclosure) {
+    is_managed_session_disclosure_shown_ =
+        !is_managed_session_disclosure_shown_;
+    debug_data_dispatcher_->OnPublicSessionShowFullManagementDisclosureChanged(
+        is_managed_session_disclosure_shown_);
+  }
+
   // Force online sign-in.
   if (sender->id() == ButtonId::kPerUserForceOnlineSignIn)
     debug_data_dispatcher_->ForceOnlineSignInForUserIndex(sender->tag());
@@ -757,6 +995,19 @@ void LockDebugView::ButtonPressed(views::Button* sender,
     debug_detachable_base_model_->SetBaseLastUsedForUser(
         debug_data_dispatcher_->GetAccountIdForUserIndex(sender->tag()));
   }
+
+  // Convert this user to regular user or public account.
+  if (sender->id() == ButtonId::kPerUserTogglePublicAccount) {
+    debug_data_dispatcher_->TogglePublicAccountForUserIndex(sender->tag());
+    UpdatePerUserActionContainer();
+    Layout();
+  }
+
+  // Toggle parent access view.
+  if (sender->id() == ButtonId::kGlobalToggleParentAccess) {
+    is_parent_access_shown_ = !is_parent_access_shown_;
+    lock_->OnSetShowParentAccessDialog(is_parent_access_shown_);
+  }
 }
 
 void LockDebugView::UpdatePerUserActionContainer() {
@@ -778,8 +1029,18 @@ void LockDebugView::UpdatePerUserActionContainer() {
     row->AddChildView(name);
 
     AddButton("Toggle PIN", ButtonId::kPerUserTogglePin, row)->set_tag(i);
+    AddButton("Toggle Tap", ButtonId::kPerUserToggleTap, row)->set_tag(i);
     AddButton("Cycle easy unlock", ButtonId::kPerUserCycleEasyUnlockState, row)
         ->set_tag(i);
+    AddButton("Cycle fingerprint state",
+              ButtonId::kPerUserCycleFingerprintState, row)
+        ->set_tag(i);
+    AddButton("Send fingerprint auth success",
+              ButtonId::kPerUserAuthFingerprintSuccessState, row)
+        ->set_tag(i);
+    AddButton("Send fingerprint auth fail",
+              ButtonId::kPerUserAuthFingerprintFailState, row)
+        ->set_tag(i);
     AddButton("Force online sign-in", ButtonId::kPerUserForceOnlineSignIn, row)
         ->set_tag(i);
     AddButton("Toggle auth enabled", ButtonId::kPerUserToggleAuthEnabled, row)
@@ -792,6 +1053,10 @@ void LockDebugView::UpdatePerUserActionContainer() {
           ->set_tag(i);
     }
 
+    AddButton("Toggle Public Account", ButtonId::kPerUserTogglePublicAccount,
+              row)
+        ->set_tag(i);
+
     per_user_action_view_container_->AddChildView(row);
   }
 }
@@ -834,7 +1099,7 @@ views::LabelButton* LockDebugView::AddButton(const std::string& text,
       this, base::ASCIIToUTF16(text));
   button->set_id(id);
   button->SetFocusBehavior(views::View::FocusBehavior::NEVER);
-  container->AddChildView(login_layout_util::WrapViewForPreferredSize(button));
+  container->AddChildView(login_views_utils::WrapViewForPreferredSize(button));
   return button;
 }
 
