@@ -5,8 +5,8 @@
 #include "ash/login/ui/login_password_view.h"
 
 #include "ash/login/ui/login_test_base.h"
-#include "ash/public/cpp/config.h"
 #include "ash/shell.h"
+#include "base/bind.h"
 #include "base/strings/utf_string_conversions.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/views/widget/widget.h"
@@ -61,17 +61,17 @@ class LoginPasswordViewTest : public LoginTestBase {
 // Verifies that the submit button updates its UI state.
 TEST_F(LoginPasswordViewTest, SubmitButtonUpdatesUiState) {
   LoginPasswordView::TestApi test_api(view_);
-  ui::test::EventGenerator& generator = GetEventGenerator();
+  ui::test::EventGenerator* generator = GetEventGenerator();
 
   // The submit button starts with the disabled state.
   EXPECT_TRUE(is_password_field_empty_);
   EXPECT_FALSE(test_api.submit_button()->enabled());
   // Enter 'a'. The submit button is enabled.
-  generator.PressKey(ui::KeyboardCode::VKEY_A, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_A, 0);
   EXPECT_FALSE(is_password_field_empty_);
   EXPECT_TRUE(test_api.submit_button()->enabled());
   // Enter 'b'. The submit button stays enabled.
-  generator.PressKey(ui::KeyboardCode::VKEY_B, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_B, 0);
   EXPECT_FALSE(is_password_field_empty_);
   EXPECT_TRUE(test_api.submit_button()->enabled());
 
@@ -81,7 +81,7 @@ TEST_F(LoginPasswordViewTest, SubmitButtonUpdatesUiState) {
   EXPECT_FALSE(test_api.submit_button()->enabled());
 
   // Enter 'a'. The submit button is enabled.
-  generator.PressKey(ui::KeyboardCode::VKEY_A, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_A, 0);
   EXPECT_FALSE(is_password_field_empty_);
   EXPECT_TRUE(test_api.submit_button()->enabled());
   // Set the text field to be read-only. The submit button is disabled.
@@ -98,12 +98,12 @@ TEST_F(LoginPasswordViewTest, SubmitButtonUpdatesUiState) {
 TEST_F(LoginPasswordViewTest, PasswordSubmitIncludesPasswordText) {
   LoginPasswordView::TestApi test_api(view_);
 
-  ui::test::EventGenerator& generator = GetEventGenerator();
-  generator.PressKey(ui::KeyboardCode::VKEY_A, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_B, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_C, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_1, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->PressKey(ui::KeyboardCode::VKEY_A, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_B, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_C, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_1, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
 
   ASSERT_TRUE(password_.has_value());
   EXPECT_EQ(base::ASCIIToUTF16("abc1"), *password_);
@@ -113,14 +113,14 @@ TEST_F(LoginPasswordViewTest, PasswordSubmitIncludesPasswordText) {
 TEST_F(LoginPasswordViewTest, PasswordSubmitViaButton) {
   LoginPasswordView::TestApi test_api(view_);
 
-  ui::test::EventGenerator& generator = GetEventGenerator();
-  generator.PressKey(ui::KeyboardCode::VKEY_A, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_B, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_C, 0);
-  generator.PressKey(ui::KeyboardCode::VKEY_1, 0);
-  generator.MoveMouseTo(
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->PressKey(ui::KeyboardCode::VKEY_A, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_B, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_C, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_1, 0);
+  generator->MoveMouseTo(
       test_api.submit_button()->GetBoundsInScreen().CenterPoint());
-  generator.ClickLeftButton();
+  generator->ClickLeftButton();
 
   ASSERT_TRUE(password_.has_value());
   EXPECT_EQ(base::ASCIIToUTF16("abc1"), *password_);
@@ -129,13 +129,13 @@ TEST_F(LoginPasswordViewTest, PasswordSubmitViaButton) {
 // Verifies that text is not cleared after submitting a password.
 TEST_F(LoginPasswordViewTest, PasswordSubmitClearsPassword) {
   LoginPasswordView::TestApi test_api(view_);
-  ui::test::EventGenerator& generator = GetEventGenerator();
+  ui::test::EventGenerator* generator = GetEventGenerator();
 
   // Submit 'a' password.
   EXPECT_TRUE(is_password_field_empty_);
-  generator.PressKey(ui::KeyboardCode::VKEY_A, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_A, 0);
   EXPECT_FALSE(is_password_field_empty_);
-  generator.PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
   EXPECT_FALSE(is_password_field_empty_);
   ASSERT_TRUE(password_.has_value());
   EXPECT_EQ(base::ASCIIToUTF16("a"), *password_);
@@ -146,9 +146,9 @@ TEST_F(LoginPasswordViewTest, PasswordSubmitClearsPassword) {
   EXPECT_TRUE(is_password_field_empty_);
 
   // Submit 'b' password.
-  generator.PressKey(ui::KeyboardCode::VKEY_B, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_B, 0);
   EXPECT_FALSE(is_password_field_empty_);
-  generator.PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
+  generator->PressKey(ui::KeyboardCode::VKEY_RETURN, 0);
   EXPECT_FALSE(is_password_field_empty_);
   ASSERT_TRUE(password_.has_value());
   // The submitted password is 'b' instead of "ab".
@@ -158,7 +158,7 @@ TEST_F(LoginPasswordViewTest, PasswordSubmitClearsPassword) {
 // Verifies that clicking the easy unlock icon fires the click event.
 TEST_F(LoginPasswordViewTest, EasyUnlockClickFiresEvent) {
   LoginPasswordView::TestApi test_api(view_);
-  ui::test::EventGenerator& generator = GetEventGenerator();
+  ui::test::EventGenerator* generator = GetEventGenerator();
 
   // Enable icon.
   view_->SetEasyUnlockIcon(mojom::EasyUnlockIconId::SPINNER,
@@ -167,17 +167,17 @@ TEST_F(LoginPasswordViewTest, EasyUnlockClickFiresEvent) {
 
   // Click to the right of the icon, call is not generated.
   EXPECT_FALSE(easy_unlock_icon_tapped_called_);
-  generator.MoveMouseTo(
+  generator->MoveMouseTo(
       test_api.easy_unlock_icon()->GetBoundsInScreen().bottom_right() +
       gfx::Vector2d(2, 0));
-  generator.ClickLeftButton();
+  generator->ClickLeftButton();
   EXPECT_FALSE(easy_unlock_icon_tapped_called_);
 
   // Click the icon.
   EXPECT_FALSE(easy_unlock_icon_tapped_called_);
-  generator.MoveMouseTo(
+  generator->MoveMouseTo(
       test_api.easy_unlock_icon()->GetBoundsInScreen().CenterPoint());
-  generator.ClickLeftButton();
+  generator->ClickLeftButton();
   EXPECT_TRUE(easy_unlock_icon_tapped_called_);
 
   // Icon was not hovered (since we did not enable immediate hover).
@@ -187,7 +187,7 @@ TEST_F(LoginPasswordViewTest, EasyUnlockClickFiresEvent) {
 // Verifies that hovering the icon fires the hover event.
 TEST_F(LoginPasswordViewTest, EasyUnlockMouseHover) {
   LoginPasswordView::TestApi test_api(view_);
-  ui::test::EventGenerator& generator = GetEventGenerator();
+  ui::test::EventGenerator* generator = GetEventGenerator();
 
   // Enable icon, enable immediate hovering.
   view_->SetEasyUnlockIcon(mojom::EasyUnlockIconId::SPINNER,
@@ -197,7 +197,7 @@ TEST_F(LoginPasswordViewTest, EasyUnlockMouseHover) {
 
   // Hover over the icon.
   EXPECT_FALSE(easy_unlock_icon_hovered_called_);
-  generator.MoveMouseTo(
+  generator->MoveMouseTo(
       test_api.easy_unlock_icon()->GetBoundsInScreen().CenterPoint());
   EXPECT_TRUE(easy_unlock_icon_hovered_called_);
 
