@@ -6,6 +6,7 @@
 #include "ash/login/ui/login_display_style.h"
 #include "ash/login/ui/login_test_base.h"
 #include "ash/login/ui/login_test_utils.h"
+#include "base/bind.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/gfx/geometry/rect.h"
@@ -23,12 +24,26 @@ class LoginUserViewUnittest : public LoginTestBase {
 
   // Builds a new LoginUserView instance and adds it to |container_|.
   LoginUserView* AddUserView(LoginDisplayStyle display_style,
-                             bool show_dropdown) {
+                             bool show_dropdown,
+                             bool public_account) {
+    LoginUserView::OnRemoveWarningShown on_remove_warning_shown;
+    LoginUserView::OnRemove on_remove;
+    if (show_dropdown) {
+      on_remove_warning_shown = base::BindRepeating(
+          &LoginUserViewUnittest::OnRemoveWarningShown, base::Unretained(this));
+      on_remove = base::BindRepeating(&LoginUserViewUnittest::OnRemove,
+                                      base::Unretained(this));
+    }
+
     auto* view =
-        new LoginUserView(display_style, show_dropdown,
+        new LoginUserView(display_style, show_dropdown, public_account,
                           base::BindRepeating(&LoginUserViewUnittest::OnTapped,
-                                              base::Unretained(this)));
-    mojom::LoginUserInfoPtr user = CreateUser("foo@foo.com");
+                                              base::Unretained(this)),
+                          on_remove_warning_shown, on_remove);
+
+    std::string email = "foo@foo.com";
+    mojom::LoginUserInfoPtr user =
+        public_account ? CreatePublicAccountUser(email) : CreateUser(email);
     view->UpdateForUser(user, false /*animate*/);
     container_->AddChildView(view);
     widget()->GetContentsView()->Layout();
@@ -51,11 +66,15 @@ class LoginUserViewUnittest : public LoginTestBase {
   }
 
   int tap_count_ = 0;
+  int remove_show_warning_count_ = 0;
+  int remove_count_ = 0;
 
   views::View* container_ = nullptr;  // Owned by test widget view hierarchy.
 
  private:
   void OnTapped() { ++tap_count_; }
+  void OnRemoveWarningShown() { ++remove_show_warning_count_; }
+  void OnRemove() { ++remove_count_; }
 
   DISALLOW_COPY_AND_ASSIGN(LoginUserViewUnittest);
 };
@@ -64,10 +83,15 @@ class LoginUserViewUnittest : public LoginTestBase {
 
 // Verifies that the user view does not change width for short/long usernames.
 TEST_F(LoginUserViewUnittest, DifferentUsernamesHaveSameWidth) {
-  LoginUserView* large = AddUserView(LoginDisplayStyle::kLarge, false);
-  LoginUserView* small = AddUserView(LoginDisplayStyle::kSmall, false);
+  LoginUserView* large =
+      AddUserView(LoginDisplayStyle::kLarge, false /*show_dropdown*/,
+                  false /*public_account*/);
+  LoginUserView* small =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
   LoginUserView* extra_small =
-      AddUserView(LoginDisplayStyle::kExtraSmall, false);
+      AddUserView(LoginDisplayStyle::kExtraSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
 
   int large_width = large->size().width();
   int small_width = small->size().width();
@@ -92,10 +116,15 @@ TEST_F(LoginUserViewUnittest, DifferentUsernamesHaveSameWidth) {
 // Verifies that the user views all have different sizes with different display
 // styles.
 TEST_F(LoginUserViewUnittest, DifferentStylesHaveDifferentSizes) {
-  LoginUserView* large = AddUserView(LoginDisplayStyle::kLarge, false);
-  LoginUserView* small = AddUserView(LoginDisplayStyle::kSmall, false);
+  LoginUserView* large =
+      AddUserView(LoginDisplayStyle::kLarge, false /*show_dropdown*/,
+                  false /*public_account*/);
+  LoginUserView* small =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
   LoginUserView* extra_small =
-      AddUserView(LoginDisplayStyle::kExtraSmall, false);
+      AddUserView(LoginDisplayStyle::kExtraSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
 
   EXPECT_NE(large->size(), gfx::Size());
   EXPECT_NE(large->size(), small->size());
@@ -106,8 +135,12 @@ TEST_F(LoginUserViewUnittest, DifferentStylesHaveDifferentSizes) {
 // Verifies that displaying the dropdown does not change the view size. Further,
 // the dropdown should not change the centering for the user label.
 TEST_F(LoginUserViewUnittest, DropdownDoesNotChangeSize) {
-  LoginUserView* with = AddUserView(LoginDisplayStyle::kLarge, true);
-  LoginUserView* without = AddUserView(LoginDisplayStyle::kLarge, false);
+  LoginUserView* with =
+      AddUserView(LoginDisplayStyle::kLarge, true /*show_dropdown*/,
+                  false /*public_account*/);
+  LoginUserView* without =
+      AddUserView(LoginDisplayStyle::kLarge, false /*show_dropdown*/,
+                  false /*public_account*/);
   EXPECT_NE(with->size(), gfx::Size());
   EXPECT_EQ(with->size(), without->size());
 
@@ -123,14 +156,16 @@ TEST_F(LoginUserViewUnittest, DropdownDoesNotChangeSize) {
 // Verifies that the entire user view is a tap target, and not just (for
 // example) the user icon.
 TEST_F(LoginUserViewUnittest, EntireViewIsTapTarget) {
-  LoginUserView* view = AddUserView(LoginDisplayStyle::kSmall, false);
+  LoginUserView* view =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
   EXPECT_NE(view->size(), gfx::Size());
 
   // Returns true if there is a tap at |point| offset by |dx|, |dy|.
   auto tap = [this](gfx::Point point, int dx, int dy) -> bool {
     point.Offset(dx, dy);
-    GetEventGenerator().MoveMouseTo(point);
-    GetEventGenerator().ClickLeftButton();
+    GetEventGenerator()->MoveMouseTo(point);
+    GetEventGenerator()->ClickLeftButton();
     bool result = tap_count_ == 1;
     tap_count_ = 0;
     return result;
@@ -150,8 +185,12 @@ TEST_F(LoginUserViewUnittest, EntireViewIsTapTarget) {
 // Verifies the focused user view is opaque. Verifies that a hovered view is
 // opaque. Verifies the interaction between focus and hovered opaqueness.
 TEST_F(LoginUserViewUnittest, FocusHoverOpaqueInteractions) {
-  LoginUserView* one = AddUserView(LoginDisplayStyle::kSmall, false);
-  LoginUserView* two = AddUserView(LoginDisplayStyle::kSmall, false);
+  LoginUserView* one =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
+  LoginUserView* two =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
   LoginUserView::TestApi one_test(one);
   LoginUserView::TestApi two_test(two);
 
@@ -168,17 +207,17 @@ TEST_F(LoginUserViewUnittest, FocusHoverOpaqueInteractions) {
   EXPECT_TRUE(two_test.is_opaque());
 
   // Non-focused element can be opaque if the mouse is over it.
-  GetEventGenerator().MoveMouseTo(one->GetBoundsInScreen().CenterPoint());
+  GetEventGenerator()->MoveMouseTo(one->GetBoundsInScreen().CenterPoint());
   EXPECT_TRUE(one_test.is_opaque());
   EXPECT_TRUE(two_test.is_opaque());
 
   // Focused element stays opaque when mouse is over it.
-  GetEventGenerator().MoveMouseTo(two->GetBoundsInScreen().CenterPoint());
+  GetEventGenerator()->MoveMouseTo(two->GetBoundsInScreen().CenterPoint());
   EXPECT_FALSE(one_test.is_opaque());
   EXPECT_TRUE(two_test.is_opaque());
 
   // Focused element stays opaque when mouse leaves it.
-  GetEventGenerator().MoveMouseTo(one->GetBoundsInScreen().CenterPoint());
+  GetEventGenerator()->MoveMouseTo(one->GetBoundsInScreen().CenterPoint());
   EXPECT_TRUE(one_test.is_opaque());
   EXPECT_TRUE(two_test.is_opaque());
 
@@ -192,8 +231,12 @@ TEST_F(LoginUserViewUnittest, FocusHoverOpaqueInteractions) {
 // focus, and that a forced opaque element can transition to both
 // opaque/transparent when losing forced opaque.
 TEST_F(LoginUserViewUnittest, ForcedOpaque) {
-  LoginUserView* one = AddUserView(LoginDisplayStyle::kSmall, false);
-  LoginUserView* two = AddUserView(LoginDisplayStyle::kSmall, false);
+  LoginUserView* one =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
+  LoginUserView* two =
+      AddUserView(LoginDisplayStyle::kSmall, false /*show_dropdown*/,
+                  false /*public_account*/);
   LoginUserView::TestApi one_test(one);
   LoginUserView::TestApi two_test(two);
 
@@ -230,4 +273,42 @@ TEST_F(LoginUserViewUnittest, ForcedOpaque) {
   EXPECT_TRUE(two_test.is_opaque());
 }
 
+// Verifies that a long user name does not push the label or dropdown button
+// outside of the LoginUserView bounds.
+TEST_F(LoginUserViewUnittest, ElideUserLabel) {
+  LoginUserView* view =
+      AddUserView(LoginDisplayStyle::kLarge, true /*show_dropdown*/,
+                  false /*public_account*/);
+  LoginUserView::TestApi view_test(view);
+
+  mojom::LoginUserInfoPtr user =
+      CreateUser("verylongusernamethatfillsthebox@domain.com");
+  view->UpdateForUser(user, false /*animate*/);
+  container_->Layout();
+
+  EXPECT_TRUE(view->GetVisibleBounds().Contains(
+      view_test.user_label()->GetVisibleBounds()));
+  EXPECT_TRUE(view->GetVisibleBounds().Contains(
+      view_test.dropdown()->GetVisibleBounds()));
+}
+
+// Verifies that displaying the domain does not change the view width.
+// Also domain should have the same horizontal centering as user label.
+TEST_F(LoginUserViewUnittest, DomainDoesNotChangeWidth) {
+  LoginUserView* public_account =
+      AddUserView(LoginDisplayStyle::kLarge, false /*show_dropdown*/,
+                  true /*public_account*/);
+  LoginUserView* regular_user =
+      AddUserView(LoginDisplayStyle::kLarge, false /*show_dropdown*/,
+                  false /*public_account*/);
+  EXPECT_NE(regular_user->size().width(), 0);
+  EXPECT_EQ(regular_user->size().width(), public_account->size().width());
+
+  views::View* user_label = LoginUserView::TestApi(public_account).user_label();
+  views::View* user_domain =
+      LoginUserView::TestApi(public_account).user_label();
+  EXPECT_EQ(user_label->GetBoundsInScreen().CenterPoint().x(),
+            user_domain->GetBoundsInScreen().CenterPoint().x());
+}
+
 }  // namespace ash
