@@ -8,7 +8,7 @@
 
 #include "ash/accessibility/accessibility_controller.h"
 #include "ash/host/ash_window_tree_host.h"
-#include "ash/magnifier/magnifier_scale_utils.h"
+#include "ash/magnifier/magnifier_utils.h"
 #include "ash/public/cpp/ash_pref_names.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/root_window_controller.h"
@@ -16,7 +16,9 @@
 #include "ash/shelf/shelf.h"
 #include "ash/shelf/shelf_layout_manager.h"
 #include "ash/shell.h"
-#include "ash/system/tray/system_tray_notifier.h"
+#include "ash/wm/overview/overview_controller.h"
+#include "ash/wm/splitview/split_view_controller.h"
+#include "base/bind.h"
 #include "base/numerics/ranges.h"
 #include "components/prefs/pref_change_registrar.h"
 #include "components/prefs/pref_registry_simple.h"
@@ -24,10 +26,12 @@
 #include "ui/aura/client/drag_drop_client.h"
 #include "ui/aura/env.h"
 #include "ui/aura/window_tree_host.h"
+#include "ui/base/ime/ime_bridge.h"
 #include "ui/base/ime/input_method.h"
 #include "ui/base/ime/text_input_client.h"
 #include "ui/compositor/layer.h"
 #include "ui/compositor/reflector.h"
+#include "ui/compositor/scoped_layer_animation_settings.h"
 #include "ui/display/screen.h"
 #include "ui/views/widget/widget.h"
 #include "ui/wm/core/coordinate_conversion.h"
@@ -48,7 +52,7 @@ constexpr char kDockedMagnifierViewportWindowName[] =
 
 // Returns true if High Contrast mode is enabled.
 bool IsHighContrastEnabled() {
-  return Shell::Get()->accessibility_controller()->IsHighContrastEnabled();
+  return Shell::Get()->accessibility_controller()->high_contrast_enabled();
 }
 
 // Returns the current cursor location in screen coordinates.
@@ -56,15 +60,6 @@ inline gfx::Point GetCursorScreenPoint() {
   return display::Screen::GetScreen()->GetCursorScreenPoint();
 }
 
-// Returns the InputMethod associated with the WindowTreeHost of the given
-// window.
-ui::InputMethod* GetInputMethodForWindow(aura::Window* window) {
-  DCHECK(window);
-  aura::WindowTreeHost* host = window->GetHost();
-  DCHECK(host);
-  return host->GetInputMethod();
-}
-
 // Updates the workarea of the display associated with |window| such that the
 // given magnifier viewport |height| is allocated at the top of the screen.
 void SetViewportHeightInWorkArea(aura::Window* window, int height) {
@@ -85,7 +80,7 @@ gfx::Rect GetViewportWidgetBoundsInRoot(aura::Window* root) {
 
   auto root_bounds = root->GetBoundsInRootWindow();
   root_bounds.set_height(root_bounds.height() /
-                         DockedMagnifierController::kScreenHeightDevisor);
+                         DockedMagnifierController::kScreenHeightDivisor);
   return root_bounds;
 }
 
@@ -131,9 +126,17 @@ aura::Window* GetViewportParentContainerForRoot(aura::Window* root) {
 
 DockedMagnifierController::DockedMagnifierController() : binding_(this) {
   Shell::Get()->session_controller()->AddObserver(this);
+  if (ui::IMEBridge::Get())
+    ui::IMEBridge::Get()->AddObserver(this);
 }
 
 DockedMagnifierController::~DockedMagnifierController() {
+  if (input_method_)
+    input_method_->RemoveObserver(this);
+  input_method_ = nullptr;
+  if (ui::IMEBridge::Get())
+    ui::IMEBridge::Get()->RemoveObserver(this);
+
   Shell* shell = Shell::Get();
   shell->session_controller()->RemoveObserver(this);
 
@@ -145,9 +148,16 @@ DockedMagnifierController::~DockedMagnifierController() {
 
 // static
 void DockedMagnifierController::RegisterProfilePrefs(
-    PrefRegistrySimple* registry) {
-  registry->RegisterBooleanPref(prefs::kDockedMagnifierEnabled, false,
-                                PrefRegistry::PUBLIC);
+    PrefRegistrySimple* registry,
+    bool for_test) {
+  if (for_test) {
+    // In tests there is no remote pref service. Make ash own the prefs.
+    registry->RegisterBooleanPref(prefs::kDockedMagnifierEnabled, false,
+                                  PrefRegistry::PUBLIC);
+  } else {
+    // TODO(warx): move ownership to ash.
+    registry->RegisterForeignPref(prefs::kDockedMagnifierEnabled);
+  }
   registry->RegisterDoublePref(prefs::kDockedMagnifierScale,
                                kDefaultMagnifierScale, PrefRegistry::PUBLIC);
 }
@@ -185,16 +195,10 @@ void DockedMagnifierController::SetScale(float scale) {
 }
 
 void DockedMagnifierController::StepToNextScaleValue(int delta_index) {
-  SetScale(magnifier_scale_utils::GetNextMagnifierScaleValue(
+  SetScale(magnifier_utils::GetNextMagnifierScaleValue(
       delta_index, GetScale(), kMinMagnifierScale, kMaxMagnifierScale));
 }
 
-void DockedMagnifierController::SetClient(
-    mojom::DockedMagnifierClientPtr client) {
-  client_ = std::move(client);
-  NotifyClientWithStatusChanged();
-}
-
 void DockedMagnifierController::CenterOnPoint(
     const gfx::Point& point_in_screen) {
   if (!GetEnabled())
@@ -202,8 +206,14 @@ void DockedMagnifierController::CenterOnPoint(
 
   auto* screen = display::Screen::GetScreen();
   auto* window = screen->GetWindowAtScreenPoint(point_in_screen);
-  if (!window)
-    return;
+  if (!window) {
+    // In tests and sometimes initially on signin screen, |point_in_screen|
+    // maybe invalid and doesn't belong to any existing root window. However, we
+    // are here because the Docked Magnifier is enabled. We need to create the
+    // viewport widget somewhere, so we'll use the primary root window until we
+    // get a valid cursor event.
+    window = Shell::GetPrimaryRootWindow();
+  }
 
   auto* root_window = window->GetRootWindow();
   DCHECK(root_window);
@@ -280,6 +290,11 @@ void DockedMagnifierController::OnActiveUserPrefServiceChanged(
   InitFromUserPrefs();
 }
 
+void DockedMagnifierController::OnSigninScreenPrefServiceInitialized(
+    PrefService* prefs) {
+  OnActiveUserPrefServiceChanged(prefs);
+}
+
 void DockedMagnifierController::OnMouseEvent(ui::MouseEvent* event) {
   DCHECK(GetEnabled());
   CenterOnPoint(GetCursorScreenPoint());
@@ -300,16 +315,57 @@ void DockedMagnifierController::OnScrollEvent(ui::ScrollEvent* event) {
     // Notes: - Clamping of the new scale value happens inside SetScale().
     //        - Refreshing the viewport happens in the handler of the scale pref
     //          changes.
-    SetScale(magnifier_scale_utils::GetScaleFromScroll(
+    SetScale(magnifier_utils::GetScaleFromScroll(
         event->y_offset() * kScrollScaleFactor, GetScale(), kMaxMagnifierScale,
         kMinMagnifierScale));
     event->StopPropagation();
   }
 }
 
+void DockedMagnifierController::OnTouchEvent(ui::TouchEvent* event) {
+  DCHECK(GetEnabled());
+
+  aura::Window* target = static_cast<aura::Window*>(event->target());
+  aura::Window* event_root = target->GetRootWindow();
+  gfx::Point event_screen_point = event->root_location();
+  ::wm::ConvertPointToScreen(event_root, &event_screen_point);
+  CenterOnPoint(event_screen_point);
+}
+
+void DockedMagnifierController::OnInputContextHandlerChanged() {
+  if (!GetEnabled())
+    return;
+
+  auto* new_input_method =
+      magnifier_utils::GetInputMethod(current_source_root_window_);
+  if (new_input_method == input_method_)
+    return;
+
+  if (input_method_)
+    input_method_->RemoveObserver(this);
+  input_method_ = new_input_method;
+  if (input_method_)
+    input_method_->AddObserver(this);
+}
+
+void DockedMagnifierController::OnInputMethodDestroyed(
+    const ui::InputMethod* input_method) {
+  DCHECK_EQ(input_method, input_method_);
+  input_method_->RemoveObserver(this);
+  input_method_ = nullptr;
+}
+
 void DockedMagnifierController::OnCaretBoundsChanged(
     const ui::TextInputClient* client) {
-  DCHECK(GetEnabled());
+  if (!GetEnabled()) {
+    // There is a small window between the time the "enabled" pref is updated to
+    // false, and the time we're notified with this change, upon which we remove
+    // the magnifier's viewport widget and stop observing the input method.
+    // During this short interval, if focus is in an editable element, the input
+    // method can notify us here. In this case, we should just return.
+    return;
+  }
+
   aura::client::DragDropClient* drag_drop_client =
       aura::client::GetDragDropClient(current_source_root_window_);
   if (drag_drop_client && drag_drop_client->IsDragDropInProgress()) {
@@ -381,10 +437,6 @@ void DockedMagnifierController::SetFullscreenMagnifierEnabled(bool enabled) {
   }
 }
 
-void DockedMagnifierController::FlushClientPtrForTesting() {
-  client_.FlushForTesting();
-}
-
 const views::Widget* DockedMagnifierController::GetViewportWidgetForTesting()
     const {
   return viewport_widget_;
@@ -421,10 +473,10 @@ void DockedMagnifierController::SwitchCurrentSourceRootWindowIfNeeded(
     }
     if (update_old_root_workarea)
       SetViewportHeightInWorkArea(old_root_window, 0);
-    ui::InputMethod* old_input_method =
-        GetInputMethodForWindow(old_root_window);
-    if (old_input_method)
-      old_input_method->RemoveObserver(this);
+
+    if (input_method_)
+      input_method_->RemoveObserver(this);
+    input_method_ = nullptr;
 
     // Reset mouse cursor confinement to default.
     RootWindowController::ForWindow(old_root_window)
@@ -436,7 +488,7 @@ void DockedMagnifierController::SwitchCurrentSourceRootWindowIfNeeded(
   // reflector and the viewport widget and its layers. New viewport and
   // reflector may be recreated later if |new_root_window| is not |nullptr|.
   if (reflector_) {
-    aura::Env::GetInstance()->context_factory_private()->RemoveReflector(
+    Shell::Get()->aura_env()->context_factory_private()->RemoveReflector(
         reflector_.get());
     reflector_.reset();
   }
@@ -459,15 +511,14 @@ void DockedMagnifierController::SwitchCurrentSourceRootWindowIfNeeded(
 
   CreateMagnifierViewport();
 
-  ui::InputMethod* new_input_method =
-      GetInputMethodForWindow(current_source_root_window_);
-  if (new_input_method)
-    new_input_method->AddObserver(this);
+  input_method_ = magnifier_utils::GetInputMethod(current_source_root_window_);
+  if (input_method_)
+    input_method_->AddObserver(this);
 
-  DCHECK(aura::Env::GetInstance()->context_factory_private());
+  DCHECK(Shell::Get()->aura_env()->context_factory_private());
   DCHECK(viewport_widget_);
   reflector_ =
-      aura::Env::GetInstance()->context_factory_private()->CreateReflector(
+      Shell::Get()->aura_env()->context_factory_private()->CreateReflector(
           current_source_root_window_->layer()->GetCompositor(),
           viewport_magnifier_layer_.get());
 }
@@ -497,12 +548,42 @@ void DockedMagnifierController::InitFromUserPrefs() {
           base::Unretained(this)));
 
   OnEnabledPrefChanged();
-  NotifyClientWithStatusChanged();
 }
 
 void DockedMagnifierController::OnEnabledPrefChanged() {
+  // When switching from the signin screen to a newly created profile while the
+  // Docked Magnifier is enabled, the prefs will be copied from the signin
+  // profile to the user profile, and the Docked Magnifier will remain enabled.
+  // We don't want to redo the below operations if the status doesn't change,
+  // for example readding the same observer to the WindowTreeHostManager will
+  // cause a crash on DCHECK on debug builds.
+  const bool current_enabled = !!current_source_root_window_;
+  const bool new_enabled = GetEnabled();
+  if (current_enabled == new_enabled)
+    return;
+
+  // Toggling the status of the docked magnifier, changes the display's work
+  // area. However, display's work area changes are not allowed while overview
+  // mode is active (See https://crbug.com/834400). For this reason, we exit
+  // overview mode, before we actually update the state of docked magnifier
+  // below. https://crbug.com/894256.
   Shell* shell = Shell::Get();
-  if (GetEnabled()) {
+  auto* overview_controller = shell->overview_controller();
+  if (overview_controller->IsSelecting()) {
+    auto* split_view_controller = shell->split_view_controller();
+    if (split_view_controller->IsSplitViewModeActive()) {
+      // In this case, we're in a single-split-view mode, i.e. a window is
+      // snapped to one side of the split view, while the other side has
+      // overview active. We need to exit split view as well as exiting overview
+      // mode, otherwise we'll be in an invalid state.
+      split_view_controller->EndSplitView(
+          SplitViewController::EndReason::kNormal);
+    }
+
+    overview_controller->ToggleOverview();
+  }
+
+  if (new_enabled) {
     // Enabling the Docked Magnifier disables the Fullscreen Magnifier.
     SetFullscreenMagnifierEnabled(false);
     // Calling refresh will result in the creation of the magnifier viewport and
@@ -510,7 +591,7 @@ void DockedMagnifierController::OnEnabledPrefChanged() {
     Refresh();
     // Make sure we are in front of the fullscreen magnifier which also handles
     // scroll events.
-    shell->PrependPreTargetHandler(this);
+    shell->AddPreTargetHandler(this, ui::EventTarget::Priority::kSystem);
     shell->window_tree_host_manager()->AddObserver(this);
   } else {
     shell->window_tree_host_manager()->RemoveObserver(this);
@@ -524,14 +605,11 @@ void DockedMagnifierController::OnEnabledPrefChanged() {
 
   // Update the green checkmark status in the accessibility menu in the system
   // tray.
-  shell->system_tray_notifier()->NotifyAccessibilityStatusChanged(
-      A11Y_NOTIFICATION_NONE);
+  shell->accessibility_controller()->NotifyAccessibilityStatusChanged();
 
   // We use software composited mouse cursor so that it can be mirrored into the
   // magnifier viewport.
   shell->UpdateCursorCompositingEnabled();
-
-  NotifyClientWithStatusChanged();
 }
 
 void DockedMagnifierController::OnScalePrefChanged() {
@@ -561,11 +639,6 @@ void DockedMagnifierController::Refresh() {
   CenterOnPoint(GetCursorScreenPoint());
 }
 
-void DockedMagnifierController::NotifyClientWithStatusChanged() {
-  if (client_)
-    client_->OnEnabledStatusChanged(GetEnabled());
-}
-
 void DockedMagnifierController::CreateMagnifierViewport() {
   DCHECK(GetEnabled());
   DCHECK(current_source_root_window_);
