@@ -12,10 +12,8 @@
 #include "ash/display/window_tree_host_manager.h"
 #include "ash/host/ash_window_tree_host.h"
 #include "ash/magnifier/magnifier_test_utils.h"
-#include "ash/public/cpp/ash_features.h"
 #include "ash/public/cpp/ash_pref_names.h"
 #include "ash/public/cpp/ash_switches.h"
-#include "ash/public/cpp/config.h"
 #include "ash/public/interfaces/docked_magnifier_controller.mojom.h"
 #include "ash/session/session_controller.h"
 #include "ash/session/test_session_controller_client.h"
@@ -23,17 +21,20 @@
 #include "ash/shell.h"
 #include "ash/test/ash_test_base.h"
 #include "ash/test/ash_test_helper.h"
+#include "ash/wm/overview/overview_controller.h"
+#include "ash/wm/splitview/split_view_controller.h"
+#include "ash/wm/tablet_mode/tablet_mode_controller.h"
+#include "ash/wm/window_state.h"
 #include "base/command_line.h"
-#include "base/test/scoped_feature_list.h"
 #include "components/prefs/pref_service.h"
 #include "components/session_manager/session_manager_types.h"
-#include "mojo/public/cpp/bindings/binding.h"
 #include "ui/aura/window.h"
 #include "ui/compositor/layer.h"
 #include "ui/display/display.h"
 #include "ui/display/manager/managed_display_info.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/views/widget/widget.h"
+#include "ui/wm/core/coordinate_conversion.h"
 
 namespace ash {
 
@@ -42,33 +43,11 @@ namespace {
 constexpr char kUser1Email[] = "user1@dockedmagnifier";
 constexpr char kUser2Email[] = "user2@dockedmagnifier";
 
-// Mock mojo client of the Docked Magnifier.
-class DockedMagnifierTestClient : public mojom::DockedMagnifierClient {
- public:
-  DockedMagnifierTestClient() : binding_(this) {}
-  ~DockedMagnifierTestClient() override = default;
-
-  bool docked_magnifier_enabled() const { return docked_magnifier_enabled_; }
-
-  // Connects to the DockedMagnifierController.
-  void Start() {
-    ash::mojom::DockedMagnifierClientPtr client;
-    binding_.Bind(mojo::MakeRequest(&client));
-    Shell::Get()->docked_magnifier_controller()->SetClient(std::move(client));
-  }
-
-  // ash::mojom::DockedMagnifierClient:
-  void OnEnabledStatusChanged(bool enabled) override {
-    docked_magnifier_enabled_ = enabled;
-  }
-
- private:
-  mojo::Binding<ash::mojom::DockedMagnifierClient> binding_;
-
-  bool docked_magnifier_enabled_ = false;
-
-  DISALLOW_COPY_AND_ASSIGN(DockedMagnifierTestClient);
-};
+// Returns the magnifier area height given the display height.
+int GetMagnifierHeight(int display_height) {
+  return (display_height / DockedMagnifierController::kScreenHeightDivisor) +
+         DockedMagnifierController::kSeparatorHeight;
+}
 
 class DockedMagnifierTest : public NoSessionAshTestBase {
  public:
@@ -79,8 +58,6 @@ class DockedMagnifierTest : public NoSessionAshTestBase {
     return Shell::Get()->docked_magnifier_controller();
   }
 
-  DockedMagnifierTestClient* test_client() { return &test_client_; }
-
   PrefService* user1_pref_service() {
     return Shell::Get()->session_controller()->GetUserPrefServiceForUser(
         AccountId::FromUserEmail(kUser1Email));
@@ -93,9 +70,6 @@ class DockedMagnifierTest : public NoSessionAshTestBase {
 
   // AshTestBase:
   void SetUp() override {
-    // Explicitly enable the Docked Magnifier feature for the tests.
-    scoped_feature_list_.InitAndEnableFeature(features::kDockedMagnifier);
-
     // Explicitly enable --ash-constrain-pointer-to-root to be able to test
     // mouse cursor confinement outside the magnifier viewport.
     base::CommandLine::ForCurrentProcess()->AppendSwitch(
@@ -109,7 +83,8 @@ class DockedMagnifierTest : public NoSessionAshTestBase {
     // Create user 2 session.
     GetSessionControllerClient()->AddUserSession(kUser2Email);
 
-    test_client_.Start();
+    // Place the cursor in the first display.
+    GetEventGenerator()->MoveMouseTo(gfx::Point(0, 0));
   }
 
   void SwitchActiveUser(const std::string& email) {
@@ -117,14 +92,49 @@ class DockedMagnifierTest : public NoSessionAshTestBase {
         AccountId::FromUserEmail(email));
   }
 
- protected:
-  MagnifierTextInputTestHelper text_input_helper_;
+  // Tests that when the magnifier layer's transform is applied on the point in
+  // the |root_window| coordinates that corresponds to the
+  // |point_of_interest_in_screen|, the resulting point is at the center of the
+  // magnifier viewport widget.
+  void TestMagnifierLayerTransform(
+      const gfx::Point& point_of_interest_in_screen,
+      const aura::Window* root_window) {
+    // Convert to root coordinates.
+    gfx::Point point_of_interest_in_root = point_of_interest_in_screen;
+    ::wm::ConvertPointFromScreen(root_window, &point_of_interest_in_root);
+    // Account for point of interest being outside the minimum height threshold.
+    // Do this in gfx::PointF to avoid rounding errors.
+    gfx::PointF point_of_interest_in_root_f(point_of_interest_in_root);
+    const float min_pov_height =
+        controller()->GetMinimumPointOfInterestHeightForTesting();
+    if (point_of_interest_in_root_f.y() < min_pov_height)
+      point_of_interest_in_root_f.set_y(min_pov_height);
+
+    const ui::Layer* magnifier_layer =
+        controller()->GetViewportMagnifierLayerForTesting();
+    // The magnifier layer's transform, when applied to the point of interest
+    // (in root coordinates), should take it to the point at the center of the
+    // viewport widget (also in root coordinates).
+    magnifier_layer->transform().TransformPoint(&point_of_interest_in_root_f);
+    const views::Widget* viewport_widget =
+        controller()->GetViewportWidgetForTesting();
+    const gfx::Point viewport_center_in_root =
+        viewport_widget->GetNativeWindow()
+            ->GetBoundsInRootWindow()
+            .CenterPoint();
+    EXPECT_EQ(viewport_center_in_root,
+              gfx::ToFlooredPoint(point_of_interest_in_root_f));
+  }
+
+  void TouchPoint(const gfx::Point& touch_point_in_screen) {
+    // TODO(oshima): Currently touch event doesn't update the
+    // event dispatcher in the event generator. Fix it and use
+    // touch event insteead.
+    auto* generator = GetEventGenerator();
+    generator->GestureTapAt(touch_point_in_screen);
+  }
 
  private:
-  DockedMagnifierTestClient test_client_;
-
-  base::test::ScopedFeatureList scoped_feature_list_;
-
   DISALLOW_COPY_AND_ASSIGN(DockedMagnifierTest);
 };
 
@@ -154,37 +164,17 @@ TEST_F(DockedMagnifierTest, MutuallyExclusiveMagnifiers) {
   EXPECT_FALSE(controller()->GetFullscreenMagnifierEnabled());
 }
 
-// Tests the changes in the magnifier's status, user switches and the
-// interaction with the client.
+// Tests the changes in the magnifier's status, user switches.
 TEST_F(DockedMagnifierTest, TestEnableAndDisable) {
-  // Client should receive status updates.
-  EXPECT_FALSE(controller()->GetEnabled());
-  EXPECT_FALSE(test_client()->docked_magnifier_enabled());
+  // Enable for user 1, and switch to user 2. User 2 should have it disabled.
   controller()->SetEnabled(true);
-  controller()->FlushClientPtrForTesting();
   EXPECT_TRUE(controller()->GetEnabled());
-  EXPECT_TRUE(test_client()->docked_magnifier_enabled());
-  controller()->SetEnabled(false);
-  controller()->FlushClientPtrForTesting();
-  EXPECT_FALSE(controller()->GetEnabled());
-  EXPECT_FALSE(test_client()->docked_magnifier_enabled());
-
-  // Enable again for user 1, and switch to user 2. User 2 should have it
-  // disabled, the client should be updated accordingly.
-  controller()->SetEnabled(true);
-  controller()->FlushClientPtrForTesting();
-  EXPECT_TRUE(controller()->GetEnabled());
-  EXPECT_TRUE(test_client()->docked_magnifier_enabled());
   SwitchActiveUser(kUser2Email);
-  controller()->FlushClientPtrForTesting();
   EXPECT_FALSE(controller()->GetEnabled());
-  EXPECT_FALSE(test_client()->docked_magnifier_enabled());
 
   // Switch back to user 1, expect it to be enabled.
   SwitchActiveUser(kUser1Email);
-  controller()->FlushClientPtrForTesting();
   EXPECT_TRUE(controller()->GetEnabled());
-  EXPECT_TRUE(test_client()->docked_magnifier_enabled());
 }
 
 // Tests the magnifier's scale changes.
@@ -211,17 +201,12 @@ TEST_F(DockedMagnifierTest, TestScale) {
 // DockedMagnifierController (such as Settings UI) are observed and applied.
 TEST_F(DockedMagnifierTest, TestOutsidePrefsUpdates) {
   EXPECT_FALSE(controller()->GetEnabled());
-  EXPECT_FALSE(test_client()->docked_magnifier_enabled());
   user1_pref_service()->SetBoolean(prefs::kDockedMagnifierEnabled, true);
-  controller()->FlushClientPtrForTesting();
   EXPECT_TRUE(controller()->GetEnabled());
-  EXPECT_TRUE(test_client()->docked_magnifier_enabled());
   user1_pref_service()->SetDouble(prefs::kDockedMagnifierScale, 7.3f);
   EXPECT_FLOAT_EQ(7.3f, controller()->GetScale());
   user1_pref_service()->SetBoolean(prefs::kDockedMagnifierEnabled, false);
-  controller()->FlushClientPtrForTesting();
   EXPECT_FALSE(controller()->GetEnabled());
-  EXPECT_FALSE(test_client()->docked_magnifier_enabled());
 }
 
 // Tests that the workareas of displays are adjusted properly when the Docked
@@ -232,7 +217,7 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
   ASSERT_EQ(2u, root_windows.size());
 
   // Place the cursor in the first display.
-  GetEventGenerator().MoveMouseTo(gfx::Point(0, 0));
+  GetEventGenerator()->MoveMouseTo(gfx::Point(0, 0));
 
   // Before the magnifier is enabled, the work areas of both displays are their
   // full size minus the shelf height.
@@ -240,7 +225,7 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
   const gfx::Rect disp_1_bounds(0, 0, 800, 600);
   EXPECT_EQ(disp_1_bounds, display_1.bounds());
   gfx::Rect disp_1_workarea_no_magnifier = disp_1_bounds;
-  disp_1_workarea_no_magnifier.Inset(0, 0, 0, kShelfSize);
+  disp_1_workarea_no_magnifier.Inset(0, 0, 0, ShelfConstants::shelf_size());
   EXPECT_EQ(disp_1_workarea_no_magnifier, display_1.work_area());
   // At this point, normal mouse cursor confinement should be used.
   AshWindowTreeHost* host1 =
@@ -254,7 +239,7 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
   const gfx::Rect disp_2_bounds(800, 0, 400, 300);
   EXPECT_EQ(disp_2_bounds, display_2.bounds());
   gfx::Rect disp_2_workarea_no_magnifier = disp_2_bounds;
-  disp_2_workarea_no_magnifier.Inset(0, 0, 0, kShelfSize);
+  disp_2_workarea_no_magnifier.Inset(0, 0, 0, ShelfConstants::shelf_size());
   EXPECT_EQ(disp_2_workarea_no_magnifier, display_2.work_area());
   AshWindowTreeHost* host2 =
       Shell::Get()
@@ -276,9 +261,7 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
   // the separator layer.
   gfx::Rect disp_1_workspace_with_magnifier = disp_1_workarea_no_magnifier;
   const int disp_1_magnifier_height =
-      (disp_1_bounds.height() /
-       DockedMagnifierController::kScreenHeightDevisor) +
-      DockedMagnifierController::kSeparatorHeight;
+      GetMagnifierHeight(disp_1_bounds.height());
   disp_1_workspace_with_magnifier.Inset(0, disp_1_magnifier_height, 0, 0);
   EXPECT_EQ(disp_1_bounds, display_1.bounds());
   EXPECT_EQ(disp_1_workspace_with_magnifier, display_1.work_area());
@@ -298,7 +281,7 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
   // Now, move mouse cursor to display 2, and expect that the workarea of
   // display 1 is restored to its original value, while that of display 2 is
   // shrunk to fit the Docked Magnifier's viewport.
-  GetEventGenerator().MoveMouseTo(gfx::Point(800, 0));
+  GetEventGenerator()->MoveMouseTo(gfx::Point(800, 0));
   const views::Widget* viewport_2_widget =
       controller()->GetViewportWidgetForTesting();
   ASSERT_NE(nullptr, viewport_2_widget);
@@ -313,9 +296,7 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
   EXPECT_EQ(disp_2_bounds, display_2.bounds());
   gfx::Rect disp_2_workspace_with_magnifier = disp_2_workarea_no_magnifier;
   const int disp_2_magnifier_height =
-      (disp_2_bounds.height() /
-       DockedMagnifierController::kScreenHeightDevisor) +
-      DockedMagnifierController::kSeparatorHeight;
+      GetMagnifierHeight(disp_2_bounds.height());
   disp_2_workspace_with_magnifier.Inset(0, disp_2_magnifier_height, 0, 0);
   EXPECT_EQ(disp_2_workspace_with_magnifier, display_2.work_area());
   // Display 2's mouse is confined outside the viewport.
@@ -339,6 +320,150 @@ TEST_F(DockedMagnifierTest, DisplaysWorkAreas) {
             gfx::Rect(gfx::Point(0, 0), disp_2_bounds.size()));
 }
 
+// Test that we exit overview mode when enabling the docked magnifier.
+TEST_F(DockedMagnifierTest, DisplaysWorkAreasOverviewMode) {
+  std::unique_ptr<aura::Window> window(
+      CreateTestWindowInShell(SK_ColorWHITE, 100, gfx::Rect(0, 0, 200, 200)));
+  wm::GetWindowState(window.get())->Maximize();
+
+  // Enable overview mode followed by the magnifier.
+  auto* overview_controller = Shell::Get()->overview_controller();
+  overview_controller->ToggleOverview();
+  EXPECT_TRUE(overview_controller->IsSelecting());
+  controller()->SetEnabled(true);
+  EXPECT_TRUE(controller()->GetEnabled());
+
+  // Expect that overview mode is exited, the display's work area is updated,
+  // and the window's bounds are updated to be equal to the new display's work
+  // area bounds.
+  EXPECT_FALSE(overview_controller->IsSelecting());
+  const display::Display& display = display_manager()->GetDisplayAt(0);
+  gfx::Rect workarea = display.bounds();
+  const int magnifier_height = GetMagnifierHeight(display.bounds().height());
+  workarea.Inset(0, magnifier_height, 0, ShelfConstants::shelf_size());
+  EXPECT_EQ(workarea, display.work_area());
+  EXPECT_EQ(workarea, window->bounds());
+  EXPECT_TRUE(wm::GetWindowState(window.get())->IsMaximized());
+}
+
+// Test that we exist split view and over view modes when a single window is
+// snapped and the other snap region is hosting overview mode.
+TEST_F(DockedMagnifierTest, DisplaysWorkAreasSingleSplitView) {
+  // Verify that we're in tablet mode.
+  Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(true);
+  EXPECT_TRUE(Shell::Get()
+                  ->tablet_mode_controller()
+                  ->IsTabletModeWindowManagerEnabled());
+
+  std::unique_ptr<aura::Window> window(
+      CreateTestWindowInShell(SK_ColorWHITE, 100, gfx::Rect(0, 0, 200, 200)));
+  wm::GetWindowState(window.get())->Maximize();
+
+  auto* split_view_controller = Shell::Get()->split_view_controller();
+  EXPECT_EQ(split_view_controller->state(), SplitViewController::NO_SNAP);
+  EXPECT_EQ(split_view_controller->IsSplitViewModeActive(), false);
+
+  // Simulate going into split view, by enabling overview mode, and snapping
+  // a window to the left.
+  auto* overview_controller = Shell::Get()->overview_controller();
+  overview_controller->ToggleOverview();
+  EXPECT_TRUE(overview_controller->IsSelecting());
+  split_view_controller->SnapWindow(window.get(), SplitViewController::LEFT);
+  EXPECT_EQ(split_view_controller->state(), SplitViewController::LEFT_SNAPPED);
+  EXPECT_EQ(split_view_controller->left_window(), window.get());
+  EXPECT_TRUE(overview_controller->IsSelecting());
+
+  // Enable the docked magnifier and expect that both overview and split view
+  // modes are exited, and the window remains maximized, and its bounds are
+  // updated to match the new display's work area.
+  controller()->SetEnabled(true);
+  EXPECT_TRUE(controller()->GetEnabled());
+  EXPECT_FALSE(overview_controller->IsSelecting());
+  EXPECT_EQ(split_view_controller->state(), SplitViewController::NO_SNAP);
+  EXPECT_EQ(split_view_controller->IsSplitViewModeActive(), false);
+  const display::Display& display = display_manager()->GetDisplayAt(0);
+  const int magnifier_height = GetMagnifierHeight(display.bounds().height());
+  gfx::Rect work_area = display.bounds();
+  work_area.Inset(0, magnifier_height, 0, ShelfConstants::shelf_size());
+  EXPECT_EQ(work_area, display.work_area());
+  EXPECT_EQ(work_area, window->bounds());
+  EXPECT_TRUE(wm::GetWindowState(window.get())->IsMaximized());
+}
+
+// Test that we don't exit split view with two windows snapped on both sides
+// when we enable the docked magnifier, but rather their bounds are updated.
+TEST_F(DockedMagnifierTest, DisplaysWorkAreasDoubleSplitView) {
+  // Verify that we're in tablet mode.
+  Shell::Get()->tablet_mode_controller()->EnableTabletModeWindowManager(true);
+  EXPECT_TRUE(Shell::Get()
+                  ->tablet_mode_controller()
+                  ->IsTabletModeWindowManagerEnabled());
+
+  std::unique_ptr<aura::Window> window1(
+      CreateTestWindowInShell(SK_ColorWHITE, 100, gfx::Rect(0, 0, 200, 200)));
+  std::unique_ptr<aura::Window> window2(
+      CreateTestWindowInShell(SK_ColorWHITE, 200, gfx::Rect(0, 0, 200, 200)));
+
+  auto* overview_controller = Shell::Get()->overview_controller();
+  overview_controller->ToggleOverview();
+  EXPECT_TRUE(overview_controller->IsSelecting());
+
+  auto* split_view_controller = Shell::Get()->split_view_controller();
+  EXPECT_EQ(split_view_controller->IsSplitViewModeActive(), false);
+  split_view_controller->SnapWindow(window1.get(), SplitViewController::LEFT);
+  split_view_controller->SnapWindow(window2.get(), SplitViewController::RIGHT);
+  EXPECT_EQ(split_view_controller->IsSplitViewModeActive(), true);
+  EXPECT_EQ(split_view_controller->state(), SplitViewController::BOTH_SNAPPED);
+
+  // Snapping both windows should exit overview mode.
+  EXPECT_FALSE(overview_controller->IsSelecting());
+
+  // Enable the docked magnifier, and expect that split view does not exit, and
+  // the two windows heights are updated to be equal to the height of the
+  // updated display's work area.
+  controller()->SetEnabled(true);
+  EXPECT_TRUE(controller()->GetEnabled());
+  EXPECT_EQ(split_view_controller->IsSplitViewModeActive(), true);
+  EXPECT_EQ(split_view_controller->state(), SplitViewController::BOTH_SNAPPED);
+  const display::Display& display = display_manager()->GetDisplayAt(0);
+  const int magnifier_height = GetMagnifierHeight(display.bounds().height());
+  gfx::Rect work_area = display.bounds();
+  work_area.Inset(0, magnifier_height, 0, ShelfConstants::shelf_size());
+  EXPECT_EQ(work_area, display.work_area());
+  EXPECT_EQ(work_area.height(), window1->bounds().height());
+  EXPECT_EQ(work_area.height(), window2->bounds().height());
+}
+
+// Tests that the Docked Magnifier follows touch events.
+TEST_F(DockedMagnifierTest, TouchEvents) {
+  UpdateDisplay("800x600,800+0-400x300");
+  const auto root_windows = Shell::GetAllRootWindows();
+  ASSERT_EQ(2u, root_windows.size());
+
+  controller()->SetEnabled(true);
+  EXPECT_TRUE(controller()->GetEnabled());
+  controller()->SetScale(4.0f);
+
+  // Generate some touch events in both displays and expect the magnifier
+  // viewport moves accordingly.
+  gfx::Point touch_point(200, 350);
+  TouchPoint(touch_point);
+  const views::Widget* viewport_widget =
+      controller()->GetViewportWidgetForTesting();
+  EXPECT_EQ(root_windows[0], viewport_widget->GetNativeView()->GetRootWindow());
+  TestMagnifierLayerTransform(touch_point, root_windows[0]);
+
+  // Touch a new point in the other display.
+  touch_point = gfx::Point(900, 200);
+  TouchPoint(touch_point);
+
+  // New viewport widget is created in the second display.
+  ASSERT_NE(viewport_widget, controller()->GetViewportWidgetForTesting());
+  viewport_widget = controller()->GetViewportWidgetForTesting();
+  EXPECT_EQ(root_windows[1], viewport_widget->GetNativeView()->GetRootWindow());
+  TestMagnifierLayerTransform(touch_point, root_windows[1]);
+}
+
 // Tests the behavior of the magnifier when displays are added or removed.
 TEST_F(DockedMagnifierTest, AddRemoveDisplays) {
   // Start with a single display.
@@ -358,7 +483,7 @@ TEST_F(DockedMagnifierTest, AddRemoveDisplays) {
   ASSERT_NE(nullptr, viewport_widget);
   EXPECT_EQ(root_windows[0], viewport_widget->GetNativeView()->GetRootWindow());
   const int viewport_1_height =
-      800 / DockedMagnifierController::kScreenHeightDevisor;
+      800 / DockedMagnifierController::kScreenHeightDivisor;
   EXPECT_EQ(gfx::Rect(0, 0, 600, viewport_1_height),
             viewport_widget->GetWindowBoundsInScreen());
 
@@ -377,13 +502,13 @@ TEST_F(DockedMagnifierTest, AddRemoveDisplays) {
 
   // Move the cursor to the second display, expect the viewport widget to get
   // updated accordingly.
-  GetEventGenerator().MoveMouseTo(gfx::Point(800, 0));
+  GetEventGenerator()->MoveMouseTo(gfx::Point(800, 0));
   // New viewport widget is created.
   ASSERT_NE(viewport_widget, controller()->GetViewportWidgetForTesting());
   viewport_widget = controller()->GetViewportWidgetForTesting();
   EXPECT_EQ(root_windows[1], viewport_widget->GetNativeView()->GetRootWindow());
   const int viewport_2_height =
-      600 / DockedMagnifierController::kScreenHeightDevisor;
+      600 / DockedMagnifierController::kScreenHeightDivisor;
   EXPECT_EQ(gfx::Rect(600, 0, 400, viewport_2_height),
             viewport_widget->GetWindowBoundsInScreen());
 
@@ -398,11 +523,6 @@ TEST_F(DockedMagnifierTest, AddRemoveDisplays) {
   base::RunLoop().RunUntilIdle();
   root_windows = Shell::GetAllRootWindows();
   ASSERT_EQ(1u, root_windows.size());
-  if (Shell::GetAshConfig() == Config::MUS) {
-    // TODO: Figure out why do we need to manually simulate the mouse cursor
-    // being teleported to the first display. https://crbug.com/814590.
-    GetEventGenerator().MoveMouseTo(gfx::Point(300, 400));
-  }
   viewport_widget = controller()->GetViewportWidgetForTesting();
   ASSERT_NE(nullptr, viewport_widget);
   EXPECT_EQ(root_windows[0], viewport_widget->GetNativeView()->GetRootWindow());
@@ -427,42 +547,37 @@ TEST_F(DockedMagnifierTest, TransformSimple) {
   ASSERT_NE(nullptr, viewport_widget);
   EXPECT_EQ(root_windows[0], viewport_widget->GetNativeView()->GetRootWindow());
   const int viewport_height =
-      800 / DockedMagnifierController::kScreenHeightDevisor;
+      800 / DockedMagnifierController::kScreenHeightDivisor;
   EXPECT_EQ(gfx::Rect(0, 0, 800, viewport_height),
             viewport_widget->GetWindowBoundsInScreen());
 
   // Move the cursor to the center of the screen.
   gfx::Point point_of_interest(400, 400);
-  GetEventGenerator().MoveMouseTo(point_of_interest);
-  const ui::Layer* magnifier_layer =
-      controller()->GetViewportMagnifierLayerForTesting();
-  // The magnifier layer's transform, when applied to the point of interest,
-  // should take it to the point at the center of the viewport widget.
-  magnifier_layer->transform().TransformPoint(&point_of_interest);
-  const gfx::Point viewport_center(
-      viewport_widget->GetWindowBoundsInScreen().CenterPoint());
-  EXPECT_EQ(viewport_center, point_of_interest);
+  GetEventGenerator()->MoveMouseTo(point_of_interest);
+  TestMagnifierLayerTransform(point_of_interest, root_windows[0]);
 
   // Move the cursor to the bottom right corner.
   point_of_interest = gfx::Point(799, 799);
-  GetEventGenerator().MoveMouseTo(point_of_interest);
-  // Similar to the above case the expected transform should still take the
-  // point to the center of the viewport widget.
-  magnifier_layer->transform().TransformPoint(&point_of_interest);
-  EXPECT_EQ(viewport_center, point_of_interest);
+  GetEventGenerator()->MoveMouseTo(point_of_interest);
+  TestMagnifierLayerTransform(point_of_interest, root_windows[0]);
 
   // Tricky: Move the cursor to the top right corner, such that the cursor is
   // over the magnifier viewport. The transform should be such that the viewport
   // doesn't show itself.
   point_of_interest = gfx::Point(799, 0);
-  GetEventGenerator().MoveMouseTo(point_of_interest);
+  GetEventGenerator()->MoveMouseTo(point_of_interest);
+  TestMagnifierLayerTransform(point_of_interest, root_windows[0]);
   // In this case, our point of interest is changed to be at the bottom of the
   // separator, and it should go to the center of the top *edge* of the viewport
   // widget.
   point_of_interest.set_y(viewport_height +
                           DockedMagnifierController::kSeparatorHeight);
+  const gfx::Point viewport_center =
+      viewport_widget->GetNativeWindow()->GetBoundsInRootWindow().CenterPoint();
   gfx::Point viewport_top_edge_center = viewport_center;
   viewport_top_edge_center.set_y(0);
+  const ui::Layer* magnifier_layer =
+      controller()->GetViewportMagnifierLayerForTesting();
   magnifier_layer->transform().TransformPoint(&point_of_interest);
   EXPECT_EQ(viewport_top_edge_center, point_of_interest);
   // The minimum height for the point of interest is the bottom of the viewport
@@ -479,6 +594,7 @@ TEST_F(DockedMagnifierTest, TransformSimple) {
   EXPECT_FLOAT_EQ(scale2, controller()->GetScale());
   // The transform behaves exactly as above even with a different scale.
   point_of_interest = gfx::Point(799, 0);
+  TestMagnifierLayerTransform(point_of_interest, root_windows[0]);
   point_of_interest.set_y(viewport_height +
                           DockedMagnifierController::kSeparatorHeight);
   magnifier_layer->transform().TransformPoint(&point_of_interest);
@@ -497,7 +613,8 @@ TEST_F(DockedMagnifierTest, TextInputFieldEvents) {
   const auto root_windows = Shell::GetAllRootWindows();
   ASSERT_EQ(1u, root_windows.size());
 
-  text_input_helper_.CreateAndShowTextInputView(gfx::Rect(500, 400, 80, 80));
+  MagnifierTextInputTestHelper text_input_helper;
+  text_input_helper.CreateAndShowTextInputView(gfx::Rect(500, 400, 80, 80));
 
   // Enable the docked magnifier.
   controller()->SetEnabled(true);
@@ -507,29 +624,49 @@ TEST_F(DockedMagnifierTest, TextInputFieldEvents) {
   EXPECT_FLOAT_EQ(scale1, controller()->GetScale());
 
   // Focus on the text input field.
-  text_input_helper_.FocusOnTextInputView();
+  text_input_helper.FocusOnTextInputView();
 
   // The text input caret center point will be our point of interest. When it
   // goes through the magnifier layer transform, it should end up being in the
   // center of the viewport.
-  const ui::Layer* magnifier_layer =
-      controller()->GetViewportMagnifierLayerForTesting();
-  gfx::Point caret_center(text_input_helper_.GetCaretBounds().CenterPoint());
-  magnifier_layer->transform().TransformPoint(&caret_center);
-  const views::Widget* viewport_widget =
-      controller()->GetViewportWidgetForTesting();
-  const gfx::Point viewport_center(
-      viewport_widget->GetWindowBoundsInScreen().CenterPoint());
-  EXPECT_EQ(caret_center, viewport_center);
+  gfx::Point caret_center(text_input_helper.GetCaretBounds().CenterPoint());
+  TestMagnifierLayerTransform(caret_center, root_windows[0]);
 
   // Simulate typing by pressing some keys while focus is in the text field. The
   // transformed caret center should always go to the viewport center.
-  GetEventGenerator().PressKey(ui::VKEY_A, 0);
-  GetEventGenerator().ReleaseKey(ui::VKEY_A, 0);
-  gfx::Point new_caret_center(
-      text_input_helper_.GetCaretBounds().CenterPoint());
-  magnifier_layer->transform().TransformPoint(&new_caret_center);
-  EXPECT_EQ(new_caret_center, viewport_center);
+  GetEventGenerator()->PressKey(ui::VKEY_A, 0);
+  GetEventGenerator()->ReleaseKey(ui::VKEY_A, 0);
+  gfx::Point new_caret_center(text_input_helper.GetCaretBounds().CenterPoint());
+  TestMagnifierLayerTransform(new_caret_center, root_windows[0]);
+}
+
+TEST_F(DockedMagnifierTest, FocusChangeEvents) {
+  UpdateDisplay("600x900");
+  const auto root_windows = Shell::GetAllRootWindows();
+  ASSERT_EQ(1u, root_windows.size());
+
+  MagnifierFocusTestHelper focus_test_helper;
+  focus_test_helper.CreateAndShowFocusTestView(gfx::Point(70, 500));
+
+  // Enable the docked magnifier.
+  controller()->SetEnabled(true);
+  const float scale = 2.0f;
+  controller()->SetScale(scale);
+  EXPECT_TRUE(controller()->GetEnabled());
+  EXPECT_FLOAT_EQ(scale, controller()->GetScale());
+
+  // Focus on the first button and expect the magnifier to be centered around
+  // its center.
+  focus_test_helper.FocusFirstButton();
+  gfx::Point button_1_center(
+      focus_test_helper.GetFirstButtonBoundsInRoot().CenterPoint());
+  TestMagnifierLayerTransform(button_1_center, root_windows[0]);
+
+  // Similarly if we focus on the second button.
+  focus_test_helper.FocusSecondButton();
+  gfx::Point button_2_center(
+      focus_test_helper.GetSecondButtonBoundsInRoot().CenterPoint());
+  TestMagnifierLayerTransform(button_2_center, root_windows[0]);
 }
 
 // Tests that viewport layer is inverted properly when the status of the High
@@ -551,13 +688,13 @@ TEST_F(DockedMagnifierTest, HighContrastMode) {
   // Enable High Contrast mode, and expect the viewport layer to be inverted.
   Shell::Get()->accessibility_controller()->SetHighContrastEnabled(true);
   EXPECT_TRUE(
-      Shell::Get()->accessibility_controller()->IsHighContrastEnabled());
+      Shell::Get()->accessibility_controller()->high_contrast_enabled());
   EXPECT_TRUE(viewport_layer->layer_inverted());
 
   // Disable High Contrast, the layer should be updated accordingly.
   Shell::Get()->accessibility_controller()->SetHighContrastEnabled(false);
   EXPECT_FALSE(
-      Shell::Get()->accessibility_controller()->IsHighContrastEnabled());
+      Shell::Get()->accessibility_controller()->high_contrast_enabled());
   EXPECT_FALSE(viewport_layer->layer_inverted());
 
   // Now, disable the Docked Magnifier, enable High Contrast, and then re-enable
@@ -566,7 +703,7 @@ TEST_F(DockedMagnifierTest, HighContrastMode) {
   EXPECT_FALSE(magnifier->GetEnabled());
   Shell::Get()->accessibility_controller()->SetHighContrastEnabled(true);
   EXPECT_TRUE(
-      Shell::Get()->accessibility_controller()->IsHighContrastEnabled());
+      Shell::Get()->accessibility_controller()->high_contrast_enabled());
   magnifier->SetEnabled(true);
   EXPECT_TRUE(magnifier->GetEnabled());
   const ui::Layer* new_viewport_layer =
