@@ -7,11 +7,13 @@
 #include <memory>
 
 #include "ash/login/ui/lock_screen.h"
+#include "ash/metrics/demo_session_metrics_recorder.h"
 #include "ash/metrics/desktop_task_switch_metric_recorder.h"
 #include "ash/metrics/pointer_metrics_recorder.h"
 #include "ash/public/cpp/shelf_item.h"
 #include "ash/public/cpp/shelf_model.h"
 #include "ash/public/cpp/shell_window_ids.h"
+#include "ash/public/interfaces/accessibility_controller.mojom-shared.h"
 #include "ash/public/interfaces/window_state_type.mojom.h"
 #include "ash/session/session_controller.h"
 #include "ash/shelf/shelf.h"
@@ -37,6 +39,7 @@ enum ActiveWindowStateType {
   ACTIVE_WINDOW_STATE_TYPE_SNAPPED,
   ACTIVE_WINDOW_STATE_TYPE_PINNED,
   ACTIVE_WINDOW_STATE_TYPE_TRUSTED_PINNED,
+  ACTIVE_WINDOW_STATE_TYPE_PIP,
   ACTIVE_WINDOW_STATE_TYPE_COUNT,
 };
 
@@ -62,12 +65,14 @@ ActiveWindowStateType GetActiveWindowState() {
       case mojom::WindowStateType::TRUSTED_PINNED:
         active_window_state_type = ACTIVE_WINDOW_STATE_TYPE_TRUSTED_PINNED;
         break;
+      case mojom::WindowStateType::PIP:
+        active_window_state_type = ACTIVE_WINDOW_STATE_TYPE_PIP;
+        break;
       case mojom::WindowStateType::DEFAULT:
       case mojom::WindowStateType::NORMAL:
       case mojom::WindowStateType::MINIMIZED:
       case mojom::WindowStateType::INACTIVE:
       case mojom::WindowStateType::AUTO_POSITIONED:
-        // TODO: We probably want to recorde PINNED state.
         active_window_state_type = ACTIVE_WINDOW_STATE_TYPE_OTHER;
         break;
     }
@@ -98,7 +103,7 @@ bool IsUserActive() {
 // UMA statistics. Note the containers are ordered from top most visible
 // container to the lowest to allow the |GetNumVisibleWindows| method to short
 // circuit when processing a maximized or fullscreen window.
-int kVisibleWindowContainerIds[] = {kShellWindowId_PanelContainer,
+int kVisibleWindowContainerIds[] = {kShellWindowId_PipContainer,
                                     kShellWindowId_AlwaysOnTopContainer,
                                     kShellWindowId_DefaultContainer};
 
@@ -138,9 +143,7 @@ int GetNumVisibleWindowsInPrimaryDisplay() {
       // windows. Only windows in the kShellWindowId_DefaultContainer and
       // kShellWindowId_AlwaysOnTopContainer can be maximized or fullscreened
       // and completely obscure windows beneath them.
-      if ((kShellWindowId_DefaultContainer == current_container_id ||
-           kShellWindowId_AlwaysOnTopContainer == current_container_id) &&
-          child_window_state->IsMaximizedOrFullscreenOrPinned()) {
+      if (child_window_state->IsMaximizedOrFullscreenOrPinned()) {
         maximized_or_fullscreen_window_present = true;
         break;
       }
@@ -199,6 +202,13 @@ void UserMetricsRecorder::RecordUserClickOnShelfButton(
   recorder->RecordUserShelfButtonClick(target);
 }
 
+// static
+void UserMetricsRecorder::RecordUserToggleDictation(
+    mojom::DictationToggleSource source) {
+  UMA_HISTOGRAM_ENUMERATION("Accessibility.CrosDictation.ToggleDictationMethod",
+                            source);
+}
+
 void UserMetricsRecorder::RecordUserMetricsAction(UserMetricsAction action) {
   using base::RecordAction;
   using base::UserMetricsAction;
@@ -364,6 +374,9 @@ void UserMetricsRecorder::RecordUserMetricsAction(UserMetricsAction action) {
     case UMA_STATUS_AREA_OS_UPDATE_DEFAULT_SELECTED:
       RecordAction(UserMetricsAction("StatusArea_OS_Update_Default_Selected"));
       break;
+    case UMA_STATUS_AREA_SCREEN_CAPTURE_CHANGE_SOURCE:
+      RecordAction(UserMetricsAction("StatusArea_ScreenCapture_Change_Source"));
+      break;
     case UMA_STATUS_AREA_SCREEN_CAPTURE_DEFAULT_STOP:
       RecordAction(UserMetricsAction("StatusArea_ScreenCapture_Default_Stop"));
       break;
@@ -419,21 +432,14 @@ void UserMetricsRecorder::RecordUserMetricsAction(UserMetricsAction action) {
     case UMA_TRAY_SHUT_DOWN:
       RecordAction(UserMetricsAction("Tray_ShutDown"));
       break;
-    case UMA_TRAY_SWIPE_TO_CLOSE_SUCCESSFUL:
-      RecordAction(UserMetricsAction("Tray_SwipeToClose_Successful"));
-      break;
-    case UMA_TRAY_SWIPE_TO_CLOSE_UNSUCCESSFUL:
-      RecordAction(UserMetricsAction("Tray_SwipeToClose_Unsuccessful"));
-      break;
-    case UMA_TRAY_SWIPE_TO_OPEN_SUCCESSFUL:
-      RecordAction(UserMetricsAction("Tray_SwipeToOpen_Successful"));
-      break;
-    case UMA_TRAY_SWIPE_TO_OPEN_UNSUCCESSFUL:
-      RecordAction(UserMetricsAction("Tray_SwipeToOpen_Unsuccessful"));
-      break;
   }
 }
 
+void UserMetricsRecorder::StartDemoSessionMetricsRecording() {
+  demo_session_metrics_recorder_ =
+      std::make_unique<DemoSessionMetricsRecorder>();
+}
+
 void UserMetricsRecorder::OnShellInitialized() {
   // Lazy creation of the DesktopTaskSwitchMetricRecorder because it accesses
   // Shell::Get() which is not available when |this| is instantiated.
@@ -445,6 +451,7 @@ void UserMetricsRecorder::OnShellInitialized() {
 }
 
 void UserMetricsRecorder::OnShellShuttingDown() {
+  demo_session_metrics_recorder_.reset();
   desktop_task_switch_metric_recorder_.reset();
 
   // To clean up pointer_metrics_recorder_ properly, a valid shell instance is
