@@ -8,15 +8,26 @@
 
 #include "ash/accelerators/accelerator_controller.h"
 #include "ash/accessibility/accessibility_controller.h"
+#include "ash/accessibility/accessibility_focus_ring_controller.h"
 #include "ash/app_list/app_list_controller_impl.h"
+#include "ash/assistant/assistant_alarm_timer_controller.h"
+#include "ash/assistant/assistant_controller.h"
+#include "ash/assistant/assistant_notification_controller.h"
+#include "ash/assistant/assistant_screen_context_controller.h"
+#include "ash/assistant/assistant_setup_controller.h"
 #include "ash/cast_config_controller.h"
+#include "ash/contained_shell/contained_shell_controller.h"
 #include "ash/display/ash_display_controller.h"
+#include "ash/display/cros_display_config.h"
+#include "ash/display/display_output_protection.h"
+#include "ash/events/event_rewriter_controller.h"
+#include "ash/first_run/first_run_helper.h"
 #include "ash/highlighter/highlighter_controller.h"
 #include "ash/ime/ime_controller.h"
+#include "ash/keyboard/ash_keyboard_controller.h"
 #include "ash/login/login_screen_controller.h"
 #include "ash/magnifier/docked_magnifier_controller.h"
-#include "ash/media_controller.h"
-#include "ash/message_center/message_center_controller.h"
+#include "ash/media/media_controller.h"
 #include "ash/metrics/time_to_first_present_recorder.h"
 #include "ash/new_window_controller.h"
 #include "ash/note_taking_controller.h"
@@ -27,10 +38,11 @@
 #include "ash/shell.h"
 #include "ash/shell_delegate.h"
 #include "ash/shutdown_controller.h"
-#include "ash/system/locale/locale_notification_controller.h"
+#include "ash/system/locale/locale_update_controller.h"
+#include "ash/system/message_center/message_center_controller.h"
+#include "ash/system/model/system_tray_model.h"
 #include "ash/system/network/vpn_list.h"
 #include "ash/system/night_light/night_light_controller.h"
-#include "ash/system/tray/system_tray_controller.h"
 #include "ash/tray_action/tray_action.h"
 #include "ash/voice_interaction/voice_interaction_controller.h"
 #include "ash/wallpaper/wallpaper_controller.h"
@@ -39,7 +51,8 @@
 #include "base/bind.h"
 #include "base/lazy_instance.h"
 #include "base/single_thread_task_runner.h"
-#include "ui/app_list/presenter/app_list.h"
+#include "chromeos/constants/chromeos_switches.h"
+#include "ui/keyboard/keyboard_controller.h"
 
 namespace ash {
 namespace mojo_interface_factory {
@@ -58,8 +71,10 @@ void BindAccessibilityControllerRequestOnMainThread(
   Shell::Get()->accessibility_controller()->BindRequest(std::move(request));
 }
 
-void BindAppListRequestOnMainThread(app_list::mojom::AppListRequest request) {
-  Shell::Get()->app_list()->BindRequest(std::move(request));
+void BindAccessibilityFocusRingControllerRequestOnMainThread(
+    mojom::AccessibilityFocusRingControllerRequest request) {
+  Shell::Get()->accessibility_focus_ring_controller()->BindRequest(
+      std::move(request));
 }
 
 void BindAppListControllerRequestOnMainThread(
@@ -72,6 +87,47 @@ void BindAshDisplayControllerRequestOnMainThread(
   Shell::Get()->ash_display_controller()->BindRequest(std::move(request));
 }
 
+void BindAssistantAlarmTimerControllerRequestOnMainThread(
+    mojom::AssistantAlarmTimerControllerRequest request) {
+  Shell::Get()->assistant_controller()->alarm_timer_controller()->BindRequest(
+      std::move(request));
+}
+
+void BindAssistantControllerRequestOnMainThread(
+    mojom::AssistantControllerRequest request) {
+  Shell::Get()->assistant_controller()->BindRequest(std::move(request));
+}
+
+void BindAssistantNotificationControllerRequestOnMainThread(
+    mojom::AssistantNotificationControllerRequest request) {
+  Shell::Get()->assistant_controller()->notification_controller()->BindRequest(
+      std::move(request));
+}
+
+void BindAssistantScreenContextControllerRequestOnMainThread(
+    mojom::AssistantScreenContextControllerRequest request) {
+  Shell::Get()
+      ->assistant_controller()
+      ->screen_context_controller()
+      ->BindRequest(std::move(request));
+}
+
+void BindAssistantSetupControllerRequestOnMainThread(
+    mojom::AssistantSetupControllerRequest request) {
+  Shell::Get()->assistant_controller()->setup_controller()->BindRequest(
+      std::move(request));
+}
+
+void BindAssistantVolumeControlRequestOnMainThread(
+    mojom::AssistantVolumeControlRequest request) {
+  Shell::Get()->assistant_controller()->BindRequest(std::move(request));
+}
+
+void BindCrosDisplayConfigControllerRequestOnMainThread(
+    mojom::CrosDisplayConfigControllerRequest request) {
+  Shell::Get()->cros_display_config()->BindRequest(std::move(request));
+}
+
 void BindAshMessageCenterControllerRequestOnMainThread(
     mojom::AshMessageCenterControllerRequest request) {
   Shell::Get()->message_center_controller()->BindRequest(std::move(request));
@@ -81,11 +137,31 @@ void BindCastConfigOnMainThread(mojom::CastConfigRequest request) {
   Shell::Get()->cast_config()->BindRequest(std::move(request));
 }
 
+void BindContainedShellControllerRequestOnMainThread(
+    mojom::ContainedShellControllerRequest request) {
+  Shell::Get()->contained_shell_controller()->BindRequest(std::move(request));
+}
+
+void BindDisplayOutputProtectionRequestOnMainThread(
+    mojom::DisplayOutputProtectionRequest request) {
+  Shell::Get()->display_output_protection()->BindRequest(std::move(request));
+}
+
 void BindDockedMagnifierControllerRequestOnMainThread(
     mojom::DockedMagnifierControllerRequest request) {
   Shell::Get()->docked_magnifier_controller()->BindRequest(std::move(request));
 }
 
+void BindEventRewriterControllerRequestOnMainThread(
+    mojom::EventRewriterControllerRequest request) {
+  Shell::Get()->event_rewriter_controller()->BindRequest(std::move(request));
+}
+
+void BindFirstRunHelperRequestOnMainThread(
+    mojom::FirstRunHelperRequest request) {
+  Shell::Get()->first_run_helper()->BindRequest(std::move(request));
+}
+
 void BindHighlighterControllerRequestOnMainThread(
     mojom::HighlighterControllerRequest request) {
   Shell::Get()->highlighter_controller()->BindRequest(std::move(request));
@@ -95,10 +171,14 @@ void BindImeControllerRequestOnMainThread(mojom::ImeControllerRequest request) {
   Shell::Get()->ime_controller()->BindRequest(std::move(request));
 }
 
-void BindLocaleNotificationControllerOnMainThread(
-    mojom::LocaleNotificationControllerRequest request) {
-  Shell::Get()->locale_notification_controller()->BindRequest(
-      std::move(request));
+void BindKeyboardControllerRequestOnMainThread(
+    mojom::KeyboardControllerRequest request) {
+  Shell::Get()->ash_keyboard_controller()->BindRequest(std::move(request));
+}
+
+void BindLocaleUpdateControllerOnMainThread(
+    mojom::LocaleUpdateControllerRequest request) {
+  Shell::Get()->locale_update_controller()->BindRequest(std::move(request));
 }
 
 void BindLockScreenRequestOnMainThread(mojom::LoginScreenRequest request) {
@@ -145,7 +225,7 @@ void BindShutdownControllerRequestOnMainThread(
 }
 
 void BindSystemTrayRequestOnMainThread(mojom::SystemTrayRequest request) {
-  Shell::Get()->system_tray_controller()->BindRequest(std::move(request));
+  Shell::Get()->system_tray_model()->BindRequest(std::move(request));
 }
 
 void BindTabletModeRequestOnMainThread(
@@ -182,74 +262,124 @@ void RegisterInterfaces(
     service_manager::BinderRegistry* registry,
     scoped_refptr<base::SingleThreadTaskRunner> main_thread_task_runner) {
   registry->AddInterface(
-      base::Bind(&BindAcceleratorControllerRequestOnMainThread),
+      base::BindRepeating(&BindAcceleratorControllerRequestOnMainThread),
       main_thread_task_runner);
   registry->AddInterface(
-      base::Bind(&BindAccessibilityControllerRequestOnMainThread),
-      main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindAppListRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindAppListControllerRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(
-      base::Bind(&BindAshDisplayControllerRequestOnMainThread),
+      base::BindRepeating(&BindAccessibilityControllerRequestOnMainThread),
       main_thread_task_runner);
   registry->AddInterface(
-      base::Bind(&BindAshMessageCenterControllerRequestOnMainThread),
+      base::BindRepeating(
+          &BindAccessibilityFocusRingControllerRequestOnMainThread),
       main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindCastConfigOnMainThread),
-                         main_thread_task_runner);
-  if (features::IsDockedMagnifierEnabled()) {
+  registry->AddInterface(
+      base::BindRepeating(&BindAppListControllerRequestOnMainThread),
+      main_thread_task_runner);
+  if (chromeos::switches::IsAssistantEnabled()) {
     registry->AddInterface(
-        base::BindRepeating(&BindDockedMagnifierControllerRequestOnMainThread),
+        base::BindRepeating(
+            &BindAssistantAlarmTimerControllerRequestOnMainThread),
+        main_thread_task_runner);
+    registry->AddInterface(
+        base::BindRepeating(&BindAssistantControllerRequestOnMainThread),
+        main_thread_task_runner);
+    registry->AddInterface(
+        base::BindRepeating(
+            &BindAssistantNotificationControllerRequestOnMainThread),
+        main_thread_task_runner);
+    registry->AddInterface(
+        base::BindRepeating(
+            &BindAssistantScreenContextControllerRequestOnMainThread),
+        main_thread_task_runner);
+    registry->AddInterface(
+        base::BindRepeating(&BindAssistantSetupControllerRequestOnMainThread),
+        main_thread_task_runner);
+    registry->AddInterface(
+        base::BindRepeating(&BindAssistantVolumeControlRequestOnMainThread),
         main_thread_task_runner);
   }
   registry->AddInterface(
-      base::Bind(&BindHighlighterControllerRequestOnMainThread),
+      base::BindRepeating(&BindAshDisplayControllerRequestOnMainThread),
       main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindImeControllerRequestOnMainThread),
-                         main_thread_task_runner);
   registry->AddInterface(
-      base::Bind(&BindLocaleNotificationControllerOnMainThread),
+      base::BindRepeating(&BindCrosDisplayConfigControllerRequestOnMainThread),
       main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindLockScreenRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindMediaControllerRequestOnMainThread),
-                         main_thread_task_runner);
   registry->AddInterface(
-      base::Bind(&BindNewWindowControllerRequestOnMainThread),
+      base::BindRepeating(&BindAshMessageCenterControllerRequestOnMainThread),
       main_thread_task_runner);
-  if (switches::IsNightLightEnabled()) {
+  registry->AddInterface(base::BindRepeating(&BindCastConfigOnMainThread),
+                         main_thread_task_runner);
+  if (base::FeatureList::IsEnabled(features::kContainedShell)) {
     registry->AddInterface(
-        base::Bind(&BindNightLightControllerRequestOnMainThread),
+        base::BindRepeating(&BindContainedShellControllerRequestOnMainThread),
         main_thread_task_runner);
   }
   registry->AddInterface(
-      base::Bind(&BindNoteTakingControllerRequestOnMainThread),
+      base::BindRepeating(&BindDisplayOutputProtectionRequestOnMainThread),
       main_thread_task_runner);
   registry->AddInterface(
-      base::Bind(&BindProcessCreationTimeRecorderOnMainThread),
+      base::BindRepeating(&BindDockedMagnifierControllerRequestOnMainThread),
       main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindSessionControllerRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindShelfRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindShutdownControllerRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindSystemTrayRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindTabletModeRequestOnMainThread),
-                         main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindTrayActionRequestOnMainThread),
+  registry->AddInterface(
+      base::BindRepeating(&BindEventRewriterControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindFirstRunHelperRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindHighlighterControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindImeControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindKeyboardControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindLocaleUpdateControllerOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindLockScreenRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindMediaControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindNewWindowControllerRequestOnMainThread),
+      main_thread_task_runner);
+    registry->AddInterface(
+        base::BindRepeating(&BindNightLightControllerRequestOnMainThread),
+        main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindNoteTakingControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindProcessCreationTimeRecorderOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindSessionControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(base::BindRepeating(&BindShelfRequestOnMainThread),
                          main_thread_task_runner);
   registry->AddInterface(
-      base::Bind(&BindVoiceInteractionControllerRequestOnMainThread),
+      base::BindRepeating(&BindShutdownControllerRequestOnMainThread),
       main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindVpnListRequestOnMainThread),
+  registry->AddInterface(
+      base::BindRepeating(&BindSystemTrayRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindTabletModeRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindTrayActionRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(
+      base::BindRepeating(&BindVoiceInteractionControllerRequestOnMainThread),
+      main_thread_task_runner);
+  registry->AddInterface(base::BindRepeating(&BindVpnListRequestOnMainThread),
                          main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindWallpaperRequestOnMainThread),
+  registry->AddInterface(base::BindRepeating(&BindWallpaperRequestOnMainThread),
                          main_thread_task_runner);
-  registry->AddInterface(base::Bind(&BindSplitViewRequestOnMainThread),
+  registry->AddInterface(base::BindRepeating(&BindSplitViewRequestOnMainThread),
                          main_thread_task_runner);
 
   // Inject additional optional interfaces.
