@@ -11,16 +11,14 @@
 
 #include "ash/login_status.h"
 #include "ash/public/interfaces/session_controller.mojom.h"
-#include "ash/session/session_controller.h"
 #include "ash/session/session_observer.h"
 #include "ash/session/test_session_controller_client.h"
 #include "ash/shell.h"
-#include "ash/system/message_center/notification_tray.h"
-#include "ash/system/screen_security/screen_tray_item.h"
 #include "ash/system/tray/system_tray_notifier.h"
 #include "ash/test/ash_test_base.h"
-#include "ash/wm/overview/window_selector_controller.h"
+#include "ash/wm/overview/overview_controller.h"
 #include "ash/wm/window_util.h"
+#include "base/bind.h"
 #include "base/callback.h"
 #include "base/macros.h"
 #include "base/run_loop.h"
@@ -51,6 +49,8 @@ class TestSessionObserver : public SessionObserver {
     user_session_account_ids_.push_back(account_id);
   }
 
+  void OnFirstSessionStarted() override { first_session_started_ = true; }
+
   void OnSessionStateChanged(SessionState state) override { state_ = state; }
 
   void OnActiveUserPrefServiceChanged(PrefService* pref_service) override {
@@ -67,6 +67,7 @@ class TestSessionObserver : public SessionObserver {
 
   SessionState state() const { return state_; }
   const AccountId& active_account_id() const { return active_account_id_; }
+  bool first_session_started() const { return first_session_started_; }
   const std::vector<AccountId>& user_session_account_ids() const {
     return user_session_account_ids_;
   }
@@ -78,6 +79,7 @@ class TestSessionObserver : public SessionObserver {
  private:
   SessionState state_ = SessionState::UNKNOWN;
   AccountId active_account_id_;
+  bool first_session_started_ = false;
   std::vector<AccountId> user_session_account_ids_;
   PrefService* last_user_pref_service_ = nullptr;
 
@@ -172,6 +174,18 @@ TEST_F(SessionControllerTest, SimpleSessionInfo) {
   EXPECT_TRUE(controller()->IsRunningInAppMode());
 }
 
+TEST_F(SessionControllerTest, OnFirstSessionStarted) {
+  // Simulate chrome starting a user session.
+  mojom::SessionInfo info;
+  FillDefaultSessionInfo(&info);
+  SetSessionInfo(info);
+  UpdateSession(1u, "user1@test.com");
+  controller()->SetUserSessionOrder({1u});
+
+  // Observer is notified.
+  EXPECT_TRUE(observer()->first_session_started());
+}
+
 // Tests that the CanLockScreen is only true with an active user session.
 TEST_F(SessionControllerTest, CanLockScreen) {
   mojom::SessionInfo info;
@@ -302,6 +316,26 @@ TEST_F(SessionControllerTest, GetLoginStateForActiveSession) {
   }
 }
 
+TEST_F(SessionControllerTest, GetLoginStateForOwner) {
+  // Simulate an active user session.
+  mojom::SessionInfo info;
+  FillDefaultSessionInfo(&info);
+  info.state = SessionState::ACTIVE;
+  SetSessionInfo(info);
+
+  mojom::UserSessionPtr session = mojom::UserSession::New();
+  session->session_id = 1u;
+  session->user_info = mojom::UserInfo::New();
+  session->user_info->type = user_manager::USER_TYPE_REGULAR;
+  session->user_info->account_id = AccountId::FromUserEmail("owner@test.com");
+  session->user_info->display_name = "Owner";
+  session->user_info->display_email = "owner@test.com";
+  session->user_info->is_device_owner = true;
+  controller()->UpdateUserSession(std::move(session));
+
+  EXPECT_EQ(LoginStatus::OWNER, controller()->login_status());
+}
+
 // Tests that user sessions can be set and updated.
 TEST_F(SessionControllerTest, UserSessions) {
   EXPECT_FALSE(controller()->IsActiveUserSessionStarted());
@@ -557,14 +591,8 @@ class CanSwitchUserTest : public AshTestBase {
   CanSwitchUserTest() = default;
   ~CanSwitchUserTest() override = default;
 
-  void SetUp() override {
-    AshTestBase::SetUp();
-    NotificationTray::DisableAnimationsForTest(true);
-  }
-
   void TearDown() override {
-    RunAllPendingInMessageLoop();
-    NotificationTray::DisableAnimationsForTest(false);
+    base::RunLoop().RunUntilIdle();
     AshTestBase::TearDown();
   }
 
@@ -574,7 +602,7 @@ class CanSwitchUserTest : public AshTestBase {
     Shell::Get()->system_tray_notifier()->NotifyScreenCaptureStart(
         base::BindRepeating(&CanSwitchUserTest::StopCaptureCallback,
                             base::Unretained(this)),
-        base::EmptyString16());
+        base::RepeatingClosure(), base::string16());
   }
 
   // The callback which gets called when the screen capture gets stopped.
@@ -591,7 +619,7 @@ class CanSwitchUserTest : public AshTestBase {
     Shell::Get()->system_tray_notifier()->NotifyScreenShareStart(
         base::BindRepeating(&CanSwitchUserTest::StopShareCallback,
                             base::Unretained(this)),
-        base::EmptyString16());
+        base::string16());
   }
 
   // Simulates a screen share session stop.
@@ -621,10 +649,10 @@ class CanSwitchUserTest : public AshTestBase {
 
   // Methods needed to test with overview mode.
   bool ToggleOverview() {
-    return Shell::Get()->window_selector_controller()->ToggleOverview();
+    return Shell::Get()->overview_controller()->ToggleOverview();
   }
   bool IsSelecting() const {
-    return Shell::Get()->window_selector_controller()->IsSelecting();
+    return Shell::Get()->overview_controller()->IsSelecting();
   }
 
   // Various counter accessors.
@@ -785,5 +813,20 @@ TEST_F(CanSwitchUserTest, OverviewModeDismissed) {
   EXPECT_EQ(1, switch_callback_hit_count());
 }
 
+using SessionControllerUnblockTest = NoSessionAshTestBase;
+
+TEST_F(SessionControllerUnblockTest, ActiveWindowAfterUnblocking) {
+  EXPECT_TRUE(Shell::Get()->session_controller()->IsUserSessionBlocked());
+  auto widget = CreateTestWidget();
+  // |widget| should not be active as it is blocked by SessionController.
+  EXPECT_FALSE(widget->IsActive());
+  SimulateUserLogin("user@test.com");
+  EXPECT_FALSE(Shell::Get()->session_controller()->IsUserSessionBlocked());
+
+  // |widget| should now be active as SessionController no longer is blocking
+  // windows from becoming active.
+  EXPECT_TRUE(widget->IsActive());
+}
+
 }  // namespace
 }  // namespace ash
