@@ -4,12 +4,12 @@
 
 #include "ash/shelf/app_list_shelf_item_delegate.h"
 
+#include <algorithm>
 #include <utility>
 
+#include "ash/app_list/app_list_controller_impl.h"
 #include "ash/public/cpp/shelf_model.h"
 #include "ash/shell.h"
-#include "ui/app_list/app_list_constants.h"
-#include "ui/app_list/presenter/app_list.h"
 
 namespace ash {
 
@@ -22,8 +22,13 @@ void AppListShelfItemDelegate::ItemSelected(std::unique_ptr<ui::Event> event,
                                             int64_t display_id,
                                             ShelfLaunchSource source,
                                             ItemSelectedCallback callback) {
-  Shell::Get()->app_list()->ToggleAppList(display_id, app_list::kShelfButton);
-  std::move(callback).Run(SHELF_ACTION_APP_LIST_SHOWN, base::nullopt);
+  std::move(callback).Run(
+      Shell::Get()->app_list_controller()->OnAppListButtonPressed(
+          display_id,
+          event->IsShiftDown() ? app_list::kShelfButtonFullscreen
+                               : app_list::kShelfButton,
+          event->time_stamp()),
+      base::nullopt);
 }
 
 void AppListShelfItemDelegate::ExecuteCommand(bool from_context_menu,
