@@ -10,8 +10,10 @@
 #include "ash/focus_cycler.h"
 #include "ash/lock_screen_action/lock_screen_action_background_controller.h"
 #include "ash/lock_screen_action/test_lock_screen_action_background_controller.h"
+#include "ash/login/login_screen_controller.h"
 #include "ash/login/mock_login_screen_client.h"
 #include "ash/login/ui/login_test_base.h"
+#include "ash/login/ui/views_utils.h"
 #include "ash/public/interfaces/kiosk_app_info.mojom.h"
 #include "ash/root_window_controller.h"
 #include "ash/session/session_controller.h"
@@ -26,8 +28,10 @@
 #include "ash/tray_action/test_tray_action_client.h"
 #include "ash/tray_action/tray_action.h"
 #include "ash/wm/lock_state_controller.h"
+#include "base/bind.h"
 #include "base/command_line.h"
-#include "chromeos/chromeos_switches.h"
+#include "base/run_loop.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "ui/events/event_utils.h"
 #include "ui/events/test/event_generator.h"
 #include "ui/views/controls/button/label_button.h"
@@ -37,25 +41,14 @@ using session_manager::SessionState;
 namespace ash {
 namespace {
 
-// Returns true if |view| or any child of it has focus.
-bool HasFocusInAnyChildView(views::View* view) {
-  if (view->HasFocus())
-    return true;
-  for (int i = 0; i < view->child_count(); ++i) {
-    if (HasFocusInAnyChildView(view->child_at(i)))
-      return true;
-  }
-  return false;
-}
-
 void ExpectFocused(views::View* view) {
   EXPECT_TRUE(view->GetWidget()->IsActive());
-  EXPECT_TRUE(HasFocusInAnyChildView(view));
+  EXPECT_TRUE(login_views_utils::HasFocusInAnyChildView(view));
 }
 
 void ExpectNotFocused(views::View* view) {
   EXPECT_FALSE(view->GetWidget()->IsActive());
-  EXPECT_FALSE(HasFocusInAnyChildView(view));
+  EXPECT_FALSE(login_views_utils::HasFocusInAnyChildView(view));
 }
 
 class LoginShelfViewTest : public LoginTestBase {
@@ -71,7 +64,7 @@ class LoginShelfViewTest : public LoginTestBase {
         &action_background_controller_factory_);
 
     LoginTestBase::SetUp();
-    login_shelf_view_ = GetPrimaryShelf()->GetLoginShelfViewForTesting();
+    login_shelf_view_ = GetPrimaryShelf()->shelf_widget()->login_shelf_view();
     Shell::Get()->tray_action()->SetClient(
         tray_action_client_.CreateInterfacePtrAndBind(),
         mojom::TrayActionState::kNotAvailable);
@@ -106,7 +99,7 @@ class LoginShelfViewTest : public LoginTestBase {
                                ui::EventTimeForNow(), 0, 0);
     login_shelf_view_->ButtonPressed(
         static_cast<views::Button*>(login_shelf_view_->GetViewByID(id)), event);
-    RunAllPendingInMessageLoop();
+    base::RunLoop().RunUntilIdle();
   }
 
   // Checks if the shelf is only showing the buttons in the list. The IDs in
@@ -124,6 +117,11 @@ class LoginShelfViewTest : public LoginTestBase {
     return visible_button_count == ids.size();
   }
 
+  // Check whether the button is enabled.
+  bool IsButtonEnabled(LoginShelfView::ButtonId id) {
+    return login_shelf_view_->GetViewByID(id)->enabled();
+  }
+
   TestTrayActionClient tray_action_client_;
 
   LoginShelfView* login_shelf_view_;  // Unowned.
@@ -156,6 +154,7 @@ class LoginShelfViewTest : public LoginTestBase {
 TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterSessionStateChange) {
   EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
 
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
   NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
   EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
                                  LoginShelfView::kBrowseAsGuest,
@@ -258,6 +257,7 @@ TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterLockScreenNoteState) {
 }
 
 TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterKioskAppsLoaded) {
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
   NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
   EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
                                  LoginShelfView::kBrowseAsGuest,
@@ -277,6 +277,111 @@ TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterKioskAppsLoaded) {
                                  LoginShelfView::kAddUser}));
 }
 
+TEST_F(LoginShelfViewTest, SetAllowLoginByGuest) {
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
+  NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+
+  // SetAllowLoginAsGuest(false) always hides the guest button.
+  login_shelf_view_->SetAllowLoginAsGuest(false /*allow_guest*/);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kAddUser}));
+
+  // SetAllowLoginAsGuest(true) brings the guest button back.
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+
+  // However, SetAllowLoginAsGuest(true) does not mean that the guest button is
+  // always visible.
+  login_shelf_view_->SetLoginDialogState(
+      mojom::OobeDialogState::SAML_PASSWORD_CONFIRM);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+}
+
+TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterDialogStateChange) {
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
+  // The conditions in this test should only hold while there are user pods on
+  // the signin screen.
+  DataDispatcher()->AddObserver(login_shelf_view_);
+  AddUsers(1);
+
+  NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+
+  // Add user button is always hidden if dialog state !=
+  // OobeDialogState::HIDDEN.
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::GAIA_SIGNIN);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  // Guest button is hidden if dialog state ==
+  // OobeDialogState::WRONG_HWID_WARNING or SAML_PASSWORD_CONFIRM.
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::HIDDEN);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  login_shelf_view_->SetLoginDialogState(
+      mojom::OobeDialogState::WRONG_HWID_WARNING);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::HIDDEN);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  login_shelf_view_->SetLoginDialogState(
+      mojom::OobeDialogState::SAML_PASSWORD_CONFIRM);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  // By default guest login during gaia is not allowed.
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::GAIA_SIGNIN);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  // Guest button is hidden if SetAllowLoginAsGuest(false).
+  login_shelf_view_->SetAllowLoginAsGuest(false /*allow_guest*/);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  // Kiosk app button is visible when dialog state == OobeDialogState::HIDDEN
+  // or GAIA_SIGNIN.
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::GAIA_SIGNIN);
+  std::vector<mojom::KioskAppInfoPtr> kiosk_apps;
+  kiosk_apps.push_back(mojom::KioskAppInfo::New());
+  login_shelf_view_->SetKioskApps(std::move(kiosk_apps));
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kApps}));
+
+  login_shelf_view_->SetLoginDialogState(
+      mojom::OobeDialogState::SAML_PASSWORD_CONFIRM);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::HIDDEN);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kAddUser,
+                         LoginShelfView::kApps}));
+
+  // Kiosk app button is hidden when no app exists.
+  login_shelf_view_->SetKioskApps(std::vector<mojom::KioskAppInfoPtr>());
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kAddUser}));
+}
+
+TEST_F(LoginShelfViewTest, ShouldShowGuestButtonWhenNoUserPods) {
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
+  DataDispatcher()->AddObserver(login_shelf_view_);
+  SetUserCount(0);
+
+  NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
+  // When no user pods are visible, the Gaia dialog would normally pop up. We
+  // need to simulate that behavior in this test.
+  login_shelf_view_->SetLoginDialogState(mojom::OobeDialogState::GAIA_SIGNIN);
+  EXPECT_TRUE(ShowsShelfButtons(
+      {LoginShelfView::kShutdown, LoginShelfView::kBrowseAsGuest}));
+}
+
 TEST_F(LoginShelfViewTest, ClickShutdownButton) {
   Click(LoginShelfView::kShutdown);
   EXPECT_TRUE(Shell::Get()->lock_state_controller()->ShutdownRequested());
@@ -355,24 +460,109 @@ TEST_F(LoginShelfViewTest, TabGoesFromShelfToStatusAreaAndBackToShelf) {
       login_shelf_view_->GetViewByID(LoginShelfView::kShutdown)->HasFocus());
 
   // Focus from the first button to the second button.
-  GetEventGenerator().PressKey(ui::KeyboardCode::VKEY_TAB, 0);
+  GetEventGenerator()->PressKey(ui::KeyboardCode::VKEY_TAB, 0);
   ExpectFocused(shelf);
   ExpectNotFocused(status_area);
   EXPECT_TRUE(
       login_shelf_view_->GetViewByID(LoginShelfView::kSignOut)->HasFocus());
 
   // Focus from the second button to the status area.
-  GetEventGenerator().PressKey(ui::KeyboardCode::VKEY_TAB, 0);
+  GetEventGenerator()->PressKey(ui::KeyboardCode::VKEY_TAB, 0);
   ExpectNotFocused(shelf);
   ExpectFocused(status_area);
 
   // A single shift+tab brings focus back to the second shelf button.
-  GetEventGenerator().PressKey(ui::KeyboardCode::VKEY_TAB, ui::EF_SHIFT_DOWN);
+  GetEventGenerator()->PressKey(ui::KeyboardCode::VKEY_TAB, ui::EF_SHIFT_DOWN);
   ExpectFocused(shelf);
   ExpectNotFocused(status_area);
   EXPECT_TRUE(
       login_shelf_view_->GetViewByID(LoginShelfView::kSignOut)->HasFocus());
 }
 
+TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterAddButtonStatusChange) {
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
+  NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  EXPECT_TRUE(IsButtonEnabled(LoginShelfView::kAddUser));
+
+  login_shelf_view_->SetAddUserButtonEnabled(false /*enable_add_user*/);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  EXPECT_FALSE(IsButtonEnabled(LoginShelfView::kAddUser));
+
+  login_shelf_view_->SetAddUserButtonEnabled(true /*enable_add_user*/);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  EXPECT_TRUE(IsButtonEnabled(LoginShelfView::kAddUser));
+}
+
+TEST_F(LoginShelfViewTest, ShouldUpdateUiAfterShutdownButtonStatusChange) {
+  login_shelf_view_->SetAllowLoginAsGuest(true /*allow_guest*/);
+  NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  EXPECT_TRUE(IsButtonEnabled(LoginShelfView::kShutdown));
+
+  login_shelf_view_->SetShutdownButtonEnabled(false /*enable_shutdown_button*/);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  EXPECT_FALSE(IsButtonEnabled(LoginShelfView::kShutdown));
+
+  login_shelf_view_->SetShutdownButtonEnabled(true /*enable_shutdown_button*/);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+  EXPECT_TRUE(IsButtonEnabled(LoginShelfView::kShutdown));
+}
+
+TEST_F(LoginShelfViewTest, ParentAccessButtonVisibility) {
+  // Parent access button should only be visible on lock screen.
+  Shell::Get()->login_screen_controller()->SetShowParentAccessButton(true);
+
+  NotifySessionStateChanged(SessionState::LOGIN_PRIMARY);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown,
+                                 LoginShelfView::kBrowseAsGuest,
+                                 LoginShelfView::kAddUser}));
+
+  NotifySessionStateChanged(SessionState::LOGGED_IN_NOT_ACTIVE);
+  EXPECT_TRUE(ShowsShelfButtons({LoginShelfView::kShutdown}));
+
+  NotifySessionStateChanged(SessionState::ACTIVE);
+  EXPECT_TRUE(ShowsShelfButtons({}));
+
+  NotifySessionStateChanged(SessionState::LOGIN_SECONDARY);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kCancel}));
+
+  NotifySessionStateChanged(SessionState::ACTIVE);
+  EXPECT_TRUE(ShowsShelfButtons({}));
+
+  NotifySessionStateChanged(SessionState::LOCKED);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kSignOut,
+                         LoginShelfView::kParentAccess}));
+}
+
+TEST_F(LoginShelfViewTest, ParentAccessButtonVisibilityChangeOnLockScreen) {
+  NotifySessionStateChanged(SessionState::LOCKED);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kSignOut}));
+
+  Shell::Get()->login_screen_controller()->SetShowParentAccessButton(true);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kSignOut,
+                         LoginShelfView::kParentAccess}));
+
+  Shell::Get()->login_screen_controller()->SetShowParentAccessButton(false);
+  EXPECT_TRUE(
+      ShowsShelfButtons({LoginShelfView::kShutdown, LoginShelfView::kSignOut}));
+}
+
 }  // namespace
 }  // namespace ash
