@@ -7,7 +7,6 @@
 #include <memory>
 
 #include "ash/public/cpp/ash_pref_names.h"
-#include "ash/public/cpp/config.h"
 #include "ash/public/cpp/remote_shelf_item_delegate.h"
 #include "ash/public/cpp/shelf_prefs.h"
 #include "ash/root_window_controller.h"
@@ -15,12 +14,14 @@
 #include "ash/shelf/app_list_shelf_item_delegate.h"
 #include "ash/shelf/shelf.h"
 #include "ash/shelf/shelf_constants.h"
+#include "ash/shelf/shelf_widget.h"
 #include "ash/shell.h"
 #include "ash/strings/grit/ash_strings.h"
 #include "ash/system/message_center/arc/arc_notification_constants.h"
 #include "ash/wm/tablet_mode/tablet_mode_controller.h"
 #include "base/auto_reset.h"
-#include "base/strings/utf_string_conversions.h"
+#include "base/bind.h"
+#include "chromeos/strings/grit/chromeos_strings.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "components/prefs/pref_change_registrar.h"
 #include "components/prefs/pref_registry_simple.h"
@@ -93,15 +94,16 @@ void UpdateShelfVisibility() {
 
 // Set each Shelf's auto-hide behavior and alignment from the per-display prefs.
 void SetShelfBehaviorsFromPrefs() {
-  // The shelf should always be bottom-aligned and not hidden in tablet mode;
-  // alignment and auto-hide are assigned from prefs when tablet mode is exited.
+  SetShelfAutoHideFromPrefs();
+
+  // The shelf should always be bottom-aligned in tablet mode; alignment is
+  // assigned from prefs when tablet mode is exited.
   if (Shell::Get()
           ->tablet_mode_controller()
           ->IsTabletModeWindowManagerEnabled()) {
     return;
   }
 
-  SetShelfAutoHideFromPrefs();
   SetShelfAlignmentFromPrefs();
 }
 
@@ -346,8 +348,8 @@ void ShelfController::OnTabletModeStarted() {
       // Only animate into tablet mode if the shelf alignment will not change.
       if (shelf->IsHorizontalAlignment())
         shelf->set_is_tablet_mode_animation_running(true);
-      shelf->SetAutoHideBehavior(SHELF_AUTO_HIDE_BEHAVIOR_NEVER);
       shelf->SetAlignment(SHELF_ALIGNMENT_BOTTOM);
+      shelf->shelf_widget()->OnTabletModeChanged();
     }
   }
 }
@@ -361,8 +363,11 @@ void ShelfController::OnTabletModeEnded() {
   // Only animate out of tablet mode if the shelf alignment will not change.
   for (const auto& display : display::Screen::GetScreen()->GetAllDisplays()) {
     Shelf* shelf = GetShelfForDisplay(display.id());
-    if (shelf && shelf->IsHorizontalAlignment())
-      shelf->set_is_tablet_mode_animation_running(true);
+    if (shelf) {
+      if (shelf->IsHorizontalAlignment())
+        shelf->set_is_tablet_mode_animation_running(true);
+      shelf->shelf_widget()->OnTabletModeChanged();
+    }
   }
 }
 
@@ -411,9 +416,9 @@ void ShelfController::OnNotificationAdded(const std::string& notification_id) {
 
   // Skip this if the notification shouldn't badge an app.
   if (notification->notifier_id().type !=
-          message_center::NotifierId::APPLICATION &&
+          message_center::NotifierType::APPLICATION &&
       notification->notifier_id().type !=
-          message_center::NotifierId::ARC_APPLICATION) {
+          message_center::NotifierType::ARC_APPLICATION) {
     return;
   }
 
