@@ -7,27 +7,28 @@
 
 #include <memory>
 
+#include "ash/app_list/app_list_controller_observer.h"
 #include "ash/ash_export.h"
 #include "ash/public/cpp/shelf_types.h"
 #include "ash/session/session_observer.h"
 #include "ash/shelf/shelf.h"
 #include "ash/shell_observer.h"
+#include "ash/system/locale/locale_update_controller.h"
+#include "ash/wallpaper/wallpaper_controller_observer.h"
 #include "ash/wm/lock_state_observer.h"
+#include "ash/wm/overview/overview_observer.h"
 #include "ash/wm/wm_snap_to_pixel_layout_manager.h"
 #include "ash/wm/workspace/workspace_types.h"
 #include "base/macros.h"
 #include "base/observer_list.h"
 #include "base/scoped_observer.h"
 #include "base/timer/timer.h"
+#include "ui/display/display_observer.h"
 #include "ui/gfx/geometry/insets.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/keyboard/keyboard_controller_observer.h"
 #include "ui/wm/public/activation_change_observer.h"
 
-namespace keyboard {
-class KeyboardController;
-}
-
 namespace ui {
 class ImplicitAnimationObserver;
 class MouseEvent;
@@ -50,31 +51,21 @@ class ShelfWidget;
 // closely with ShelfLayoutManager.
 // On mus, widget bounds management is handled by the window manager.
 class ASH_EXPORT ShelfLayoutManager
-    : public ShellObserver,
+    : public AppListControllerObserver,
+      public ShellObserver,
+      public OverviewObserver,
       public ::wm::ActivationChangeObserver,
       public keyboard::KeyboardControllerObserver,
       public LockStateObserver,
       public wm::WmSnapToPixelLayoutManager,
-      public SessionObserver {
+      public display::DisplayObserver,
+      public SessionObserver,
+      public WallpaperControllerObserver,
+      public LocaleChangeObserver {
  public:
-  // The snapping threshold for dragging app list from shelf in tablet mode,
-  // measured in DIPs.
-  static constexpr int kAppListDragSnapToFullscreenThreshold = 320;
-
-  // The snapping thresholds for dragging app list from shelf in laptop mode,
-  // measured in DIPs.
-  static constexpr int kAppListDragSnapToClosedThreshold = 144;
-  static constexpr int kAppListDragSnapToPeekingThreshold = 561;
-
-  // The velocity the app list must be dragged in order to change the state of
-  // the app list for fling event, measured in DIPs/event.
-  static constexpr int kAppListDragVelocityThreshold = 6;
-
   ShelfLayoutManager(ShelfWidget* shelf_widget, Shelf* shelf);
   ~ShelfLayoutManager() override;
 
-  bool updating_bounds() const { return updating_bounds_; }
-
   // Clears internal data for shutdown process.
   void PrepareForShutdown();
   // Returns whether the shelf and its contents (shelf, status) are visible
@@ -82,19 +73,13 @@ class ASH_EXPORT ShelfLayoutManager
   bool IsVisible() const;
 
   // Returns the ideal bounds of the shelf assuming it is visible.
-  gfx::Rect GetIdealBounds();
+  gfx::Rect GetIdealBounds() const;
 
   // Returns the preferred size of the shelf for the target visibility state.
   gfx::Size GetPreferredSize();
 
-  // Returns the bounds within the root window not occupied by the shelf nor the
-  // virtual keyboard.
-  const gfx::Rect& user_work_area_bounds() const {
-    return user_work_area_bounds_;
-  }
-
   // Stops any animations and sets the bounds of the shelf and status widgets.
-  void LayoutShelfAndUpdateBounds(bool change_work_area);
+  void LayoutShelfAndUpdateBounds();
 
   // Stops any animations, sets the bounds of the shelf and status widgets, and
   // changes the work area
@@ -110,29 +95,13 @@ class ASH_EXPORT ShelfLayoutManager
   // Invoked by the shelf when the auto-hide state may have changed.
   void UpdateAutoHideState();
 
-  // TODO(mash): Add similar event handling support for mash.
   // Updates the auto-hide state for mouse events.
   void UpdateAutoHideForMouseEvent(ui::MouseEvent* event, aura::Window* target);
 
-  // Process the gesture events on |target|.
-  void ProcessGestureEventOnWindow(ui::GestureEvent* event,
-                                   aura::Window* target);
-
-  ShelfVisibilityState visibility_state() const {
-    return state_.visibility_state;
-  }
-  ShelfAutoHideState auto_hide_state() const { return state_.auto_hide_state; }
-
-  int chromevox_panel_height() const { return chromevox_panel_height_; }
-
-  int docked_magnifier_height() const { return docked_magnifier_height_; }
-
-  ShelfWidget* shelf_widget() { return shelf_widget_; }
-
-  // Sets whether any windows overlap the shelf. If a window overlaps the shelf
-  // the shelf renders slightly differently.
-  void SetWindowOverlapsShelf(bool value);
-  bool window_overlaps_shelf() const { return window_overlaps_shelf_; }
+  // Called by AutoHideEventHandler to process the gesture events when shelf is
+  // auto hide.
+  void ProcessGestureEventOfAutoHideShelf(ui::GestureEvent* event,
+                                          aura::Window* target);
 
   void AddObserver(ShelfLayoutManagerObserver* observer);
   void RemoveObserver(ShelfLayoutManagerObserver* observer);
@@ -142,37 +111,102 @@ class ASH_EXPORT ShelfLayoutManager
   // be processed any further, false otherwise.
   bool ProcessGestureEvent(const ui::GestureEvent& event_in_screen);
 
-  // Overridden from wm::WmSnapToPixelLayoutManager:
+  // Returns true if a maximized or fullscreen window is being dragged from the
+  // top of the display or from the caption area. Note currently for this case
+  // it's only allowed in tablet mode, not in laptop mode.
+  bool IsDraggingWindowFromTopOrCaptionArea() const;
+
+  // Returns how the shelf background should be painted.
+  ShelfBackgroundType GetShelfBackgroundType() const;
+
+  // Set the height of the accessibility panel, which takes away space from the
+  // available work area from the top of the screen. Used by ChromeVox.
+  void SetAccessibilityPanelHeight(int height);
+
+  // Set the height of the Docked Magnifier viewport at the top of the screen,
+  // which will reduce the available screen work area similarly to the ChromeVox
+  // panel height. The Docked Magnifier appears above the ChromeVox panel.
+  void SetDockedMagnifierHeight(int height);
+
+  // Updates the background of the shelf if it has changed.
+  void MaybeUpdateShelfBackground(AnimationChangeType change_type);
+
+  // Returns whether the shelf should show a blurred background. This may
+  // return false even if background blur is enabled depending on the session
+  // state.
+  bool ShouldBlurShelfBackground();
+
+  // wm::WmSnapToPixelLayoutManager:
   void OnWindowResized() override;
   void SetChildBounds(aura::Window* child,
                       const gfx::Rect& requested_bounds) override;
 
-  // Overridden from ShellObserver:
+  // ShellObserver:
   void OnShelfAutoHideBehaviorChanged(aura::Window* root_window) override;
   void OnPinnedStateChanged(aura::Window* pinned_window) override;
-  void OnVirtualKeyboardStateChanged(bool activated,
-                                     aura::Window* root_window) override;
-  void OnAppListVisibilityChanged(bool shown,
-                                  aura::Window* root_window) override;
+  void OnSplitViewModeStarted() override;
+  void OnSplitViewModeEnded() override;
 
-  // Overridden from wm::ActivationChangeObserver:
+  // OverviewObserver:
+  void OnOverviewModeStartingAnimationComplete(bool canceled) override;
+  void OnOverviewModeEndingAnimationComplete(bool canceled) override;
+
+  // AppListControllerObserver:
+  void OnAppListVisibilityChanged(bool shown, int64_t display_id) override;
+  void OnHomeLauncherTargetPositionChanged(bool showing,
+                                           int64_t display_id) override;
+  void OnHomeLauncherAnimationComplete(bool shown, int64_t display_id) override;
+
+  // wm::ActivationChangeObserver:
   void OnWindowActivated(ActivationReason reason,
                          aura::Window* gained_active,
                          aura::Window* lost_active) override;
 
-  // Overridden from keyboard::KeyboardControllerObserver:
+  // keyboard::KeyboardControllerObserver:
   void OnKeyboardAppearanceChanged(
       const keyboard::KeyboardStateDescriptor& state) override;
-  void OnKeyboardAvailabilityChanged(const bool is_available) override;
-  void OnKeyboardClosed() override;
+  void OnKeyboardVisibilityStateChanged(bool is_visible) override;
 
-  // Overridden from LockStateObserver:
+  // LockStateObserver:
   void OnLockStateEvent(LockStateObserver::EventType event) override;
 
-  // Overridden from SessionObserver:
+  // SessionObserver:
   void OnSessionStateChanged(session_manager::SessionState state) override;
   void OnLoginStatusChanged(LoginStatus loing_status) override;
 
+  // WallpaperControllerObserver:
+  void OnWallpaperBlurChanged() override;
+  void OnFirstWallpaperShown() override;
+
+  // DisplayObserver:
+  void OnDisplayMetricsChanged(const display::Display& display,
+                               uint32_t changed_metrics) override;
+
+  // LocaleChangeObserver:
+  void OnLocaleChanged() override;
+
+  // Returns the bounds within the root window not occupied by the shelf nor the
+  // virtual keyboard.
+  const gfx::Rect& user_work_area_bounds() const {
+    return user_work_area_bounds_;
+  }
+
+  // Returns the stable work area which is the work area when the shelf is
+  // visible.
+  gfx::Rect ComputeStableWorkArea() const;
+
+  ShelfVisibilityState visibility_state() const {
+    return state_.visibility_state;
+  }
+  bool updating_bounds() const { return updating_bounds_; }
+  ShelfAutoHideState auto_hide_state() const { return state_.auto_hide_state; }
+  int accessibility_panel_height() const { return accessibility_panel_height_; }
+  int docked_magnifier_height() const { return docked_magnifier_height_; }
+  ShelfWidget* shelf_widget() { return shelf_widget_; }
+
+  // Returns whether background blur is enabled.
+  bool IsBackgroundBlurEnabled() { return is_background_blur_enabled_; }
+
   // TODO(harrym|oshima): These templates will be moved to a new Shelf class.
   // A helper function for choosing values specific to a shelf alignment.
   template <typename T>
@@ -195,23 +229,11 @@ class ASH_EXPORT ShelfLayoutManager
     return shelf_->IsHorizontalAlignment() ? horizontal : vertical;
   }
 
-  // Returns how the shelf background should be painted.
-  ShelfBackgroundType GetShelfBackgroundType() const;
-
-  // Set the height of the ChromeVox panel, which takes away space from the
-  // available work area from the top of the screen.
-  void SetChromeVoxPanelHeight(int height);
-
-  // Set the height of the Docked Magnifier viewport at the top of the screen,
-  // which will reduce the available screen work area similarly to the ChromeVox
-  // panel height. The Docked Magnifier appears above the ChromeVox panel.
-  void SetDockedMagnifierHeight(int height);
-
  private:
   class UpdateShelfObserver;
   friend class PanelLayoutManagerTest;
   friend class ShelfLayoutManagerTest;
-  friend class WebNotificationTrayTest;
+  friend class NotificationTrayTest;
 
   struct TargetBounds {
     TargetBounds();
@@ -219,9 +241,9 @@ class ASH_EXPORT ShelfLayoutManager
 
     float opacity;
     float status_opacity;
-    gfx::Rect shelf_bounds_in_root;
-    gfx::Rect shelf_bounds_in_shelf;
-    gfx::Rect status_bounds_in_shelf;
+    gfx::Rect shelf_bounds;            // Bounds of the shelf within the screen
+    gfx::Rect shelf_bounds_in_shelf;   // Bounds of the shelf minus status area
+    gfx::Rect status_bounds_in_shelf;  // Bounds of status area within shelf
     gfx::Insets work_area_insets;
   };
 
@@ -233,6 +255,9 @@ class ASH_EXPORT ShelfLayoutManager
 
     bool IsScreenLocked() const;
 
+    // Returns whether the session is in an active state.
+    bool IsActiveSessionState() const;
+
     // Returns true if the two states are considered equal. As
     // |auto_hide_state| only matters if |visibility_state| is
     // |SHELF_AUTO_HIDE|, Equals() ignores the |auto_hide_state| as
@@ -252,26 +277,30 @@ class ASH_EXPORT ShelfLayoutManager
 
   // Updates the bounds and opacity of the shelf and status widgets.
   // If |observer| is specified, it will be called back when the animations, if
-  // any, are complete. |change_work_area| specifies whether or not to update
-  // the work area of the screen.
+  // any, are complete.
   void UpdateBoundsAndOpacity(const TargetBounds& target_bounds,
                               bool animate,
-                              bool change_work_area,
                               ui::ImplicitAnimationObserver* observer);
 
+  // Returns whether the virtual keyboard is currently being shown.
+  bool IsKeyboardShown() const;
+
   // Stops any animations and progresses them to the end.
   void StopAnimating();
 
-  // Calculates the target bounds assuming visibility of |visible|.
-  void CalculateTargetBounds(const State& state, TargetBounds* target_bounds);
+  // Calculates the target bounds assuming visibility of
+  // |state.visibilty_state|, and returns the work area.
+  gfx::Rect CalculateTargetBounds(const State& state,
+                                  TargetBounds* target_bounds) const;
+
+  // Calculate the target bounds using |state_|, and updates the
+  // |user_work_area_bounds_|.
+  void CalculateTargetBoundsAndUpdateWorkArea(TargetBounds* target_bounds);
 
   // Updates the target bounds if a gesture-drag is in progress. This is only
   // used by |CalculateTargetBounds()|.
   void UpdateTargetBoundsForGesture(TargetBounds* target_bounds) const;
 
-  // Updates the background of the shelf if it has changed.
-  void MaybeUpdateShelfBackground(AnimationChangeType change_type);
-
   // Updates the auto hide state immediately.
   void UpdateAutoHideStateNow();
 
@@ -279,6 +308,11 @@ class ASH_EXPORT ShelfLayoutManager
   // |mouse_over_shelf_when_auto_hide_timer_started_|.
   void StopAutoHideTimer();
 
+  // Returns the bounds of the shelf on the screen. The returned rect does
+  // not include portions of the shelf that extend beyond its own display,
+  // as those are not visible to the user.
+  gfx::Rect GetVisibleShelfBounds() const;
+
   // Returns the bounds of an additional region which can trigger showing the
   // shelf. This region exists to make it easier to trigger showing the shelf
   // when the shelf is auto hidden and the shelf is on the boundary between
@@ -305,7 +339,7 @@ class ASH_EXPORT ShelfLayoutManager
   void UpdateShelfVisibilityAfterLoginUIChange();
 
   // Compute |target_bounds| opacity based on gesture and shelf visibility.
-  float ComputeTargetOpacity(const State& state);
+  float ComputeTargetOpacity(const State& state) const;
 
   // Returns true if there is a fullscreen window open that causes the shelf
   // to be hidden.
@@ -315,8 +349,11 @@ class ASH_EXPORT ShelfLayoutManager
   // the shelf to be autohidden.
   bool IsShelfAutoHideForFullscreenMaximized() const;
 
+  // Returns true if the home gesture handler should handle the event.
+  bool ShouldHomeGestureHandleEvent(float scroll_y) const;
+
   // Gesture related functions:
-  void StartGestureDrag(const ui::GestureEvent& gesture_in_screen);
+  bool StartGestureDrag(const ui::GestureEvent& gesture_in_screen);
   void UpdateGestureDrag(const ui::GestureEvent& gesture_in_screen);
   void CompleteGestureDrag(const ui::GestureEvent& gesture_in_screen);
   void CompleteAppListDrag(const ui::GestureEvent& gesture_in_screen);
@@ -328,9 +365,18 @@ class ASH_EXPORT ShelfLayoutManager
   // on a visible shelf; other gestures should not change shelf visibility.
   bool IsSwipingCorrectDirection();
 
+  // Returns true if should change the visibility of the shelf after drag.
+  bool ShouldChangeVisibilityAfterDrag(
+      const ui::GestureEvent& gesture_in_screen);
+
+  // Updates the mask to limit the content to the non lock screen container.
+  // The mask will be removed if the workspace state is either in fullscreen
+  // or maximized.
+  void UpdateWorkspaceMask(wm::WorkspaceWindowState window_state);
+
   // True when inside UpdateBoundsAndOpacity() method. Used to prevent calling
   // UpdateBoundsAndOpacity() again from SetChildBounds().
-  bool updating_bounds_;
+  bool updating_bounds_ = false;
 
   bool in_shutdown_ = false;
 
@@ -343,20 +389,25 @@ class ASH_EXPORT ShelfLayoutManager
   ShelfWidget* shelf_widget_;
   Shelf* shelf_;
 
-  // Do any windows overlap the shelf? This is maintained by WorkspaceManager.
-  bool window_overlaps_shelf_ = false;
-
   // Whether the app list is visible. This is maintained by
   // OnAppListVisibilityChanged.
   bool is_app_list_visible_ = false;
 
+  // Whether the HomeLauncher is being dragged to, or animating to fullscreen.
+  // This is maintained by OnHomeLauncherTargetPositionChanged.
+  bool is_home_launcher_target_position_shown_ = false;
+
+  // Whether the HomeLauncher is shown. This is maintained by
+  // OnHomeLauncherAnimationComplete.
+  bool is_home_launcher_shown_ = false;
+
   base::OneShotTimer auto_hide_timer_;
 
   // Whether the mouse was over the shelf when the auto hide timer started.
   // False when neither the auto hide timer nor the timer task are running.
   bool mouse_over_shelf_when_auto_hide_timer_started_ = false;
 
-  base::ObserverList<ShelfLayoutManagerObserver> observers_;
+  base::ObserverList<ShelfLayoutManagerObserver>::Unchecked observers_;
 
   // The shelf reacts to gesture-drags, and can be set to auto-hide for certain
   // gestures. Swiping up from the shelf in tablet mode can open the
@@ -386,39 +437,46 @@ class ASH_EXPORT ShelfLayoutManager
   // Used to delay updating shelf background.
   UpdateShelfObserver* update_shelf_observer_ = nullptr;
 
-  // The bounds of the keyboard.
-  gfx::Rect keyboard_bounds_;
+  // The occluded bounds of the keyboard. See
+  // ui/keyboard/keyboard_controller_observer.h for details.
+  gfx::Rect keyboard_occluded_bounds_;
+
+  // The displaced bounds of the keyboard. See
+  // ui/keyboard/keyboard_controller_observer.h for details.
+  gfx::Rect keyboard_displaced_bounds_;
 
   // The bounds within the root window not occupied by the shelf nor the virtual
   // keyboard.
   gfx::Rect user_work_area_bounds_;
 
-  // The height of the ChromeVox panel at the top of the screen, which
-  // needs to be removed from the available work area.
-  int chromevox_panel_height_ = 0;
+  // The height of the accessibility panel at the top of the screen, which
+  // needs to be removed from the available work area. Used by ChromeVox.
+  int accessibility_panel_height_ = 0;
 
   // The height of the Docked Magnifier viewport at the top of the screen, which
-  // similarly to |chromevox_panel_height_| needs to be removed from the
+  // similarly to |accessibility_panel_height_| needs to be removed from the
   // available work area.
   int docked_magnifier_height_ = 0;
 
   // Whether background blur is enabled.
   const bool is_background_blur_enabled_;
 
+  // The display on which this shelf is shown.
+  display::Display display_;
+
   // The current shelf background. Should not be assigned to directly, use
   // MaybeUpdateShelfBackground() instead.
-  ShelfBackgroundType shelf_background_type_ = SHELF_BACKGROUND_OVERLAP;
+  ShelfBackgroundType shelf_background_type_ = SHELF_BACKGROUND_DEFAULT;
 
   // Shelf will become transparent if launcher is opened. Stores the shelf
   // background type before open the launcher when start to drag the launcher
   // from shelf.
   ShelfBackgroundType shelf_background_type_before_drag_ =
-      SHELF_BACKGROUND_OVERLAP;
+      SHELF_BACKGROUND_DEFAULT;
 
-  ScopedObserver<keyboard::KeyboardController,
-                 keyboard::KeyboardControllerObserver>
-      keyboard_observer_;
-  ScopedSessionObserver scoped_session_observer_;
+  ScopedSessionObserver scoped_session_observer_{this};
+  ScopedObserver<WallpaperController, ShelfLayoutManager>
+      wallpaper_controller_observer_{this};
 
   DISALLOW_COPY_AND_ASSIGN(ShelfLayoutManager);
 };
