@@ -14,12 +14,18 @@
 #include "ash/public/cpp/shelf_model_observer.h"
 #include "ash/public/interfaces/shelf.mojom.h"
 #include "ash/shelf/ink_drop_button_listener.h"
+#include "ash/shelf/overflow_bubble.h"
+#include "ash/shelf/overflow_bubble_view.h"
 #include "ash/shelf/shelf_button_pressed_metric_tracker.h"
 #include "ash/shelf/shelf_tooltip_manager.h"
+#include "ash/shell_observer.h"
+#include "ash/system/model/virtual_keyboard_model.h"
 #include "ash/wm/tablet_mode/tablet_mode_observer.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
+#include "base/timer/timer.h"
 #include "third_party/skia/include/core/SkColor.h"
+#include "ui/views/accessibility/view_accessibility.h"
 #include "ui/views/animation/bounds_animator_observer.h"
 #include "ui/views/animation/ink_drop_state.h"
 #include "ui/views/context_menu_controller.h"
@@ -36,7 +42,8 @@ class SimpleMenuModel;
 namespace views {
 class BoundsAnimator;
 class MenuRunner;
-}
+class Separator;
+}  // namespace views
 
 namespace ash {
 class AppListButton;
@@ -46,7 +53,7 @@ class OverflowBubble;
 class OverflowButton;
 class ScopedRootWindowForNewWindows;
 class Shelf;
-class ShelfButton;
+class ShelfAppButton;
 class ShelfModel;
 struct ShelfItem;
 class ShelfMenuModelAdapter;
@@ -62,13 +69,58 @@ enum ShelfAlignmentUmaEnumValue {
 
 // ShelfView contains the shelf items visible within an active user session.
 // ShelfView and LoginShelfView should never be shown together.
+
+// In the following example, there are 12 apps to place on the shelf, plus
+// the app list and back buttons, which make 14 shelf items in total.
+//
+// If there is enough screen space, all icons can fit:
+//
+// -----------------------------------------------------------------
+// | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 10 | 11 | 12 | 13 |
+// -----------------------------------------------------------------
+//   ^                                               ^
+//   |                                               |
+// first_visible_index = 0                 last_visible_index = 13
+// (back button = 0 is hidden)
+//
+// Where:
+//     0 = back button (only shown in tablet mode)
+//     1 = app list button
+//
+// If screen space is more constrained, some icons are placed in an overflow
+// menu (which holds its own instance of ShelfView):
+//
+//            first_visible_index = 10
+//               (for the overflow)     last_visible_index = 13 (for overflow)
+//                                |               |
+//                                v               v
+//                              ---------------------
+//                              | 10 | 11 | 12 | 13 |
+//                              ---------------------
+//                                        ^
+// -------------------------------------------
+// | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | ... |
+// -------------------------------------------
+//   ^                                    ^
+//   |                                    |
+// first_visible_index = 0       last_visible_index = 10
+//   (for the main shelf)         (the overflow button)
+//  (back button = 0
+//           is hidden)
+//
+// Note that last_visible_index is 10 (not 9) even though the overflow button
+// doesn't shift the array of indices.
+
 class ASH_EXPORT ShelfView : public views::View,
                              public ShelfModelObserver,
+                             public ShellObserver,
                              public InkDropButtonListener,
                              public views::ContextMenuController,
                              public views::FocusTraversable,
                              public views::BoundsAnimatorObserver,
-                             public app_list::ApplicationDragAndDropHost {
+                             public app_list::ApplicationDragAndDropHost,
+                             public ash::TabletModeObserver,
+                             public VirtualKeyboardModel::Observer {
  public:
   ShelfView(ShelfModel* model, Shelf* shelf, ShelfWidget* shelf_widget);
   ~ShelfView() override;
@@ -78,22 +130,18 @@ class ASH_EXPORT ShelfView : public views::View,
 
   void Init();
 
-  void OnShelfAlignmentChanged();
-
   // Returns the ideal bounds of the specified item, or an empty rect if id
   // isn't know. If the item is in an overflow shelf, the overflow icon location
   // will be returned.
   gfx::Rect GetIdealBoundsOfItemIcon(const ShelfID& id);
 
-  // Repositions the icon for the specified item by the midpoint of the window.
-  void UpdatePanelIconPosition(const ShelfID& id, const gfx::Point& midpoint);
-
-  // Returns true if we're showing a menu.
+  // Returns true if we're showing a menu. Note the menu could be either the
+  // context menu or the application select menu.
   bool IsShowingMenu() const;
 
   // Returns true if we're showing a menu for |view|. |view| could be a
-  // ShelfButton or the ShelfView.
-  bool IsShowingMenuForView(views::View* view) const;
+  // ShelfAppButton or the ShelfView.
+  bool IsShowingMenuForView(const views::View* view) const;
 
   // Returns true if overflow bubble is shown.
   bool IsShowingOverflowBubble() const;
@@ -106,6 +154,12 @@ class ASH_EXPORT ShelfView : public views::View,
 
   AppListButton* GetAppListButton() const;
   BackButton* GetBackButton() const;
+  OverflowButton* GetOverflowButton() const;
+
+  // Updates the union of all the shelf item bounds shown by this shelf view.
+  // This is used to determine the common area where the mouse can hover
+  // for showing tooltips without stuttering over gaps.
+  void UpdateVisibleShelfItemBoundsUnion();
 
   // Returns true if the mouse cursor exits the area for launcher tooltip.
   // There are thin gaps between launcher buttons but the tooltip shouldn't hide
@@ -116,13 +170,17 @@ class ASH_EXPORT ShelfView : public views::View,
   // Returns true if a tooltip should be shown for the shelf item |view|.
   bool ShouldShowTooltipForView(const views::View* view) const;
 
-  // Returns the title of the shelf item |view|.
-  base::string16 GetTitleForView(const views::View* view) const;
-
   // Returns rectangle bounding all visible launcher items. Used screen
   // coordinate system.
   gfx::Rect GetVisibleItemsBoundsInScreen();
 
+  // Overridden from views::View:
+  gfx::Size CalculatePreferredSize() const override;
+  void OnBoundsChanged(const gfx::Rect& previous_bounds) override;
+  FocusTraversable* GetPaneFocusTraversable() override;
+  void GetAccessibleNodeData(ui::AXNodeData* node_data) override;
+  View* GetTooltipHandlerForPoint(const gfx::Point& point) override;
+
   // InkDropButtonListener:
   void ButtonPressed(views::Button* sender,
                      const ui::Event& event,
@@ -140,11 +198,19 @@ class ASH_EXPORT ShelfView : public views::View,
                            const gfx::Vector2d& cursor_offset_from_center,
                            float scale_factor) override;
 
+  // Overridden from ash::TabletModeObserver:
+  void OnTabletModeStarted() override;
+  void OnTabletModeEnded() override;
+
+  // Overridden from VirtualKeyboardModel::Observer:
+  void OnVirtualKeyboardVisibilityChanged() override;
+
   void CreateDragIconProxyByLocationWithNoAnimation(
       const gfx::Point& origin_in_screen_coordinates,
       const gfx::ImageSkia& icon,
       views::View* replaced_view,
-      float scale_factor) override;
+      float scale_factor,
+      int blur_radius) override;
 
   void UpdateDragIconProxy(
       const gfx::Point& location_in_screen_coordinates) override;
@@ -174,27 +240,64 @@ class ASH_EXPORT ShelfView : public views::View,
                                Pointer pointer,
                                bool canceled);
 
-  // Updates the background for the shelf items.
-  void UpdateShelfItemBackground(SkColor color);
+  // Returns whether |item| should belong in the pinned section of the shelf.
+  bool IsItemPinned(const ShelfItem& item) const;
+
+  // Enumerates the shelf items that are centered in the new UI and returns
+  // the total size they occupy.
+  int GetDimensionOfCenteredShelfItems() const;
+
+  // Returns the index of the item after which the separator should be shown,
+  // or -1 if no separator is required.
+  int GetSeparatorIndex() const;
+
+  // Update the layout when entering or exiting tablet mode. Have the owning
+  // widget call this instead of observing changes ourselves to ensure this
+  // happens after the tablet related changes in ShelfController.
+  void OnTabletModeChanged();
 
   // True if the current |drag_view_| is the given |drag_view|.
-  bool IsDraggedView(const ShelfButton* drag_view) const;
+  bool IsDraggedView(const ShelfAppButton* drag_view) const;
 
   // Returns the list of open windows that correspond to the app represented by
   // this shelf view.
   const std::vector<aura::Window*> GetOpenWindowsForShelfView(
       views::View* view);
 
+  views::View* FindFirstOrLastFocusableChild(bool last) const;
+
   // Return the view model for test purposes.
   const views::ViewModel* view_model_for_test() const {
     return view_model_.get();
   }
 
-  // Return the main shelf. This will return nullptr if this is not called on
-  // the overflow shelf.
-  ShelfView* main_shelf() { return main_shelf_; }
+  // Returns the main shelf. This can be called on either the main shelf
+  // or the overflow shelf.
+  ShelfView* main_shelf() { return main_shelf_ ? main_shelf_ : this; }
+  // Returns the overflow shelf. This can be called on either the main shelf
+  // or the overflow shelf. Returns nullptr if there is no overflow shelf.
+  ShelfView* overflow_shelf() {
+    if (is_overflow_mode())
+      return this;
+    return overflow_bubble_ ? overflow_bubble_->bubble_view()->shelf_view()
+                            : nullptr;
+  }
 
-  const ShelfButton* drag_view() const { return drag_view_; }
+  const ShelfAppButton* drag_view() const { return drag_view_; }
+
+  // Returns true when this ShelfView is used for Overflow Bubble.
+  // In this mode, it does not show app list and overflow button.
+  // Note:
+  //   * When Shelf can contain only one item (the overflow button) due to very
+  //     small resolution screen, the overflow bubble can show the app list
+  //     button.
+  bool is_overflow_mode() const { return overflow_mode_; }
+
+  int first_visible_index() const { return first_visible_index_; }
+  int last_visible_index() const { return last_visible_index_; }
+  ShelfWidget* shelf_widget() const { return shelf_widget_; }
+  OverflowBubble* overflow_bubble() { return overflow_bubble_.get(); }
+  views::ViewModel* view_model() { return view_model_.get(); }
 
  private:
   friend class ShelfViewTestAPI;
@@ -211,14 +314,6 @@ class ASH_EXPORT ShelfView : public views::View,
   // Minimum distance before drag starts.
   static const int kMinimumDragDistance;
 
-  // Returns true when this ShelfView is used for Overflow Bubble.
-  // In this mode, it does not show app list, panel and overflow button.
-  // Note:
-  //   * When Shelf can contain only one item (overflow button) due to very
-  //     small resolution screen, overflow bubble can show app list and panel
-  //     button.
-  bool is_overflow_mode() const { return overflow_mode_; }
-
   bool dragging() const { return drag_pointer_ != NONE; }
 
   // Sets the bounds of each view to its ideal bounds.
@@ -227,17 +322,15 @@ class ASH_EXPORT ShelfView : public views::View,
   // Update all button's visibility in overflow.
   void UpdateAllButtonsVisibilityInOverflowMode();
 
+  void LayoutAppListAndBackButtonHighlight() const;
+
   // Calculates the ideal bounds. The bounds of each button corresponding to an
   // item in the model is set in |view_model_|.
   void CalculateIdealBounds(gfx::Rect* overflow_bounds) const;
 
   // Returns the index of the last view whose max primary axis coordinate is
   // less than |max_value|. Returns -1 if nothing fits, or there are no views.
-  int DetermineLastVisibleIndex(int max_value) const;
-
-  // Returns the index of the first panel whose min primary axis coordinate is
-  // at least |min_value|. Returns the index past the last panel if none fit.
-  int DetermineFirstVisiblePanelIndex(int min_value) const;
+  int IndexOfLastItemThatFitsSize(int max_value) const;
 
   // Animates the bounds of each view to its ideal bounds.
   void AnimateToIdealBounds();
@@ -255,6 +348,15 @@ class ASH_EXPORT ShelfView : public views::View,
   // Invoked when the mouse is dragged. Updates the models as appropriate.
   void ContinueDrag(const ui::LocatedEvent& event);
 
+  // Scroll the view to show more content in the direction of the user's drag.
+  void ScrollForUserDrag(int offset);
+
+  // Increase the speed of an existing scroll.
+  void SpeedUpDragScrolling();
+
+  // Reorder |drag_view_| according to the latest dragging coordinate.
+  void MoveDragViewTo(int primary_axis_coordinate);
+
   // Ends the drag on the other shelf. (ie if we are on main shelf, ends drag on
   // the overflow shelf). Invoked when a shelf item is being dragged from one
   // shelf to the other.
@@ -285,8 +387,6 @@ class ASH_EXPORT ShelfView : public views::View,
   // Note:
   //  * When overflow button is visible, returns bounds from first item
   //    to overflow button.
-  //  * When overflow button is visible and one or more panel items exists,
-  //    returns bounds from first item to last panel item.
   //  * In the overflow mode, returns only bubble's bounds.
   gfx::Rect GetBoundsForDragInsertInScreen();
 
@@ -312,21 +412,8 @@ class ASH_EXPORT ShelfView : public views::View,
                               const gfx::Point& location,
                               bool context_menu) const;
 
-  // Gets the menu anchor position for a menu. |for_item| is true if the menu is
-  // for an item on the shelf, or false if the menu is for the shelf view
-  // itself, |context_menu| is whether the menu will be an application menu or
-  // context menu, and |touch_menu| is whether the menu was initiated by touch.
-  views::MenuAnchorPosition GetMenuAnchorPosition(bool for_item,
-                                                  bool context_menu) const;
-
-  // Overridden from views::View:
-  gfx::Size CalculatePreferredSize() const override;
-  void OnBoundsChanged(const gfx::Rect& previous_bounds) override;
-  void OnPaint(gfx::Canvas* canvas) override;
-  FocusTraversable* GetPaneFocusTraversable() override;
-  void GetAccessibleNodeData(ui::AXNodeData* node_data) override;
-  void ViewHierarchyChanged(
-      const ViewHierarchyChangedDetails& details) override;
+  void AnnounceShelfAlignment();
+  void AnnounceShelfAutohideBehavior();
 
   // Overridden from ui::EventHandler:
   void OnGestureEvent(ui::GestureEvent* event) override;
@@ -340,6 +427,11 @@ class ASH_EXPORT ShelfView : public views::View,
   void ShelfItemDelegateChanged(const ShelfID& id,
                                 ShelfItemDelegate* old_delegate,
                                 ShelfItemDelegate* delegate) override;
+  void ShelfItemStatusChanged(const ShelfID& id) override;
+
+  // Overridden from ShellObserver:
+  void OnShelfAlignmentChanged(aura::Window* root_window) override;
+  void OnShelfAutoHideBehaviorChanged(aura::Window* root_window) override;
 
   // Handles the result when querying ShelfItemDelegates for context menu items.
   // Shows a default shelf context menu with optional extra custom |menu_items|.
@@ -401,6 +493,9 @@ class ASH_EXPORT ShelfView : public views::View,
   // Updates the back button opacity and focus behavior based on tablet mode.
   void UpdateBackButton();
 
+  // Set background blur to the dragged image. |size| is the image size.
+  void SetDragImageBlur(const gfx::Size& size, int blur_radius);
+
   // The model; owned by Launcher.
   ShelfModel* model_;
 
@@ -415,8 +510,12 @@ class ASH_EXPORT ShelfView : public views::View,
   // item in |model_|.
   std::unique_ptr<views::ViewModel> view_model_;
 
-  // Index of first visible launcher item.
-  int first_visible_index_ = 0;
+  // Index of the first visible launcher item. This is either:
+  // * 0 (back button) for the main shelf when tablet mode is on
+  // * 1 (app list button) for the main shelf when tablet mode is off
+  // * > 1 when this shelf view is the overflow shelf view and only shows a
+  //   subset of items.
+  mutable int first_visible_index_ = 0;
 
   // Last index of a launcher button that is visible
   // (does not go into overflow).
@@ -438,10 +537,10 @@ class ASH_EXPORT ShelfView : public views::View,
 
   // The view being dragged. This is set immediately when the mouse is pressed.
   // |dragging_| is set only if the mouse is dragged far enough.
-  ShelfButton* drag_view_ = nullptr;
+  ShelfAppButton* drag_view_ = nullptr;
 
   // The view showing a context menu. This can be either a ShelfView or
-  // ShelfButton.
+  // ShelfAppButton.
   views::View* menu_owner_ = nullptr;
 
   // Position of the mouse down event in |drag_view_|'s coordinates.
@@ -463,9 +562,10 @@ class ASH_EXPORT ShelfView : public views::View,
   // True when an item being inserted or removed in the model cancels a drag.
   bool cancelling_drag_model_changed_ = false;
 
-  // Index of the last hidden launcher item. If there are no hidden items this
-  // will be equal to last_visible_index_ + 1.
-  mutable int last_hidden_index_ = 0;
+  // The item with an in-flight async request for a context menu or selection
+  // (which shows a shelf item application menu if multiple windows are open).
+  // Used to avoid multiple concurrent menu requests. The value is null if none.
+  ShelfID item_awaiting_response_;
 
   // The timestamp of the event which closed the last menu - or 0.
   base::TimeTicks closing_event_time_;
@@ -484,6 +584,9 @@ class ASH_EXPORT ShelfView : public views::View,
   // the item can be dragged outside the app grid.
   std::unique_ptr<ash::DragImageView> drag_image_;
 
+  // The owner of a mask layer used to clip the background blur.
+  std::unique_ptr<ui::LayerOwner> drag_image_mask_;
+
   // The cursor offset to the middle of the dragged item.
   gfx::Vector2d drag_image_offset_;
 
@@ -520,9 +623,30 @@ class ASH_EXPORT ShelfView : public views::View,
   // Tracks UMA metrics based on shelf button press actions.
   ShelfButtonPressedMetricTracker shelf_button_pressed_metric_tracker_;
 
-  // Color used to paint the background behind the app list button and back
-  // button.
-  SkColor shelf_item_background_color_;
+  // A reference to the view used as a separator between pinned and unpinned
+  // items.
+  views::Separator* separator_ = nullptr;
+
+  // The union of all visible shelf item bounds. Used for showing tooltips in
+  // a continuous manner.
+  gfx::Rect visible_shelf_item_bounds_union_;
+
+  // A view to draw a background behind the app list and back buttons.
+  // Owned by the view hierarchy.
+  views::View* back_and_app_list_background_ = nullptr;
+
+  // A view used to make accessibility announcements (changes in the shelf's
+  // alignment or auto-hide state).
+  views::View* announcement_view_ = nullptr;  // Owned by ShelfView
+
+  // For dragging: -1 if scrolling back, 1 if scrolling forward, 0 if neither.
+  int drag_scroll_dir_ = 0;
+
+  // Used to periodically call ScrollForUserDrag.
+  base::RepeatingTimer scrolling_timer_;
+
+  // Used to call SpeedUpDragScrolling.
+  base::OneShotTimer speed_up_drag_scrolling_;
 
   base::WeakPtrFactory<ShelfView> weak_factory_;
 
