@@ -11,11 +11,12 @@
 #include "ash/public/cpp/ash_features.h"
 #include "ash/public/cpp/ash_switches.h"
 #include "ash/public/cpp/window_properties.h"
-#include "ash/public/interfaces/kiosk_app_info.mojom.h"
 #include "ash/root_window_controller.h"
 #include "ash/session/session_controller.h"
 #include "ash/shelf/app_list_button.h"
 #include "ash/shelf/login_shelf_view.h"
+#include "ash/shelf/overflow_bubble.h"
+#include "ash/shelf/overflow_bubble_view.h"
 #include "ash/shelf/shelf.h"
 #include "ash/shelf/shelf_background_animator_observer.h"
 #include "ash/shelf/shelf_constants.h"
@@ -24,9 +25,11 @@
 #include "ash/shell.h"
 #include "ash/system/status_area_layout_manager.h"
 #include "ash/system/status_area_widget.h"
-#include "ash/system/tray/system_tray.h"
+#include "ash/wm/window_util.h"
 #include "base/command_line.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "ui/compositor/layer.h"
+#include "ui/compositor/layer_owner.h"
 #include "ui/compositor/scoped_layer_animation_settings.h"
 #include "ui/gfx/skbitmap_operations.h"
 #include "ui/views/accessible_pane_view.h"
@@ -39,6 +42,10 @@
 namespace ash {
 namespace {
 
+constexpr int kShelfRoundedCornerRadius = 28;
+constexpr int kShelfBlurRadius = 10;
+constexpr float kShelfBlurQuality = 0.33f;
+
 // Return the first or last focusable child of |root|.
 views::View* FindFirstOrLastFocusableChild(views::View* root,
                                            bool find_last_child) {
@@ -73,8 +80,6 @@ class ShelfWidget::DelegateView : public views::WidgetDelegate,
 
   FocusCycler* focus_cycler() { return focus_cycler_; }
 
-  ui::Layer* opaque_background() { return &opaque_background_; }
-
   void SetParentLayer(ui::Layer* layer);
 
   void set_default_last_focusable_child(bool default_last_focusable_child) {
@@ -88,6 +93,8 @@ class ShelfWidget::DelegateView : public views::WidgetDelegate,
 
   bool CanActivate() const override;
   void ReorderChildLayers(ui::Layer* parent_layer) override;
+  void UpdateBackgroundBlur();
+  void UpdateOpaqueBackground();
   // This will be called when the parent local bounds change.
   void OnBoundsChanged(const gfx::Rect& old_bounds) override;
 
@@ -97,6 +104,8 @@ class ShelfWidget::DelegateView : public views::WidgetDelegate,
   // ShelfBackgroundAnimatorObserver:
   void UpdateShelfBackground(SkColor color) override;
 
+  SkColor GetShelfBackgroundColor() const;
+
  private:
   ShelfWidget* shelf_widget_;
   FocusCycler* focus_cycler_;
@@ -104,9 +113,16 @@ class ShelfWidget::DelegateView : public views::WidgetDelegate,
   // ShelfBackgroundAnimator.
   ui::Layer opaque_background_;
 
+  // A mask to show rounded corners when appropriate.
+  std::unique_ptr<ui::LayerOwner> mask_ = nullptr;
+
   // When true, the default focus of the shelf is the last focusable child.
   bool default_last_focusable_child_ = false;
 
+  // Cache the state of the background blur so that it can be updated only
+  // when necessary.
+  bool background_is_currently_blurred_ = false;
+
   DISALLOW_COPY_AND_ASSIGN(DelegateView);
 };
 
@@ -119,7 +135,8 @@ ShelfWidget::DelegateView::DelegateView(ShelfWidget* shelf_widget)
 
   SetLayoutManager(std::make_unique<views::FillLayout>());
   set_allow_deactivate_on_esc(true);
-  opaque_background_.SetBounds(GetLocalBounds());
+
+  UpdateOpaqueBackground();
 }
 
 ShelfWidget::DelegateView::~DelegateView() = default;
@@ -131,12 +148,12 @@ bool ShelfWidget::IsUsingViewsShelf() {
       return true;
     // See https://crbug.com/798869.
     case session_manager::SessionState::OOBE:
-      return false;
+    case session_manager::SessionState::LOGIN_PRIMARY:
+      return true;
     case session_manager::SessionState::LOCKED:
     case session_manager::SessionState::LOGIN_SECONDARY:
       return switches::IsUsingViewsLock();
     case session_manager::SessionState::UNKNOWN:
-    case session_manager::SessionState::LOGIN_PRIMARY:
     case session_manager::SessionState::LOGGED_IN_NOT_ACTIVE:
       return features::IsViewsLoginEnabled();
   }
@@ -148,6 +165,22 @@ void ShelfWidget::DelegateView::SetParentLayer(ui::Layer* layer) {
 }
 
 bool ShelfWidget::DelegateView::CanActivate() const {
+  // Allow activations coming from the overflow bubble if it is currently shown
+  // and active.
+  aura::Window* active_window = wm::GetActiveWindow();
+  aura::Window* bubble_window = nullptr;
+  aura::Window* shelf_window = shelf_widget_->GetNativeWindow();
+  if (shelf_widget_->IsShowingOverflowBubble()) {
+    bubble_window = shelf_widget_->shelf_view_->overflow_bubble()
+                        ->bubble_view()
+                        ->GetWidget()
+                        ->GetNativeWindow();
+  }
+  if (active_window &&
+      (active_window == bubble_window || active_window == shelf_window)) {
+    return true;
+  }
+
   // Only allow activation from the focus cycler, not from mouse events, etc.
   return focus_cycler_ && focus_cycler_->widget_activating() == GetWidget();
 }
@@ -157,33 +190,130 @@ void ShelfWidget::DelegateView::ReorderChildLayers(ui::Layer* parent_layer) {
   parent_layer->StackAtBottom(&opaque_background_);
 }
 
+void ShelfWidget::DelegateView::UpdateBackgroundBlur() {
+  // Blur only if the background is visible.
+  const bool should_blur_background =
+      opaque_background_.visible() &&
+      shelf_widget_->shelf_layout_manager()->ShouldBlurShelfBackground();
+  if (should_blur_background == background_is_currently_blurred_)
+    return;
+
+  opaque_background_.SetBackgroundBlur(should_blur_background ? kShelfBlurRadius
+                                                              : 0);
+  opaque_background_.SetBackdropFilterQuality(kShelfBlurQuality);
+
+  background_is_currently_blurred_ = should_blur_background;
+}
+
+void ShelfWidget::DelegateView::UpdateOpaqueBackground() {
+  const gfx::Rect local_bounds = GetLocalBounds();
+  gfx::Rect opaque_background_bounds = local_bounds;
+
+  const Shelf* shelf = shelf_widget_->shelf();
+  const ShelfBackgroundType background_type =
+      shelf_widget_->GetBackgroundType();
+
+  // If the app list is showing in clamshell mode, we should hide the shelf.
+  // otherwise, we should show it again. This creates a 'blending' effect
+  // between the two
+  if (background_type == SHELF_BACKGROUND_APP_LIST) {
+    opaque_background_.SetVisible(false);
+    UpdateBackgroundBlur();
+    return;
+  }
+
+  if (!opaque_background_.visible())
+    opaque_background_.SetVisible(true);
+
+  // Extend the opaque layer a little bit to handle "overshoot" gestures
+  // gracefully (the user drags the shelf further than it can actually go).
+  // That way:
+  // 1) When the shelf has rounded corners, only two of them are visible,
+  // 2) Even when the shelf is squared, it doesn't tear off the screen edge
+  // when dragged away.
+  // To achieve this, we extend the layer in the same direction where the shelf
+  // is aligned (downwards for a bottom shelf, etc.).
+  const int radius = kShelfRoundedCornerRadius;
+  const int safety_margin = 3 * radius;
+  opaque_background_bounds.Inset(
+      -shelf->SelectValueForShelfAlignment(0, safety_margin, 0), 0,
+      -shelf->SelectValueForShelfAlignment(0, 0, safety_margin),
+      -shelf->SelectValueForShelfAlignment(safety_margin, 0, 0));
+
+  // Show rounded corners except in maximized (which includes split view) mode.
+  if (background_type == SHELF_BACKGROUND_MAXIMIZED) {
+    mask_ = nullptr;
+    opaque_background_.SetMaskLayer(nullptr);
+  } else {
+    if (!mask_) {
+      mask_ = views::Painter::CreatePaintedLayer(
+          views::Painter::CreateSolidRoundRectPainter(SK_ColorBLACK, radius));
+      mask_->layer()->SetFillsBoundsOpaquely(false);
+      opaque_background_.SetMaskLayer(mask_->layer());
+    }
+    if (mask_->layer()->bounds() != opaque_background_bounds)
+      mask_->layer()->SetBounds(opaque_background_bounds);
+  }
+  opaque_background_.SetBounds(opaque_background_bounds);
+  UpdateBackgroundBlur();
+  SchedulePaint();
+}
+
 void ShelfWidget::DelegateView::OnBoundsChanged(const gfx::Rect& old_bounds) {
-  opaque_background_.SetBounds(GetLocalBounds());
+  UpdateOpaqueBackground();
 }
 
 views::View* ShelfWidget::DelegateView::GetDefaultFocusableChild() {
-  // If views-based login shelf is shown, we want to focus either its first or
-  // last child, otherwise focus on the first child as default.
-  if (IsUsingViewsShelf())
+  if (!IsUsingViewsShelf())
+    return GetFirstFocusableChild();
+
+  if (shelf_widget_->login_shelf_view_->visible())
     return FindFirstOrLastFocusableChild(shelf_widget_->login_shelf_view_,
                                          default_last_focusable_child_);
-  return GetFirstFocusableChild();
+  else
+    return shelf_widget_->shelf_view_->FindFirstOrLastFocusableChild(
+        default_last_focusable_child_);
 }
 
 void ShelfWidget::DelegateView::UpdateShelfBackground(SkColor color) {
   opaque_background_.SetColor(color);
+  UpdateOpaqueBackground();
+}
+
+SkColor ShelfWidget::DelegateView::GetShelfBackgroundColor() const {
+  return opaque_background_.background_color();
+}
+
+bool ShelfWidget::GetHitTestRects(aura::Window* target,
+                                  gfx::Rect* hit_test_rect_mouse,
+                                  gfx::Rect* hit_test_rect_touch) {
+  // This should only get called when the login shelf is visible, i.e. not
+  // during an active session. In an active session, hit test rects should be
+  // calculated higher up in the class hierarchy by |EasyResizeWindowTargeter|.
+  // When in OOBE or locked/login screen, let events pass through empty parts
+  // of the shelf.
+  DCHECK(login_shelf_view_->visible());
+  gfx::Rect login_view_button_bounds =
+      login_shelf_view_->get_button_union_bounds();
+  aura::Window* source = login_shelf_view_->GetWidget()->GetNativeWindow();
+  aura::Window::ConvertRectToTarget(source, target->parent(),
+                                    &login_view_button_bounds);
+  *hit_test_rect_mouse = login_view_button_bounds;
+  *hit_test_rect_touch = login_view_button_bounds;
+  return true;
 }
 
 ShelfWidget::ShelfWidget(aura::Window* shelf_container, Shelf* shelf)
     : shelf_(shelf),
+      background_animator_(SHELF_BACKGROUND_DEFAULT,
+                           shelf_,
+                           Shell::Get()->wallpaper_controller()),
+      shelf_layout_manager_(new ShelfLayoutManager(this, shelf)),
       delegate_view_(new DelegateView(this)),
       shelf_view_(new ShelfView(Shell::Get()->shelf_model(), shelf_, this)),
       login_shelf_view_(
           new LoginShelfView(RootWindowController::ForWindow(shelf_container)
                                  ->lock_screen_action_background_controller())),
-      background_animator_(SHELF_BACKGROUND_DEFAULT,
-                           shelf_,
-                           Shell::Get()->wallpaper_controller()),
       scoped_session_observer_(this) {
   DCHECK(shelf_container);
   DCHECK(shelf_);
@@ -191,10 +321,11 @@ ShelfWidget::ShelfWidget(aura::Window* shelf_container, Shelf* shelf)
   views::Widget::InitParams params(
       views::Widget::InitParams::TYPE_WINDOW_FRAMELESS);
   params.name = "ShelfWidget";
-  params.opacity = views::Widget::InitParams::TRANSLUCENT_WINDOW;
+  params.layer_type = ui::LAYER_NOT_DRAWN;
   params.ownership = views::Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
   params.delegate = delegate_view_;
   params.parent = shelf_container;
+
   Init(params);
 
   // The shelf should not take focus when initially shown.
@@ -208,7 +339,6 @@ ShelfWidget::ShelfWidget(aura::Window* shelf_container, Shelf* shelf)
   GetContentsView()->AddChildView(shelf_view_);
   GetContentsView()->AddChildView(login_shelf_view_);
 
-  shelf_layout_manager_ = new ShelfLayoutManager(this, shelf_);
   shelf_layout_manager_->AddObserver(this);
   shelf_container->SetLayoutManager(shelf_layout_manager_);
   background_animator_.PaintBackground(
@@ -217,9 +347,8 @@ ShelfWidget::ShelfWidget(aura::Window* shelf_container, Shelf* shelf)
 
   views::Widget::AddObserver(this);
 
-  // Calls back into |this| and depends on |shelf_view_|.
-  background_animator_.AddObserver(this);
   background_animator_.AddObserver(delegate_view_);
+  shelf_->AddObserver(this);
 }
 
 ShelfWidget::~ShelfWidget() {
@@ -245,7 +374,7 @@ void ShelfWidget::Shutdown() {
 
   // Don't need to update the shelf background during shutdown.
   background_animator_.RemoveObserver(delegate_view_);
-  background_animator_.RemoveObserver(this);
+  shelf_->RemoveObserver(this);
 
   // Don't need to observe focus/activation during shutdown.
   Shell::Get()->focus_cycler()->RemoveWidget(this);
@@ -281,17 +410,18 @@ int ShelfWidget::GetBackgroundAlphaValue(
 void ShelfWidget::OnShelfAlignmentChanged() {
   // Check added for http://crbug.com/738011.
   CHECK(status_area_widget_);
-  shelf_view_->OnShelfAlignmentChanged();
   status_area_widget_->UpdateAfterShelfAlignmentChange();
-  delegate_view_->SchedulePaint();
+  // This call will in turn trigger a call to delegate_view_->SchedulePaint().
+  delegate_view_->UpdateOpaqueBackground();
+}
+
+void ShelfWidget::OnTabletModeChanged() {
+  shelf_view_->OnTabletModeChanged();
 }
 
 void ShelfWidget::PostCreateShelf() {
   SetFocusCycler(Shell::Get()->focus_cycler());
 
-  // Ensure the newly created |shelf_| gets current values.
-  background_animator_.NotifyObserver(this);
-
   shelf_layout_manager_->LayoutShelf();
   shelf_layout_manager_->UpdateAutoHideState();
   ShowIfHidden();
@@ -301,7 +431,7 @@ bool ShelfWidget::IsShowingAppList() const {
   return GetAppListButton() && GetAppListButton()->is_showing_app_list();
 }
 
-bool ShelfWidget::IsShowingContextMenu() const {
+bool ShelfWidget::IsShowingMenu() const {
   return shelf_view_->IsShowingMenu();
 }
 
@@ -319,17 +449,6 @@ FocusCycler* ShelfWidget::GetFocusCycler() {
   return delegate_view_->focus_cycler();
 }
 
-void ShelfWidget::UpdateIconPositionForPanel(aura::Window* panel) {
-  ShelfID id = ShelfID::Deserialize(panel->GetProperty(kShelfIDKey));
-  if (id.IsNull())
-    return;
-
-  aura::Window* shelf_window = this->GetNativeWindow();
-  gfx::Rect bounds = panel->GetBoundsInScreen();
-  ::wm::ConvertRectFromScreen(shelf_window, &bounds);
-  shelf_view_->UpdatePanelIconPosition(id, bounds.CenterPoint());
-}
-
 gfx::Rect ShelfWidget::GetScreenBoundsOfItemIconForWindow(
     aura::Window* window) {
   ShelfID id = ShelfID::Deserialize(window->GetProperty(kShelfIDKey));
@@ -357,11 +476,6 @@ ShelfWidget::GetDragAndDropHostForAppList() {
   return shelf_view_;
 }
 
-void ShelfWidget::SetLoginKioskApps(
-    std::vector<mojom::KioskAppInfoPtr> kiosk_apps) {
-  login_shelf_view_->SetKioskApps(std::move(kiosk_apps));
-}
-
 void ShelfWidget::set_default_last_focusable_child(
     bool default_last_focusable_child) {
   delegate_view_->set_default_last_focusable_child(
@@ -370,14 +484,19 @@ void ShelfWidget::set_default_last_focusable_child(
 
 void ShelfWidget::OnWidgetActivationChanged(views::Widget* widget,
                                             bool active) {
-  if (active)
+  if (active) {
+    // Do not focus the default element if the widget activation came from the
+    // overflow bubble focus cycling. The setter of
+    // |activated_from_overflow_bubble_| should handle focusing the correct
+    // view.
+    if (activated_from_overflow_bubble_) {
+      activated_from_overflow_bubble_ = false;
+      return;
+    }
     delegate_view_->SetPaneFocusAndFocusDefault();
-  else
+  } else {
     delegate_view_->GetFocusManager()->ClearFocus();
-}
-
-void ShelfWidget::UpdateShelfItemBackground(SkColor color) {
-  shelf_view_->UpdateShelfItemBackground(color);
+  }
 }
 
 void ShelfWidget::WillDeleteShelfLayoutManager() {
@@ -385,6 +504,11 @@ void ShelfWidget::WillDeleteShelfLayoutManager() {
   shelf_layout_manager_ = nullptr;
 }
 
+void ShelfWidget::OnBackgroundTypeChanged(ShelfBackgroundType background_type,
+                                          AnimationChangeType change_type) {
+  delegate_view_->UpdateOpaqueBackground();
+}
+
 void ShelfWidget::OnSessionStateChanged(session_manager::SessionState state) {
   // Do not show shelf widget:
   // * when views based shelf is disabled
@@ -424,6 +548,10 @@ void ShelfWidget::OnSessionStateChanged(session_manager::SessionState state) {
   login_shelf_view_->UpdateAfterSessionStateChange(state);
 }
 
+SkColor ShelfWidget::GetShelfBackgroundColor() const {
+  return delegate_view_->GetShelfBackgroundColor();
+}
+
 void ShelfWidget::HideIfShown() {
   if (IsVisible())
     Hide();
@@ -434,4 +562,16 @@ void ShelfWidget::ShowIfHidden() {
     Show();
 }
 
+void ShelfWidget::OnMouseEvent(ui::MouseEvent* event) {
+  if (event->type() == ui::ET_MOUSE_PRESSED)
+    keyboard::KeyboardController::Get()->HideKeyboardImplicitlyByUser();
+  views::Widget::OnMouseEvent(event);
+}
+
+void ShelfWidget::OnGestureEvent(ui::GestureEvent* event) {
+  if (event->type() == ui::ET_GESTURE_TAP_DOWN)
+    keyboard::KeyboardController::Get()->HideKeyboardImplicitlyByUser();
+  views::Widget::OnGestureEvent(event);
+}
+
 }  // namespace ash
