@@ -11,17 +11,16 @@
 
 #include "ash/ash_export.h"
 #include "ash/metrics/user_metrics_recorder.h"
-#include "ash/public/cpp/app_list/app_list_constants.h"
 #include "ash/public/cpp/shelf_types.h"
 #include "ash/session/session_observer.h"
-#include "ash/wm/cursor_manager_chromeos.h"
 #include "ash/wm/system_modal_container_event_filter_delegate.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
-#include "chromeos/chromeos_switches.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "ui/aura/window.h"
+#include "ui/base/ui_base_types.h"
 #include "ui/display/screen.h"
 #include "ui/events/event_target.h"
 #include "ui/wm/core/cursor_manager.h"
@@ -31,11 +30,9 @@ class PrefRegistrySimple;
 class PrefService;
 
 namespace aura {
+class Env;
 class RootWindow;
-class UserActivityForwarder;
 class Window;
-class WindowManagerClient;
-class WindowTreeClient;
 }  // namespace aura
 
 namespace display {
@@ -50,11 +47,12 @@ class FileHelper;
 
 namespace gfx {
 class Insets;
-}
+class Point;
+}  // namespace gfx
 
 namespace keyboard {
-class KeyboardController;
-}  // namespace keyboard
+class KeyboardUIFactory;
+}
 
 namespace service_manager {
 class Connector;
@@ -65,9 +63,6 @@ class ContextFactory;
 class ContextFactoryPrivate;
 class UserActivityDetector;
 class UserActivityPowerManagerNotifier;
-namespace ws2 {
-class GpuInterfaceProvider;
-}
 }  // namespace ui
 
 namespace views {
@@ -83,11 +78,16 @@ class AcceleratorFilter;
 class ActivationClient;
 class CompoundEventFilter;
 class FocusController;
+class FocusRules;
 class ShadowController;
 class VisibilityController;
 class WindowModalityController;
 }  // namespace wm
 
+namespace ws {
+class GpuInterfaceProvider;
+}
+
 namespace ash {
 
 class AcceleratorController;
@@ -106,8 +106,10 @@ class BluetoothNotificationController;
 class BluetoothPowerController;
 class BrightnessControlDelegate;
 class CastConfigController;
-class ClientImageRegistry;
+class DisplayOutputProtection;
+class ContainedShellController;
 class CrosDisplayConfig;
+class DesksController;
 class DetachableBaseHandler;
 class DetachableBaseNotificationController;
 class DisplayColorManager;
@@ -128,19 +130,19 @@ class HighContrastController;
 class HighlighterController;
 class ImeController;
 class ImeFocusHandler;
-class ImmersiveContextAsh;
-class ImmersiveHandlerFactoryAsh;
+class ImmersiveContext;
 class KeyAccessibilityEnabler;
 class KeyboardBrightnessControlDelegate;
-class KeyboardUI;
+class AshKeyboardController;
 class LaserPointerController;
-class LocaleNotificationController;
+class LocaleUpdateController;
 class LockStateController;
 class LogoutConfirmationController;
 class LoginScreenController;
 class MagnificationController;
 class TabletModeController;
 class MediaController;
+class MediaNotificationController;
 class MessageCenterController;
 class MouseCursorEventFilter;
 class MruWindowTracker;
@@ -148,7 +150,6 @@ class MultiDeviceNotificationPresenter;
 class NewWindowController;
 class NightLightController;
 class NoteTakingController;
-class NotificationTray;
 class OverlayEventFilter;
 class PartialMagnificationController;
 class PeripheralBatteryNotifier;
@@ -161,7 +162,6 @@ class ProjectingObserver;
 class ResizeShadowController;
 class ResolutionNotificationController;
 class RootWindowController;
-class ShellPort;
 class ScreenLayoutObserver;
 class ScreenOrientationController;
 class ScreenshotController;
@@ -183,8 +183,6 @@ class StickyKeysController;
 class SystemGestureEventFilter;
 class SystemModalContainerEventFilter;
 class SystemNotificationController;
-class SystemTray;
-class SystemTrayController;
 class SystemTrayModel;
 class SystemTrayNotifier;
 class TimeToFirstPresentRecorder;
@@ -193,7 +191,6 @@ class ToastManager;
 class TouchDevicesController;
 class TrayAction;
 class TrayBluetoothHelper;
-class VirtualKeyboardController;
 class VideoActivityNotifier;
 class VideoDetector;
 class VoiceInteractionController;
@@ -203,10 +200,9 @@ class WaylandServerController;
 class WindowServiceOwner;
 class WindowCycleController;
 class WindowPositioner;
-class WindowSelectorController;
+class OverviewController;
 class WindowTreeHostManager;
 
-enum class Config;
 enum class LoginStatus;
 
 // Shell is a singleton object that presents the Shell API and implements the
@@ -275,25 +271,10 @@ class ASH_EXPORT Shell : public SessionObserver,
   // Returns true if a system-modal dialog window is currently open.
   static bool IsSystemModalWindowOpen();
 
-  // TODO(sky): move this and WindowManagerClient into ShellMash that is owned
-  // by Shell. Doing the move is gated on having mash create Shell.
-  static void set_window_tree_client(aura::WindowTreeClient* client) {
-    window_tree_client_ = client;
-  }
-
-  static aura::WindowTreeClient* window_tree_client() {
-    return window_tree_client_;
-  }
-
-  static void set_window_manager_client(aura::WindowManagerClient* client) {
-    window_manager_client_ = client;
-  }
-  static aura::WindowManagerClient* window_manager_client() {
-    return window_manager_client_;
-  }
-
-  static Config GetAshConfig();
-  static bool ShouldUseIMEService();
+  // Returns true if |window| is a proxy window. A proxy window is a window that
+  // was created by way of a WindowService client (e.g. the keyboard shortcut
+  // viewer app under classic ash, or a browser window under mash).
+  static bool IsProxyWindow(aura::Window* window);
 
   // Registers all ash related local state prefs to the given |registry|.
   static void RegisterLocalStatePrefs(PrefRegistrySimple* registry,
@@ -382,17 +363,15 @@ class ASH_EXPORT Shell : public SessionObserver,
     return brightness_control_delegate_.get();
   }
   CastConfigController* cast_config() { return cast_config_.get(); }
-  ClientImageRegistry* client_image_registry() {
-    return client_image_registry_.get();
-  }
   service_manager::Connector* connector() { return connector_; }
+  ContainedShellController* contained_shell_controller() {
+    return contained_shell_controller_.get();
+  }
   CrosDisplayConfig* cros_display_config() {
     return cros_display_config_.get();
   }
-
-  // Returns nullptr in mash which has no global cursor manager.
   ::wm::CursorManager* cursor_manager() { return cursor_manager_.get(); }
-
+  DesksController* desks_controller() { return desks_controller_.get(); }
   DetachableBaseHandler* detachable_base_handler() {
     return detachable_base_handler_.get();
   }
@@ -413,8 +392,12 @@ class ASH_EXPORT Shell : public SessionObserver,
   DisplayErrorObserver* display_error_observer() {
     return display_error_observer_.get();
   }
+  DisplayOutputProtection* display_output_protection() {
+    return display_output_protection_.get();
+  }
 
   DockedMagnifierController* docked_magnifier_controller();
+  aura::Env* aura_env() { return aura_env_; }
   ::wm::CompoundEventFilter* env_filter() { return env_filter_.get(); }
   EventRewriterController* event_rewriter_controller() {
     return event_rewriter_controller_.get();
@@ -425,6 +408,7 @@ class ASH_EXPORT Shell : public SessionObserver,
   }
   FirstRunHelper* first_run_helper() { return first_run_helper_.get(); }
   ::wm::FocusController* focus_controller() { return focus_controller_.get(); }
+  ::wm::FocusRules* focus_rules() { return focus_rules_; }
   FocusCycler* focus_cycler() { return focus_cycler_.get(); }
   HighlighterController* highlighter_controller() {
     return highlighter_controller_.get();
@@ -433,18 +417,21 @@ class ASH_EXPORT Shell : public SessionObserver,
     return high_contrast_controller_.get();
   }
   ImeController* ime_controller() { return ime_controller_.get(); }
+  ImmersiveContext* immersive_context() { return immersive_context_.get(); }
   KeyAccessibilityEnabler* key_accessibility_enabler() {
     return key_accessibility_enabler_.get();
   }
   KeyboardBrightnessControlDelegate* keyboard_brightness_control_delegate() {
     return keyboard_brightness_control_delegate_.get();
   }
-  KeyboardUI* keyboard_ui() { return keyboard_ui_.get(); }
+  AshKeyboardController* ash_keyboard_controller() {
+    return ash_keyboard_controller_.get();
+  }
   LaserPointerController* laser_pointer_controller() {
     return laser_pointer_controller_.get();
   }
-  LocaleNotificationController* locale_notification_controller() {
-    return locale_notification_controller_.get();
+  LocaleUpdateController* locale_update_controller() {
+    return locale_update_controller_.get();
   }
   LoginScreenController* login_screen_controller() {
     return login_screen_controller_.get();
@@ -459,6 +446,9 @@ class ASH_EXPORT Shell : public SessionObserver,
     return magnification_controller_.get();
   }
   MediaController* media_controller() { return media_controller_.get(); }
+  MediaNotificationController* media_notification_controller() {
+    return media_notification_controller_.get();
+  }
   MessageCenterController* message_center_controller() {
     return message_center_controller_.get();
   }
@@ -527,9 +517,6 @@ class ASH_EXPORT Shell : public SessionObserver,
   SystemNotificationController* system_notification_controller() {
     return system_notification_controller_.get();
   }
-  SystemTrayController* system_tray_controller() {
-    return system_tray_controller_.get();
-  }
   SystemTrayModel* system_tray_model() { return system_tray_model_.get(); }
   SystemTrayNotifier* system_tray_notifier() {
     return system_tray_notifier_.get();
@@ -556,9 +543,6 @@ class ASH_EXPORT Shell : public SessionObserver,
   }
   UserMetricsRecorder* metrics() { return user_metrics_recorder_.get(); }
   VideoDetector* video_detector() { return video_detector_.get(); }
-  VirtualKeyboardController* virtual_keyboard_controller() {
-    return virtual_keyboard_controller_.get();
-  }
   VoiceInteractionController* voice_interaction_controller() {
     return voice_interaction_controller_.get();
   }
@@ -570,10 +554,9 @@ class ASH_EXPORT Shell : public SessionObserver,
     return window_cycle_controller_.get();
   }
   WindowPositioner* window_positioner() { return window_positioner_.get(); }
-  WindowSelectorController* window_selector_controller() {
-    return window_selector_controller_.get();
+  OverviewController* overview_controller() {
+    return overview_controller_.get();
   }
-  // WindowServiceOwner is null in mash.
   WindowServiceOwner* window_service_owner() {
     return window_service_owner_.get();
   }
@@ -589,15 +572,9 @@ class ASH_EXPORT Shell : public SessionObserver,
   // TODO(jamescook): Move to Shelf.
   void UpdateShelfVisibility();
 
-  // Returns NotificationTray on the primary root window.
-  NotificationTray* GetNotificationTray();
-
   // Does the primary display have status area?
   bool HasPrimaryStatusArea();
 
-  // Returns the system tray on primary display.
-  SystemTray* GetPrimarySystemTray();
-
   // Starts the animation that occurs on first login.
   void DoInitialWorkspaceAnimation();
 
@@ -613,6 +590,10 @@ class ASH_EXPORT Shell : public SessionObserver,
   // Returns true if split view mode is active.
   bool IsSplitViewModeActive() const;
 
+  // Shows the context menu for the wallpaper or shelf at |location_in_screen|.
+  void ShowContextMenu(const gfx::Point& location_in_screen,
+                       ui::MenuSourceType source_type);
+
   void AddShellObserver(ShellObserver* observer);
   void RemoveShellObserver(ShellObserver* observer);
 
@@ -620,17 +601,6 @@ class ASH_EXPORT Shell : public SessionObserver,
   // TODO(oshima): Investigate if we can merge this and |OnLoginStateChanged|.
   void UpdateAfterLoginStatusChange(LoginStatus status);
 
-  // Notifies observers that overview mode is about to be started (before the
-  // windows get re-arranged).
-  void NotifyOverviewModeStarting();
-
-  // Notifies observers that overview mode is about to end (bofore the windows
-  // restore themselves).
-  void NotifyOverviewModeEnding();
-
-  // Notifies observers that overview mode has ended.
-  void NotifyOverviewModeEnded();
-
   // Notifies observers that split view mode is about to be started (before the
   // window gets snapped and activated).
   void NotifySplitViewModeStarting();
@@ -648,10 +618,6 @@ class ASH_EXPORT Shell : public SessionObserver,
   // Notifies observers that |pinned_window| changed its pinned window state.
   void NotifyPinnedStateChanged(aura::Window* pinned_window);
 
-  // Notifies observers that the shelf was created for |root_window|.
-  // TODO(jamescook): Move to Shelf.
-  void NotifyShelfCreatedForRootWindow(aura::Window* root_window);
-
   // Notifies observers that |root_window|'s shelf changed alignment.
   // TODO(jamescook): Move to Shelf.
   void NotifyShelfAlignmentChanged(aura::Window* root_window);
@@ -663,8 +629,6 @@ class ASH_EXPORT Shell : public SessionObserver,
   // Used to provide better error messages for Shell::Get() under mash.
   static void SetIsBrowserProcessWithMash();
 
-  void NotifyAppListVisibilityChanged(bool visible, aura::Window* root_window);
-
  private:
   FRIEND_TEST_ALL_PREFIXES(ExtendedDesktopTest, TestCursor);
   FRIEND_TEST_ALL_PREFIXES(WindowManagerTest, MouseEventCursors);
@@ -677,15 +641,14 @@ class ASH_EXPORT Shell : public SessionObserver,
   friend class SmsObserverTest;
 
   Shell(std::unique_ptr<ShellDelegate> shell_delegate,
-        std::unique_ptr<ShellPort> shell_port);
+        service_manager::Connector* connector);
   ~Shell() override;
 
-  void Init(
-      ui::ContextFactory* context_factory,
-      ui::ContextFactoryPrivate* context_factory_private,
-      std::unique_ptr<base::Value> initial_display_prefs,
-      std::unique_ptr<ui::ws2::GpuInterfaceProvider> gpu_interface_provider,
-      service_manager::Connector* connector);
+  void Init(ui::ContextFactory* context_factory,
+            ui::ContextFactoryPrivate* context_factory_private,
+            std::unique_ptr<base::Value> initial_display_prefs,
+            std::unique_ptr<ws::GpuInterfaceProvider> gpu_interface_provider,
+            std::unique_ptr<keyboard::KeyboardUIFactory> keyboard_ui_factory);
 
   // Initializes the display manager and related components.
   void InitializeDisplayManager();
@@ -722,11 +685,12 @@ class ASH_EXPORT Shell : public SessionObserver,
 
   static Shell* instance_;
 
-  // Only valid in mash, for classic ash this is null.
-  static aura::WindowTreeClient* window_tree_client_;
-  static aura::WindowManagerClient* window_manager_client_;
+  // |owned_aura_env_| is non-null if Shell created aura::Env. Shell creates
+  // aura::Env only in single-process-mash mode.
+  std::unique_ptr<aura::Env> owned_aura_env_;
 
-  std::unique_ptr<ShellPort> shell_port_;
+  // This is either |owned_aura_env_|, or Env::GetInstance().
+  aura::Env* aura_env_;
 
   // The CompoundEventFilter owned by aura::Env object.
   std::unique_ptr<::wm::CompoundEventFilter> env_filter_;
@@ -748,9 +712,10 @@ class ASH_EXPORT Shell : public SessionObserver,
   std::unique_ptr<BacklightsForcedOffSetter> backlights_forced_off_setter_;
   std::unique_ptr<BrightnessControlDelegate> brightness_control_delegate_;
   std::unique_ptr<CastConfigController> cast_config_;
-  std::unique_ptr<ClientImageRegistry> client_image_registry_;
   std::unique_ptr<CrosDisplayConfig> cros_display_config_;
-  service_manager::Connector* connector_ = nullptr;
+  service_manager::Connector* const connector_;
+  std::unique_ptr<ContainedShellController> contained_shell_controller_;
+  std::unique_ptr<DesksController> desks_controller_;
   std::unique_ptr<DetachableBaseHandler> detachable_base_handler_;
   std::unique_ptr<DetachableBaseNotificationController>
       detachable_base_notification_controller_;
@@ -760,16 +725,15 @@ class ASH_EXPORT Shell : public SessionObserver,
   std::unique_ptr<FocusCycler> focus_cycler_;
   std::unique_ptr<ImeController> ime_controller_;
   std::unique_ptr<ImeFocusHandler> ime_focus_handler_;
-  std::unique_ptr<ImmersiveContextAsh> immersive_context_;
+  std::unique_ptr<ImmersiveContext> immersive_context_;
   std::unique_ptr<KeyboardBrightnessControlDelegate>
       keyboard_brightness_control_delegate_;
-  std::unique_ptr<KeyboardUI> keyboard_ui_;
-  std::unique_ptr<keyboard::KeyboardController> keyboard_controller_;
-  std::unique_ptr<LocaleNotificationController> locale_notification_controller_;
+  std::unique_ptr<LocaleUpdateController> locale_update_controller_;
   std::unique_ptr<LoginScreenController> login_screen_controller_;
   std::unique_ptr<LogoutConfirmationController> logout_confirmation_controller_;
   std::unique_ptr<TabletModeController> tablet_mode_controller_;
   std::unique_ptr<MediaController> media_controller_;
+  std::unique_ptr<MediaNotificationController> media_notification_controller_;
   std::unique_ptr<MruWindowTracker> mru_window_tracker_;
   std::unique_ptr<MultiDeviceNotificationPresenter>
       multidevice_notification_presenter_;
@@ -786,7 +750,6 @@ class ASH_EXPORT Shell : public SessionObserver,
   std::unique_ptr<ShellState> shell_state_;
   std::unique_ptr<ShutdownController> shutdown_controller_;
   std::unique_ptr<SystemNotificationController> system_notification_controller_;
-  std::unique_ptr<SystemTrayController> system_tray_controller_;
   std::unique_ptr<SystemTrayModel> system_tray_model_;
   std::unique_ptr<SystemTrayNotifier> system_tray_notifier_;
   std::unique_ptr<ToastManager> toast_manager_;
@@ -797,7 +760,9 @@ class ASH_EXPORT Shell : public SessionObserver,
   std::unique_ptr<VpnList> vpn_list_;
   std::unique_ptr<WallpaperController> wallpaper_controller_;
   std::unique_ptr<WindowCycleController> window_cycle_controller_;
-  std::unique_ptr<WindowSelectorController> window_selector_controller_;
+  std::unique_ptr<OverviewController> overview_controller_;
+  // Owned by |focus_controller_|.
+  ::wm::FocusRules* focus_rules_ = nullptr;
   std::unique_ptr<::wm::ShadowController> shadow_controller_;
   std::unique_ptr<::wm::VisibilityController> visibility_controller_;
   std::unique_ptr<::wm::WindowModalityController> window_modality_controller_;
@@ -845,10 +810,6 @@ class ASH_EXPORT Shell : public SessionObserver,
 
   std::unique_ptr<ScreenPinningController> screen_pinning_controller_;
 
-  // Forwards user activity ui::mojom::UserActivityMonitor to
-  // |user_activity_detector_|. Only initialized for mash.
-  std::unique_ptr<aura::UserActivityForwarder> user_activity_forwarder_;
-
   std::unique_ptr<PeripheralBatteryNotifier> peripheral_battery_notifier_;
   std::unique_ptr<PowerEventObserver> power_event_observer_;
   std::unique_ptr<PowerPrefs> power_prefs_;
@@ -861,9 +822,10 @@ class ASH_EXPORT Shell : public SessionObserver,
       bluetooth_notification_controller_;
   std::unique_ptr<BluetoothPowerController> bluetooth_power_controller_;
   std::unique_ptr<TrayBluetoothHelper> tray_bluetooth_helper_;
-  std::unique_ptr<VirtualKeyboardController> virtual_keyboard_controller_;
+  std::unique_ptr<AshKeyboardController> ash_keyboard_controller_;
   // Controls video output device state.
   std::unique_ptr<display::DisplayConfigurator> display_configurator_;
+  std::unique_ptr<DisplayOutputProtection> display_output_protection_;
   std::unique_ptr<DisplayColorManager> display_color_manager_;
   std::unique_ptr<DisplayErrorObserver> display_error_observer_;
   std::unique_ptr<ProjectingObserver> projecting_observer_;
@@ -910,11 +872,9 @@ class ASH_EXPORT Shell : public SessionObserver,
   // For testing only: simulate that a modal window is open
   bool simulate_modal_window_open_for_test_ = false;
 
-  std::unique_ptr<ImmersiveHandlerFactoryAsh> immersive_handler_factory_;
-
   std::unique_ptr<MessageCenterController> message_center_controller_;
 
-  base::ObserverList<ShellObserver> shell_observers_;
+  base::ObserverList<ShellObserver>::Unchecked shell_observers_;
 
   base::WeakPtrFactory<Shell> weak_factory_;
 
