@@ -10,6 +10,7 @@
 #include "ash/test/ash_test_base.h"
 #include "ash/test/ash_test_helper.h"
 #include "ash/test_shell_delegate.h"
+#include "base/bind_helpers.h"
 #include "base/run_loop.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/testing_pref_service.h"
@@ -116,10 +117,10 @@ TEST_F(BluetoothPowerControllerTest, ToggleBluetoothEnabled) {
   // local state prefs.
   EXPECT_FALSE(
       local_state_prefs_.GetBoolean(prefs::kSystemBluetoothAdapterEnabled));
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(true);
   EXPECT_TRUE(
       local_state_prefs_.GetBoolean(prefs::kSystemBluetoothAdapterEnabled));
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(false);
   EXPECT_FALSE(
       local_state_prefs_.GetBoolean(prefs::kSystemBluetoothAdapterEnabled));
 
@@ -128,10 +129,10 @@ TEST_F(BluetoothPowerControllerTest, ToggleBluetoothEnabled) {
   AddUserSessionAndStartWatchingPrefsChanges(kUser1Email);
   EXPECT_FALSE(
       active_user_prefs_.GetBoolean(prefs::kUserBluetoothAdapterEnabled));
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(true);
   EXPECT_TRUE(
       active_user_prefs_.GetBoolean(prefs::kUserBluetoothAdapterEnabled));
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(false);
   EXPECT_FALSE(
       active_user_prefs_.GetBoolean(prefs::kUserBluetoothAdapterEnabled));
 }
@@ -356,11 +357,11 @@ TEST_F(BluetoothPowerControllerIntegrationTest, Basics) {
   device::BluetoothAdapter* adapter = GetBluetoothAdapter();
 
   // Verify toggling bluetooth before login.
-  PrefService* local_state = Shell::Get()->GetLocalStatePrefService();
-  GetController()->ToggleBluetoothEnabled();
+  PrefService* local_state = ash_test_helper()->GetLocalStatePrefService();
+  GetController()->SetBluetoothEnabled(true);
   EXPECT_TRUE(local_state->GetBoolean(prefs::kSystemBluetoothAdapterEnabled));
   EXPECT_TRUE(adapter->IsPowered());
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(false);
   EXPECT_FALSE(local_state->GetBoolean(prefs::kSystemBluetoothAdapterEnabled));
   EXPECT_FALSE(adapter->IsPowered());
 
@@ -368,10 +369,10 @@ TEST_F(BluetoothPowerControllerIntegrationTest, Basics) {
   SimulateUserLogin(kUser1Email);
   PrefService* user_prefs =
       Shell::Get()->session_controller()->GetLastActiveUserPrefService();
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(true);
   EXPECT_TRUE(user_prefs->GetBoolean(prefs::kUserBluetoothAdapterEnabled));
   EXPECT_TRUE(adapter->IsPowered());
-  GetController()->ToggleBluetoothEnabled();
+  GetController()->SetBluetoothEnabled(false);
   EXPECT_FALSE(user_prefs->GetBoolean(prefs::kUserBluetoothAdapterEnabled));
   EXPECT_FALSE(adapter->IsPowered());
 }
