@@ -6,18 +6,15 @@
 
 #include <memory>
 
-#include "ash/components/quick_launch/public/mojom/constants.mojom.h"
+#include "ash/components/shortcut_viewer/public/mojom/shortcut_viewer.mojom.h"
 #include "ash/public/cpp/ash_typography.h"
-#include "ash/public/cpp/config.h"
 #include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/shelf/shelf.h"
 #include "ash/shell.h"
-#include "ash/shell_delegate.h"
 #include "ash/system/tray/tray_constants.h"
 #include "ash/system/tray/tray_container.h"
 #include "base/logging.h"
 #include "base/strings/utf_string_conversions.h"
-#include "mash/public/mojom/launchable.mojom.h"
 #include "services/service_manager/public/cpp/connector.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/base/ui_base_features.h"
@@ -28,20 +25,11 @@
 #include "ui/views/layout/fill_layout.h"
 
 namespace ash {
-namespace {
-
-const char kTooltipText[] = "Running with feature mash";
-
-}  // namespace
 
 FlagWarningTray::FlagWarningTray(Shelf* shelf) : shelf_(shelf) {
   DCHECK(shelf_);
   SetLayoutManager(std::make_unique<views::FillLayout>());
 
-  // Flag warning tray is not currently used in non-MASH environments, because
-  // mus will roll out via experiment/Finch trial and showing the tray would
-  // reveal the experiment state to users.
-  DCHECK(!::features::IsAshInBrowserProcess());
   container_ = new TrayContainer(shelf);
   AddChildView(container_);
 
@@ -50,7 +38,7 @@ FlagWarningTray::FlagWarningTray(Shelf* shelf) : shelf_(shelf) {
   button_->SetProminent(true);
   button_->SetBgColorOverride(gfx::kGoogleYellow300);
   button_->SetEnabledTextColors(SK_ColorBLACK);
-  button_->SetTooltipText(base::ASCIIToUTF16(kTooltipText));
+  button_->SetFocusBehavior(views::View::FocusBehavior::NEVER);
   UpdateButton();
   container_->AddChildView(button_);
   SetVisible(true);
@@ -70,11 +58,11 @@ void FlagWarningTray::ButtonPressed(views::Button* sender,
                                     const ui::Event& event) {
   DCHECK_EQ(button_, sender);
 
-  // Open the quick launch mojo mini-app to demonstrate that mini-apps work.
-  mash::mojom::LaunchablePtr launchable;
-  Shell::Get()->shell_delegate()->GetShellConnector()->BindInterface(
-      quick_launch::mojom::kServiceName, &launchable);
-  launchable->Launch(mash::mojom::kWindow, mash::mojom::LaunchMode::DEFAULT);
+  // Open the shortcut viewer mini-app to demonstrate that mini-apps work.
+  shortcut_viewer::mojom::ShortcutViewerPtr shortcut_viewer_ptr;
+  Shell::Get()->connector()->BindInterface(shortcut_viewer::mojom::kServiceName,
+                                           &shortcut_viewer_ptr);
+  shortcut_viewer_ptr->Toggle(base::TimeTicks::Now());
 }
 
 void FlagWarningTray::GetAccessibleNodeData(ui::AXNodeData* node_data) {
@@ -88,17 +76,23 @@ void FlagWarningTray::UpdateButton() {
 
   // Only the horizontal shelf is wide enough for a text label. Use an icon for
   // vertical shelf alignments.
-  if (shelf_->IsHorizontalAlignment()) {
-    button_->SetText(base::ASCIIToUTF16("mash"));
-    button_->SetImage(views::Button::STATE_NORMAL, gfx::ImageSkia());
-    button_->SetMinSize(gfx::Size(0, kTrayItemSize));
-  } else {
+  if (!shelf_->IsHorizontalAlignment()) {
     button_->SetText(base::string16());
     button_->SetImage(
         views::Button::STATE_NORMAL,
         gfx::CreateVectorIcon(kSystemMenuInfoIcon, SK_ColorBLACK));
     button_->SetMinSize(gfx::Size(kTrayItemSize, kTrayItemSize));
+    return;
   }
+
+  if (::features::IsMultiProcessMash()) {
+    button_->SetText(base::ASCIIToUTF16("Mash"));
+    button_->SetTooltipText(base::ASCIIToUTF16("Running with feature Mash"));
+  } else {
+    NOTREACHED();
+  }
+  button_->SetImage(views::Button::STATE_NORMAL, gfx::ImageSkia());
+  button_->SetMinSize(gfx::Size(0, kTrayItemSize));
 }
 
 }  // namespace ash
