@@ -6,17 +6,19 @@
 
 #include "ash/accessibility/accessibility_controller.h"
 #include "ash/ime/ime_controller.h"
+#include "ash/keyboard/ash_keyboard_controller.h"
 #include "ash/keyboard/virtual_keyboard_controller.h"
 #include "ash/public/cpp/ash_constants.h"
+#include "ash/resources/vector_icons/vector_icons.h"
 #include "ash/root_window_controller.h"
 #include "ash/session/session_controller.h"
 #include "ash/shelf/shelf.h"
 #include "ash/shell.h"
 #include "ash/strings/grit/ash_strings.h"
 #include "ash/system/ime_menu/ime_list_view.h"
+#include "ash/system/model/system_tray_model.h"
 #include "ash/system/tray/detailed_view_delegate.h"
 #include "ash/system/tray/system_menu_button.h"
-#include "ash/system/tray/system_tray_controller.h"
 #include "ash/system/tray/system_tray_notifier.h"
 #include "ash/system/tray/tray_constants.h"
 #include "ash/system/tray/tray_container.h"
@@ -27,16 +29,18 @@
 #include "base/metrics/user_metrics.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/session_manager/session_manager_types.h"
+#include "ui/base/ime/chromeos/extension_ime_util.h"
 #include "ui/base/ime/ime_bridge.h"
 #include "ui/base/ime/text_input_client.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/base/ui_base_features.h"
 #include "ui/gfx/geometry/insets.h"
 #include "ui/gfx/paint_vector_icon.h"
 #include "ui/gfx/range/range.h"
 #include "ui/keyboard/keyboard_controller.h"
-#include "ui/keyboard/keyboard_util.h"
 #include "ui/views/controls/button/button.h"
+#include "ui/views/controls/image_view.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/controls/scroll_view.h"
 #include "ui/views/controls/separator.h"
@@ -60,7 +64,7 @@ gfx::Range GetImeListViewRange() {
 // Shows language and input settings page.
 void ShowIMESettings() {
   base::RecordAction(base::UserMetricsAction("StatusArea_IME_Detailed"));
-  Shell::Get()->system_tray_controller()->ShowIMESettings();
+  Shell::Get()->system_tray_model()->client_ptr()->ShowIMESettings();
 }
 
 // Returns true if the current screen is login or lock screen.
@@ -74,8 +78,12 @@ bool IsInLoginOrLockScreen() {
 
 // Returns true if the current input context type is password.
 bool IsInPasswordInputContext() {
-  return ui::IMEBridge::Get()->GetCurrentInputContext().type ==
-         ui::TEXT_INPUT_TYPE_PASSWORD;
+  // Avoid getting IMEBridge instance if ash is not in browser.
+  // This is to temporarily mute the crash (http://crbug.com/867084).
+  // TODO(shuchen): This will be eventually be solved by the Mojo-based IMF.
+  return !::features::IsMultiProcessMash() &&
+         ui::IMEBridge::Get()->GetCurrentInputContext().type ==
+             ui::TEXT_INPUT_TYPE_PASSWORD;
 }
 
 class ImeMenuLabel : public views::Label {
@@ -98,6 +106,15 @@ class ImeMenuLabel : public views::Label {
   DISALLOW_COPY_AND_ASSIGN(ImeMenuLabel);
 };
 
+class ImeMenuImageView : public views::ImageView {
+ public:
+  ImeMenuImageView() { SetBorder(views::CreateEmptyBorder(gfx::Insets(0, 6))); }
+  ~ImeMenuImageView() override = default;
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(ImeMenuImageView);
+};
+
 SystemMenuButton* CreateImeMenuButton(views::ButtonListener* listener,
                                       const gfx::VectorIcon& icon,
                                       int accessible_name_id,
@@ -110,9 +127,9 @@ class ImeTitleView : public views::View, public views::ButtonListener {
  public:
   explicit ImeTitleView(bool show_settings_button) : settings_button_(nullptr) {
     SetBorder(views::CreatePaddedBorder(
-        views::CreateSolidSidedBorder(0, 0, kSeparatorWidth, 0,
+        views::CreateSolidSidedBorder(0, 0, kMenuSeparatorWidth, 0,
                                       kMenuSeparatorColor),
-        gfx::Insets(kMenuSeparatorVerticalPadding - kSeparatorWidth, 0)));
+        gfx::Insets(kMenuSeparatorVerticalPadding - kMenuSeparatorWidth, 0)));
     auto box_layout =
         std::make_unique<views::BoxLayout>(views::BoxLayout::kHorizontal);
     box_layout->set_minimum_cross_axis_size(kTrayPopupItemMinHeight);
@@ -207,9 +224,9 @@ class ImeButtonsView : public views::View, public views::ButtonListener {
     box_layout->set_minimum_cross_axis_size(kTrayPopupItemMinHeight);
     SetLayoutManager(std::move(box_layout));
     SetBorder(views::CreatePaddedBorder(
-        views::CreateSolidSidedBorder(kSeparatorWidth, 0, 0, 0,
+        views::CreateSolidSidedBorder(kMenuSeparatorWidth, 0, 0, 0,
                                       kMenuSeparatorColor),
-        gfx::Insets(kMenuSeparatorVerticalPadding - kSeparatorWidth,
+        gfx::Insets(kMenuSeparatorVerticalPadding - kMenuSeparatorWidth,
                     kMenuExtraMarginFromLeftEdge)));
 
     const int right_border = 1;
@@ -253,55 +270,44 @@ class ImeButtonsView : public views::View, public views::ButtonListener {
 
 // A list of available IMEs shown in the opt-in IME menu, which has a different
 // height depending on the number of IMEs in the list.
-class ImeMenuListView : public ImeListView, public DetailedViewDelegate {
+class ImeMenuListView : public ImeListView {
  public:
-  ImeMenuListView() : ImeListView(this, false /* use_unified_theme */) {
-    set_should_focus_ime_after_selection_with_keyboard(true);
-  }
-
+  ImeMenuListView() : ImeMenuListView(std::make_unique<Delegate>()) {}
   ~ImeMenuListView() override = default;
 
-  // DetailedViewDelegate:
-  void TransitionToMainView(bool restore_focus) override {}
-  void CloseBubble() override {}
-  SkColor GetBackgroundColor(ui::NativeTheme* native_theme) override {
-    return native_theme->GetSystemColor(
-        ui::NativeTheme::kColorId_BubbleBackground);
-  }
-  bool IsOverflowIndicatorEnabled() const override { return true; }
-  TriView* CreateTitleRow(int string_id) override { return nullptr; }
-  views::View* CreateTitleSeparator() override { return nullptr; }
-  void ShowStickyHeaderSeparator(views::View* view,
-                                 bool show_separator) override {}
-  views::Separator* CreateListSubHeaderSeparator() override { return nullptr; }
-  HoverHighlightView* CreateScrollListItem(
-      ViewClickListener* listener,
-      const gfx::VectorIcon& icon,
-      const base::string16& text) override {
-    return nullptr;
-  }
-  views::Button* CreateBackButton(views::ButtonListener* listener) override {
-    return nullptr;
-  }
-  views::Button* CreateInfoButton(views::ButtonListener* listener,
-                                  int info_accessible_name_id) override {
-    return nullptr;
-  }
-  views::Button* CreateSettingsButton(views::ButtonListener* listener,
-                                      int setting_accessible_name_id) override {
-    return nullptr;
-  }
-  views::Button* CreateHelpButton(views::ButtonListener* listener) override {
-    return nullptr;
+ private:
+  class Delegate : public DetailedViewDelegate {
+   public:
+    Delegate() : DetailedViewDelegate(nullptr /* tray_controller */) {}
+
+    // DetailedViewDelegate:
+    void TransitionToMainView(bool restore_focus) override {}
+    void CloseBubble() override {}
+    SkColor GetBackgroundColor(ui::NativeTheme* native_theme) override {
+      return native_theme->GetSystemColor(
+          ui::NativeTheme::kColorId_BubbleBackground);
+    }
+    bool IsOverflowIndicatorEnabled() const override { return true; }
+
+   private:
+    DISALLOW_COPY_AND_ASSIGN(Delegate);
+  };
+
+  ImeMenuListView(std::unique_ptr<Delegate> delegate)
+      : ImeListView(delegate.get(), false /* use_unified_theme */) {
+    set_should_focus_ime_after_selection_with_keyboard(true);
+    delegate_ = std::move(delegate);
   }
 
- protected:
+  // ImeListView:
   void Layout() override {
     gfx::Range height_range = GetImeListViewRange();
     scroller()->ClipHeightTo(height_range.start(), height_range.end());
     ImeListView::Layout();
   }
 
+  std::unique_ptr<Delegate> delegate_;
+
   DISALLOW_COPY_AND_ASSIGN(ImeMenuListView);
 };
 
@@ -310,7 +316,8 @@ class ImeMenuListView : public ImeListView, public DetailedViewDelegate {
 ImeMenuTray::ImeMenuTray(Shelf* shelf)
     : TrayBackgroundView(shelf),
       ime_controller_(Shell::Get()->ime_controller()),
-      label_(new ImeMenuLabel()),
+      label_(nullptr),
+      image_view_(nullptr),
       keyboard_suppressed_(false),
       show_bubble_after_keyboard_hidden_(false),
       is_emoji_enabled_(false),
@@ -319,13 +326,14 @@ ImeMenuTray::ImeMenuTray(Shelf* shelf)
       weak_ptr_factory_(this) {
   DCHECK(ime_controller_);
   SetInkDropMode(InkDropMode::ON);
-  SetupLabelForTray(label_);
-  label_->SetElideBehavior(gfx::TRUNCATE);
-  label_->SetTooltipText(l10n_util::GetStringUTF16(IDS_ASH_STATUS_TRAY_IME));
-  tray_container()->AddChildView(label_);
+  CreateLabel();
   SystemTrayNotifier* tray_notifier = Shell::Get()->system_tray_notifier();
   tray_notifier->AddIMEObserver(this);
   tray_notifier->AddVirtualKeyboardObserver(this);
+
+  // Show the tray even if virtual keyboard is shown. (Other tray buttons will
+  // be hidden).
+  set_show_with_virtual_keyboard(true);
 }
 
 ImeMenuTray::~ImeMenuTray() {
@@ -335,12 +343,12 @@ ImeMenuTray::~ImeMenuTray() {
   tray_notifier->RemoveIMEObserver(this);
   tray_notifier->RemoveVirtualKeyboardObserver(this);
   auto* keyboard_controller = keyboard::KeyboardController::Get();
-  if (keyboard_controller->enabled())
+  if (keyboard_controller->HasObserver(this))
     keyboard_controller->RemoveObserver(this);
 }
 
 void ImeMenuTray::ShowImeMenuBubbleInternal(bool show_by_click) {
-  views::TrayBubbleView::InitParams init_params;
+  TrayBubbleView::InitParams init_params;
   init_params.delegate = this;
   init_params.parent_window = GetBubbleWindowContainer();
   init_params.anchor_view = GetBubbleAnchor();
@@ -350,7 +358,7 @@ void ImeMenuTray::ShowImeMenuBubbleInternal(bool show_by_click) {
   init_params.close_on_deactivate = true;
   init_params.show_by_click = show_by_click;
 
-  views::TrayBubbleView* bubble_view = new views::TrayBubbleView(init_params);
+  TrayBubbleView* bubble_view = new TrayBubbleView(init_params);
   bubble_view->set_anchor_view_insets(GetBubbleAnchorInsets());
 
   // Add a title item with a separator on the top of the IME menu.
@@ -377,8 +385,10 @@ void ImeMenuTray::ShowKeyboardWithKeyset(
     chromeos::input_method::mojom::ImeKeyset keyset) {
   CloseBubble();
 
-  Shell::Get()->virtual_keyboard_controller()->ForceShowKeyboardWithKeyset(
-      keyset);
+  Shell::Get()
+      ->ash_keyboard_controller()
+      ->virtual_keyboard_controller()
+      ->ForceShowKeyboardWithKeyset(keyset);
 }
 
 bool ImeMenuTray::ShouldShowBottomButtons() {
@@ -407,14 +417,14 @@ bool ImeMenuTray::ShouldShowBottomButtons() {
 
 bool ImeMenuTray::ShouldShowKeyboardToggle() const {
   return keyboard_suppressed_ &&
-         !Shell::Get()->accessibility_controller()->IsVirtualKeyboardEnabled();
+         !Shell::Get()->accessibility_controller()->virtual_keyboard_enabled();
 }
 
 base::string16 ImeMenuTray::GetAccessibleNameForTray() {
   return l10n_util::GetStringUTF16(IDS_ASH_IME_MENU_ACCESSIBLE_NAME);
 }
 
-void ImeMenuTray::HideBubbleWithView(const views::TrayBubbleView* bubble_view) {
+void ImeMenuTray::HideBubbleWithView(const TrayBubbleView* bubble_view) {
   if (bubble_->bubble_view() == bubble_view)
     CloseBubble();
 }
@@ -442,8 +452,7 @@ void ImeMenuTray::CloseBubble() {
 
 void ImeMenuTray::ShowBubble(bool show_by_click) {
   auto* keyboard_controller = keyboard::KeyboardController::Get();
-  if (keyboard_controller->enabled() &&
-      keyboard_controller->keyboard_visible()) {
+  if (keyboard_controller->IsKeyboardVisible()) {
     show_bubble_after_keyboard_hidden_ = true;
     keyboard_controller->AddObserver(this);
     keyboard_controller->HideKeyboardExplicitlyBySystem();
@@ -453,7 +462,7 @@ void ImeMenuTray::ShowBubble(bool show_by_click) {
   }
 }
 
-views::TrayBubbleView* ImeMenuTray::GetBubbleView() {
+TrayBubbleView* ImeMenuTray::GetBubbleView() {
   return bubble_ ? bubble_->bubble_view() : nullptr;
 }
 
@@ -475,26 +484,19 @@ void ImeMenuTray::OnIMEMenuActivationChanged(bool is_activated) {
     CloseBubble();
 }
 
-void ImeMenuTray::BubbleViewDestroyed() {
-}
-
-void ImeMenuTray::OnMouseEnteredView() {}
-
-void ImeMenuTray::OnMouseExitedView() {}
-
 base::string16 ImeMenuTray::GetAccessibleNameForBubble() {
   return l10n_util::GetStringUTF16(IDS_ASH_IME_MENU_ACCESSIBLE_NAME);
 }
 
 bool ImeMenuTray::ShouldEnableExtraKeyboardAccessibility() {
-  return Shell::Get()->accessibility_controller()->IsSpokenFeedbackEnabled();
+  return Shell::Get()->accessibility_controller()->spoken_feedback_enabled();
 }
 
-void ImeMenuTray::HideBubble(const views::TrayBubbleView* bubble_view) {
+void ImeMenuTray::HideBubble(const TrayBubbleView* bubble_view) {
   HideBubbleWithView(bubble_view);
 }
 
-void ImeMenuTray::OnKeyboardHidden() {
+void ImeMenuTray::OnKeyboardHidden(bool is_temporary_hide) {
   if (show_bubble_after_keyboard_hidden_) {
     show_bubble_after_keyboard_hidden_ = false;
     auto* keyboard_controller = keyboard::KeyboardController::Get();
@@ -514,11 +516,52 @@ void ImeMenuTray::OnKeyboardSuppressionChanged(bool suppressed) {
 void ImeMenuTray::UpdateTrayLabel() {
   const mojom::ImeInfo& current_ime = ime_controller_->current_ime();
 
+  // For ARC IMEs, we use the globe icon instead of the short name of the active
+  // IME.
+  if (chromeos::extension_ime_util::IsArcIME(current_ime.id)) {
+    CreateImageView();
+    image_view_->SetImage(
+        gfx::CreateVectorIcon(kShelfGlobeIcon, kTrayIconSize, kTrayIconColor));
+    return;
+  }
+
   // Updates the tray label based on the current input method.
+  CreateLabel();
   if (current_ime.third_party)
     label_->SetText(current_ime.short_name + base::UTF8ToUTF16("*"));
   else
     label_->SetText(current_ime.short_name);
 }
 
+void ImeMenuTray::CreateLabel() {
+  // Do nothing if label_ is already created.
+  if (label_)
+    return;
+  // Remove image_view_ at first if it's created.
+  if (image_view_) {
+    tray_container()->RemoveChildView(image_view_);
+    image_view_ = nullptr;
+  }
+  label_ = new ImeMenuLabel();
+  SetupLabelForTray(label_);
+  label_->SetElideBehavior(gfx::TRUNCATE);
+  label_->SetTooltipText(l10n_util::GetStringUTF16(IDS_ASH_STATUS_TRAY_IME));
+  tray_container()->AddChildView(label_);
+}
+
+void ImeMenuTray::CreateImageView() {
+  // Do nothing if image_view_ is already created.
+  if (image_view_)
+    return;
+  // Remove label_ at first if it's created.
+  if (label_) {
+    tray_container()->RemoveChildView(label_);
+    label_ = nullptr;
+  }
+  image_view_ = new ImeMenuImageView();
+  image_view_->set_tooltip_text(
+      l10n_util::GetStringUTF16(IDS_ASH_STATUS_TRAY_IME));
+  tray_container()->AddChildView(image_view_);
+}
+
 }  // namespace ash
