@@ -14,12 +14,10 @@
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
 #include "base/strings/utf_string_conversions.h"
-#include "chromeos/network/device_state.h"
-#include "chromeos/network/network_connection_handler.h"
 #include "chromeos/network/network_state.h"
-#include "chromeos/network/network_state_handler.h"
-#include "chromeos/network/portal_detector/network_portal_detector.h"
+#include "chromeos/network/network_type_pattern.h"
 #include "chromeos/network/tether_constants.h"
+#include "components/vector_icons/vector_icons.h"
 #include "third_party/cros_system_api/dbus/service_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/gfx/image/image_skia_operations.h"
@@ -28,12 +26,7 @@
 #include "ui/gfx/skia_util.h"
 #include "ui/gfx/vector_icon_types.h"
 
-using chromeos::DeviceState;
-using chromeos::NetworkConnectionHandler;
-using chromeos::NetworkHandler;
-using chromeos::NetworkPortalDetector;
 using chromeos::NetworkState;
-using chromeos::NetworkStateHandler;
 using chromeos::NetworkTypePattern;
 
 namespace ash {
@@ -41,16 +34,6 @@ namespace network_icon {
 
 namespace {
 
-// Constants for offseting the badge displayed on top of the signal strength
-// icon. The badge will extend outside of the base icon bounds by these amounts.
-// All values are in dp.
-
-// The badge offsets are different depending on whether the icon is in the tray
-// or menu.
-const int kTrayIconBadgeOffset = 3;
-const int kMenuIconBadgeOffset = 2;
-
-//------------------------------------------------------------------------------
 // class used for maintaining a map of network state and images.
 class NetworkIconImpl {
  public:
@@ -60,7 +43,7 @@ class NetworkIconImpl {
 
   // Determines whether or not the associated network might be dirty and if so
   // updates and generates the icon. Does nothing if network no longer exists.
-  void Update(const chromeos::NetworkState* network);
+  void Update(const chromeos::NetworkState* network, bool show_vpn_badge);
 
   const gfx::ImageSkia& image() const { return image_; }
 
@@ -71,12 +54,6 @@ class NetworkIconImpl {
   // Updates the local state for cellular networks. Returns true if changed.
   bool UpdateCellularState(const chromeos::NetworkState* network);
 
-  // Updates the portal state for wireless networks. Returns true if changed.
-  bool UpdatePortalState(const chromeos::NetworkState* network);
-
-  // Updates the VPN badge. Returns true if changed.
-  bool UpdateVPNBadge();
-
   // Gets |badges| based on |network| and the current state.
   void GetBadges(const NetworkState* network, Badges* badges);
 
@@ -90,22 +67,12 @@ class NetworkIconImpl {
   const IconType icon_type_;
 
   // Cached state of the network when the icon was last generated.
-  std::string state_;
-
-  // Cached strength index of the network when the icon was last generated.
-  int strength_index_;
-
-  // Cached technology badge for the network when the icon was last generated.
+  std::string connection_state_;
+  int strength_index_ = -1;
   Badge technology_badge_ = {};
-
-  // Cached vpn badge for the network when the icon was last generated.
-  Badge vpn_badge_ = {};
-
-  // Cached roaming state of the network when the icon was last generated.
-  std::string roaming_state_;
-
-  // Cached portal state of the network when the icon was last generated.
-  bool behind_captive_portal_;
+  bool show_vpn_badge_ = false;
+  bool is_roaming_ = false;
+  bool behind_captive_portal_ = false;
 
   // Generated icon image.
   gfx::ImageSkia image_;
@@ -164,17 +131,22 @@ const double kConnectingImageAlpha = 0.5;
 // Number of discrete images to use for alpha fade animation
 const int kNumFadeImages = 10;
 
+bool IsTrayIcon(IconType icon_type) {
+  return icon_type == ICON_TYPE_TRAY_REGULAR ||
+         icon_type == ICON_TYPE_TRAY_OOBE;
+}
+
 SkColor GetDefaultColorForIconType(IconType icon_type) {
-  if (icon_type == ICON_TYPE_TRAY)
+  if (icon_type == ICON_TYPE_TRAY_REGULAR)
     return kTrayIconColor;
-  if (features::IsSystemTrayUnifiedEnabled())
-    return kUnifiedMenuIconColor;
-  else
-    return kMenuIconColor;
+  if (icon_type == ICON_TYPE_TRAY_OOBE)
+    return kOobeTrayIconColor;
+  return kUnifiedMenuIconColor;
 }
 
 bool IconTypeIsDark(IconType icon_type) {
-  return (icon_type != ICON_TYPE_TRAY);
+  // Dark icon is used for OOBE tray icon because the background is white.
+  return icon_type == ICON_TYPE_TRAY_OOBE;
 }
 
 bool IconTypeHasVPNBadge(IconType icon_type) {
@@ -182,12 +154,7 @@ bool IconTypeHasVPNBadge(IconType icon_type) {
 }
 
 gfx::Size GetSizeForBaseIconSize(const gfx::Size& base_icon_size) {
-  gfx::Size size = base_icon_size;
-  const int badge_offset = base_icon_size.width() == kTrayIconSize
-                               ? kTrayIconBadgeOffset
-                               : kMenuIconBadgeOffset;
-  size.Enlarge(badge_offset * 2, badge_offset * 2);
-  return size;
+  return base_icon_size;
 }
 
 gfx::ImageSkia CreateNetworkIconImage(const gfx::ImageSkia& icon,
@@ -213,7 +180,7 @@ ImageType ImageTypeForNetworkType(const std::string& type) {
 // have an associated Tether network. Used to display the correct icon.
 std::string GetEffectiveNetworkType(const NetworkState* network,
                                     IconType icon_type) {
-  if (icon_type == ICON_TYPE_TRAY && network->type() == shill::kTypeWifi &&
+  if (IsTrayIcon(icon_type) && network->type() == shill::kTypeWifi &&
       !network->tether_guid().empty()) {
     return chromeos::kTypeTether;
   }
@@ -226,8 +193,19 @@ ImageType ImageTypeForNetwork(const NetworkState* network, IconType icon_type) {
 }
 
 gfx::Size GetSizeForIconType(IconType icon_type) {
-  int side = icon_type == ICON_TYPE_TRAY ? kTrayIconSize : kMenuIconSize;
-  return gfx::Size(side, side);
+  int size = kMenuIconSize;
+  if (IsTrayIcon(icon_type)) {
+    size = kUnifiedTrayIconSize;
+  } else if (icon_type == ICON_TYPE_DEFAULT_VIEW) {
+    size = kUnifiedFeaturePodVectorIconSize;
+  }
+  return gfx::Size(size, size);
+}
+
+int GetPaddingForIconType(IconType icon_type) {
+  if (IsTrayIcon(icon_type))
+    return kUnifiedTrayNetworkIconPadding;
+  return kTrayNetworkIconPadding;
 }
 
 gfx::ImageSkia GetImageForIndex(ImageType image_type,
@@ -235,17 +213,7 @@ gfx::ImageSkia GetImageForIndex(ImageType image_type,
                                 int index) {
   return gfx::CanvasImageSource::MakeImageSkia<SignalStrengthImageSource>(
       image_type, GetDefaultColorForIconType(icon_type),
-      GetSizeForIconType(icon_type), index);
-}
-
-// Returns an image to represent either a fully connected network or a
-// disconnected network.
-const gfx::ImageSkia GetBasicImage(bool connected,
-                                   IconType icon_type,
-                                   const std::string& network_type) {
-  DCHECK_NE(shill::kTypeVPN, network_type);
-  return GetImageForIndex(ImageTypeForNetworkType(network_type), icon_type,
-                          connected ? kNumNetworkImages - 1 : 0);
+      GetSizeForIconType(icon_type), index, GetPaddingForIconType(icon_type));
 }
 
 gfx::ImageSkia* ConnectingWirelessImage(ImageType image_type,
@@ -285,21 +253,17 @@ gfx::ImageSkia ConnectingVpnImage(double animation) {
           SkColorSetA(kMenuIconColor, kConnectingImageAlpha), kMenuIconColor));
 }
 
-Badge ConnectingVpnBadge(double animation, IconType icon_type) {
-  return {&kNetworkBadgeVpnIcon,
-          SkColorSetA(GetDefaultColorForIconType(icon_type), 0xFF * animation)};
-}
-
 int StrengthIndex(int strength) {
-  if (strength == 0)
+  if (strength <= 0)
     return 0;
+
+  if (strength >= 100)
+    return kNumNetworkImages - 1;
+
   // Return an index in the range [1, kNumNetworkImages - 1].
   // This logic is equivalent to cr_network_icon.js:strengthToIndex_().
-  const float findex = (static_cast<float>(strength - 1) / 100.0f) *
-                       static_cast<float>(kNumNetworkImages - 1);
-  int index = 1 + static_cast<int>(findex);
-  index = std::max(std::min(index, kNumNetworkImages - 1), 1);
-  return index;
+  int zero_based_index = (strength - 1) * (kNumNetworkImages - 1) / 100;
+  return zero_based_index + 1;
 }
 
 Badge BadgeForNetworkTechnology(const NetworkState* network,
@@ -335,7 +299,7 @@ gfx::ImageSkia GetIcon(const NetworkState* network,
                        IconType icon_type,
                        int strength_index) {
   if (network->Matches(NetworkTypePattern::Ethernet())) {
-    return gfx::CreateVectorIcon(kNetworkEthernetIcon,
+    return gfx::CreateVectorIcon(vector_icons::kEthernetIcon,
                                  GetDefaultColorForIconType(icon_type));
   }
   if (network->Matches(NetworkTypePattern::Wireless())) {
@@ -343,7 +307,7 @@ gfx::ImageSkia GetIcon(const NetworkState* network,
                             strength_index);
   }
   if (network->Matches(NetworkTypePattern::VPN())) {
-    DCHECK_NE(ICON_TYPE_TRAY, icon_type);
+    DCHECK(!IsTrayIcon(icon_type));
     return gfx::CreateVectorIcon(kNetworkVpnIcon,
                                  GetDefaultColorForIconType(ICON_TYPE_LIST));
   }
@@ -352,39 +316,10 @@ gfx::ImageSkia GetIcon(const NetworkState* network,
   return gfx::ImageSkia();
 }
 
-//------------------------------------------------------------------------------
-// Get connecting images
-
 gfx::ImageSkia GetConnectingVpnImage(IconType icon_type) {
-  NetworkStateHandler* handler = NetworkHandler::Get()->network_state_handler();
-  const NetworkState* connected_network = nullptr;
-  if (icon_type == ICON_TYPE_TRAY) {
-    connected_network =
-        handler->ConnectedNetworkByType(NetworkTypePattern::NonVirtual());
-  }
   double animation = NetworkIconAnimation::GetInstance()->GetAnimation();
-
-  gfx::ImageSkia icon;
-  Badges badges;
-  if (connected_network) {
-    icon = GetImageForNetwork(connected_network, icon_type);
-    badges.bottom_left = ConnectingVpnBadge(animation, icon_type);
-  } else {
-    icon = ConnectingVpnImage(animation);
-  }
-  return CreateNetworkIconImage(icon, badges);
-}
-
-gfx::ImageSkia GetConnectingImage(IconType icon_type,
-                                  const std::string& network_type) {
-  if (network_type == shill::kTypeVPN)
-    return GetConnectingVpnImage(icon_type);
-
-  ImageType image_type = ImageTypeForNetworkType(network_type);
-  double animation = NetworkIconAnimation::GetInstance()->GetAnimation();
-
-  return CreateNetworkIconImage(
-      *ConnectingWirelessImage(image_type, icon_type, animation), Badges());
+  gfx::ImageSkia icon = ConnectingVpnImage(animation);
+  return CreateNetworkIconImage(icon, Badges());
 }
 
 }  // namespace
@@ -395,36 +330,37 @@ gfx::ImageSkia GetConnectingImage(IconType icon_type,
 NetworkIconImpl::NetworkIconImpl(const std::string& path,
                                  IconType icon_type,
                                  const std::string& network_type)
-    : network_path_(path),
-      icon_type_(icon_type),
-      strength_index_(-1),
-      behind_captive_portal_(false) {
+    : network_path_(path), icon_type_(icon_type) {
   // Default image is null.
 }
 
-void NetworkIconImpl::Update(const NetworkState* network) {
+void NetworkIconImpl::Update(const NetworkState* network, bool show_vpn_badge) {
   DCHECK(network);
   // Determine whether or not we need to update the icon.
   bool dirty = image_.isNull();
 
-  // If the network state has changed, the icon needs updating.
-  if (state_ != network->connection_state()) {
-    state_ = network->connection_state();
+  std::string connection_state = network->connection_state();
+  if (connection_state != connection_state_) {
+    connection_state_ = connection_state;
     dirty = true;
   }
 
-  dirty |= UpdatePortalState(network);
-
-  if (network->Matches(NetworkTypePattern::Wireless())) {
-    dirty |= UpdateWirelessStrengthIndex(network);
+  bool behind_captive_portal = network->IsCaptivePortal();
+  if (behind_captive_portal != behind_captive_portal_) {
+    behind_captive_portal_ = behind_captive_portal;
+    dirty = true;
   }
 
+  if (network->Matches(NetworkTypePattern::Wireless()))
+    dirty |= UpdateWirelessStrengthIndex(network);
+
   if (network->Matches(NetworkTypePattern::Cellular()))
     dirty |= UpdateCellularState(network);
 
-  if (IconTypeHasVPNBadge(icon_type_) &&
-      network->Matches(NetworkTypePattern::NonVirtual())) {
-    dirty |= UpdateVPNBadge();
+  bool new_show_vpn_badge = show_vpn_badge && IconTypeHasVPNBadge(icon_type_);
+  if (new_show_vpn_badge != show_vpn_badge_) {
+    show_vpn_badge_ = new_show_vpn_badge;
+    dirty = true;
   }
 
   if (dirty) {
@@ -444,49 +380,22 @@ bool NetworkIconImpl::UpdateWirelessStrengthIndex(const NetworkState* network) {
 
 bool NetworkIconImpl::UpdateCellularState(const NetworkState* network) {
   bool dirty = false;
-  const Badge technology_badge = BadgeForNetworkTechnology(network, icon_type_);
-  if (technology_badge != technology_badge_) {
-    technology_badge_ = technology_badge;
-    dirty = true;
+  if (!features::IsSeparateNetworkIconsEnabled()) {
+    const Badge technology_badge =
+        BadgeForNetworkTechnology(network, icon_type_);
+    if (technology_badge != technology_badge_) {
+      technology_badge_ = technology_badge;
+      dirty = true;
+    }
   }
-  std::string roaming_state = network->roaming();
-  if (roaming_state != roaming_state_) {
-    roaming_state_ = roaming_state;
+  bool is_roaming = network->IndicateRoaming();
+  if (is_roaming != is_roaming_) {
+    is_roaming_ = is_roaming;
     dirty = true;
   }
   return dirty;
 }
 
-bool NetworkIconImpl::UpdatePortalState(const NetworkState* network) {
-  bool behind_captive_portal = false;
-  if (network && chromeos::network_portal_detector::IsInitialized()) {
-    NetworkPortalDetector::CaptivePortalState state =
-        chromeos::network_portal_detector::GetInstance()->GetCaptivePortalState(
-            network->guid());
-    behind_captive_portal =
-        state.status == NetworkPortalDetector::CAPTIVE_PORTAL_STATUS_PORTAL;
-  }
-
-  if (behind_captive_portal == behind_captive_portal_)
-    return false;
-  behind_captive_portal_ = behind_captive_portal;
-  return true;
-}
-
-bool NetworkIconImpl::UpdateVPNBadge() {
-  const NetworkState* vpn =
-      NetworkHandler::Get()->network_state_handler()->ConnectedNetworkByType(
-          NetworkTypePattern::VPN());
-  Badge vpn_badge = {};
-  if (vpn)
-    vpn_badge = {&kNetworkBadgeVpnIcon, GetDefaultColorForIconType(icon_type_)};
-  if (vpn_badge != vpn_badge_) {
-    vpn_badge_ = vpn_badge;
-    return true;
-  }
-  return false;
-}
-
 void NetworkIconImpl::GetBadges(const NetworkState* network, Badges* badges) {
   DCHECK(network);
 
@@ -494,33 +403,23 @@ void NetworkIconImpl::GetBadges(const NetworkState* network, Badges* badges) {
   const SkColor icon_color = GetDefaultColorForIconType(icon_type_);
   if (type == shill::kTypeWifi) {
     if (network->security_class() != shill::kSecurityNone &&
-        IconTypeIsDark(icon_type_)) {
-      badges->bottom_right = {&kNetworkBadgeSecureIcon, icon_color};
+        !IsTrayIcon(icon_type_)) {
+      badges->bottom_right = {&kUnifiedNetworkBadgeSecureIcon, icon_color};
     }
   } else if (type == shill::kTypeWimax) {
     technology_badge_ = {&kNetworkBadgeTechnology4gIcon, icon_color};
   } else if (type == shill::kTypeCellular) {
     // technology_badge_ is set in UpdateCellularState.
-    if (network->IsConnectedState() &&
-        network->roaming() == shill::kRoamingStateRoaming) {
-      // For networks that are always in roaming don't show roaming badge.
-      const DeviceState* device =
-          NetworkHandler::Get()->network_state_handler()->GetDeviceState(
-              network->device_path());
-      LOG_IF(WARNING, !device)
-          << "Could not find device state for " << network->device_path();
-      if (!device || !device->provider_requires_roaming()) {
-        badges->bottom_right = {&kNetworkBadgeRoamingIcon, icon_color};
-      }
-    }
+    if (network->IsConnectedState() && network->IndicateRoaming())
+      badges->bottom_right = {&kNetworkBadgeRoamingIcon, icon_color};
   }
-  // Only show technology, VPN, and captive portal badges when connected.
-  if (network->IsConnectedState()) {
+  // Only show technology badge when connected.
+  if (network->IsConnectedState() && !features::IsSeparateNetworkIconsEnabled())
     badges->top_left = technology_badge_;
-    badges->bottom_left = vpn_badge_;
-    if (behind_captive_portal_)
-      badges->bottom_right = {&kNetworkBadgeCaptivePortalIcon, icon_color};
-  }
+  if (show_vpn_badge_)
+    badges->bottom_left = {&kUnifiedNetworkBadgeVpnIcon, icon_color};
+  if (behind_captive_portal_)
+    badges->bottom_right = {&kUnifiedNetworkBadgeCaptivePortalIcon, icon_color};
 }
 
 void NetworkIconImpl::GenerateImage(const NetworkState* network) {
@@ -534,7 +433,8 @@ void NetworkIconImpl::GenerateImage(const NetworkState* network) {
 namespace {
 
 NetworkIconImpl* FindAndUpdateImageImpl(const NetworkState* network,
-                                        IconType icon_type) {
+                                        IconType icon_type,
+                                        bool show_vpn_badge) {
   // Find or add the icon.
   NetworkIconMap* icon_map = GetIconMap(icon_type);
   NetworkIconImpl* icon;
@@ -548,7 +448,7 @@ NetworkIconImpl* FindAndUpdateImageImpl(const NetworkState* network,
   }
 
   // Update and return the icon's image.
-  icon->Update(network);
+  icon->Update(network, show_vpn_badge);
   return icon;
 }
 
@@ -557,24 +457,68 @@ NetworkIconImpl* FindAndUpdateImageImpl(const NetworkState* network,
 //------------------------------------------------------------------------------
 // Public interface
 
-gfx::ImageSkia GetImageForNetwork(const NetworkState* network,
-                                  IconType icon_type) {
+const gfx::ImageSkia GetBasicImage(IconType icon_type,
+                                   const std::string& network_type,
+                                   bool connected) {
+  DCHECK_NE(shill::kTypeVPN, network_type);
+  return GetImageForIndex(ImageTypeForNetworkType(network_type), icon_type,
+                          connected ? kNumNetworkImages - 1 : 0);
+}
+
+gfx::ImageSkia GetImageForNonVirtualNetwork(const NetworkState* network,
+                                            IconType icon_type,
+                                            bool show_vpn_badge,
+                                            bool* animating) {
   DCHECK(network);
+  DCHECK(!network->Matches(NetworkTypePattern::VPN()));
   const std::string network_type = GetEffectiveNetworkType(network, icon_type);
 
-  if (!network->visible())
-    return GetBasicImage(false /* is_connected */, icon_type, network_type);
+  if (!network->visible()) {
+    if (animating)
+      *animating = false;
+    return GetBasicImage(icon_type, network_type, false /* connected */);
+  }
 
-  if (network->IsConnectingState())
-    return GetConnectingImage(icon_type, network_type);
+  if (network->IsConnectingState()) {
+    if (animating)
+      *animating = true;
+    return GetConnectingImageForNetworkType(network_type, icon_type);
+  }
 
-  NetworkIconImpl* icon = FindAndUpdateImageImpl(network, icon_type);
+  NetworkIconImpl* icon =
+      FindAndUpdateImageImpl(network, icon_type, show_vpn_badge);
+  if (animating)
+    *animating = false;
+  return icon->image();
+}
+
+gfx::ImageSkia GetImageForVPN(const NetworkState* vpn,
+                              IconType icon_type,
+                              bool* animating) {
+  DCHECK(vpn);
+  DCHECK(vpn->Matches(NetworkTypePattern::VPN()));
+  if (vpn->IsConnectingState()) {
+    if (animating)
+      *animating = true;
+    return GetConnectingVpnImage(icon_type);
+  }
+
+  NetworkIconImpl* icon =
+      FindAndUpdateImageImpl(vpn, icon_type, false /* show_vpn_badge */);
+  if (animating)
+    *animating = false;
   return icon->image();
 }
 
 gfx::ImageSkia GetImageForWiFiEnabledState(bool enabled, IconType icon_type) {
+  if (!enabled) {
+    return gfx::CreateVectorIcon(kUnifiedMenuWifiOffIcon,
+                                 GetSizeForIconType(icon_type).width(),
+                                 GetDefaultColorForIconType(icon_type));
+  }
+
   gfx::ImageSkia image =
-      GetBasicImage(true /* connected */, icon_type, shill::kTypeWifi);
+      GetBasicImage(icon_type, shill::kTypeWifi, true /* connected */);
   Badges badges;
   if (!enabled) {
     badges.center = {&kNetworkBadgeOffIcon,
@@ -583,9 +527,33 @@ gfx::ImageSkia GetImageForWiFiEnabledState(bool enabled, IconType icon_type) {
   return CreateNetworkIconImage(image, badges);
 }
 
-gfx::ImageSkia GetImageForDisconnectedCellNetwork() {
-  return GetBasicImage(false /* not connected */, ICON_TYPE_LIST,
-                       shill::kTypeCellular);
+gfx::ImageSkia GetConnectingImageForNetworkType(const std::string& network_type,
+                                                IconType icon_type) {
+  DCHECK(network_type != shill::kTypeVPN);
+  ImageType image_type = ImageTypeForNetworkType(network_type);
+  double animation = NetworkIconAnimation::GetInstance()->GetAnimation();
+
+  return CreateNetworkIconImage(
+      *ConnectingWirelessImage(image_type, icon_type, animation), Badges());
+}
+
+gfx::ImageSkia GetConnectedNetworkWithConnectingVpnImage(
+    const NetworkState* connected_network,
+    IconType icon_type) {
+  DCHECK(connected_network);
+  gfx::ImageSkia icon = GetImageForNonVirtualNetwork(
+      connected_network, icon_type, false /* show_vpn_badge */);
+  double animation = NetworkIconAnimation::GetInstance()->GetAnimation();
+  Badges badges;
+  badges.bottom_left = {
+      &kUnifiedNetworkBadgeVpnIcon,
+      SkColorSetA(GetDefaultColorForIconType(icon_type), 0xFF * animation)};
+  return CreateNetworkIconImage(icon, badges);
+}
+
+gfx::ImageSkia GetDisconnectedImageForNetworkType(
+    const std::string& network_type) {
+  return GetBasicImage(ICON_TYPE_LIST, network_type, false /* connected */);
 }
 
 gfx::ImageSkia GetImageForNewWifiNetwork(SkColor icon_color,
@@ -604,15 +572,7 @@ base::string16 GetLabelForNetwork(const chromeos::NetworkState* network,
   DCHECK(network);
   std::string activation_state = network->activation_state();
   if (icon_type == ICON_TYPE_LIST || icon_type == ICON_TYPE_MENU_LIST) {
-    // Show "<network>: [Connecting|Activating|Reconnecting]..."
-    // TODO(varkha): Remaining states should migrate to secondary status in the
-    // network item and no longer be part of the label.
-    // See http://crbug.com/676181 .
-    if (network->IsReconnecting()) {
-      return l10n_util::GetStringFUTF16(
-          IDS_ASH_STATUS_TRAY_NETWORK_LIST_RECONNECTING,
-          base::UTF8ToUTF16(network->name()));
-    }
+    // Show "<network>: [Connecting|Activating]..."
     if (icon_type != ICON_TYPE_MENU_LIST && network->IsConnectingState()) {
       return l10n_util::GetStringFUTF16(
           IDS_ASH_STATUS_TRAY_NETWORK_LIST_CONNECTING,
@@ -631,13 +591,7 @@ base::string16 GetLabelForNetwork(const chromeos::NetworkState* network,
           base::UTF8ToUTF16(network->name()));
     }
   } else {
-    // Show "[Connected to|Connecting to|Activating|Reconnecting to] <network>"
-    // (non-list view).
-    if (network->IsReconnecting()) {
-      return l10n_util::GetStringFUTF16(
-          IDS_ASH_STATUS_TRAY_NETWORK_RECONNECTING,
-          base::UTF8ToUTF16(network->name()));
-    }
+    // Show "[Connected to|Connecting to|Activating] <network>" (non-list view).
     if (network->IsConnectedState()) {
       return l10n_util::GetStringFUTF16(IDS_ASH_STATUS_TRAY_NETWORK_CONNECTED,
                                         base::UTF8ToUTF16(network->name()));
@@ -660,126 +614,9 @@ base::string16 GetLabelForNetwork(const chromeos::NetworkState* network,
   }
 }
 
-int GetCellularUninitializedMsg() {
-  static base::Time s_uninitialized_state_time;
-  static int s_uninitialized_msg(0);
-
-  NetworkStateHandler* handler = NetworkHandler::Get()->network_state_handler();
-
-  if (handler->GetTechnologyState(NetworkTypePattern::Cellular()) ==
-      NetworkStateHandler::TECHNOLOGY_UNINITIALIZED) {
-    s_uninitialized_msg = IDS_ASH_STATUS_TRAY_INITIALIZING_CELLULAR;
-    s_uninitialized_state_time = base::Time::Now();
-    return s_uninitialized_msg;
-  }
-
-  if (handler->GetScanningByType(NetworkTypePattern::Cellular())) {
-    s_uninitialized_msg = IDS_ASH_STATUS_TRAY_MOBILE_SCANNING;
-    s_uninitialized_state_time = base::Time::Now();
-    return s_uninitialized_msg;
-  }
-
-  // There can be a delay between leaving the Initializing state and when
-  // a Cellular device shows up, so keep showing the initializing
-  // animation for a bit to avoid flashing the disconnect icon.
-  const int kInitializingDelaySeconds = 1;
-  base::TimeDelta dtime = base::Time::Now() - s_uninitialized_state_time;
-  if (dtime.InSeconds() < kInitializingDelaySeconds)
-    return s_uninitialized_msg;
-  return 0;
-}
-
-void GetDefaultNetworkImageAndLabel(IconType icon_type,
-                                    gfx::ImageSkia* image,
-                                    base::string16* label,
-                                    bool* animating) {
-  NetworkStateHandler* state_handler =
-      NetworkHandler::Get()->network_state_handler();
-  NetworkConnectionHandler* connect_handler =
-      NetworkHandler::Get()->network_connection_handler();
-  const NetworkState* connected_network =
-      state_handler->ConnectedNetworkByType(NetworkTypePattern::NonVirtual());
-  const NetworkState* connecting_network =
-      state_handler->ConnectingNetworkByType(NetworkTypePattern::Wireless());
-  if (!connecting_network && icon_type == ICON_TYPE_TRAY) {
-    connecting_network =
-        state_handler->ConnectingNetworkByType(NetworkTypePattern::VPN());
-  }
-
-  const NetworkState* network;
-  // If we are connecting to a network, and there is either no connected
-  // network, or the connection was user requested, or shill triggered a
-  // reconnection, use the connecting network.
-  if (connecting_network &&
-      (!connected_network || connecting_network->IsReconnecting() ||
-       connect_handler->HasConnectingNetwork(connecting_network->path()))) {
-    network = connecting_network;
-  } else {
-    network = connected_network;
-  }
-
-  // Don't show ethernet in the tray
-  if (icon_type == ICON_TYPE_TRAY && network &&
-      network->Matches(NetworkTypePattern::Ethernet())) {
-    *image = gfx::ImageSkia();
-    *animating = false;
-    return;
-  }
-
-  if (!network) {
-    // If no connecting network, check if we are activating a network.
-    const NetworkState* mobile_network =
-        state_handler->FirstNetworkByType(NetworkTypePattern::Mobile());
-    if (mobile_network && (mobile_network->activation_state() ==
-                           shill::kActivationStateActivating)) {
-      network = mobile_network;
-    }
-  }
-  if (!network) {
-    // If no connecting network, check for mobile initializing. Do not display
-    // the message about enabling Bluetooth for Tether.
-    int uninitialized_msg = GetCellularUninitializedMsg();
-    if (uninitialized_msg != 0) {
-      *image = GetConnectingImage(icon_type, shill::kTypeCellular);
-      if (label)
-        *label = l10n_util::GetStringUTF16(uninitialized_msg);
-      *animating = true;
-    } else {
-      // Otherwise show a Wi-Fi icon. If Wi-Fi is disabled, show a full icon
-      // with a strikethrough. If it's enabled then it's disconnected, so show
-      // an empty wedge.
-      bool wifi_enabled =
-          NetworkHandler::Get()->network_state_handler()->IsTechnologyEnabled(
-              NetworkTypePattern::WiFi());
-      *image = wifi_enabled ? GetBasicImage(false /* not connected */,
-                                            icon_type, shill::kTypeWifi)
-                            : GetImageForWiFiEnabledState(
-                                  false /* not enabled*/, icon_type);
-      if (label) {
-        *label = l10n_util::GetStringUTF16(
-            IDS_ASH_STATUS_TRAY_NETWORK_NOT_CONNECTED);
-      }
-      *animating = false;
-    }
-    return;
-  }
-  *animating = network->IsConnectingState();
-  // Get icon and label for connected or connecting network.
-  *image = GetImageForNetwork(network, icon_type);
-  if (label)
-    *label = GetLabelForNetwork(network, icon_type);
-}
-
-void PurgeNetworkIconCache() {
-  NetworkStateHandler::NetworkStateList networks;
-  NetworkHandler::Get()->network_state_handler()->GetVisibleNetworkList(
-      &networks);
-  std::set<std::string> network_paths;
-  for (NetworkStateHandler::NetworkStateList::iterator iter = networks.begin();
-       iter != networks.end(); ++iter) {
-    network_paths.insert((*iter)->path());
-  }
-  PurgeIconMap(ICON_TYPE_TRAY, network_paths);
+void PurgeNetworkIconCache(const std::set<std::string>& network_paths) {
+  PurgeIconMap(ICON_TYPE_TRAY_OOBE, network_paths);
+  PurgeIconMap(ICON_TYPE_TRAY_REGULAR, network_paths);
   PurgeIconMap(ICON_TYPE_DEFAULT_VIEW, network_paths);
   PurgeIconMap(ICON_TYPE_LIST, network_paths);
   PurgeIconMap(ICON_TYPE_MENU_LIST, network_paths);
