@@ -5,6 +5,8 @@
 #ifndef ASH_SYSTEM_NETWORK_TRAY_NETWORK_STATE_OBSERVER_H_
 #define ASH_SYSTEM_NETWORK_TRAY_NETWORK_STATE_OBSERVER_H_
 
+#include <vector>
+
 #include "base/macros.h"
 #include "base/timer/timer.h"
 #include "chromeos/network/network_state_handler_observer.h"
@@ -27,14 +29,12 @@ class TrayNetworkStateObserver : public chromeos::NetworkStateHandlerObserver {
 
   ~TrayNetworkStateObserver() override;
 
-  // NetworkStateHandlerObserver overrides.
+  // NetworkStateHandlerObserver
   void NetworkListChanged() override;
   void DeviceListChanged() override;
-  void DefaultNetworkChanged(const chromeos::NetworkState* network) override;
-  void NetworkConnectionStateChanged(
-      const chromeos::NetworkState* network) override;
+  void ActiveNetworksChanged(const std::vector<const chromeos::NetworkState*>&
+                                 active_networks) override;
   void NetworkPropertiesUpdated(const chromeos::NetworkState* network) override;
-  void DevicePropertiesUpdated(const chromeos::DeviceState* device) override;
 
  private:
   void SignalUpdate(bool notify_a11y);
@@ -43,9 +43,6 @@ class TrayNetworkStateObserver : public chromeos::NetworkStateHandlerObserver {
   // Unowned Delegate pointer (must outlive this instance).
   Delegate* delegate_;
 
-  // Set to true when we should purge stale icons in the cache.
-  bool purge_icons_;
-
   // Frequency at which to push NetworkStateChanged updates. This avoids
   // unnecessarily frequent UI updates (which can be expensive). We set this
   // to 0 for tests to eliminate timing variance.
