@@ -13,7 +13,8 @@
 #include "ash/system/tray/tray_constants.h"
 #include "ash/system/tray/tray_container.h"
 #include "ash/wm/mru_window_tracker.h"
-#include "ash/wm/overview/window_selector_controller.h"
+#include "ash/wm/overview/overview_controller.h"
+#include "ash/wm/overview/overview_session.h"
 #include "ash/wm/splitview/split_view_controller.h"
 #include "ash/wm/tablet_mode/tablet_mode_controller.h"
 #include "ash/wm/window_state.h"
@@ -21,6 +22,7 @@
 #include "base/metrics/user_metrics_action.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/gfx/paint_vector_icon.h"
+#include "ui/views/animation/ink_drop.h"
 #include "ui/views/border.h"
 #include "ui/views/controls/image_view.h"
 #include "ui/wm/core/window_util.h"
@@ -48,24 +50,29 @@ OverviewButtonTray::OverviewButtonTray(Shelf* shelf)
   // horizontal shelf, no separator is required.
   set_separator_visibility(false);
 
-  Shell::Get()->AddShellObserver(this);
+  Shell::Get()->overview_controller()->AddObserver(this);
   Shell::Get()->tablet_mode_controller()->AddObserver(this);
 }
 
 OverviewButtonTray::~OverviewButtonTray() {
+  if (Shell::Get()->overview_controller())
+    Shell::Get()->overview_controller()->RemoveObserver(this);
   if (Shell::Get()->tablet_mode_controller())
     Shell::Get()->tablet_mode_controller()->RemoveObserver(this);
-  Shell::Get()->RemoveShellObserver(this);
 }
 
 void OverviewButtonTray::UpdateAfterLoginStatusChange(LoginStatus status) {
   UpdateIconVisibility();
 }
 
+void OverviewButtonTray::SnapRippleToActivated() {
+  GetInkDrop()->SnapToActivated();
+}
+
 void OverviewButtonTray::OnGestureEvent(ui::GestureEvent* event) {
   Button::OnGestureEvent(event);
   if (event->type() == ui::ET_GESTURE_LONG_PRESS) {
-    Shell::Get()->window_selector_controller()->OnOverviewButtonTrayLongPressed(
+    Shell::Get()->overview_controller()->OnOverviewButtonTrayLongPressed(
         event->location());
   }
 }
@@ -74,13 +81,12 @@ bool OverviewButtonTray::PerformAction(const ui::Event& event) {
   DCHECK(event.type() == ui::ET_MOUSE_RELEASED ||
          event.type() == ui::ET_GESTURE_TAP);
 
-  if (last_press_event_time_ &&
+  OverviewController* overview_controller = Shell::Get()->overview_controller();
+  // Skip if the second tap happened outside of overview. This can happen if a
+  // window gets activated in between, which cancels overview mode.
+  if (overview_controller->IsSelecting() && last_press_event_time_ &&
       event.time_stamp() - last_press_event_time_.value() <
           kDoubleTapThresholdMs) {
-    // Second taps should not be processed outside of overview mode. (First
-    // taps should be outside of overview).
-    DCHECK(Shell::Get()->window_selector_controller()->IsSelecting());
-
     base::RecordAction(base::UserMetricsAction("Tablet_QuickSwitch"));
 
     // Build mru window list. Use cycle as it excludes some windows we are not
@@ -89,8 +95,11 @@ bool OverviewButtonTray::PerformAction(const ui::Event& event) {
         Shell::Get()->mru_window_tracker()->BuildWindowForCycleList();
 
     // Switch to the second most recently used window (most recent is the
-    // current window) if it exists, unless splitview mode is active.
-    if (mru_window_list.size() > 1u) {
+    // current window) if it exists, unless splitview mode is active. Do not
+    // switch we entered overview mode will all windows minimized.
+    if (mru_window_list.size() > 1u &&
+        overview_controller->overview_session()->enter_exit_overview_type() !=
+            OverviewSession::EnterExitOverviewType::kWindowsMinimized) {
       aura::Window* new_active_window = mru_window_list[1];
 
       // In splitview mode, quick switch will only affect the windows on the non
@@ -110,7 +119,7 @@ bool OverviewButtonTray::PerformAction(const ui::Event& event) {
       }
 
       AnimateInkDrop(views::InkDropState::DEACTIVATED, nullptr);
-      wm::GetWindowState(new_active_window)->Activate();
+      ::wm::ActivateWindow(new_active_window);
       last_press_event_time_ = base::nullopt;
       return true;
     }
@@ -118,13 +127,11 @@ bool OverviewButtonTray::PerformAction(const ui::Event& event) {
 
   // If not in overview mode record the time of this tap. A subsequent tap will
   // be checked against this to see if we should quick switch.
-  last_press_event_time_ =
-      Shell::Get()->window_selector_controller()->IsSelecting()
-          ? base::nullopt
-          : base::make_optional(event.time_stamp());
+  last_press_event_time_ = Shell::Get()->overview_controller()->IsSelecting()
+                               ? base::nullopt
+                               : base::make_optional(event.time_stamp());
 
-  WindowSelectorController* controller =
-      Shell::Get()->window_selector_controller();
+  OverviewController* controller = Shell::Get()->overview_controller();
   // Note: Toggling overview mode will fail if there is no window to show, the
   // screen is locked, a modal dialog is open or is running in kiosk app
   // session.
@@ -138,11 +145,7 @@ void OverviewButtonTray::OnSessionStateChanged(
   UpdateIconVisibility();
 }
 
-void OverviewButtonTray::OnTabletModeStarted() {
-  UpdateIconVisibility();
-}
-
-void OverviewButtonTray::OnTabletModeEnded() {
+void OverviewButtonTray::OnTabletModeEventsBlockingChanged() {
   UpdateIconVisibility();
 }
 
@@ -160,24 +163,22 @@ base::string16 OverviewButtonTray::GetAccessibleNameForTray() {
   return l10n_util::GetStringUTF16(IDS_ASH_OVERVIEW_BUTTON_ACCESSIBLE_NAME);
 }
 
-void OverviewButtonTray::HideBubbleWithView(
-    const views::TrayBubbleView* bubble_view) {
+void OverviewButtonTray::HideBubbleWithView(const TrayBubbleView* bubble_view) {
   // This class has no bubbles to hide.
 }
 
 void OverviewButtonTray::UpdateIconVisibility() {
   // The visibility of the OverviewButtonTray has diverged from
-  // WindowSelectorController::CanSelect. The visibility of the button should
+  // OverviewController::CanSelect. The visibility of the button should
   // not change during transient times in which CanSelect is false. Such as when
   // a modal dialog is present.
   SessionController* session_controller = Shell::Get()->session_controller();
-
-  Shell* shell = Shell::Get();
-  SetVisible(
-      shell->tablet_mode_controller()->IsTabletModeWindowManagerEnabled() &&
-      session_controller->GetSessionState() ==
-          session_manager::SessionState::ACTIVE &&
-      !session_controller->IsRunningInAppMode());
+  SetVisible(Shell::Get()
+                 ->tablet_mode_controller()
+                 ->AreInternalInputDeviceEventsBlocked() &&
+             session_controller->GetSessionState() ==
+                 session_manager::SessionState::ACTIVE &&
+             !session_controller->IsRunningInAppMode());
 }
 
 }  // namespace ash
