@@ -4,19 +4,20 @@
 
 #include "ash/system/palette/palette_welcome_bubble.h"
 
+#include <memory>
+
 #include "ash/public/cpp/ash_pref_names.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/session/session_controller.h"
 #include "ash/shell.h"
-#include "ash/shell_port.h"
 #include "ash/strings/grit/ash_strings.h"
 #include "ash/system/palette/palette_tray.h"
-#include "chromeos/chromeos_switches.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/pref_service.h"
 #include "ui/aura/window.h"
 #include "ui/base/l10n/l10n_util.h"
-#include "ui/views/bubble/bubble_dialog_delegate.h"
+#include "ui/views/bubble/bubble_dialog_delegate_view.h"
 #include "ui/views/controls/label.h"
 #include "ui/views/layout/fill_layout.h"
 
@@ -36,7 +37,7 @@ class PaletteWelcomeBubble::WelcomeBubbleView
   WelcomeBubbleView(views::View* anchor, views::BubbleBorder::Arrow arrow)
       : views::BubbleDialogDelegateView(anchor, arrow) {
     set_close_on_deactivate(true);
-    set_can_activate(false);
+    SetCanActivate(false);
     set_accept_events(true);
     set_parent_window(
         anchor_widget()->GetNativeWindow()->GetRootWindow()->GetChildById(
@@ -58,7 +59,7 @@ class PaletteWelcomeBubble::WelcomeBubbleView
   void Init() override {
     SetLayoutManager(std::make_unique<views::FillLayout>());
     auto* label = new views::Label(l10n_util::GetStringUTF16(
-        chromeos::switches::IsVoiceInteractionEnabled()
+        chromeos::switches::IsAssistantEnabled()
             ? IDS_ASH_STYLUS_WARM_WELCOME_BUBBLE_WITH_ASSISTANT_DESCRIPTION
             : IDS_ASH_STYLUS_WARM_WELCOME_BUBBLE_DESCRIPTION));
     label->SetHorizontalAlignment(gfx::ALIGN_LEFT);
@@ -80,7 +81,7 @@ PaletteWelcomeBubble::PaletteWelcomeBubble(PaletteTray* tray) : tray_(tray) {
 PaletteWelcomeBubble::~PaletteWelcomeBubble() {
   if (bubble_view_) {
     bubble_view_->GetWidget()->RemoveObserver(this);
-    ShellPort::Get()->RemovePointerWatcher(this);
+    Shell::Get()->RemovePreTargetHandler(this);
   }
   Shell::Get()->session_controller()->RemoveObserver(this);
 }
@@ -93,7 +94,7 @@ void PaletteWelcomeBubble::RegisterProfilePrefs(PrefRegistrySimple* registry) {
 void PaletteWelcomeBubble::OnWidgetClosing(views::Widget* widget) {
   widget->RemoveObserver(this);
   bubble_view_ = nullptr;
-  ShellPort::Get()->RemovePointerWatcher(this);
+  Shell::Get()->RemovePreTargetHandler(this);
 }
 
 void PaletteWelcomeBubble::OnActiveUserPrefServiceChanged(
@@ -101,7 +102,7 @@ void PaletteWelcomeBubble::OnActiveUserPrefServiceChanged(
   active_user_pref_service_ = pref_service;
 }
 
-void PaletteWelcomeBubble::ShowIfNeeded(bool shown_by_stylus) {
+void PaletteWelcomeBubble::ShowIfNeeded() {
   if (!active_user_pref_service_)
     return;
 
@@ -115,11 +116,13 @@ void PaletteWelcomeBubble::ShowIfNeeded(bool shown_by_stylus) {
   if (user_type && *user_type == user_manager::USER_TYPE_GUEST)
     return;
 
-  if (!active_user_pref_service_->GetBoolean(
-          prefs::kShownPaletteWelcomeBubble) &&
-      !bubble_shown()) {
-    Show(shown_by_stylus);
-  }
+  if (!HasBeenShown())
+    Show();
+}
+
+bool PaletteWelcomeBubble::HasBeenShown() const {
+  return active_user_pref_service_ && active_user_pref_service_->GetBoolean(
+                                          prefs::kShownPaletteWelcomeBubble);
 }
 
 void PaletteWelcomeBubble::MarkAsShown() {
@@ -128,31 +131,20 @@ void PaletteWelcomeBubble::MarkAsShown() {
                                         true);
 }
 
-base::Optional<gfx::Rect> PaletteWelcomeBubble::GetBubbleBoundsForTest() {
-  if (bubble_view_)
-    return base::make_optional(bubble_view_->GetBoundsInScreen());
-
-  return base::nullopt;
+views::View* PaletteWelcomeBubble::GetBubbleViewForTesting() {
+  return bubble_view_;
 }
 
-void PaletteWelcomeBubble::Show(bool shown_by_stylus) {
+void PaletteWelcomeBubble::Show() {
   if (!bubble_view_) {
     DCHECK(tray_);
     bubble_view_ =
         new WelcomeBubbleView(tray_, views::BubbleBorder::BOTTOM_RIGHT);
   }
-  active_user_pref_service_->SetBoolean(prefs::kShownPaletteWelcomeBubble,
-                                        true);
+  MarkAsShown();
   bubble_view_->GetWidget()->Show();
   bubble_view_->GetWidget()->AddObserver(this);
-  ShellPort::Get()->AddPointerWatcher(this,
-                                      views::PointerWatcherEventTypes::BASIC);
-
-  // If the bubble is shown after the device first reads a stylus, ignore the
-  // first up event so the event responsible for showing the bubble does not
-  // also cause the bubble to close.
-  if (shown_by_stylus)
-    ignore_stylus_event_ = true;
+  Shell::Get()->AddPreTargetHandler(this);
 }
 
 void PaletteWelcomeBubble::Hide() {
@@ -160,30 +152,18 @@ void PaletteWelcomeBubble::Hide() {
     bubble_view_->GetWidget()->Close();
 }
 
-void PaletteWelcomeBubble::OnPointerEventObserved(
-    const ui::PointerEvent& event,
-    const gfx::Point& location_in_screen,
-    gfx::NativeView target) {
-  if (!bubble_view_)
-    return;
-
-  // For devices with external stylus, the bubble is activated after it receives
-  // a pen pointer event. However, on attaching |this| as a pointer watcher, it
-  // receives the same set of events which activated the bubble in the first
-  // place (ET_POINTER_DOWN, ET_POINTER_UP, ET_POINTER_CAPTURE_CHANGED). By only
-  // looking at ET_POINTER_UP events and skipping the first up event if the
-  // bubble is shown because of a stylus event, we ensure the same event that
-  // opened the bubble does not close it as well.
-  if (event.type() != ui::ET_POINTER_UP)
-    return;
-
-  if (ignore_stylus_event_) {
-    ignore_stylus_event_ = false;
-    return;
-  }
-
-  if (!bubble_view_->GetBoundsInScreen().Contains(location_in_screen))
+void PaletteWelcomeBubble::OnMouseEvent(ui::MouseEvent* event) {
+  if (bubble_view_ && event->type() == ui::ET_MOUSE_PRESSED &&
+      event->target() != bubble_view_->GetWidget()->GetNativeView()) {
     bubble_view_->GetWidget()->Close();
+  }
+}
+
+void PaletteWelcomeBubble::OnTouchEvent(ui::TouchEvent* event) {
+  if (bubble_view_ && event->type() == ui::ET_TOUCH_PRESSED &&
+      event->target() != bubble_view_->GetWidget()->GetNativeView()) {
+    bubble_view_->GetWidget()->Close();
+  }
 }
 
 }  // namespace ash
