@@ -6,15 +6,15 @@
 
 #include "ash/shell.h"
 #include "ash/strings/grit/ash_strings.h"
-#include "ash/system/tray/system_tray.h"
-#include "ash/system/web_notification/web_notification_tray.h"
 #include "ash/test/ash_test_base.h"
 #include "ash/wm/tablet_mode/tablet_mode_controller.h"
 #include "base/command_line.h"
+#include "base/run_loop.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/test/scoped_feature_list.h"
 #include "ui/accessibility/ax_node_data.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/chromeos/devicetype_utils.h"
@@ -37,16 +37,6 @@ class ScreenLayoutObserverTest : public AshTestBase {
   ~ScreenLayoutObserverTest() override;
 
  protected:
-  void SetUp() override {
-    AshTestBase::SetUp();
-    WebNotificationTray::DisableAnimationsForTest(true);
-  }
-
-  void TearDown() override {
-    WebNotificationTray::DisableAnimationsForTest(false);
-    AshTestBase::TearDown();
-  }
-
   ScreenLayoutObserver* GetScreenLayoutObserver();
   void CheckUpdate();
 
@@ -80,12 +70,12 @@ ScreenLayoutObserver* ScreenLayoutObserverTest::GetScreenLayoutObserver() {
 void ScreenLayoutObserverTest::CloseNotification() {
   message_center::MessageCenter::Get()->RemoveNotification(
       ScreenLayoutObserver::kNotificationId, false);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 }
 
 void ScreenLayoutObserverTest::ClickNotification() {
   const message_center::Notification* notification = GetDisplayNotification();
-  notification->Click();
+  notification->delegate()->Click(base::nullopt, base::nullopt);
 }
 
 base::string16 ScreenLayoutObserverTest::GetDisplayNotificationText() const {
@@ -138,34 +128,7 @@ ScreenLayoutObserverTest::GetDisplayNotification() const {
   return nullptr;
 }
 
-class ScreenLayoutObserverTestMultiMirroring
-    : public ScreenLayoutObserverTest,
-      public testing::WithParamInterface<bool> {
- public:
-  ScreenLayoutObserverTestMultiMirroring() = default;
-  ~ScreenLayoutObserverTestMultiMirroring() override = default;
-
- protected:
-  void SetUp() override {
-    bool should_disable_multi_mirroring = GetParam();
-    if (should_disable_multi_mirroring) {
-      base::CommandLine::ForCurrentProcess()->AppendSwitch(
-          ::switches::kDisableMultiMirroring);
-    }
-    ScreenLayoutObserverTest::SetUp();
-  }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(ScreenLayoutObserverTestMultiMirroring);
-};
-
-// Instantiate the boolean which is used to enable/disable multi-mirroring in
-// the parameterized tests.
-INSTANTIATE_TEST_CASE_P(,
-                        ScreenLayoutObserverTestMultiMirroring,
-                        testing::Bool());
-
-TEST_P(ScreenLayoutObserverTestMultiMirroring, DisplayNotifications) {
+TEST_F(ScreenLayoutObserverTest, DisplayNotifications) {
   Shell::Get()->screen_layout_observer()->set_show_notifications_for_testing(
       true);
 
@@ -191,28 +154,6 @@ TEST_P(ScreenLayoutObserverTestMultiMirroring, DisplayNotifications) {
             GetDisplayNotificationAdditionalText());
   EXPECT_TRUE(GetDisplayNotificationText().empty());
 
-  // UI-scale
-  CloseNotification();
-  UpdateDisplay("400x400@1.5");
-  EXPECT_EQ(l10n_util::GetStringFUTF16(
-                IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED,
-                GetFirstDisplayName(), base::UTF8ToUTF16("600x600")),
-            GetDisplayNotificationAdditionalText());
-  EXPECT_EQ(l10n_util::GetStringUTF16(
-                IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED_TITLE),
-            GetDisplayNotificationText());
-
-  // UI-scale to 1.0
-  CloseNotification();
-  UpdateDisplay("400x400");
-  EXPECT_EQ(l10n_util::GetStringFUTF16(
-                IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED,
-                GetFirstDisplayName(), base::UTF8ToUTF16("400x400")),
-            GetDisplayNotificationAdditionalText());
-  EXPECT_EQ(l10n_util::GetStringUTF16(
-                IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED_TITLE),
-            GetDisplayNotificationText());
-
   // No-update
   CloseNotification();
   UpdateDisplay("400x400");
@@ -267,26 +208,9 @@ TEST_P(ScreenLayoutObserverTestMultiMirroring, DisplayNotifications) {
   // Turn off mirror mode.
   CloseNotification();
   display_manager()->SetMirrorMode(display::MirrorMode::kOff, base::nullopt);
-  if (display_manager()->is_multi_mirroring_enabled()) {
-    EXPECT_EQ(
-        l10n_util::GetStringUTF16(IDS_ASH_STATUS_TRAY_DISPLAY_MIRROR_EXIT),
-        GetDisplayNotificationText());
-  } else {
-    EXPECT_EQ(l10n_util::GetStringFUTF16(IDS_ASH_STATUS_TRAY_DISPLAY_EXTENDED,
-                                         GetSecondDisplayName()),
-              GetDisplayNotificationText());
-  }
-  EXPECT_TRUE(GetDisplayNotificationAdditionalText().empty());
-
-  // Resize the first display.
-  UpdateDisplay("400x400@1.5,200x200");
-  EXPECT_EQ(l10n_util::GetStringFUTF16(
-                IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED,
-                GetFirstDisplayName(), base::UTF8ToUTF16("600x600")),
-            GetDisplayNotificationAdditionalText());
-  EXPECT_EQ(l10n_util::GetStringUTF16(
-                IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED_TITLE),
+  EXPECT_EQ(l10n_util::GetStringUTF16(IDS_ASH_STATUS_TRAY_DISPLAY_MIRROR_EXIT),
             GetDisplayNotificationText());
+  EXPECT_TRUE(GetDisplayNotificationAdditionalText().empty());
 
   // Rotate the second.
   UpdateDisplay("400x400@1.5,200x200/r");
@@ -319,7 +243,8 @@ TEST_F(ScreenLayoutObserverTest, ZoomingInUnifiedModeNotification) {
   // Using keyboard shortcuts to change the zoom should result in a
   // notification.
   CloseNotification();
-  EXPECT_TRUE(display_manager()->ZoomInternalDisplay(false /* up */));
+  int64_t display_id = display::Screen::GetScreen()->GetPrimaryDisplay().id();
+  EXPECT_TRUE(display_manager()->ZoomDisplay(display_id, false /* up */));
   EXPECT_EQ(l10n_util::GetStringFUTF16(
                 IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED,
                 GetUnifiedDisplayName(), base::UTF8ToUTF16("400x200")),
@@ -329,7 +254,7 @@ TEST_F(ScreenLayoutObserverTest, ZoomingInUnifiedModeNotification) {
             GetDisplayNotificationText());
 
   CloseNotification();
-  EXPECT_TRUE(display_manager()->ZoomInternalDisplay(true /* up */));
+  EXPECT_TRUE(display_manager()->ZoomDisplay(display_id, true /* up */));
   EXPECT_EQ(l10n_util::GetStringFUTF16(
                 IDS_ASH_STATUS_TRAY_DISPLAY_RESOLUTION_CHANGED,
                 GetUnifiedDisplayName(), base::UTF8ToUTF16("800x400")),
@@ -344,7 +269,7 @@ TEST_F(ScreenLayoutObserverTest, ZoomingInUnifiedModeNotification) {
   CloseNotification();
   Shell::Get()->screen_layout_observer()->SetDisplayChangedFromSettingsUI(
       display::kUnifiedDisplayId);
-  EXPECT_TRUE(display_manager()->ZoomInternalDisplay(false /* up */));
+  EXPECT_TRUE(display_manager()->ZoomDisplay(display_id, false /* up */));
   EXPECT_TRUE(GetDisplayNotificationAdditionalText().empty());
   EXPECT_TRUE(GetDisplayNotificationText().empty());
 }
@@ -436,37 +361,6 @@ TEST_F(ScreenLayoutObserverTest, ExitMirrorModeBecauseOfDockedModeMessage) {
             GetDisplayNotificationText());
 }
 
-// TODO(crbug.com/774795) Remove this test when multi mirroring is enabled by
-// default.
-// Tests that exiting mirror mode because of adding a third display shows the
-// correct "3+ displays mirror mode is not supported" message.
-TEST_P(ScreenLayoutObserverTestMultiMirroring,
-       ExitMirrorModeBecauseOfThirdDisplayMessage) {
-  if (display_manager()->is_multi_mirroring_enabled()) {
-    // This test is not neccessary when mirroring across 3+ displays is
-    // supported.
-    return;
-  }
-  Shell::Get()->screen_layout_observer()->set_show_notifications_for_testing(
-      true);
-  UpdateDisplay("400x400,200x200");
-  display::Display::SetInternalDisplayId(
-      display_manager()->GetSecondaryDisplay().id());
-
-  // Mirroring.
-  UpdateDisplay("400x400,200x200");
-  display_manager()->SetMirrorMode(display::MirrorMode::kNormal, base::nullopt);
-  EXPECT_EQ(l10n_util::GetStringFUTF16(IDS_ASH_STATUS_TRAY_DISPLAY_MIRRORING,
-                                       GetMirroringDisplayNames()),
-            GetDisplayNotificationText());
-
-  // Adding a third display. Mirror mode for 3+ displays is not supported.
-  CloseNotification();
-  UpdateDisplay("400x400,200x200,100x100");
-  EXPECT_EQ(l10n_util::GetStringUTF16(IDS_ASH_DISPLAY_MIRRORING_NOT_SUPPORTED),
-            GetDisplayNotificationText());
-}
-
 // Special case: tests that exiting mirror mode by removing a display shows the
 // correct message.
 TEST_F(ScreenLayoutObserverTest,
