@@ -5,30 +5,31 @@
 #include "ash/system/status_area_widget.h"
 
 #include "ash/focus_cycler.h"
-#include "ash/public/cpp/ash_features.h"
 #include "ash/public/cpp/ash_switches.h"
-#include "ash/public/cpp/config.h"
+#include "ash/public/cpp/system_tray_focus_observer.h"
 #include "ash/session/session_controller.h"
 #include "ash/session/test_session_controller_client.h"
 #include "ash/shell.h"
 #include "ash/system/ime_menu/ime_menu_tray.h"
-#include "ash/system/message_center/notification_tray.h"
 #include "ash/system/overview/overview_button_tray.h"
 #include "ash/system/palette/palette_tray.h"
 #include "ash/system/session/logout_button_tray.h"
 #include "ash/system/status_area_widget_test_helper.h"
-#include "ash/system/system_tray_focus_observer.h"
-#include "ash/system/tray/system_tray.h"
 #include "ash/system/tray/system_tray_notifier.h"
 #include "ash/system/unified/unified_system_tray.h"
 #include "ash/system/virtual_keyboard/virtual_keyboard_tray.h"
 #include "ash/test/ash_test_base.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
-#include "base/test/scoped_feature_list.h"
 #include "chromeos/dbus/dbus_thread_manager.h"
 #include "chromeos/network/network_handler.h"
 #include "components/prefs/testing_pref_service.h"
 #include "components/session_manager/session_manager_types.h"
+#include "ui/events/test/event_generator.h"
+#include "ui/keyboard/keyboard_controller.h"
+#include "ui/keyboard/keyboard_util.h"
+#include "ui/keyboard/public/keyboard_switches.h"
+#include "ui/keyboard/test/keyboard_test_util.h"
 
 using session_manager::SessionState;
 
@@ -51,29 +52,15 @@ TEST_F(StatusAreaWidgetTest, Basics) {
 
   // Default trays are constructed.
   EXPECT_TRUE(status->overview_button_tray());
-  if (features::IsSystemTrayUnifiedEnabled()) {
-    EXPECT_TRUE(status->unified_system_tray());
-  } else {
-    EXPECT_TRUE(status->system_tray());
-    EXPECT_TRUE(status->notification_tray());
-  }
+  EXPECT_TRUE(status->unified_system_tray());
   EXPECT_TRUE(status->logout_button_tray_for_testing());
   EXPECT_TRUE(status->ime_menu_tray());
   EXPECT_TRUE(status->virtual_keyboard_tray_for_testing());
   EXPECT_TRUE(status->palette_tray());
 
-  // Needed because NotificationTray updates its initial visibility
-  // asynchronously.
-  RunAllPendingInMessageLoop();
-
   // Default trays are visible.
   EXPECT_FALSE(status->overview_button_tray()->visible());
-  if (features::IsSystemTrayUnifiedEnabled()) {
-    EXPECT_TRUE(status->unified_system_tray()->visible());
-  } else {
-    EXPECT_TRUE(status->system_tray()->visible());
-    EXPECT_TRUE(status->notification_tray()->visible());
-  }
+  EXPECT_TRUE(status->unified_system_tray()->visible());
   EXPECT_FALSE(status->logout_button_tray_for_testing()->visible());
   EXPECT_FALSE(status->ime_menu_tray()->visible());
   EXPECT_FALSE(status->virtual_keyboard_tray_for_testing()->visible());
@@ -137,74 +124,9 @@ class StatusAreaWidgetFocusTest : public AshTestBase {
   DISALLOW_COPY_AND_ASSIGN(StatusAreaWidgetFocusTest);
 };
 
-// Tests that tab traversal through status area widget in non-active session
-// could properly send FocusOut event.
-TEST_F(StatusAreaWidgetFocusTest, FocusOutObserver) {
-  // This is old SystemTray version.
-  if (features::IsSystemTrayUnifiedEnabled())
-    return;
-
-  // Set session state to LOCKED.
-  SessionController* session = Shell::Get()->session_controller();
-  ASSERT_TRUE(session->IsActiveUserSessionStarted());
-  TestSessionControllerClient* client = GetSessionControllerClient();
-  client->SetSessionState(SessionState::LOCKED);
-  ASSERT_TRUE(session->IsScreenLocked());
-
-  StatusAreaWidget* status = StatusAreaWidgetTestHelper::GetStatusAreaWidget();
-  // Default trays are constructed.
-  ASSERT_TRUE(status->overview_button_tray());
-  ASSERT_TRUE(status->system_tray());
-  ASSERT_TRUE(status->notification_tray());
-  ASSERT_TRUE(status->logout_button_tray_for_testing());
-  ASSERT_TRUE(status->ime_menu_tray());
-  ASSERT_TRUE(status->virtual_keyboard_tray_for_testing());
-
-  // Needed because NotificationTray updates its initial visibility
-  // asynchronously.
-  base::RunLoop().RunUntilIdle();
-
-  // Default trays are visible.
-  ASSERT_FALSE(status->overview_button_tray()->visible());
-  ASSERT_TRUE(status->system_tray()->visible());
-  ASSERT_TRUE(status->notification_tray()->visible());
-  ASSERT_FALSE(status->logout_button_tray_for_testing()->visible());
-  ASSERT_FALSE(status->ime_menu_tray()->visible());
-  ASSERT_FALSE(status->virtual_keyboard_tray_for_testing()->visible());
-
-  // Set focus to status area widget, which will be be system tray.
-  ASSERT_TRUE(Shell::Get()->focus_cycler()->FocusWidget(status));
-  views::FocusManager* focus_manager = status->GetFocusManager();
-  EXPECT_EQ(status->system_tray(), focus_manager->GetFocusedView());
-
-  // A tab key event will move focus to notification tray.
-  GenerateTabEvent(false);
-  EXPECT_EQ(status->notification_tray(), focus_manager->GetFocusedView());
-  EXPECT_EQ(0, test_observer_->focus_out_count());
-  EXPECT_EQ(0, test_observer_->reverse_focus_out_count());
-
-  // Another tab key event will send FocusOut event, since we are not handling
-  // this event, focus will still be moved to system tray.
-  GenerateTabEvent(false);
-  EXPECT_EQ(status->system_tray(), focus_manager->GetFocusedView());
-  EXPECT_EQ(1, test_observer_->focus_out_count());
-  EXPECT_EQ(0, test_observer_->reverse_focus_out_count());
-
-  // A reverse tab key event will send reverse FocusOut event, since we are not
-  // handling this event, focus will still be moved to notification tray.
-  GenerateTabEvent(true);
-  EXPECT_EQ(status->notification_tray(), focus_manager->GetFocusedView());
-  EXPECT_EQ(1, test_observer_->focus_out_count());
-  EXPECT_EQ(1, test_observer_->reverse_focus_out_count());
-}
-
 // Tests that tab traversal through status area widget in non-active session
 // could properly send FocusOut event.
 TEST_F(StatusAreaWidgetFocusTest, FocusOutObserverUnified) {
-  // This is UnifiedSystemTray version.
-  if (!features::IsSystemTrayUnifiedEnabled())
-    return;
-
   // Set session state to LOCKED.
   SessionController* session = Shell::Get()->session_controller();
   ASSERT_TRUE(session->IsActiveUserSessionStarted());
@@ -220,10 +142,6 @@ TEST_F(StatusAreaWidgetFocusTest, FocusOutObserverUnified) {
   ASSERT_TRUE(status->ime_menu_tray());
   ASSERT_TRUE(status->virtual_keyboard_tray_for_testing());
 
-  // Needed because NotificationTray updates its initial visibility
-  // asynchronously.
-  base::RunLoop().RunUntilIdle();
-
   // Default trays are visible.
   ASSERT_FALSE(status->overview_button_tray()->visible());
   ASSERT_TRUE(status->unified_system_tray()->visible());
@@ -295,35 +213,25 @@ class UnifiedStatusAreaWidgetTest : public AshTestBase {
 
   // AshTestBase:
   void SetUp() override {
-    scoped_feature_list_.InitAndEnableFeature(features::kSystemTrayUnified);
-
     chromeos::DBusThreadManager::Initialize();
     // Initializing NetworkHandler before ash is more like production.
     chromeos::NetworkHandler::Initialize();
     AshTestBase::SetUp();
-    // Mash doesn't do this yet, so don't do it in tests either.
-    // http://crbug.com/718072
-    if (Shell::GetAshConfig() != Config::MASH) {
-      chromeos::NetworkHandler::Get()->InitializePrefServices(&profile_prefs_,
-                                                              &local_state_);
-    }
+    chromeos::NetworkHandler::Get()->InitializePrefServices(&profile_prefs_,
+                                                            &local_state_);
     // Networking stubs may have asynchronous initialization.
     base::RunLoop().RunUntilIdle();
   }
 
   void TearDown() override {
     // This roughly matches production shutdown order.
-    if (Shell::GetAshConfig() != Config::MASH) {
-      chromeos::NetworkHandler::Get()->ShutdownPrefServices();
-    }
+    chromeos::NetworkHandler::Get()->ShutdownPrefServices();
     AshTestBase::TearDown();
     chromeos::NetworkHandler::Shutdown();
     chromeos::DBusThreadManager::Shutdown();
   }
 
  private:
-  base::test::ScopedFeatureList scoped_feature_list_;
-
   TestingPrefServiceSimple profile_prefs_;
   TestingPrefServiceSimple local_state_;
 
@@ -335,4 +243,113 @@ TEST_F(UnifiedStatusAreaWidgetTest, Basics) {
   EXPECT_TRUE(status->unified_system_tray());
 }
 
+class StatusAreaWidgetVirtualKeyboardTest : public AshTestBase {
+ protected:
+  void SetUp() override {
+    base::CommandLine::ForCurrentProcess()->AppendSwitch(
+        keyboard::switches::kEnableVirtualKeyboard);
+    AshTestBase::SetUp();
+    ASSERT_TRUE(keyboard::IsKeyboardEnabled());
+    keyboard::test::WaitUntilLoaded();
+
+    // These tests only apply to the floating virtual keyboard, as it is the
+    // only case where both the virtual keyboard and the shelf are visible.
+    const gfx::Rect keyboard_bounds(0, 0, 1, 1);
+    keyboard_controller()->SetContainerType(
+        keyboard::mojom::ContainerType::kFloating, keyboard_bounds,
+        base::DoNothing());
+  }
+
+  keyboard::KeyboardController* keyboard_controller() {
+    return keyboard::KeyboardController::Get();
+  }
+};
+
+// See https://crbug.com/897672.
+TEST_F(StatusAreaWidgetVirtualKeyboardTest,
+       ClickingVirtualKeyboardTrayHidesShownKeyboard) {
+  // Set up the virtual keyboard tray icon along with some other tray icons.
+  StatusAreaWidget* status = StatusAreaWidgetTestHelper::GetStatusAreaWidget();
+  status->virtual_keyboard_tray_for_testing()->SetVisible(true);
+  status->ime_menu_tray()->SetVisible(true);
+
+  keyboard_controller()->ShowKeyboard(false /* locked */);
+  ASSERT_TRUE(keyboard::WaitUntilShown());
+
+  // The keyboard should hide when clicked.
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->set_current_screen_location(
+      status->virtual_keyboard_tray_for_testing()
+          ->GetBoundsInScreen()
+          .CenterPoint());
+  generator->ClickLeftButton();
+  ASSERT_TRUE(keyboard::WaitUntilHidden());
+}
+
+// See https://crbug.com/897672.
+TEST_F(StatusAreaWidgetVirtualKeyboardTest,
+       TappingVirtualKeyboardTrayHidesShownKeyboard) {
+  // Set up the virtual keyboard tray icon along with some other tray icons.
+  StatusAreaWidget* status = StatusAreaWidgetTestHelper::GetStatusAreaWidget();
+  status->virtual_keyboard_tray_for_testing()->SetVisible(true);
+  status->ime_menu_tray()->SetVisible(true);
+
+  keyboard_controller()->ShowKeyboard(false /* locked */);
+  ASSERT_TRUE(keyboard::WaitUntilShown());
+
+  // The keyboard should hide when tapped.
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->GestureTapAt(status->virtual_keyboard_tray_for_testing()
+                              ->GetBoundsInScreen()
+                              .CenterPoint());
+  ASSERT_TRUE(keyboard::WaitUntilHidden());
+}
+
+TEST_F(StatusAreaWidgetVirtualKeyboardTest, ClickingHidesVirtualKeyboard) {
+  keyboard_controller()->ShowKeyboard(false /* locked */);
+  ASSERT_TRUE(keyboard_controller()->IsKeyboardVisible());
+
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->set_current_screen_location(
+      StatusAreaWidgetTestHelper::GetStatusAreaWidget()
+          ->GetWindowBoundsInScreen()
+          .CenterPoint());
+  generator->ClickLeftButton();
+
+  // Times out if test fails.
+  ASSERT_TRUE(keyboard::WaitUntilHidden());
+}
+
+TEST_F(StatusAreaWidgetVirtualKeyboardTest, TappingHidesVirtualKeyboard) {
+  keyboard_controller()->ShowKeyboard(false /* locked */);
+  ASSERT_TRUE(keyboard::WaitUntilShown());
+
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->set_current_screen_location(
+      StatusAreaWidgetTestHelper::GetStatusAreaWidget()
+          ->GetWindowBoundsInScreen()
+          .CenterPoint());
+  generator->PressTouch();
+
+  // Times out if test fails.
+  ASSERT_TRUE(keyboard::WaitUntilHidden());
+}
+
+TEST_F(StatusAreaWidgetVirtualKeyboardTest, DoesNotHideLockedVirtualKeyboard) {
+  keyboard_controller()->ShowKeyboard(true /* locked */);
+  ASSERT_TRUE(keyboard::WaitUntilShown());
+
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->set_current_screen_location(
+      StatusAreaWidgetTestHelper::GetStatusAreaWidget()
+          ->GetWindowBoundsInScreen()
+          .CenterPoint());
+
+  generator->ClickLeftButton();
+  EXPECT_FALSE(keyboard::IsKeyboardHiding());
+
+  generator->PressTouch();
+  EXPECT_FALSE(keyboard::IsKeyboardHiding());
+}
+
 }  // namespace ash
