@@ -53,8 +53,13 @@ const SkColor kHeaderTextColorNormal = SkColorSetARGB(0x7f, 0, 0, 0);
 
 const SkColor kMobileNotConnectedXIconColor = SkColorSetRGB(0xb2, 0xb2, 0xb2);
 
-const int kTrayIconSize = 16;
-const SkColor kTrayIconColor = SK_ColorWHITE;
+const SkColor kTrayIconColor = gfx::kGoogleGrey200;
+// Note that the alpha value should match kSignalStrengthImageBgAlpha in
+// ash/public/cpp/network_icon_image_source.cc
+const SkColor kTrayIconBackgroundColor =
+    SkColorSetA(gfx::kGoogleGrey200, 0x4D /* 30% */);
+const SkColor kOobeTrayIconColor = gfx::kGoogleGrey600;
+
 const int kMenuIconSize = 20;
 const SkColor kMenuIconColor = gfx::kChromeIconGrey;
 const SkColor kMenuIconColorDisabled = SkColorSetA(gfx::kChromeIconGrey, 0x61);
@@ -65,7 +70,6 @@ const int kMenuEdgeEffectivePadding =
     kMenuExtraMarginFromLeftEdge + (kMenuButtonSize - kMenuIconSize) / 2;
 
 const int kHitRegionPadding = 4;
-const int kSeparatorWidth = 1;
 
 const SkColor kMenuSeparatorColor = SkColorSetA(SK_ColorBLACK, 0x1F);
 
