@@ -11,12 +11,13 @@
 #include "ash/session/session_controller.h"
 #include "ash/shell.h"
 #include "ash/shell_delegate.h"
+#include "base/bind.h"
 #include "base/metrics/histogram_macros.h"
 #include "components/pref_registry/pref_registry_syncable.h"
 #include "components/prefs/pref_change_registrar.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/pref_service.h"
-#include "services/ui/public/cpp/input_devices/input_device_controller_client.h"
+#include "services/ws/public/cpp/input_devices/input_device_controller_client.h"
 #include "ui/wm/core/cursor_manager.h"
 
 namespace ash {
@@ -41,7 +42,7 @@ void OnSetTouchpadEnabledDone(bool enabled, bool succeeded) {
     cursor_manager->HideCursor();
 }
 
-ui::InputDeviceControllerClient* GetInputDeviceControllerClient() {
+ws::InputDeviceControllerClient* GetInputDeviceControllerClient() {
   return Shell::Get()->shell_delegate()->GetInputDeviceControllerClient();
 }
 
