@@ -12,7 +12,6 @@
 #include "ash/display/window_tree_host_manager.h"
 #include "ash/public/cpp/ash_pref_names.h"
 #include "ash/public/cpp/ash_switches.h"
-#include "ash/public/cpp/config.h"
 #include "ash/public/cpp/login_constants.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/root_window_controller.h"
@@ -26,6 +25,8 @@
 #include "ash/wallpaper/wallpaper_view.h"
 #include "ash/wallpaper/wallpaper_widget_controller.h"
 #include "ash/wallpaper/wallpaper_window_state_manager.h"
+#include "ash/wm/overview/overview_constants.h"
+#include "ash/wm/overview/overview_controller.h"
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/files/file_enumerator.h"
@@ -36,12 +37,13 @@
 #include "base/path_service.h"
 #include "base/sequenced_task_runner.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/task/post_task.h"
 #include "base/task_runner_util.h"
-#include "base/task_scheduler/post_task.h"
 #include "base/values.h"
-#include "chromeos/chromeos_switches.h"
+#include "chromeos/constants/chromeos_switches.h"
 #include "components/prefs/pref_registry_simple.h"
 #include "components/prefs/scoped_user_pref_update.h"
+#include "services/data_decoder/public/cpp/decode_image.h"
 #include "ui/display/manager/display_manager.h"
 #include "ui/display/manager/managed_display_info.h"
 #include "ui/display/screen.h"
@@ -101,12 +103,6 @@ base::FilePath& GlobalChromeOSCustomWallpapersDir() {
   return *dir_chrome_os_custom_wallpapers;
 }
 
-// The file path of the device policy wallpaper (if any).
-base::FilePath& GlobalDevicePolicyWallpaperFile() {
-  static base::NoDestructor<base::FilePath> device_policy_wallpaper_file;
-  return *device_policy_wallpaper_file;
-}
-
 void SetGlobalUserDataDir(const base::FilePath& path) {
   base::FilePath& global_path = GlobalUserDataDir();
   global_path = path;
@@ -122,11 +118,6 @@ void SetGlobalChromeOSCustomWallpapersDir(const base::FilePath& path) {
   global_path = path;
 }
 
-void SetGlobalDevicePolicyWallpaperFile(const base::FilePath& path) {
-  base::FilePath& global_path = GlobalDevicePolicyWallpaperFile();
-  global_path = path;
-}
-
 // Returns the appropriate wallpaper resolution for all root windows.
 WallpaperController::WallpaperResolution GetAppropriateResolution() {
   gfx::Size size = WallpaperController::GetMaxDisplaySizeInNative();
@@ -161,14 +152,13 @@ std::string GetCustomWallpaperSubdirForCurrentResolution() {
 }
 
 // Resizes |image| to a resolution which is nearest to |preferred_width| and
-// |preferred_height| while respecting the |layout| choice. |output_skia| is
-// optional (may be null). Returns true on success.
-bool ResizeImage(const gfx::ImageSkia& image,
-                 WallpaperLayout layout,
-                 int preferred_width,
-                 int preferred_height,
-                 scoped_refptr<base::RefCountedBytes>* output,
-                 gfx::ImageSkia* output_skia) {
+// |preferred_height| while respecting the |layout| choice. Encodes the image to
+// JPEG and saves to |output|. Returns true on success.
+bool ResizeAndEncodeImage(const gfx::ImageSkia& image,
+                          WallpaperLayout layout,
+                          int preferred_width,
+                          int preferred_height,
+                          scoped_refptr<base::RefCountedBytes>* output) {
   int width = image.width();
   int height = image.height();
   int resized_width;
@@ -205,33 +195,25 @@ bool ResizeImage(const gfx::ImageSkia& image,
 
   SkBitmap bitmap = *(resized_image.bitmap());
   gfx::JPEGCodec::Encode(bitmap, kDefaultEncodingQuality, &(*output)->data());
-
-  if (output_skia) {
-    resized_image.MakeThreadSafe();
-    *output_skia = resized_image;
-  }
-
   return true;
 }
 
 // Resizes |image| to a resolution which is nearest to |preferred_width| and
 // |preferred_height| while respecting the |layout| choice and saves the
-// resized wallpaper to |path|. |output_skia| is optional (may be
-// null). Returns true on success.
+// resized wallpaper to |path|. Returns true on success.
 bool ResizeAndSaveWallpaper(const gfx::ImageSkia& image,
                             const base::FilePath& path,
                             WallpaperLayout layout,
                             int preferred_width,
-                            int preferred_height,
-                            gfx::ImageSkia* output_skia) {
+                            int preferred_height) {
   if (layout == WALLPAPER_LAYOUT_CENTER) {
     if (base::PathExists(path))
       base::DeleteFile(path, false);
     return false;
   }
   scoped_refptr<base::RefCountedBytes> data;
-  if (!ResizeImage(image, layout, preferred_width, preferred_height, &data,
-                   output_skia)) {
+  if (!ResizeAndEncodeImage(image, layout, preferred_width, preferred_height,
+                            &data)) {
     return false;
   }
 
@@ -241,36 +223,29 @@ bool ResizeAndSaveWallpaper(const gfx::ImageSkia& image,
   return written_bytes == data->size();
 }
 
-// Creates a 1x1 solid color image to be used as the backup default wallpaper.
-gfx::ImageSkia CreateSolidColorWallpaper() {
+// Creates a 1x1 solid color image.
+gfx::ImageSkia CreateSolidColorWallpaper(SkColor color) {
   SkBitmap bitmap;
   bitmap.allocN32Pixels(1, 1);
-  bitmap.eraseColor(kDefaultWallpaperColor);
+  bitmap.eraseColor(color);
   return gfx::ImageSkia::CreateFrom1xBitmap(bitmap);
 }
 
 // Returns true if a color should be extracted from the wallpaper based on the
 // command kAshShelfColor line arg.
 bool IsShelfColoringEnabled() {
-  const bool kDefaultValue = true;
-
-  if (!base::CommandLine::ForCurrentProcess()->HasSwitch(
-          switches::kAshShelfColor)) {
-    return kDefaultValue;
-  }
-
-  const std::string switch_value =
+  const std::string explicit_switch_value =
       base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
           switches::kAshShelfColor);
-  if (switch_value != switches::kAshShelfColorEnabled &&
-      switch_value != switches::kAshShelfColorDisabled) {
-    LOG(WARNING) << "Invalid '--" << switches::kAshShelfColor << "' value of '"
-                 << switch_value << "'. Defaulting to "
-                 << (kDefaultValue ? "enabled." : "disabled.");
-    return kDefaultValue;
+
+  // Always enabled, unless explicitly disabled.
+  if (explicit_switch_value == switches::kAshShelfColorDisabled) {
+    LOG(WARNING) << "Shelf coloring explicitly disabled. "
+                 << "This should only happen in tests.";
+    return false;
   }
 
-  return switch_value == switches::kAshShelfColorEnabled;
+  return true;
 }
 
 // Gets the color profiles for extracting wallpaper prominent colors.
@@ -312,9 +287,11 @@ void OnWallpaperDataRead(LoadedCallback callback,
     base::ThreadTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
         base::BindOnce(std::move(callback), base::Passed(gfx::ImageSkia())));
-  } else {
-    DecodeWallpaper(*data, std::move(callback));
+    return;
   }
+  // This image was once encoded to JPEG by |ResizeAndEncodeImage|.
+  DecodeWallpaper(*data, data_decoder::mojom::ImageCodec::ROBUST_JPEG,
+                  std::move(callback));
 }
 
 // Deletes a list of wallpaper files in |file_list|.
@@ -381,13 +358,11 @@ void SaveCustomWallpaper(const std::string& wallpaper_files_id,
   // resized wallpaper is not generated (i.e. chrome shutdown before resized
   // wallpaper is saved).
   ResizeAndSaveWallpaper(*image, original_path, WALLPAPER_LAYOUT_STRETCH,
-                         image->width(), image->height(), nullptr);
+                         image->width(), image->height());
   ResizeAndSaveWallpaper(*image, small_wallpaper_path, layout,
-                         kSmallWallpaperMaxWidth, kSmallWallpaperMaxHeight,
-                         nullptr);
+                         kSmallWallpaperMaxWidth, kSmallWallpaperMaxHeight);
   ResizeAndSaveWallpaper(*image, large_wallpaper_path, layout,
-                         kLargeWallpaperMaxWidth, kLargeWallpaperMaxHeight,
-                         nullptr);
+                         kLargeWallpaperMaxWidth, kLargeWallpaperMaxHeight);
 }
 
 // Checks if kiosk app is running. Note: it returns false either when there's
@@ -442,13 +417,13 @@ void SaveOnlineWallpaper(const std::string& url,
       *image,
       GetOnlineWallpaperPath(url,
                              WallpaperController::WALLPAPER_RESOLUTION_LARGE),
-      layout, image->width(), image->height(), nullptr);
+      layout, image->width(), image->height());
   ResizeAndSaveWallpaper(
       *image,
       GetOnlineWallpaperPath(url,
                              WallpaperController::WALLPAPER_RESOLUTION_SMALL),
       WALLPAPER_LAYOUT_CENTER_CROPPED, kSmallWallpaperMaxWidth,
-      kSmallWallpaperMaxHeight, nullptr);
+      kSmallWallpaperMaxHeight);
 }
 
 // Implementation of |WallpaperController::GetOfflineWallpaper|.
@@ -484,7 +459,7 @@ WallpaperController::WallpaperController()
       color_profiles_(GetProminentColorProfiles()),
       wallpaper_reload_delay_(kWallpaperReloadDelay),
       sequenced_task_runner_(base::CreateSequencedTaskRunnerWithTraits(
-          {base::MayBlock(), base::TaskPriority::USER_BLOCKING,
+          {base::MayBlock(), base::TaskPriority::USER_VISIBLE,
            base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN})),
       scoped_session_observer_(this),
       weak_factory_(this) {
@@ -522,15 +497,10 @@ gfx::Size WallpaperController::GetMaxDisplaySizeInNative() {
   for (const auto& display : display::Screen::GetScreen()->GetAllDisplays()) {
     // Use the native size, not ManagedDisplayInfo::size_in_pixel or
     // Display::size.
-    // TODO(msw): Avoid using Display::size here; see http://crbug.com/613657.
-    gfx::Size size = display.size();
-    if (Shell::HasInstance()) {
-      display::ManagedDisplayInfo info =
-          Shell::Get()->display_manager()->GetDisplayInfo(display.id());
-      // TODO(mash): Mash returns a fake ManagedDisplayInfo. crbug.com/622480
-      if (info.id() == display.id())
-        size = info.bounds_in_native().size();
-    }
+    display::ManagedDisplayInfo info =
+        Shell::Get()->display_manager()->GetDisplayInfo(display.id());
+    DCHECK_EQ(display.id(), info.id());
+    gfx::Size size = info.bounds_in_native().size();
     if (display.rotation() == display::Display::ROTATE_90 ||
         display.rotation() == display::Display::ROTATE_270) {
       size = gfx::Size(size.height(), size.width());
@@ -658,9 +628,18 @@ bool WallpaperController::CanOpenWallpaperPicker() {
          !IsActiveUserWallpaperControlledByPolicyImpl();
 }
 
+bool WallpaperController::HasShownAnyWallpaper() const {
+  return !!current_wallpaper_;
+}
+
 void WallpaperController::ShowWallpaperImage(const gfx::ImageSkia& image,
                                              WallpaperInfo info,
-                                             bool preview_mode) {
+                                             bool preview_mode,
+                                             bool always_on_top) {
+  // Prevent showing other wallpapers if there is an always-on-top wallpaper.
+  if (is_always_on_top_wallpaper_ && !always_on_top)
+    return;
+
   // Ignore show wallpaper requests during preview mode. This could happen if a
   // custom wallpaper previously set on another device is being synced.
   if (confirm_preview_wallpaper_callback_ && !preview_mode)
@@ -699,6 +678,10 @@ void WallpaperController::ShowWallpaperImage(const gfx::ImageSkia& image,
   current_wallpaper_->AddObserver(this);
   current_wallpaper_->StartResize();
 
+  if (is_first_wallpaper_) {
+    for (auto& observer : observers_)
+      observer.OnFirstWallpaperShown();
+  }
   mojo_observers_.ForAllPtrs([this](mojom::WallpaperObserver* observer) {
     observer->OnWallpaperChanged(current_wallpaper_->original_image_id());
   });
@@ -715,8 +698,8 @@ bool WallpaperController::IsPolicyControlled(const AccountId& account_id,
          info.type == POLICY;
 }
 
-void WallpaperController::PrepareWallpaperForLockScreenChange(bool locking) {
-  bool needs_blur = locking && IsBlurEnabled();
+void WallpaperController::UpdateWallpaperBlur(bool blur) {
+  bool needs_blur = blur && IsBlurAllowed();
   if (needs_blur == is_wallpaper_blurred_)
     return;
 
@@ -735,14 +718,15 @@ void WallpaperController::PrepareWallpaperForLockScreenChange(bool locking) {
 
 bool WallpaperController::ShouldApplyDimming() const {
   return Shell::Get()->session_controller()->IsUserSessionBlocked() &&
-         !base::CommandLine::ForCurrentProcess()->HasSwitch(
-             switches::kAshDisableLoginDimAndBlur);
+         !IsOneShotWallpaper();
 }
 
-bool WallpaperController::IsBlurEnabled() const {
-  return !IsDevicePolicyWallpaper() &&
-         !base::CommandLine::ForCurrentProcess()->HasSwitch(
-             switches::kAshDisableLoginDimAndBlur);
+bool WallpaperController::IsBlurAllowed() const {
+  return !IsDevicePolicyWallpaper() && !IsOneShotWallpaper();
+}
+
+bool WallpaperController::IsWallpaperBlurred() const {
+  return is_wallpaper_blurred_;
 }
 
 bool WallpaperController::SetUserWallpaperInfo(const AccountId& account_id,
@@ -770,7 +754,7 @@ bool WallpaperController::SetUserWallpaperInfo(const AccountId& account_id,
   auto wallpaper_info_dict = std::make_unique<base::DictionaryValue>();
   wallpaper_info_dict->SetString(
       kNewWallpaperDateNodeName,
-      base::Int64ToString(info.date.ToInternalValue()));
+      base::NumberToString(info.date.ToInternalValue()));
   wallpaper_info_dict->SetString(kNewWallpaperLocationNodeName, info.location);
   wallpaper_info_dict->SetInteger(kNewWallpaperLayoutNodeName, info.layout);
   wallpaper_info_dict->SetInteger(kNewWallpaperTypeNodeName, info.type);
@@ -878,15 +862,13 @@ void WallpaperController::Init(
     const base::FilePath& user_data_path,
     const base::FilePath& chromeos_wallpapers_path,
     const base::FilePath& chromeos_custom_wallpapers_path,
-    const base::FilePath& device_policy_wallpaper_path,
-    bool is_device_wallpaper_policy_enforced) {
+    const base::FilePath& device_policy_wallpaper_path) {
   DCHECK(!wallpaper_controller_client_.get());
   wallpaper_controller_client_ = std::move(client);
   SetGlobalUserDataDir(user_data_path);
   SetGlobalChromeOSWallpapersDir(chromeos_wallpapers_path);
   SetGlobalChromeOSCustomWallpapersDir(chromeos_custom_wallpapers_path);
-  SetGlobalDevicePolicyWallpaperFile(device_policy_wallpaper_path);
-  is_device_wallpaper_policy_enforced_ = is_device_wallpaper_policy_enforced;
+  SetDevicePolicyWallpaperPath(device_policy_wallpaper_path);
 }
 
 void WallpaperController::SetCustomWallpaper(
@@ -912,7 +894,7 @@ void WallpaperController::SetCustomWallpaper(
                             weak_factory_.GetWeakPtr(), image,
                             WallpaperInfo{std::string(), layout, CUSTOMIZED,
                                           base::Time::Now().LocalMidnight()},
-                            /*preview_mode=*/true);
+                            /*preview_mode=*/true, /*always_on_top=*/false);
     // Show the preview wallpaper.
     reload_preview_wallpaper_callback_.Run();
   } else {
@@ -961,10 +943,17 @@ void WallpaperController::SetOnlineWallpaperFromData(
       base::BindOnce(&WallpaperController::OnOnlineWallpaperDecoded,
                      weak_factory_.GetWeakPtr(), params, /*save_file=*/true,
                      std::move(callback));
-  if (bypass_decode_for_testing_)
-    std::move(decoded_callback).Run(CreateSolidColorWallpaper());
-  else
-    DecodeWallpaper(image_data, std::move(decoded_callback));
+  if (bypass_decode_for_testing_) {
+    std::move(decoded_callback)
+        .Run(CreateSolidColorWallpaper(kDefaultWallpaperColor));
+    return;
+  }
+  // Use default codec because 1) online wallpapers may have various formats,
+  // 2) the image data comes from the Chrome OS wallpaper picker and is
+  // trusted (third-party wallpaper apps use |SetThirdPartyWallpaper|), 3) the
+  // code path is never used on login screen (enforced by the check above).
+  DecodeWallpaper(image_data, data_decoder::mojom::ImageCodec::DEFAULT,
+                  std::move(decoded_callback));
 }
 
 void WallpaperController::SetDefaultWallpaper(
@@ -1020,19 +1009,25 @@ void WallpaperController::SetPolicyWallpaper(
       base::Passed(&user_info), wallpaper_files_id, kPolicyWallpaperFile,
       POLICY, WALLPAPER_LAYOUT_CENTER_CROPPED, show_wallpaper);
 
-  if (bypass_decode_for_testing_)
-    std::move(callback).Run(CreateSolidColorWallpaper());
-  else
-    DecodeWallpaper(data, std::move(callback));
+  if (bypass_decode_for_testing_) {
+    std::move(callback).Run(CreateSolidColorWallpaper(kDefaultWallpaperColor));
+    return;
+  }
+  // The default codec cannot be used here because the image data is provided by
+  // user and thus not trusted. In addition, only JPEG |data| is accepted.
+  DecodeWallpaper(data, data_decoder::mojom::ImageCodec::ROBUST_JPEG,
+                  std::move(callback));
 }
 
-void WallpaperController::SetDeviceWallpaperPolicyEnforced(bool enforced) {
-  bool previous_enforced = is_device_wallpaper_policy_enforced_;
-  is_device_wallpaper_policy_enforced_ = enforced;
-
+void WallpaperController::SetDevicePolicyWallpaperPath(
+    const base::FilePath& device_policy_wallpaper_path) {
+  const bool was_device_policy_wallpaper_enforced =
+      !device_policy_wallpaper_path_.empty();
+  device_policy_wallpaper_path_ = device_policy_wallpaper_path;
   if (ShouldSetDevicePolicyWallpaper()) {
     SetDevicePolicyWallpaper();
-  } else if ((previous_enforced != enforced) && !enforced) {
+  } else if (was_device_policy_wallpaper_enforced &&
+             device_policy_wallpaper_path.empty()) {
     // If the device wallpaper policy is cleared, the wallpaper should revert to
     // the wallpaper of the current user with the large pod in the users list in
     // the login screen. If there is no such user, use the first user in the
@@ -1074,6 +1069,10 @@ void WallpaperController::ConfirmPreviewWallpaper() {
 }
 
 void WallpaperController::CancelPreviewWallpaper() {
+  if (!confirm_preview_wallpaper_callback_) {
+    DCHECK(!reload_preview_wallpaper_callback_);
+    return;
+  }
   confirm_preview_wallpaper_callback_.Reset();
   reload_preview_wallpaper_callback_.Reset();
   ReloadWallpaper(/*clear_cache=*/false);
@@ -1128,17 +1127,6 @@ void WallpaperController::ShowUserWallpaper(
 
   const AccountId account_id = current_user_->account_id;
   const bool is_ephemeral = current_user_->is_ephemeral;
-  // Guest user or regular user in ephemeral mode.
-  if ((is_ephemeral && current_user_->has_gaia_account) ||
-      current_user_->type == user_manager::USER_TYPE_GUEST) {
-    if (!InitializeUserWallpaperInfo(account_id, is_ephemeral))
-      return;
-    SetDefaultWallpaperImpl(account_id, current_user_->type,
-                            /*show_wallpaper=*/true);
-    VLOG(1) << "User is ephemeral. Fallback to default wallpaper.";
-    return;
-  }
-
   WallpaperInfo info;
   if (!GetUserWallpaperInfo(account_id, &info, is_ephemeral)) {
     if (!InitializeUserWallpaperInfo(account_id, is_ephemeral))
@@ -1146,14 +1134,21 @@ void WallpaperController::ShowUserWallpaper(
     GetUserWallpaperInfo(account_id, &info, is_ephemeral);
   }
 
+  // For ephemeral users, the cache is the only place to access their wallpaper
+  // because it is not saved to disk. If the image doesn't exist in cache, it
+  // means the user's wallpaper type is default (i.e. the user never sets their
+  // own wallpaper), and it's a bug if it's not.
+  //
+  // For regular users, the image will be read from disk if the cache is not
+  // hit (e.g. when the first time the wallpaper is shown on login screen).
   gfx::ImageSkia user_wallpaper;
   if (GetWallpaperFromCache(account_id, &user_wallpaper)) {
-    ShowWallpaperImage(user_wallpaper, info, /*preview_mode=*/false);
+    ShowWallpaperImage(user_wallpaper, info, /*preview_mode=*/false,
+                       /*always_on_top=*/false);
     return;
   }
 
-  if (info.location.empty()) {
-    // Uses default wallpaper when file is empty.
+  if (info.type == DEFAULT) {
     SetDefaultWallpaperImpl(account_id, current_user_->type,
                             /*show_wallpaper=*/true);
     return;
@@ -1168,7 +1163,8 @@ void WallpaperController::ShowUserWallpaper(
 
   base::FilePath wallpaper_path;
   if (info.type == DEVICE) {
-    wallpaper_path = GlobalDevicePolicyWallpaperFile();
+    DCHECK(!device_policy_wallpaper_path_.empty());
+    wallpaper_path = device_policy_wallpaper_path_;
   } else {
     std::string sub_dir = GetCustomWallpaperSubdirForCurrentResolution();
     // Wallpaper is not resized when layout is
@@ -1208,6 +1204,36 @@ void WallpaperController::ShowSigninWallpaper() {
   }
 }
 
+void WallpaperController::ShowOneShotWallpaper(const gfx::ImageSkia& image) {
+  const WallpaperInfo info = {
+      std::string(), WallpaperLayout::WALLPAPER_LAYOUT_STRETCH,
+      WallpaperType::ONE_SHOT, base::Time::Now().LocalMidnight()};
+  ShowWallpaperImage(image, info, /*preview_mode=*/false,
+                     /*always_on_top=*/false);
+}
+
+void WallpaperController::ShowAlwaysOnTopWallpaper(
+    const base::FilePath& image_path) {
+  is_always_on_top_wallpaper_ = true;
+  const WallpaperInfo info = {
+      std::string(), WallpaperLayout::WALLPAPER_LAYOUT_CENTER_CROPPED,
+      WallpaperType::ONE_SHOT, base::Time::Now().LocalMidnight()};
+  ReadAndDecodeWallpaper(
+      base::BindOnce(&WallpaperController::OnAlwaysOnTopWallpaperDecoded,
+                     weak_factory_.GetWeakPtr(), info),
+      sequenced_task_runner_, image_path);
+}
+
+void WallpaperController::RemoveAlwaysOnTopWallpaper() {
+  if (!is_always_on_top_wallpaper_) {
+    DCHECK(!reload_always_on_top_wallpaper_callback_);
+    return;
+  }
+  is_always_on_top_wallpaper_ = false;
+  reload_always_on_top_wallpaper_callback_.Reset();
+  ReloadWallpaper(/*clear_cache=*/false);
+}
+
 void WallpaperController::RemoveUserWallpaper(
     mojom::WallpaperUserInfoPtr user_info,
     const std::string& wallpaper_files_id) {
@@ -1218,7 +1244,9 @@ void WallpaperController::RemoveUserWallpaper(
 void WallpaperController::RemovePolicyWallpaper(
     mojom::WallpaperUserInfoPtr user_info,
     const std::string& wallpaper_files_id) {
-  DCHECK(IsPolicyControlled(user_info->account_id, user_info->is_ephemeral));
+  if (!IsPolicyControlled(user_info->account_id, user_info->is_ephemeral))
+    return;
+
   // Updates the screen only when the user has logged in.
   const bool show_wallpaper =
       Shell::Get()->session_controller()->IsActiveUserSessionStarted();
@@ -1434,9 +1462,11 @@ void WallpaperController::FlushForTesting() {
 void WallpaperController::InstallDesktopController(aura::Window* root_window) {
   DCHECK_EQ(WALLPAPER_IMAGE, wallpaper_mode_);
 
-  auto* session_controller = Shell::Get()->session_controller();
+  bool session_blocked =
+      Shell::Get()->session_controller()->IsUserSessionBlocked();
+  bool in_overview = Shell::Get()->overview_controller()->IsSelecting();
   bool is_wallpaper_blurred =
-      session_controller->IsUserSessionBlocked() && IsBlurEnabled();
+      (session_blocked || in_overview) && IsBlurAllowed();
 
   if (is_wallpaper_blurred_ != is_wallpaper_blurred) {
     is_wallpaper_blurred_ = is_wallpaper_blurred;
@@ -1448,12 +1478,14 @@ void WallpaperController::InstallDesktopController(aura::Window* root_window) {
   }
 
   const int container_id = GetWallpaperContainerId(locked_);
+  float blur = login_constants::kClearBlurSigma;
+  if (is_wallpaper_blurred) {
+    blur = session_blocked ? login_constants::kBlurSigma : kWallpaperBlurSigma;
+  }
   RootWindowController::ForWindow(root_window)
       ->wallpaper_widget_controller()
       ->SetWallpaperWidget(CreateWallpaperWidget(root_window, container_id),
-                           is_wallpaper_blurred
-                               ? login_constants::kBlurSigma
-                               : login_constants::kClearBlurSigma);
+                           blur);
 }
 
 void WallpaperController::InstallDesktopControllerForAllWindows() {
@@ -1474,6 +1506,9 @@ bool WallpaperController::ReparentWallpaper(int container) {
 }
 
 int WallpaperController::GetWallpaperContainerId(bool locked) {
+  if (is_always_on_top_wallpaper_)
+    return kShellWindowId_AlwaysOnTopWallpaperContainer;
+
   return locked ? kShellWindowId_LockScreenWallpaperContainer
                 : kShellWindowId_WallpaperContainer;
 }
@@ -1519,7 +1554,7 @@ void WallpaperController::RemoveUserWallpaperImpl(
 
   base::PostTaskWithTraits(
       FROM_HERE,
-      {base::MayBlock(), base::TaskPriority::BACKGROUND,
+      {base::MayBlock(), base::TaskPriority::BEST_EFFORT,
        base::TaskShutdownBehavior::CONTINUE_ON_SHUTDOWN},
       base::BindOnce(&DeleteWallpaperInList, std::move(files_to_remove)));
 }
@@ -1617,7 +1652,7 @@ void WallpaperController::ReadAndDecodeWallpaper(
     const base::FilePath& file_path) {
   decode_requests_for_testing_.push_back(file_path);
   if (bypass_decode_for_testing_) {
-    std::move(callback).Run(CreateSolidColorWallpaper());
+    std::move(callback).Run(CreateSolidColorWallpaper(kDefaultWallpaperColor));
     return;
   }
   std::string* data = new std::string;
@@ -1684,7 +1719,7 @@ void WallpaperController::OnOnlineWallpaperDecoded(
                             weak_factory_.GetWeakPtr(), image,
                             WallpaperInfo{params.url, params.layout, ONLINE,
                                           base::Time::Now().LocalMidnight()},
-                            /*preview_mode=*/true);
+                            /*preview_mode=*/true, /*always_on_top=*/false);
     // Show the preview wallpaper.
     reload_preview_wallpaper_callback_.Run();
   } else {
@@ -1703,8 +1738,10 @@ void WallpaperController::SetOnlineWallpaperImpl(
     LOG(ERROR) << "Setting user wallpaper info fails. This should never happen "
                   "except in tests.";
   }
-  if (show_wallpaper)
-    ShowWallpaperImage(image, wallpaper_info, /*preview_mode=*/false);
+  if (show_wallpaper) {
+    ShowWallpaperImage(image, wallpaper_info, /*preview_mode=*/false,
+                       /*always_on_top=*/false);
+  }
 
   wallpaper_cache_map_[params.account_id] =
       CustomWallpaperElement(base::FilePath(), image);
@@ -1775,7 +1812,8 @@ void WallpaperController::OnDefaultWallpaperDecoded(
     const gfx::ImageSkia& image) {
   if (image.isNull()) {
     // Create a solid color wallpaper if the default wallpaper decoding fails.
-    cached_default_wallpaper_.image = CreateSolidColorWallpaper();
+    cached_default_wallpaper_.image =
+        CreateSolidColorWallpaper(kDefaultWallpaperColor);
     cached_default_wallpaper_.file_path.clear();
   } else {
     cached_default_wallpaper_.image = image;
@@ -1786,7 +1824,7 @@ void WallpaperController::OnDefaultWallpaperDecoded(
     WallpaperInfo info(cached_default_wallpaper_.file_path.value(), layout,
                        DEFAULT, base::Time::Now().LocalMidnight());
     ShowWallpaperImage(cached_default_wallpaper_.image, info,
-                       /*preview_mode=*/false);
+                       /*preview_mode=*/false, /*always_on_top=*/false);
   }
 }
 
@@ -1842,8 +1880,10 @@ void WallpaperController::SaveAndSetWallpaper(
                        layout, base::Passed(std::move(deep_copy))));
   }
 
-  if (show_wallpaper)
-    ShowWallpaperImage(image, info, /*preview_mode=*/false);
+  if (show_wallpaper) {
+    ShowWallpaperImage(image, info, /*preview_mode=*/false,
+                       /*always_on_top=*/false);
+  }
 
   wallpaper_cache_map_[user_info->account_id] =
       CustomWallpaperElement(wallpaper_path, image);
@@ -1865,8 +1905,10 @@ void WallpaperController::OnWallpaperDecoded(
   }
 
   wallpaper_cache_map_[account_id] = CustomWallpaperElement(path, image);
-  if (show_wallpaper)
-    ShowWallpaperImage(image, info, /*preview_mode=*/false);
+  if (show_wallpaper) {
+    ShowWallpaperImage(image, info, /*preview_mode=*/false,
+                       /*always_on_top=*/false);
+  }
 }
 
 void WallpaperController::ReloadWallpaper(bool clear_cache) {
@@ -1874,7 +1916,9 @@ void WallpaperController::ReloadWallpaper(bool clear_cache) {
   if (clear_cache)
     wallpaper_cache_map_.clear();
 
-  if (reload_preview_wallpaper_callback_)
+  if (reload_always_on_top_wallpaper_callback_)
+    reload_always_on_top_wallpaper_callback_.Run();
+  else if (reload_preview_wallpaper_callback_)
     reload_preview_wallpaper_callback_.Run();
   else if (current_user_)
     ShowUserWallpaper(std::move(current_user_));
@@ -1977,6 +2021,24 @@ base::Optional<std::vector<SkColor>> WallpaperController::GetCachedColors(
   return cached_colors_out;
 }
 
+void WallpaperController::OnAlwaysOnTopWallpaperDecoded(
+    const WallpaperInfo& info,
+    const gfx::ImageSkia& image) {
+  // Do nothing if |RemoveAlwaysOnTopWallpaper| was called before decoding
+  // completes.
+  if (!is_always_on_top_wallpaper_)
+    return;
+  if (image.isNull()) {
+    is_always_on_top_wallpaper_ = false;
+    return;
+  }
+  reload_always_on_top_wallpaper_callback_ =
+      base::BindRepeating(&WallpaperController::ShowWallpaperImage,
+                          weak_factory_.GetWeakPtr(), image, info,
+                          /*preview_mode=*/false, /*always_on_top=*/true);
+  reload_always_on_top_wallpaper_callback_.Run();
+}
+
 bool WallpaperController::MoveToLockedContainer() {
   if (locked_)
     return false;
@@ -1998,19 +2060,20 @@ bool WallpaperController::IsDevicePolicyWallpaper() const {
          current_wallpaper_->wallpaper_info().type == WallpaperType::DEVICE;
 }
 
+bool WallpaperController::IsOneShotWallpaper() const {
+  return current_wallpaper_ &&
+         current_wallpaper_->wallpaper_info().type == WallpaperType::ONE_SHOT;
+}
+
 bool WallpaperController::ShouldSetDevicePolicyWallpaper() const {
   // Only allow the device wallpaper if the policy is in effect for enterprise
   // managed devices.
-  if (!is_device_wallpaper_policy_enforced_)
+  if (device_policy_wallpaper_path_.empty())
     return false;
 
   // Only set the device wallpaper if we're at the login screen.
-  if (Shell::Get()->session_controller()->GetSessionState() !=
-      session_manager::SessionState::LOGIN_PRIMARY) {
-    return false;
-  }
-
-  return true;
+  return Shell::Get()->session_controller()->GetSessionState() ==
+         session_manager::SessionState::LOGIN_PRIMARY;
 }
 
 void WallpaperController::SetDevicePolicyWallpaper() {
@@ -2018,7 +2081,7 @@ void WallpaperController::SetDevicePolicyWallpaper() {
   ReadAndDecodeWallpaper(
       base::BindRepeating(&WallpaperController::OnDevicePolicyWallpaperDecoded,
                           weak_factory_.GetWeakPtr()),
-      sequenced_task_runner_.get(), GlobalDevicePolicyWallpaperFile());
+      sequenced_task_runner_.get(), device_policy_wallpaper_path_);
 }
 
 void WallpaperController::OnDevicePolicyWallpaperDecoded(
@@ -2034,10 +2097,11 @@ void WallpaperController::OnDevicePolicyWallpaperDecoded(
     SetDefaultWallpaperImpl(EmptyAccountId(), user_manager::USER_TYPE_REGULAR,
                             /*show_wallpaper=*/true);
   } else {
-    WallpaperInfo info(GlobalDevicePolicyWallpaperFile().value(),
+    WallpaperInfo info(device_policy_wallpaper_path_.value(),
                        WALLPAPER_LAYOUT_CENTER_CROPPED, DEVICE,
                        base::Time::Now().LocalMidnight());
-    ShowWallpaperImage(image, info, /*preview_mode=*/false);
+    ShowWallpaperImage(image, info, /*preview_mode=*/false,
+                       /*always_on_top=*/false);
   }
 }
 
