@@ -148,30 +148,41 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   // Returns true if the active user is allowed to open the wallpaper picker.
   bool CanOpenWallpaperPicker();
 
-  // Shows the wallpaper and alerts observers of changes. Does not show the
-  // image if |preview_mode| is false and the current wallpaper is still being
-  // previewed. See comments for |confirm_preview_wallpaper_callback_|.
+  // Returns whether any wallpaper has been shown. It returns false before the
+  // first wallpaper is set (which happens momentarily after startup), and will
+  // always return true thereafter.
+  bool HasShownAnyWallpaper() const;
+
+  // Shows the wallpaper and alerts observers of changes.
+  // Does not show the image if:
+  // 1)  |preview_mode| is false and the current wallpaper is still being
+  //     previewed. See comments for |confirm_preview_wallpaper_callback_|.
+  // 2)  |always_on_top| is false but the current wallpaper is always-on-top.
   void ShowWallpaperImage(const gfx::ImageSkia& image,
                           WallpaperInfo info,
-                          bool preview_mode);
+                          bool preview_mode,
+                          bool always_on_top);
 
   // Returns whether a wallpaper policy is enforced for |account_id| (not
   // including device policy).
   bool IsPolicyControlled(const AccountId& account_id, bool is_ephemeral) const;
 
-  // Prepares wallpaper to lock screen transition. Will apply blur if
-  // |locking| is true and remove it otherwise.
-  void PrepareWallpaperForLockScreenChange(bool locking);
+  // Update the blurred state of the current wallpaper. Applies blur if |blur|
+  // is true and blur is allowed by the controller, otherwise any existing blur
+  // is removed.
+  void UpdateWallpaperBlur(bool blur);
 
   // Wallpaper should be dimmed for login, lock, OOBE and add user screens.
   bool ShouldApplyDimming() const;
 
-  // Returns whether blur is enabled for login, lock, OOBE and add user screens.
-  // See crbug.com/775591.
-  bool IsBlurEnabled() const;
+  // Returns whether the current wallpaper is allowed to be blurred. See
+  // https://crbug.com/775591.
+  bool IsBlurAllowed() const;
 
   // Returns whether the current wallpaper is blurred.
-  bool IsWallpaperBlurred() const { return is_wallpaper_blurred_; }
+  // Note: this returns false when there's no wallpaper yet. Check
+  // |HasShownAnyWallpaper| if there's need to distinguish.
+  bool IsWallpaperBlurred() const;
 
   // Sets wallpaper info for |account_id| and saves it to local state if
   // |is_ephemeral| is false. Returns false if it fails (which happens if local
@@ -210,8 +221,7 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
             const base::FilePath& user_data_path,
             const base::FilePath& chromeos_wallpapers_path,
             const base::FilePath& chromeos_custom_wallpapers_path,
-            const base::FilePath& device_policy_wallpaper_path,
-            bool is_device_wallpaper_policy_enforced) override;
+            const base::FilePath& device_policy_wallpaper_path) override;
   void SetCustomWallpaper(mojom::WallpaperUserInfoPtr user_info,
                           const std::string& wallpaper_files_id,
                           const std::string& file_name,
@@ -240,7 +250,8 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   void SetPolicyWallpaper(mojom::WallpaperUserInfoPtr user_info,
                           const std::string& wallpaper_files_id,
                           const std::string& data) override;
-  void SetDeviceWallpaperPolicyEnforced(bool enforced) override;
+  void SetDevicePolicyWallpaperPath(
+      const base::FilePath& device_policy_wallpaper_path) override;
   void SetThirdPartyWallpaper(mojom::WallpaperUserInfoPtr user_info,
                               const std::string& wallpaper_files_id,
                               const std::string& file_name,
@@ -253,6 +264,9 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
                                    WallpaperLayout layout) override;
   void ShowUserWallpaper(mojom::WallpaperUserInfoPtr user_info) override;
   void ShowSigninWallpaper() override;
+  void ShowOneShotWallpaper(const gfx::ImageSkia& image) override;
+  void ShowAlwaysOnTopWallpaper(const base::FilePath& image_path) override;
+  void RemoveAlwaysOnTopWallpaper() override;
   void RemoveUserWallpaper(mojom::WallpaperUserInfoPtr user_info,
                            const std::string& wallpaper_files_id) override;
   void RemovePolicyWallpaper(mojom::WallpaperUserInfoPtr user_info,
@@ -491,6 +505,10 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   base::Optional<std::vector<SkColor>> GetCachedColors(
       const std::string& current_location) const;
 
+  // The callback when decoding of the always-on-top wallpaper completes.
+  void OnAlwaysOnTopWallpaperDecoded(const WallpaperInfo& info,
+                                     const gfx::ImageSkia& image);
+
   // Move all wallpaper widgets to the locked container.
   // Returns true if the wallpaper moved.
   bool MoveToLockedContainer();
@@ -502,6 +520,9 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   // Returns whether the current wallpaper is set by device policy.
   bool IsDevicePolicyWallpaper() const;
 
+  // Returns whether the current wallpaper has type of ONE_SHOT.
+  bool IsOneShotWallpaper() const;
+
   // Returns true if device wallpaper policy is in effect and we are at the
   // login screen right now.
   bool ShouldSetDevicePolicyWallpaper() const;
@@ -537,7 +558,7 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   // Bindings for the WallpaperController interface.
   mojo::BindingSet<mojom::WallpaperController> bindings_;
 
-  base::ObserverList<WallpaperControllerObserver> observers_;
+  base::ObserverList<WallpaperControllerObserver>::Unchecked observers_;
 
   mojo::AssociatedInterfacePtrSet<mojom::WallpaperObserver> mojo_observers_;
 
@@ -585,13 +606,15 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   // The wallpaper animation duration. An empty value disables the animation.
   base::TimeDelta animation_duration_;
 
-  // Whether the device wallpaper policy is enforced on this device.
-  bool is_device_wallpaper_policy_enforced_ = false;
+  base::FilePath device_policy_wallpaper_path_;
 
   // Whether the current wallpaper (if any) is the first wallpaper since the
   // controller initialization. Empty wallpapers for testing don't count.
   bool is_first_wallpaper_ = false;
 
+  // If true, the current wallpaper should always stay on top.
+  bool is_always_on_top_wallpaper_ = false;
+
   scoped_refptr<base::SequencedTaskRunner> sequenced_task_runner_;
 
   ScopedSessionObserver scoped_session_observer_;
@@ -609,6 +632,11 @@ class ASH_EXPORT WallpaperController : public mojom::WallpaperController,
   // change). Has the same lifetime with |confirm_preview_wallpaper_callback_|.
   base::RepeatingClosure reload_preview_wallpaper_callback_;
 
+  // Called when the always-on-top wallpaper needs to be reloaded (e.g. display
+  // size change). Non-empty if and only if |is_always_on_top_wallpaper_| is
+  // true.
+  base::RepeatingClosure reload_always_on_top_wallpaper_callback_;
+
   // If true, use a solid color wallpaper as if it is the decoded image.
   bool bypass_decode_for_testing_ = false;
 
