@@ -7,6 +7,9 @@
 
 #include <memory>
 
+#include "ash/wm/tablet_mode/tablet_mode_controller.h"
+#include "ash/wm/tablet_mode/tablet_mode_observer.h"
+#include "base/scoped_observer.h"
 #include "ui/views/context_menu_controller.h"
 #include "ui/views/view.h"
 
@@ -16,9 +19,9 @@ class Window;
 
 namespace ash {
 
-class PreEventDispatchHandler;
-
-class WallpaperView : public views::View, public views::ContextMenuController {
+class WallpaperView : public views::View,
+                      public views::ContextMenuController,
+                      TabletModeObserver {
  public:
   WallpaperView();
   ~WallpaperView() override;
@@ -30,16 +33,25 @@ class WallpaperView : public views::View, public views::ContextMenuController {
   void OnPaint(gfx::Canvas* canvas) override;
   bool OnMousePressed(const ui::MouseEvent& event) override;
 
+  // Overridden from TabletModeObserver:
+  void OnTabletModeStarted() override;
+  void OnTabletModeEnded() override;
+  void OnTabletControllerDestroyed() override;
+
   // Overridden from views::ContextMenuController:
   void ShowContextMenuForView(views::View* source,
                               const gfx::Point& point,
                               ui::MenuSourceType source_type) override;
-  std::unique_ptr<PreEventDispatchHandler> pre_dispatch_handler_;
+
+  ScopedObserver<TabletModeController, TabletModeObserver>
+      tablet_mode_observer_{this};
+  bool is_tablet_mode_ = false;
 
   DISALLOW_COPY_AND_ASSIGN(WallpaperView);
 };
 
-views::Widget* CreateWallpaper(aura::Window* root_window, int container_id);
+views::Widget* CreateWallpaperWidget(aura::Window* root_window,
+                                     int container_id);
 
 }  // namespace ash
 
