@@ -7,9 +7,11 @@
 #include "ash/display/mouse_cursor_event_filter.h"
 #include "ash/public/cpp/shell_window_ids.h"
 #include "ash/root_window_controller.h"
+#include "ash/shelf/shelf_constants.h"
 #include "ash/shelf/shelf_layout_manager.h"
 #include "ash/shell.h"
 #include "ash/test/ash_test_base.h"
+#include "ash/window_factory.h"
 #include "ash/wm/cursor_manager_test_api.h"
 #include "ash/wm/drag_window_controller.h"
 #include "ash/wm/window_positioning_utils.h"
@@ -72,20 +74,20 @@ class DragWindowResizerTest : public AshTestBase {
     gfx::Rect root_bounds(root->bounds());
     EXPECT_EQ(kRootHeight, root_bounds.height());
     EXPECT_EQ(800, root_bounds.width());
-    window_.reset(new aura::Window(&delegate_));
+    window_ = window_factory::NewWindow(&delegate_);
     window_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     window_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(window_.get());
     window_->set_id(1);
 
-    always_on_top_window_.reset(new aura::Window(&delegate2_));
+    always_on_top_window_ = window_factory::NewWindow(&delegate2_);
     always_on_top_window_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     always_on_top_window_->SetProperty(aura::client::kAlwaysOnTopKey, true);
     always_on_top_window_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(always_on_top_window_.get());
     always_on_top_window_->set_id(2);
 
-    system_modal_window_.reset(new aura::Window(&delegate3_));
+    system_modal_window_ = window_factory::NewWindow(&delegate3_);
     system_modal_window_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     system_modal_window_->SetProperty(aura::client::kModalKey,
                                       ui::MODAL_TYPE_SYSTEM);
@@ -93,23 +95,18 @@ class DragWindowResizerTest : public AshTestBase {
     ParentWindowInPrimaryRootWindow(system_modal_window_.get());
     system_modal_window_->set_id(3);
 
-    transient_child_ = new aura::Window(&delegate4_);
+    transient_child_ = window_factory::NewWindow(&delegate4_).release();
     transient_child_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     transient_child_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(transient_child_);
     transient_child_->set_id(4);
 
-    transient_parent_.reset(new aura::Window(&delegate5_));
+    transient_parent_ = window_factory::NewWindow(&delegate5_);
     transient_parent_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     transient_parent_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(transient_parent_.get());
     ::wm::AddTransientChild(transient_parent_.get(), transient_child_);
     transient_parent_->set_id(5);
-
-    panel_window_.reset(new aura::Window(&delegate6_));
-    panel_window_->SetType(aura::client::WINDOW_TYPE_PANEL);
-    panel_window_->Init(ui::LAYER_NOT_DRAWN);
-    ParentWindowInPrimaryRootWindow(panel_window_.get());
   }
 
   void TearDown() override {
@@ -117,7 +114,6 @@ class DragWindowResizerTest : public AshTestBase {
     always_on_top_window_.reset();
     system_modal_window_.reset();
     transient_parent_.reset();
-    panel_window_.reset();
     AshTestBase::TearDown();
   }
 
@@ -159,7 +155,6 @@ class DragWindowResizerTest : public AshTestBase {
   std::unique_ptr<aura::Window> window_;
   std::unique_ptr<aura::Window> always_on_top_window_;
   std::unique_ptr<aura::Window> system_modal_window_;
-  std::unique_ptr<aura::Window> panel_window_;
   aura::Window* transient_child_;
   std::unique_ptr<aura::Window> transient_parent_;
 
@@ -230,13 +225,14 @@ TEST_F(DragWindowResizerTest, WindowDragWithMultiDisplays) {
     // Window origin should be adjusted for minimum visibility (25px).
     int expected_x = -50 + wm::kMinimumOnScreenArea;
 
-    EXPECT_EQ(base::IntToString(expected_x) + ",10 50x60",
+    EXPECT_EQ(base::NumberToString(expected_x) + ",10 50x60",
               window_->bounds().ToString());
   }
   // Dropping a window that is larger than the destination work area
   // will shrink to fit to the work area.
   window_->SetBoundsInScreen(gfx::Rect(0, 0, 700, 500),
                              display::Screen::GetScreen()->GetPrimaryDisplay());
+  const int shelf_inset = 300 - ShelfConstants::shelf_size();
   EXPECT_EQ(root_windows[0], window_->GetRootWindow());
   {
     // Grab the top-right edge of the window and move the pointer to (0, 10)
@@ -248,7 +244,7 @@ TEST_F(DragWindowResizerTest, WindowDragWithMultiDisplays) {
     resizer->CompleteDrag();
     EXPECT_EQ(root_windows[1], window_->GetRootWindow());
     // Window size should be adjusted to fit to the work area
-    EXPECT_EQ(gfx::Size(400, 252).ToString(),
+    EXPECT_EQ(gfx::Size(400, shelf_inset).ToString(),
               window_->bounds().size().ToString());
     gfx::Rect window_bounds_in_screen = window_->GetBoundsInScreen();
     gfx::Rect intersect(window_->GetRootWindow()->GetBoundsInScreen());
@@ -276,7 +272,7 @@ TEST_F(DragWindowResizerTest, WindowDragWithMultiDisplays) {
     resizer->CompleteDrag();
     EXPECT_EQ(root_windows[1], window_->GetRootWindow());
     // Window size should be adjusted to fit to the work area
-    EXPECT_EQ(gfx::Size(400, 252).ToString(),
+    EXPECT_EQ(gfx::Size(400, shelf_inset).ToString(),
               window_->bounds().size().ToString());
     gfx::Rect window_bounds_in_screen = window_->GetBoundsInScreen();
     gfx::Rect intersect(window_->GetRootWindow()->GetBoundsInScreen());
@@ -297,7 +293,7 @@ TEST_F(DragWindowResizerTest, WindowDragWithMultiDisplaysActiveRoot) {
   ASSERT_EQ(2U, root_windows.size());
 
   aura::test::TestWindowDelegate delegate;
-  std::unique_ptr<aura::Window> window(new aura::Window(&delegate));
+  std::unique_ptr<aura::Window> window = window_factory::NewWindow(&delegate);
   window->SetType(aura::client::WINDOW_TYPE_NORMAL);
   window->Init(ui::LAYER_TEXTURED);
   ParentWindowInPrimaryRootWindow(window.get());
@@ -656,7 +652,7 @@ TEST_F(DragWindowResizerTest, MoveWindowAcrossDisplays) {
     resizer->Drag(CalculateDragPoint(*resizer, 399, 200), 0);
     EXPECT_TRUE(TestIfMouseWarpsAt(gfx::Point(399, 200)));
     EXPECT_EQ("401,200",
-              aura::Env::GetInstance()->last_mouse_location().ToString());
+              Shell::Get()->aura_env()->last_mouse_location().ToString());
     resizer->CompleteDrag();
   }
 
@@ -673,7 +669,7 @@ TEST_F(DragWindowResizerTest, MoveWindowAcrossDisplays) {
     resizer->Drag(CalculateDragPoint(*resizer, 399, 200), 0);
     EXPECT_TRUE(TestIfMouseWarpsAt(gfx::Point(399, 200)));
     EXPECT_EQ("401,200",
-              aura::Env::GetInstance()->last_mouse_location().ToString());
+              Shell::Get()->aura_env()->last_mouse_location().ToString());
     resizer->CompleteDrag();
   }
 
@@ -683,7 +679,7 @@ TEST_F(DragWindowResizerTest, MoveWindowAcrossDisplays) {
     window->SetBoundsInScreen(
         gfx::Rect(0, 0, 50, 60),
         display::Screen::GetScreen()->GetPrimaryDisplay());
-    aura::Env::GetInstance()->set_last_mouse_location(gfx::Point(0, 0));
+    Shell::Get()->aura_env()->SetLastMouseLocation(gfx::Point(0, 0));
     // Grab (0, 0) of the window.
     std::unique_ptr<WindowResizer> resizer(
         CreateDragWindowResizer(window, gfx::Point(), HTCAPTION));
@@ -691,7 +687,7 @@ TEST_F(DragWindowResizerTest, MoveWindowAcrossDisplays) {
     resizer->Drag(CalculateDragPoint(*resizer, 399, 200), 0);
     EXPECT_TRUE(TestIfMouseWarpsAt(gfx::Point(399, 200)));
     EXPECT_EQ("401,200",
-              aura::Env::GetInstance()->last_mouse_location().ToString());
+              Shell::Get()->aura_env()->last_mouse_location().ToString());
     resizer->CompleteDrag();
   }
 
@@ -708,7 +704,7 @@ TEST_F(DragWindowResizerTest, MoveWindowAcrossDisplays) {
     resizer->Drag(CalculateDragPoint(*resizer, 399, 200), 0);
     EXPECT_FALSE(TestIfMouseWarpsAt(gfx::Point(399, 200)));
     EXPECT_EQ("399,200",
-              aura::Env::GetInstance()->last_mouse_location().ToString());
+              Shell::Get()->aura_env()->last_mouse_location().ToString());
     resizer->CompleteDrag();
   }
 
@@ -725,24 +721,7 @@ TEST_F(DragWindowResizerTest, MoveWindowAcrossDisplays) {
     resizer->Drag(CalculateDragPoint(*resizer, 399, 200), 0);
     EXPECT_TRUE(TestIfMouseWarpsAt(gfx::Point(399, 200)));
     EXPECT_EQ("401,200",
-              aura::Env::GetInstance()->last_mouse_location().ToString());
-    resizer->CompleteDrag();
-  }
-
-  // Panel window can be moved across display.
-  {
-    aura::Window* window = panel_window_.get();
-    window->SetBoundsInScreen(
-        gfx::Rect(0, 0, 50, 60),
-        display::Screen::GetScreen()->GetPrimaryDisplay());
-    // Grab (0, 0) of the window.
-    std::unique_ptr<WindowResizer> resizer(
-        CreateDragWindowResizer(window, gfx::Point(), HTCAPTION));
-    ASSERT_TRUE(resizer.get());
-    resizer->Drag(CalculateDragPoint(*resizer, 399, 200), 0);
-    EXPECT_TRUE(TestIfMouseWarpsAt(gfx::Point(399, 200)));
-    EXPECT_EQ("401,200",
-              aura::Env::GetInstance()->last_mouse_location().ToString());
+              Shell::Get()->aura_env()->last_mouse_location().ToString());
     resizer->CompleteDrag();
   }
 }
