@@ -10,7 +10,6 @@
 #include "ash/accessibility/accessibility_controller.h"
 #include "ash/accessibility/test_accessibility_controller_client.h"
 #include "ash/public/cpp/ash_switches.h"
-#include "ash/public/cpp/config.h"
 #include "ash/session/session_controller.h"
 #include "ash/shell.h"
 #include "ash/shutdown_controller.h"
@@ -22,6 +21,7 @@
 #include "ash/wm/lock_state_controller_test_api.h"
 #include "ash/wm/session_state_animator.h"
 #include "ash/wm/test_session_state_animator.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/run_loop.h"
 #include "base/time/time.h"
@@ -342,9 +342,7 @@ TEST_F(LockStateControllerTest, LegacyLockAndShutDown) {
   EXPECT_EQ(0, NumShutdownRequests());
   // Make sure a mouse move event won't show the cursor.
   GenerateMouseMoveEvent();
-  // TODO: CursorManager not created in mash. http://crbug.com/631103.
-  if (Shell::GetAshConfig() != Config::MASH)
-    EXPECT_FALSE(cursor_visible());
+  EXPECT_FALSE(cursor_visible());
 
   ShutdownSoundPlayed();
   EXPECT_TRUE(lock_state_test_api_->real_shutdown_timer_is_running());
@@ -533,9 +531,7 @@ TEST_F(LockStateControllerTest, ShutdownWithoutButton) {
       SessionStateAnimator::kAllNonRootContainersMask,
       SessionStateAnimator::ANIMATION_HIDE_IMMEDIATELY));
   GenerateMouseMoveEvent();
-  // TODO: CursorManager not created in mash. http://crbug.com/631103.
-  if (Shell::GetAshConfig() != Config::MASH)
-    EXPECT_FALSE(cursor_visible());
+  EXPECT_FALSE(cursor_visible());
 }
 
 // Test that we display the fast-close animation and shut down when we get an
@@ -550,9 +546,7 @@ TEST_F(LockStateControllerTest, RequestShutdownFromLoginScreen) {
   Advance(SessionStateAnimator::ANIMATION_SPEED_SHUTDOWN);
 
   GenerateMouseMoveEvent();
-  // TODO: CursorManager not created in mash. http://crbug.com/631103.
-  if (Shell::GetAshConfig() != Config::MASH)
-    EXPECT_FALSE(cursor_visible());
+  EXPECT_FALSE(cursor_visible());
 
   EXPECT_EQ(0, NumShutdownRequests());
   ShutdownSoundPlayed();
@@ -576,9 +570,7 @@ TEST_F(LockStateControllerTest, RequestShutdownFromLockScreen) {
   Advance(SessionStateAnimator::ANIMATION_SPEED_SHUTDOWN);
 
   GenerateMouseMoveEvent();
-  // TODO: CursorManager not created in mash. http://crbug.com/631103.
-  if (Shell::GetAshConfig() != Config::MASH)
-    EXPECT_FALSE(cursor_visible());
+  EXPECT_FALSE(cursor_visible());
 
   EXPECT_EQ(0, NumShutdownRequests());
   ShutdownSoundPlayed();
