@@ -8,13 +8,15 @@
 
 #include "ash/screen_util.h"
 #include "ash/shell.h"
+#include "ash/wm/overview/overview_constants.h"
+#include "ash/wm/overview/overview_controller.h"
+#include "ash/wm/overview/overview_grid.h"
+#include "ash/wm/overview/overview_item.h"
+#include "ash/wm/overview/overview_session.h"
 #include "ash/wm/overview/overview_utils.h"
-#include "ash/wm/overview/scoped_transform_overview_window.h"
-#include "ash/wm/overview/window_grid.h"
-#include "ash/wm/overview/window_selector.h"
-#include "ash/wm/overview/window_selector_item.h"
 #include "ash/wm/splitview/split_view_constants.h"
 #include "ash/wm/splitview/split_view_drag_indicators.h"
+#include "ash/wm/splitview/split_view_utils.h"
 #include "ash/wm/window_positioning_utils.h"
 #include "base/numerics/ranges.h"
 #include "ui/aura/window.h"
@@ -24,10 +26,6 @@ namespace ash {
 
 namespace {
 
-// Before dragging an overview window, the window will scale up |kPreDragScale|
-// to indicate its selection.
-constexpr float kDragWindowScale = 0.04f;
-
 // The amount of distance from the start of drag the item needs to be dragged
 // vertically for it to be closed on release.
 constexpr float kDragToCloseDistanceThresholdDp = 160.f;
@@ -48,24 +46,32 @@ constexpr int kMinimumDragDistanceAlreadyInSnapRegionDp = 48;
 constexpr float kFlingToCloseVelocityThreshold = 2000.f;
 constexpr float kItemMinOpacity = 0.4f;
 
+void UnpauseOcclusionTracker() {
+  Shell::Get()->overview_controller()->UnpauseOcclusionTracker(
+      kOcclusionPauseDurationForDragMs);
+}
+
 }  // namespace
 
 OverviewWindowDragController::OverviewWindowDragController(
-    WindowSelector* window_selector)
-    : window_selector_(window_selector),
+    OverviewSession* overview_session)
+    : overview_session_(overview_session),
       split_view_controller_(Shell::Get()->split_view_controller()) {}
 
 OverviewWindowDragController::~OverviewWindowDragController() = default;
 
 void OverviewWindowDragController::InitiateDrag(
-    WindowSelectorItem* item,
+    OverviewItem* item,
     const gfx::Point& location_in_screen) {
   item_ = item;
   previous_event_location_ = location_in_screen;
   initial_event_location_ = location_in_screen;
-  started_in_snap_region_ =
-      GetSnapPosition(location_in_screen) != SplitViewController::NONE;
+  if (ShouldAllowSplitView()) {
+    started_in_snap_region_ =
+        GetSnapPosition(location_in_screen) != SplitViewController::NONE;
+  }
   current_drag_behavior_ = DragBehavior::kUndefined;
+  Shell::Get()->overview_controller()->PauseOcclusionTracker();
 }
 
 void OverviewWindowDragController::Drag(const gfx::Point& location_in_screen) {
@@ -78,14 +84,13 @@ void OverviewWindowDragController::Drag(const gfx::Point& location_in_screen) {
       return;
     }
 
-    if (IsOverviewSwipeToCloseEnabled() &&
-        std::abs(distance.x()) < std::abs(distance.y())) {
+    if (std::abs(distance.x()) < std::abs(distance.y())) {
       current_drag_behavior_ = DragBehavior::kDragToClose;
       original_opacity_ = item_->GetOpacity();
-      window_selector_->GetGridWithRootWindow(item_->root_window())
+      overview_session_->GetGridWithRootWindow(item_->root_window())
           ->StartNudge(item_);
       did_move_ = true;
-    } else {
+    } else if (ShouldAllowSplitView()) {
       StartSplitViewDragMode(location_in_screen);
     }
   }
@@ -98,19 +103,24 @@ void OverviewWindowDragController::Drag(const gfx::Point& location_in_screen) {
     float val = std::abs(static_cast<float>(location_in_screen.y()) -
                          initial_event_location_.y()) /
                 kDragToCloseDistanceThresholdDp;
-    window_selector_->GetGridWithRootWindow(item_->root_window())
+    overview_session_->GetGridWithRootWindow(item_->root_window())
         ->UpdateNudge(item_, val);
     val = base::ClampToRange(val, 0.f, 1.f);
     float opacity = original_opacity_;
-    if (opacity > kItemMinOpacity) {
+    if (opacity > kItemMinOpacity)
       opacity = original_opacity_ - val * (original_opacity_ - kItemMinOpacity);
-    }
     item_->SetOpacity(opacity);
   } else if (current_drag_behavior_ == DragBehavior::kDragToSnap) {
-    UpdateDragIndicatorsAndWindowGrid(location_in_screen);
+    UpdateDragIndicatorsAndOverviewGrid(location_in_screen);
     x_offset = location_in_screen.x() - previous_event_location_.x();
   }
 
+  // Update the split view divider bar status if necessary. If splitview is
+  // active when dragging the overview window, the split divider bar should be
+  // placed below the dragged window during dragging.
+  if (ShouldAllowSplitView())
+    split_view_controller_->OnWindowDragStarted(item_->GetWindow());
+
   // Update the dragged |item_|'s bounds accordingly.
   gfx::Rect bounds(item_->target_bounds());
   bounds.Offset(x_offset,
@@ -121,11 +131,20 @@ void OverviewWindowDragController::Drag(const gfx::Point& location_in_screen) {
 
 void OverviewWindowDragController::CompleteDrag(
     const gfx::Point& location_in_screen) {
+  // Update the split view divider bar stuatus if necessary. The divider bar
+  // should be placed above the dragged window after drag ends. Note here the
+  // passed paramters |snap_position_| and |location_in_screen| won't be used in
+  // this function for this case, but they are passed in as placeholders.
+  if (ShouldAllowSplitView()) {
+    split_view_controller_->OnWindowDragEnded(
+        item_->GetWindow(), snap_position_, location_in_screen);
+  }
+
   // Update window grid bounds and |snap_position_| in case the screen
   // orientation was changed.
   if (current_drag_behavior_ == DragBehavior::kDragToSnap) {
-    UpdateDragIndicatorsAndWindowGrid(location_in_screen);
-    window_selector_->SetSplitViewDragIndicatorsIndicatorState(
+    UpdateDragIndicatorsAndOverviewGrid(location_in_screen);
+    overview_session_->SetSplitViewDragIndicatorsIndicatorState(
         IndicatorState::kNone, gfx::Point());
   }
 
@@ -135,23 +154,22 @@ void OverviewWindowDragController::CompleteDrag(
     // If we are in drag to close mode close the window if it has been dragged
     // enough, otherwise reposition it and set its opacity back to its original
     // value.
-    window_selector_->GetGridWithRootWindow(item_->root_window())->EndNudge();
+    overview_session_->GetGridWithRootWindow(item_->root_window())->EndNudge();
     if (std::abs((location_in_screen - initial_event_location_).y()) >
         kDragToCloseDistanceThresholdDp) {
       item_->AnimateAndCloseWindow(
           (location_in_screen - initial_event_location_).y() < 0);
     } else {
       item_->SetOpacity(original_opacity_);
-      window_selector_->PositionWindows(/*animate=*/true);
+      overview_session_->PositionWindows(/*animate=*/true);
     }
-  } else {
-    DCHECK_EQ(current_drag_behavior_, DragBehavior::kDragToSnap);
+  } else if (current_drag_behavior_ == DragBehavior::kDragToSnap) {
     // If the window was dragged around but should not be snapped, move it back
     // to overview window grid.
     if (!ShouldUpdateDragIndicatorsOrSnap(location_in_screen) ||
         snap_position_ == SplitViewController::NONE) {
       item_->set_should_restack_on_animation_end(true);
-      window_selector_->PositionWindows(/*animate=*/true);
+      overview_session_->PositionWindows(/*animate=*/true);
     } else {
       SnapWindow(snap_position_);
     }
@@ -159,22 +177,21 @@ void OverviewWindowDragController::CompleteDrag(
   did_move_ = false;
   item_ = nullptr;
   current_drag_behavior_ = DragBehavior::kNoDrag;
+  UnpauseOcclusionTracker();
 }
 
 void OverviewWindowDragController::StartSplitViewDragMode(
     const gfx::Point& location_in_screen) {
-  // Increase the bounds of the dragged item.
-  gfx::Rect scaled_bounds(item_->target_bounds());
-  scaled_bounds.Inset(-scaled_bounds.width() * kDragWindowScale,
-                      -scaled_bounds.height() * kDragWindowScale);
-  item_->SetBounds(scaled_bounds, OVERVIEW_ANIMATION_LAY_OUT_SELECTOR_ITEMS);
+  DCHECK(ShouldAllowSplitView());
+
+  item_->ScaleUpSelectedItem(
+      OVERVIEW_ANIMATION_LAYOUT_OVERVIEW_ITEMS_IN_OVERVIEW);
 
   did_move_ = true;
   current_drag_behavior_ = DragBehavior::kDragToSnap;
-  window_selector_->SetSplitViewDragIndicatorsIndicatorState(
-      split_view_controller_->CanSnap(item_->GetWindow())
-          ? IndicatorState::kDragArea
-          : IndicatorState::kCannotSnap,
+  overview_session_->SetSplitViewDragIndicatorsIndicatorState(
+      CanSnapInSplitview(item_->GetWindow()) ? IndicatorState::kDragArea
+                                             : IndicatorState::kCannotSnap,
       location_in_screen);
 }
 
@@ -184,11 +201,17 @@ void OverviewWindowDragController::Fling(const gfx::Point& location_in_screen,
   if (current_drag_behavior_ == DragBehavior::kDragToClose ||
       current_drag_behavior_ == DragBehavior::kUndefined) {
     if (std::abs(velocity_y) > kFlingToCloseVelocityThreshold) {
+      if (ShouldAllowSplitView()) {
+        split_view_controller_->OnWindowDragEnded(
+            item_->GetWindow(), snap_position_, location_in_screen);
+      }
+
       item_->AnimateAndCloseWindow(
           (location_in_screen - initial_event_location_).y() < 0);
       did_move_ = false;
       item_ = nullptr;
       current_drag_behavior_ = DragBehavior::kNoDrag;
+      UnpauseOcclusionTracker();
       return;
     }
   }
@@ -206,42 +229,47 @@ void OverviewWindowDragController::ActivateDraggedWindow() {
   // and the selected window cannot be snapped, exit splitview and activate
   // the selected window, and also exit the overview.
   SplitViewController::State split_state = split_view_controller_->state();
-  if (split_state == SplitViewController::NO_SNAP) {
-    window_selector_->SelectWindow(item_);
-  } else if (split_view_controller_->CanSnap(item_->GetWindow())) {
+  if (!ShouldAllowSplitView() || split_state == SplitViewController::NO_SNAP) {
+    overview_session_->SelectWindow(item_);
+  } else if (CanSnapInSplitview(item_->GetWindow())) {
     SnapWindow(split_state == SplitViewController::LEFT_SNAPPED
                    ? SplitViewController::RIGHT
                    : SplitViewController::LEFT);
   } else {
     split_view_controller_->EndSplitView();
-    window_selector_->SelectWindow(item_);
+    overview_session_->SelectWindow(item_);
     split_view_controller_->ShowAppCannotSnapToast();
   }
   current_drag_behavior_ = DragBehavior::kNoDrag;
+  UnpauseOcclusionTracker();
 }
 
 void OverviewWindowDragController::ResetGesture() {
-  window_selector_->PositionWindows(/*animate=*/true);
-  window_selector_->SetSplitViewDragIndicatorsIndicatorState(
-      IndicatorState::kNone, gfx::Point());
+  overview_session_->PositionWindows(/*animate=*/true);
+  if (ShouldAllowSplitView()) {
+    overview_session_->SetSplitViewDragIndicatorsIndicatorState(
+        IndicatorState::kNone, gfx::Point());
+  }
   // This function gets called after a long press release, which bypasses
   // CompleteDrag but stops dragging as well, so reset |item_|.
   item_ = nullptr;
   current_drag_behavior_ = DragBehavior::kNoDrag;
+  UnpauseOcclusionTracker();
 }
 
-void OverviewWindowDragController::ResetWindowSelector() {
-  window_selector_ = nullptr;
+void OverviewWindowDragController::ResetOverviewSession() {
+  overview_session_ = nullptr;
 }
 
-void OverviewWindowDragController::UpdateDragIndicatorsAndWindowGrid(
+void OverviewWindowDragController::UpdateDragIndicatorsAndOverviewGrid(
     const gfx::Point& location_in_screen) {
+  DCHECK(ShouldAllowSplitView());
   if (!ShouldUpdateDragIndicatorsOrSnap(location_in_screen))
     return;
 
   // Attempt to update the drag indicators and move the window grid only if the
   // window is snappable.
-  if (!split_view_controller_->CanSnap(item_->GetWindow())) {
+  if (!CanSnapInSplitview(item_->GetWindow())) {
     snap_position_ = SplitViewController::NONE;
     return;
   }
@@ -249,42 +277,28 @@ void OverviewWindowDragController::UpdateDragIndicatorsAndWindowGrid(
   SplitViewController::SnapPosition last_snap_position = snap_position_;
   snap_position_ = GetSnapPosition(location_in_screen);
 
-  // If there is already a snapped window in |snap_position_|, do not allow
-  // another window snap in the same position.
-  SplitViewController::State snapped_state = split_view_controller_->state();
-  if ((snap_position_ == SplitViewController::LEFT &&
-       snapped_state == SplitViewController::LEFT_SNAPPED) ||
-      (snap_position_ == SplitViewController::RIGHT &&
-       snapped_state == SplitViewController::RIGHT_SNAPPED)) {
-    snap_position_ = SplitViewController::NONE;
-    window_selector_->SetSplitViewDragIndicatorsIndicatorState(
-        IndicatorState::kNone, gfx::Point());
-    return;
-  }
-
   // If there is no current snapped window, update the window grid size if the
   // dragged window can be snapped if dropped.
-  if (snapped_state == SplitViewController::NO_SNAP &&
+  if (split_view_controller_->state() == SplitViewController::NO_SNAP &&
       snap_position_ != last_snap_position) {
     // Do not reposition the item that is currently being dragged.
-    window_selector_->SetBoundsForWindowGridsInScreenIgnoringWindow(
+    overview_session_->SetBoundsForOverviewGridsInScreenIgnoringWindow(
         GetGridBounds(snap_position_), item_);
   }
 
   // Show the cannot snap ui on the split view drag indicators if the window
   // cannot be snapped, otherwise show the drag ui.
   if (snap_position_ == SplitViewController::NONE) {
-    window_selector_->SetSplitViewDragIndicatorsIndicatorState(
-        split_view_controller_->CanSnap(item_->GetWindow())
-            ? IndicatorState::kDragArea
-            : IndicatorState::kCannotSnap,
+    overview_session_->SetSplitViewDragIndicatorsIndicatorState(
+        CanSnapInSplitview(item_->GetWindow()) ? IndicatorState::kDragArea
+                                               : IndicatorState::kCannotSnap,
         gfx::Point());
     return;
   }
 
   // Display the preview area on the split view drag indicators. The split
   // view drag indicators will calculate the preview area bounds.
-  window_selector_->SetSplitViewDragIndicatorsIndicatorState(
+  overview_session_->SetSplitViewDragIndicatorsIndicatorState(
       snap_position_ == SplitViewController::LEFT
           ? IndicatorState::kPreviewAreaLeft
           : IndicatorState::kPreviewAreaRight,
@@ -294,8 +308,7 @@ void OverviewWindowDragController::UpdateDragIndicatorsAndWindowGrid(
 bool OverviewWindowDragController::ShouldUpdateDragIndicatorsOrSnap(
     const gfx::Point& event_location) {
   auto snap_position = GetSnapPosition(event_location);
-  const bool inverted =
-      !split_view_controller_->IsCurrentScreenOrientationPrimary();
+  const bool inverted = !IsCurrentScreenOrientationPrimary();
   // Note: in some orientations SplitViewController::LEFT is not physically on
   // the left/top.
   const bool on_the_left_or_top =
@@ -303,8 +316,7 @@ bool OverviewWindowDragController::ShouldUpdateDragIndicatorsOrSnap(
       (inverted && snap_position == SplitViewController::RIGHT);
 
   // Snap the window if it is less than |kDistanceFromEdgeDp| from the edge.
-  const bool landscape =
-      split_view_controller_->IsCurrentScreenOrientationLandscape();
+  const bool landscape = IsCurrentScreenOrientationLandscape();
   gfx::Rect area(
       screen_util::GetDisplayWorkAreaBoundsInParent(item_->GetWindow()));
   ::wm::ConvertRectToScreen(item_->GetWindow()->GetRootWindow(), &area);
@@ -353,53 +365,62 @@ bool OverviewWindowDragController::ShouldUpdateDragIndicatorsOrSnap(
 SplitViewController::SnapPosition OverviewWindowDragController::GetSnapPosition(
     const gfx::Point& location_in_screen) const {
   DCHECK(item_);
+  DCHECK(ShouldAllowSplitView());
   gfx::Rect area(
       screen_util::GetDisplayWorkAreaBoundsInParent(item_->GetWindow()));
   ::wm::ConvertRectToScreen(item_->GetWindow()->GetRootWindow(), &area);
 
-  switch (split_view_controller_->GetCurrentScreenOrientation()) {
-    case OrientationLockType::kLandscapePrimary:
-    case OrientationLockType::kLandscapeSecondary: {
-      // The window can be snapped if it reaches close enough to the screen
-      // edge of the screen (on primary axis). The edge insets are a fixed ratio
-      // of the screen plus some padding. This matches the drag indicators ui.
-      const int screen_edge_inset_for_drag =
-          area.width() * kHighlightScreenPrimaryAxisRatio +
-          kHighlightScreenEdgePaddingDp;
-      area.Inset(screen_edge_inset_for_drag, 0);
-      if (location_in_screen.x() <= area.x()) {
-        return split_view_controller_->IsCurrentScreenOrientationPrimary()
-                   ? SplitViewController::LEFT
-                   : SplitViewController::RIGHT;
-      }
-      if (location_in_screen.x() >= area.right() - 1) {
-        return split_view_controller_->IsCurrentScreenOrientationPrimary()
-                   ? SplitViewController::RIGHT
-                   : SplitViewController::LEFT;
-      }
-      return SplitViewController::NONE;
+  const bool is_landscape = IsCurrentScreenOrientationLandscape();
+  const bool is_primary = IsCurrentScreenOrientationPrimary();
+
+  // If split view mode is active at the moment, and dragging an overview window
+  // to snap it to a position that already has a snapped window in place, we
+  // should show the preview window as soon as the window past the split divider
+  // bar.
+  if (split_view_controller_->IsSplitViewModeActive()) {
+    const int position =
+        is_landscape ? location_in_screen.x() : location_in_screen.y();
+    SplitViewController::SnapPosition default_snap_position =
+        split_view_controller_->default_snap_position();
+    // If we're trying to snap to a position that already has a snapped window:
+    const bool re_snap =
+        is_primary == (position < split_view_controller_->divider_position() ==
+                       (default_snap_position == SplitViewController::LEFT));
+    if (re_snap)
+      return default_snap_position;
+  }
+
+  if (is_landscape) {
+    // The window can be snapped if it reaches close enough to the screen
+    // edge of the screen (on primary axis). The edge insets are a fixed ratio
+    // of the screen plus some padding. This matches the drag indicators ui.
+    const int screen_edge_inset_for_drag =
+        area.width() * kHighlightScreenPrimaryAxisRatio +
+        kHighlightScreenEdgePaddingDp;
+    area.Inset(screen_edge_inset_for_drag, 0);
+    if (location_in_screen.x() <= area.x()) {
+      return is_primary ? SplitViewController::LEFT
+                        : SplitViewController::RIGHT;
     }
-    case OrientationLockType::kPortraitPrimary:
-    case OrientationLockType::kPortraitSecondary: {
-      const int screen_edge_inset_for_drag =
-          area.height() * kHighlightScreenPrimaryAxisRatio +
-          kHighlightScreenEdgePaddingDp;
-      area.Inset(0, screen_edge_inset_for_drag);
-      if (location_in_screen.y() <= area.y()) {
-        return split_view_controller_->IsCurrentScreenOrientationPrimary()
-                   ? SplitViewController::LEFT
-                   : SplitViewController::RIGHT;
-      }
-      if (location_in_screen.y() >= area.bottom() - 1) {
-        return split_view_controller_->IsCurrentScreenOrientationPrimary()
-                   ? SplitViewController::RIGHT
-                   : SplitViewController::LEFT;
-      }
-      return SplitViewController::NONE;
+    if (location_in_screen.x() >= area.right() - 1) {
+      return is_primary ? SplitViewController::RIGHT
+                        : SplitViewController::LEFT;
     }
-    default:
-      NOTREACHED();
-      return SplitViewController::NONE;
+    return SplitViewController::NONE;
+  } else {
+    const int screen_edge_inset_for_drag =
+        area.height() * kHighlightScreenPrimaryAxisRatio +
+        kHighlightScreenEdgePaddingDp;
+    area.Inset(0, screen_edge_inset_for_drag);
+    if (location_in_screen.y() <= area.y()) {
+      return is_primary ? SplitViewController::LEFT
+                        : SplitViewController::RIGHT;
+    }
+    if (location_in_screen.y() >= area.bottom() - 1) {
+      return is_primary ? SplitViewController::RIGHT
+                        : SplitViewController::LEFT;
+    }
+    return SplitViewController::NONE;
   }
 }
 
@@ -408,8 +429,9 @@ gfx::Rect OverviewWindowDragController::GetGridBounds(
   aura::Window* pending_snapped_window = item_->GetWindow();
   switch (snap_position) {
     case SplitViewController::NONE:
-      return gfx::Rect(split_view_controller_->GetDisplayWorkAreaBoundsInParent(
-          pending_snapped_window));
+      return gfx::Rect(
+          screen_util::GetDisplayWorkAreaBoundsInParentForDefaultContainer(
+              pending_snapped_window));
     case SplitViewController::LEFT:
       return split_view_controller_->GetSnappedWindowBoundsInScreen(
           pending_snapped_window, SplitViewController::RIGHT);
