@@ -9,12 +9,16 @@
 #include "ash/ash_export.h"
 #include "ash/display/screen_orientation_controller.h"
 #include "ash/public/interfaces/split_view.mojom.h"
+#include "ash/session/session_observer.h"
 #include "ash/shell_observer.h"
+#include "ash/wm/overview/overview_observer.h"
+#include "ash/wm/tablet_mode/tablet_mode_controller.h"
 #include "ash/wm/tablet_mode/tablet_mode_observer.h"
 #include "ash/wm/window_state_observer.h"
 #include "base/containers/flat_map.h"
 #include "base/macros.h"
 #include "base/observer_list.h"
+#include "base/scoped_observer.h"
 #include "base/time/time.h"
 #include "mojo/public/cpp/bindings/binding_set.h"
 #include "mojo/public/cpp/bindings/interface_ptr_set.h"
@@ -29,11 +33,10 @@ class Layer;
 }  // namespace ui
 
 namespace ash {
-
+class OverviewSession;
 class SplitViewControllerTest;
 class SplitViewDivider;
-class SplitViewWindowSelectorTest;
-class OverviewWindowAnimationObserver;
+class SplitViewOverviewSessionTest;
 
 // The controller for the split view. It snaps a window to left/right side of
 // the screen. It also observes the two snapped windows and decides when to exit
@@ -44,9 +47,11 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
                                        public ash::wm::WindowStateObserver,
                                        public ::wm::ActivationChangeObserver,
                                        public ShellObserver,
+                                       public OverviewObserver,
                                        public display::DisplayObserver,
                                        public TabletModeObserver,
-                                       public AccessibilityObserver {
+                                       public AccessibilityObserver,
+                                       public SessionObserver {
  public:
   enum State { NO_SNAP, LEFT_SNAPPED, RIGHT_SNAPPED, BOTH_SNAPPED };
 
@@ -58,6 +63,16 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // top of the screen.
   enum SnapPosition { NONE, LEFT, RIGHT };
 
+  // Why splitview was ended. For now, all reasons will be kNormal except when
+  // the home launcher button is pressed, an unsnappable window just got
+  // activated, or the active user session changed.
+  enum class EndReason {
+    kNormal = 0,
+    kHomeLauncherPressed,
+    kUnsnappableWindowActivated,
+    kActiveUserChanged,
+  };
+
   class Observer {
    public:
     // Called when split view state changed from |previous_state| to |state|.
@@ -72,29 +87,12 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   SplitViewController();
   ~SplitViewController() override;
 
-  // Returns true if split view mode is supported. Currently the split view
-  // mode is only supported in tablet mode.
-  static bool ShouldAllowSplitView();
-
   // Binds the mojom::SplitViewController interface to this object.
   void BindRequest(mojom::SplitViewControllerRequest request);
 
-  // Returns true if |window| can be activated and snapped.
-  bool CanSnap(aura::Window* window);
-
   // Returns true if split view mode is active.
   bool IsSplitViewModeActive() const;
 
-  OrientationLockType GetCurrentScreenOrientation() const;
-
-  // Returns true if |screen_orientation_| is a landscape orientation.
-  bool IsCurrentScreenOrientationLandscape() const;
-
-  // Returns true if |screen_orientation_| is a primary orientation. Note,
-  // |left_window_| should be placed on the left or top side if the screen is
-  // primary orientation.
-  bool IsCurrentScreenOrientationPrimary() const;
-
   // Snaps window to left/right. It will try to remove |window| from the
   // overview window grid first before snapping it if |window| is currently
   // showing in the overview window grid.
@@ -119,8 +117,6 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
                                            SnapPosition snap_position);
   gfx::Rect GetSnappedWindowBoundsInScreen(aura::Window* window,
                                            SnapPosition snap_position);
-  gfx::Rect GetDisplayWorkAreaBoundsInParent(aura::Window* window) const;
-  gfx::Rect GetDisplayWorkAreaBoundsInScreen(aura::Window* window) const;
 
   // Gets the desired snapped window bounds accoridng to the snap state
   // |snap_state| and the divider pistion |divider_position_|.
@@ -139,9 +135,13 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   void ShowAppCannotSnapToast();
 
   // Ends the split view mode.
-  void EndSplitView();
+  void EndSplitView(EndReason end_reason = EndReason::kNormal);
 
-  // Called when a window's tab(s) start/end being dragging around.
+  // Returns true if |window| is a snapped window in splitview.
+  bool IsWindowInSplitView(const aura::Window* window) const;
+
+  // Called when a window (either it's browser window or an app window) start/
+  // end being dragged.
   void OnWindowDragStarted(aura::Window* dragged_window);
   void OnWindowDragEnded(aura::Window* dragged_window,
                          SnapPosition desired_snap_position,
@@ -150,6 +150,8 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   void AddObserver(Observer* observer);
   void RemoveObserver(Observer* observer);
 
+  void FlushForTesting();
+
   // mojom::SplitViewObserver:
   void AddObserver(mojom::SplitViewObserverPtr observer) override;
 
@@ -170,19 +172,27 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
                          aura::Window* lost_active) override;
 
   // ShellObserver:
+  void OnPinnedStateChanged(aura::Window* pinned_window) override;
+
+  // OverviewObserver:
   void OnOverviewModeStarting() override;
-  void OnOverviewModeEnding() override;
+  void OnOverviewModeEnding(OverviewSession* overview_session) override;
 
   // display::DisplayObserver:
   void OnDisplayMetricsChanged(const display::Display& display,
                                uint32_t metrics) override;
 
   // TabletModeObserver:
+  void OnTabletModeStarted() override;
   void OnTabletModeEnding() override;
+  void OnTabletControllerDestroyed() override;
 
   // AccessibilityObserver:
   void OnAccessibilityStatusChanged() override;
 
+  // SessionObserver:
+  void OnActiveUserSessionChanged(const AccountId& account_id) override;
+
   aura::Window* left_window() { return left_window_; }
   aura::Window* right_window() { return right_window_; }
   int divider_position() const { return divider_position_; }
@@ -190,14 +200,11 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   SnapPosition default_snap_position() const { return default_snap_position_; }
   SplitViewDivider* split_view_divider() { return split_view_divider_.get(); }
   bool is_resizing() const { return is_resizing_; }
-  base::WeakPtr<OverviewWindowAnimationObserver>
-  snapped_window_animation_observer() {
-    return snapped_window_animation_observer_;
-  }
+  EndReason end_reason() const { return end_reason_; }
 
  private:
   friend class SplitViewControllerTest;
-  friend class SplitViewWindowSelectorTest;
+  friend class SplitViewOverviewSessionTest;
   class TabDraggedWindowObserver;
 
   // Start observing |window|.
@@ -256,8 +263,9 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // needs to be activated. Returns nullptr if there is no such window.
   aura::Window* GetActiveWindowAfterResizingUponExit();
 
-  // Returns the maximum value of the |divider_position_|. It should always be
-  // the length of the longer side of the current display's work area bounds.
+  // Returns the maximum value of the |divider_position_|. It is the width of
+  // the current display's work area bounds in landscape orientation, or height
+  // of the current display's work area bounds in portrait orientation.
   int GetDividerEndPosition();
 
   // Called after a to-be-snapped window |window| got snapped. It updates the
@@ -287,7 +295,8 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // moved to the positions in |kFixedPositionRatios|. Whether the divider can
   // be moved to |kOneThirdPositionRatio| or |kTwoThirdPositionRatio| depends
   // on the minimum size of current snapped windows.
-  void GetDividerOptionalPositionRatios(std::vector<float>* positionRatios);
+  void GetDividerOptionalPositionRatios(
+      std::vector<float>* out_position_ratios);
 
   // Gets the expected window component depending on current screen orientation
   // for resizing purpose.
@@ -310,19 +319,26 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // translation to the snapped window to make it visually be placed outside of
   // the workspace area.
   void SetWindowsTransformDuringResizing();
+
   // Restore the snapped windows transform to identity transform after resizing.
   void RestoreWindowsTransformAfterResizing();
 
-  // Set |transform| for |window| and its transient descendants.
-  void SetTransform(aura::Window* window, const gfx::Transform& transform);
+  // Animates to |target_transform| for |window| and its transient descendants.
+  // |window| will be applied |start_transform| first and then animate to
+  // |target_transform|. Note |start_transform| and |end_transform| are for
+  // |window| and need to be adjusted for its transient child windows.
+  void SetTransformWithAnimation(aura::Window* window,
+                                 const gfx::Transform& start_transform,
+                                 const gfx::Transform& target_transform);
 
   // Removes the window item that contains |window| from the overview window
   // grid if |window| is currently showing in overview window grid. It should be
   // called before trying to snap the window.
   void RemoveWindowFromOverviewIfApplicable(aura::Window* window);
 
-  // Returns the window that is currently snapped at |snap_position|.
-  aura::Window* GetSnappedWindowAt(SnapPosition snap_position);
+  // Updates the |snapping_window_transformed_bounds_map_| on |window|. It
+  // should be called before trying to snap the window.
+  void UpdateSnappingWindowTransformedBounds(aura::Window* window);
 
   // Inserts |window| into overview window grid if overview mode is active. Do
   // nothing if overview mode is inactive at the moment.
@@ -336,6 +352,21 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // snapped windows.
   void FinishWindowResizing(aura::Window* window);
 
+  // Called by OnWindowDragEnded to do the actual work of finishing the window
+  // dragging. If |is_being_destroyed| equals true, the dragged window is to be
+  // destroyed, and SplitViewController should not try to put it in splitview.
+  void EndWindowDragImpl(aura::Window* window,
+                         bool is_being_destroyed,
+                         SnapPosition desired_snap_position,
+                         const gfx::Point& last_location_in_screen);
+
+  // Gets the snap position of |window| according to last mouse/gesture event
+  // location on |window|. Used when |desired_snap_position_| was NONE but
+  // SplitViewController needs to snap the |window| after dragging.
+  SplitViewController::SnapPosition GetSnapPosition(
+      aura::Window* window,
+      const gfx::Point& last_location_in_screen);
+
   // Bindings for the SplitViewController interface.
   mojo::BindingSet<mojom::SplitViewController> bindings_;
 
@@ -357,8 +388,8 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // The window observer that obseves the tab-dragged window.
   std::unique_ptr<TabDraggedWindowObserver> dragged_window_observer_;
 
-  // The distance between the origin of the divider and the origin of the screen
-  // in screen coordinates.
+  // The distance between the origin of the divider and the origin of the
+  // current display's work area in screen coordinates.
   //     |<---     divider_position_    --->|
   //     ----------------------------------------------------------
   //     |                                  | |                    |
@@ -367,6 +398,12 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   //     ----------------------------------------------------------
   int divider_position_ = -1;
 
+  // The closest position ratio of divider among kFixedPositionRatios,
+  // kOneThirdPositionRatio and kTwoThirdPositionRatio based on current
+  // |divider_position_|. Used to update |divider_position_| on work area
+  // changes.
+  float divider_closest_ratio_ = 0.f;
+
   // The location of the previous mouse/gesture event in screen coordinates.
   gfx::Point previous_event_location_;
 
@@ -385,20 +422,20 @@ class ASH_EXPORT SplitViewController : public mojom::SplitViewController,
   // If the divider is currently being dragging.
   bool is_resizing_ = false;
 
+  // Stores the reason which cause splitview to end.
+  EndReason end_reason_ = EndReason::kNormal;
+
   // The time when splitview starts. Used for metric collection purpose.
   base::Time splitview_start_time_;
 
-  // The map from a to-be-snapped window to its overview item's bounds if the
-  // window comes from the overview.
-  base::flat_map<aura::Window*, gfx::Rect> overview_window_item_bounds_map_;
+  // The map from a to-be-snapped window to its transformed bounds.
+  base::flat_map<aura::Window*, gfx::Rect>
+      snapping_window_transformed_bounds_map_;
 
-  // Weak ptr to the observer that observes the snapped window's transform
-  // animaiton if it comes from the overview.
-  base::WeakPtr<OverviewWindowAnimationObserver>
-      snapped_window_animation_observer_ = nullptr;
-
-  base::ObserverList<Observer> observers_;
+  base::ObserverList<Observer>::Unchecked observers_;
   mojo::InterfacePtrSet<mojom::SplitViewObserver> mojo_observers_;
+  ScopedObserver<TabletModeController, TabletModeObserver>
+      tablet_mode_observer_{this};
 
   DISALLOW_COPY_AND_ASSIGN(SplitViewController);
 };
