@@ -6,6 +6,7 @@
 
 #include "ash/shell.h"
 #include "ash/wm/window_state.h"
+#include "base/bind.h"
 #include "base/run_loop.h"
 #include "ui/aura/client/cursor_client.h"
 #include "ui/aura/env.h"
@@ -51,9 +52,8 @@ bool ToplevelWindowEventHandler::AttemptToStartDrag(
                                       ? ::wm::WINDOW_MOVE_SOURCE_TOUCH
                                       : ::wm::WINDOW_MOVE_SOURCE_MOUSE;
   if (gesture_target) {
-    ui::GestureRecognizer::Get()->TransferEventsTo(
-        gesture_target, window,
-        ui::GestureRecognizer::ShouldCancelTouches::DontCancel);
+    window->env()->gesture_recognizer()->TransferEventsTo(
+        gesture_target, window, ui::TransferTouchesBehavior::kDontCancel);
   }
   return wm_toplevel_window_event_handler_.AttemptToStartDrag(
       window, point_in_parent, window_component, source,
@@ -69,10 +69,11 @@ bool ToplevelWindowEventHandler::AttemptToStartDrag(
   DCHECK(root_window);
   gfx::Point drag_location;
   if (move_source == ::wm::WINDOW_MOVE_SOURCE_TOUCH &&
-      aura::Env::GetInstance()->is_touch_down()) {
+      Shell::Get()->aura_env()->is_touch_down()) {
     gfx::PointF drag_location_f;
-    bool has_point = ui::GestureRecognizer::Get()->GetLastTouchPointForTarget(
-        source, &drag_location_f);
+    bool has_point =
+        source->env()->gesture_recognizer()->GetLastTouchPointForTarget(
+            source, &drag_location_f);
     drag_location = gfx::ToFlooredPoint(drag_location_f);
     DCHECK(has_point);
   } else {
