@@ -8,15 +8,18 @@
 #include "ash/root_window_controller.h"
 #include "ash/shell.h"
 #include "ash/test/ash_test_base.h"
+#include "ash/window_factory.h"
 #include "ash/wm/resize_shadow.h"
 #include "ash/wm/resize_shadow_controller.h"
 #include "ash/wm/window_state.h"
 #include "ash/wm/window_util.h"
 #include "ash/wm/wm_event.h"
 #include "ash/wm/workspace_controller.h"
+#include "base/bind.h"
 #include "base/compiler_specific.h"
+#include "base/run_loop.h"
 #include "base/threading/thread_task_runner_handle.h"
-#include "services/ui/public/interfaces/window_manager_constants.mojom.h"
+#include "services/ws/public/mojom/window_tree_constants.mojom.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/client/capture_client.h"
@@ -60,8 +63,9 @@ class ToplevelWindowEventHandlerTest : public AshTestBase {
  protected:
   aura::Window* CreateWindow(int hittest_code) {
     TestWindowDelegate* d1 = new TestWindowDelegate(hittest_code);
-    aura::Window* w1 = new aura::Window(d1);
-    w1->SetType(aura::client::WINDOW_TYPE_NORMAL);
+    aura::Window* w1 =
+        window_factory::NewWindow(d1, aura::client::WINDOW_TYPE_NORMAL)
+            .release();
     w1->set_id(1);
     w1->Init(ui::LAYER_TEXTURED);
     aura::Window* parent = Shell::GetContainer(Shell::GetPrimaryRootWindow(),
@@ -134,8 +138,9 @@ TEST_F(ToplevelWindowEventHandlerTest, WindowPositionAutoManagement) {
       ::wm::GetWindowMoveClient(w1->GetRootWindow());
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
-      base::Bind(&ContinueAndCompleteDrag, base::Unretained(&generator),
-                 base::Unretained(window_state), base::Unretained(w1.get())));
+      base::BindOnce(&ContinueAndCompleteDrag, base::Unretained(&generator),
+                     base::Unretained(window_state),
+                     base::Unretained(w1.get())));
   EXPECT_EQ(::wm::MOVE_SUCCESSFUL,
             move_client->RunMoveLoop(w1.get(), gfx::Vector2d(100, 100),
                                      ::wm::WINDOW_MOVE_SOURCE_MOUSE));
@@ -152,8 +157,9 @@ TEST_F(ToplevelWindowEventHandlerTest, WindowPositionAutoManagement) {
   generator.PressLeftButton();
   base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
-      base::Bind(&ContinueAndCompleteDrag, base::Unretained(&generator),
-                 base::Unretained(window_state), base::Unretained(w1.get())));
+      base::BindOnce(&ContinueAndCompleteDrag, base::Unretained(&generator),
+                     base::Unretained(window_state),
+                     base::Unretained(w1.get())));
   EXPECT_EQ(::wm::MOVE_SUCCESSFUL,
             move_client->RunMoveLoop(w1.get(), gfx::Vector2d(100, 100),
                                      ::wm::WINDOW_MOVE_SOURCE_MOUSE));
@@ -459,9 +465,9 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDrag) {
   gfx::Rect old_bounds = target->bounds();
   gfx::Point location(5, 5);
   target->SetProperty(aura::client::kResizeBehaviorKey,
-                      ui::mojom::kResizeBehaviorCanResize |
-                          ui::mojom::kResizeBehaviorCanMaximize |
-                          ui::mojom::kResizeBehaviorCanMinimize);
+                      ws::mojom::kResizeBehaviorCanResize |
+                          ws::mojom::kResizeBehaviorCanMaximize |
+                          ws::mojom::kResizeBehaviorCanMinimize);
 
   gfx::Point end = location;
 
@@ -469,7 +475,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDrag) {
   end.Offset(100, 0);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Verify that the window has moved after the gesture.
   EXPECT_NE(old_bounds.ToString(), target->bounds().ToString());
@@ -483,7 +489,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDrag) {
   end.Offset(-100, 0);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_NE(old_bounds.ToString(), target->bounds().ToString());
   EXPECT_EQ(mojom::WindowStateType::LEFT_SNAPPED, window_state->GetStateType());
@@ -498,7 +504,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDrag) {
   end.Offset(0, -100);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_NE(old_bounds.ToString(), target->bounds().ToString());
   EXPECT_TRUE(window_state->IsMaximized());
@@ -513,7 +519,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDrag) {
   end.Offset(0, 100);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
   EXPECT_NE(old_bounds.ToString(), target->bounds().ToString());
   EXPECT_TRUE(window_state->IsMinimized());
   EXPECT_TRUE(window_state->unminimize_to_restore_bounds());
@@ -521,6 +527,33 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDrag) {
             window_state->GetRestoreBoundsInScreen().ToString());
 }
 
+// Verifies that window dragged by touch-gestures to the edge of display
+// will not lead to system crash (see https://crbug.com/917060).
+TEST_F(ToplevelWindowEventHandlerTest, GestureDragMultiDisplays) {
+  UpdateDisplay("800x600, 800x600");
+  std::unique_ptr<aura::Window> target(CreateTestWindowInShellWithDelegate(
+      new TestWindowDelegate(HTCAPTION), 0, gfx::Rect(0, 0, 100, 100)));
+  wm::WindowState* window_state = wm::GetWindowState(target.get());
+  ui::test::EventGenerator generator(Shell::GetPrimaryRootWindow(),
+                                     target.get());
+  gfx::Rect old_bounds = target->bounds();
+  gfx::Point location(5, 5);
+  gfx::Point end = location;
+
+  // On real device, gesture event's location may not be accurate. For example,
+  // when window is dragged by touch-gestures to the edge of display, it may
+  // create gesture events with location out of the display bounds. Let |end| be
+  // out of the primary display's bounds to emulate this situation.
+  end.Offset(800, 0);
+  generator.GestureScrollSequence(location, end,
+                                  base::TimeDelta::FromMilliseconds(5), 10);
+
+  // Verify that the window has moved after the gesture.
+  EXPECT_NE(old_bounds.ToString(), target->bounds().ToString());
+  EXPECT_EQ(mojom::WindowStateType::RIGHT_SNAPPED,
+            window_state->GetStateType());
+}
+
 // Tests that a gesture cannot minimize an unminimizeable window.
 TEST_F(ToplevelWindowEventHandlerTest,
        GestureAttemptMinimizeUnminimizeableWindow) {
@@ -529,13 +562,13 @@ TEST_F(ToplevelWindowEventHandlerTest,
                                      target.get());
   gfx::Point location(5, 5);
   target->SetProperty(aura::client::kResizeBehaviorKey,
-                      ui::mojom::kResizeBehaviorCanMaximize);
+                      ws::mojom::kResizeBehaviorCanMaximize);
 
   gfx::Point end = location;
   end.Offset(0, 100);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(wm::GetWindowState(target.get())->IsMinimized());
 }
 
@@ -694,7 +727,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDragToRestore) {
   end.Offset(0, 100);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
   EXPECT_NE(old_bounds.ToString(), window->bounds().ToString());
   EXPECT_TRUE(window_state->IsMinimized());
   EXPECT_TRUE(window_state->unminimize_to_restore_bounds());
@@ -718,9 +751,9 @@ TEST_F(ToplevelWindowEventHandlerTest, EasyResizerUsedForTopLevel) {
   // Make |w1| resizable to allow touch events to go to it (and not |w2|) thanks
   // to EasyResizeWindowTargeter.
   w1->SetProperty(aura::client::kResizeBehaviorKey,
-                  ui::mojom::kResizeBehaviorCanMaximize |
-                      ui::mojom::kResizeBehaviorCanMinimize |
-                      ui::mojom::kResizeBehaviorCanResize);
+                  ws::mojom::kResizeBehaviorCanMaximize |
+                      ws::mojom::kResizeBehaviorCanMinimize |
+                      ws::mojom::kResizeBehaviorCanResize);
   // Clicking a point within |w2| but close to |w1| should not activate |w2|.
   const gfx::Point touch_point(105, 105);
   generator.MoveTouch(touch_point);
@@ -730,8 +763,8 @@ TEST_F(ToplevelWindowEventHandlerTest, EasyResizerUsedForTopLevel) {
   // Make |w1| not resizable to allow touch events to go to |w2| even when close
   // to |w1|.
   w1->SetProperty(aura::client::kResizeBehaviorKey,
-                  ui::mojom::kResizeBehaviorCanMaximize |
-                      ui::mojom::kResizeBehaviorCanMinimize);
+                  ws::mojom::kResizeBehaviorCanMaximize |
+                      ws::mojom::kResizeBehaviorCanMinimize);
   // Clicking a point within |w2| should activate that window.
   generator.PressMoveAndReleaseTouchTo(touch_point);
   EXPECT_TRUE(wm::IsActiveWindow(w2.get()));
@@ -751,8 +784,8 @@ TEST_F(ToplevelWindowEventHandlerTest, EasyResizerUsedForTransient) {
   // Make |w11| non-resizable to avoid touch events inside its transient parent
   // |w1| from going to |w11| because of EasyResizeWindowTargeter.
   w11->SetProperty(aura::client::kResizeBehaviorKey,
-                   ui::mojom::kResizeBehaviorCanMaximize |
-                       ui::mojom::kResizeBehaviorCanMinimize);
+                   ws::mojom::kResizeBehaviorCanMaximize |
+                       ws::mojom::kResizeBehaviorCanMinimize);
   // Clicking a point within w1 should activate that window.
   generator.PressMoveAndReleaseTouchTo(gfx::Point(10, 10));
   EXPECT_TRUE(wm::IsActiveWindow(w1.get()));
@@ -761,9 +794,9 @@ TEST_F(ToplevelWindowEventHandlerTest, EasyResizerUsedForTransient) {
   // |w1| that are close to |w11| border to go to |w11| thanks to
   // EasyResizeWindowTargeter.
   w11->SetProperty(aura::client::kResizeBehaviorKey,
-                   ui::mojom::kResizeBehaviorCanMaximize |
-                       ui::mojom::kResizeBehaviorCanMinimize |
-                       ui::mojom::kResizeBehaviorCanResize);
+                   ws::mojom::kResizeBehaviorCanMaximize |
+                       ws::mojom::kResizeBehaviorCanMinimize |
+                       ws::mojom::kResizeBehaviorCanResize);
   // Clicking a point within |w1| but close to |w11| should activate |w11|.
   generator.PressMoveAndReleaseTouchTo(gfx::Point(10, 10));
   EXPECT_TRUE(wm::IsActiveWindow(w11.get()));
@@ -780,7 +813,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDragForUnresizableWindow) {
   gfx::Point location(5, 5);
 
   target->SetProperty(aura::client::kResizeBehaviorKey,
-                      ui::mojom::kResizeBehaviorNone);
+                      ws::mojom::kResizeBehaviorNone);
 
   gfx::Point end = location;
 
@@ -788,7 +821,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDragForUnresizableWindow) {
   end.Offset(100, 0);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Verify that the window has moved after the gesture.
   gfx::Rect expected_bounds(old_bounds);
@@ -805,7 +838,7 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDragForUnresizableWindow) {
   end.Offset(-100, 0);
   generator.GestureScrollSequence(location, end,
                                   base::TimeDelta::FromMilliseconds(5), 10);
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   // Verify that the window has moved after the gesture.
   expected_bounds = old_bounds;
@@ -854,7 +887,7 @@ TEST_F(ToplevelWindowEventHandlerTest, EscapeReverts) {
   generator.MoveMouseBy(10, 11);
 
   // Execute any scheduled draws so that pending mouse events are processed.
-  RunAllPendingInMessageLoop();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_EQ("0,0 110x111", target->bounds().ToString());
   generator.PressKey(ui::VKEY_ESCAPE, 0);
@@ -872,7 +905,7 @@ TEST_F(ToplevelWindowEventHandlerTest, MinimizeMaximizeCompletes) {
                                        target.get());
     generator.PressLeftButton();
     generator.MoveMouseBy(10, 11);
-    RunAllPendingInMessageLoop();
+    base::RunLoop().RunUntilIdle();
     EXPECT_EQ("10,11 100x100", target->bounds().ToString());
     wm::WindowState* window_state = wm::GetWindowState(target.get());
     window_state->Minimize();
@@ -880,7 +913,7 @@ TEST_F(ToplevelWindowEventHandlerTest, MinimizeMaximizeCompletes) {
 
     generator.PressLeftButton();
     generator.MoveMouseBy(10, 11);
-    RunAllPendingInMessageLoop();
+    base::RunLoop().RunUntilIdle();
     EXPECT_EQ("10,11 100x100", target->bounds().ToString());
   }
 
@@ -892,7 +925,7 @@ TEST_F(ToplevelWindowEventHandlerTest, MinimizeMaximizeCompletes) {
                                        target.get());
     generator.PressLeftButton();
     generator.MoveMouseBy(10, 11);
-    RunAllPendingInMessageLoop();
+    base::RunLoop().RunUntilIdle();
     EXPECT_EQ("10,11 100x100", target->bounds().ToString());
     wm::WindowState* window_state = wm::GetWindowState(target.get());
     window_state->Maximize();
@@ -900,7 +933,7 @@ TEST_F(ToplevelWindowEventHandlerTest, MinimizeMaximizeCompletes) {
 
     generator.PressLeftButton();
     generator.MoveMouseBy(10, 11);
-    RunAllPendingInMessageLoop();
+    base::RunLoop().RunUntilIdle();
     EXPECT_EQ("10,11 100x100", target->bounds().ToString());
   }
 }
@@ -952,9 +985,9 @@ TEST_F(ToplevelWindowEventHandlerTest, CaptureLossAfterMouseRelease) {
   ::wm::WindowMoveClient* move_client =
       ::wm::GetWindowMoveClient(window->GetRootWindow());
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE,
-      base::Bind(&SendMouseReleaseAndReleaseCapture,
-                 base::Unretained(&generator), base::Unretained(window.get())));
+      FROM_HERE, base::BindOnce(&SendMouseReleaseAndReleaseCapture,
+                                base::Unretained(&generator),
+                                base::Unretained(window.get())));
   EXPECT_EQ(::wm::MOVE_SUCCESSFUL,
             move_client->RunMoveLoop(window.get(), gfx::Vector2d(),
                                      ::wm::WINDOW_MOVE_SOURCE_MOUSE));
@@ -979,8 +1012,8 @@ TEST_F(ToplevelWindowEventHandlerTest, GestureDragCaptureLoss) {
   ::wm::WindowMoveClient* move_client =
       ::wm::GetWindowMoveClient(window->GetRootWindow());
   base::ThreadTaskRunnerHandle::Get()->PostTask(
-      FROM_HERE, base::Bind(&CheckHasCaptureAndReleaseCapture,
-                            base::Unretained(window.get())));
+      FROM_HERE, base::BindOnce(&CheckHasCaptureAndReleaseCapture,
+                                base::Unretained(window.get())));
   EXPECT_EQ(::wm::MOVE_SUCCESSFUL,
             move_client->RunMoveLoop(window.get(), gfx::Vector2d(),
                                      ::wm::WINDOW_MOVE_SOURCE_TOUCH));
