@@ -95,14 +95,6 @@ class WindowCycleControllerTest : public AshTestBase {
     shelf_view_test_->SetAnimationDuration(1);
   }
 
-  aura::Window* CreatePanelWindow() {
-    gfx::Rect rect(100, 100);
-    aura::Window* window = CreateTestWindowInShellWithDelegateAndType(
-        NULL, aura::client::WINDOW_TYPE_PANEL, 0, rect);
-    shelf_view_test_->RunMessageLoopUntilAnimationsDone();
-    return window;
-  }
-
   const aura::Window::Windows GetWindows(WindowCycleController* controller) {
     return controller->window_cycle_list()->windows();
   }
@@ -532,87 +524,6 @@ TEST_F(WindowCycleControllerTest, CyclePreservesMinimization) {
   EXPECT_TRUE(IsWindowMinimized(window1.get()));
 }
 
-// Tests cycles between panel and normal windows.
-TEST_F(WindowCycleControllerTest, CyclePanels) {
-  WindowCycleController* controller = Shell::Get()->window_cycle_controller();
-
-  std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
-  std::unique_ptr<aura::Window> panel0(CreatePanelWindow());
-  std::unique_ptr<aura::Window> panel1(CreatePanelWindow());
-  wm::ActivateWindow(window0.get());
-  wm::ActivateWindow(panel1.get());
-  wm::ActivateWindow(panel0.get());
-  EXPECT_TRUE(wm::IsActiveWindow(panel0.get()));
-
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->CompleteCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(panel1.get()));
-
-  // Cycling again should select the most recently used panel.
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->CompleteCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(panel0.get()));
-
-  // Cycling twice again should select the first window.
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->CompleteCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-}
-
-// Tests cycles between panel and normal windows.
-TEST_F(WindowCycleControllerTest, CyclePanelsDestroyed) {
-  WindowCycleController* controller = Shell::Get()->window_cycle_controller();
-
-  std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
-  std::unique_ptr<aura::Window> window1(CreateTestWindowInShellWithId(1));
-  std::unique_ptr<aura::Window> window2(CreateTestWindowInShellWithId(2));
-  std::unique_ptr<aura::Window> panel0(CreatePanelWindow());
-  std::unique_ptr<aura::Window> panel1(CreatePanelWindow());
-  wm::ActivateWindow(window2.get());
-  wm::ActivateWindow(panel1.get());
-  wm::ActivateWindow(panel0.get());
-  wm::ActivateWindow(window1.get());
-  wm::ActivateWindow(window0.get());
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-
-  // Cycling once highlights window2.
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  // All panels are destroyed.
-  panel0.reset();
-  panel1.reset();
-  // Cycling again should now select window2.
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->CompleteCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(window2.get()));
-}
-
-// Tests cycles between panel and normal windows.
-TEST_F(WindowCycleControllerTest, CycleMruPanelDestroyed) {
-  WindowCycleController* controller = Shell::Get()->window_cycle_controller();
-
-  std::unique_ptr<aura::Window> window0(CreateTestWindowInShellWithId(0));
-  std::unique_ptr<aura::Window> window1(CreateTestWindowInShellWithId(1));
-  std::unique_ptr<aura::Window> panel0(CreatePanelWindow());
-  std::unique_ptr<aura::Window> panel1(CreatePanelWindow());
-  wm::ActivateWindow(panel1.get());
-  wm::ActivateWindow(panel0.get());
-  wm::ActivateWindow(window1.get());
-  wm::ActivateWindow(window0.get());
-  EXPECT_TRUE(wm::IsActiveWindow(window0.get()));
-
-  // Cycling once highlights window2.
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-
-  // Panel 1 is the next item as the MRU panel, removing it should make panel 2
-  // the next window to be selected.
-  panel0.reset();
-  // Cycling again should now select panel1.
-  controller->HandleCycleWindow(WindowCycleController::FORWARD);
-  controller->CompleteCycling();
-  EXPECT_TRUE(wm::IsActiveWindow(panel1.get()));
-}
-
 // Tests that the tab key events are not sent to the window.
 TEST_F(WindowCycleControllerTest, TabKeyNotLeaked) {
   std::unique_ptr<Window> w0(CreateTestWindowInShellWithId(0));
@@ -620,15 +531,15 @@ TEST_F(WindowCycleControllerTest, TabKeyNotLeaked) {
   EventCounter event_count;
   w0->AddPreTargetHandler(&event_count);
   w1->AddPreTargetHandler(&event_count);
-  ui::test::EventGenerator& generator = GetEventGenerator();
+  ui::test::EventGenerator* generator = GetEventGenerator();
   wm::GetWindowState(w0.get())->Activate();
-  generator.PressKey(ui::VKEY_MENU, ui::EF_NONE);
+  generator->PressKey(ui::VKEY_MENU, ui::EF_NONE);
   EXPECT_EQ(1, event_count.GetKeyEventCountAndReset());
-  generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  generator->PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
   EXPECT_EQ(0, event_count.GetKeyEventCountAndReset());
-  generator.ReleaseKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  generator->ReleaseKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
   EXPECT_EQ(0, event_count.GetKeyEventCountAndReset());
-  generator.ReleaseKey(ui::VKEY_MENU, ui::EF_NONE);
+  generator->ReleaseKey(ui::VKEY_MENU, ui::EF_NONE);
   EXPECT_TRUE(wm::GetWindowState(w1.get())->IsActive());
   EXPECT_EQ(0, event_count.GetKeyEventCountAndReset());
 }
@@ -643,12 +554,12 @@ TEST_F(WindowCycleControllerTest, MouseEventsCaptured) {
   EventCounter event_count;
   w0->AddPreTargetHandler(&event_count);
   w1->SetTargetHandler(&event_count);
-  ui::test::EventGenerator& generator = GetEventGenerator();
+  ui::test::EventGenerator* generator = GetEventGenerator();
   wm::ActivateWindow(w0.get());
 
   // Events get through.
-  generator.MoveMouseToCenterOf(w0.get());
-  generator.ClickLeftButton();
+  generator->MoveMouseToCenterOf(w0.get());
+  generator->ClickLeftButton();
   EXPECT_LT(0, event_count.GetMouseEventCountAndReset());
 
   // Start cycling.
@@ -656,16 +567,16 @@ TEST_F(WindowCycleControllerTest, MouseEventsCaptured) {
   controller->HandleCycleWindow(WindowCycleController::FORWARD);
 
   // Most mouse events don't get through.
-  generator.PressLeftButton();
+  generator->PressLeftButton();
   EXPECT_EQ(0, event_count.GetMouseEventCountAndReset());
 
   // Although releases do.
-  generator.ReleaseLeftButton();
+  generator->ReleaseLeftButton();
   EXPECT_LT(0, event_count.GetMouseEventCountAndReset());
 
   // Stop cycling: once again, events get through.
   controller->CompleteCycling();
-  generator.ClickLeftButton();
+  generator->ClickLeftButton();
   EXPECT_LT(0, event_count.GetMouseEventCountAndReset());
 }
 
@@ -688,11 +599,11 @@ TEST_F(WindowCycleControllerTest, TabPastFullscreenWindow) {
   wm::GetWindowState(w0.get())->Activate();
   EXPECT_TRUE(wm::GetWindowState(w0.get())->IsActive());
 
-  ui::test::EventGenerator& generator = GetEventGenerator();
-  generator.PressKey(ui::VKEY_MENU, ui::EF_NONE);
+  ui::test::EventGenerator* generator = GetEventGenerator();
+  generator->PressKey(ui::VKEY_MENU, ui::EF_NONE);
 
-  generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
-  generator.ReleaseKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  generator->PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  generator->ReleaseKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
 
   // Because w0 and w1 are full-screen, the event should be passed to the
   // browser window to handle it (which if the browser doesn't handle it will
@@ -702,7 +613,7 @@ TEST_F(WindowCycleControllerTest, TabPastFullscreenWindow) {
   EventCounter event_count;
   w0->AddPreTargetHandler(&event_count);
   w1->AddPreTargetHandler(&event_count);
-  generator.PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
+  generator->PressKey(ui::VKEY_TAB, ui::EF_ALT_DOWN);
   EXPECT_EQ(1, event_count.GetKeyEventCountAndReset());
 }
 
