@@ -4,7 +4,7 @@
 
 #include "ash/wm/wm_toplevel_window_event_handler.h"
 
-#include "ash/public/cpp/config.h"
+#include "ash/public/cpp/app_types.h"
 #include "ash/shell.h"
 #include "ash/wm/resize_shadow_controller.h"
 #include "ash/wm/tablet_mode/tablet_mode_controller.h"
@@ -13,23 +13,86 @@
 #include "ash/wm/window_state_observer.h"
 #include "ash/wm/window_util.h"
 #include "ash/wm/wm_event.h"
+#include "ui/aura/client/aura_constants.h"
 #include "ui/aura/client/window_types.h"
+#include "ui/aura/env.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_delegate.h"
 #include "ui/aura/window_observer.h"
 #include "ui/base/hit_test.h"
 #include "ui/events/event.h"
-
-namespace {
-const double kMinHorizVelocityForWindowSwipe = 1100;
-const double kMinVertVelocityForWindowMinimize = 1000;
-}
+#include "ui/events/gestures/gesture_recognizer.h"
+#include "ui/views/widget/widget.h"
+#include "ui/wm/core/coordinate_conversion.h"
 
 namespace ash {
 namespace wm {
 
 namespace {
 
+// How many pixels are reserved for gesture events to start dragging the app
+// window from the top of the screen in tablet mode.
+constexpr int kDragStartTopEdgeInset = 8;
+
+// Returns the toplevel window that should be dragged for a gesture event that
+// occurs in the HTCLIENT area of a window. Returns null if there shouldn't be
+// special casing for this HTCLIENT area gesture. This is used to drag app
+// windows which are fullscreened/maximized in tablet mode from the top of the
+// screen, which don't have a window frame.
+aura::Window* GetTargetForClientAreaGesture(ui::GestureEvent* event,
+                                            aura::Window* target) {
+  if (event->type() != ui::ET_GESTURE_SCROLL_BEGIN)
+    return nullptr;
+
+  if (!Shell::Get()
+           ->tablet_mode_controller()
+           ->IsTabletModeWindowManagerEnabled()) {
+    return nullptr;
+  }
+
+  views::Widget* widget = views::Widget::GetTopLevelWidgetForNativeView(target);
+  if (!widget)
+    return nullptr;
+
+  aura::Window* toplevel = widget->GetNativeWindow();
+  wm::WindowState* window_state = wm::GetWindowState(toplevel);
+  if (!window_state ||
+      (!window_state->IsMaximized() && !window_state->IsFullscreen() &&
+       !window_state->IsSnapped())) {
+    return nullptr;
+  }
+
+  if (toplevel->GetProperty(aura::client::kAppType) ==
+      static_cast<int>(AppType::BROWSER)) {
+    return nullptr;
+  }
+
+  if (event->details().scroll_y_hint() < 0)
+    return nullptr;
+
+  const gfx::Point location_in_screen =
+      event->target()->GetScreenLocation(*event);
+  const gfx::Rect work_area_bounds =
+      display::Screen::GetScreen()
+          ->GetDisplayNearestWindow(static_cast<aura::Window*>(event->target()))
+          .work_area();
+
+  gfx::Rect hit_bounds_in_screen(work_area_bounds);
+  hit_bounds_in_screen.set_height(kDragStartTopEdgeInset);
+
+  // There may be a bezel sensor off screen logically above
+  // |hit_bounds_in_screen|. Handles the ET_GESTURE_SCROLL_BEGIN event
+  // triggered in the bezel area too.
+  bool in_bezel = location_in_screen.y() < hit_bounds_in_screen.y() &&
+                  location_in_screen.x() >= hit_bounds_in_screen.x() &&
+                  location_in_screen.x() < hit_bounds_in_screen.right();
+
+  if (hit_bounds_in_screen.Contains(location_in_screen) || in_bezel)
+    return toplevel;
+
+  return nullptr;
+}
+
 // Returns whether |window| can be moved via a two finger drag given
 // the hittest results of the two fingers.
 bool CanStartTwoFingerMove(aura::Window* window,
@@ -40,8 +103,8 @@ bool CanStartTwoFingerMove(aura::Window* window,
   // the tab strip is full and hitting the caption area is difficult. We check
   // the window type and the state type so that we do not steal touches from the
   // web contents.
-  if (!GetWindowState(window)->IsNormalOrSnapped() ||
-      window->type() != aura::client::WINDOW_TYPE_NORMAL) {
+  if (window->type() != aura::client::WINDOW_TYPE_NORMAL ||
+      !GetWindowState(window)->IsNormalOrSnapped()) {
     return false;
   }
   int component1_behavior =
@@ -60,11 +123,6 @@ bool CanStartOneFingerDrag(int window_component) {
 }
 
 void ShowResizeShadow(aura::Window* window, int component) {
-  if (Shell::GetAshConfig() == Config::MASH) {
-    // TODO: http://crbug.com/640773.
-    return;
-  }
-
   // Window resize in tablet mode is disabled (except in splitscreen).
   if (Shell::Get()
           ->tablet_mode_controller()
@@ -79,11 +137,6 @@ void ShowResizeShadow(aura::Window* window, int component) {
 }
 
 void HideResizeShadow(aura::Window* window) {
-  if (Shell::GetAshConfig() == Config::MASH) {
-    // TODO: http://crbug.com/640773.
-    return;
-  }
-
   ResizeShadowController* resize_shadow_controller =
       Shell::Get()->resize_shadow_controller();
   if (resize_shadow_controller)
@@ -172,12 +225,29 @@ void WmToplevelWindowEventHandler::ScopedWindowResizer::OnWindowDestroying(
 WmToplevelWindowEventHandler::WmToplevelWindowEventHandler()
     : first_finger_hittest_(HTNOWHERE) {
   Shell::Get()->window_tree_host_manager()->AddObserver(this);
+  display::Screen::GetScreen()->AddObserver(this);
 }
 
 WmToplevelWindowEventHandler::~WmToplevelWindowEventHandler() {
+  display::Screen::GetScreen()->RemoveObserver(this);
   Shell::Get()->window_tree_host_manager()->RemoveObserver(this);
 }
 
+void WmToplevelWindowEventHandler::OnDisplayMetricsChanged(
+    const display::Display& display,
+    uint32_t metrics) {
+  if (!window_resizer_ || !(metrics & DISPLAY_METRIC_ROTATION))
+    return;
+
+  display::Display current_display =
+      display::Screen::GetScreen()->GetDisplayNearestWindow(
+          window_resizer_->resizer()->GetTarget());
+  if (display.id() != current_display.id())
+    return;
+
+  RevertDrag();
+}
+
 void WmToplevelWindowEventHandler::OnKeyEvent(ui::KeyEvent* event) {
   if (window_resizer_.get() && event->type() == ui::ET_KEY_PRESSED &&
       event->key_code() == ui::VKEY_ESCAPE) {
@@ -228,23 +298,51 @@ void WmToplevelWindowEventHandler::OnMouseEvent(ui::MouseEvent* event,
 
 void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
                                                   aura::Window* target) {
+  DCHECK_EQ(event->target(), target);
+  int component = GetNonClientComponent(target, event->location());
+  gfx::Point event_location = event->location();
+
+  aura::Window* original_target = target;
+  bool client_area_drag = false;
+  if (component == HTCLIENT) {
+    // When dragging on a client area starts a gesture drag, |this| stops the
+    // propagation of the ET_GESTURE_SCROLL_BEGIN event. Subsequent gestures on
+    // the HTCLIENT area should also be stopped lest the client receive an
+    // ET_GESTURE_SCROLL_UPDATE without the ET_GESTURE_SCROLL_BEGIN.
+    if (in_gesture_drag_ && target != gesture_target_) {
+      event->StopPropagation();
+      return;
+    }
+
+    aura::Window* new_target = GetTargetForClientAreaGesture(event, target);
+
+    client_area_drag = !!new_target;
+    if (new_target && (target != new_target)) {
+      DCHECK_EQ(ui::ET_GESTURE_SCROLL_BEGIN, event->type());
+      aura::Window::ConvertPointToTarget(target, new_target, &event_location);
+
+      original_target->env()->gesture_recognizer()->TransferEventsTo(
+          original_target, new_target, ui::TransferTouchesBehavior::kCancel);
+      UpdateGestureTarget(new_target, event_location);
+      target = new_target;
+    }
+  }
+
   if (event->type() == ui::ET_GESTURE_END)
     UpdateGestureTarget(nullptr);
   else if (event->type() == ui::ET_GESTURE_BEGIN)
-    UpdateGestureTarget(target);
+    UpdateGestureTarget(target, event_location);
 
   if (event->handled())
     return;
   if (!target->delegate())
     return;
 
-  if (window_resizer_.get() && !in_gesture_drag_)
+  if (window_resizer_ && !in_gesture_drag_)
     return;
 
-  if (window_resizer_.get() &&
-      window_resizer_->resizer()->GetTarget() != target) {
+  if (window_resizer_ && window_resizer_->resizer()->GetTarget() != target)
     return;
-  }
 
   if (event->details().touch_points() > 2) {
     if (CompleteDrag(DragResult::SUCCESS))
@@ -254,7 +352,6 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
 
   switch (event->type()) {
     case ui::ET_GESTURE_TAP_DOWN: {
-      int component = GetNonClientComponent(target, event->location());
       if (!(WindowResizer::GetBoundsChangeForWindowComponent(component) &
             WindowResizer::kBoundsChange_Resizes))
         return;
@@ -264,9 +361,8 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
     case ui::ET_GESTURE_END: {
       HideResizeShadow(target);
 
-      if (window_resizer_.get() &&
-          (event->details().touch_points() == 1 ||
-           !CanStartOneFingerDrag(first_finger_hittest_))) {
+      if (window_resizer_ && (event->details().touch_points() == 1 ||
+                              !CanStartOneFingerDrag(first_finger_hittest_))) {
         CompleteDrag(DragResult::SUCCESS);
         event->StopPropagation();
       }
@@ -274,12 +370,11 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
     }
     case ui::ET_GESTURE_BEGIN: {
       if (event->details().touch_points() == 1) {
-        first_finger_touch_point_ = event->location();
+        first_finger_touch_point_ = event_location;
         aura::Window::ConvertPointToTarget(target, target->parent(),
                                            &first_finger_touch_point_);
-        first_finger_hittest_ =
-            GetNonClientComponent(target, event->location());
-      } else if (window_resizer_.get()) {
+        first_finger_hittest_ = component;
+      } else if (window_resizer_) {
         if (!window_resizer_->IsMove()) {
           // The transition from resizing with one finger to resizing with two
           // fingers causes unintended resizing because the location of
@@ -290,8 +385,7 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
           event->StopPropagation();
         }
       } else {
-        int second_finger_hittest =
-            GetNonClientComponent(target, event->location());
+        int second_finger_hittest = component;
         if (CanStartTwoFingerMove(target, first_finger_hittest_,
                                   second_finger_hittest)) {
           AttemptToStartDrag(target, first_finger_touch_point_, HTCAPTION,
@@ -309,10 +403,11 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
       // finger's position to the position in the middle of the two fingers.
       if (window_resizer_.get())
         return;
-      int component = GetNonClientComponent(target, event->location());
-      if (!CanStartOneFingerDrag(component))
+
+      if (!client_area_drag && !CanStartOneFingerDrag(component))
         return;
-      gfx::Point location_in_parent = event->location();
+
+      gfx::Point location_in_parent = event_location;
       aura::Window::ConvertPointToTarget(target, target->parent(),
                                          &location_in_parent);
       AttemptToStartDrag(target, location_in_parent, component,
@@ -328,10 +423,33 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
     return;
 
   switch (event->type()) {
-    case ui::ET_GESTURE_SCROLL_UPDATE:
+    case ui::ET_GESTURE_SCROLL_UPDATE: {
+      gfx::Rect bounds_in_screen = target->GetRootWindow()->GetBoundsInScreen();
+      gfx::Point screen_location = event->location();
+      ::wm::ConvertPointToScreen(target, &screen_location);
+
+      // It is physically not possible to move a touch pointer from one display
+      // to another, so constrain the bounds to the display. This is important,
+      // as it is possible for touch points to extend outside the bounds of the
+      // display (as happens with gestures on the bezel), and dragging via touch
+      // should not trigger moving to a new display.(see
+      // https://crbug.com/917060)
+      if (!bounds_in_screen.Contains(screen_location)) {
+        int x = std::max(
+            std::min(screen_location.x(), bounds_in_screen.right() - 1),
+            bounds_in_screen.x());
+        int y = std::max(
+            std::min(screen_location.y(), bounds_in_screen.bottom() - 1),
+            bounds_in_screen.y());
+        gfx::Point updated_location(x, y);
+        ::wm::ConvertPointFromScreen(target, &updated_location);
+        event->set_location(updated_location);
+      }
+
       HandleDrag(target, event);
       event->StopPropagation();
       return;
+    }
     case ui::ET_GESTURE_SCROLL_END:
       // We must complete the drag here instead of as a result of ET_GESTURE_END
       // because otherwise the drag will be reverted when EndMoveLoop() is
@@ -342,57 +460,9 @@ void WmToplevelWindowEventHandler::OnGestureEvent(ui::GestureEvent* event,
       event->StopPropagation();
       return;
     case ui::ET_SCROLL_FLING_START:
-      CompleteDrag(DragResult::SUCCESS);
-
-      // TODO(pkotwicz): Fix tests which inadvertently start flings and check
-      // window_resizer_->IsMove() instead of the hittest component at |event|'s
-      // location.
-      if (GetNonClientComponent(target, event->location()) != HTCAPTION ||
-          !GetWindowState(target)->IsNormalOrSnapped()) {
-        return;
-      }
-
-      if (event->details().velocity_y() > kMinVertVelocityForWindowMinimize) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::MINIMIZED);
-      } else if (event->details().velocity_y() <
-                 -kMinVertVelocityForWindowMinimize) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::MAXIMIZED);
-      } else if (event->details().velocity_x() >
-                 kMinHorizVelocityForWindowSwipe) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::RIGHT_SNAPPED);
-      } else if (event->details().velocity_x() <
-                 -kMinHorizVelocityForWindowSwipe) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::LEFT_SNAPPED);
-      }
-      event->StopPropagation();
-      return;
+      FALLTHROUGH;
     case ui::ET_GESTURE_SWIPE:
-      DCHECK_GT(event->details().touch_points(), 0);
-      if (event->details().touch_points() == 1)
-        return;
-      if (!GetWindowState(target)->IsNormalOrSnapped())
-        return;
-
-      CompleteDrag(DragResult::SUCCESS);
-
-      if (event->details().swipe_down()) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::MINIMIZED);
-      } else if (event->details().swipe_up()) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::MAXIMIZED);
-      } else if (event->details().swipe_right()) {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::RIGHT_SNAPPED);
-      } else {
-        SetWindowStateTypeFromGesture(target,
-                                      mojom::WindowStateType::LEFT_SNAPPED);
-      }
-      event->StopPropagation();
+      HandleFlingOrSwipe(event);
       return;
     default:
       return;
@@ -413,8 +483,12 @@ bool WmToplevelWindowEventHandler::AttemptToStartDrag(
   }
 
   end_closure_ = std::move(end_closure);
-  pre_drag_window_bounds_ = window->bounds();
   in_gesture_drag_ = (source == ::wm::WINDOW_MOVE_SOURCE_TOUCH);
+  // |gesture_target_| needs to be updated if the drag originated from a
+  // client (i.e. |this| never handled ET_GESTURE_EVENT_BEGIN).
+  if (in_gesture_drag_ && !gesture_target_)
+    UpdateGestureTarget(window);
+
   return true;
 }
 
@@ -558,44 +632,17 @@ void WmToplevelWindowEventHandler::HandleCaptureLost(ui::LocatedEvent* event) {
   }
 }
 
-void WmToplevelWindowEventHandler::SetWindowStateTypeFromGesture(
-    aura::Window* window,
-    mojom::WindowStateType new_state_type) {
-  wm::WindowState* window_state = GetWindowState(window);
-  // TODO(oshima): Move extra logic (set_unminimize_to_restore_bounds,
-  // SetRestoreBoundsInParent) that modifies the window state
-  // into WindowState.
-  switch (new_state_type) {
-    case mojom::WindowStateType::MINIMIZED:
-      if (window_state->CanMinimize()) {
-        window_state->Minimize();
-        window_state->set_unminimize_to_restore_bounds(true);
-        window_state->SetRestoreBoundsInParent(pre_drag_window_bounds_);
-      }
-      break;
-    case mojom::WindowStateType::MAXIMIZED:
-      if (window_state->CanMaximize()) {
-        window_state->SetRestoreBoundsInParent(pre_drag_window_bounds_);
-        window_state->Maximize();
-      }
-      break;
-    case mojom::WindowStateType::LEFT_SNAPPED:
-      if (window_state->CanSnap()) {
-        window_state->SetRestoreBoundsInParent(pre_drag_window_bounds_);
-        const wm::WMEvent event(wm::WM_EVENT_SNAP_LEFT);
-        window_state->OnWMEvent(&event);
-      }
-      break;
-    case mojom::WindowStateType::RIGHT_SNAPPED:
-      if (window_state->CanSnap()) {
-        window_state->SetRestoreBoundsInParent(pre_drag_window_bounds_);
-        const wm::WMEvent event(wm::WM_EVENT_SNAP_RIGHT);
-        window_state->OnWMEvent(&event);
-      }
-      break;
-    default:
-      NOTREACHED();
-  }
+void WmToplevelWindowEventHandler::HandleFlingOrSwipe(ui::GestureEvent* event) {
+  UpdateGestureTarget(nullptr);
+  if (!window_resizer_)
+    return;
+
+  std::unique_ptr<ScopedWindowResizer> resizer(std::move(window_resizer_));
+  resizer->resizer()->FlingOrSwipe(event);
+  first_finger_hittest_ = HTNOWHERE;
+  in_gesture_drag_ = false;
+  if (end_closure_)
+    std::move(end_closure_).Run(DragResult::SUCCESS);
 }
 
 void WmToplevelWindowEventHandler::ResizerWindowDestroyed() {
@@ -612,7 +659,10 @@ void WmToplevelWindowEventHandler::OnWindowDestroying(aura::Window* window) {
     UpdateGestureTarget(nullptr);
 }
 
-void WmToplevelWindowEventHandler::UpdateGestureTarget(aura::Window* target) {
+void WmToplevelWindowEventHandler::UpdateGestureTarget(
+    aura::Window* target,
+    const gfx::Point& location) {
+  event_location_in_gesture_target_ = location;
   if (gesture_target_ == target)
     return;
 
