@@ -10,6 +10,7 @@
 #include "ash/shelf/shelf_constants.h"
 #include "ash/shell.h"
 #include "ash/test/ash_test_base.h"
+#include "ash/window_factory.h"
 #include "ash/wm/window_positioning_utils.h"
 #include "ash/wm/window_state.h"
 #include "ash/wm/window_util.h"
@@ -19,9 +20,10 @@
 #include "base/command_line.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/stringprintf.h"
-#include "services/ui/public/interfaces/window_tree_constants.mojom.h"
+#include "services/ws/public/mojom/window_tree_constants.mojom.h"
 #include "ui/aura/client/aura_constants.h"
 #include "ui/aura/test/test_window_delegate.h"
+#include "ui/aura/test/test_windows.h"
 #include "ui/aura/window_event_dispatcher.h"
 #include "ui/base/hit_test.h"
 #include "ui/display/display_layout.h"
@@ -60,6 +62,38 @@ class TestWindowDelegate : public aura::test::TestWindowDelegate {
   DISALLOW_COPY_AND_ASSIGN(TestWindowDelegate);
 };
 
+// WindowState based on a given initial state. Records the last resize bounds.
+class FakeWindowState : public wm::WindowState::State {
+ public:
+  explicit FakeWindowState(mojom::WindowStateType initial_state_type)
+      : state_type_(initial_state_type) {}
+  ~FakeWindowState() override = default;
+
+  // WindowState::State overrides:
+  void OnWMEvent(wm::WindowState* window_state,
+                 const wm::WMEvent* event) override {
+    if (event->IsBoundsEvent()) {
+      if (event->type() == wm::WM_EVENT_SET_BOUNDS) {
+        const auto* set_bounds_event =
+            static_cast<const wm::SetBoundsEvent*>(event);
+        last_bounds_ = set_bounds_event->requested_bounds();
+      }
+    }
+  }
+  mojom::WindowStateType GetType() const override { return state_type_; }
+  void AttachState(wm::WindowState* window_state,
+                   wm::WindowState::State* previous_state) override {}
+  void DetachState(wm::WindowState* window_state) override {}
+
+  const gfx::Rect& last_bounds() { return last_bounds_; }
+
+ private:
+  mojom::WindowStateType state_type_;
+  gfx::Rect last_bounds_;
+
+  DISALLOW_COPY_AND_ASSIGN(FakeWindowState);
+};
+
 }  // namespace
 
 class WorkspaceWindowResizerTest : public AshTestBase {
@@ -78,25 +112,25 @@ class WorkspaceWindowResizerTest : public AshTestBase {
     gfx::Rect root_bounds(root->bounds());
     EXPECT_EQ(800, root_bounds.width());
     Shell::Get()->SetDisplayWorkAreaInsets(root, gfx::Insets());
-    window_.reset(new aura::Window(&delegate_));
+    window_ = window_factory::NewWindow(&delegate_);
     window_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     window_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(window_.get());
     window_->set_id(1);
 
-    window2_.reset(new aura::Window(&delegate2_));
+    window2_ = window_factory::NewWindow(&delegate2_);
     window2_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     window2_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(window2_.get());
     window2_->set_id(2);
 
-    window3_.reset(new aura::Window(&delegate3_));
+    window3_ = window_factory::NewWindow(&delegate3_);
     window3_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     window3_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(window3_.get());
     window3_->set_id(3);
 
-    window4_.reset(new aura::Window(&delegate4_));
+    window4_ = window_factory::NewWindow(&delegate4_);
     window4_->SetType(aura::client::WINDOW_TYPE_NORMAL);
     window4_->Init(ui::LAYER_NOT_DRAWN);
     ParentWindowInPrimaryRootWindow(window4_.get());
@@ -123,7 +157,7 @@ class WorkspaceWindowResizerTest : public AshTestBase {
       if (*i == window_.get() || *i == window2_.get() || *i == window3_.get()) {
         if (!result.empty())
           result += " ";
-        result += base::IntToString((*i)->id());
+        result += base::NumberToString((*i)->id());
       }
     }
     return result;
@@ -537,8 +571,8 @@ TEST_F(WorkspaceWindowResizerTest, Edge) {
   // http://crbug.com/292238.
   window_->SetBounds(gfx::Rect(20, 30, 400, 60));
   window_->SetProperty(aura::client::kResizeBehaviorKey,
-                       ui::mojom::kResizeBehaviorCanResize |
-                           ui::mojom::kResizeBehaviorCanMaximize);
+                       ws::mojom::kResizeBehaviorCanResize |
+                           ws::mojom::kResizeBehaviorCanMaximize);
   wm::WindowState* window_state = wm::GetWindowState(window_.get());
 
   {
@@ -597,7 +631,7 @@ TEST_F(WorkspaceWindowResizerTest, Edge) {
     // With the resolution of 500x600 we will hit in this case the 50% screen
     // size setting.
     // TODO(varkha): Insets are updated because of http://crbug.com/292238
-    EXPECT_EQ("250,0 250x" + base::IntToString(bottom),
+    EXPECT_EQ("250,0 250x" + base::NumberToString(bottom),
               window_->bounds().ToString());
     EXPECT_EQ("800,10 400x60",
               window_state->GetRestoreBoundsInScreen().ToString());
@@ -608,7 +642,7 @@ TEST_F(WorkspaceWindowResizerTest, Edge) {
 TEST_F(WorkspaceWindowResizerTest, NonResizableWindows) {
   window_->SetBounds(gfx::Rect(20, 30, 50, 60));
   window_->SetProperty(aura::client::kResizeBehaviorKey,
-                       ui::mojom::kResizeBehaviorNone);
+                       ws::mojom::kResizeBehaviorNone);
 
   std::unique_ptr<WindowResizer> resizer(
       CreateResizerForTest(window_.get(), gfx::Point(), HTCAPTION));
@@ -628,8 +662,8 @@ TEST_F(WorkspaceWindowResizerTest, CancelSnapPhantom) {
 
   // Make the window snappable by making it resizable and maximizable.
   window_->SetProperty(aura::client::kResizeBehaviorKey,
-                       ui::mojom::kResizeBehaviorCanResize |
-                           ui::mojom::kResizeBehaviorCanMaximize);
+                       ws::mojom::kResizeBehaviorCanResize |
+                           ws::mojom::kResizeBehaviorCanMaximize);
   EXPECT_EQ(root_windows[0], window_->GetRootWindow());
   EXPECT_FLOAT_EQ(1.0f, window_->layer()->opacity());
   {
@@ -785,7 +819,7 @@ TEST_F(WorkspaceWindowResizerTest, DontDragOffBottom) {
   resizer->Drag(CalculateDragPoint(*resizer, 0, 600), 0);
   int expected_y =
       kRootHeight - WorkspaceWindowResizer::kMinOnscreenHeight - 10;
-  EXPECT_EQ("100," + base::IntToString(expected_y) + " 300x400",
+  EXPECT_EQ("100," + base::NumberToString(expected_y) + " 300x400",
             window_->bounds().ToString());
 }
 
@@ -815,7 +849,7 @@ TEST_F(WorkspaceWindowResizerTest, DontDragOffBottomWithMultiDisplay) {
     // When the mouse cursor is in the primary display, the window cannot move
     // on non-work area but can get all the way towards the bottom,
     // restricted only by the window height.
-    EXPECT_EQ("100," + base::IntToString(expected_y) + " 300x20",
+    EXPECT_EQ("100," + base::NumberToString(expected_y) + " 300x20",
               window_->bounds().ToString());
     // Revert the drag in order to not remember the restore bounds.
     resizer->RevertDrag();
@@ -834,7 +868,7 @@ TEST_F(WorkspaceWindowResizerTest, DontDragOffBottomWithMultiDisplay) {
         kRootHeight - WorkspaceWindowResizer::kMinOnscreenHeight - 10;
     // When the mouse cursor is in the primary display, the window cannot move
     // on non-work area with kMinOnscreenHeight margin.
-    EXPECT_EQ("100," + base::IntToString(expected_y) + " 300x400",
+    EXPECT_EQ("100," + base::NumberToString(expected_y) + " 300x400",
               window_->bounds().ToString());
     resizer->CompleteDrag();
   }
@@ -890,8 +924,9 @@ TEST_F(WorkspaceWindowResizerTest, ResizeWindowOutsideLeftWorkArea) {
       window_.get(), gfx::Point(pixels_to_left_border, 0), HTRIGHT));
   ASSERT_TRUE(resizer.get());
   resizer->Drag(CalculateDragPoint(*resizer, -window_width, 0), 0);
-  EXPECT_EQ(base::IntToString(window_x) + ",100 " +
-                base::IntToString(wm::kMinimumOnScreenArea - window_x) + "x380",
+  EXPECT_EQ(base::NumberToString(window_x) + ",100 " +
+                base::NumberToString(wm::kMinimumOnScreenArea - window_x) +
+                "x380",
             window_->bounds().ToString());
 }
 
@@ -908,9 +943,9 @@ TEST_F(WorkspaceWindowResizerTest, ResizeWindowOutsideRightWorkArea) {
       CreateResizerForTest(window_.get(), gfx::Point(window_x, 0), HTLEFT));
   ASSERT_TRUE(resizer.get());
   resizer->Drag(CalculateDragPoint(*resizer, window_width, 0), 0);
-  EXPECT_EQ(base::IntToString(right - wm::kMinimumOnScreenArea) + ",100 " +
-                base::IntToString(window_width - pixels_to_right_border +
-                                  wm::kMinimumOnScreenArea) +
+  EXPECT_EQ(base::NumberToString(right - wm::kMinimumOnScreenArea) + ",100 " +
+                base::NumberToString(window_width - pixels_to_right_border +
+                                     wm::kMinimumOnScreenArea) +
                 "x380",
             window_->bounds().ToString());
 }
@@ -927,10 +962,10 @@ TEST_F(WorkspaceWindowResizerTest, ResizeWindowOutsideBottomWorkArea) {
       window_.get(), gfx::Point(0, bottom - delta_to_bottom), HTTOP));
   ASSERT_TRUE(resizer.get());
   resizer->Drag(CalculateDragPoint(*resizer, 0, bottom), 0);
-  EXPECT_EQ("100," + base::IntToString(bottom - wm::kMinimumOnScreenArea) +
+  EXPECT_EQ("100," + base::NumberToString(bottom - wm::kMinimumOnScreenArea) +
                 " 300x" +
-                base::IntToString(height -
-                                  (delta_to_bottom - wm::kMinimumOnScreenArea)),
+                base::NumberToString(
+                    height - (delta_to_bottom - wm::kMinimumOnScreenArea)),
             window_->bounds().ToString());
 }
 
@@ -951,8 +986,8 @@ TEST_F(WorkspaceWindowResizerTest, DragWindowOutsideRightToSecondaryDisplay) {
       CreateResizerForTest(window_.get(), gfx::Point(window_x, 0), HTCAPTION));
   ASSERT_TRUE(resizer.get());
   resizer->Drag(CalculateDragPoint(*resizer, window_width, 0), 0);
-  EXPECT_EQ(base::IntToString(right - wm::kMinimumOnScreenArea) + ",100 " +
-                base::IntToString(window_width) + "x380",
+  EXPECT_EQ(base::NumberToString(right - wm::kMinimumOnScreenArea) + ",100 " +
+                base::NumberToString(window_width) + "x380",
             window_->bounds().ToString());
 
   // With secondary display.  Operation itself is same but doesn't change
@@ -962,8 +997,8 @@ TEST_F(WorkspaceWindowResizerTest, DragWindowOutsideRightToSecondaryDisplay) {
                                          gfx::Insets(0, 0, 50, 0));
   window_->SetBounds(gfx::Rect(window_x, 100, window_width, 380));
   resizer->Drag(CalculateDragPoint(*resizer, window_width, 0), 0);
-  EXPECT_EQ(base::IntToString(window_x + window_width) + ",100 " +
-                base::IntToString(window_width) + "x380",
+  EXPECT_EQ(base::NumberToString(window_x + window_width) + ",100 " +
+                base::NumberToString(window_width) + "x380",
             window_->bounds().ToString());
 }
 
@@ -1408,7 +1443,7 @@ TEST_F(WorkspaceWindowResizerTest, MagneticallyResize_LEFT) {
 TEST_F(WorkspaceWindowResizerTest, CheckUserWindowManagedFlags) {
   window_->SetBounds(gfx::Rect(0, 50, 400, 200));
   window_->SetProperty(aura::client::kResizeBehaviorKey,
-                       ui::mojom::kResizeBehaviorCanMaximize);
+                       ws::mojom::kResizeBehaviorCanMaximize);
 
   std::vector<aura::Window*> no_attached_windows;
   // Check that an abort doesn't change anything.
@@ -1465,8 +1500,8 @@ TEST_F(WorkspaceWindowResizerTest, TestPartialMaxSizeEnforced) {
 TEST_F(WorkspaceWindowResizerTest, PhantomSnapMaxSize) {
   // Make the window snappable by making it resizable and maximizable.
   window_->SetProperty(aura::client::kResizeBehaviorKey,
-                       ui::mojom::kResizeBehaviorCanResize |
-                           ui::mojom::kResizeBehaviorCanMaximize);
+                       ws::mojom::kResizeBehaviorCanResize |
+                           ws::mojom::kResizeBehaviorCanMaximize);
   {
     // With max size not set we get a phantom window controller for dragging off
     // the right hand side.
