@@ -30,11 +30,11 @@
 #include "base/allocator/buildflags.h"
 #include "base/allocator/malloc_zone_functions_mac.h"
 #include "base/bind.h"
+#include "base/bits.h"
 #include "base/logging.h"
 #include "base/mac/mac_util.h"
 #include "base/mac/mach_logging.h"
 #include "base/process/memory.h"
-#include "base/scoped_clear_errno.h"
 #include "base/threading/sequenced_task_runner_handle.h"
 #include "build/build_config.h"
 #include "third_party/apple_apsl/CFBase.h"
@@ -148,7 +148,7 @@ void* oom_killer_memalign(struct _malloc_zone_t* zone,
   // other reasons why NULL might be returned (see
   // http://opensource.apple.com/source/Libc/Libc-583/gen/malloc.c ).
   if (!result && size && alignment >= sizeof(void*) &&
-      (alignment & (alignment - 1)) == 0) {
+      base::bits::IsPowerOfTwo(alignment)) {
     TerminateBecauseOutOfMemory(size);
   }
   return result;
@@ -198,7 +198,7 @@ void* oom_killer_memalign_purgeable(struct _malloc_zone_t* zone,
   // other reasons why NULL might be returned (see
   // http://opensource.apple.com/source/Libc/Libc-583/gen/malloc.c ).
   if (!result && size && alignment >= sizeof(void*) &&
-      (alignment & (alignment - 1)) == 0) {
+      base::bits::IsPowerOfTwo(alignment)) {
     TerminateBecauseOutOfMemory(size);
   }
   return result;
@@ -211,7 +211,7 @@ void* oom_killer_memalign_purgeable(struct _malloc_zone_t* zone,
 // === Core Foundation CFAllocators ===
 
 bool CanGetContextForCFAllocator() {
-  return !base::mac::IsOSLaterThan10_13_DontCallThis();
+  return !base::mac::IsOSLaterThan10_14_DontCallThis();
 }
 
 CFAllocatorContext* ContextForCFAllocator(CFAllocatorRef allocator) {
@@ -500,7 +500,7 @@ void ShimNewMallocZonesAndReschedule(base::Time end_time,
   base::TimeDelta next_delay = delay * 2;
   SequencedTaskRunnerHandle::Get()->PostDelayedTask(
       FROM_HERE,
-      base::Bind(&ShimNewMallocZonesAndReschedule, end_time, next_delay),
+      base::BindOnce(&ShimNewMallocZonesAndReschedule, end_time, next_delay),
       delay);
 }
 
