@@ -8,37 +8,40 @@ import static org.chromium.base.metrics.CachedMetrics.EnumeratedHistogramSample;
 
 import android.annotation.SuppressLint;
 import android.content.Context;
-import android.os.AsyncTask;
 import android.os.Build;
 import android.os.Build.VERSION_CODES;
-import android.os.Process;
 import android.os.StrictMode;
 import android.os.SystemClock;
 import android.support.annotation.NonNull;
+import android.support.annotation.Nullable;
 import android.support.v4.content.ContextCompat;
 import android.system.Os;
 
+import org.chromium.base.BaseSwitches;
 import org.chromium.base.BuildConfig;
 import org.chromium.base.BuildInfo;
 import org.chromium.base.CommandLine;
 import org.chromium.base.ContextUtils;
 import org.chromium.base.FileUtils;
 import org.chromium.base.Log;
+import org.chromium.base.StreamUtil;
+import org.chromium.base.StrictModeContext;
 import org.chromium.base.SysUtils;
 import org.chromium.base.TraceEvent;
 import org.chromium.base.VisibleForTesting;
 import org.chromium.base.annotations.JNINamespace;
 import org.chromium.base.annotations.MainDex;
+import org.chromium.base.compat.ApiHelperForM;
 import org.chromium.base.metrics.RecordHistogram;
+import org.chromium.base.task.AsyncTask;
 
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.zip.ZipEntry;
 import java.util.zip.ZipFile;
 
-import javax.annotation.Nullable;
-
 /**
  * This class provides functionality to load and register the native libraries.
  * Callers are allowed to separate loading the libraries from initializing them.
@@ -84,6 +87,9 @@ public class LibraryLoader {
     // SharedPreferences key for "don't prefetch libraries" flag
     private static final String DONT_PREFETCH_LIBRARIES_KEY = "dont_prefetch_libraries";
 
+    // Shared preferences key for the reached code profiler.
+    private static final String REACHED_CODE_PROFILER_ENABLED_KEY = "reached_code_profiler_enabled";
+
     private static final EnumeratedHistogramSample sRelinkerCountHistogram =
             new EnumeratedHistogramSample("ChromiumAndroidLinker.RelinkerFallbackCount", 2);
 
@@ -135,6 +141,27 @@ public class LibraryLoader {
     // report.
     private int mLibraryPreloaderStatus = -1;
 
+    /**
+     * Call this method to determine if this chromium project must
+     * use this linker. If not, System.loadLibrary() should be used to load
+     * libraries instead.
+     */
+    public static boolean useCrazyLinker() {
+        // The auto-generated NativeLibraries.sUseLinker variable will be true if the
+        // build has not explicitly disabled Linker features.
+        return NativeLibraries.sUseLinker;
+    }
+
+    /**
+     * Call this method to determine if the chromium project must load the library
+     * directly from a zip file.
+     */
+    private static boolean isInZipFile() {
+        // The auto-generated NativeLibraries.sUseLibraryInZipFile variable will be true
+        // iff the library remains embedded in the APK zip file on the target.
+        return NativeLibraries.sUseLibraryInZipFile;
+    }
+
     /**
      * Set native library preloader, if set, the NativeLibraryPreloader.loadLibrary will be invoked
      * before calling System.loadLibrary, this only applies when not using the chromium linker.
@@ -186,7 +213,7 @@ public class LibraryLoader {
      */
     public void preloadNowOverrideApplicationContext(Context appContext) {
         synchronized (mLock) {
-            if (!Linker.isUsed()) {
+            if (!useCrazyLinker()) {
                 preloadAlreadyLocked(appContext);
             }
         }
@@ -195,7 +222,7 @@ public class LibraryLoader {
     private void preloadAlreadyLocked(Context appContext) {
         try (TraceEvent te = TraceEvent.scoped("LibraryLoader.preloadAlreadyLocked")) {
             // Preloader uses system linker, we shouldn't preload if Chromium linker is used.
-            assert !Linker.isUsed();
+            assert !useCrazyLinker();
             if (mLibraryPreloader != null && !mLibraryPreloaderCalled) {
                 mLibraryPreloaderStatus = mLibraryPreloader.loadLibrary(appContext);
                 mLibraryPreloaderCalled = true;
@@ -284,6 +311,33 @@ public class LibraryLoader {
         }
     }
 
+    /**
+     * Enables the reached code profiler. The value comes from "ReachedCodeProfiler"
+     * finch experiment, and is pushed on every run. I.e. the effect of the finch experiment
+     * lags by one run, which is the best we can do considering that the profiler has to be enabled
+     * before finch is initialized. Note that since LibraryLoader is in //base, it can't depend
+     * on ChromeFeatureList, and has to rely on external code pushing the value.
+     *
+     * @param enabled whether to enable the reached code profiler.
+     */
+    public static void setReachedCodeProfilerEnabledOnNextRuns(boolean enabled) {
+        ContextUtils.getAppSharedPreferences()
+                .edit()
+                .putBoolean(REACHED_CODE_PROFILER_ENABLED_KEY, enabled)
+                .apply();
+    }
+
+    /**
+     * @return whether to enable reached code profiler (see
+     *         setReachedCodeProfilerEnabledOnNextRuns()).
+     */
+    private static boolean isReachedCodeProfilerEnabled() {
+        try (StrictModeContext unused = StrictModeContext.allowDiskReads()) {
+            return ContextUtils.getAppSharedPreferences().getBoolean(
+                    REACHED_CODE_PROFILER_ENABLED_KEY, false);
+        }
+    }
+
     /** Prefetches the native libraries in a background thread.
      *
      * Launches an AsyncTask that, through a short-lived forked process, reads a
@@ -312,7 +366,7 @@ public class LibraryLoader {
         new LibraryPrefetchTask(coldStart).executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
     }
 
-    private static class LibraryPrefetchTask extends AsyncTask<Void, Void, Void> {
+    private static class LibraryPrefetchTask extends AsyncTask<Void> {
         private final boolean mColdStart;
 
         public LibraryPrefetchTask(boolean coldStart) {
@@ -320,9 +374,10 @@ public class LibraryLoader {
         }
 
         @Override
-        protected Void doInBackground(Void... params) {
-            try (TraceEvent e = TraceEvent.scoped("LibraryLoader.asyncPrefetchLibrariesToMemory")) {
-                int percentage = nativePercentageOfResidentNativeLibraryCode();
+        protected Void doInBackground() {
+            int percentage = nativePercentageOfResidentNativeLibraryCode();
+            try (TraceEvent e = TraceEvent.scoped("LibraryLoader.asyncPrefetchLibrariesToMemory",
+                         Integer.toString(percentage))) {
                 // Arbitrary percentage threshold. If most of the native library is already
                 // resident (likely with monochrome), don't bother creating a prefetch process.
                 boolean prefetch = mColdStart && percentage < 90;
@@ -388,7 +443,7 @@ public class LibraryLoader {
     // Invoke either Linker.loadLibrary(...), System.loadLibrary(...) or System.load(...),
     // triggering JNI_OnLoad in native code.
     // TODO(crbug.com/635567): Fix this properly.
-    @SuppressLint({"DefaultLocale", "NewApi", "UnsafeDynamicallyLoadedCode"})
+    @SuppressLint({"DefaultLocale", "UnsafeDynamicallyLoadedCode"})
     private void loadAlreadyLocked(Context appContext) throws ProcessInitException {
         try (TraceEvent te = TraceEvent.scoped("LibraryLoader.loadAlreadyLocked")) {
             if (!mLoaded) {
@@ -396,12 +451,12 @@ public class LibraryLoader {
 
                 long startTime = SystemClock.uptimeMillis();
 
-                if (Linker.isUsed()) {
+                if (useCrazyLinker()) {
                     // Load libraries using the Chromium linker.
                     Linker linker = Linker.getInstance();
 
                     String apkFilePath =
-                            Linker.isInZipFile() ? appContext.getApplicationInfo().sourceDir : null;
+                            isInZipFile() ? appContext.getApplicationInfo().sourceDir : null;
                     linker.prepareLibraryLoad(apkFilePath);
 
                     for (String library : NativeLibraries.LIBRARIES) {
@@ -427,7 +482,7 @@ public class LibraryLoader {
                                     linker, apkFilePath, libFilePath);
                             incrementRelinkerCountNotHitHistogram();
                         } catch (UnsatisfiedLinkError e) {
-                            if (!Linker.isInZipFile()
+                            if (!isInZipFile()
                                     && PLATFORM_REQUIRES_NATIVE_FALLBACK_EXTRACTION) {
                                 loadLibraryWithCustomLinkerAlreadyLocked(
                                         linker, null, getExtractedLibraryPath(appContext, library));
@@ -447,18 +502,18 @@ public class LibraryLoader {
                     // If the libraries are located in the zip file, assert that the device API
                     // level is M or higher. On devices lower than M, the libraries should
                     // always be loaded by Linker.
-                    assert !Linker.isInZipFile() || Build.VERSION.SDK_INT >= VERSION_CODES.M;
+                    assert !isInZipFile() || Build.VERSION.SDK_INT >= VERSION_CODES.M;
 
                     // Load libraries using the system linker.
                     for (String library : NativeLibraries.LIBRARIES) {
                         try {
-                            if (!Linker.isInZipFile()) {
+                            if (!isInZipFile()) {
                                 // The extract and retry logic isn't needed because this path is
                                 // used only for local development.
                                 System.loadLibrary(library);
                             } else {
                                 // Load directly from the APK.
-                                boolean is64Bit = Process.is64Bit();
+                                boolean is64Bit = ApiHelperForM.isProcess64Bit();
                                 String zipFilePath = appContext.getApplicationInfo().sourceDir;
                                 // In API level 23 and above, it’s possible to open a .so file
                                 // directly from the APK of the path form
@@ -562,6 +617,13 @@ public class LibraryLoader {
         }
         mLibraryProcessType = processType;
 
+        // Add a switch for the reached code profiler as late as possible since it requires a read
+        // from the shared preferences. At this point the shared preferences are usually warmed up.
+        if (mLibraryProcessType == LibraryProcessType.PROCESS_BROWSER
+                && isReachedCodeProfilerEnabled()) {
+            CommandLine.getInstance().appendSwitch(BaseSwitches.ENABLE_REACHED_CODE_PROFILER);
+        }
+
         ensureCommandLineSwitchedAlreadyLocked();
 
         if (!nativeLibraryLoaded(mLibraryProcessType)) {
@@ -632,7 +694,7 @@ public class LibraryLoader {
     // onNativeInitializationComplete().
     private void recordBrowserProcessHistogramAlreadyLocked() {
         assert Thread.holdsLock(mLock);
-        if (Linker.isUsed()) {
+        if (useCrazyLinker()) {
             nativeRecordChromiumAndroidLinkerBrowserHistogram(mIsUsingBrowserSharedRelros,
                     mLoadAtFixedAddressFailed,
                     mLibraryWasLoadedFromApk ? LibraryLoadFromApkStatusCodes.SUCCESSFUL
@@ -651,7 +713,7 @@ public class LibraryLoader {
     public void registerRendererProcessHistogram(boolean requestedSharedRelro,
                                                  boolean loadAtFixedAddressFailed) {
         synchronized (mLock) {
-            if (Linker.isUsed()) {
+            if (useCrazyLinker()) {
                 nativeRegisterChromiumAndroidLinkerRendererHistogram(
                         requestedSharedRelro, loadAtFixedAddressFailed, mLibraryLoadTimeMs);
             }
@@ -704,11 +766,13 @@ public class LibraryLoader {
         File libraryFile = new File(destDir, fileName);
 
         if (!libraryFile.exists()) {
-            try (ZipFile zipFile = new ZipFile(apkPath);
-                    InputStream inputStream =
-                            zipFile.getInputStream(zipFile.getEntry(pathWithinApk))) {
-                if (zipFile.getEntry(pathWithinApk) == null)
+            ZipFile zipFile = null;
+            try {
+                zipFile = new ZipFile(apkPath);
+                ZipEntry zipEntry = zipFile.getEntry(pathWithinApk);
+                if (zipEntry == null)
                     throw new RuntimeException("Cannot find ZipEntry" + pathWithinApk);
+                InputStream inputStream = zipFile.getInputStream(zipEntry);
 
                 FileUtils.copyFileStreamAtomicWithBuffer(
                         inputStream, libraryFile, new byte[16 * 1024]);
@@ -716,6 +780,8 @@ public class LibraryLoader {
                 libraryFile.setExecutable(true, false);
             } catch (IOException e) {
                 throw new RuntimeException(e);
+            } finally {
+                StreamUtil.closeQuietly(zipFile);
             }
         }
         return libraryFile.getAbsolutePath();
