@@ -19,11 +19,12 @@
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-using ::testing::_;
-using ::testing::Mock;
+using ::testing::AnyNumber;
 using ::testing::ByMove;
+using ::testing::Mock;
 using ::testing::Return;
 using ::testing::StrictMock;
+using ::testing::_;
 
 namespace base {
 namespace {
@@ -54,6 +55,7 @@ class HasRef : public NoRef {
 
   MOCK_CONST_METHOD0(AddRef, void());
   MOCK_CONST_METHOD0(Release, bool());
+  MOCK_CONST_METHOD0(HasAtLeastOneRef, bool());
 
  private:
   // Particularly important in this test to ensure no copies are made.
@@ -72,6 +74,7 @@ class Parent {
  public:
   void AddRef() const {}
   void Release() const {}
+  bool HasAtLeastOneRef() const { return true; }
   virtual void VirtualSet() { value = kParentValue; }
   void NonVirtualSet() { value = kParentValue; }
   int value;
@@ -311,6 +314,10 @@ void TakesACallback(const Closure& callback) {
   callback.Run();
 }
 
+int Noexcept() noexcept {
+  return 42;
+}
+
 class BindTest : public ::testing::Test {
  public:
   BindTest() {
@@ -319,13 +326,15 @@ class BindTest : public ::testing::Test {
     static_func_mock_ptr = &static_func_mock_;
   }
 
-  virtual ~BindTest() = default;
+  ~BindTest() override = default;
 
   static void VoidFunc0() {
     static_func_mock_ptr->VoidMethod0();
   }
 
   static int IntFunc0() { return static_func_mock_ptr->IntMethod0(); }
+  int NoexceptMethod() noexcept { return 42; }
+  int ConstNoexceptMethod() const noexcept { return 42; }
 
  protected:
   StrictMock<NoRef> no_ref_;
@@ -437,6 +446,7 @@ TEST_F(BindTest, IgnoreResultForRepeating) {
   EXPECT_CALL(static_func_mock_, IntMethod0()).WillOnce(Return(1337));
   EXPECT_CALL(has_ref_, AddRef()).Times(2);
   EXPECT_CALL(has_ref_, Release()).Times(2);
+  EXPECT_CALL(has_ref_, HasAtLeastOneRef()).WillRepeatedly(Return(true));
   EXPECT_CALL(has_ref_, IntMethod0()).WillOnce(Return(10));
   EXPECT_CALL(has_ref_, IntConstMethod0()).WillOnce(Return(11));
   EXPECT_CALL(no_ref_, IntMethod0()).WillOnce(Return(12));
@@ -475,6 +485,7 @@ TEST_F(BindTest, IgnoreResultForOnce) {
   EXPECT_CALL(static_func_mock_, IntMethod0()).WillOnce(Return(1337));
   EXPECT_CALL(has_ref_, AddRef()).Times(2);
   EXPECT_CALL(has_ref_, Release()).Times(2);
+  EXPECT_CALL(has_ref_, HasAtLeastOneRef()).WillRepeatedly(Return(true));
   EXPECT_CALL(has_ref_, IntMethod0()).WillOnce(Return(10));
   EXPECT_CALL(has_ref_, IntConstMethod0()).WillOnce(Return(11));
 
@@ -763,7 +774,7 @@ struct OnceTestConfig {
 
 using BindVariantsTestConfig = ::testing::Types<
   RepeatingTestConfig, OnceTestConfig>;
-TYPED_TEST_CASE(BindVariantsTest, BindVariantsTestConfig);
+TYPED_TEST_SUITE(BindVariantsTest, BindVariantsTestConfig);
 
 template <typename TypeParam, typename Signature>
 using CallbackType = typename TypeParam::template CallbackType<Signature>;
@@ -790,6 +801,7 @@ TYPED_TEST(BindVariantsTest, FunctionTypeSupport) {
   EXPECT_CALL(static_func_mock, VoidMethod0());
   EXPECT_CALL(has_ref, AddRef()).Times(4);
   EXPECT_CALL(has_ref, Release()).Times(4);
+  EXPECT_CALL(has_ref, HasAtLeastOneRef()).WillRepeatedly(Return(true));
   EXPECT_CALL(has_ref, VoidMethod0()).Times(2);
   EXPECT_CALL(has_ref, VoidConstMethod0()).Times(2);
 
@@ -838,6 +850,7 @@ TYPED_TEST(BindVariantsTest, ReturnValues) {
   EXPECT_CALL(static_func_mock, IntMethod0()).WillOnce(Return(1337));
   EXPECT_CALL(has_ref, AddRef()).Times(4);
   EXPECT_CALL(has_ref, Release()).Times(4);
+  EXPECT_CALL(has_ref, HasAtLeastOneRef()).WillRepeatedly(Return(true));
   EXPECT_CALL(has_ref, IntMethod0()).WillOnce(Return(31337));
   EXPECT_CALL(has_ref, IntConstMethod0())
       .WillOnce(Return(41337))
@@ -958,6 +971,7 @@ TYPED_TEST(BindVariantsTest, ScopedRefptr) {
   StrictMock<HasRef> has_ref;
   EXPECT_CALL(has_ref, AddRef()).Times(1);
   EXPECT_CALL(has_ref, Release()).Times(1);
+  EXPECT_CALL(has_ref, HasAtLeastOneRef()).WillRepeatedly(Return(true));
 
   const scoped_refptr<HasRef> refptr(&has_ref);
   CallbackType<TypeParam, int()> scoped_refptr_const_ref_cb =
@@ -989,7 +1003,7 @@ struct CustomDeleter {
 using MoveOnlyTypesToTest =
     ::testing::Types<std::unique_ptr<DeleteCounter>,
                      std::unique_ptr<DeleteCounter, CustomDeleter>>;
-TYPED_TEST_CASE(BindMoveOnlyTypeTest, MoveOnlyTypesToTest);
+TYPED_TEST_SUITE(BindMoveOnlyTypeTest, MoveOnlyTypesToTest);
 
 TYPED_TEST(BindMoveOnlyTypeTest, PassedToBoundCallback) {
   int deletes = 0;
@@ -1059,7 +1073,7 @@ TEST_F(BindTest, BindMoveOnlyVector) {
   using MoveOnlyVector = std::vector<std::unique_ptr<int>>;
 
   MoveOnlyVector v;
-  v.push_back(WrapUnique(new int(12345)));
+  v.push_back(std::make_unique<int>(12345));
 
   // Early binding should work:
   base::Callback<MoveOnlyVector()> bound_cb =
@@ -1469,6 +1483,16 @@ TEST_F(BindTest, UnwrapPassed) {
   EXPECT_EQ(p, internal::Unwrap(Passed(WrapUnique(p))).get());
 }
 
+TEST_F(BindTest, BindNoexcept) {
+  EXPECT_EQ(42, base::BindOnce(&Noexcept).Run());
+  EXPECT_EQ(
+      42,
+      base::BindOnce(&BindTest::NoexceptMethod, base::Unretained(this)).Run());
+  EXPECT_EQ(
+      42, base::BindOnce(&BindTest::ConstNoexceptMethod, base::Unretained(this))
+              .Run());
+}
+
 // Test null callbacks cause a DCHECK.
 TEST(BindDeathTest, NullCallback) {
   base::Callback<void(int)> null_cb;
@@ -1476,5 +1500,13 @@ TEST(BindDeathTest, NullCallback) {
   EXPECT_DCHECK_DEATH(base::Bind(null_cb, 42));
 }
 
+TEST(BindDeathTest, BanFirstOwnerOfRefCountedType) {
+  StrictMock<HasRef> has_ref;
+  EXPECT_DCHECK_DEATH({
+    EXPECT_CALL(has_ref, HasAtLeastOneRef()).WillOnce(Return(false));
+    base::BindOnce(&HasRef::VoidMethod0, &has_ref);
+  });
+}
+
 }  // namespace
 }  // namespace base
