@@ -7,6 +7,7 @@
 #include "base/callback.h"
 #include "base/compiler_specific.h"
 #include "base/macros.h"
+#include "base/sanitizer_buildflags.h"
 #include "base/strings/string_piece.h"
 #include "base/test/scoped_feature_list.h"
 #include "build/build_config.h"
@@ -30,13 +31,16 @@
 #endif  // OS_WIN
 
 #if defined(OS_FUCHSIA)
+#include <lib/zx/event.h>
+#include <lib/zx/port.h>
+#include <lib/zx/process.h>
+#include <lib/zx/thread.h>
+#include <lib/zx/time.h>
 #include <zircon/process.h>
-#include <zircon/syscalls.h>
 #include <zircon/syscalls/debug.h>
-#include <zircon/syscalls/exception.h>
 #include <zircon/syscalls/port.h>
+#include <zircon/types.h>
 #include "base/fuchsia/fuchsia_logging.h"
-#include "base/fuchsia/scoped_zx_handle.h"
 #endif
 
 namespace logging {
@@ -308,9 +312,9 @@ static const unsigned int kThreadEndedPortKey = 2u;
 
 struct thread_data_t {
   // For signaling the thread ended properly.
-  zx_handle_t event;
+  zx::unowned_event event;
   // For registering thread termination.
-  zx_handle_t port;
+  zx::unowned_port port;
   // Location where the thread is expected to crash.
   int death_location;
 };
@@ -322,10 +326,10 @@ void* CrashThread(void* arg) {
   int death_location = data->death_location;
 
   // Register the exception handler on the port.
-  status = zx_task_bind_exception_port(zx_thread_self(), data->port,
-                                       kExceptionPortKey, 0);
+  status = zx::thread::self()->bind_exception_port(*data->port,
+                                                   kExceptionPortKey, 0);
   if (status != ZX_OK) {
-    zx_object_signal(data->event, 0, ZX_USER_SIGNAL_0);
+    data->event->signal(0, ZX_USER_SIGNAL_0);
     return nullptr;
   }
 
@@ -335,47 +339,48 @@ void* CrashThread(void* arg) {
 
   // We should never reach this point, signal the thread incorrectly ended
   // properly.
-  zx_object_signal(data->event, 0, ZX_USER_SIGNAL_0);
+  data->event->signal(0, ZX_USER_SIGNAL_0);
   return nullptr;
 }
 
 // Runs the CrashThread function in a separate thread.
 void SpawnCrashThread(int death_location, uintptr_t* child_crash_addr) {
-  base::ScopedZxHandle port;
-  base::ScopedZxHandle event;
+  zx::port port;
+  zx::event event;
   zx_status_t status;
 
-  status = zx_port_create(0, port.receive());
+  status = zx::port::create(0, &port);
   ASSERT_EQ(status, ZX_OK);
-  status = zx_event_create(0, event.receive());
+  status = zx::event::create(0, &event);
   ASSERT_EQ(status, ZX_OK);
 
   // Register the thread ended event on the port.
-  status = zx_object_wait_async(event.get(), port.get(), kThreadEndedPortKey,
-                                ZX_USER_SIGNAL_0, ZX_WAIT_ASYNC_ONCE);
+  status = event.wait_async(port, kThreadEndedPortKey, ZX_USER_SIGNAL_0,
+                            ZX_WAIT_ASYNC_ONCE);
   ASSERT_EQ(status, ZX_OK);
 
   // Run the thread.
-  thread_data_t thread_data = {event.get(), port.get(), death_location};
+  thread_data_t thread_data = {zx::unowned_event(event), zx::unowned_port(port),
+                               death_location};
   pthread_t thread;
   int ret = pthread_create(&thread, nullptr, CrashThread, &thread_data);
   ASSERT_EQ(ret, 0);
 
   // Wait on the port.
   zx_port_packet_t packet;
-  status = zx_port_wait(port.get(), ZX_TIME_INFINITE, &packet);
+  status = port.wait(zx::time::infinite(), &packet);
   ASSERT_EQ(status, ZX_OK);
   // Check the thread did crash and not terminate.
   ASSERT_EQ(packet.key, kExceptionPortKey);
 
   // Get the crash address.
-  zx_handle_t zircon_thread;
-  status = zx_object_get_child(zx_process_self(), packet.exception.tid,
-                               ZX_RIGHT_SAME_RIGHTS, &zircon_thread);
+  zx::thread zircon_thread;
+  status = zx::process::self()->get_child(packet.exception.tid,
+                                          ZX_RIGHT_SAME_RIGHTS, &zircon_thread);
   ASSERT_EQ(status, ZX_OK);
   zx_thread_state_general_regs_t buffer;
-  status = zx_thread_read_state(zircon_thread, ZX_THREAD_STATE_GENERAL_REGS,
-                                &buffer, sizeof(buffer));
+  status = zircon_thread.read_state(ZX_THREAD_STATE_GENERAL_REGS, &buffer,
+                                    sizeof(buffer));
   ASSERT_EQ(status, ZX_OK);
 #if defined(ARCH_CPU_X86_64)
   *child_crash_addr = static_cast<uintptr_t>(buffer.rip);
@@ -385,7 +390,7 @@ void SpawnCrashThread(int death_location, uintptr_t* child_crash_addr) {
 #error Unsupported architecture
 #endif
 
-  status = zx_task_kill(zircon_thread);
+  status = zircon_thread.kill();
   ASSERT_EQ(status, ZX_OK);
 }
 
@@ -461,7 +466,7 @@ void CrashChildMain(int death_location) {
   // Should never reach this point.
   const uintptr_t failed = 0;
   HANDLE_EINTR(write(g_child_crash_pipe, &failed, sizeof(uintptr_t)));
-};
+}
 
 void SpawnChildAndCrash(int death_location, uintptr_t* child_crash_addr) {
   int pipefd[2];
@@ -536,7 +541,7 @@ void DcheckEmptyFunction1() {
 }
 void DcheckEmptyFunction2() {}
 
-#if DCHECK_IS_CONFIGURABLE
+#if defined(DCHECK_IS_CONFIGURABLE)
 class ScopedDcheckSeverity {
  public:
   ScopedDcheckSeverity(LogSeverity new_severity) : old_severity_(LOG_DCHECK) {
@@ -548,7 +553,7 @@ class ScopedDcheckSeverity {
  private:
   LogSeverity old_severity_;
 };
-#endif  // DCHECK_IS_CONFIGURABLE
+#endif  // defined(DCHECK_IS_CONFIGURABLE)
 
 // https://crbug.com/709067 tracks test flakiness on iOS.
 #if defined(OS_IOS)
@@ -557,12 +562,12 @@ class ScopedDcheckSeverity {
 #define MAYBE_Dcheck Dcheck
 #endif
 TEST_F(LoggingTest, MAYBE_Dcheck) {
-#if DCHECK_IS_CONFIGURABLE
+#if defined(DCHECK_IS_CONFIGURABLE)
   // DCHECKs are enabled, and LOG_DCHECK is mutable, but defaults to non-fatal.
   // Set it to LOG_FATAL to get the expected behavior from the rest of this
   // test.
   ScopedDcheckSeverity dcheck_severity(LOG_FATAL);
-#endif  // DCHECK_IS_CONFIGURABLE
+#endif  // defined(DCHECK_IS_CONFIGURABLE)
 
 #if defined(NDEBUG) && !defined(DCHECK_ALWAYS_ON)
   // Release build.
@@ -723,7 +728,7 @@ namespace nested_test {
   }
 }  // namespace nested_test
 
-#if DCHECK_IS_CONFIGURABLE
+#if defined(DCHECK_IS_CONFIGURABLE)
 TEST_F(LoggingTest, ConfigurableDCheck) {
   // Verify that DCHECKs default to non-fatal in configurable-DCHECK builds.
   // Note that we require only that DCHECK is non-fatal by default, rather
@@ -773,7 +778,7 @@ TEST_F(LoggingTest, ConfigurableDCheckFeature) {
     EXPECT_LT(LOG_DCHECK, LOG_FATAL);
   }
 }
-#endif  // DCHECK_IS_CONFIGURABLE
+#endif  // defined(DCHECK_IS_CONFIGURABLE)
 
 #if defined(OS_FUCHSIA)
 TEST_F(LoggingTest, FuchsiaLogging) {
@@ -795,6 +800,87 @@ TEST_F(LoggingTest, FuchsiaLogging) {
 }
 #endif  // defined(OS_FUCHSIA)
 
+TEST_F(LoggingTest, LogPrefix) {
+  // Set up a callback function to capture the log output string.
+  auto old_log_message_handler = GetLogMessageHandler();
+  // Use a static because only captureless lambdas can be converted to a
+  // function pointer for SetLogMessageHandler().
+  static std::string* log_string_ptr = nullptr;
+  std::string log_string;
+  log_string_ptr = &log_string;
+  SetLogMessageHandler([](int severity, const char* file, int line,
+                          size_t start, const std::string& str) -> bool {
+    *log_string_ptr = str;
+    return true;
+  });
+
+  // Logging with a prefix includes the prefix string after the opening '['.
+  const char kPrefix[] = "prefix";
+  SetLogPrefix(kPrefix);
+  LOG(ERROR) << "test";  // Writes into |log_string|.
+  EXPECT_EQ(1u, log_string.find(kPrefix));
+
+  // Logging without a prefix does not include the prefix string.
+  SetLogPrefix(nullptr);
+  LOG(ERROR) << "test";  // Writes into |log_string|.
+  EXPECT_EQ(std::string::npos, log_string.find(kPrefix));
+
+  // Clean up.
+  SetLogMessageHandler(old_log_message_handler);
+  log_string_ptr = nullptr;
+}
+
+#if !defined(ADDRESS_SANITIZER) && !defined(MEMORY_SANITIZER) && \
+    !BUILDFLAG(IS_HWASAN)
+// Since we scan potentially uninitialized portions of the stack, we can't run
+// this test under any sanitizer that checks for uninitialized reads.
+TEST_F(LoggingTest, LogMessageMarkersOnStack) {
+  const uint32_t kLogStartMarker = 0xbedead01;
+  const uint32_t kLogEndMarker = 0x5050dead;
+  const char kTestMessage[] = "Oh noes! I have crashed! 💩";
+
+  uint32_t stack_start = 0;
+
+  // Install a LogAssertHandler which will scan between |stack_start| and its
+  // local-scope stack for the start & end markers, and verify the message.
+  ScopedLogAssertHandler assert_handler(base::BindRepeating(
+      [](uint32_t* stack_start_ptr, const char* file, int line,
+         const base::StringPiece message, const base::StringPiece stack_trace) {
+        uint32_t stack_end;
+        uint32_t* stack_end_ptr = &stack_end;
+
+        // Scan the stack for the expected markers.
+        uint32_t* start_marker = nullptr;
+        uint32_t* end_marker = nullptr;
+        for (uint32_t* ptr = stack_end_ptr; ptr <= stack_start_ptr; ++ptr) {
+          if (*ptr == kLogStartMarker)
+            start_marker = ptr;
+          else if (*ptr == kLogEndMarker)
+            end_marker = ptr;
+        }
+
+        // Verify that start & end markers were found, somewhere, in-between
+        // this and the LogAssertHandler scope, in the LogMessage destructor's
+        // stack frame.
+        ASSERT_TRUE(start_marker);
+        ASSERT_TRUE(end_marker);
+
+        // Verify that the |message| is found in-between the markers.
+        const char* start_char_marker =
+            reinterpret_cast<char*>(start_marker + 1);
+        const char* end_char_marker = reinterpret_cast<char*>(end_marker);
+
+        const base::StringPiece stack_view(start_char_marker,
+                                           end_char_marker - start_char_marker);
+        ASSERT_FALSE(stack_view.find(message) == base::StringPiece::npos);
+      },
+      &stack_start));
+
+  // Trigger a log assertion, with a test message we can check for.
+  LOG(FATAL) << kTestMessage;
+}
+#endif  // !defined(ADDRESS_SANITIZER)
+
 }  // namespace
 
 }  // namespace logging
