@@ -6,7 +6,6 @@
 #define BASE_MESSAGE_LOOP_MESSAGE_LOOP_H_
 
 #include <memory>
-#include <queue>
 #include <string>
 
 #include "base/base_export.h"
@@ -14,29 +13,33 @@
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
 #include "base/memory/scoped_refptr.h"
-#include "base/message_loop/incoming_task_queue.h"
 #include "base/message_loop/message_loop_current.h"
-#include "base/message_loop/message_loop_task_runner.h"
 #include "base/message_loop/message_pump.h"
 #include "base/message_loop/timer_slack.h"
-#include "base/observer_list.h"
 #include "base/pending_task.h"
 #include "base/run_loop.h"
-#include "base/synchronization/lock.h"
-#include "base/threading/sequence_local_storage_map.h"
 #include "base/threading/thread_checker.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
 
 namespace base {
 
-class ThreadTaskRunnerHandle;
+class MessageLoopImpl;
+
+namespace sequence_manager {
+class TaskQueue;
+class LazyThreadControllerForTest;
+namespace internal {
+class SequenceManagerImpl;
+class ThreadControllerImpl;
+}
+}  // namespace sequence_manager
 
 // A MessageLoop is used to process events for a particular thread.  There is
 // at most one MessageLoop instance per thread.
 //
 // Events include at a minimum Task instances submitted to the MessageLoop's
-// TaskRunner. Depending on the type of message pump used by the MessageLoop
+// TaskRunner. Depending on the Type of message pump used by the MessageLoop
 // other events such as UI messages may be processed.  On Windows APC calls (as
 // time permits) and signals sent to a registered set of HANDLEs may also be
 // processed.
@@ -74,17 +77,11 @@ class ThreadTaskRunnerHandle;
 //
 // Please be SURE your task is reentrant (nestable) and all global variables
 // are stable and accessible before calling SetNestableTasksAllowed(true).
-//
-// TODO(gab): MessageLoop doesn't need to be a MessageLoopCurrent once callers
-// that store MessageLoop::current() in a MessageLoop* variable have been
-// updated to use a MessageLoopCurrent variable.
-class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
-                                public RunLoop::Delegate,
-                                public MessageLoopCurrent {
+
+class BASE_EXPORT MessageLoopBase {
  public:
-  // TODO(gab): Migrate usage of this class to MessageLoopCurrent and remove
-  // this forwarded declaration.
-  using DestructionObserver = MessageLoopCurrent::DestructionObserver;
+  MessageLoopBase() = default;
+  virtual ~MessageLoopBase() = default;
 
   // A MessageLoop has a particular type, which indicates the set of
   // asynchronous events it may process in addition to tasks and timers.
@@ -119,17 +116,122 @@ class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
 #endif  // defined(OS_ANDROID)
   };
 
+  // Returns true if this loop is |type|. This allows subclasses (especially
+  // those in tests) to specialize how they are identified.
+  virtual bool IsType(Type type) const = 0;
+
+  // Returns the name of the thread this message loop is bound to. This function
+  // is only valid when this message loop is running, BindToCurrentThread has
+  // already been called and has an "happens-before" relationship with this call
+  // (this relationship is obtained implicitly by the MessageLoop's task posting
+  // system unless calling this very early).
+  virtual std::string GetThreadName() const = 0;
+
+  using DestructionObserver = MessageLoopCurrent::DestructionObserver;
+
+  // Add a DestructionObserver, which will start receiving notifications
+  // immediately.
+  virtual void AddDestructionObserver(
+      DestructionObserver* destruction_observer) = 0;
+
+  // Remove a DestructionObserver.  It is safe to call this method while a
+  // DestructionObserver is receiving a notification callback.
+  virtual void RemoveDestructionObserver(
+      DestructionObserver* destruction_observer) = 0;
+
+  // TODO(altimin,yutak): Replace with base::TaskObserver.
+  using TaskObserver = MessageLoopCurrent::TaskObserver;
+
+  // These functions can only be called on the same thread that |this| is
+  // running on.
+  // These functions must not be called from a TaskObserver callback.
+  virtual void AddTaskObserver(TaskObserver* task_observer) = 0;
+  virtual void RemoveTaskObserver(TaskObserver* task_observer) = 0;
+
+  // When this functionality is enabled, the queue time will be recorded for
+  // posted tasks.
+  virtual void SetAddQueueTimeToTasks(bool enable) = 0;
+
+  // Returns true if this is the active MessageLoop for the current thread.
+  virtual bool IsBoundToCurrentThread() const = 0;
+
+  // Returns true if the message loop is idle (ignoring delayed tasks). This is
+  // the same condition which triggers DoWork() to return false: i.e.
+  // out of tasks which can be processed at the current run-level -- there might
+  // be deferred non-nestable tasks remaining if currently in a nested run
+  // level.
+  virtual bool IsIdleForTesting() = 0;
+
+  // Returns the MessagePump owned by this MessageLoop if any.
+  virtual MessagePump* GetMessagePump() const = 0;
+
+  // Sets a new TaskRunner for this message loop. If the message loop was
+  // already bound, this must be called on the thread to which it is bound.
+  // TODO(alexclarke): Remove this as part of https://crbug.com/825327.
+  virtual void SetTaskRunner(
+      scoped_refptr<SingleThreadTaskRunner> task_runner) = 0;
+
+  // Gets the TaskRunner associated with this message loop.
+  // TODO(alexclarke): Remove this as part of https://crbug.com/825327.
+  virtual scoped_refptr<SingleThreadTaskRunner> GetTaskRunner() = 0;
+
+  // Binds the MessageLoop to the current thread using |pump|.
+  virtual void BindToCurrentThread(std::unique_ptr<MessagePump> pump) = 0;
+
+  // Returns true if the MessageLoop retains any tasks inside it.
+  virtual bool HasTasks() = 0;
+
+  // Deletes all tasks associated with this MessageLoop. Note that the tasks
+  // can post other tasks when destructed.
+  virtual void DeletePendingTasks() = 0;
+
+ protected:
+  friend class MessageLoop;
+  friend class MessageLoopForUI;
+  friend class MessageLoopCurrent;
+  friend class MessageLoopCurrentForIO;
+  friend class MessageLoopCurrentForUI;
+  friend class Thread;
+  friend class sequence_manager::internal::ThreadControllerImpl;
+
+  // Explicitly allow or disallow task execution. Task execution is disallowed
+  // implicitly when we enter a nested runloop.
+  virtual void SetTaskExecutionAllowed(bool allowed) = 0;
+
+  // Whether task execution is allowed at the moment.
+  virtual bool IsTaskExecutionAllowed() const = 0;
+
+#if defined(OS_IOS)
+  virtual void AttachToMessagePump() = 0;
+#endif
+
+  virtual Type GetType() const = 0;
+
+  // Set the timer slack for this message loop.
+  // TODO(alexclarke): Remove this as part of https://crbug.com/891670.
+  virtual void SetTimerSlack(TimerSlack timer_slack) = 0;
+};
+
+class BASE_EXPORT MessageLoop {
+ public:
+  // For migration convenience we define the Type enum.
+  using Type = MessageLoopBase::Type;
+  static constexpr Type TYPE_DEFAULT = Type::TYPE_DEFAULT;
+  static constexpr Type TYPE_UI = Type::TYPE_UI;
+  static constexpr Type TYPE_CUSTOM = Type::TYPE_CUSTOM;
+  static constexpr Type TYPE_IO = Type::TYPE_IO;
+#if defined(OS_ANDROID)
+  static constexpr Type TYPE_JAVA = Type::TYPE_JAVA;
+#endif  // defined(OS_ANDROID)
+
   // Normally, it is not necessary to instantiate a MessageLoop.  Instead, it
   // is typical to make use of the current thread's MessageLoop instance.
   explicit MessageLoop(Type type = TYPE_DEFAULT);
   // Creates a TYPE_CUSTOM MessageLoop with the supplied MessagePump, which must
   // be non-NULL.
-  explicit MessageLoop(std::unique_ptr<MessagePump> pump);
+  explicit MessageLoop(std::unique_ptr<MessagePump> custom_pump);
 
-  ~MessageLoop() override;
-
-  // TODO(gab): Mass migrate callers to MessageLoopCurrent::Get().
-  static MessageLoopCurrent current();
+  virtual ~MessageLoop();
 
   using MessagePumpFactory = std::unique_ptr<MessagePump>();
   // Uses the given base::MessagePumpForUIFactory to override the default
@@ -142,9 +244,7 @@ class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
   static std::unique_ptr<MessagePump> CreateMessagePumpForType(Type type);
 
   // Set the timer slack for this message loop.
-  void SetTimerSlack(TimerSlack timer_slack) {
-    pump_->SetTimerSlack(timer_slack);
-  }
+  void SetTimerSlack(TimerSlack timer_slack);
 
   // Returns true if this loop is |type|. This allows subclasses (especially
   // those in tests) to specialize how they are identified.
@@ -160,45 +260,37 @@ class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
   // system unless calling this very early).
   std::string GetThreadName() const;
 
-  // Gets the TaskRunner associated with this message loop.
-  const scoped_refptr<SingleThreadTaskRunner>& task_runner() const {
-    return task_runner_;
-  }
-
-  // Sets a new TaskRunner for this message loop. The message loop must already
-  // have been bound to a thread prior to this call, and the task runner must
-  // belong to that thread. Note that changing the task runner will also affect
-  // the ThreadTaskRunnerHandle for the target thread. Must be called on the
-  // thread to which the message loop is bound.
+  // Sets a new TaskRunner for this message loop. If the message loop was
+  // already bound, this must be called on the thread to which it is bound.
   void SetTaskRunner(scoped_refptr<SingleThreadTaskRunner> task_runner);
 
-  // Clears task_runner() and the ThreadTaskRunnerHandle for the target thread.
-  // Must be called on the thread to which the message loop is bound.
-  void ClearTaskRunnerForTesting();
+  // Gets the TaskRunner associated with this message loop.
+  scoped_refptr<SingleThreadTaskRunner> task_runner() const;
 
-  // TODO(https://crbug.com/825327): Remove users of TaskObservers through
-  // MessageLoop::current() and migrate the type back here.
+  // TODO(yutak): Replace all the use sites with base::TaskObserver.
   using TaskObserver = MessageLoopCurrent::TaskObserver;
 
   // These functions can only be called on the same thread that |this| is
   // running on.
+  // These functions must not be called from a TaskObserver callback.
   void AddTaskObserver(TaskObserver* task_observer);
   void RemoveTaskObserver(TaskObserver* task_observer);
 
+  // Returns true if this is the active MessageLoop for the current thread.
+  bool IsBoundToCurrentThread() const;
+
   // Returns true if the message loop is idle (ignoring delayed tasks). This is
   // the same condition which triggers DoWork() to return false: i.e.
   // out of tasks which can be processed at the current run-level -- there might
   // be deferred non-nestable tasks remaining if currently in a nested run
   // level.
+  // TODO(alexclarke): Make this const when MessageLoopImpl goes away.
   bool IsIdleForTesting();
 
-  // Runs the specified PendingTask.
-  void RunTask(PendingTask* pending_task);
+  MessageLoopBase* GetMessageLoopBase();
 
   //----------------------------------------------------------------------------
  protected:
-  std::unique_ptr<MessagePump> pump_;
-
   using MessagePumpFactoryCallback =
       OnceCallback<std::unique_ptr<MessagePump>()>;
 
@@ -208,24 +300,33 @@ class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
   // specific type with a custom loop. The implementation does not call
   // BindToCurrentThread. If this constructor is invoked directly by a subclass,
   // then the subclass must subsequently bind the message loop.
-  MessageLoop(Type type, MessagePumpFactoryCallback pump_factory);
+  MessageLoop(Type type, std::unique_ptr<MessagePump> pump);
 
   // Configure various members and bind this message loop to the current thread.
   void BindToCurrentThread();
 
+  // A raw pointer to the MessagePump handed-off to |backend_|.
+  // Valid for the lifetime of |backend_|.
+  MessagePump* pump_ = nullptr;
+
+  // The SequenceManager-based implementation of the MessageLoop.
+  // TODO(crbug.com/891670): MessageLoopBase is now always a
+  // SequenceManagerImpl, this can be simplified but we also shouldn't publicly
+  // expose all of SequenceManagerImpl either.
+  const std::unique_ptr<MessageLoopBase> backend_;
+  // SequenceManager-based backend requires an explicit initialisation of the
+  // default task queue.
+  const scoped_refptr<sequence_manager::TaskQueue> default_task_queue_;
+
  private:
-  friend class internal::IncomingTaskQueue;
-  friend class MessageLoopCurrent;
-  friend class MessageLoopCurrentForIO;
-  friend class MessageLoopCurrentForUI;
+  friend class MessageLoopTypedTest;
   friend class ScheduleWorkTest;
   friend class Thread;
+  friend class sequence_manager::LazyThreadControllerForTest;
+  friend class sequence_manager::internal::SequenceManagerImpl;
   FRIEND_TEST_ALL_PREFIXES(MessageLoopTest, DeleteUnboundLoop);
 
   // Creates a MessageLoop without binding to a thread.
-  // If |type| is TYPE_CUSTOM non-null |pump_factory| must be also given
-  // to create a message pump for this message loop.  Otherwise a default
-  // message pump for the |type| is created.
   //
   // It is valid to call this to create a new message loop on one thread,
   // and then pass it to the thread where the message loop actually runs.
@@ -233,89 +334,22 @@ class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
   // thread the message loop runs on, before calling Run().
   // Before BindToCurrentThread() is called, only Post*Task() functions can
   // be called on the message loop.
-  static std::unique_ptr<MessageLoop> CreateUnbound(
-      Type type,
-      MessagePumpFactoryCallback pump_factory);
+  static std::unique_ptr<MessageLoop> CreateUnbound(Type type);
 
-  // Sets the ThreadTaskRunnerHandle for the current thread to point to the
-  // task runner for this message loop.
-  void SetThreadTaskRunnerHandle();
+  scoped_refptr<sequence_manager::TaskQueue> CreateDefaultTaskQueue();
 
-  // RunLoop::Delegate:
-  void Run(bool application_tasks_allowed) override;
-  void Quit() override;
-  void EnsureWorkScheduled() override;
-
-  // Called to process any delayed non-nestable tasks.
-  bool ProcessNextDelayedNonNestableTask();
-
-  // Calls RunTask or queues the pending_task on the deferred task list if it
-  // cannot be run right now.  Returns true if the task was run.
-  bool DeferOrRunPendingTask(PendingTask pending_task);
-
-  // Delete tasks that haven't run yet without running them.  Used in the
-  // destructor to make sure all the task's destructors get called.
-  void DeletePendingTasks();
-
-  // Wakes up the message pump. Can be called on any thread. The caller is
-  // responsible for synchronizing ScheduleWork() calls.
-  void ScheduleWork();
-
-  // MessagePump::Delegate methods:
-  bool DoWork() override;
-  bool DoDelayedWork(TimeTicks* next_delayed_work_time) override;
-  bool DoIdleWork() override;
+  std::unique_ptr<MessagePump> CreateMessagePump();
 
   const Type type_;
 
-#if defined(OS_WIN)
-  // Tracks if we have requested high resolution timers. Its only use is to
-  // turn off the high resolution timer upon loop destruction.
-  bool in_high_res_mode_ = false;
-#endif
-
-  // A recent snapshot of Time::Now(), used to check delayed_work_queue_.
-  TimeTicks recent_time_;
-
-  ObserverList<DestructionObserver> destruction_observers_;
-
-  // A boolean which prevents unintentional reentrant task execution (e.g. from
-  // induced nested message loops). As such, nested message loops will only
-  // process system messages (not application tasks) by default. A nested loop
-  // layer must have been explicitly granted permission to be able to execute
-  // application tasks. This is granted either by
-  // RunLoop::Type::kNestableTasksAllowed when the loop is driven by the
-  // application or by a ScopedNestableTaskAllower preceding a system call that
-  // is known to generate a system-driven nested loop.
-  bool task_execution_allowed_ = true;
-
-  // pump_factory_.Run() is called to create a message pump for this loop
-  // if type_ is TYPE_CUSTOM and pump_ is null.
-  MessagePumpFactoryCallback pump_factory_;
-
-  ObserverList<TaskObserver> task_observers_;
-
-  scoped_refptr<internal::IncomingTaskQueue> incoming_task_queue_;
-
-  // A task runner which we haven't bound to a thread yet.
-  scoped_refptr<internal::MessageLoopTaskRunner> unbound_task_runner_;
-
-  // The task runner associated with this message loop.
-  scoped_refptr<SingleThreadTaskRunner> task_runner_;
-  std::unique_ptr<ThreadTaskRunnerHandle> thread_task_runner_handle_;
+  // If set this will be returned by the next call to CreateMessagePump().
+  // This is only set if |type_| is TYPE_CUSTOM and |pump_| is null.
+  std::unique_ptr<MessagePump> custom_pump_;
 
   // Id of the thread this message loop is bound to. Initialized once when the
   // MessageLoop is bound to its thread and constant forever after.
   PlatformThreadId thread_id_ = kInvalidThreadId;
 
-  // Holds data stored through the SequenceLocalStorageSlot API.
-  internal::SequenceLocalStorageMap sequence_local_storage_map_;
-
-  // Enables the SequenceLocalStorageSlot API within its scope.
-  // Instantiated in BindToCurrentThread().
-  std::unique_ptr<internal::ScopedSetSequenceLocalStorageMapForCurrentThread>
-      scoped_set_sequence_local_storage_map_for_current_thread_;
-
   // Verifies that calls are made on the thread on which BindToCurrentThread()
   // was invoked.
   THREAD_CHECKER(bound_thread_checker_);
@@ -337,14 +371,7 @@ class BASE_EXPORT MessageLoop : public MessagePump::Delegate,
 //
 class BASE_EXPORT MessageLoopForUI : public MessageLoop {
  public:
-  MessageLoopForUI() : MessageLoop(TYPE_UI) {
-  }
-
-  explicit MessageLoopForUI(std::unique_ptr<MessagePump> pump);
-
-  // TODO(gab): Mass migrate callers to MessageLoopCurrentForUI::Get()/IsSet().
-  static MessageLoopCurrentForUI current();
-  static bool IsCurrent();
+  explicit MessageLoopForUI(Type type = TYPE_UI);
 
 #if defined(OS_IOS)
   // On iOS, the main message loop cannot be Run().  Instead call Attach(),
@@ -354,14 +381,23 @@ class BASE_EXPORT MessageLoopForUI : public MessageLoop {
 #endif
 
 #if defined(OS_ANDROID)
-  // On Android, the UI message loop is handled by Java side. So Run() should
-  // never be called. Instead use Start(), which will forward all the native UI
-  // events to the Java message loop.
-  void Start();
-
-  // In Android there are cases where we want to abort immediately without
+  // On Android there are cases where we want to abort immediately without
   // calling Quit(), in these cases we call Abort().
   void Abort();
+
+  // True if this message pump has been aborted.
+  bool IsAborted();
+
+  // Since Run() is never called on Android, and the message loop is run by the
+  // java Looper, quitting the RunLoop won't join the thread, so we need a
+  // callback to run when the RunLoop goes idle to let the Java thread know when
+  // it can safely quit.
+  void QuitWhenIdle(base::OnceClosure callback);
+#endif
+
+#if defined(OS_WIN)
+  // See method of the same name in the Windows MessagePumpForUI implementation.
+  void EnableWmQuit();
 #endif
 };
 
@@ -386,10 +422,6 @@ static_assert(sizeof(MessageLoop) == sizeof(MessageLoopForUI),
 class BASE_EXPORT MessageLoopForIO : public MessageLoop {
  public:
   MessageLoopForIO() : MessageLoop(TYPE_IO) {}
-
-  // TODO(gab): Mass migrate callers to MessageLoopCurrentForIO::Get()/IsSet().
-  static MessageLoopCurrentForIO current();
-  static bool IsCurrent();
 };
 
 // Do not add any member variables to MessageLoopForIO!  This is important b/c
