@@ -38,9 +38,9 @@
 #include "base/strings/stringize_macros.h"
 #include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
-#include "base/sys_info.h"
-#include "base/task_scheduler/post_task.h"
-#include "base/task_scheduler/task_scheduler.h"
+#include "base/system/sys_info.h"
+#include "base/task/post_task.h"
+#include "base/task/task_scheduler/task_scheduler.h"
 #include "base/test/gtest_util.h"
 #include "base/test/launcher/test_launcher_tracer.h"
 #include "base/test/launcher/test_results_tracker.h"
@@ -67,9 +67,14 @@
 #endif
 
 #if defined(OS_FUCHSIA)
+#include <lib/fdio/namespace.h>
 #include <lib/zx/job.h>
+#include "base/atomic_sequence_num.h"
+#include "base/base_paths_fuchsia.h"
 #include "base/fuchsia/default_job.h"
+#include "base/fuchsia/file_utils.h"
 #include "base/fuchsia/fuchsia_logging.h"
+#include "base/path_service.h"
 #endif
 
 namespace base {
@@ -84,8 +89,7 @@ const char kTestShardIndex[] = "GTEST_SHARD_INDEX";
 
 namespace {
 
-// Global tag for test runs where the results are incomplete or unreliable
-// for any reason, e.g. early exit because of too many broken tests.
+// Global tag for test runs where the results are unreliable for any reason.
 const char kUnreliableResultsTag[] = "UNRELIABLE_RESULTS";
 
 // Maximum time of no output after which we print list of processes still
@@ -135,18 +139,14 @@ TestLauncherTracer* GetTestLauncherTracer() {
 void CreateAndStartTaskScheduler(int num_parallel_jobs) {
   // These values are taken from TaskScheduler::StartWithDefaultParams(), which
   // is not used directly to allow a custom number of threads in the foreground
-  // blocking pool.
-  constexpr int kMaxBackgroundThreads = 1;
-  constexpr int kMaxBackgroundBlockingThreads = 2;
-  const int max_foreground_threads =
-      std::max(1, base::SysInfo::NumberOfProcessors());
+  // pool.
+  // TODO(etiennep): Change this to 2 in future CL.
+  constexpr int kMaxBackgroundThreads = 3;
   constexpr base::TimeDelta kSuggestedReclaimTime =
       base::TimeDelta::FromSeconds(30);
   base::TaskScheduler::Create("TestLauncher");
   base::TaskScheduler::GetInstance()->Start(
       {{kMaxBackgroundThreads, kSuggestedReclaimTime},
-       {kMaxBackgroundBlockingThreads, kSuggestedReclaimTime},
-       {max_foreground_threads, kSuggestedReclaimTime},
        {num_parallel_jobs, kSuggestedReclaimTime}});
 }
 
@@ -248,6 +248,7 @@ CommandLine PrepareCommandLineForGTest(const CommandLine& command_line,
 
   // Handled by the launcher process.
   switches.erase(kGTestRepeatFlag);
+  switches.erase(kIsolatedScriptTestRepeatFlag);
   switches.erase(kGTestShuffleFlag);
   switches.erase(kGTestRandomSeedFlag);
 
@@ -320,10 +321,58 @@ int LaunchChildTestProcessWithOptions(const CommandLine& command_line,
 #elif defined(OS_FUCHSIA)
   DCHECK(!new_options.job_handle);
 
+  // Set the clone policy, deliberately omitting FDIO_SPAWN_CLONE_NAMESPACE so
+  // that we can install a different /data.
+  new_options.spawn_flags = FDIO_SPAWN_CLONE_STDIO | FDIO_SPAWN_CLONE_JOB;
+
+  const base::FilePath kDataPath("/data");
+
+  // Clone all namespace entries from the current process, except /data, which
+  // is overridden below.
+  fdio_flat_namespace_t* flat_namespace = nullptr;
+  zx_status_t result = fdio_ns_export_root(&flat_namespace);
+  ZX_CHECK(ZX_OK == result, result) << "fdio_ns_export_root";
+  for (size_t i = 0; i < flat_namespace->count; ++i) {
+    base::FilePath path(flat_namespace->path[i]);
+    if (path == kDataPath) {
+      result = zx_handle_close(flat_namespace->handle[i]);
+      ZX_CHECK(ZX_OK == result, result) << "zx_handle_close";
+    } else {
+      new_options.paths_to_transfer.push_back(
+          {path, flat_namespace->handle[i]});
+    }
+  }
+  free(flat_namespace);
+
   zx::job job_handle;
-  zx_status_t result = zx::job::create(GetDefaultJob(), 0, &job_handle);
+  result = zx::job::create(*GetDefaultJob(), 0, &job_handle);
   ZX_CHECK(ZX_OK == result, result) << "zx_job_create";
   new_options.job_handle = job_handle.get();
+
+  // Give this test its own isolated /data directory by creating a new temporary
+  // subdirectory under data (/data/test-$PID) and binding that to /data on the
+  // child process.
+  CHECK(base::PathExists(kDataPath));
+
+  // Create the test subdirectory with a name that is unique to the child test
+  // process (qualified by parent PID and an autoincrementing test process
+  // index).
+  static base::AtomicSequenceNumber child_launch_index;
+  base::FilePath nested_data_path = kDataPath.AppendASCII(
+      base::StringPrintf("test-%" PRIuS "-%d", base::Process::Current().Pid(),
+                         child_launch_index.GetNext()));
+  CHECK(!base::DirectoryExists(nested_data_path));
+  CHECK(base::CreateDirectory(nested_data_path));
+  DCHECK(base::DirectoryExists(nested_data_path));
+
+  // Bind the new test subdirectory to /data in the child process' namespace.
+  new_options.paths_to_transfer.push_back(
+      {kDataPath, base::fuchsia::GetHandleFromFile(
+                      base::File(nested_data_path,
+                                 base::File::FLAG_OPEN | base::File::FLAG_READ |
+                                     base::File::FLAG_DELETE_ON_CLOSE))
+                      .release()});
+
 #endif  // defined(OS_FUCHSIA)
 
 #if defined(OS_LINUX)
@@ -400,7 +449,12 @@ int LaunchChildTestProcessWithOptions(const CommandLine& command_line,
     AutoLock lock(*GetLiveProcessesLock());
 
 #if defined(OS_FUCHSIA)
-    CHECK_EQ(job_handle.kill(), ZX_OK);
+    zx_status_t status = job_handle.kill();
+    ZX_CHECK(status == ZX_OK, status);
+
+    // The child process' data dir should have been deleted automatically,
+    // thanks to the DELETE_ON_CLOSE flag.
+    DCHECK(!base::DirectoryExists(nested_data_path));
 #elif defined(OS_POSIX)
     if (exit_code != 0) {
       // On POSIX, in case the test does not exit cleanly, either due to a crash
@@ -508,6 +562,20 @@ void DoLaunchChildTestProcess(
                output_file_contents));
 }
 
+std::vector<std::string> ExtractTestsFromFilter(const std::string& filter,
+                                                bool double_colon_supported) {
+  std::vector<std::string> tests;
+  if (double_colon_supported) {
+    tests =
+        SplitString(filter, "::", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
+  }
+  if (tests.size() <= 1) {
+    tests =
+        SplitString(filter, ":", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
+  }
+  return tests;
+}
+
 }  // namespace
 
 const char kGTestBreakOnFailure[] = "gtest_break_on_failure";
@@ -520,6 +588,10 @@ const char kGTestRunDisabledTestsFlag[] = "gtest_also_run_disabled_tests";
 const char kGTestOutputFlag[] = "gtest_output";
 const char kGTestShuffleFlag[] = "gtest_shuffle";
 const char kGTestRandomSeedFlag[] = "gtest_random_seed";
+const char kIsolatedScriptRunDisabledTestsFlag[] =
+    "isolated-script-test-also-run-disabled-tests";
+const char kIsolatedScriptTestFilterFlag[] = "isolated-script-test-filter";
+const char kIsolatedScriptTestRepeatFlag[] = "isolated-script-test-repeat";
 
 TestLauncherDelegate::~TestLauncherDelegate() = default;
 
@@ -726,7 +798,7 @@ void TestLauncher::OnTestFinished(const TestResult& original_result) {
     KillSpawnedTestProcesses();
 #endif  // defined(OS_POSIX)
 
-    MaybeSaveSummaryAsJSON({"BROKEN_TEST_EARLY_EXIT", kUnreliableResultsTag});
+    MaybeSaveSummaryAsJSON({"BROKEN_TEST_EARLY_EXIT"});
 
     exit(1);
   }
@@ -746,7 +818,6 @@ void TestLauncher::OnTestFinished(const TestResult& original_result) {
     fflush(stdout);
 
     results_tracker_.AddGlobalTag("BROKEN_TEST_SKIPPED_RETRIES");
-    results_tracker_.AddGlobalTag(kUnreliableResultsTag);
 
     OnTestIterationFinished();
     return;
@@ -886,6 +957,13 @@ bool TestLauncher::Init() {
     LOG(ERROR) << "Invalid value for " << kGTestRepeatFlag;
     return false;
   }
+  if (command_line->HasSwitch(kIsolatedScriptTestRepeatFlag) &&
+      !StringToInt(
+          command_line->GetSwitchValueASCII(kIsolatedScriptTestRepeatFlag),
+          &cycles_)) {
+    LOG(ERROR) << "Invalid value for " << kIsolatedScriptTestRepeatFlag;
+    return false;
+  }
 
   if (command_line->HasSwitch(switches::kTestLauncherRetryLimit)) {
     int retry_limit = -1;
@@ -897,7 +975,22 @@ bool TestLauncher::Init() {
     }
 
     retry_limit_ = retry_limit;
-  } else if (!command_line->HasSwitch(kGTestFilterFlag) || BotModeEnabled()) {
+  } else if (command_line->HasSwitch(
+                 switches::kIsolatedScriptTestLauncherRetryLimit)) {
+    int retry_limit = -1;
+    if (!StringToInt(command_line->GetSwitchValueASCII(
+                         switches::kIsolatedScriptTestLauncherRetryLimit),
+                     &retry_limit) ||
+        retry_limit < 0) {
+      LOG(ERROR) << "Invalid value for "
+                 << switches::kIsolatedScriptTestLauncherRetryLimit;
+      return false;
+    }
+
+    retry_limit_ = retry_limit;
+  } else if (BotModeEnabled() ||
+             !(command_line->HasSwitch(kGTestFilterFlag) ||
+               command_line->HasSwitch(kIsolatedScriptTestFilterFlag))) {
     // Retry failures 3 times by default if we are running all of the tests or
     // in bot mode.
     retry_limit_ = 3;
@@ -947,30 +1040,37 @@ bool TestLauncher::Init() {
   std::vector<std::string> positive_gtest_filter;
 
   if (command_line->HasSwitch(switches::kTestLauncherFilterFile)) {
-    base::FilePath filter_file_path = base::MakeAbsoluteFilePath(
-        command_line->GetSwitchValuePath(switches::kTestLauncherFilterFile));
-    if (!LoadFilterFile(filter_file_path, &positive_file_filter,
-                        &negative_test_filter_))
-      return false;
+    auto filter =
+        command_line->GetSwitchValueNative(switches::kTestLauncherFilterFile);
+    for (auto filter_file :
+         SplitString(filter, FILE_PATH_LITERAL(";"), base::TRIM_WHITESPACE,
+                     base::SPLIT_WANT_ALL)) {
+      base::FilePath filter_file_path =
+          base::MakeAbsoluteFilePath(FilePath(filter_file));
+      if (!LoadFilterFile(filter_file_path, &positive_file_filter,
+                          &negative_test_filter_))
+        return false;
+    }
   }
 
   // Split --gtest_filter at '-', if there is one, to separate into
   // positive filter and negative filter portions.
-  std::string filter = command_line->GetSwitchValueASCII(kGTestFilterFlag);
+  bool double_colon_supported = !command_line->HasSwitch(kGTestFilterFlag);
+  std::string filter = command_line->GetSwitchValueASCII(
+      double_colon_supported ? kIsolatedScriptTestFilterFlag
+                             : kGTestFilterFlag);
   size_t dash_pos = filter.find('-');
   if (dash_pos == std::string::npos) {
     positive_gtest_filter =
-        SplitString(filter, ":", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
+        ExtractTestsFromFilter(filter, double_colon_supported);
   } else {
     // Everything up to the dash.
-    positive_gtest_filter =
-        SplitString(filter.substr(0, dash_pos), ":", base::TRIM_WHITESPACE,
-                    base::SPLIT_WANT_ALL);
+    positive_gtest_filter = ExtractTestsFromFilter(filter.substr(0, dash_pos),
+                                                   double_colon_supported);
 
     // Everything after the dash.
-    for (std::string pattern :
-         SplitString(filter.substr(dash_pos + 1), ":", base::TRIM_WHITESPACE,
-                     base::SPLIT_WANT_ALL)) {
+    for (std::string pattern : ExtractTestsFromFilter(
+             filter.substr(dash_pos + 1), double_colon_supported)) {
       negative_test_filter_.push_back(pattern);
     }
   }
@@ -1067,16 +1167,15 @@ void TestLauncher::CombinePositiveTestFilters(
   // If two positive filters are present, only run tests that match a pattern
   // in both filters.
   if (!filter_a.empty() && !filter_b.empty()) {
-    for (size_t i = 0; i < tests_.size(); i++) {
-      std::string test_name =
-          FormatFullTestName(tests_[i].test_case_name, tests_[i].test_name);
+    for (const auto& i : tests_) {
+      std::string test_name = FormatFullTestName(i.test_case_name, i.test_name);
       bool found_a = false;
       bool found_b = false;
-      for (size_t k = 0; k < filter_a.size(); ++k) {
-        found_a = found_a || MatchPattern(test_name, filter_a[k]);
+      for (const auto& k : filter_a) {
+        found_a = found_a || MatchPattern(test_name, k);
       }
-      for (size_t k = 0; k < filter_b.size(); ++k) {
-        found_b = found_b || MatchPattern(test_name, filter_b[k]);
+      for (const auto& k : filter_b) {
+        found_b = found_b || MatchPattern(test_name, k);
       }
       if (found_a && found_b) {
         positive_test_filter_.push_back(test_name);
@@ -1102,7 +1201,8 @@ void TestLauncher::RunTests() {
       results_tracker_.AddDisabledTest(test_name);
 
       // Skip disabled tests unless explicitly requested.
-      if (!command_line->HasSwitch(kGTestRunDisabledTestsFlag))
+      if (!command_line->HasSwitch(kGTestRunDisabledTestsFlag) &&
+          !command_line->HasSwitch(kIsolatedScriptRunDisabledTestsFlag))
         continue;
     }
 
@@ -1165,7 +1265,8 @@ void TestLauncher::RunTests() {
   }
 
   // Save an early test summary in case the launcher crashes or gets killed.
-  MaybeSaveSummaryAsJSON({"EARLY_SUMMARY", kUnreliableResultsTag});
+  results_tracker_.GeneratePlaceholderIteration();
+  MaybeSaveSummaryAsJSON({"EARLY_SUMMARY"});
 
   test_started_count_ = launcher_delegate_->RunTests(this, test_names);
 
@@ -1213,7 +1314,7 @@ void TestLauncher::OnShutdownPipeReadable() {
 
   KillSpawnedTestProcesses();
 
-  MaybeSaveSummaryAsJSON({"CAUGHT_TERMINATION_SIGNAL", kUnreliableResultsTag});
+  MaybeSaveSummaryAsJSON({"CAUGHT_TERMINATION_SIGNAL"});
 
   // The signal would normally kill the process, so exit now.
   _exit(1);
@@ -1300,7 +1401,9 @@ size_t NumParallelJobs() {
     }
     return jobs;
   }
-  if (command_line->HasSwitch(kGTestFilterFlag) && !BotModeEnabled()) {
+  if (!BotModeEnabled() &&
+      (command_line->HasSwitch(kGTestFilterFlag) ||
+       command_line->HasSwitch(kIsolatedScriptTestFilterFlag))) {
     // Do not run jobs in parallel by default if we are running a subset of
     // the tests and if bot mode is off.
     return 1U;
