@@ -12,8 +12,9 @@
 #include "base/build_time.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
-#include "base/macros.h"
+#include "base/stl_util.h"
 #include "base/strings/stringprintf.h"
+#include "base/test/gtest_util.h"
 #include "base/threading/platform_thread.h"
 #include "base/time/time_override.h"
 #include "build/build_config.h"
@@ -158,7 +159,7 @@ TEST_F(TimeTest, UTCTimeT) {
   struct tm tms;
 #if defined(OS_WIN)
   gmtime_s(&tms, &now_t_1);
-#elif defined(OS_POSIX)
+#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
   gmtime_r(&now_t_1, &tms);
 #endif
 
@@ -204,7 +205,7 @@ TEST_F(TimeTest, LocalTimeT) {
   struct tm tms;
 #if defined(OS_WIN)
   localtime_s(&tms, &now_t_1);
-#elif defined(OS_POSIX)
+#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
   localtime_r(&now_t_1, &tms);
 #endif
 
@@ -240,13 +241,13 @@ TEST_F(TimeTest, JsTime) {
   EXPECT_EQ(800730.0, t.ToJsTime());
 }
 
-#if defined(OS_POSIX)
+#if defined(OS_POSIX) || defined(OS_FUCHSIA)
 TEST_F(TimeTest, FromTimeVal) {
   Time now = Time::Now();
   Time also_now = Time::FromTimeVal(now.ToTimeVal());
   EXPECT_EQ(now, also_now);
 }
-#endif  // OS_POSIX
+#endif  // defined(OS_POSIX) || defined(OS_FUCHSIA)
 
 TEST_F(TimeTest, FromExplodedWithMilliseconds) {
   // Some platform implementations of FromExploded are liable to drop
@@ -293,6 +294,15 @@ TEST_F(TimeTest, UTCExplode) {
   EXPECT_TRUE((a - b) < TimeDelta::FromSeconds(1));
 }
 
+TEST_F(TimeTest, UTCMidnight) {
+  Time::Exploded exploded;
+  Time::Now().UTCMidnight().UTCExplode(&exploded);
+  EXPECT_EQ(0, exploded.hour);
+  EXPECT_EQ(0, exploded.minute);
+  EXPECT_EQ(0, exploded.second);
+  EXPECT_EQ(0, exploded.millisecond);
+}
+
 TEST_F(TimeTest, LocalMidnight) {
   Time::Exploded exploded;
   Time::Now().LocalMidnight().LocalExplode(&exploded);
@@ -310,8 +320,8 @@ TEST_F(TimeTest, ParseTimeTest1) {
   char time_buf[64] = {};
 #if defined(OS_WIN)
   localtime_s(&local_time, &current_time);
-  asctime_s(time_buf, arraysize(time_buf), &local_time);
-#elif defined(OS_POSIX)
+  asctime_s(time_buf, base::size(time_buf), &local_time);
+#elif defined(OS_POSIX) || defined(OS_FUCHSIA)
   localtime_r(&current_time, &local_time);
   asctime_r(&local_time, time_buf);
 #endif
@@ -647,7 +657,7 @@ TEST_F(TimeTest, MaxConversions) {
   EXPECT_TRUE(t.is_max());
   EXPECT_EQ(std::numeric_limits<time_t>::max(), t.ToTimeT());
 
-#if defined(OS_POSIX)
+#if defined(OS_POSIX) || defined(OS_FUCHSIA)
   struct timeval tval;
   tval.tv_sec = std::numeric_limits<time_t>::max();
   tval.tv_usec = static_cast<suseconds_t>(Time::kMicrosecondsPerSecond) - 1;
@@ -719,7 +729,7 @@ TEST_F(TimeTest, FromExploded_MinMax) {
   if (Time::kExplodedMinYear != std::numeric_limits<int>::min()) {
     exploded.year = Time::kExplodedMinYear;
     EXPECT_TRUE(Time::FromUTCExploded(exploded, &parsed_time));
-#if !defined(OS_WIN)
+#if defined(OS_POSIX) || defined(OS_FUCHSIA)
     // On Windows, January 1, 1601 00:00:00 is actually the null time.
     EXPECT_FALSE(parsed_time.is_null());
 #endif
@@ -989,14 +999,7 @@ TEST(ThreadTicks, MAYBE_NowOverride) {
   EXPECT_GT(ThreadTicks::Max(), subtle::ThreadTicksNowIgnoringOverride());
 }
 
-// Fails frequently on Android http://crbug.com/352633 with:
-// Expected: (delta_thread.InMicroseconds()) > (0), actual: 0 vs 0
-#if defined(OS_ANDROID)
-#define MAYBE_ThreadNow DISABLED_ThreadNow
-#else
-#define MAYBE_ThreadNow ThreadNow
-#endif
-TEST(ThreadTicks, MAYBE_ThreadNow) {
+TEST(ThreadTicks, ThreadNow) {
   if (ThreadTicks::IsSupported()) {
     ThreadTicks::WaitUntilInitialized();
     TimeTicks begin = TimeTicks::Now();
@@ -1010,7 +1013,7 @@ TEST(ThreadTicks, MAYBE_ThreadNow) {
     TimeDelta delta = end - begin;
     TimeDelta delta_thread = end_thread - begin_thread;
     // Make sure that some thread time have elapsed.
-    EXPECT_GT(delta_thread.InMicroseconds(), 0);
+    EXPECT_GE(delta_thread.InMicroseconds(), 0);
     // But the thread time is at least 9ms less than clock time.
     TimeDelta difference = delta - delta_thread;
     EXPECT_GE(difference.InMicroseconds(), 9000);
@@ -1129,7 +1132,48 @@ TEST(TimeDelta, FromAndIn) {
   EXPECT_EQ(TimeDelta::FromNanosecondsD(12345.678).InNanoseconds(), 12000);
 }
 
-#if defined(OS_POSIX)
+TEST(TimeDelta, InRoundsTowardsZero) {
+  EXPECT_EQ(TimeDelta::FromHours(23).InDays(), 0);
+  EXPECT_EQ(TimeDelta::FromHours(-23).InDays(), 0);
+  EXPECT_EQ(TimeDelta::FromMinutes(59).InHours(), 0);
+  EXPECT_EQ(TimeDelta::FromMinutes(-59).InHours(), 0);
+  EXPECT_EQ(TimeDelta::FromSeconds(59).InMinutes(), 0);
+  EXPECT_EQ(TimeDelta::FromSeconds(-59).InMinutes(), 0);
+  EXPECT_EQ(TimeDelta::FromMilliseconds(999).InSeconds(), 0);
+  EXPECT_EQ(TimeDelta::FromMilliseconds(-999).InSeconds(), 0);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(999).InMilliseconds(), 0);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(-999).InMilliseconds(), 0);
+}
+
+TEST(TimeDelta, InDaysFloored) {
+  EXPECT_EQ(TimeDelta::FromHours(-25).InDaysFloored(), -2);
+  EXPECT_EQ(TimeDelta::FromHours(-24).InDaysFloored(), -1);
+  EXPECT_EQ(TimeDelta::FromHours(-23).InDaysFloored(), -1);
+
+  EXPECT_EQ(TimeDelta::FromHours(-1).InDaysFloored(), -1);
+  EXPECT_EQ(TimeDelta::FromHours(0).InDaysFloored(), 0);
+  EXPECT_EQ(TimeDelta::FromHours(1).InDaysFloored(), 0);
+
+  EXPECT_EQ(TimeDelta::FromHours(23).InDaysFloored(), 0);
+  EXPECT_EQ(TimeDelta::FromHours(24).InDaysFloored(), 1);
+  EXPECT_EQ(TimeDelta::FromHours(25).InDaysFloored(), 1);
+}
+
+TEST(TimeDelta, InMillisecondsRoundedUp) {
+  EXPECT_EQ(TimeDelta::FromMicroseconds(-1001).InMillisecondsRoundedUp(), -1);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(-1000).InMillisecondsRoundedUp(), -1);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(-999).InMillisecondsRoundedUp(), 0);
+
+  EXPECT_EQ(TimeDelta::FromMicroseconds(-1).InMillisecondsRoundedUp(), 0);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(0).InMillisecondsRoundedUp(), 0);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(1).InMillisecondsRoundedUp(), 1);
+
+  EXPECT_EQ(TimeDelta::FromMicroseconds(999).InMillisecondsRoundedUp(), 1);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(1000).InMillisecondsRoundedUp(), 1);
+  EXPECT_EQ(TimeDelta::FromMicroseconds(1001).InMillisecondsRoundedUp(), 2);
+}
+
+#if defined(OS_POSIX) || defined(OS_FUCHSIA)
 TEST(TimeDelta, TimeSpecConversion) {
   TimeDelta delta = TimeDelta::FromSeconds(0);
   struct timespec result = delta.ToTimeSpec();
@@ -1155,7 +1199,7 @@ TEST(TimeDelta, TimeSpecConversion) {
   EXPECT_EQ(result.tv_nsec, 1000);
   EXPECT_EQ(delta, TimeDelta::FromTimeSpec(result));
 }
-#endif  // OS_POSIX
+#endif  // defined(OS_POSIX) || defined(OS_FUCHSIA)
 
 // Our internal time format is serialized in things like databases, so it's
 // important that it's consistent across all our platforms.  We use the 1601
@@ -1291,8 +1335,10 @@ TEST(TimeDelta, MaxConversions) {
   EXPECT_TRUE(TimeDelta::FromSecondsD(std::numeric_limits<double>::infinity())
                   .is_max());
 
-  constexpr double max_d = max_int;
-  constexpr double min_d = min_int;
+  // Note that max_int/min_int will be rounded when converted to doubles - they
+  // can't be exactly represented.
+  constexpr double max_d = static_cast<double>(max_int);
+  constexpr double min_d = static_cast<double>(min_int);
 
   static_assert(
       TimeDelta::FromSecondsD(max_d / Time::kMicrosecondsPerSecond + 1)
@@ -1445,6 +1491,14 @@ TEST(TimeDelta, Overflows) {
   EXPECT_TRUE((large_delta / 0.5).is_max());
   EXPECT_TRUE((large_delta / -0.5).is_min());
 
+  // Test that double conversions overflow to infinity.
+  EXPECT_EQ((large_delta + kOneSecond).InSecondsF(),
+            std::numeric_limits<double>::infinity());
+  EXPECT_EQ((large_delta + kOneSecond).InMillisecondsF(),
+            std::numeric_limits<double>::infinity());
+  EXPECT_EQ((large_delta + kOneSecond).InMicrosecondsF(),
+            std::numeric_limits<double>::infinity());
+
   // Test +=, -=, *= and /= operators.
   TimeDelta delta = large_delta;
   delta += kOneSecond;
@@ -1489,6 +1543,126 @@ TEST(TimeDelta, Overflows) {
   EXPECT_EQ(kOneSecond, (ticks_now + kOneSecond) - ticks_now);
 }
 
+TEST(TimeBase, AddSubDeltaSaturates) {
+  constexpr TimeTicks kLargeTimeTicks =
+      TimeTicks::FromInternalValue(std::numeric_limits<int64_t>::max() - 1);
+
+  constexpr TimeTicks kLargeNegativeTimeTicks =
+      TimeTicks::FromInternalValue(std::numeric_limits<int64_t>::min() + 1);
+
+  EXPECT_TRUE((kLargeTimeTicks + TimeDelta::Max()).is_max())
+      << (kLargeTimeTicks + TimeDelta::Max());
+  EXPECT_TRUE((kLargeNegativeTimeTicks + TimeDelta::Max()).is_max())
+      << (kLargeNegativeTimeTicks + TimeDelta::Max());
+  EXPECT_TRUE((kLargeTimeTicks - TimeDelta::Max()).is_min())
+      << (kLargeTimeTicks - TimeDelta::Max());
+  EXPECT_TRUE((kLargeNegativeTimeTicks - TimeDelta::Max()).is_min())
+      << (kLargeNegativeTimeTicks - TimeDelta::Max());
+  EXPECT_TRUE((TimeTicks() + TimeDelta::Max()).is_max())
+      << (TimeTicks() + TimeDelta::Max());
+  EXPECT_TRUE((TimeTicks() - TimeDelta::Max()).is_min())
+      << (TimeTicks() - TimeDelta::Max());
+  EXPECT_TRUE((TimeTicks::Now() + TimeDelta::Max()).is_max())
+      << (TimeTicks::Now() + TimeDelta::Max());
+  EXPECT_TRUE((TimeTicks::Now() - TimeDelta::Max()).is_min())
+      << (TimeTicks::Now() - TimeDelta::Max());
+
+  EXPECT_TRUE((kLargeTimeTicks + TimeDelta::Min()).is_min())
+      << (kLargeTimeTicks + TimeDelta::Min());
+  EXPECT_TRUE((kLargeNegativeTimeTicks + TimeDelta::Min()).is_min())
+      << (kLargeNegativeTimeTicks + TimeDelta::Min());
+  EXPECT_TRUE((kLargeTimeTicks - TimeDelta::Min()).is_max())
+      << (kLargeTimeTicks - TimeDelta::Min());
+  EXPECT_TRUE((kLargeNegativeTimeTicks - TimeDelta::Min()).is_max())
+      << (kLargeNegativeTimeTicks - TimeDelta::Min());
+  EXPECT_TRUE((TimeTicks() + TimeDelta::Min()).is_min())
+      << (TimeTicks() + TimeDelta::Min());
+  EXPECT_TRUE((TimeTicks() - TimeDelta::Min()).is_max())
+      << (TimeTicks() - TimeDelta::Min());
+  EXPECT_TRUE((TimeTicks::Now() + TimeDelta::Min()).is_min())
+      << (TimeTicks::Now() + TimeDelta::Min());
+  EXPECT_TRUE((TimeTicks::Now() - TimeDelta::Min()).is_max())
+      << (TimeTicks::Now() - TimeDelta::Min());
+}
+
+TEST(TimeBase, AddSubInfinities) {
+  // CHECK when adding opposite signs or subtracting same sign.
+  EXPECT_CHECK_DEATH({ TimeTicks::Min() + TimeDelta::Max(); });
+  EXPECT_CHECK_DEATH({ TimeTicks::Max() + TimeDelta::Min(); });
+  EXPECT_CHECK_DEATH({ TimeTicks::Min() - TimeDelta::Min(); });
+  EXPECT_CHECK_DEATH({ TimeTicks::Max() - TimeDelta::Max(); });
+
+  // Saturates when adding same sign or subtracting opposite signs.
+  EXPECT_TRUE((TimeTicks::Max() + TimeDelta::Max()).is_max());
+  EXPECT_TRUE((TimeTicks::Min() + TimeDelta::Min()).is_min());
+  EXPECT_TRUE((TimeTicks::Max() - TimeDelta::Min()).is_max());
+  EXPECT_TRUE((TimeTicks::Min() - TimeDelta::Max()).is_min());
+}
+
+constexpr TimeTicks TestTimeTicksConstexprCopyAssignment() {
+  TimeTicks a = TimeTicks::FromInternalValue(12345);
+  TimeTicks b;
+  b = a;
+  return b;
+}
+
+TEST(TimeTicks, ConstexprAndTriviallyCopiable) {
+  // "Trivially copyable" is necessary for use in std::atomic<TimeTicks>.
+  static_assert(std::is_trivially_copyable<TimeTicks>(), "");
+
+  // Copy ctor.
+  constexpr TimeTicks a = TimeTicks::FromInternalValue(12345);
+  constexpr TimeTicks b{a};
+  static_assert(a.ToInternalValue() == b.ToInternalValue(), "");
+
+  // Copy assignment.
+  static_assert(a.ToInternalValue() ==
+                    TestTimeTicksConstexprCopyAssignment().ToInternalValue(),
+                "");
+}
+
+constexpr ThreadTicks TestThreadTicksConstexprCopyAssignment() {
+  ThreadTicks a = ThreadTicks::FromInternalValue(12345);
+  ThreadTicks b;
+  b = a;
+  return b;
+}
+
+TEST(ThreadTicks, ConstexprAndTriviallyCopiable) {
+  // "Trivially copyable" is necessary for use in std::atomic<ThreadTicks>.
+  static_assert(std::is_trivially_copyable<ThreadTicks>(), "");
+
+  // Copy ctor.
+  constexpr ThreadTicks a = ThreadTicks::FromInternalValue(12345);
+  constexpr ThreadTicks b{a};
+  static_assert(a.ToInternalValue() == b.ToInternalValue(), "");
+
+  // Copy assignment.
+  static_assert(a.ToInternalValue() ==
+                    TestThreadTicksConstexprCopyAssignment().ToInternalValue(),
+                "");
+}
+
+constexpr TimeDelta TestTimeDeltaConstexprCopyAssignment() {
+  TimeDelta a = TimeDelta::FromSeconds(1);
+  TimeDelta b;
+  b = a;
+  return b;
+}
+
+TEST(TimeDelta, ConstexprAndTriviallyCopiable) {
+  // "Trivially copyable" is necessary for use in std::atomic<TimeDelta>.
+  static_assert(std::is_trivially_copyable<TimeDelta>(), "");
+
+  // Copy ctor.
+  constexpr TimeDelta a = TimeDelta::FromSeconds(1);
+  constexpr TimeDelta b{a};
+  static_assert(a == b, "");
+
+  // Copy assignment.
+  static_assert(a == TestTimeDeltaConstexprCopyAssignment(), "");
+}
+
 TEST(TimeDeltaLogging, DCheckEqCompiles) {
   DCHECK_EQ(TimeDelta(), TimeDelta());
 }
