@@ -49,7 +49,6 @@ const char* const kDumpProviderWhitelist[] = {
     "MojoMessages",
     "PartitionAlloc",
     "ProcessMemoryMetrics",
-    "RenderProcessHost",
     "SharedMemoryTracker",
     "Skia",
     "Sql",
@@ -67,6 +66,7 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "blink_gc/allocated_objects",
     "blink_objects/AdSubframe",
     "blink_objects/AudioHandler",
+    "blink_objects/ContextLifecycleStateObserver",
     "blink_objects/DetachedScriptState",
     "blink_objects/Document",
     "blink_objects/Frame",
@@ -78,7 +78,6 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "blink_objects/Resource",
     "blink_objects/RTCPeerConnection",
     "blink_objects/ScriptPromise",
-    "blink_objects/PausableObject",
     "blink_objects/V8PerContextData",
     "blink_objects/WorkerGlobalScope",
     "blink_objects/UACSSResource",
@@ -94,9 +93,9 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "extensions/value_store/Extensions.Database.Value.Restore/0x?",
     "font_caches/font_platform_data_cache",
     "font_caches/shape_caches",
-    "gpu/gl/buffers/share_group_0x?",
-    "gpu/gl/renderbuffers/share_group_0x?",
-    "gpu/gl/textures/share_group_0x?",
+    "gpu/gl/buffers/context_group_0x?",
+    "gpu/gl/renderbuffers/context_group_0x?",
+    "gpu/gl/textures/context_group_0x?",
     "history/delta_file_service/leveldb_0x?",
     "history/usage_reports_buffer/leveldb_0x?",
     "java_heap",
@@ -120,7 +119,6 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "mojo/message_pipe",
     "mojo/platform_handle",
     "mojo/queued_ipc_channel_message/0x?",
-    "mojo/render_process_host/0x?",
     "mojo/shared_buffer",
     "mojo/unknown",
     "mojo/watcher",
@@ -128,8 +126,8 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "net/http_network_session_0x?/quic_stream_factory",
     "net/http_network_session_0x?/socket_pool",
     "net/http_network_session_0x?/spdy_session_pool",
+    "net/http_network_session_0x?/ssl_client_session_cache",
     "net/http_network_session_0x?/stream_factory",
-    "net/ssl_session_cache",
     "net/url_request_context",
     "net/url_request_context/app_request",
     "net/url_request_context/app_request/0x?",
@@ -189,6 +187,17 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "net/url_request_context/main_media/0x?/http_cache/memory_backend",
     "net/url_request_context/main_media/0x?/http_cache/simple_backend",
     "net/url_request_context/main_media/0x?/http_network_session",
+    "net/url_request_context/mirroring",
+    "net/url_request_context/mirroring/0x?",
+    "net/url_request_context/mirroring/0x?/cookie_monster",
+    "net/url_request_context/mirroring/0x?/cookie_monster/cookies",
+    "net/url_request_context/mirroring/0x?/cookie_monster/tasks_pending_global",
+    "net/url_request_context/mirroring/0x?/cookie_monster/"
+    "tasks_pending_for_key",
+    "net/url_request_context/mirroring/0x?/http_cache",
+    "net/url_request_context/mirroring/0x?/http_cache/memory_backend",
+    "net/url_request_context/mirroring/0x?/http_cache/simple_backend",
+    "net/url_request_context/mirroring/0x?/http_network_session",
     "net/url_request_context/proxy",
     "net/url_request_context/proxy/0x?",
     "net/url_request_context/proxy/0x?/cookie_monster",
@@ -250,18 +259,47 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "sqlite",
     "ui/resource_manager_0x?/default_resource/0x?",
     "ui/resource_manager_0x?/tinted_resource",
-    "v8/isolate_0x?/contexts/detached_context",
-    "v8/isolate_0x?/contexts/native_context",
-    "v8/isolate_0x?/heap_spaces",
-    "v8/isolate_0x?/heap_spaces/code_space",
-    "v8/isolate_0x?/heap_spaces/large_object_space",
-    "v8/isolate_0x?/heap_spaces/map_space",
-    "v8/isolate_0x?/heap_spaces/new_space",
-    "v8/isolate_0x?/heap_spaces/old_space",
-    "v8/isolate_0x?/heap_spaces/read_only_space",
-    "v8/isolate_0x?/malloc",
-    "v8/isolate_0x?/zapped_for_debug",
     "site_storage/blob_storage/0x?",
+    "v8/main/code_stats",
+    "v8/main/contexts/detached_context",
+    "v8/main/contexts/native_context",
+    "v8/main/heap/code_space",
+    "v8/main/heap/code_stats",
+    "v8/main/heap/code_large_object_space",
+    "v8/main/heap/large_object_space",
+    "v8/main/heap/map_space",
+    "v8/main/heap/new_large_object_space",
+    "v8/main/heap/new_space",
+    "v8/main/heap/old_space",
+    "v8/main/heap/read_only_space",
+    "v8/main/malloc",
+    "v8/main/zapped_for_debug",
+    "v8/utility/code_stats",
+    "v8/utility/contexts/detached_context",
+    "v8/utility/contexts/native_context",
+    "v8/utility/heap/code_space",
+    "v8/utility/heap/code_large_object_space",
+    "v8/utility/heap/large_object_space",
+    "v8/utility/heap/map_space",
+    "v8/utility/heap/new_large_object_space",
+    "v8/utility/heap/new_space",
+    "v8/utility/heap/old_space",
+    "v8/utility/heap/read_only_space",
+    "v8/utility/malloc",
+    "v8/utility/zapped_for_debug",
+    "v8/workers/code_stats/isolate_0x?",
+    "v8/workers/contexts/detached_context/isolate_0x?",
+    "v8/workers/contexts/native_context/isolate_0x?",
+    "v8/workers/heap/code_space/isolate_0x?",
+    "v8/workers/heap/code_large_object_space/isolate_0x?",
+    "v8/workers/heap/large_object_space/isolate_0x?",
+    "v8/workers/heap/map_space/isolate_0x?",
+    "v8/workers/heap/new_large_object_space/isolate_0x?",
+    "v8/workers/heap/new_space/isolate_0x?",
+    "v8/workers/heap/old_space/isolate_0x?",
+    "v8/workers/heap/read_only_space/isolate_0x?",
+    "v8/workers/malloc/isolate_0x?",
+    "v8/workers/zapped_for_debug/isolate_0x?",
     "site_storage/index_db/db_0x?",
     "site_storage/index_db/memenv_0x?",
     "site_storage/localstorage/0x?/cache_size",
@@ -300,6 +338,8 @@ const char* const kAllocatorDumpNameWhitelist[] = {
     "sync/0x?/model_type/PRIORITY_PREFERENCE",
     "sync/0x?/model_type/READING_LIST",
     "sync/0x?/model_type/SEARCH_ENGINE",
+    "sync/0x?/model_type/SECURITY_EVENT",
+    "sync/0x?/model_type/SEND_TAB_TO_SELF",
     "sync/0x?/model_type/SESSION",
     "sync/0x?/model_type/SYNCED_NOTIFICATION",
     "sync/0x?/model_type/SYNCED_NOTIFICATION_APP_INFO",
