@@ -6,6 +6,7 @@
 
 #include <stddef.h>
 
+#include <algorithm>
 #include <utility>
 
 #include "base/json/json_reader.h"
@@ -31,6 +32,7 @@ const char kEnableArgumentFilter[] = "enable-argument-filter";
 
 // String parameters that can be used to parse the trace config string.
 const char kRecordModeParam[] = "record_mode";
+const char kTraceBufferSizeInEvents[] = "trace_buffer_size_in_events";
 const char kEnableSystraceParam[] = "enable_systrace";
 const char kEnableArgumentFilterParam[] = "enable_argument_filter";
 
@@ -51,6 +53,11 @@ const char kEventFiltersParam[] = "event_filters";
 const char kFilterPredicateParam[] = "filter_predicate";
 const char kFilterArgsParam[] = "filter_args";
 
+// String parameter used to parse process filter.
+const char kIncludedProcessesParam[] = "included_process_ids";
+
+const char kHistogramNamesParam[] = "histogram_names";
+
 class ConvertableTraceConfigToTraceFormat
     : public base::trace_event::ConvertableToTraceFormat {
  public:
@@ -115,6 +122,56 @@ void TraceConfig::MemoryDumpConfig::Merge(
                config.heap_profiler_options.breakdown_threshold_bytes);
 }
 
+TraceConfig::ProcessFilterConfig::ProcessFilterConfig() = default;
+
+TraceConfig::ProcessFilterConfig::ProcessFilterConfig(
+    const ProcessFilterConfig& other) = default;
+
+TraceConfig::ProcessFilterConfig::ProcessFilterConfig(
+    const std::unordered_set<base::ProcessId>& included_process_ids)
+    : included_process_ids_(included_process_ids) {}
+
+TraceConfig::ProcessFilterConfig::~ProcessFilterConfig() = default;
+
+void TraceConfig::ProcessFilterConfig::Clear() {
+  included_process_ids_.clear();
+}
+
+void TraceConfig::ProcessFilterConfig::Merge(
+    const ProcessFilterConfig& config) {
+  included_process_ids_.insert(config.included_process_ids_.begin(),
+                               config.included_process_ids_.end());
+}
+
+void TraceConfig::ProcessFilterConfig::InitializeFromConfigDict(
+    const base::DictionaryValue& dict) {
+  included_process_ids_.clear();
+  const Value* value =
+      dict.FindKeyOfType(kIncludedProcessesParam, Value::Type::LIST);
+  if (!value)
+    return;
+  for (auto& pid_value : value->GetList()) {
+    if (pid_value.is_int())
+      included_process_ids_.insert(pid_value.GetInt());
+  }
+}
+
+void TraceConfig::ProcessFilterConfig::ToDict(DictionaryValue* dict) const {
+  if (included_process_ids_.empty())
+    return;
+  Value* list = dict->SetKey(kIncludedProcessesParam, Value(Value::Type::LIST));
+  std::set<base::ProcessId> ordered_set(included_process_ids_.begin(),
+                                        included_process_ids_.end());
+  for (auto process_id : ordered_set)
+    list->GetList().emplace_back(static_cast<int>(process_id));
+}
+
+bool TraceConfig::ProcessFilterConfig::IsEnabled(
+    base::ProcessId process_id) const {
+  return included_process_ids_.empty() ||
+         included_process_ids_.count(process_id);
+}
+
 TraceConfig::EventFilterConfig::EventFilterConfig(
     const std::string& predicate_name)
     : predicate_name_(predicate_name) {}
@@ -234,11 +291,14 @@ TraceConfig& TraceConfig::operator=(const TraceConfig& rhs) {
     return *this;
 
   record_mode_ = rhs.record_mode_;
+  trace_buffer_size_in_events_ = rhs.trace_buffer_size_in_events_;
   enable_systrace_ = rhs.enable_systrace_;
   enable_argument_filter_ = rhs.enable_argument_filter_;
   category_filter_ = rhs.category_filter_;
+  process_filter_config_ = rhs.process_filter_config_;
   memory_dump_config_ = rhs.memory_dump_config_;
   event_filters_ = rhs.event_filters_;
+  histogram_names_ = rhs.histogram_names_;
   return *this;
 }
 
@@ -272,26 +332,34 @@ void TraceConfig::Merge(const TraceConfig& config) {
     DLOG(ERROR) << "Attempting to merge trace config with a different "
                 << "set of options.";
   }
+  DCHECK_EQ(trace_buffer_size_in_events_, config.trace_buffer_size_in_events_)
+      << "Cannot change trace buffer size";
 
   category_filter_.Merge(config.category_filter_);
-
   memory_dump_config_.Merge(config.memory_dump_config_);
+  process_filter_config_.Merge(config.process_filter_config_);
 
   event_filters_.insert(event_filters_.end(), config.event_filters().begin(),
                         config.event_filters().end());
+  histogram_names_.insert(config.histogram_names().begin(),
+                          config.histogram_names().end());
 }
 
 void TraceConfig::Clear() {
   record_mode_ = RECORD_UNTIL_FULL;
+  trace_buffer_size_in_events_ = 0;
   enable_systrace_ = false;
   enable_argument_filter_ = false;
   category_filter_.Clear();
   memory_dump_config_.Clear();
+  process_filter_config_.Clear();
   event_filters_.clear();
+  histogram_names_.clear();
 }
 
 void TraceConfig::InitializeDefault() {
   record_mode_ = RECORD_UNTIL_FULL;
+  trace_buffer_size_in_events_ = 0;
   enable_systrace_ = false;
   enable_argument_filter_ = false;
 }
@@ -310,6 +378,9 @@ void TraceConfig::InitializeFromConfigDict(const DictionaryValue& dict) {
       record_mode_ = RECORD_AS_MUCH_AS_POSSIBLE;
     }
   }
+  int buffer_size = 0;
+  trace_buffer_size_in_events_ =
+      dict.GetInteger(kTraceBufferSizeInEvents, &buffer_size) ? buffer_size : 0;
 
   bool val;
   enable_systrace_ = dict.GetBoolean(kEnableSystraceParam, &val) ? val : false;
@@ -317,10 +388,14 @@ void TraceConfig::InitializeFromConfigDict(const DictionaryValue& dict) {
       dict.GetBoolean(kEnableArgumentFilterParam, &val) ? val : false;
 
   category_filter_.InitializeFromConfigDict(dict);
+  process_filter_config_.InitializeFromConfigDict(dict);
 
   const base::ListValue* category_event_filters = nullptr;
   if (dict.GetList(kEventFiltersParam, &category_event_filters))
     SetEventFiltersFromConfigList(*category_event_filters);
+  const base::ListValue* histogram_names = nullptr;
+  if (dict.GetList(kHistogramNamesParam, &histogram_names))
+    SetHistogramNamesFromConfigList(*histogram_names);
 
   if (category_filter_.IsCategoryEnabled(MemoryDumpManager::kTraceCategory)) {
     // If dump triggers not set, the client is using the legacy with just
@@ -334,7 +409,7 @@ void TraceConfig::InitializeFromConfigDict(const DictionaryValue& dict) {
 }
 
 void TraceConfig::InitializeFromConfigString(StringPiece config_string) {
-  auto dict = DictionaryValue::From(JSONReader::Read(config_string));
+  auto dict = DictionaryValue::From(JSONReader::ReadDeprecated(config_string));
   if (dict)
     InitializeFromConfigDict(*dict);
   else
@@ -347,6 +422,7 @@ void TraceConfig::InitializeFromStrings(StringPiece category_filter_string,
     category_filter_.InitializeFromString(category_filter_string);
 
   record_mode_ = RECORD_UNTIL_FULL;
+  trace_buffer_size_in_events_ = 0;
   enable_systrace_ = false;
   enable_argument_filter_ = false;
   if (!trace_options_string.empty()) {
@@ -434,7 +510,7 @@ void TraceConfig::SetMemoryDumpConfigFromConfigDict(
                                          &min_size_bytes)
         && min_size_bytes >= 0) {
       memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes =
-          static_cast<size_t>(min_size_bytes);
+          min_size_bytes;
     } else {
       memory_dump_config_.heap_profiler_options.breakdown_threshold_bytes =
           MemoryDumpConfig::HeapProfiler::kDefaultBreakdownThresholdBytes;
@@ -447,6 +523,17 @@ void TraceConfig::SetDefaultMemoryDumpConfig() {
   memory_dump_config_.allowed_dump_modes = GetDefaultAllowedMemoryDumpModes();
 }
 
+void TraceConfig::SetProcessFilterConfig(const ProcessFilterConfig& config) {
+  process_filter_config_ = config;
+}
+
+void TraceConfig::SetHistogramNamesFromConfigList(
+    const base::ListValue& histogram_names) {
+  histogram_names_.clear();
+  for (const Value& value : histogram_names.GetList())
+    histogram_names_.insert(value.GetString());
+}
+
 void TraceConfig::SetEventFiltersFromConfigList(
     const base::ListValue& category_event_filters) {
   event_filters_.clear();
@@ -475,8 +562,11 @@ std::unique_ptr<DictionaryValue> TraceConfig::ToDict() const {
                   TraceConfig::TraceRecordModeToStr(record_mode_));
   dict->SetBoolean(kEnableSystraceParam, enable_systrace_);
   dict->SetBoolean(kEnableArgumentFilterParam, enable_argument_filter_);
+  if (trace_buffer_size_in_events_ > 0)
+    dict->SetInteger(kTraceBufferSizeInEvents, trace_buffer_size_in_events_);
 
   category_filter_.ToDict(dict.get());
+  process_filter_config_.ToDict(dict.get());
 
   if (!event_filters_.empty()) {
     std::unique_ptr<base::ListValue> filter_list(new base::ListValue());
@@ -525,9 +615,21 @@ std::unique_ptr<DictionaryValue> TraceConfig::ToDict() const {
     }
     dict->Set(kMemoryDumpConfigParam, std::move(memory_dump_config));
   }
+
+  if (!histogram_names_.empty()) {
+    std::unique_ptr<base::ListValue> histogram_names(new base::ListValue());
+    for (const std::string& histogram_name : histogram_names_)
+      histogram_names->AppendString(histogram_name);
+    dict->Set(kHistogramNamesParam, std::move(histogram_names));
+  }
+
   return dict;
 }
 
+void TraceConfig::EnableHistogram(const std::string& histogram_name) {
+  histogram_names_.insert(histogram_name);
+}
+
 std::string TraceConfig::ToTraceOptionsString() const {
   std::string ret;
   switch (record_mode_) {
