@@ -13,13 +13,11 @@
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "gpu/command_buffer/client/cmd_buffer_helper.h"
 #include "gpu/command_buffer/client/command_buffer_direct_locked.h"
 #include "gpu/command_buffer/service/mocks.h"
-#include "gpu/command_buffer/service/transfer_buffer_manager.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace gpu {
@@ -42,10 +40,8 @@ const int32_t kUnusedCommandId = 5;  // we use 0 and 2 currently.
 // interface (calling it directly, not through the RPC mechanism).
 class CommandBufferHelperTest : public testing::Test {
  protected:
-  virtual void SetUp() {
-    transfer_buffer_manager_ = std::make_unique<TransferBufferManager>(nullptr);
-    command_buffer_.reset(
-        new CommandBufferDirectLocked(transfer_buffer_manager_.get()));
+  void SetUp() override {
+    command_buffer_.reset(new CommandBufferDirectLocked());
     api_mock_.reset(new AsyncAPIMock(true, command_buffer_->service()));
     command_buffer_->set_handler(api_mock_.get());
 
@@ -60,7 +56,7 @@ class CommandBufferHelperTest : public testing::Test {
     test_command_next_id_ = kUnusedCommandId;
   }
 
-  virtual void TearDown() {
+  void TearDown() override {
     // If the CommandExecutor posts any tasks, this forces them to run.
     base::RunLoop().RunUntilIdle();
     test_command_args_.clear();
@@ -200,7 +196,6 @@ class CommandBufferHelperTest : public testing::Test {
     helper_->WaitForGetOffsetInRange(start, end);
   }
 
-  std::unique_ptr<TransferBufferManager> transfer_buffer_manager_;
   std::unique_ptr<CommandBufferDirectLocked> command_buffer_;
   std::unique_ptr<AsyncAPIMock> api_mock_;
   std::unique_ptr<CommandBufferHelper> helper_;
@@ -335,6 +330,34 @@ TEST_F(CommandBufferHelperTest, TestCalcImmediateEntriesAutoFlushing) {
   EXPECT_EQ(error::kNoError, GetError());
 }
 
+// Checks that automatic flushing treats ordering barriers as flushes.
+TEST_F(CommandBufferHelperTest,
+       TestCalcImmediateEntriesAutoFlushingOrderingBarrier) {
+  // Check that auto flush happens without an ordering barrier.
+  AddUniqueCommandWithExpect(error::kNoError, kAutoFlushSmall - 1);
+  EXPECT_EQ(0, command_buffer_->FlushCount());
+  AddUniqueCommandWithExpect(error::kNoError, 1);
+  // Auto flush should be triggered by going past the threshold.
+  EXPECT_EQ(1, command_buffer_->FlushCount());
+  helper_->Finish();
+  EXPECT_EQ(2, command_buffer_->FlushCount());
+
+  // Check that an ordering barrier prevents auto flush.
+  AddUniqueCommandWithExpect(error::kNoError, kAutoFlushSmall - 1);
+  EXPECT_EQ(2, command_buffer_->FlushCount());
+  helper_->OrderingBarrier();
+  EXPECT_EQ(3, command_buffer_->FlushCount());
+  AddUniqueCommandWithExpect(error::kNoError, 1);
+  // Adding a command should not have caused a flush because there was an
+  // ordering barrier.
+  EXPECT_EQ(3, command_buffer_->FlushCount());
+
+  // Check that the commands did happen.
+  helper_->Finish();
+  Mock::VerifyAndClearExpectations(api_mock_.get());
+  EXPECT_EQ(error::kNoError, GetError());
+}
+
 // Checks immediate_entry_count_ calc when automatic flushing is enabled, and
 // we allocate commands over the immediate_entry_count_ size.
 TEST_F(CommandBufferHelperTest, TestCalcImmediateEntriesOverFlushLimit) {
@@ -376,7 +399,7 @@ TEST_F(CommandBufferHelperTest, TestCommandProcessing) {
   EXPECT_EQ(0, GetGetOffset());
 
   // Add 3 commands through the helper
-  AddCommandWithExpect(error::kNoError, kUnusedCommandId, 0, NULL);
+  AddCommandWithExpect(error::kNoError, kUnusedCommandId, 0, nullptr);
 
   CommandBufferEntry args1[2];
   args1[0].value_uint32 = 3;
@@ -473,8 +496,8 @@ TEST_F(CommandBufferHelperTest, TestAvailableEntries) {
   args[1].value_float = 4.f;
 
   // Add 2 commands through the helper - 8 entries
-  AddCommandWithExpect(error::kNoError, kUnusedCommandId + 1, 0, NULL);
-  AddCommandWithExpect(error::kNoError, kUnusedCommandId + 2, 0, NULL);
+  AddCommandWithExpect(error::kNoError, kUnusedCommandId + 1, 0, nullptr);
+  AddCommandWithExpect(error::kNoError, kUnusedCommandId + 2, 0, nullptr);
   AddCommandWithExpect(error::kNoError, kUnusedCommandId + 3, 2, args);
   AddCommandWithExpect(error::kNoError, kUnusedCommandId + 4, 2, args);
 
@@ -584,7 +607,7 @@ TEST_F(CommandBufferHelperTest, FreeRingBuffer) {
   EXPECT_FALSE(helper_->HaveRingBuffer());
 
   // Test that WaitForAvailableEntries allocates a new one
-  AddCommandWithExpect(error::kNoError, kUnusedCommandId, 0, NULL);
+  AddCommandWithExpect(error::kNoError, kUnusedCommandId, 0, nullptr);
   EXPECT_TRUE(helper_->HaveRingBuffer());
   helper_->Finish();
   helper_->FreeRingBuffer();
@@ -594,7 +617,7 @@ TEST_F(CommandBufferHelperTest, FreeRingBuffer) {
   Mock::VerifyAndClearExpectations(api_mock_.get());
 
   // Test that FreeRingBuffer doesn't force a finish
-  AddCommandWithExpect(error::kNoError, kUnusedCommandId, 0, NULL);
+  AddCommandWithExpect(error::kNoError, kUnusedCommandId, 0, nullptr);
   EXPECT_TRUE(helper_->HaveRingBuffer());
   int32_t old_get_offset = command_buffer_->GetLastState().get_offset;
   EXPECT_NE(helper_->GetPutOffsetForTest(), old_get_offset);
@@ -602,8 +625,8 @@ TEST_F(CommandBufferHelperTest, FreeRingBuffer) {
 
   helper_->FreeRingBuffer();
   EXPECT_FALSE(helper_->HaveRingBuffer());
-  // FreeRingBuffer should have caused a flush.
-  EXPECT_EQ(command_buffer_->FlushCount(), old_flush_count + 1);
+  // FreeRingBuffer should have caused an ordering barrier and a flush.
+  EXPECT_EQ(command_buffer_->FlushCount(), old_flush_count + 2);
   // However it shouldn't force a finish.
   EXPECT_EQ(command_buffer_->GetLastState().get_offset, old_get_offset);
 
@@ -611,7 +634,7 @@ TEST_F(CommandBufferHelperTest, FreeRingBuffer) {
   // should work.
   helper_->Finish();
   EXPECT_FALSE(helper_->HaveRingBuffer());
-  EXPECT_EQ(command_buffer_->FlushCount(), old_flush_count + 1);
+  EXPECT_EQ(command_buffer_->FlushCount(), old_flush_count + 2);
   EXPECT_EQ(command_buffer_->GetLastState().get_offset,
             helper_->GetPutOffsetForTest());
 }
