@@ -13,6 +13,7 @@
 #include <memory>
 #include <set>
 #include <string>
+#include <unordered_map>
 #include <utility>
 #include <vector>
 
@@ -33,10 +34,10 @@
 #include "gpu/command_buffer/client/logging.h"
 #include "gpu/command_buffer/client/mapped_memory.h"
 #include "gpu/command_buffer/client/query_tracker.h"
+#include "gpu/command_buffer/client/readback_buffer_shadow_tracker.h"
 #include "gpu/command_buffer/client/ref_counted.h"
 #include "gpu/command_buffer/client/share_group.h"
 #include "gpu/command_buffer/client/transfer_buffer.h"
-#include "gpu/command_buffer/common/capabilities.h"
 #include "gpu/command_buffer/common/context_creation_attribs.h"
 #include "gpu/command_buffer/common/context_result.h"
 #include "gpu/command_buffer/common/debug_marker_manager.h"
@@ -49,6 +50,7 @@ namespace gles2 {
 
 class GLES2CmdHelper;
 class VertexArrayObjectManager;
+class ReadbackBufferShadowTracker;
 
 // This class emulates GLES2 over command buffers. It can be used by a client
 // program so that the program does not need deal with shared memory and command
@@ -105,31 +107,54 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
 
   // ContextSupport implementation.
   void SetAggressivelyFreeResources(bool aggressively_free_resources) override;
-  void Swap() override;
-  void SwapWithBounds(const std::vector<gfx::Rect>& rects) override;
-  void PartialSwapBuffers(const gfx::Rect& sub_buffer) override;
-  void CommitOverlayPlanes() override;
+  void Swap(uint32_t flags,
+            SwapCompletedCallback complete_callback,
+            PresentationCallback presentation_callback) override;
+  void SwapWithBounds(const std::vector<gfx::Rect>& rects,
+                      uint32_t flags,
+                      SwapCompletedCallback swap_completed,
+                      PresentationCallback presentation_callback) override;
+  void PartialSwapBuffers(const gfx::Rect& sub_buffer,
+                          uint32_t flags,
+                          SwapCompletedCallback swap_completed,
+                          PresentationCallback presentation_callback) override;
+  void CommitOverlayPlanes(uint32_t flags,
+                           SwapCompletedCallback swap_completed,
+                           PresentationCallback presentation_callback) override;
   void ScheduleOverlayPlane(int plane_z_order,
                             gfx::OverlayTransform plane_transform,
                             unsigned overlay_texture_id,
                             const gfx::Rect& display_bounds,
-                            const gfx::RectF& uv_rect) override;
+                            const gfx::RectF& uv_rect,
+                            bool enable_blend,
+                            unsigned gpu_fence_id) override;
   uint64_t ShareGroupTracingGUID() const override;
   void SetErrorMessageCallback(
       base::RepeatingCallback<void(const char*, int32_t)> callback) override;
-  void SetSnapshotRequested() override;
   bool ThreadSafeShallowLockDiscardableTexture(uint32_t texture_id) override;
   void CompleteLockDiscardableTexureOnContextThread(
       uint32_t texture_id) override;
   bool ThreadsafeDiscardableTextureIsDeletedForTracing(
       uint32_t texture_id) override;
-
+  void* MapTransferCacheEntry(uint32_t serialized_size) override;
+  void UnmapAndCreateTransferCacheEntry(uint32_t type, uint32_t id) override;
+  bool ThreadsafeLockTransferCacheEntry(uint32_t type, uint32_t id) override;
+  void UnlockTransferCacheEntries(
+      const std::vector<std::pair<uint32_t, uint32_t>>& entries) override;
+  void DeleteTransferCacheEntry(uint32_t type, uint32_t id) override;
+  unsigned int GetTransferBufferFreeSize() const override;
   void GetProgramInfoCHROMIUMHelper(GLuint program,
                                     std::vector<int8_t>* result);
   GLint GetAttribLocationHelper(GLuint program, const char* name);
   GLint GetUniformLocationHelper(GLuint program, const char* name);
   GLint GetFragDataIndexEXTHelper(GLuint program, const char* name);
   GLint GetFragDataLocationHelper(GLuint program, const char* name);
+
+  // Writes the result bucket into a buffer pointed by name and of maximum size
+  // buffsize. If length is !null, it receives the number of characters written
+  // (excluding the final \0). This is a helper function for GetActive*Helper
+  // functions that return names.
+  void GetResultNameHelper(GLsizei bufsize, GLsizei* length, char* name);
   bool GetActiveAttribHelper(
       GLuint program, GLuint index, GLsizei bufsize, GLsizei* length,
       GLint* size, GLenum* type, char* name);
@@ -161,13 +186,22 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
       GLint* values);
   bool GetQueryObjectValueHelper(
       const char* function_name, GLuint id, GLenum pname, GLuint64* params);
+  bool GetProgramInterfaceivHelper(
+      GLuint program, GLenum program_interface, GLenum pname, GLint* params);
+  GLuint GetProgramResourceIndexHelper(
+      GLuint program, GLenum program_interface, const char* name);
+  bool GetProgramResourceNameHelper(
+      GLuint program, GLenum program_interface, GLuint index, GLsizei bufsize,
+      GLsizei* length, char* name);
+  bool GetProgramResourceivHelper(
+      GLuint program, GLenum program_interface, GLuint index,
+      GLsizei prop_count, const GLenum* props, GLsizei bufsize, GLsizei* length,
+      GLint* params);
+  GLint GetProgramResourceLocationHelper(
+      GLuint program, GLenum program_interface, const char* name);
 
   const scoped_refptr<ShareGroup>& share_group() const { return share_group_; }
 
-  const Capabilities& capabilities() const {
-    return capabilities_;
-  }
-
   GpuControl* gpu_control() {
     return gpu_control_;
   }
@@ -195,19 +229,7 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
                   const char* function_name,
                   const char* msg) override;
 
-  // ClientTransferCache::Client implementation.
-  void IssueCreateTransferCacheEntry(GLuint entry_type,
-                                     GLuint entry_id,
-                                     GLuint handle_shm_id,
-                                     GLuint handle_shm_offset,
-                                     GLuint data_shm_id,
-                                     GLuint data_shm_offset,
-                                     GLuint data_size) override;
-  void IssueDeleteTransferCacheEntry(GLuint entry_type,
-                                     GLuint entry_id) override;
-  void IssueUnlockTransferCacheEntry(GLuint entry_type,
-                                     GLuint entry_id) override;
-  CommandBuffer* command_buffer() const override;
+  CommandBuffer* command_buffer() const;
 
  private:
   friend class GLES2ImplementationTest;
@@ -309,21 +331,42 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
   };
 
   struct TextureUnit {
-    TextureUnit()
-        : bound_texture_2d(0),
-          bound_texture_cube_map(0),
-          bound_texture_external_oes(0) {}
+    TextureUnit() {}
 
     // texture currently bound to this unit's GL_TEXTURE_2D with glBindTexture
-    GLuint bound_texture_2d;
+    GLuint bound_texture_2d = 0;
 
     // texture currently bound to this unit's GL_TEXTURE_CUBE_MAP with
     // glBindTexture
-    GLuint bound_texture_cube_map;
+    GLuint bound_texture_cube_map = 0;
 
     // texture currently bound to this unit's GL_TEXTURE_EXTERNAL_OES with
     // glBindTexture
-    GLuint bound_texture_external_oes;
+    GLuint bound_texture_external_oes = 0;
+
+    // texture currently bound to this unit's GL_TEXTURE_RECTANGLE_ARB with
+    // glBindTexture
+    GLuint bound_texture_rectangle_arb = 0;
+  };
+
+  // Prevents problematic reentrancy during error callbacks.
+  class DeferErrorCallbacks {
+   public:
+    explicit DeferErrorCallbacks(GLES2Implementation* gles2_implementation);
+    ~DeferErrorCallbacks();
+
+   private:
+    GLES2Implementation* gles2_implementation_;
+  };
+
+  struct DeferredErrorCallback {
+    // This takes std::string by value and uses std::move in the
+    // implementation, allowing the compiler to achieve zero copies
+    // when passing in a temporary.
+    DeferredErrorCallback(std::string message, int32_t id);
+
+    std::string message;
+    int32_t id = 0;
   };
 
   // Checks for single threaded access.
@@ -343,6 +386,13 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
   void OnGpuControlLostContext() final;
   void OnGpuControlLostContextMaybeReentrant() final;
   void OnGpuControlErrorMessage(const char* message, int32_t id) final;
+  void OnGpuControlSwapBuffersCompleted(
+      const SwapBuffersCompleteParams& params) final;
+  void OnSwapBufferPresented(uint64_t swap_id,
+                             const gfx::PresentationFeedback& feedback) final;
+
+  void SendErrorMessage(std::string message, int32_t id);
+  void CallDeferredErrorCallbacks();
 
   bool IsChromiumFramebufferMultisampleAvailable();
 
@@ -361,6 +411,11 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
     return last_error_;
   }
 
+  void AllocateShadowCopiesForReadback();
+  static void BufferShadowWrittenCallback(
+      const ReadbackBufferShadowTracker::BufferList& buffers,
+      uint64_t serial);
+
   // Returns true if id is reserved.
   bool IsBufferReservedId(GLuint id);
   bool IsFramebufferReservedId(GLuint id) { return false; }
@@ -371,6 +426,10 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
   bool IsSamplerReservedId(GLuint id) { return false; }
   bool IsTransformFeedbackReservedId(GLuint id) { return false; }
 
+  bool UpdateIndexedBufferState(GLenum target,
+                                GLuint index,
+                                GLuint buffer_id,
+                                const char* function_name);
   void BindBufferHelper(GLenum target, GLuint buffer);
   void BindBufferBaseHelper(GLenum target, GLuint index, GLuint buffer);
   void BindBufferRangeHelper(GLenum target, GLuint index, GLuint buffer,
@@ -431,6 +490,30 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
       GLenum target, GLintptr offset, GLsizeiptr size, const void* data,
       ScopedTransferBufferPtr* buffer);
 
+  void MultiDrawArraysWEBGLHelper(GLenum mode,
+                                  const GLint* firsts,
+                                  const GLsizei* counts,
+                                  GLsizei drawcount);
+
+  void MultiDrawArraysInstancedWEBGLHelper(GLenum mode,
+                                           const GLint* firsts,
+                                           const GLsizei* counts,
+                                           const GLsizei* instanceCounts,
+                                           GLsizei drawcount);
+
+  void MultiDrawElementsWEBGLHelper(GLenum mode,
+                                    const GLsizei* counts,
+                                    GLenum type,
+                                    const GLsizei* offsets,
+                                    GLsizei drawcount);
+
+  void MultiDrawElementsInstancedWEBGLHelper(GLenum mode,
+                                             const GLsizei* counts,
+                                             GLenum type,
+                                             const GLsizei* offsets,
+                                             const GLsizei* instanceCounts,
+                                             GLsizei drawcount);
+
   GLuint CreateImageCHROMIUMHelper(ClientBuffer buffer,
                                    GLsizei width,
                                    GLsizei height,
@@ -545,6 +628,8 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
       GLuint buffer_id,
       const char* function_name, GLuint offset, GLsizei size);
 
+  void OnBufferWrite(GLenum target);
+
   // Pack 2D arrays of char into a bucket.
   // Helper function for ShaderSource(), TransformFeedbackVaryings(), etc.
   bool PackStringsToBucket(GLsizei count,
@@ -562,9 +647,9 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
                                    const GLfloat* transform_values,
                                    ScopedTransferBufferPtr* buffer,
                                    uint32_t* out_paths_shm_id,
-                                   size_t* out_paths_offset,
+                                   uint32_t* out_paths_offset,
                                    uint32_t* out_transforms_shm_id,
-                                   size_t* out_transforms_offset);
+                                   uint32_t* out_transforms_offset);
 
 // Set to 1 to have the client fail when a GL error is generated.
 // This helps find bugs in the renderer since the debugger stops on the error.
@@ -593,6 +678,9 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
 
   PixelStoreParams GetUnpackParameters(Dimension dimension);
 
+  uint64_t PrepareNextSwapId(SwapCompletedCallback complete_callback,
+                             PresentationCallback present_callback);
+
   GLES2Util util_;
   GLES2CmdHelper* helper_;
   std::string last_error_;
@@ -649,10 +737,13 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
   GLuint current_program_;
 
   GLuint bound_array_buffer_;
+  GLuint bound_atomic_counter_buffer_;
   GLuint bound_copy_read_buffer_;
   GLuint bound_copy_write_buffer_;
   GLuint bound_pixel_pack_buffer_;
   GLuint bound_pixel_unpack_buffer_;
+  GLuint bound_shader_storage_buffer_;
+  GLuint bound_transform_feedback_buffer_;
   GLuint bound_uniform_buffer_;
   // We don't cache the currently bound transform feedback buffer, because
   // it is part of the current transform feedback object. Caching the transform
@@ -706,7 +797,7 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
   MappedBufferMap mapped_buffers_;
 
   // TODO(zmo): Consolidate |mapped_buffers_| and |mapped_buffer_range_map_|.
-  typedef base::hash_map<GLuint, MappedBuffer> MappedBufferRangeMap;
+  typedef std::unordered_map<GLuint, MappedBuffer> MappedBufferRangeMap;
   MappedBufferRangeMap mapped_buffer_range_map_;
 
   typedef std::map<const void*, MappedTexture> MappedTextureMap;
@@ -719,15 +810,17 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
       id_allocators_[static_cast<int>(IdNamespaces::kNumIdNamespaces)];
 
   std::unique_ptr<BufferTracker> buffer_tracker_;
+  std::unique_ptr<ReadbackBufferShadowTracker> readback_buffer_shadow_tracker_;
 
+  base::Optional<ScopedMappedMemoryPtr> font_mapped_buffer_;
   base::Optional<ScopedTransferBufferPtr> raster_mapped_buffer_;
 
-  base::Callback<void(const char*, int32_t)> error_message_callback_;
+  base::RepeatingCallback<void(const char*, int32_t)> error_message_callback_;
+  bool deferring_error_callbacks_ = false;
+  std::deque<DeferredErrorCallback> deferred_error_callbacks_;
 
   int current_trace_stack_;
 
-  Capabilities capabilities_;
-
   // Flag to indicate whether the implementation can retain resources, or
   // whether it should aggressively free them.
   bool aggressively_free_resources_;
@@ -742,6 +835,15 @@ class GLES2_IMPL_EXPORT GLES2Implementation : public GLES2Interface,
   // gl_strings, valid forever.
   std::vector<const char*> cached_extensions_;
 
+  // The next swap ID to send.
+  uint64_t swap_id_ = 0;
+  // A map of swap IDs to callbacks to run when that ID completes.
+  base::flat_map<uint64_t, SwapCompletedCallback> pending_swap_callbacks_;
+  base::flat_map<uint64_t, PresentationCallback>
+      pending_presentation_callbacks_;
+
+  std::string last_active_url_;
+
   base::WeakPtrFactory<GLES2Implementation> weak_ptr_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(GLES2Implementation);
