@@ -1,6 +1,7 @@
 // Copyright 2016 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
+
 #include <stddef.h>
 #include <stdint.h>
 
@@ -9,16 +10,17 @@
 
 #include "base/at_exit.h"
 #include "base/bind.h"
+#include "base/bind_helpers.h"
 #include "base/command_line.h"
+#include "base/i18n/icu_util.h"
 #include "base/logging.h"
-#include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
+#include "base/stl_util.h"
 #include "base/strings/string_piece.h"
 #include "base/strings/string_util.h"
 #include "build/build_config.h"
 #include "gpu/command_buffer/common/constants.h"
 #include "gpu/command_buffer/common/context_creation_attribs.h"
-#include "gpu/command_buffer/common/sync_token.h"
 #include "gpu/command_buffer/service/buffer_manager.h"
 #include "gpu/command_buffer/service/command_buffer_direct.h"
 #include "gpu/command_buffer/service/context_group.h"
@@ -28,8 +30,11 @@
 #include "gpu/command_buffer/service/image_manager.h"
 #include "gpu/command_buffer/service/logger.h"
 #include "gpu/command_buffer/service/mailbox_manager_impl.h"
+#include "gpu/command_buffer/service/passthrough_discardable_manager.h"
+#include "gpu/command_buffer/service/raster_decoder.h"
 #include "gpu/command_buffer/service/service_discardable_manager.h"
-#include "gpu/command_buffer/service/sync_point_manager.h"
+#include "gpu/command_buffer/service/shared_context_state.h"
+#include "gpu/command_buffer/service/shared_image_manager.h"
 #include "gpu/command_buffer/service/transfer_buffer_manager.h"
 #include "ui/gfx/geometry/size.h"
 #include "ui/gl/gl_context.h"
@@ -47,10 +52,10 @@
 namespace gpu {
 namespace {
 
-const size_t kCommandBufferSize = 16384;
-const size_t kTransferBufferSize = 16384;
-const size_t kSmallTransferBufferSize = 16;
-const size_t kTinyTransferBufferSize = 3;
+const uint32_t kCommandBufferSize = 16384;
+const uint32_t kTransferBufferSize = 16384;
+const uint32_t kSmallTransferBufferSize = 16;
+const uint32_t kTinyTransferBufferSize = 3;
 
 #if !defined(GPU_FUZZER_USE_ANGLE) && !defined(GPU_FUZZER_USE_SWIFTSHADER)
 #define GPU_FUZZER_USE_STUB
@@ -171,7 +176,7 @@ constexpr const char* kExtensions[] = {
     "GL_OES_texture_half_float_linear",
     "GL_OES_texture_npot",
     "GL_OES_vertex_array_object"};
-constexpr size_t kExtensionCount = arraysize(kExtensions);
+constexpr size_t kExtensionCount = base::size(kExtensions);
 
 #if defined(GPU_FUZZER_USE_STUB)
 constexpr const char* kDriverVersions[] = {"OpenGL ES 2.0", "OpenGL ES 3.1",
@@ -226,8 +231,14 @@ struct Config {
     attrib_helper.bind_generates_resource = it.GetBit();
     attrib_helper.single_buffer = it.GetBit();
     bool es3 = it.GetBit();
+#if defined(GPU_FUZZER_USE_RASTER_DECODER)
+    ALLOW_UNUSED_LOCAL(es3);
+    attrib_helper.context_type = CONTEXT_TYPE_OPENGLES2;
+#else
     attrib_helper.context_type =
         es3 ? CONTEXT_TYPE_OPENGLES3 : CONTEXT_TYPE_OPENGLES2;
+#endif
+    attrib_helper.enable_oop_rasterization = it.GetBit();
 
 #if defined(GPU_FUZZER_USE_STUB)
     std::vector<base::StringPiece> enabled_extensions;
@@ -251,11 +262,25 @@ struct Config {
     GPU_DRIVER_BUG_WORKAROUNDS(GPU_OP)
 #undef GPU_OP
 
+#if defined(GPU_FUZZER_USE_PASSTHROUGH_CMD_DECODER)
+    gl_context_attribs.bind_generates_resource =
+        attrib_helper.bind_generates_resource;
+    gl_context_attribs.webgl_compatibility_context =
+        IsWebGLContextType(attrib_helper.context_type);
+    gl_context_attribs.global_texture_share_group = true;
+    gl_context_attribs.robust_resource_initialization = true;
+    gl_context_attribs.robust_buffer_access = true;
+    gl_context_attribs.client_major_es_version =
+        IsWebGL2OrES3ContextType(attrib_helper.context_type) ? 3 : 2;
+    gl_context_attribs.client_minor_es_version = 0;
+#endif
+
     return it.consumed_bytes();
-  };
+  }
 
   GpuDriverBugWorkarounds workarounds;
   ContextCreationAttribs attrib_helper;
+  gl::GLContextAttribs gl_context_attribs;
 #if defined(GPU_FUZZER_USE_STUB)
   const char* version;
   std::string extensions;
@@ -273,12 +298,13 @@ GpuPreferences GetGpuPreferences() {
 class CommandBufferSetup {
  public:
   CommandBufferSetup()
-      : atexit_manager_(),
+      : at_exit_manager_(),
         gpu_preferences_(GetGpuPreferences()),
         share_group_(new gl::GLShareGroup),
         translator_cache_(gpu_preferences_) {
     logging::SetMinLogLevel(logging::LOG_FATAL);
-    base::CommandLine::Init(0, NULL);
+    CHECK(base::i18n::InitializeICU());
+    base::CommandLine::Init(0, nullptr);
 
     auto* command_line = base::CommandLine::ForCurrentProcess();
     ALLOW_UNUSED_LOCAL(command_line);
@@ -304,6 +330,8 @@ class CommandBufferSetup {
 #error invalid configuration
 #endif
     discardable_manager_ = std::make_unique<ServiceDiscardableManager>();
+    passthrough_discardable_manager_ =
+        std::make_unique<PassthroughDiscardableManager>();
 
     if (gpu_preferences_.use_passthrough_cmd_decoder)
       recreate_context_ = true;
@@ -315,29 +343,50 @@ class CommandBufferSetup {
   }
 
   bool InitDecoder() {
-    if (recreate_context_) {
+    if (!context_) {
       InitContext();
     }
 
     context_->MakeCurrent(surface_.get());
+    GpuFeatureInfo gpu_feature_info;
+#if defined(GPU_FUZZER_USE_RASTER_DECODER)
+    // Cause feature_info's |chromium_raster_transport| to be enabled.
+    gpu_feature_info.status_values[GPU_FEATURE_TYPE_OOP_RASTERIZATION] =
+        kGpuFeatureStatusEnabled;
+#endif
     scoped_refptr<gles2::FeatureInfo> feature_info =
-        new gles2::FeatureInfo(config_.workarounds);
+        new gles2::FeatureInfo(config_.workarounds, gpu_feature_info);
     scoped_refptr<gles2::ContextGroup> context_group = new gles2::ContextGroup(
         gpu_preferences_, true, &mailbox_manager_, nullptr /* memory_tracker */,
         &translator_cache_, &completeness_cache_, feature_info,
         config_.attrib_helper.bind_generates_resource, &image_manager_,
         nullptr /* image_factory */, nullptr /* progress_reporter */,
-        GpuFeatureInfo(), discardable_manager_.get());
-    command_buffer_.reset(new CommandBufferDirect(
-        context_group->transfer_buffer_manager(), &sync_point_manager_));
+        gpu_feature_info, discardable_manager_.get(),
+        passthrough_discardable_manager_.get(), &shared_image_manager_);
+    command_buffer_.reset(new CommandBufferDirect());
 
+#if defined(GPU_FUZZER_USE_RASTER_DECODER)
+    CHECK(feature_info->feature_flags().chromium_raster_transport);
+    scoped_refptr<SharedContextState> context_state =
+        base::MakeRefCounted<SharedContextState>(
+            share_group_, surface_, context_,
+            config_.workarounds.use_virtualized_gl_contexts, base::DoNothing());
+    context_state->InitializeGrContext(config_.workarounds, nullptr);
+    context_state->InitializeGL(gpu_preferences_, feature_info);
+    auto* context = context_state->context();
+    decoder_.reset(raster::RasterDecoder::Create(
+        command_buffer_.get(), command_buffer_->service(), &outputter_,
+        context_group.get(), std::move(context_state)));
+#else
+    auto* context = context_.get();
     decoder_.reset(gles2::GLES2Decoder::Create(
         command_buffer_.get(), command_buffer_->service(), &outputter_,
         context_group.get()));
+#endif
 
     decoder_->GetLogger()->set_log_synthesized_gl_errors(false);
 
-    auto result = decoder_->Initialize(surface_.get(), context_.get(), true,
+    auto result = decoder_->Initialize(surface_.get(), context, true,
                                        gles2::DisallowedFeatures(),
                                        config_.attrib_helper);
     if (result != gpu::ContextResult::kSuccess)
@@ -352,22 +401,33 @@ class CommandBufferSetup {
   }
 
   void ResetDecoder() {
-    // Keep a reference to the translators, which keeps them in the cache even
-    // after the decoder is reset. They are expensive to initialize, but they
-    // don't keep state.
-    scoped_refptr<gles2::ShaderTranslatorInterface> translator =
-        decoder_->GetTranslator(GL_VERTEX_SHADER);
-    if (translator)
-      translator->AddRef();
-    translator = decoder_->GetTranslator(GL_FRAGMENT_SHADER);
-    if (translator)
-      translator->AddRef();
-    decoder_->Destroy(true);
-    decoder_.reset();
-    if (recreate_context_) {
-      context_->ReleaseCurrent(nullptr);
-      context_ = nullptr;
+    bool context_lost = false;
+    if (decoder_) {
+#if !defined(GPU_FUZZER_USE_RASTER_DECODER)
+      // Keep a reference to the translators, which keeps them in the cache even
+      // after the decoder is reset. They are expensive to initialize, but they
+      // don't keep state.
+      scoped_refptr<gles2::ShaderTranslatorInterface> translator =
+          decoder_->GetTranslator(GL_VERTEX_SHADER);
+      if (translator)
+        translator->AddRef();
+      translator = decoder_->GetTranslator(GL_FRAGMENT_SHADER);
+      if (translator)
+        translator->AddRef();
+#endif
+      context_lost =
+          decoder_->WasContextLost() || !decoder_->CheckResetStatus();
+      decoder_->Destroy(!context_lost);
+      decoder_.reset();
     }
+
+    if (context_) {
+      if (recreate_context_ || context_lost) {
+        context_->ReleaseCurrent(nullptr);
+        context_ = nullptr;
+      }
+    }
+
     command_buffer_.reset();
   }
 
@@ -386,10 +446,12 @@ class CommandBufferSetup {
     if (padded_size > kCommandBufferSize)
       return;
 
-    if (!InitDecoder())
+    if (!InitDecoder()) {
+      ResetDecoder();
       return;
+    }
 
-    size_t buffer_size = buffer_->size();
+    uint32_t buffer_size = buffer_->size();
     CHECK_LE(padded_size, buffer_size);
     command_buffer_->SetGetBuffer(buffer_id_);
     auto* memory = static_cast<char*>(buffer_->memory());
@@ -401,7 +463,7 @@ class CommandBufferSetup {
   }
 
  private:
-  void CreateTransferBuffer(size_t size, int32_t id) {
+  void CreateTransferBuffer(uint32_t size, int32_t id) {
     scoped_refptr<Buffer> buffer =
         command_buffer_->CreateTransferBufferWithId(size, id);
     memset(buffer->memory(), 0, size);
@@ -424,7 +486,7 @@ class CommandBufferSetup {
   void InitContext() {
 #if !defined(GPU_FUZZER_USE_STUB)
     context_ = new gl::GLContextEGL(share_group_.get());
-    context_->Initialize(surface_.get(), gl::GLContextAttribs());
+    context_->Initialize(surface_.get(), config_.gl_context_attribs);
 #else
     scoped_refptr<gl::GLContextStub> context_stub =
         new gl::GLContextStub(share_group_.get());
@@ -458,11 +520,11 @@ class CommandBufferSetup {
                                          GLenum severity,
                                          GLsizei length,
                                          const GLchar* message,
-                                         GLvoid* user_param) {
+                                         const GLvoid* user_param) {
     LOG_IF(FATAL, (id != GL_OUT_OF_MEMORY)) << "GL Driver Message: " << message;
   }
 
-  base::AtExitManager atexit_manager_;
+  base::AtExitManager at_exit_manager_;
 
   GpuPreferences gpu_preferences_;
 
@@ -471,9 +533,11 @@ class CommandBufferSetup {
   gles2::MailboxManagerImpl mailbox_manager_;
   gles2::TraceOutputter outputter_;
   scoped_refptr<gl::GLShareGroup> share_group_;
-  SyncPointManager sync_point_manager_;
   gles2::ImageManager image_manager_;
   std::unique_ptr<ServiceDiscardableManager> discardable_manager_;
+  std::unique_ptr<PassthroughDiscardableManager>
+      passthrough_discardable_manager_;
+  SharedImageManager shared_image_manager_;
 
   bool recreate_context_ = false;
   scoped_refptr<gl::GLSurface> surface_;
@@ -484,7 +548,11 @@ class CommandBufferSetup {
 
   std::unique_ptr<CommandBufferDirect> command_buffer_;
 
+#if defined(GPU_FUZZER_USE_RASTER_DECODER)
+  std::unique_ptr<raster::RasterDecoder> decoder_;
+#else
   std::unique_ptr<gles2::GLES2Decoder> decoder_;
+#endif
 
   scoped_refptr<Buffer> buffer_;
   int32_t buffer_id_ = 0;
