@@ -22,19 +22,23 @@
 #include "gpu/command_buffer/client/gles2_cmd_helper.h"
 #include "gpu/command_buffer/client/gles2_implementation.h"
 #include "gpu/command_buffer/client/gles2_lib.h"
-#include "gpu/command_buffer/client/shared_memory_limits.h"
 #include "gpu/command_buffer/client/transfer_buffer.h"
 #include "gpu/command_buffer/common/constants.h"
 #include "gpu/command_buffer/common/context_creation_attribs.h"
+#include "gpu/command_buffer/common/gpu_memory_buffer_support.h"
 #include "gpu/command_buffer/common/sync_token.h"
 #include "gpu/command_buffer/service/command_buffer_direct.h"
 #include "gpu/command_buffer/service/context_group.h"
 #include "gpu/command_buffer/service/gl_context_virtual.h"
 #include "gpu/command_buffer/service/gles2_cmd_decoder.h"
 #include "gpu/command_buffer/service/gpu_switches.h"
+#include "gpu/command_buffer/service/image_factory.h"
 #include "gpu/command_buffer/service/memory_tracking.h"
 #include "gpu/command_buffer/service/service_utils.h"
 #include "gpu/command_buffer/service/transfer_buffer_manager.h"
+#include "gpu/ipc/common/gpu_client_ids.h"
+#include "gpu/ipc/in_process_command_buffer.h"
+#include "gpu/ipc/service/gpu_memory_buffer_factory.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/gfx/buffer_format_util.h"
 #include "ui/gfx/gpu_memory_buffer.h"
@@ -97,13 +101,21 @@ class GpuMemoryBufferImpl : public gfx::GpuMemoryBuffer {
     NOTREACHED();
     return gfx::GpuMemoryBufferId(0);
   }
-  gfx::GpuMemoryBufferHandle GetHandle() const override {
+  gfx::GpuMemoryBufferType GetType() const override {
+    return gfx::NATIVE_PIXMAP;
+  }
+  gfx::GpuMemoryBufferHandle CloneHandle() const override {
     NOTREACHED();
     return gfx::GpuMemoryBufferHandle();
   }
   ClientBuffer AsClientBuffer() override {
     return reinterpret_cast<ClientBuffer>(this);
   }
+  void OnMemoryDump(
+      base::trace_event::ProcessMemoryDump* pmd,
+      const base::trace_event::MemoryAllocatorDumpGuid& buffer_dump_guid,
+      uint64_t tracing_process_id,
+      int importance) const override {}
 
   base::RefCountedBytes* bytes() { return bytes_.get(); }
 
@@ -155,13 +167,21 @@ class IOSurfaceGpuMemoryBuffer : public gfx::GpuMemoryBuffer {
     NOTREACHED();
     return gfx::GpuMemoryBufferId(0);
   }
-  gfx::GpuMemoryBufferHandle GetHandle() const override {
+  gfx::GpuMemoryBufferType GetType() const override {
+    return gfx::IO_SURFACE_BUFFER;
+  }
+  gfx::GpuMemoryBufferHandle CloneHandle() const override {
     NOTREACHED();
     return gfx::GpuMemoryBufferHandle();
   }
   ClientBuffer AsClientBuffer() override {
     return reinterpret_cast<ClientBuffer>(this);
   }
+  void OnMemoryDump(
+      base::trace_event::ProcessMemoryDump* pmd,
+      const base::trace_event::MemoryAllocatorDumpGuid& buffer_dump_guid,
+      uint64_t tracing_process_id,
+      int importance) const override {}
 
   IOSurfaceRef iosurface() { return iosurface_; }
 
@@ -175,11 +195,8 @@ class IOSurfaceGpuMemoryBuffer : public gfx::GpuMemoryBuffer {
 
 class CommandBufferCheckLostContext : public CommandBufferDirect {
  public:
-  CommandBufferCheckLostContext(TransferBufferManager* transfer_buffer_manager,
-                                SyncPointManager* sync_point_manager,
-                                bool context_lost_allowed)
-      : CommandBufferDirect(transfer_buffer_manager, sync_point_manager),
-        context_lost_allowed_(context_lost_allowed) {}
+  explicit CommandBufferCheckLostContext(bool context_lost_allowed)
+      : context_lost_allowed_(context_lost_allowed) {}
 
   ~CommandBufferCheckLostContext() override = default;
 
@@ -207,7 +224,9 @@ GpuFeatureInfo GLManager::g_gpu_feature_info;
 
 GLManager::Options::Options() = default;
 
-GLManager::GLManager() {
+GLManager::GLManager()
+    : gpu_memory_buffer_factory_(
+          gpu::GpuMemoryBufferFactory::CreateNativeType()) {
   SetupBaseContext();
 }
 
@@ -215,16 +234,16 @@ GLManager::~GLManager() {
   --use_count_;
   if (!use_count_) {
     if (base_share_group_) {
-      delete base_context_;
-      base_context_ = NULL;
+      delete base_share_group_;
+      base_share_group_ = nullptr;
     }
     if (base_surface_) {
       delete base_surface_;
-      base_surface_ = NULL;
+      base_surface_ = nullptr;
     }
     if (base_context_) {
       delete base_context_;
-      base_context_ = NULL;
+      base_context_ = nullptr;
     }
   }
 }
@@ -239,7 +258,7 @@ std::unique_ptr<gfx::GpuMemoryBuffer> GLManager::CreateGpuMemoryBuffer(
   }
 #endif  // defined(OS_MACOSX)
   std::vector<uint8_t> data(gfx::BufferSizeForBufferFormat(size, format), 0);
-  scoped_refptr<base::RefCountedBytes> bytes(new base::RefCountedBytes(data));
+  auto bytes = base::RefCountedBytes::TakeVector(&data);
   return base::WrapUnique<gfx::GpuMemoryBuffer>(
       new GpuMemoryBufferImpl(bytes.get(), size, format));
 }
@@ -262,7 +281,7 @@ void GLManager::InitializeWithWorkarounds(
 void GLManager::InitializeWithWorkaroundsImpl(
     const GLManager::Options& options,
     const GpuDriverBugWorkarounds& workarounds) {
-  const SharedMemoryLimits limits;
+  const SharedMemoryLimits limits = options.shared_memory_limits;
   const base::CommandLine& command_line =
       *base::CommandLine::ForCurrentProcess();
   DCHECK(!command_line.HasSwitch(switches::kDisableGLExtensions));
@@ -278,14 +297,14 @@ void GLManager::InitializeWithWorkaroundsImpl(
     mailbox_manager_ = &owned_mailbox_manager_;
   }
 
-  gl::GLShareGroup* share_group = NULL;
+  gl::GLShareGroup* share_group = nullptr;
   if (options.share_group_manager) {
     share_group = options.share_group_manager->share_group();
   } else if (options.share_mailbox_manager) {
     share_group = options.share_mailbox_manager->share_group();
   }
 
-  gles2::ContextGroup* context_group = NULL;
+  gles2::ContextGroup* context_group = nullptr;
   scoped_refptr<gles2::ShareGroup> client_share_group;
   if (options.share_group_manager) {
     context_group = options.share_group_manager->decoder_->GetContextGroup();
@@ -293,7 +312,7 @@ void GLManager::InitializeWithWorkaroundsImpl(
       options.share_group_manager->gles2_implementation()->share_group();
   }
 
-  gl::GLContext* real_gl_context = NULL;
+  gl::GLContext* real_gl_context = nullptr;
   if (options.virtual_manager &&
       !gpu_preferences_.use_passthrough_cmd_decoder) {
     real_gl_context = options.virtual_manager->context();
@@ -321,21 +340,22 @@ void GLManager::InitializeWithWorkaroundsImpl(
       std::make_unique<gles2::ShaderTranslatorCache>(gpu_preferences_);
 
   if (!context_group) {
+    GpuFeatureInfo gpu_feature_info;
     scoped_refptr<gles2::FeatureInfo> feature_info =
-        new gles2::FeatureInfo(workarounds);
+        new gles2::FeatureInfo(workarounds, gpu_feature_info);
     // Always mark the passthrough command decoder as supported so that tests do
     // not unexpectedly use the wrong command decoder
     context_group = new gles2::ContextGroup(
         gpu_preferences_, true, mailbox_manager_, nullptr /* memory_tracker */,
         translator_cache_.get(), &completeness_cache_, feature_info,
         options.bind_generates_resource, &image_manager_, options.image_factory,
-        nullptr /* progress_reporter */, GpuFeatureInfo(),
-        &discardable_manager_);
+        nullptr /* progress_reporter */, gpu_feature_info,
+        &discardable_manager_, &passthrough_discardable_manager_,
+        &shared_image_manager_);
   }
 
-  command_buffer_.reset(new CommandBufferCheckLostContext(
-      context_group->transfer_buffer_manager(), options.sync_point_manager,
-      options.context_lost_allowed));
+  command_buffer_.reset(
+      new CommandBufferCheckLostContext(options.context_lost_allowed));
 
   decoder_.reset(::gpu::gles2::GLES2Decoder::Create(
       command_buffer_.get(), command_buffer_->service(), &outputter_,
@@ -347,7 +367,8 @@ void GLManager::InitializeWithWorkaroundsImpl(
   command_buffer_->set_handler(decoder_.get());
 
   surface_ = gl::init::CreateOffscreenGLSurface(gfx::Size());
-  ASSERT_TRUE(surface_.get() != NULL) << "could not create offscreen surface";
+  ASSERT_TRUE(surface_.get() != nullptr)
+      << "could not create offscreen surface";
 
   if (base_context_) {
     context_ = scoped_refptr<gl::GLContext>(new gpu::GLContextVirtual(
@@ -367,7 +388,7 @@ void GLManager::InitializeWithWorkaroundsImpl(
       g_gpu_feature_info.ApplyToGLContext(context_.get());
     }
   }
-  ASSERT_TRUE(context_.get() != NULL) << "could not create GL context";
+  ASSERT_TRUE(context_.get() != nullptr) << "could not create GL context";
 
   ASSERT_TRUE(context_->MakeCurrent(surface_.get()));
 
@@ -404,9 +425,7 @@ void GLManager::InitializeWithWorkaroundsImpl(
 }
 
 size_t GLManager::GetSharedMemoryBytesAllocated() const {
-  return decoder_->GetContextGroup()
-      ->transfer_buffer_manager()
-      ->shared_memory_bytes_allocated();
+  return command_buffer_->service()->GetSharedMemoryBytesAllocated();
 }
 
 void GLManager::SetupBaseContext() {
@@ -441,10 +460,6 @@ void GLManager::PerformIdleWork() {
   decoder_->PerformIdleWork();
 }
 
-void GLManager::SetCommandsPaused(bool paused) {
-  command_buffer_->SetCommandsPaused(paused);
-}
-
 void GLManager::Destroy() {
   if (gles2_implementation_.get()) {
     MakeCurrent();
@@ -461,6 +476,7 @@ void GLManager::Destroy() {
     decoder_->Destroy(have_context);
     decoder_.reset();
   }
+  context_ = nullptr;
 }
 
 const GpuDriverBugWorkarounds& GLManager::workarounds() const {
@@ -477,8 +493,7 @@ const Capabilities& GLManager::GetCapabilities() const {
 
 int32_t GLManager::CreateImage(ClientBuffer buffer,
                                size_t width,
-                               size_t height,
-                               unsigned internalformat) {
+                               size_t height) {
   gfx::Size size(width, height);
   scoped_refptr<gl::GLImage> gl_image;
 
@@ -486,6 +501,8 @@ int32_t GLManager::CreateImage(ClientBuffer buffer,
   if (use_iosurface_memory_buffers_) {
     IOSurfaceGpuMemoryBuffer* gpu_memory_buffer =
         IOSurfaceGpuMemoryBuffer::FromClientBuffer(buffer);
+    unsigned internalformat = gpu::InternalFormatForGpuMemoryBufferFormat(
+        gpu_memory_buffer->GetFormat());
     scoped_refptr<gl::GLImageIOSurface> image(
         gl::GLImageIOSurface::Create(size, internalformat));
     if (!image->Initialize(gpu_memory_buffer->iosurface(),
@@ -496,14 +513,31 @@ int32_t GLManager::CreateImage(ClientBuffer buffer,
     gl_image = image;
   }
 #endif  // defined(OS_MACOSX)
+
+  if (use_native_pixmap_memory_buffers_) {
+    gfx::GpuMemoryBuffer* gpu_memory_buffer =
+        reinterpret_cast<gfx::GpuMemoryBuffer*>(buffer);
+    DCHECK(gpu_memory_buffer);
+    if (gpu_memory_buffer->GetType() == gfx::NATIVE_PIXMAP) {
+      gfx::GpuMemoryBufferHandle handle = gpu_memory_buffer->CloneHandle();
+      gfx::BufferFormat format = gpu_memory_buffer->GetFormat();
+      gl_image = gpu_memory_buffer_factory_->AsImageFactory()
+                     ->CreateImageForGpuMemoryBuffer(
+                         std::move(handle), size, format,
+                         gpu::kInProcessCommandBufferClientId,
+                         gpu::kNullSurfaceHandle);
+      if (!gl_image)
+        return -1;
+    }
+  }
+
   if (!gl_image) {
     GpuMemoryBufferImpl* gpu_memory_buffer =
         GpuMemoryBufferImpl::FromClientBuffer(buffer);
 
-    scoped_refptr<gl::GLImageRefCountedMemory> image(
-        new gl::GLImageRefCountedMemory(size, internalformat));
-    if (!image->Initialize(gpu_memory_buffer->bytes(),
-                           gpu_memory_buffer->GetFormat())) {
+    gfx::BufferFormat format = gpu_memory_buffer->GetFormat();
+    auto image = base::MakeRefCounted<gl::GLImageRefCountedMemory>(size);
+    if (!image->Initialize(gpu_memory_buffer->bytes(), format)) {
       return -1;
     }
     gl_image = image;
@@ -520,62 +554,69 @@ void GLManager::DestroyImage(int32_t id) {
 }
 
 void GLManager::SignalQuery(uint32_t query, base::OnceClosure callback) {
-  NOTIMPLEMENTED();
+  NOTREACHED();
 }
 
 void GLManager::CreateGpuFence(uint32_t gpu_fence_id, ClientGpuFence source) {
-  NOTIMPLEMENTED();
+  NOTREACHED();
 }
 
 void GLManager::GetGpuFence(
     uint32_t gpu_fence_id,
     base::OnceCallback<void(std::unique_ptr<gfx::GpuFence>)> callback) {
-  NOTIMPLEMENTED();
+  NOTREACHED();
 }
 
 void GLManager::SetLock(base::Lock*) {
-  NOTIMPLEMENTED();
+  NOTREACHED();
 }
 
 void GLManager::EnsureWorkVisible() {
-  // This is only relevant for out-of-process command buffers.
+  NOTREACHED();
 }
 
 gpu::CommandBufferNamespace GLManager::GetNamespaceID() const {
-  return command_buffer_->GetNamespaceID();
+  return CommandBufferNamespace::INVALID;
 }
 
 CommandBufferId GLManager::GetCommandBufferID() const {
-  return command_buffer_->GetCommandBufferID();
+  return CommandBufferId();
 }
 
 void GLManager::FlushPendingWork() {
-  // This is only relevant for out-of-process command buffers.
+  NOTREACHED();
 }
 
 uint64_t GLManager::GenerateFenceSyncRelease() {
-  return next_fence_sync_release_++;
+  NOTREACHED();
+  return 0;
 }
 
 bool GLManager::IsFenceSyncReleased(uint64_t release) {
-  return release <= command_buffer_->GetLastState().release_count;
+  NOTREACHED();
+  return false;
 }
 
 void GLManager::SignalSyncToken(const gpu::SyncToken& sync_token,
                                 base::OnceClosure callback) {
-  command_buffer_->SignalSyncToken(
-      sync_token, base::AdaptCallbackForRepeating(std::move(callback)));
+  NOTREACHED();
 }
 
-void GLManager::WaitSyncTokenHint(const gpu::SyncToken& sync_token) {}
+void GLManager::WaitSyncToken(const gpu::SyncToken& sync_token) {
+  NOTREACHED();
+}
 
 bool GLManager::CanWaitUnverifiedSyncToken(const gpu::SyncToken& sync_token) {
+  NOTREACHED();
   return false;
 }
 
-void GLManager::SetSnapshotRequested() {}
-
 ContextType GLManager::GetContextType() const {
   return context_type_;
 }
+
+void GLManager::Reset() {
+  Destroy();
+  SetupBaseContext();
+}
 }  // namespace gpu
