@@ -35,28 +35,26 @@
 #include "gpu/command_buffer/service/image_factory.h"
 #include "gpu/command_buffer/service/image_manager.h"
 #include "gpu/command_buffer/service/mailbox_manager.h"
+#include "gpu/command_buffer/service/memory_tracking.h"
 #include "gpu/command_buffer/service/scheduler.h"
+#include "gpu/command_buffer/service/service_utils.h"
+#include "gpu/ipc/common/command_buffer_id.h"
 #include "gpu/ipc/common/gpu_messages.h"
 #include "gpu/ipc/service/gles2_command_buffer_stub.h"
 #include "gpu/ipc/service/gpu_channel_manager.h"
 #include "gpu/ipc/service/gpu_channel_manager_delegate.h"
 #include "gpu/ipc/service/gpu_memory_buffer_factory.h"
+#include "gpu/ipc/service/image_decode_accelerator_stub.h"
 #include "gpu/ipc/service/raster_command_buffer_stub.h"
+#include "gpu/ipc/service/webgpu_command_buffer_stub.h"
 #include "ipc/ipc_channel.h"
 #include "ipc/message_filter.h"
 #include "ui/gl/gl_context.h"
 #include "ui/gl/gl_image_shared_memory.h"
 #include "ui/gl/gl_surface.h"
+#include "ui/gl/gl_utils.h"
 
 namespace gpu {
-namespace {
-
-CommandBufferId GenerateCommandBufferId(int channel_id, int32_t route_id) {
-  return CommandBufferId::FromUnsafeValue(
-      (static_cast<uint64_t>(channel_id) << 32) | route_id);
-}
-
-}  // anonymous namespace
 
 struct GpuChannelMessage {
   IPC::Message message;
@@ -83,6 +81,7 @@ class GPU_IPC_SERVICE_EXPORT GpuChannelMessageFilter
   GpuChannelMessageFilter(
       GpuChannel* gpu_channel,
       Scheduler* scheduler,
+      ImageDecodeAcceleratorWorker* image_decode_accelerator_worker,
       scoped_refptr<base::SingleThreadTaskRunner> main_task_runner);
 
   // Methods called on main thread.
@@ -107,6 +106,10 @@ class GPU_IPC_SERVICE_EXPORT GpuChannelMessageFilter
  private:
   ~GpuChannelMessageFilter() override;
 
+  SequenceId GetSequenceId(int32_t route_id) const;
+
+  bool HandleFlushMessage(const IPC::Message& message);
+
   bool MessageErrorHandler(const IPC::Message& message, const char* error_msg);
 
   IPC::Channel* ipc_channel_ = nullptr;
@@ -121,19 +124,32 @@ class GPU_IPC_SERVICE_EXPORT GpuChannelMessageFilter
   Scheduler* scheduler_;
   scoped_refptr<base::SingleThreadTaskRunner> main_task_runner_;
 
+  scoped_refptr<ImageDecodeAcceleratorStub> image_decode_accelerator_stub_;
   base::ThreadChecker io_thread_checker_;
 
+  bool allow_crash_for_testing_ = false;
+
   DISALLOW_COPY_AND_ASSIGN(GpuChannelMessageFilter);
 };
 
 GpuChannelMessageFilter::GpuChannelMessageFilter(
     GpuChannel* gpu_channel,
     Scheduler* scheduler,
+    ImageDecodeAcceleratorWorker* image_decode_accelerator_worker,
     scoped_refptr<base::SingleThreadTaskRunner> main_task_runner)
     : gpu_channel_(gpu_channel),
       scheduler_(scheduler),
-      main_task_runner_(std::move(main_task_runner)) {
+      main_task_runner_(std::move(main_task_runner)),
+      image_decode_accelerator_stub_(
+          base::MakeRefCounted<ImageDecodeAcceleratorStub>(
+              image_decode_accelerator_worker,
+              gpu_channel,
+              static_cast<int32_t>(
+                  GpuChannelReservedRoutes::kImageDecodeAccelerator))) {
   io_thread_checker_.DetachFromThread();
+  allow_crash_for_testing_ = gpu_channel->gpu_channel_manager()
+                                 ->gpu_preferences()
+                                 .enable_gpu_benchmarking_extension;
 }
 
 GpuChannelMessageFilter::~GpuChannelMessageFilter() {
@@ -142,6 +158,7 @@ GpuChannelMessageFilter::~GpuChannelMessageFilter() {
 
 void GpuChannelMessageFilter::Destroy() {
   base::AutoLock auto_lock(gpu_channel_lock_);
+  image_decode_accelerator_stub_->Shutdown();
   gpu_channel_ = nullptr;
 }
 
@@ -221,6 +238,29 @@ bool GpuChannelMessageFilter::OnMessageReceived(const IPC::Message& message) {
   if (message.should_unblock() || message.is_reply())
     return MessageErrorHandler(message, "Unexpected message type");
 
+  switch (message.type()) {
+    case GpuCommandBufferMsg_AsyncFlush::ID:
+    case GpuCommandBufferMsg_DestroyTransferBuffer::ID:
+    case GpuCommandBufferMsg_ReturnFrontBuffer::ID:
+    case GpuChannelMsg_CreateSharedImage::ID:
+    case GpuChannelMsg_DestroySharedImage::ID:
+      return MessageErrorHandler(message, "Invalid message");
+    case GpuChannelMsg_CrashForTesting::ID:
+      // Handle this message early, on the IO thread, in case the main
+      // thread is hung. This is the purpose of this message: generating
+      // minidumps on the bots, which are symbolized later by the test
+      // harness. Only pay attention to this message if Telemetry's GPU
+      // benchmarking extension was enabled via the command line, which
+      // exposes privileged APIs to JavaScript.
+      if (allow_crash_for_testing_) {
+        gl::Crash();
+      }
+      // Won't be reached if the extension is enabled.
+      return MessageErrorHandler(message, "Crashes for testing are disabled");
+    default:
+      break;
+  }
+
   if (message.type() == GpuChannelMsg_Nop::ID) {
     IPC::Message* reply = IPC::SyncMessage::GenerateReply(&message);
     ipc_channel_->Send(reply);
@@ -236,62 +276,78 @@ bool GpuChannelMessageFilter::OnMessageReceived(const IPC::Message& message) {
   if (!gpu_channel_)
     return MessageErrorHandler(message, "Channel destroyed");
 
-  // TODO(sunnyps): Remove the async flush message once the non-scheduler code
-  // path is removed.
-  if (message.type() == GpuCommandBufferMsg_AsyncFlush::ID)
-    return MessageErrorHandler(message, "Invalid flush message");
+  // Handle flush first so that it doesn't get handled out of order.
+  if (message.type() == GpuChannelMsg_FlushDeferredMessages::ID)
+    return HandleFlushMessage(message);
 
-  std::vector<Scheduler::Task> tasks;
-
-  if (message.type() == GpuChannelMsg_FlushCommandBuffers::ID) {
-    GpuChannelMsg_FlushCommandBuffers::Param params;
-
-    if (!GpuChannelMsg_FlushCommandBuffers::Read(&message, &params))
-      return MessageErrorHandler(message, "Invalid flush message");
-
-    std::vector<FlushParams> flush_list = std::get<0>(std::move(params));
-
-    for (auto& flush_info : flush_list) {
-      GpuCommandBufferMsg_AsyncFlush flush_message(
-          flush_info.route_id, flush_info.put_offset, flush_info.flush_id,
-          flush_info.snapshot_requested);
-
-      auto it = route_sequences_.find(flush_info.route_id);
-      if (it == route_sequences_.end()) {
-        DLOG(ERROR) << "Invalid route id in flush list";
-        continue;
-      }
-
-      tasks.emplace_back(
-          it->second /* sequence_id */,
-          base::BindOnce(&GpuChannel::HandleMessage, gpu_channel_->AsWeakPtr(),
-                         flush_message),
-          std::move(flush_info.sync_token_fences));
-    }
-
-    scheduler_->ScheduleTasks(std::move(tasks));
-
-  } else if (message.routing_id() == MSG_ROUTING_CONTROL ||
-             message.type() == GpuCommandBufferMsg_WaitForTokenInRange::ID ||
-             message.type() ==
-                 GpuCommandBufferMsg_WaitForGetOffsetInRange::ID) {
-    // It's OK to post task that may never run even for sync messages, because
-    // if the channel is destroyed, the client Send will fail.
-    main_task_runner_->PostTask(FROM_HERE,
-                                base::Bind(&GpuChannel::HandleOutOfOrderMessage,
-                                           gpu_channel_->AsWeakPtr(), message));
-  } else {
-    auto it = route_sequences_.find(message.routing_id());
-    if (it == route_sequences_.end())
-      return MessageErrorHandler(message, "Invalid route id");
-
-    scheduler_->ScheduleTask(
-        Scheduler::Task(it->second /* sequence_id */,
-                        base::BindOnce(&GpuChannel::HandleMessage,
-                                       gpu_channel_->AsWeakPtr(), message),
-                        std::vector<SyncToken>()));
+  if (message.routing_id() ==
+      static_cast<int32_t>(GpuChannelReservedRoutes::kImageDecodeAccelerator)) {
+    if (!image_decode_accelerator_stub_->OnMessageReceived(message))
+      return MessageErrorHandler(message, "Invalid image decode request");
+    return true;
   }
 
+  bool handle_out_of_order =
+      message.routing_id() == MSG_ROUTING_CONTROL ||
+      message.type() == GpuCommandBufferMsg_WaitForTokenInRange::ID ||
+      message.type() == GpuCommandBufferMsg_WaitForGetOffsetInRange::ID;
+
+  if (handle_out_of_order) {
+    // It's OK to post task that may never run even for sync messages, because
+    // if the channel is destroyed, the client Send will fail.
+    main_task_runner_->PostTask(
+        FROM_HERE, base::BindOnce(&GpuChannel::HandleOutOfOrderMessage,
+                                  gpu_channel_->AsWeakPtr(), message));
+    return true;
+  }
+
+  // Messages which do not have sync token dependencies.
+  SequenceId sequence_id = GetSequenceId(message.routing_id());
+  if (sequence_id.is_null())
+    return MessageErrorHandler(message, "Invalid route id");
+
+  scheduler_->ScheduleTask(
+      Scheduler::Task(sequence_id,
+                      base::BindOnce(&GpuChannel::HandleMessage,
+                                     gpu_channel_->AsWeakPtr(), message),
+                      std::vector<SyncToken>()));
+  return true;
+}
+
+SequenceId GpuChannelMessageFilter::GetSequenceId(int32_t route_id) const {
+  gpu_channel_lock_.AssertAcquired();
+  auto it = route_sequences_.find(route_id);
+  if (it == route_sequences_.end())
+    return SequenceId();
+  return it->second;
+}
+
+bool GpuChannelMessageFilter::HandleFlushMessage(const IPC::Message& message) {
+  DCHECK_EQ(message.type(), GpuChannelMsg_FlushDeferredMessages::ID);
+  gpu_channel_lock_.AssertAcquired();
+
+  GpuChannelMsg_FlushDeferredMessages::Param params;
+  if (!GpuChannelMsg_FlushDeferredMessages::Read(&message, &params))
+    return MessageErrorHandler(message, "Invalid flush message");
+
+  std::vector<GpuDeferredMessage> deferred_messages =
+      std::get<0>(std::move(params));
+
+  std::vector<Scheduler::Task> tasks;
+  tasks.reserve(deferred_messages.size());
+  for (auto& deferred_message : deferred_messages) {
+    auto it = route_sequences_.find(deferred_message.message.routing_id());
+    if (it == route_sequences_.end()) {
+      DLOG(ERROR) << "Invalid route id in flush list";
+      continue;
+    }
+    tasks.emplace_back(
+        it->second /* sequence_id */,
+        base::BindOnce(&GpuChannel::HandleMessage, gpu_channel_->AsWeakPtr(),
+                       std::move(deferred_message.message)),
+        std::move(deferred_message.sync_token_fences));
+  }
+  scheduler_->ScheduleTasks(std::move(tasks));
   return true;
 }
 
@@ -306,39 +362,6 @@ bool GpuChannelMessageFilter::MessageErrorHandler(const IPC::Message& message,
   return true;
 }
 
-// Definitions for constructor and destructor of this interface are needed to
-// avoid MSVC LNK2019.
-FilteredSender::FilteredSender() = default;
-
-FilteredSender::~FilteredSender() = default;
-
-SyncChannelFilteredSender::SyncChannelFilteredSender(
-    IPC::ChannelHandle channel_handle,
-    IPC::Listener* listener,
-    scoped_refptr<base::SingleThreadTaskRunner> ipc_task_runner,
-    base::WaitableEvent* shutdown_event)
-    : channel_(IPC::SyncChannel::Create(channel_handle,
-                                        IPC::Channel::MODE_SERVER,
-                                        listener,
-                                        ipc_task_runner,
-                                        base::ThreadTaskRunnerHandle::Get(),
-                                        false,
-                                        shutdown_event)) {}
-
-SyncChannelFilteredSender::~SyncChannelFilteredSender() = default;
-
-bool SyncChannelFilteredSender::Send(IPC::Message* msg) {
-  return channel_->Send(msg);
-}
-
-void SyncChannelFilteredSender::AddFilter(IPC::MessageFilter* filter) {
-  channel_->AddFilter(filter);
-}
-
-void SyncChannelFilteredSender::RemoveFilter(IPC::MessageFilter* filter) {
-  channel_->RemoveFilter(filter);
-}
-
 GpuChannel::GpuChannel(
     GpuChannelManager* gpu_channel_manager,
     Scheduler* scheduler,
@@ -348,7 +371,8 @@ GpuChannel::GpuChannel(
     scoped_refptr<base::SingleThreadTaskRunner> io_task_runner,
     int32_t client_id,
     uint64_t client_tracing_id,
-    bool is_gpu_host)
+    bool is_gpu_host,
+    ImageDecodeAcceleratorWorker* image_decode_accelerator_worker)
     : gpu_channel_manager_(gpu_channel_manager),
       scheduler_(scheduler),
       sync_point_manager_(sync_point_manager),
@@ -362,23 +386,44 @@ GpuChannel::GpuChannel(
       weak_factory_(this) {
   DCHECK(gpu_channel_manager_);
   DCHECK(client_id_);
-  filter_ = new GpuChannelMessageFilter(this, scheduler, task_runner);
+  filter_ = new GpuChannelMessageFilter(
+      this, scheduler, image_decode_accelerator_worker, task_runner);
+  // SharedImageInterfaceProxy/Stub is a singleton per channel, using a reserved
+  // route.
+  const int32_t shared_image_route_id =
+      static_cast<int32_t>(GpuChannelReservedRoutes::kSharedImageInterface);
+  shared_image_stub_ =
+      std::make_unique<SharedImageStub>(this, shared_image_route_id);
+  filter_->AddRoute(shared_image_route_id, shared_image_stub_->sequence());
+  router_.AddRoute(shared_image_route_id, shared_image_stub_.get());
 }
 
 GpuChannel::~GpuChannel() {
   // Clear stubs first because of dependencies.
   stubs_.clear();
 
-  // Destroy filter first so that scheduler gets no more messages.
+  // Destroy filter first to stop posting tasks to scheduler.
   filter_->Destroy();
 
   for (const auto& kv : stream_sequences_)
     scheduler_->DestroySequence(kv.second);
 }
 
-void GpuChannel::Init(std::unique_ptr<FilteredSender> channel) {
-  channel_ = std::move(channel);
-  channel_->AddFilter(filter_.get());
+void GpuChannel::Init(IPC::ChannelHandle channel_handle,
+                      base::WaitableEvent* shutdown_event) {
+  sync_channel_ = IPC::SyncChannel::Create(
+      channel_handle, IPC::Channel::MODE_SERVER, this, io_task_runner_.get(),
+      task_runner_.get(), false, shutdown_event);
+  sync_channel_->AddFilter(filter_.get());
+  channel_ = sync_channel_.get();
+}
+
+void GpuChannel::InitForTesting(IPC::Channel* channel) {
+  channel_ = channel;
+  // |channel| is an IPC::TestSink in tests, so don't add the filter to it
+  // because it will forward sent messages back to the filter.
+  // Call OnFilterAdded() to prevent DCHECK failures.
+  filter_->OnFilterAdded(channel);
 }
 
 void GpuChannel::SetUnhandledMessageListener(IPC::Listener* listener) {
@@ -390,10 +435,14 @@ base::WeakPtr<GpuChannel> GpuChannel::AsWeakPtr() {
 }
 
 base::ProcessId GpuChannel::GetClientPID() const {
-  DCHECK_NE(peer_pid_, base::kNullProcessId);
+  DCHECK(IsConnected());
   return peer_pid_;
 }
 
+bool GpuChannel::IsConnected() const {
+  return peer_pid_ != base::kNullProcessId;
+}
+
 bool GpuChannel::OnMessageReceived(const IPC::Message& msg) {
   // All messages should be pushed to channel_messages_ and handled separately.
   NOTREACHED();
@@ -442,8 +491,7 @@ CommandBufferStub* GpuChannel::LookupCommandBuffer(int32_t route_id) {
 
 bool GpuChannel::HasActiveWebGLContext() const {
   for (auto& kv : stubs_) {
-    ContextType context_type =
-        kv.second->context_group()->feature_info()->context_type();
+    ContextType context_type = kv.second->context_type();
     if (context_type == CONTEXT_TYPE_WEBGL1 ||
         context_type == CONTEXT_TYPE_WEBGL2) {
       return true;
@@ -452,10 +500,6 @@ bool GpuChannel::HasActiveWebGLContext() const {
   return false;
 }
 
-void GpuChannel::LoseAllContexts() {
-  gpu_channel_manager_->LoseAllContexts();
-}
-
 void GpuChannel::MarkAllContextsLost() {
   for (auto& kv : stubs_)
     kv.second->MarkContextLost();
@@ -500,8 +544,7 @@ void GpuChannel::HandleMessage(const IPC::Message& msg) {
 
   // If we get descheduled or yield while processing a message.
   if (stub && (stub->HasUnprocessedCommands() || !stub->IsScheduled())) {
-    DCHECK((uint32_t)GpuCommandBufferMsg_AsyncFlush::ID == msg.type() ||
-           (uint32_t)GpuCommandBufferMsg_WaitSyncToken::ID == msg.type());
+    DCHECK_EQ(GpuCommandBufferMsg_AsyncFlush::ID, msg.type());
     scheduler_->ContinueTask(
         stub->sequence_id(),
         base::BindOnce(&GpuChannel::HandleMessage, AsWeakPtr(), msg));
@@ -552,14 +595,11 @@ const CommandBufferStub* GpuChannel::GetOneStub() const {
 void GpuChannel::OnCreateCommandBuffer(
     const GPUCreateCommandBufferConfig& init_params,
     int32_t route_id,
-    base::SharedMemoryHandle shared_state_handle,
+    base::UnsafeSharedMemoryRegion shared_state_shm,
     ContextResult* result,
     gpu::Capabilities* capabilities) {
   TRACE_EVENT2("gpu", "GpuChannel::OnCreateCommandBuffer", "route_id", route_id,
                "offscreen", (init_params.surface_handle == kNullSurfaceHandle));
-  std::unique_ptr<base::SharedMemory> shared_state_shm(
-      new base::SharedMemory(shared_state_handle, false));
-
   // Default result on failure. Override with a more accurate failure if needed,
   // or with success.
   *result = ContextResult::kFatalFailure;
@@ -572,6 +612,13 @@ void GpuChannel::OnCreateCommandBuffer(
     return;
   }
 
+  if (gpu_channel_manager_->delegate()->IsExiting()) {
+    LOG(ERROR) << "ContextResult::kTransientFailure: trying to create command "
+                  "buffer during process shutdown.";
+    *result = gpu::ContextResult::kTransientFailure;
+    return;
+  }
+
   int32_t stream_id = init_params.stream_id;
   int32_t share_group_id = init_params.share_group_id;
   CommandBufferStub* share_group = LookupCommandBuffer(share_group_id);
@@ -604,7 +651,7 @@ void GpuChannel::OnCreateCommandBuffer(
   }
 
   CommandBufferId command_buffer_id =
-      GenerateCommandBufferId(client_id_, route_id);
+      CommandBufferIdFromChannelAndRoute(client_id_, route_id);
 
   SequenceId sequence_id = stream_sequences_[stream_id];
   if (sequence_id.is_null()) {
@@ -613,10 +660,24 @@ void GpuChannel::OnCreateCommandBuffer(
   }
 
   std::unique_ptr<CommandBufferStub> stub;
-  if (gpu_channel_manager_->gpu_preferences().enable_raster_decoder &&
-      init_params.attribs.enable_oop_rasterization &&
-      init_params.attribs.enable_raster_interface &&
-      !init_params.attribs.enable_gles2_interface) {
+  bool use_passthrough_cmd_decoder =
+      gpu_channel_manager_->gpu_preferences().use_passthrough_cmd_decoder &&
+      gles2::PassthroughCommandDecoderSupported();
+  bool allow_raster_decoder =
+      !use_passthrough_cmd_decoder ||
+      gpu_channel_manager_->gpu_preferences().enable_passthrough_raster_decoder;
+
+  if (init_params.attribs.context_type == CONTEXT_TYPE_WEBGPU) {
+    if (!gpu_channel_manager_->gpu_preferences().enable_webgpu) {
+      DLOG(ERROR) << "ContextResult::kFatalFailure: WebGPU not enabled";
+      return;
+    }
+
+    stub = std::make_unique<WebGPUCommandBufferStub>(
+        this, init_params, command_buffer_id, sequence_id, stream_id, route_id);
+  } else if (allow_raster_decoder &&
+             init_params.attribs.enable_raster_interface &&
+             !init_params.attribs.enable_gles2_interface) {
     stub = std::make_unique<RasterCommandBufferStub>(
         this, init_params, command_buffer_id, sequence_id, stream_id, route_id);
   } else {
@@ -670,45 +731,45 @@ void GpuChannel::CacheShader(const std::string& key,
 
 void GpuChannel::AddFilter(IPC::MessageFilter* filter) {
   io_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&GpuChannelMessageFilter::AddChannelFilter, filter_,
-                            base::RetainedRef(filter)));
+      FROM_HERE, base::BindOnce(&GpuChannelMessageFilter::AddChannelFilter,
+                                filter_, base::RetainedRef(filter)));
 }
 
 void GpuChannel::RemoveFilter(IPC::MessageFilter* filter) {
   io_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&GpuChannelMessageFilter::RemoveChannelFilter,
-                            filter_, base::RetainedRef(filter)));
+      FROM_HERE, base::BindOnce(&GpuChannelMessageFilter::RemoveChannelFilter,
+                                filter_, base::RetainedRef(filter)));
 }
 
-uint64_t GpuChannel::GetMemoryUsage() {
+uint64_t GpuChannel::GetMemoryUsage() const {
   // Collect the unique memory trackers in use by the |stubs_|.
-  std::set<gles2::MemoryTracker*> unique_memory_trackers;
-  for (auto& kv : stubs_)
-    unique_memory_trackers.insert(kv.second->GetMemoryTracker());
-
-  // Sum the memory usage for all unique memory trackers.
+  base::flat_set<MemoryTracker*> unique_memory_trackers;
+  unique_memory_trackers.reserve(stubs_.size());
   uint64_t size = 0;
-  for (auto* tracker : unique_memory_trackers) {
-    size += gpu_channel_manager()->gpu_memory_manager()->GetTrackerMemoryUsage(
-        tracker);
+  for (const auto& kv : stubs_) {
+    MemoryTracker* tracker = kv.second->GetMemoryTracker();
+    if (!unique_memory_trackers.insert(tracker).second) {
+      // We already counted that tracker.
+      continue;
+    }
+    size += tracker->GetSize();
   }
+  size += shared_image_stub_->GetSize();
 
   return size;
 }
 
 scoped_refptr<gl::GLImage> GpuChannel::CreateImageForGpuMemoryBuffer(
-    const gfx::GpuMemoryBufferHandle& handle,
+    gfx::GpuMemoryBufferHandle handle,
     const gfx::Size& size,
     gfx::BufferFormat format,
-    uint32_t internalformat,
     SurfaceHandle surface_handle) {
   switch (handle.type) {
     case gfx::SHARED_MEMORY_BUFFER: {
       if (!base::IsValueInRangeForNumericType<size_t>(handle.stride))
         return nullptr;
-      scoped_refptr<gl::GLImageSharedMemory> image(
-          new gl::GLImageSharedMemory(size, internalformat));
-      if (!image->Initialize(handle.handle, handle.id, format, handle.offset,
+      auto image = base::MakeRefCounted<gl::GLImageSharedMemory>(size);
+      if (!image->Initialize(handle.region, handle.id, format, handle.offset,
                              handle.stride)) {
         return nullptr;
       }
@@ -722,7 +783,7 @@ scoped_refptr<gl::GLImage> GpuChannel::CreateImageForGpuMemoryBuffer(
 
       return manager->gpu_memory_buffer_factory()
           ->AsImageFactory()
-          ->CreateImageForGpuMemoryBuffer(handle, size, format, internalformat,
+          ->CreateImageForGpuMemoryBuffer(std::move(handle), size, format,
                                           client_id_, surface_handle);
     }
   }
