@@ -7,6 +7,7 @@
 #include <nacl_io/nacl_io.h>
 #include <sys/mount.h>
 
+#include <set>
 #include <string>
 #include <utility>
 #include <vector>
@@ -22,6 +23,7 @@
 #include "base/strings/string_split.h"
 #include "base/strings/stringprintf.h"
 #include "base/synchronization/lock.h"
+#include "base/task/task_scheduler/task_scheduler.h"
 #include "base/threading/platform_thread.h"
 #include "base/threading/thread.h"
 #include "base/values.h"
@@ -54,10 +56,12 @@
 #include "remoting/proto/control.pb.h"
 #include "remoting/protocol/connection_to_host.h"
 #include "remoting/protocol/host_stub.h"
+#include "remoting/protocol/native_ip_synthesizer.h"
 #include "remoting/protocol/transport_context.h"
 #include "remoting/signaling/delegating_signal_strategy.h"
 #include "third_party/webrtc/modules/desktop_capture/desktop_region.h"
 #include "third_party/webrtc/rtc_base/helpers.h"
+#include "third_party/webrtc/rtc_base/network.h"
 #include "url/gurl.h"
 
 namespace remoting {
@@ -221,6 +225,18 @@ bool ChromotingInstance::Init(uint32_t argc,
   // Start all the threads.
   context_.Start();
 
+  // Initialize TaskScheduler. TaskScheduler::StartWithDefaultParams() doesn't
+  // work on NACL.
+  base::TaskScheduler::Create("RemotingChromeApp");
+  // TODO(etiennep): Change this to 2 in future CL.
+  constexpr int kBackgroundMaxThreads = 3;
+  constexpr int kForegroundMaxThreads = 3;
+  constexpr base::TimeDelta kSuggestedReclaimTime =
+      base::TimeDelta::FromSeconds(30);
+  base::TaskScheduler::GetInstance()->Start(
+      {{kBackgroundMaxThreads, kSuggestedReclaimTime},
+       {kForegroundMaxThreads, kSuggestedReclaimTime}});
+
   return true;
 }
 
@@ -230,7 +246,7 @@ void ChromotingInstance::HandleMessage(const pp::Var& message) {
     return;
   }
 
-  std::unique_ptr<base::Value> json = base::JSONReader::Read(
+  std::unique_ptr<base::Value> json = base::JSONReader::ReadDeprecated(
       message.AsString(), base::JSON_ALLOW_TRAILING_COMMAS);
   base::DictionaryValue* message_dict = nullptr;
   std::string method;
@@ -244,7 +260,7 @@ void ChromotingInstance::HandleMessage(const pp::Var& message) {
   }
 
   if (method == "connect") {
-    HandleConnect(*data);
+    UpdateNetConfigAndConnect(*data);
   } else if (method == "disconnect") {
     HandleDisconnect(*data);
   } else if (method == "incomingIq") {
@@ -307,7 +323,7 @@ void ChromotingInstance::DidChangeView(const pp::View& view) {
   plugin_view_ = view;
   webrtc::DesktopSize size(
       webrtc::DesktopSize(view.GetRect().width(), view.GetRect().height()));
-  mouse_input_filter_.set_input_size(size);
+  mouse_input_filter_.set_input_size(webrtc::DesktopRect::MakeSize(size));
   touch_input_scaler_.set_input_size(size);
 
   if (video_renderer_)
@@ -484,7 +500,7 @@ void ChromotingInstance::SetDesktopSize(const webrtc::DesktopSize& size,
                                         const webrtc::DesktopVector& dpi) {
   DCHECK(!dpi.is_zero());
 
-  mouse_input_filter_.set_output_size(size);
+  mouse_input_filter_.set_output_size(webrtc::DesktopRect::MakeSize(size));
   touch_input_scaler_.set_output_size(size);
 
   std::unique_ptr<base::DictionaryValue> data(new base::DictionaryValue());
@@ -689,14 +705,14 @@ void ChromotingInstance::HandleConnect(const base::DictionaryValue& data) {
                  weak_factory_.GetWeakPtr())));
 
   // Create TransportContext.
-  scoped_refptr<protocol::TransportContext> transport_context(
-      new protocol::TransportContext(
-          signal_strategy_.get(),
-          std::make_unique<PepperPortAllocatorFactory>(this),
-          std::make_unique<PepperUrlRequestFactory>(this),
-          protocol::NetworkSettings(
-              protocol::NetworkSettings::NAT_TRAVERSAL_FULL),
-          protocol::TransportRole::CLIENT));
+  transport_context_ = new protocol::TransportContext(
+      signal_strategy_.get(),
+      std::make_unique<PepperPortAllocatorFactory>(
+          this, base::BindRepeating(&ChromotingInstance::SendNetworkInfo,
+                                    base::Unretained(this))),
+      std::make_unique<PepperUrlRequestFactory>(this),
+      protocol::NetworkSettings(protocol::NetworkSettings::NAT_TRAVERSAL_FULL),
+      protocol::TransportRole::CLIENT);
 
   std::unique_ptr<protocol::CandidateSessionConfig> config =
       protocol::CandidateSessionConfig::CreateDefault();
@@ -711,7 +727,7 @@ void ChromotingInstance::HandleConnect(const base::DictionaryValue& data) {
   client_->set_protocol_config(std::move(config));
 
   // Kick off the connection.
-  client_->Start(signal_strategy_.get(), client_auth_config, transport_context,
+  client_->Start(signal_strategy_.get(), client_auth_config, transport_context_,
                  host_jid, capabilities);
 
   // Connect the input pipeline to the protocol stub.
@@ -719,7 +735,7 @@ void ChromotingInstance::HandleConnect(const base::DictionaryValue& data) {
   if (!plugin_view_.is_null()) {
     webrtc::DesktopSize size(plugin_view_.GetRect().width(),
                              plugin_view_.GetRect().height());
-    mouse_input_filter_.set_input_size(size);
+    mouse_input_filter_.set_input_size(webrtc::DesktopRect::MakeSize(size));
     touch_input_scaler_.set_input_size(size);
   }
 
@@ -1011,6 +1027,21 @@ void ChromotingInstance::Disconnect() {
   video_renderer_.reset();
   audio_player_.reset();
   stats_update_timer_.Stop();
+  transport_context_ = nullptr;
+}
+
+void ChromotingInstance::UpdateNetConfigAndConnect(
+    const base::DictionaryValue& data) {
+  // We can't bind HandleConnect() directly because base::DictionaryValue has
+  // no copy constructor. CreateDeepCopy() returns a unique_ptr.
+  protocol::RefreshNativeIpSynthesizer(
+      base::BindOnce(&ChromotingInstance::OnNetConfigUpdated,
+                     weak_factory_.GetWeakPtr(), data.CreateDeepCopy()));
+}
+
+void ChromotingInstance::OnNetConfigUpdated(
+    std::unique_ptr<base::DictionaryValue> data) {
+  HandleConnect(*data);
 }
 
 void ChromotingInstance::PostChromotingMessage(const std::string& method,
@@ -1161,4 +1192,24 @@ void ChromotingInstance::UpdateUmaCustomHistogram(
   }
 }
 
+void ChromotingInstance::SendNetworkInfo() {
+  if (!transport_context_)
+    return;
+  rtc::NetworkManager* network_manager = transport_context_->network_manager();
+  if (!network_manager)
+    return;
+
+  rtc::NetworkManager::NetworkList network_list;
+  network_manager->GetNetworks(&network_list);
+
+  std::set<std::string> network_names;
+  for (const auto* network : network_list) {
+    network_names.insert(network->name());
+  }
+  pp::VarDictionary dictionary;
+  dictionary.Set(pp::Var("interfaceCount"),
+                 static_cast<int32_t>(network_names.size()));
+  PostChromotingMessage("networkInfo", dictionary);
+}
+
 }  // namespace remoting
