@@ -8,6 +8,7 @@
 
 #include <utility>
 
+#include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "base/macros.h"
@@ -23,6 +24,7 @@
 #include "remoting/host/client_session.h"
 #include "remoting/host/client_session_control.h"
 #include "remoting/host/desktop_session_connector.h"
+#include "remoting/host/ipc_action_executor.h"
 #include "remoting/host/ipc_audio_capturer.h"
 #include "remoting/host/ipc_input_injector.h"
 #include "remoting/host/ipc_mouse_cursor_monitor.h"
@@ -99,12 +101,19 @@ DesktopSessionProxy::DesktopSessionProxy(
       io_task_runner_(io_task_runner),
       client_session_control_(client_session_control),
       desktop_session_connector_(desktop_session_connector),
+      ipc_file_operations_factory_(this),
       pending_capture_frame_requests_(0),
       is_desktop_session_connected_(false),
       options_(options) {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 }
 
+std::unique_ptr<ActionExecutor> DesktopSessionProxy::CreateActionExecutor() {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  return std::make_unique<IpcActionExecutor>(this);
+}
+
 std::unique_ptr<AudioCapturer> DesktopSessionProxy::CreateAudioCapturer() {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 
@@ -135,6 +144,10 @@ DesktopSessionProxy::CreateMouseCursorMonitor() {
   return std::make_unique<IpcMouseCursorMonitor>(this);
 }
 
+std::unique_ptr<FileOperations> DesktopSessionProxy::CreateFileOperations() {
+  return ipc_file_operations_factory_.CreateFileOperations();
+}
+
 std::string DesktopSessionProxy::GetCapabilities() const {
   std::string result = protocol::kRateLimitResizeRequests;
   // Ask the client to send its resolution unconditionally.
@@ -148,6 +161,11 @@ std::string DesktopSessionProxy::GetCapabilities() const {
     result += protocol::kTouchEventsCapability;
   }
 
+  if (options_.enable_file_transfer()) {
+    result += " ";
+    result += protocol::kFileTransferCapability;
+  }
+
   return result;
 }
 
@@ -182,6 +200,8 @@ bool DesktopSessionProxy::OnMessageReceived(const IPC::Message& message) {
                         OnAudioPacket)
     IPC_MESSAGE_HANDLER(ChromotingDesktopNetworkMsg_CaptureResult,
                         OnCaptureResult)
+    IPC_MESSAGE_HANDLER(ChromotingDesktopNetworkMsg_DisplayChanged,
+                        OnDesktopDisplayChanged)
     IPC_MESSAGE_HANDLER(ChromotingDesktopNetworkMsg_MouseCursor,
                         OnMouseCursor)
     IPC_MESSAGE_HANDLER(ChromotingDesktopNetworkMsg_CreateSharedBuffer,
@@ -191,7 +211,10 @@ bool DesktopSessionProxy::OnMessageReceived(const IPC::Message& message) {
     IPC_MESSAGE_HANDLER(ChromotingDesktopNetworkMsg_InjectClipboardEvent,
                         OnInjectClipboardEvent)
     IPC_MESSAGE_HANDLER(ChromotingDesktopNetworkMsg_DisconnectSession,
-                        DisconnectSession);
+                        DisconnectSession)
+    IPC_MESSAGE_FORWARD(ChromotingDesktopNetworkMsg_FileResult,
+                        &ipc_file_operations_factory_,
+                        IpcFileOperations::ResultHandler::OnResult)
   IPC_END_MESSAGE_MAP()
 
   CHECK(handled) << "Received unexpected IPC type: " << message.type();
@@ -271,6 +294,12 @@ void DesktopSessionProxy::CaptureFrame() {
   }
 }
 
+bool DesktopSessionProxy::SelectSource(webrtc::DesktopCapturer::SourceId id) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+  SendToDesktop(new ChromotingNetworkDesktopMsg_SelectSource(id));
+  return true;
+}
+
 void DesktopSessionProxy::SetVideoCapturer(
     const base::WeakPtr<IpcVideoFrameCapturer> video_capturer) {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
@@ -370,9 +399,6 @@ void DesktopSessionProxy::SetScreenResolution(
     const ScreenResolution& resolution) {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 
-  if (resolution.IsEmpty())
-    return;
-
   screen_resolution_ = resolution;
 
   // Connect to the desktop session if it is not done yet.
@@ -389,12 +415,50 @@ void DesktopSessionProxy::SetScreenResolution(
   // Depending on the session kind the screen resolution can be set by either
   // the daemon (for example RDP sessions on Windows) or by the desktop session
   // agent (when sharing the physical console).
-  if (desktop_session_connector_.get())
+  // Desktop-size-restore functionality (via an empty resolution param) does not
+  // exist for the Daemon process.  Passing an empty resolution object is
+  // treated as a critical error so we want to prevent that here.
+  if (desktop_session_connector_.get() && !screen_resolution_.IsEmpty())
     desktop_session_connector_->SetScreenResolution(this, screen_resolution_);
+
+  // Passing an empty |screen_resolution_| value to the desktop process
+  // indicates that the original resolution, if one exists, should be restored.
   SendToDesktop(
       new ChromotingNetworkDesktopMsg_SetScreenResolution(screen_resolution_));
 }
 
+void DesktopSessionProxy::ExecuteAction(
+    const protocol::ActionRequest& request) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  SendToDesktop(new ChromotingNetworkDesktopMsg_ExecuteActionRequest(request));
+}
+
+void DesktopSessionProxy::WriteFile(uint64_t file_id,
+                                    const base::FilePath& filename) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  SendToDesktop(new ChromotingNetworkDesktopMsg_WriteFile(file_id, filename));
+}
+
+void DesktopSessionProxy::WriteChunk(uint64_t file_id, std::string data) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  SendToDesktop(new ChromotingNetworkDesktopMsg_WriteFileChunk(file_id, data));
+}
+
+void DesktopSessionProxy::Close(uint64_t file_id) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  SendToDesktop(new ChromotingNetworkDesktopMsg_CloseFile(file_id));
+}
+
+void DesktopSessionProxy::Cancel(uint64_t file_id) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  SendToDesktop(new ChromotingNetworkDesktopMsg_CancelFile(file_id));
+}
+
 DesktopSessionProxy::~DesktopSessionProxy() {
   DCHECK(caller_task_runner_->BelongsToCurrentThread());
 
@@ -428,8 +492,8 @@ void DesktopSessionProxy::OnAudioPacket(const std::string& serialized_packet) {
 
   // Pass a captured audio packet to |audio_capturer_|.
   audio_capture_task_runner_->PostTask(
-      FROM_HERE, base::Bind(&IpcAudioCapturer::OnAudioPacket, audio_capturer_,
-                            base::Passed(&packet)));
+      FROM_HERE, base::BindOnce(&IpcAudioCapturer::OnAudioPacket,
+                                audio_capturer_, std::move(packet)));
 }
 
 void DesktopSessionProxy::OnCreateSharedBuffer(
@@ -454,6 +518,17 @@ void DesktopSessionProxy::OnReleaseSharedBuffer(int id) {
   shared_buffers_.erase(id);
 }
 
+void DesktopSessionProxy::OnDesktopDisplayChanged(
+    const protocol::VideoLayout& displays) {
+  DCHECK(caller_task_runner_->BelongsToCurrentThread());
+
+  if (client_session_control_) {
+    auto layout = std::make_unique<protocol::VideoLayout>();
+    layout->CopyFrom(displays);
+    client_session_control_->OnDesktopDisplayChanged(std::move(layout));
+  }
+}
+
 void DesktopSessionProxy::OnCaptureResult(
     webrtc::DesktopCapturer::Result result,
     const SerializedDesktopFrame& serialized_frame) {
