@@ -84,6 +84,30 @@ void VerifyCommonProperties(std::unique_ptr<base::DictionaryValue> response,
   EXPECT_EQ(id, int_value);
 }
 
+base::DictionaryValue CreateConnectMessage(int id) {
+  base::DictionaryValue connect_message;
+  connect_message.SetInteger("id", id);
+  connect_message.SetString("type", "connect");
+  connect_message.SetString("xmppServerAddress", "talk.google.com:5222");
+  connect_message.SetBoolean("xmppServerUseTls", true);
+  connect_message.SetString("directoryBotJid", kTestBotJid);
+  connect_message.SetString("userName", kTestClientUsername);
+  connect_message.SetString("authServiceWithToken", "oauth2:sometoken");
+  connect_message.Set("iceConfig",
+                      base::JSONReader::ReadDeprecated(
+                          "{ \"iceServers\": [ { \"urls\": [ \"stun:" +
+                          std::string(kTestStunServer) + "\" ] } ] }"));
+
+  return connect_message;
+}
+
+base::DictionaryValue CreateDisconnectMessage(int id) {
+  base::DictionaryValue disconnect_message;
+  disconnect_message.SetInteger("id", id);
+  disconnect_message.SetString("type", "disconnect");
+  return disconnect_message;
+}
+
 }  // namespace
 
 class MockIt2MeHost : public It2MeHost {
@@ -135,15 +159,16 @@ void MockIt2MeHost::Connect(
   RunSetState(kRequestedAccessCode);
 
   host_context()->ui_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&It2MeHost::Observer::OnStoreAccessCode, observer_,
-                            kTestAccessCode, kTestAccessCodeLifetime));
+      FROM_HERE,
+      base::BindOnce(&It2MeHost::Observer::OnStoreAccessCode, observer_,
+                     kTestAccessCode, kTestAccessCodeLifetime));
 
   RunSetState(kReceivedAccessCode);
   RunSetState(kConnecting);
 
   host_context()->ui_task_runner()->PostTask(
-      FROM_HERE, base::Bind(&It2MeHost::Observer::OnClientAuthenticated,
-                            observer_, kTestClientUsername));
+      FROM_HERE, base::BindOnce(&It2MeHost::Observer::OnClientAuthenticated,
+                                observer_, kTestClientUsername));
 
   RunSetState(kConnected);
 }
@@ -152,7 +177,7 @@ void MockIt2MeHost::Disconnect() {
   if (!host_context()->network_task_runner()->BelongsToCurrentThread()) {
     DCHECK(host_context()->ui_task_runner()->BelongsToCurrentThread());
     host_context()->network_task_runner()->PostTask(
-        FROM_HERE, base::Bind(&MockIt2MeHost::Disconnect, this));
+        FROM_HERE, base::BindOnce(&MockIt2MeHost::Disconnect, this));
     return;
   }
 
@@ -162,8 +187,8 @@ void MockIt2MeHost::Disconnect() {
 void MockIt2MeHost::RunSetState(It2MeHostState state) {
   if (!host_context()->network_task_runner()->BelongsToCurrentThread()) {
     host_context()->network_task_runner()->PostTask(
-        FROM_HERE,
-        base::Bind(&It2MeHost::SetStateForTesting, this, state, ErrorCode::OK));
+        FROM_HERE, base::BindOnce(&It2MeHost::SetStateForTesting, this, state,
+                                  ErrorCode::OK));
   } else {
     SetStateForTesting(state, ErrorCode::OK);
   }
@@ -171,13 +196,15 @@ void MockIt2MeHost::RunSetState(It2MeHostState state) {
 
 class MockIt2MeHostFactory : public It2MeHostFactory {
  public:
-  MockIt2MeHostFactory() = default;
+  MockIt2MeHostFactory() : host(new MockIt2MeHost()) {}
   ~MockIt2MeHostFactory() override = default;
 
   scoped_refptr<It2MeHost> CreateIt2MeHost() override {
-    return new MockIt2MeHost();
+    return host;
   }
 
+  scoped_refptr<MockIt2MeHost> host;
+
  private:
   DISALLOW_COPY_AND_ASSIGN(MockIt2MeHostFactory);
 };
@@ -208,8 +235,8 @@ class It2MeNativeMessagingHostTest : public testing::Test {
   void TestBadRequest(const base::Value& message, bool expect_error_response);
   void TestConnect();
 
-  void SendConnectMessage(int id);
-  void SendDisconnectMessage(int id);
+  // Raw pointer to host factory (owned by It2MeNativeMessagingHost).
+  MockIt2MeHostFactory* factory_raw_ptr_ = nullptr;
 
  private:
   void StartHost();
@@ -259,9 +286,8 @@ void It2MeNativeMessagingHostTest::SetUp() {
                  base::Unretained(this)));
 
   host_task_runner_->PostTask(
-      FROM_HERE,
-      base::Bind(&It2MeNativeMessagingHostTest::StartHost,
-                 base::Unretained(this)));
+      FROM_HERE, base::BindOnce(&It2MeNativeMessagingHostTest::StartHost,
+                                base::Unretained(this)));
 
   // Wait until the host finishes starting.
   test_run_loop_->Run();
@@ -321,15 +347,16 @@ It2MeNativeMessagingHostTest::ReadMessageFromOutputPipe() {
     }
 
     std::string message_json(length, '\0');
-    read_result = output_read_file_.ReadAtCurrentPos(
-        base::string_as_array(&message_json), length);
+    read_result =
+        output_read_file_.ReadAtCurrentPos(base::data(message_json), length);
     if (read_result != static_cast<int>(length)) {
       LOG(ERROR) << "Message size (" << read_result
                  << ") doesn't match the header (" << length << ").";
       return nullptr;
     }
 
-    std::unique_ptr<base::Value> message = base::JSONReader::Read(message_json);
+    std::unique_ptr<base::Value> message =
+        base::JSONReader::ReadDeprecated(message_json);
     if (!message || !message->is_dict()) {
       LOG(ERROR) << "Malformed message:" << message_json;
       return nullptr;
@@ -521,10 +548,12 @@ void It2MeNativeMessagingHostTest::StartHost() {
   policy_loader_ = policy_loader.get();
   std::unique_ptr<PolicyWatcher> policy_watcher =
       PolicyWatcher::CreateFromPolicyLoaderForTesting(std::move(policy_loader));
+  auto factory = std::make_unique<MockIt2MeHostFactory>();
+  factory_raw_ptr_ = factory.get();
   std::unique_ptr<It2MeNativeMessagingHost> it2me_host(
       new It2MeNativeMessagingHost(
           /*needs_elevation=*/false, std::move(policy_watcher),
-          std::move(context), base::WrapUnique(new MockIt2MeHostFactory())));
+          std::move(context), std::move(factory)));
   it2me_host->SetPolicyErrorClosureForTesting(
       base::Bind(base::IgnoreResult(&base::TaskRunner::PostTask),
                  test_message_loop_->task_runner(), FROM_HERE,
@@ -542,9 +571,8 @@ void It2MeNativeMessagingHostTest::StartHost() {
 void It2MeNativeMessagingHostTest::ExitTest() {
   if (!test_message_loop_->task_runner()->RunsTasksInCurrentSequence()) {
     test_message_loop_->task_runner()->PostTask(
-        FROM_HERE,
-        base::Bind(&It2MeNativeMessagingHostTest::ExitTest,
-                   base::Unretained(this)));
+        FROM_HERE, base::BindOnce(&It2MeNativeMessagingHostTest::ExitTest,
+                                  base::Unretained(this)));
     return;
   }
   test_run_loop_->Quit();
@@ -557,36 +585,12 @@ void It2MeNativeMessagingHostTest::ExitPolicyRunLoop() {
   }
 }
 
-void It2MeNativeMessagingHostTest::SendConnectMessage(int id) {
-  base::DictionaryValue connect_message;
-  connect_message.SetInteger("id", id);
-  connect_message.SetString("type", "connect");
-  connect_message.SetString("xmppServerAddress", "talk.google.com:5222");
-  connect_message.SetBoolean("xmppServerUseTls", true);
-  connect_message.SetString("directoryBotJid", kTestBotJid);
-  connect_message.SetString("userName", kTestClientUsername);
-  connect_message.SetString("authServiceWithToken", "oauth2:sometoken");
-  connect_message.Set(
-      "iceConfig",
-      base::JSONReader::Read("{ \"iceServers\": [ { \"urls\": [ \"stun:" +
-                             std::string(kTestStunServer) + "\" ] } ] }"));
-
-  WriteMessageToInputPipe(connect_message);
-}
-
-void It2MeNativeMessagingHostTest::SendDisconnectMessage(int id) {
-  base::DictionaryValue disconnect_message;
-  disconnect_message.SetInteger("id", id);
-  disconnect_message.SetString("type", "disconnect");
-  WriteMessageToInputPipe(disconnect_message);
-}
-
 void It2MeNativeMessagingHostTest::TestConnect() {
   int next_id = 1;
-  SendConnectMessage(next_id);
+  WriteMessageToInputPipe(CreateConnectMessage(next_id));
   VerifyConnectResponses(next_id);
   ++next_id;
-  SendDisconnectMessage(next_id);
+  WriteMessageToInputPipe(CreateDisconnectMessage(next_id));
   VerifyDisconnectResponses(next_id);
 }
 
@@ -629,6 +633,24 @@ TEST_F(It2MeNativeMessagingHostTest, ConnectMultiple) {
   }
 }
 
+TEST_F(It2MeNativeMessagingHostTest,
+       ConnectRespectsNoDialogsParameterOnChromeOsOnly) {
+  int next_id = 1;
+  base::DictionaryValue connect_message = CreateConnectMessage(next_id);
+  connect_message.SetBoolean("noDialogs", true);
+  WriteMessageToInputPipe(connect_message);
+  VerifyConnectResponses(next_id);
+#if defined(OS_CHROMEOS)
+  EXPECT_FALSE(factory_raw_ptr_->host->enable_dialogs());
+#else
+  EXPECT_TRUE(factory_raw_ptr_->host->enable_dialogs());
+#endif
+  ++next_id;
+  WriteMessageToInputPipe(CreateDisconnectMessage(next_id));
+  VerifyDisconnectResponses(next_id);
+}
+
+
 // Verify non-Dictionary requests are rejected.
 TEST_F(It2MeNativeMessagingHostTest, WrongFormat) {
   base::ListValue message;
@@ -654,7 +676,7 @@ TEST_F(It2MeNativeMessagingHostTest, BadPoliciesBeforeConnect) {
   base::DictionaryValue bad_policy;
   bad_policy.SetInteger(policy::key::kRemoteAccessHostFirewallTraversal, 1);
   SetPolicies(bad_policy);
-  SendConnectMessage(1);
+  WriteMessageToInputPipe(CreateConnectMessage(1));
   VerifyPolicyErrorResponse();
 }
 
@@ -662,7 +684,7 @@ TEST_F(It2MeNativeMessagingHostTest, BadPoliciesBeforeConnect) {
 TEST_F(It2MeNativeMessagingHostTest, BadPoliciesAfterConnect) {
   base::DictionaryValue bad_policy;
   bad_policy.SetInteger(policy::key::kRemoteAccessHostFirewallTraversal, 1);
-  SendConnectMessage(1);
+  WriteMessageToInputPipe(CreateConnectMessage(1));
   VerifyConnectResponses(1);
   SetPolicies(bad_policy);
   VerifyPolicyErrorResponse();
