@@ -9,11 +9,11 @@
 #include <vector>
 
 #include "base/base64.h"
+#include "base/bind.h"
 #include "base/callback_helpers.h"
 #include "base/command_line.h"
 #include "base/macros.h"
 #include "base/memory/ptr_util.h"
-#include "base/optional.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
@@ -34,14 +34,19 @@
 #include "third_party/webrtc/api/audio_codecs/audio_encoder_factory_template.h"
 #include "third_party/webrtc/api/audio_codecs/opus/audio_decoder_opus.h"
 #include "third_party/webrtc/api/audio_codecs/opus/audio_encoder_opus.h"
-#include "third_party/webrtc/api/test/fakeconstraints.h"
+#include "third_party/webrtc/api/create_peerconnection_factory.h"
+#include "third_party/webrtc/api/stats/rtcstats_objects.h"
+#include "third_party/webrtc/api/video_codecs/builtin_video_decoder_factory.h"
 
-using buzz::QName;
-using buzz::XmlElement;
+using jingle_xmpp::QName;
+using jingle_xmpp::XmlElement;
 
 namespace remoting {
 namespace protocol {
 
+class ScopedAllowThreadJoinForWebRtcTransport
+    : public base::ScopedAllowBaseSyncPrimitivesOutsideBlockingScope {};
+
 namespace {
 
 // Delay after candidate creation before sending transport-info message to
@@ -52,6 +57,17 @@ const int kTransportInfoSendDelayMs = 20;
 // XML namespace for the transport elements.
 const char kTransportNamespace[] = "google:remoting:webrtc";
 
+// Global minimum/maximum bitrates set for the PeerConnection.
+const int kMinBitrateBps = 1e6;  // 1 Mbps.
+const int kMaxBitrateBps = 1e8;  // 100 Mbps.
+
+// Frequency of polling for RTCStats. Polling is needed because WebRTC native
+// API does not provide a route-change notification for the connection type
+// (direct/STUN/relay).
+// TODO(lambroslambrou): Remove polling when a native API is provided.
+constexpr base::TimeDelta kRtcStatsPollingInterval =
+    base::TimeDelta::FromSeconds(2);
+
 #if !defined(NDEBUG)
 // Command line switch used to disable signature verification.
 // TODO(sergeyu): Remove this flag.
@@ -64,33 +80,6 @@ bool IsValidSessionDescriptionType(const std::string& type) {
 }
 
 void UpdateCodecParameters(SdpMessage* sdp_message, bool incoming) {
-  // Set bitrate range to 1-100 Mbps.
-  //  - Setting min bitrate here enables padding.
-  //  - The default max bitrate is 600 kbps. Increasing it allows to
-  //    use higher bandwidth when it's available.
-  //
-  // TODO(sergeyu): Padding needs to be enabled to workaround BW estimator not
-  // handling spiky traffic patterns well. This won't be necessary with a
-  // better bandwidth estimator.
-  // TODO(isheriff): The need for this should go away once we have a proper
-  // API to provide max bitrate for the case of handing over encoded
-  // frames to webrtc.
-  if (sdp_message->has_video()) {
-    const char* kParameters =
-        "x-google-min-bitrate=1000; x-google-max-bitrate=100000";
-    bool param_added = sdp_message->AddCodecParameter("VP8", kParameters);
-    param_added |= sdp_message->AddCodecParameter("VP9", kParameters);
-    param_added |= sdp_message->AddCodecParameter("H264", kParameters);
-    if (!param_added) {
-      if (incoming) {
-        LOG(WARNING) << "None of VP8/VP9/H264 was found in an incoming SDP.";
-      } else {
-        LOG(FATAL)
-            << "None of VP8/VP9/H264 was found in SDP generated by WebRTC.";
-      }
-    }
-  }
-
   // Update SDP format to use 160kbps stereo for opus codec.
   if (sdp_message->has_audio() &&
       !sdp_message->AddCodecParameter("opus",
@@ -103,6 +92,53 @@ void UpdateCodecParameters(SdpMessage* sdp_message, bool incoming) {
   }
 }
 
+// Returns true if the RTC stats report indicates a relay connection. If the
+// connection type cannot be determined (which should never happen with a valid
+// RTCStatsReport), nullopt is returned.
+base::Optional<bool> IsConnectionRelayed(
+    const rtc::scoped_refptr<const webrtc::RTCStatsReport>& report) {
+  auto transport_stats_list =
+      report->GetStatsOfType<webrtc::RTCTransportStats>();
+  if (transport_stats_list.size() != 1) {
+    LOG(ERROR) << "Unexpected number of transport stats: "
+               << transport_stats_list.size();
+    return base::nullopt;
+  }
+  std::string selected_candidate_pair_id =
+      *(transport_stats_list[0]->selected_candidate_pair_id);
+  const webrtc::RTCStats* selected_candidate_pair =
+      report->Get(selected_candidate_pair_id);
+  if (!selected_candidate_pair) {
+    LOG(ERROR) << "Expected to find RTC stats for id: "
+               << selected_candidate_pair;
+    return base::nullopt;
+  }
+  std::string local_candidate_id =
+      *(selected_candidate_pair->cast_to<webrtc::RTCIceCandidatePairStats>()
+            .local_candidate_id);
+  const webrtc::RTCStats* local_candidate = report->Get(local_candidate_id);
+  if (!local_candidate) {
+    LOG(ERROR) << "Expected to find RTC stats for id: " << local_candidate_id;
+    return base::nullopt;
+  }
+  std::string local_candidate_type =
+      *(local_candidate->cast_to<webrtc::RTCLocalIceCandidateStats>()
+            .candidate_type);
+  std::string remote_candidate_id =
+      *(selected_candidate_pair->cast_to<webrtc::RTCIceCandidatePairStats>()
+            .remote_candidate_id);
+  const webrtc::RTCStats* remote_candidate = report->Get(remote_candidate_id);
+  if (!remote_candidate) {
+    LOG(ERROR) << "Expected to find RTC stats for id: " << remote_candidate_id;
+    return base::nullopt;
+  }
+  std::string remote_candidate_type =
+      *(remote_candidate->cast_to<webrtc::RTCRemoteIceCandidateStats>()
+            .candidate_type);
+
+  return local_candidate_type == "relay" || remote_candidate_type == "relay";
+}
+
 // A webrtc::CreateSessionDescriptionObserver implementation used to receive the
 // results of creating descriptions for this end of the PeerConnection.
 class CreateSessionDescriptionObserver
@@ -171,6 +207,34 @@ class SetSessionDescriptionObserver
   DISALLOW_COPY_AND_ASSIGN(SetSessionDescriptionObserver);
 };
 
+class RTCStatsCollectorCallback : public webrtc::RTCStatsCollectorCallback {
+ public:
+  typedef base::RepeatingCallback<void(
+      const rtc::scoped_refptr<const webrtc::RTCStatsReport>& report)>
+      ResultCallback;
+
+  static RTCStatsCollectorCallback* Create(
+      const ResultCallback& result_callback) {
+    return new rtc::RefCountedObject<RTCStatsCollectorCallback>(
+        result_callback);
+  }
+
+  void OnStatsDelivered(
+      const rtc::scoped_refptr<const webrtc::RTCStatsReport>& report) override {
+    base::ResetAndReturn(&result_callback_).Run(report);
+  }
+
+ protected:
+  explicit RTCStatsCollectorCallback(const ResultCallback& result_callback)
+      : result_callback_(result_callback) {}
+  ~RTCStatsCollectorCallback() override = default;
+
+ private:
+  ResultCallback result_callback_;
+
+  DISALLOW_COPY_AND_ASSIGN(RTCStatsCollectorCallback);
+};
+
 }  // namespace
 
 class WebrtcTransport::PeerConnectionWrapper
@@ -178,23 +242,25 @@ class WebrtcTransport::PeerConnectionWrapper
  public:
   PeerConnectionWrapper(
       rtc::Thread* worker_thread,
-      std::unique_ptr<cricket::WebRtcVideoEncoderFactory> encoder_factory,
+      std::unique_ptr<webrtc::VideoEncoderFactory> encoder_factory,
       std::unique_ptr<cricket::PortAllocator> port_allocator,
       base::WeakPtr<WebrtcTransport> transport)
       : transport_(transport) {
     audio_module_ = new rtc::RefCountedObject<WebrtcAudioModule>();
 
     peer_connection_factory_ = webrtc::CreatePeerConnectionFactory(
-        worker_thread, rtc::Thread::Current(), audio_module_.get(),
+        worker_thread,  // network_thread
+        worker_thread,
+        rtc::Thread::Current(),  // signaling_thread
+        audio_module_,
         webrtc::CreateAudioEncoderFactory<webrtc::AudioEncoderOpus>(),
         webrtc::CreateAudioDecoderFactory<webrtc::AudioDecoderOpus>(),
-        encoder_factory.release(), nullptr);
-
-    webrtc::FakeConstraints constraints;
-    constraints.AddMandatory(webrtc::MediaConstraintsInterface::kEnableDtlsSrtp,
-                             webrtc::MediaConstraintsInterface::kValueTrue);
+        std::move(encoder_factory), webrtc::CreateBuiltinVideoDecoderFactory(),
+        nullptr,   // audio_mixer
+        nullptr);  // audio_processing
 
     webrtc::PeerConnectionInterface::RTCConfiguration rtc_config;
+    rtc_config.enable_dtls_srtp = true;
 
     // Set bundle_policy and rtcp_mux_policy to ensure that all channels are
     // multiplexed over a single channel.
@@ -205,15 +271,16 @@ class WebrtcTransport::PeerConnectionWrapper
 
     rtc_config.media_config.video.periodic_alr_bandwidth_probing = true;
 
+    rtc_config.sdp_semantics = webrtc::SdpSemantics::kUnifiedPlan;
+
     peer_connection_ = peer_connection_factory_->CreatePeerConnection(
-        rtc_config, &constraints, std::move(port_allocator), nullptr, this);
+        rtc_config, std::move(port_allocator), nullptr, this);
   }
 
   ~PeerConnectionWrapper() override {
-    // PeerConnection creates threads internally, which are stopped when the
-    // connection is closed. Thread.Stop() is a blocking operation.
-    // See crbug.com/660081.
-    base::ThreadRestrictions::ScopedAllowIO allow_io;
+    // PeerConnection creates threads internally, which are joined when the
+    // connection is closed. See crbug.com/660081.
+    ScopedAllowThreadJoinForWebRtcTransport allow_thread_join;
     peer_connection_->Close();
     peer_connection_ = nullptr;
     peer_connection_factory_ = nullptr;
@@ -273,7 +340,7 @@ class WebrtcTransport::PeerConnectionWrapper
   }
 
  private:
-  scoped_refptr<WebrtcAudioModule> audio_module_;
+  rtc::scoped_refptr<WebrtcAudioModule> audio_module_;
   scoped_refptr<webrtc::PeerConnectionFactoryInterface>
       peer_connection_factory_;
   scoped_refptr<webrtc::PeerConnectionInterface> peer_connection_;
@@ -392,8 +459,8 @@ bool WebrtcTransport::ProcessTransportInfo(XmlElement* transport_info) {
         session_description->Attr(QName(std::string(), "signature"));
     std::string signature;
     if (!base::Base64Decode(signature_base64, &signature) ||
-        !handshake_hmac_.Verify(type + " " + sdp_message.ToString(),
-                                signature)) {
+        !handshake_hmac_.Verify(
+            type + " " + sdp_message.NormalizedForSignature(), signature)) {
       LOG(WARNING) << "Received session-description with invalid signature.";
       bool ignore_error = false;
 #if !defined(NDEBUG)
@@ -466,6 +533,29 @@ bool WebrtcTransport::ProcessTransportInfo(XmlElement* transport_info) {
   return true;
 }
 
+void WebrtcTransport::Close(ErrorCode error) {
+  DCHECK(thread_checker_.CalledOnValidThread());
+  if (!peer_connection_wrapper_)
+    return;
+
+  weak_factory_.InvalidateWeakPtrs();
+
+  // Close and delete PeerConnection asynchronously. PeerConnection may be on
+  // the stack and so it must be destroyed later.
+  base::ThreadTaskRunnerHandle::Get()->DeleteSoon(
+      FROM_HERE, peer_connection_wrapper_.release());
+
+  if (error != OK)
+    event_handler_->OnWebrtcTransportError(error);
+}
+
+void WebrtcTransport::ApplySessionOptions(const SessionOptions& options) {
+  base::Optional<std::string> video_codec = options.Get("Video-Codec");
+  if (video_codec) {
+    preferred_video_codec_ = *video_codec;
+  }
+}
+
 void WebrtcTransport::OnLocalSessionDescriptionCreated(
     std::unique_ptr<webrtc::SessionDescriptionInterface> description,
     const std::string& error) {
@@ -516,9 +606,9 @@ void WebrtcTransport::OnLocalSessionDescriptionCreated(
 
   std::string digest;
   digest.resize(handshake_hmac_.DigestLength());
-  CHECK(handshake_hmac_.Sign(description->type() + " " + description_sdp,
-                             reinterpret_cast<uint8_t*>(&(digest[0])),
-                             digest.size()));
+  CHECK(handshake_hmac_.Sign(
+      description->type() + " " + sdp_message.NormalizedForSignature(),
+      reinterpret_cast<uint8_t*>(&(digest[0])), digest.size()));
   std::string digest_base64;
   base::Base64Encode(digest, &digest_base64);
   offer_tag->SetAttr(QName(std::string(), "signature"), digest_base64);
@@ -563,11 +653,12 @@ void WebrtcTransport::OnRemoteDescriptionSet(bool send_answer,
 
   // Create and send answer on the server.
   if (send_answer) {
+    const webrtc::PeerConnectionInterface::RTCOfferAnswerOptions options;
     peer_connection()->CreateAnswer(
         CreateSessionDescriptionObserver::Create(
             base::Bind(&WebrtcTransport::OnLocalSessionDescriptionCreated,
                        weak_factory_.GetWeakPtr())),
-        nullptr);
+        options);
   }
 
   AddPendingCandidatesIfPossible();
@@ -610,17 +701,6 @@ void WebrtcTransport::OnRenegotiationNeeded() {
   }
 }
 
-void WebrtcTransport::RequestNegotiation() {
-  DCHECK(transport_context_->role() == TransportRole::SERVER);
-
-  if (!negotiation_pending_) {
-    negotiation_pending_ = true;
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE,
-        base::Bind(&WebrtcTransport::SendOffer, weak_factory_.GetWeakPtr()));
-  }
-}
-
 void WebrtcTransport::OnIceConnectionChange(
     webrtc::PeerConnectionInterface::IceConnectionState new_state) {
   DCHECK(thread_checker_.CalledOnValidThread());
@@ -628,7 +708,12 @@ void WebrtcTransport::OnIceConnectionChange(
   if (!connected_ &&
       new_state == webrtc::PeerConnectionInterface::kIceConnectionConnected) {
     connected_ = true;
+    connection_relayed_.reset();
     event_handler_->OnWebrtcTransportConnected();
+
+    // Request RTC statistics, to determine if the connection is direct or
+    // relayed.
+    RequestRtcStats();
   } else if (connected_ &&
              new_state ==
                  webrtc::PeerConnectionInterface::kIceConnectionDisconnected &&
@@ -658,13 +743,131 @@ void WebrtcTransport::OnIceCandidate(
   candidate_element->SetBodyText(candidate_str);
   candidate_element->SetAttr(QName(std::string(), "sdpMid"),
                              candidate->sdp_mid());
-  candidate_element->SetAttr(QName(std::string(), "sdpMLineIndex"),
-                             base::IntToString(candidate->sdp_mline_index()));
+  candidate_element->SetAttr(
+      QName(std::string(), "sdpMLineIndex"),
+      base::NumberToString(candidate->sdp_mline_index()));
 
   EnsurePendingTransportInfoMessage();
   pending_transport_info_message_->AddElement(candidate_element.release());
 }
 
+void WebrtcTransport::OnStatsDelivered(
+    const rtc::scoped_refptr<const webrtc::RTCStatsReport>& report) {
+  if (!connected_)
+    return;
+
+  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
+      FROM_HERE,
+      base::BindOnce(&WebrtcTransport::RequestRtcStats,
+                     weak_factory_.GetWeakPtr()),
+      kRtcStatsPollingInterval);
+
+  base::Optional<bool> connection_relayed = IsConnectionRelayed(report);
+  if (connection_relayed == connection_relayed_) {
+    // No change in connection type. Unknown -> direct/relayed is treated as a
+    // change, so the correct initial bitrate caps are set.
+    return;
+  }
+
+  connection_relayed_ = connection_relayed;
+  if (connection_relayed_.has_value()) {
+    VLOG(0) << "Relay connection: "
+            << (connection_relayed_.value() ? "true" : "false");
+  } else {
+    LOG(ERROR) << "Connection type unknown, treating as direct.";
+  }
+
+  int max_bitrate_bps = kMaxBitrateBps;
+  if (connection_relayed.value_or(false)) {
+    int turn_max_rate_kbps = transport_context_->GetTurnMaxRateKbps();
+    if (turn_max_rate_kbps <= 0) {
+      VLOG(0) << "No TURN bitrate cap set.";
+    } else {
+      // Apply the TURN bitrate cap to prevent large amounts of packet loss.
+      // The Google TURN/relay server limits the connection speed by dropping
+      // packets, which may interact badly with WebRTC's bandwidth-estimation.
+      VLOG(0) << "Capping bitrate to " << turn_max_rate_kbps << "kbps.";
+      max_bitrate_bps = turn_max_rate_kbps * 1000;
+    }
+  }
+
+  // The max-bitrate needs to be applied even for direct (non-TURN) connections.
+  // Otherwise the video-sender b/w estimate is capped to a low default value
+  // (~600kbps).
+  // Set the global bitrate caps in addition to the VideoSender bitrates. The
+  // global caps affect the probing configuration used by b/w estimator.
+  // Setting min bitrate here enables padding.
+  //
+  // TODO(sergeyu): Padding needs to be enabled to workaround b/w estimator not
+  // handling spiky traffic patterns well. This won't be necessary with a
+  // better bandwidth estimator.
+  webrtc::BitrateSettings bitrate;
+  bitrate.min_bitrate_bps = kMinBitrateBps;
+  bitrate.max_bitrate_bps = max_bitrate_bps;
+  peer_connection()->SetBitrate(bitrate);
+
+  // Only set the cap on the VideoSender, because the AudioSender (via the
+  // Opus codec) is already configured with a lower bitrate.
+  rtc::scoped_refptr<webrtc::RtpSenderInterface> sender = GetVideoSender();
+  if (!sender) {
+    LOG(ERROR) << "Video sender not found.";
+    return;
+  }
+
+  webrtc::RtpParameters parameters = sender->GetParameters();
+  if (parameters.encodings.empty()) {
+    LOG(ERROR) << "No encodings found for sender " << sender->id();
+    return;
+  }
+
+  if (parameters.encodings.size() != 1) {
+    LOG(ERROR) << "Unexpected number of encodings ("
+               << parameters.encodings.size() << ") for sender "
+               << sender->id();
+  }
+
+  parameters.encodings[0].min_bitrate_bps = kMinBitrateBps;
+  parameters.encodings[0].max_bitrate_bps = max_bitrate_bps;
+  sender->SetParameters(parameters);
+}
+
+void WebrtcTransport::RequestRtcStats() {
+  if (!connected_)
+    return;
+
+  peer_connection()->GetStats(
+      RTCStatsCollectorCallback::Create(base::BindRepeating(
+          &WebrtcTransport::OnStatsDelivered, weak_factory_.GetWeakPtr())));
+}
+
+void WebrtcTransport::RequestNegotiation() {
+  DCHECK(transport_context_->role() == TransportRole::SERVER);
+
+  if (!negotiation_pending_) {
+    negotiation_pending_ = true;
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
+        FROM_HERE, base::BindOnce(&WebrtcTransport::SendOffer,
+                                  weak_factory_.GetWeakPtr()));
+  }
+}
+
+void WebrtcTransport::SendOffer() {
+  DCHECK(transport_context_->role() == TransportRole::SERVER);
+
+  DCHECK(negotiation_pending_);
+  negotiation_pending_ = false;
+
+  webrtc::PeerConnectionInterface::RTCOfferAnswerOptions options;
+  options.offer_to_receive_video = true;
+  options.offer_to_receive_audio = false;
+  options.ice_restart = want_ice_restart_;
+  peer_connection()->CreateOffer(
+      CreateSessionDescriptionObserver::Create(base::BindRepeating(
+          &WebrtcTransport::OnLocalSessionDescriptionCreated,
+          weak_factory_.GetWeakPtr())),
+      options);
+}
+
 void WebrtcTransport::EnsurePendingTransportInfoMessage() {
   DCHECK(thread_checker_.CalledOnValidThread());
 
@@ -685,33 +888,6 @@ void WebrtcTransport::EnsurePendingTransportInfoMessage() {
   }
 }
 
-void WebrtcTransport::SendOffer() {
-  DCHECK(transport_context_->role() == TransportRole::SERVER);
-
-  DCHECK(negotiation_pending_);
-  negotiation_pending_ = false;
-
-  webrtc::FakeConstraints offer_config;
-  offer_config.AddMandatory(
-      webrtc::MediaConstraintsInterface::kOfferToReceiveVideo,
-      webrtc::MediaConstraintsInterface::kValueTrue);
-  offer_config.AddMandatory(
-      webrtc::MediaConstraintsInterface::kOfferToReceiveAudio,
-      webrtc::MediaConstraintsInterface::kValueFalse);
-  offer_config.AddMandatory(webrtc::MediaConstraintsInterface::kEnableDtlsSrtp,
-                            webrtc::MediaConstraintsInterface::kValueTrue);
-  if (want_ice_restart_) {
-    offer_config.AddMandatory(webrtc::MediaConstraintsInterface::kIceRestart,
-                              webrtc::MediaConstraintsInterface::kValueTrue);
-    want_ice_restart_ = false;
-  }
-  peer_connection()->CreateOffer(
-      CreateSessionDescriptionObserver::Create(
-          base::Bind(&WebrtcTransport::OnLocalSessionDescriptionCreated,
-                     weak_factory_.GetWeakPtr())),
-      &offer_config);
-}
-
 void WebrtcTransport::SendTransportInfo() {
   DCHECK(thread_checker_.CalledOnValidThread());
   DCHECK(pending_transport_info_message_);
@@ -735,27 +911,15 @@ void WebrtcTransport::AddPendingCandidatesIfPossible() {
   }
 }
 
-void WebrtcTransport::Close(ErrorCode error) {
-  DCHECK(thread_checker_.CalledOnValidThread());
-  if (!peer_connection_wrapper_)
-    return;
-
-  weak_factory_.InvalidateWeakPtrs();
-
-  // Close and delete PeerConnection asynchronously. PeerConnection may be on
-  // the stack and so it must be destroyed later.
-  base::ThreadTaskRunnerHandle::Get()->DeleteSoon(
-      FROM_HERE, peer_connection_wrapper_.release());
-
-  if (error != OK)
-    event_handler_->OnWebrtcTransportError(error);
-}
-
-void WebrtcTransport::ApplySessionOptions(const SessionOptions& options) {
-  base::Optional<std::string> video_codec = options.Get("Video-Codec");
-  if (video_codec) {
-    preferred_video_codec_ = *video_codec;
+rtc::scoped_refptr<webrtc::RtpSenderInterface>
+WebrtcTransport::GetVideoSender() {
+  auto senders = peer_connection()->GetSenders();
+  for (rtc::scoped_refptr<webrtc::RtpSenderInterface> sender : senders) {
+    if (sender->media_type() == cricket::MediaType::MEDIA_TYPE_VIDEO) {
+      return sender;
+    }
   }
+  return nullptr;
 }
 
 }  // namespace protocol
