@@ -6,13 +6,16 @@
 
 #include <utility>
 
+#include "base/bind.h"
 #include "base/test/mock_callback.h"
 #include "base/test/scoped_task_environment.h"
+#include "base/unguessable_token.h"
 #include "media/audio/audio_io.h"
 #include "media/audio/mock_audio_manager.h"
 #include "media/audio/test_audio_thread.h"
-#include "mojo/edk/system/core.h"
+#include "mojo/core/embedder/embedder.h"
 #include "mojo/public/cpp/bindings/associated_binding.h"
+#include "services/audio/stream_factory.h"
 #include "services/audio/test/mock_log.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -29,6 +32,12 @@ using testing::_;
 
 namespace audio {
 
+namespace {
+
+// Aliases for use with MockCreatedCallback::Created().
+const bool successfully_ = true;
+const bool unsuccessfully_ = false;
+
 class MockStream : public media::AudioOutputStream {
  public:
   MockStream() {}
@@ -44,42 +53,23 @@ class MockStream : public media::AudioOutputStream {
   DISALLOW_COPY_AND_ASSIGN(MockStream);
 };
 
-class MockClient : public media::mojom::AudioOutputStreamClient {
- public:
-  MockClient() : binding_(this) {}
-
-  // Should only be called once.
-  media::mojom::AudioOutputStreamClientPtr MakePtr() {
-    media::mojom::AudioOutputStreamClientPtr ptr;
-    binding_.Bind(mojo::MakeRequest(&ptr));
-    binding_.set_connection_error_handler(base::BindOnce(
-        &MockClient::BindingConnectionError, base::Unretained(this)));
-    return ptr;
-  }
-
-  void CloseBinding() { binding_.Close(); }
-
-  MOCK_METHOD0(OnError, void());
-
-  MOCK_METHOD0(BindingConnectionError, void());
-
- private:
-  mojo::Binding<media::mojom::AudioOutputStreamClient> binding_;
-
-  DISALLOW_COPY_AND_ASSIGN(MockClient);
-};
+const uint32_t kPlatformErrorDisconnectReason = static_cast<uint32_t>(
+    media::mojom::AudioOutputStreamObserver::DisconnectReason::kPlatformError);
+const uint32_t kTerminatedByClientDisconnectReason =
+    static_cast<uint32_t>(media::mojom::AudioOutputStreamObserver::
+                              DisconnectReason::kTerminatedByClient);
 
 class MockObserver : public media::mojom::AudioOutputStreamObserver {
  public:
   MockObserver() : binding_(this) {}
 
-  // Should only be called once.
-  media::mojom::AudioOutputStreamObserverAssociatedPtr MakePtr() {
-    media::mojom::AudioOutputStreamObserverAssociatedPtr ptr;
-    binding_.Bind(mojo::MakeRequestAssociatedWithDedicatedPipe(&ptr));
-    binding_.set_connection_error_handler(base::BindOnce(
+  media::mojom::AudioOutputStreamObserverAssociatedPtrInfo MakePtrInfo() {
+    DCHECK(!binding_.is_bound());
+    media::mojom::AudioOutputStreamObserverAssociatedPtrInfo ptr_info;
+    binding_.Bind(mojo::MakeRequest(&ptr_info));
+    binding_.set_connection_error_with_reason_handler(base::BindOnce(
         &MockObserver::BindingConnectionError, base::Unretained(this)));
-    return ptr;
+    return ptr_info;
   }
 
   void CloseBinding() { binding_.Close(); }
@@ -88,7 +78,8 @@ class MockObserver : public media::mojom::AudioOutputStreamObserver {
   MOCK_METHOD0(DidStopPlaying, void());
   MOCK_METHOD1(DidChangeAudibleState, void(bool));
 
-  MOCK_METHOD0(BindingConnectionError, void());
+  MOCK_METHOD2(BindingConnectionError,
+               void(uint32_t /*disconnect_reason*/, const std::string&));
 
  private:
   mojo::AssociatedBinding<media::mojom::AudioOutputStreamObserver> binding_;
@@ -96,20 +87,15 @@ class MockObserver : public media::mojom::AudioOutputStreamObserver {
   DISALLOW_COPY_AND_ASSIGN(MockObserver);
 };
 
-// Aliases for use with MockCreatedCallback::Created().
-
-namespace {
-const bool successfully_ = true;
-const bool unsuccessfully_ = false;
-}  // namespace
-
 class MockCreatedCallback {
  public:
   MockCreatedCallback() {}
 
   MOCK_METHOD1(Created, void(bool /*valid*/));
 
-  void OnCreated(media::mojom::AudioDataPipePtr ptr) { Created(!!ptr); }
+  void OnCreated(media::mojom::ReadWriteAudioDataPipePtr ptr) {
+    Created(!!ptr);
+  }
 
   OutputStream::CreatedCallback Get() {
     return base::BindOnce(&MockCreatedCallback::OnCreated,
@@ -120,40 +106,67 @@ class MockCreatedCallback {
   DISALLOW_COPY_AND_ASSIGN(MockCreatedCallback);
 };
 
+}  // namespace
+
 // Instantiates various classes that we're going to want in most test cases.
 class TestEnvironment {
  public:
   TestEnvironment()
-      : audio_manager_(std::make_unique<media::TestAudioThread>(false)) {
-    mojo::edk::SetDefaultProcessErrorCallback(bad_message_callback_.Get());
+      : audio_manager_(std::make_unique<media::TestAudioThread>(false)),
+        stream_factory_(&audio_manager_),
+        stream_factory_binding_(&stream_factory_,
+                                mojo::MakeRequest(&stream_factory_ptr_)) {
+    mojo::core::SetDefaultProcessErrorCallback(bad_message_callback_.Get());
   }
 
-  ~TestEnvironment() { audio_manager_.Shutdown(); }
+  ~TestEnvironment() {
+    audio_manager_.Shutdown();
+    mojo::core::SetDefaultProcessErrorCallback(
+        mojo::core::ProcessErrorCallback());
+  }
 
   using MockDeleteCallback = base::MockCallback<OutputStream::DeleteCallback>;
   using MockBadMessageCallback =
       base::MockCallback<base::RepeatingCallback<void(const std::string&)>>;
 
-  std::unique_ptr<OutputStream> CreateStream(
-      media::mojom::AudioOutputStreamRequest request) {
-    return std::make_unique<OutputStream>(
-        created_callback_.Get(), delete_callback_.Get(), std::move(request),
-        client_.MakePtr(), observer_.MakePtr(), log_.MakePtr(), &audio_manager_,
-        "", media::AudioParameters::UnavailableDeviceParams());
+  media::mojom::AudioOutputStreamPtr CreateStream() {
+    media::mojom::AudioOutputStreamPtr stream_ptr;
+    stream_factory_ptr_->CreateOutputStream(
+        mojo::MakeRequest(&stream_ptr), observer_.MakePtrInfo(), log_.MakePtr(),
+        "", media::AudioParameters::UnavailableDeviceParams(),
+        base::UnguessableToken::Create(), base::nullopt,
+        created_callback_.Get());
+    return stream_ptr;
+  }
+
+  media::mojom::AudioOutputStreamPtr CreateStreamWithNullptrObserver() {
+    media::mojom::AudioOutputStreamPtr stream_ptr;
+    stream_factory_ptr_->CreateOutputStream(
+        mojo::MakeRequest(&stream_ptr), nullptr, log_.MakePtr(), "",
+        media::AudioParameters::UnavailableDeviceParams(),
+        base::UnguessableToken::Create(), base::nullopt,
+        created_callback_.Get());
+    return stream_ptr;
+  }
+
+  media::mojom::AudioOutputStreamPtr CreateStreamWithNullptrLog() {
+    media::mojom::AudioOutputStreamPtr stream_ptr;
+    stream_factory_ptr_->CreateOutputStream(
+        mojo::MakeRequest(&stream_ptr), observer_.MakePtrInfo(), nullptr, "",
+        media::AudioParameters::UnavailableDeviceParams(),
+        base::UnguessableToken::Create(), base::nullopt,
+        created_callback_.Get());
+    return stream_ptr;
   }
 
   media::MockAudioManager& audio_manager() { return audio_manager_; }
 
-  MockClient& client() { return client_; }
-
   MockObserver& observer() { return observer_; }
 
   MockLog& log() { return log_; }
 
   MockCreatedCallback& created_callback() { return created_callback_; }
 
-  MockDeleteCallback& delete_callback() { return delete_callback_; }
-
   MockBadMessageCallback& bad_message_callback() {
     return bad_message_callback_;
   }
@@ -161,17 +174,18 @@ class TestEnvironment {
  private:
   base::test::ScopedTaskEnvironment tasks_;
   media::MockAudioManager audio_manager_;
-  StrictMock<MockClient> client_;
+  StreamFactory stream_factory_;
+  mojom::StreamFactoryPtr stream_factory_ptr_;
+  mojo::Binding<mojom::StreamFactory> stream_factory_binding_;
   StrictMock<MockObserver> observer_;
   NiceMock<MockLog> log_;
   StrictMock<MockCreatedCallback> created_callback_;
-  StrictMock<MockDeleteCallback> delete_callback_;
   StrictMock<MockBadMessageCallback> bad_message_callback_;
 
   DISALLOW_COPY_AND_ASSIGN(TestEnvironment);
 };
 
-TEST(OutputStreamTest, ConstructDestruct) {
+TEST(AudioServiceOutputStreamTest, ConstructDestruct) {
   TestEnvironment env;
   MockStream mock_stream;
   EXPECT_CALL(env.created_callback(), Created(successfully_));
@@ -184,20 +198,71 @@ TEST(OutputStreamTest, ConstructDestruct) {
   EXPECT_CALL(mock_stream, SetVolume(1));
   EXPECT_CALL(env.log(), OnCreated(_, _));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
 
   EXPECT_CALL(env.log(), OnClosed());
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kTerminatedByClientDisconnectReason, _));
+  stream_ptr.reset();
+  base::RunLoop().RunUntilIdle();
 }
 
-TEST(OutputStreamTest, ConstructStreamAndDestructObserver_DestructsStream) {
+TEST(AudioServiceOutputStreamTest, ConstructDestructNullptrObserver) {
+  TestEnvironment env;
+  MockStream mock_stream;
+  EXPECT_CALL(env.created_callback(), Created(successfully_));
+  env.audio_manager().SetMakeOutputStreamCB(base::BindRepeating(
+      [](media::AudioOutputStream* stream, const media::AudioParameters& params,
+         const std::string& device_id) { return stream; },
+      &mock_stream));
+
+  EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
+  EXPECT_CALL(mock_stream, SetVolume(1));
+  EXPECT_CALL(env.log(), OnCreated(_, _));
+
+  media::mojom::AudioOutputStreamPtr stream_ptr =
+      env.CreateStreamWithNullptrObserver();
+  base::RunLoop().RunUntilIdle();
+  Mock::VerifyAndClear(&mock_stream);
+  Mock::VerifyAndClear(&env.created_callback());
+
+  EXPECT_CALL(env.log(), OnClosed());
+  EXPECT_CALL(mock_stream, Close());
+  stream_ptr.reset();
+  base::RunLoop().RunUntilIdle();
+}
+
+TEST(AudioServiceOutputStreamTest, ConstructDestructNullptrLog) {
+  TestEnvironment env;
+  MockStream mock_stream;
+  EXPECT_CALL(env.created_callback(), Created(successfully_));
+  env.audio_manager().SetMakeOutputStreamCB(base::BindRepeating(
+      [](media::AudioOutputStream* stream, const media::AudioParameters& params,
+         const std::string& device_id) { return stream; },
+      &mock_stream));
+
+  EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
+  EXPECT_CALL(mock_stream, SetVolume(1));
+
+  media::mojom::AudioOutputStreamPtr stream_ptr =
+      env.CreateStreamWithNullptrLog();
+  base::RunLoop().RunUntilIdle();
+  Mock::VerifyAndClear(&mock_stream);
+  Mock::VerifyAndClear(&env.created_callback());
+
+  EXPECT_CALL(mock_stream, Close());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kTerminatedByClientDisconnectReason, _));
+  stream_ptr.reset();
+  base::RunLoop().RunUntilIdle();
+}
+
+TEST(AudioServiceOutputStreamTest,
+     ConstructStreamAndDestructObserver_DestructsStream) {
   TestEnvironment env;
   MockStream mock_stream;
   env.audio_manager().SetMakeOutputStreamCB(base::BindRepeating(
@@ -209,27 +274,21 @@ TEST(OutputStreamTest, ConstructStreamAndDestructObserver_DestructsStream) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
 
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.client(), BindingConnectionError());
-  EXPECT_CALL(env.delete_callback(), Run(stream.release()))
-      .WillOnce(DeleteArg<0>());
 
   env.observer().CloseBinding();
   base::RunLoop().RunUntilIdle();
 
-  Mock::VerifyAndClear(&env.delete_callback());
   Mock::VerifyAndClear(&mock_stream);
-  Mock::VerifyAndClear(&env.client());
 }
 
-TEST(OutputStreamTest, ConstructStreamAndDestructClient_DestructsStream) {
+TEST(AudioServiceOutputStreamTest,
+     ConstructStreamAndReleaseStreamPtr_DestructsStream) {
   TestEnvironment env;
   MockStream mock_stream;
   env.audio_manager().SetMakeOutputStreamCB(base::BindRepeating(
@@ -241,61 +300,23 @@ TEST(OutputStreamTest, ConstructStreamAndDestructClient_DestructsStream) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
 
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.delete_callback(), Run(stream.release()))
-      .WillOnce(DeleteArg<0>());
-
-  env.client().CloseBinding();
-  base::RunLoop().RunUntilIdle();
-
-  Mock::VerifyAndClear(&env.delete_callback());
-  Mock::VerifyAndClear(&mock_stream);
-  Mock::VerifyAndClear(&env.observer());
-}
-
-TEST(OutputStreamTest, ConstructStreamAndReleaseStreamPtr_DestructsStream) {
-  TestEnvironment env;
-  MockStream mock_stream;
-  env.audio_manager().SetMakeOutputStreamCB(base::BindRepeating(
-      [](media::AudioOutputStream* stream, const media::AudioParameters& params,
-         const std::string& device_id) { return stream; },
-      &mock_stream));
-
-  EXPECT_CALL(env.created_callback(), Created(successfully_));
-  EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
-  EXPECT_CALL(mock_stream, SetVolume(1));
-
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
-  base::RunLoop().RunUntilIdle();
-  Mock::VerifyAndClear(&mock_stream);
-  Mock::VerifyAndClear(&env.created_callback());
-
-  EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
-  EXPECT_CALL(env.delete_callback(), Run(stream.release()))
-      .WillOnce(DeleteArg<0>());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kTerminatedByClientDisconnectReason, _));
 
   stream_ptr.reset();
   base::RunLoop().RunUntilIdle();
 
-  Mock::VerifyAndClear(&env.delete_callback());
   Mock::VerifyAndClear(&mock_stream);
-  Mock::VerifyAndClear(&env.client());
   Mock::VerifyAndClear(&env.observer());
 }
 
-TEST(OutputStreamTest, Play_Plays) {
+TEST(AudioServiceOutputStreamTest, Play_Plays) {
   TestEnvironment env;
   MockStream mock_stream;
   EXPECT_CALL(env.created_callback(), Created(successfully_));
@@ -307,9 +328,7 @@ TEST(OutputStreamTest, Play_Plays) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
@@ -329,11 +348,13 @@ TEST(OutputStreamTest, Play_Plays) {
   EXPECT_CALL(mock_stream, Close());
   EXPECT_CALL(env.observer(), DidChangeAudibleState(false)).Times(AtMost(1));
   EXPECT_CALL(env.observer(), DidStopPlaying()).Times(AtMost(1));
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kTerminatedByClientDisconnectReason, _));
+  stream_ptr.reset();
+  base::RunLoop().RunUntilIdle();
 }
 
-TEST(OutputStreamTest, PlayAndPause_PlaysAndStops) {
+TEST(AudioServiceOutputStreamTest, PlayAndPause_PlaysAndStops) {
   TestEnvironment env;
   MockStream mock_stream;
   EXPECT_CALL(env.created_callback(), Created(successfully_));
@@ -345,9 +366,7 @@ TEST(OutputStreamTest, PlayAndPause_PlaysAndStops) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
@@ -372,11 +391,13 @@ TEST(OutputStreamTest, PlayAndPause_PlaysAndStops) {
   Mock::VerifyAndClear(&env.observer());
 
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kTerminatedByClientDisconnectReason, _));
+  stream_ptr.reset();
+  base::RunLoop().RunUntilIdle();
 }
 
-TEST(OutputStreamTest, SetVolume_SetsVolume) {
+TEST(AudioServiceOutputStreamTest, SetVolume_SetsVolume) {
   double new_volume = 0.618;
   TestEnvironment env;
   MockStream mock_stream;
@@ -389,9 +410,7 @@ TEST(OutputStreamTest, SetVolume_SetsVolume) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
@@ -403,11 +422,13 @@ TEST(OutputStreamTest, SetVolume_SetsVolume) {
   Mock::VerifyAndClear(&mock_stream);
 
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kTerminatedByClientDisconnectReason, _));
+  stream_ptr.reset();
+  base::RunLoop().RunUntilIdle();
 }
 
-TEST(OutputStreamTest, SetNegativeVolume_BadMessage) {
+TEST(AudioServiceOutputStreamTest, SetNegativeVolume_BadMessage) {
   TestEnvironment env;
   MockStream mock_stream;
   EXPECT_CALL(env.created_callback(), Created(successfully_));
@@ -419,24 +440,20 @@ TEST(OutputStreamTest, SetNegativeVolume_BadMessage) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
 
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kPlatformErrorDisconnectReason, _));
   EXPECT_CALL(env.bad_message_callback(), Run(_));
-  EXPECT_CALL(env.delete_callback(), Run(stream.release()))
-      .WillOnce(DeleteArg<0>());
   stream_ptr->SetVolume(-0.1);
   base::RunLoop().RunUntilIdle();
 }
 
-TEST(OutputStreamTest, SetVolumeGreaterThanOne_BadMessage) {
+TEST(AudioServiceOutputStreamTest, SetVolumeGreaterThanOne_BadMessage) {
   TestEnvironment env;
   MockStream mock_stream;
   EXPECT_CALL(env.created_callback(), Created(successfully_));
@@ -448,46 +465,36 @@ TEST(OutputStreamTest, SetVolumeGreaterThanOne_BadMessage) {
   EXPECT_CALL(mock_stream, Open()).WillOnce(Return(true));
   EXPECT_CALL(mock_stream, SetVolume(1));
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
   base::RunLoop().RunUntilIdle();
   Mock::VerifyAndClear(&mock_stream);
   Mock::VerifyAndClear(&env.created_callback());
 
   EXPECT_CALL(mock_stream, Close());
-  EXPECT_CALL(env.observer(), BindingConnectionError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kPlatformErrorDisconnectReason, _));
   EXPECT_CALL(env.bad_message_callback(), Run(_));
-  EXPECT_CALL(env.delete_callback(), Run(stream.release()))
-      .WillOnce(DeleteArg<0>());
   stream_ptr->SetVolume(1.1);
   base::RunLoop().RunUntilIdle();
 }
 
-TEST(OutputStreamTest, ConstructWithStreamCreationFailure_SignalsError) {
+TEST(AudioServiceOutputStreamTest,
+     ConstructWithStreamCreationFailure_SignalsError) {
   TestEnvironment env;
 
   // By default, the MockAudioManager fails to create a stream.
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  std::unique_ptr<OutputStream> stream =
-      env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
 
   EXPECT_CALL(env.created_callback(), Created(unsuccessfully_));
-  EXPECT_CALL(env.observer(), BindingConnectionError());
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kPlatformErrorDisconnectReason, _));
   EXPECT_CALL(env.log(), OnError());
-  EXPECT_CALL(env.client(), OnError());
-  EXPECT_CALL(env.client(), BindingConnectionError());
-  EXPECT_CALL(env.delete_callback(), Run(stream.release()))
-      .WillOnce(DeleteArg<0>());
   base::RunLoop().RunUntilIdle();
-  Mock::VerifyAndClear(&env.delete_callback());
-  Mock::VerifyAndClear(&env.client());
   Mock::VerifyAndClear(&env.observer());
 }
 
-TEST(OutputStreamTest,
+TEST(AudioServiceOutputStreamTest,
      ConstructWithStreamCreationFailureAndDestructBeforeErrorFires_NoCrash) {
   // The main purpose of this test is to make sure that that delete callback
   // call is deferred, and that it is canceled in case of destruction.
@@ -495,19 +502,13 @@ TEST(OutputStreamTest,
 
   // By default, the MockAudioManager fails to create a stream.
 
-  media::mojom::AudioOutputStreamPtr stream_ptr;
-  {
-    EXPECT_CALL(env.created_callback(), Created(unsuccessfully_));
-    std::unique_ptr<OutputStream> stream =
-        env.CreateStream(mojo::MakeRequest(&stream_ptr));
+  media::mojom::AudioOutputStreamPtr stream_ptr = env.CreateStream();
+  EXPECT_CALL(env.created_callback(), Created(unsuccessfully_));
 
-    EXPECT_CALL(env.observer(), BindingConnectionError());
-    EXPECT_CALL(env.client(), OnError());
-    EXPECT_CALL(env.client(), BindingConnectionError());
-  }
+  EXPECT_CALL(env.observer(),
+              BindingConnectionError(kPlatformErrorDisconnectReason, _));
 
   base::RunLoop().RunUntilIdle();
-  Mock::VerifyAndClear(&env.client());
   Mock::VerifyAndClear(&env.observer());
 }
 
