@@ -4,14 +4,14 @@
 
 #include <SensorsApi.h>
 #include <objbase.h>
-#include <propvarutil.h>
 #include <sensors.h>
 
 #include "base/bind.h"
-#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
+#include "base/test/scoped_task_environment.h"
 #include "base/win/iunknown_impl.h"
+#include "base/win/propvarutil.h"
 #include "base/win/scoped_propvariant.h"
 #include "services/device/generic_sensor/fake_platform_sensor_and_provider.h"
 #include "services/device/generic_sensor/generic_sensor_consts.h"
@@ -190,7 +190,12 @@ class MockISensorDataReport : public MockCOMInterface<ISensorDataReport> {
 //                         data in OnDataUpdated event.
 class PlatformSensorAndProviderTestWin : public ::testing::Test {
  public:
+  PlatformSensorAndProviderTestWin()
+      : scoped_task_environment_(
+            base::test::ScopedTaskEnvironment::MainThreadType::IO) {}
+
   void SetUp() override {
+    EXPECT_EQ(S_OK, CoInitialize(nullptr));
     sensor_ = new NiceMock<MockISensor>();
     sensor_collection_ = new NiceMock<MockISensorCollection>();
     sensor_manager_ = new NiceMock<MockISensorManager>();
@@ -243,6 +248,14 @@ class PlatformSensorAndProviderTestWin : public ::testing::Test {
 
   void QuitInnerLoop() { run_loop_->Quit(); }
 
+  void SetUnsupportedSensor(REFSENSOR_TYPE_ID sensor) {
+    EXPECT_CALL(*sensor_manager_, GetSensorsByType(sensor, _))
+        .WillRepeatedly(
+            Invoke([](REFSENSOR_TYPE_ID type, ISensorCollection** collection) {
+              return HRESULT_FROM_WIN32(ERROR_NOT_FOUND);
+            }));
+  }
+
   // Sets sensor with REFSENSOR_TYPE_ID |sensor| to be supported by mocked
   // ISensorMager and it will be present in ISensorCollection.
   void SetSupportedSensor(REFSENSOR_TYPE_ID sensor) {
@@ -281,7 +294,7 @@ class PlatformSensorAndProviderTestWin : public ::testing::Test {
           events->AddRef();
           sensor_events_.Attach(events);
           if (this->run_loop_) {
-            message_loop_.task_runner()->PostTask(
+            scoped_task_environment_.GetMainThreadTaskRunner()->PostTask(
                 FROM_HERE,
                 base::Bind(&PlatformSensorAndProviderTestWin::QuitInnerLoop,
                            base::Unretained(this)));
@@ -295,7 +308,7 @@ class PlatformSensorAndProviderTestWin : public ::testing::Test {
         .WillByDefault(Invoke([this](ISensorEvents* events) {
           sensor_events_.Reset();
           if (this->run_loop_) {
-            message_loop_.task_runner()->PostTask(
+            scoped_task_environment_.GetMainThreadTaskRunner()->PostTask(
                 FROM_HERE,
                 base::Bind(&PlatformSensorAndProviderTestWin::QuitInnerLoop,
                            base::Unretained(this)));
@@ -375,11 +388,11 @@ class PlatformSensorAndProviderTestWin : public ::testing::Test {
     sensor_events_->OnDataUpdated(sensor_.get(), data_report.Get());
   }
 
+  base::test::ScopedTaskEnvironment scoped_task_environment_;
   scoped_refptr<MockISensorManager> sensor_manager_;
   scoped_refptr<MockISensorCollection> sensor_collection_;
   scoped_refptr<MockISensor> sensor_;
   Microsoft::WRL::ComPtr<ISensorEvents> sensor_events_;
-  base::MessageLoop message_loop_;
   scoped_refptr<PlatformSensor> platform_sensor_;
   // Inner run loop used to wait for async sensor creation callback.
   std::unique_ptr<base::RunLoop> run_loop_;
@@ -737,4 +750,31 @@ TEST_F(PlatformSensorAndProviderTestWin,
   EXPECT_TRUE(sensor->StopListening(client.get(), configuration));
 }
 
+// Tests that when only the quaternion version of the absolute orientation
+// sensor is available the provider falls back to using a fusion algorithm
+// to provide the euler angles version.
+TEST_F(PlatformSensorAndProviderTestWin,
+       CheckDeviceOrientationEulerAnglesFallback) {
+  SetUnsupportedSensor(SENSOR_TYPE_INCLINOMETER_3D);
+  SetSupportedSensor(SENSOR_TYPE_AGGREGATED_DEVICE_ORIENTATION);
+
+  auto sensor = CreateSensor(SensorType::ABSOLUTE_ORIENTATION_EULER_ANGLES);
+  EXPECT_TRUE(sensor);
+}
+
+// Tests that with neither absolute orientation sensor type available
+// the fallback logic does not generate an infinite loop.
+TEST_F(PlatformSensorAndProviderTestWin,
+       CheckDeviceOrientationFallbackFailure) {
+  SetUnsupportedSensor(SENSOR_TYPE_INCLINOMETER_3D);
+  SetUnsupportedSensor(SENSOR_TYPE_AGGREGATED_DEVICE_ORIENTATION);
+
+  auto euler_angles_sensor =
+      CreateSensor(SensorType::ABSOLUTE_ORIENTATION_EULER_ANGLES);
+  EXPECT_FALSE(euler_angles_sensor);
+  auto quaternion_sensor =
+      CreateSensor(SensorType::ABSOLUTE_ORIENTATION_QUATERNION);
+  EXPECT_FALSE(quaternion_sensor);
+}
+
 }  // namespace device
