@@ -19,10 +19,18 @@ UkmEntryBuilderBase::UkmEntryBuilderBase(ukm::SourceId source_id,
   entry_->event_hash = event_hash;
 }
 
+UkmEntryBuilderBase::UkmEntryBuilderBase(base::UkmSourceId source_id,
+                                         uint64_t event_hash)
+    : entry_(mojom::UkmEntry::New()) {
+  entry_->source_id = source_id.ToInt64();
+  entry_->event_hash = event_hash;
+}
+
 UkmEntryBuilderBase::~UkmEntryBuilderBase() = default;
 
-void UkmEntryBuilderBase::AddMetric(uint64_t metric_hash, int64_t value) {
-  entry_->metrics.emplace_back(mojom::UkmMetric::New(metric_hash, value));
+void UkmEntryBuilderBase::SetMetricInternal(uint64_t metric_hash,
+                                            int64_t value) {
+  entry_->metrics.emplace(metric_hash, value);
 }
 
 void UkmEntryBuilderBase::Record(UkmRecorder* recorder) {
@@ -32,6 +40,10 @@ void UkmEntryBuilderBase::Record(UkmRecorder* recorder) {
     entry_.reset();
 }
 
+mojom::UkmEntryPtr UkmEntryBuilderBase::TakeEntry() {
+  return std::move(entry_);
+}
+
 }  // namespace internal
 
 }  // namespace ukm
