@@ -6,7 +6,6 @@
 
 #include "base/bind.h"
 #include "base/feature_list.h"
-#include "base/memory/ptr_util.h"
 #include "build/build_config.h"
 #include "services/metrics/public/cpp/delegating_ukm_recorder.h"
 #include "services/metrics/public/cpp/ukm_entry_builder.h"
@@ -21,6 +20,9 @@ UkmRecorder::~UkmRecorder() = default;
 
 // static
 UkmRecorder* UkmRecorder::Get() {
+  // Note that SourceUrlRecorderWebContentsObserver assumes that
+  // DelegatingUkmRecorder::Get() is the canonical UkmRecorder instance. If this
+  // changes, SourceUrlRecorderWebContentsObserver should be updated to match.
   return DelegatingUkmRecorder::Get();
 }
 
@@ -29,12 +31,12 @@ ukm::SourceId UkmRecorder::GetNewSourceID() {
   return AssignNewSourceId();
 }
 
-std::unique_ptr<UkmEntryBuilder> UkmRecorder::GetEntryBuilder(
-    ukm::SourceId source_id,
-    const char* event_name) {
-  return std::make_unique<UkmEntryBuilder>(
-      base::Bind(&UkmRecorder::AddEntry, base::Unretained(this)), source_id,
-      event_name);
+void UkmRecorder::RecordOtherURL(base::UkmSourceId source_id, const GURL& url) {
+  UpdateSourceURL(source_id.ToInt64(), url);
+}
+
+void UkmRecorder::RecordAppURL(base::UkmSourceId source_id, const GURL& url) {
+  UpdateAppURL(source_id.ToInt64(), url);
 }
 
 }  // namespace ukm
