@@ -11,16 +11,22 @@
 namespace network {
 namespace features {
 
+COMPONENT_EXPORT(NETWORK_CPP)
+extern const base::Feature kExpectCTReporting;
 COMPONENT_EXPORT(NETWORK_CPP)
 extern const base::Feature kNetworkErrorLogging;
 COMPONENT_EXPORT(NETWORK_CPP)
 extern const base::Feature kNetworkService;
 COMPONENT_EXPORT(NETWORK_CPP)
-extern const base::Feature kOutOfBlinkCORS;
-COMPONENT_EXPORT(NETWORK_CPP)
-extern const base::Feature kRendererSideResourceScheduler;
+extern const base::Feature kOutOfBlinkCors;
 COMPONENT_EXPORT(NETWORK_CPP)
 extern const base::Feature kReporting;
+COMPONENT_EXPORT(NETWORK_CPP)
+extern const base::Feature kThrottleDelayable;
+COMPONENT_EXPORT(NETWORK_CPP)
+extern const base::Feature kDelayRequestsOnMultiplexedConnections;
+COMPONENT_EXPORT(NETWORK_CPP)
+extern const base::Feature kEnforceRequestInitiatorLockForCorb;
 
 }  // namespace features
 }  // namespace network
