@@ -11,10 +11,10 @@ namespace network {
 ResourceResponseInfo::ResourceResponseInfo()
     : ct_policy_compliance(net::ct::CTPolicyCompliance::
                                CT_POLICY_COMPLIANCE_DETAILS_NOT_AVAILABLE),
-      is_legacy_symantec_cert(false),
       content_length(-1),
       encoded_data_length(-1),
       encoded_body_length(-1),
+      network_accessed(false),
       appcache_id(0),
       was_fetched_via_spdy(false),
       was_alpn_negotiated(false),
@@ -22,14 +22,13 @@ ResourceResponseInfo::ResourceResponseInfo()
       connection_info(net::HttpResponseInfo::CONNECTION_INFO_UNKNOWN),
       was_fetched_via_service_worker(false),
       was_fallback_required_by_service_worker(false),
-      response_type_via_service_worker(mojom::FetchResponseType::kDefault),
-      previews_state(0),
+      response_type(mojom::FetchResponseType::kDefault),
       effective_connection_type(net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN),
       cert_status(0),
-      ssl_connection_status(0),
-      ssl_key_exchange_group(0),
       did_service_worker_navigation_preload(false),
-      blocked_cross_site_document(false) {}
+      should_report_corb_blocking(false),
+      async_revalidation_requested(false),
+      did_mime_sniff(false) {}
 
 ResourceResponseInfo::ResourceResponseInfo(const ResourceResponseInfo& other) =
     default;
