@@ -11,21 +11,24 @@
 
 #include "base/bind.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/metrics/field_trial.h"
 #include "base/metrics/field_trial_params.h"
+#include "base/metrics/histogram_functions.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/optional.h"
 #include "base/sequenced_task_runner.h"
 #include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/supports_user_data.h"
+#include "base/time/default_tick_clock.h"
+#include "base/time/tick_clock.h"
 #include "base/trace_event/trace_event.h"
 #include "net/base/host_port_pair.h"
 #include "net/base/load_flags.h"
 #include "net/base/request_priority.h"
 #include "net/http/http_server_properties.h"
 #include "net/log/net_log.h"
+#include "net/nqe/effective_connection_type_observer.h"
 #include "net/nqe/network_quality_estimator.h"
 #include "net/url_request/url_request.h"
 #include "net/url_request/url_request_context.h"
@@ -36,44 +39,6 @@ namespace network {
 
 namespace {
 
-// When kPrioritySupportedRequestsDelayable is enabled, requests for
-// H2/QUIC/SPDY resources can be delayed by the ResourceScheduler just as
-// HTTP/1.1 resources are. It has good impact on performance, but breaks
-// expected behavior of H2. See intent-to-unship:
-// https://groups.google.com/a/chromium.org/forum/#!topic/blink-
-// dev/ChqGX8UyHz8. We're keeping it around for finch trials to compare
-// alternatives to.
-const base::Feature kPrioritySupportedRequestsDelayable{
-    "PrioritySupportedRequestsDelayable", base::FEATURE_DISABLED_BY_DEFAULT};
-
-// When enabled, low-priority H2 and QUIC requests are throttled, but only
-// when the parser is in head.
-const base::Feature kHeadPrioritySupportedRequestsDelayable{
-    "HeadPriorityRequestsDelayable", base::FEATURE_DISABLED_BY_DEFAULT};
-
-// In the event that many resource requests are started quickly, this feature
-// will periodically yield (e.g., delaying starting of requests) by posting a
-// task and waiting for the task to run to resume. This allows other
-// operations that rely on the IO thread (e.g., already running network
-// requests) to make progress.
-const base::Feature kNetworkSchedulerYielding{
-    "NetworkSchedulerYielding", base::FEATURE_DISABLED_BY_DEFAULT};
-const char kMaxRequestsBeforeYieldingParam[] = "MaxRequestsBeforeYieldingParam";
-const int kMaxRequestsBeforeYieldingDefault = 5;
-const char kYieldMsParam[] = "MaxYieldMs";
-const int kYieldMsDefault = 0;
-
-// Based on the field trial parameters, this feature will override the value of
-// the maximum number of delayable requests allowed in flight. The number of
-// delayable requests allowed in flight will be based on the network's
-// effective connection type ranges and the
-// corresponding number of delayable requests in flight specified in the
-// experiment configuration. Based on field trial parameters, this experiment
-// may also throttle delayable requests based on the number of non-delayable
-// requests in-flight times a weighting factor.
-const base::Feature kThrottleDelayable{"ThrottleDelayable",
-                                       base::FEATURE_DISABLED_BY_DEFAULT};
-
 enum StartMode { START_SYNC, START_ASYNC };
 
 // Flags identifying various attributes of the request that are used
@@ -93,7 +58,7 @@ enum class RequestStartTrigger {
   CLIENT_KILL,
   SPDY_PROXY_DETECTED,
   REQUEST_REPRIORITIZED,
-  START_WAS_YIELDED,
+  LONG_QUEUED_REQUESTS_TIMER_FIRED,
 };
 
 const char* RequestStartTriggerString(RequestStartTrigger trigger) {
@@ -112,32 +77,22 @@ const char* RequestStartTriggerString(RequestStartTrigger trigger) {
       return "SPDY_PROXY_DETECTED";
     case RequestStartTrigger::REQUEST_REPRIORITIZED:
       return "REQUEST_REPRIORITIZED";
-    case RequestStartTrigger::START_WAS_YIELDED:
-      return "START_WAS_YIELDED";
+    case RequestStartTrigger::LONG_QUEUED_REQUESTS_TIMER_FIRED:
+      return "LONG_QUEUED_REQUESTS_TIMER_FIRED";
   }
-  NOTREACHED();
-  return "Unknown";
 }
 
 }  // namespace
 
-// The maximum number of delayable requests to allow to be in-flight at any
-// point in time (across all hosts).
-static const size_t kDefaultMaxNumDelayableRequestsPerClient = 10;
-
 // The maximum number of requests to allow be in-flight at any point in time per
-// host.
+// host. This limit does not apply to hosts that support request prioritization
+// when |delay_requests_on_multiplexed_connections| is true.
 static const size_t kMaxNumDelayableRequestsPerHostPerClient = 6;
 
 // The maximum number of delayable requests to allow to be in-flight at any
 // point in time while in the layout-blocking phase of loading.
 static const size_t kMaxNumDelayableWhileLayoutBlockingPerClient = 1;
 
-// The priority level above which resources are considered layout-blocking if
-// the html_body has not started.
-static const net::RequestPriority kLayoutBlockingPriorityThreshold =
-    net::MEDIUM;
-
 // The priority level below which resources are considered to be delayable.
 static const net::RequestPriority kDelayablePriorityThreshold = net::MEDIUM;
 
@@ -145,6 +100,15 @@ static const net::RequestPriority kDelayablePriorityThreshold = net::MEDIUM;
 // requests should be blocked.
 static const size_t kInFlightNonDelayableRequestCountPerClientThreshold = 1;
 
+// Duration after which the timer to dispatch long queued requests should fire.
+// The request needs to be queued for at least 15 seconds before it can be
+// dispatched. Choosing 5 seconds as the checking interval ensures that the
+// queue is not checked too frequently. The interval is also not too long, so
+// we do not expect too many long queued requests to go on the network at the
+// same time.
+constexpr base::TimeDelta kLongQueuedRequestsDispatchPeriodicity =
+    base::TimeDelta::FromSeconds(5);
+
 struct ResourceScheduler::RequestPriorityParams {
   RequestPriorityParams()
       : priority(net::DEFAULT_PRIORITY), intra_priority(0) {}
@@ -173,8 +137,8 @@ struct ResourceScheduler::RequestPriorityParams {
 
 class ResourceScheduler::RequestQueue {
  public:
-  typedef std::multiset<ScheduledResourceRequestImpl*, ScheduledResourceSorter>
-      NetQueue;
+  using NetQueue =
+      std::multiset<ScheduledResourceRequestImpl*, ScheduledResourceSorter>;
 
   RequestQueue() : fifo_ordering_ids_(0) {}
   ~RequestQueue() {}
@@ -200,11 +164,11 @@ class ResourceScheduler::RequestQueue {
   }
 
   // Returns true if no requests are queued.
-  bool IsEmpty() const { return queue_.size() == 0; }
+  bool IsEmpty() const { return queue_.empty(); }
 
  private:
-  typedef std::map<ScheduledResourceRequestImpl*, NetQueue::iterator>
-      PointerMap;
+  using PointerMap =
+      std::map<ScheduledResourceRequestImpl*, NetQueue::iterator>;
 
   uint32_t MakeFifoOrderingId() {
     fifo_ordering_ids_ += 1;
@@ -358,7 +322,7 @@ class ResourceScheduler::ScheduledResourceRequestImpl
 
   // Maximum number of delayable requests in-flight when |this| was in-flight.
   size_t peak_delayable_requests_in_flight_;
-  // Cached to excessive recomputation in ShouldKeepSearching.
+  // Cached to excessive recomputation in ReachedMaxRequestsPerHostPerClient().
   const net::HostPortPair host_port_pair_;
 
   base::WeakPtrFactory<ResourceScheduler::ScheduledResourceRequestImpl>
@@ -395,33 +359,29 @@ void ResourceScheduler::RequestQueue::Insert(
 }
 
 // Each client represents a tab.
-class ResourceScheduler::Client {
+class ResourceScheduler::Client : public net::EffectiveConnectionTypeObserver {
  public:
-  Client(const net::NetworkQualityEstimator* const network_quality_estimator,
-         ResourceScheduler* resource_scheduler)
-      : deprecated_is_loaded_(false),
-        deprecated_has_html_body_(false),
-        using_spdy_proxy_(false),
-        in_flight_delayable_count_(0),
+  Client(net::NetworkQualityEstimator* network_quality_estimator,
+         ResourceScheduler* resource_scheduler,
+         const base::TickClock* tick_clock)
+      : in_flight_delayable_count_(0),
         total_layout_blocking_count_(0),
         num_skipped_scans_due_to_scheduled_start_(0),
-        started_requests_since_yielding_(0),
-        did_scheduler_yield_(false),
         network_quality_estimator_(network_quality_estimator),
-        max_delayable_requests_(
-            resource_scheduler->throttle_delayable_
-                .GetParamsForNetworkQuality(network_quality_estimator)
-                .max_delayable_requests),
         resource_scheduler_(resource_scheduler),
+        tick_clock_(tick_clock),
         weak_ptr_factory_(this) {
-    if (IsRendererSideResourceSchedulerEnabled()) {
-      // In this case, "layout blocking" concept is moved to the renderer side,
-      // so the shceduler works always with the normal mode.
-      deprecated_has_html_body_ = true;
-    }
+    DCHECK(tick_clock_);
+
+    UpdateParamsForNetworkQuality();
+    if (network_quality_estimator_)
+      network_quality_estimator_->AddEffectiveConnectionTypeObserver(this);
   }
 
-  ~Client() {}
+  ~Client() override {
+    if (network_quality_estimator_)
+      network_quality_estimator_->RemoveEffectiveConnectionTypeObserver(this);
+  }
 
   void ScheduleRequest(const net::URLRequest& url_request,
                        ScheduledResourceRequestImpl* request) {
@@ -432,8 +392,6 @@ class ResourceScheduler::Client {
       StartRequest(request, START_SYNC, RequestStartTrigger::NONE);
     } else {
       pending_requests_.Insert(request);
-      if (should_start == YIELD_SCHEDULER)
-        did_scheduler_yield_ = true;
     }
   }
 
@@ -446,8 +404,7 @@ class ResourceScheduler::Client {
 
       // Removing this request may have freed up another to load.
       LoadAnyStartablePendingRequests(
-          deprecated_has_html_body_ ? RequestStartTrigger::COMPLETION_POST_BODY
-                                    : RequestStartTrigger::COMPLETION_PRE_BODY);
+          RequestStartTrigger::COMPLETION_POST_BODY);
     }
   }
 
@@ -476,43 +433,6 @@ class ResourceScheduler::Client {
     return unowned_requests;
   }
 
-  bool deprecated_is_loaded() const { return deprecated_is_loaded_; }
-
-  void DeprecatedOnLoadingStateChanged(bool is_loaded) {
-    deprecated_is_loaded_ = is_loaded;
-  }
-
-  void DeprecatedOnNavigate() {
-    deprecated_has_html_body_ = false;
-    if (IsRendererSideResourceSchedulerEnabled()) {
-      // In this case, "layout blocking" concept is moved to the renderer side,
-      // so the shceduler works always with the normal mode.
-      deprecated_has_html_body_ = true;
-    }
-
-    deprecated_is_loaded_ = false;
-    max_delayable_requests_ =
-        resource_scheduler_->throttle_delayable_
-            .GetParamsForNetworkQuality(network_quality_estimator_)
-            .max_delayable_requests;
-  }
-
-  void DeprecatedOnWillInsertBody() {
-    // Can be called multiple times per RVH in the case of out-of-process
-    // iframes.
-    if (deprecated_has_html_body_)
-      return;
-    deprecated_has_html_body_ = true;
-    LoadAnyStartablePendingRequests(RequestStartTrigger::BODY_REACHED);
-  }
-
-  void OnReceivedSpdyProxiedHttpResponse() {
-    if (!using_spdy_proxy_) {
-      using_spdy_proxy_ = true;
-      LoadAnyStartablePendingRequests(RequestStartTrigger::SPDY_PROXY_DETECTED);
-    }
-  }
-
   void ReprioritizeRequest(ScheduledResourceRequestImpl* request,
                            RequestPriorityParams old_priority_params,
                            RequestPriorityParams new_priority_params) {
@@ -535,14 +455,36 @@ class ResourceScheduler::Client {
     }
   }
 
+  // Updates the params based on the current network quality estimate.
+  void UpdateParamsForNetworkQuality() {
+    params_for_network_quality_ =
+        resource_scheduler_->resource_scheduler_params_manager_
+            .GetParamsForEffectiveConnectionType(
+                network_quality_estimator_
+                    ? network_quality_estimator_->GetEffectiveConnectionType()
+                    : net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN);
+  }
+
+  void OnLongQueuedRequestsDispatchTimerFired() {
+    LoadAnyStartablePendingRequests(
+        RequestStartTrigger::LONG_QUEUED_REQUESTS_TIMER_FIRED);
+  }
+
+  bool HasNoPendingRequests() const { return pending_requests_.IsEmpty(); }
+
  private:
   enum ShouldStartReqResult {
     DO_NOT_START_REQUEST_AND_STOP_SEARCHING,
     DO_NOT_START_REQUEST_AND_KEEP_SEARCHING,
-    START_REQUEST,
-    YIELD_SCHEDULER
+    START_REQUEST
   };
 
+  // net::EffectiveConnectionTypeObserver implementation:
+  void OnEffectiveConnectionTypeChanged(
+      net::EffectiveConnectionType effective_connection_type) override {
+    UpdateParamsForNetworkQuality();
+  }
+
   // Records the metrics related to number of requests in flight.
   void RecordRequestCountMetrics() const {
     UMA_HISTOGRAM_COUNTS_100("ResourceScheduler.RequestsCount.All",
@@ -667,17 +609,10 @@ class ResourceScheduler::Client {
       // If a request is already marked as layout-blocking make sure to keep the
       // attribute across redirects.
       attributes |= kAttributeLayoutBlocking;
-    } else if (!deprecated_has_html_body_ &&
-               request->url_request()->priority() >
-                   kLayoutBlockingPriorityThreshold) {
-      // Requests that are above the non_delayable threshold before the HTML
-      // body has been parsed are inferred to be layout-blocking.
-      attributes |= kAttributeLayoutBlocking;
     } else if (request->url_request()->priority() <
                kDelayablePriorityThreshold) {
-      if (resource_scheduler_->priority_requests_delayable() ||
-          (resource_scheduler_->head_priority_requests_delayable() &&
-           !deprecated_has_html_body_)) {
+      if (params_for_network_quality_
+              .delay_requests_on_multiplexed_connections) {
         // Resources below the delayable priority threshold that are considered
         // delayable.
         attributes |= kAttributeDelayable;
@@ -696,7 +631,27 @@ class ResourceScheduler::Client {
     return attributes;
   }
 
-  bool ShouldKeepSearching(const net::HostPortPair& active_request_host) const {
+  bool ReachedMaxRequestsPerHostPerClient(
+      const net::HostPortPair& active_request_host,
+      bool supports_priority) const {
+    // This method should not be called for requests to origins that support
+    // prioritization (aka multiplexing) unless one of the experiments to
+    // throttle priority requests is enabled.
+    DCHECK(
+        !supports_priority ||
+        params_for_network_quality_.delay_requests_on_multiplexed_connections);
+
+    // kMaxNumDelayableRequestsPerHostPerClient limit does not apply to servers
+    // that support request priorities when
+    // |delay_requests_on_multiplexed_connections| is true. If
+    // |delay_requests_on_multiplexed_connections| is false, then
+    // kMaxNumDelayableRequestsPerHostPerClient limit still applies to other
+    // experiments that delay priority requests.
+    if (supports_priority &&
+        params_for_network_quality_.delay_requests_on_multiplexed_connections) {
+      return false;
+    }
+
     size_t same_host_count = 0;
     for (RequestSet::const_iterator it = in_flight_requests_.begin();
          it != in_flight_requests_.end(); ++it) {
@@ -712,21 +667,6 @@ class ResourceScheduler::Client {
   void StartRequest(ScheduledResourceRequestImpl* request,
                     StartMode start_mode,
                     RequestStartTrigger trigger) {
-    if (resource_scheduler_->yielding_scheduler_enabled()) {
-      started_requests_since_yielding_ += 1;
-      if (started_requests_since_yielding_ == 1) {
-        // This is the first started request since last yielding. Post a task to
-        // reset the counter and start any yielded tasks if necessary. We post
-        // this now instead of when we first yield so that if there is a pause
-        // between requests the counter is reset.
-        resource_scheduler_->task_runner()->PostDelayedTask(
-            FROM_HERE,
-            base::BindOnce(&Client::ResumeIfYielded,
-                           weak_ptr_factory_.GetWeakPtr()),
-            resource_scheduler_->yield_time());
-      }
-    }
-
     // Only log on requests that were blocked by the ResourceScheduler.
     if (start_mode == START_ASYNC) {
       DCHECK_NE(RequestStartTrigger::NONE, trigger);
@@ -742,6 +682,16 @@ class ResourceScheduler::Client {
           "ResourceScheduler.NumDelayableRequestsInFlightAtStart.NonDelayable",
           in_flight_delayable_count_);
     }
+
+    DCHECK(!request->url_request()->creation_time().is_null());
+    base::TimeDelta queuing_duration =
+        base::TimeTicks::Now() - request->url_request()->creation_time();
+    base::UmaHistogramMediumTimes(
+        "ResourceScheduler.RequestQueuingDuration.Priority" +
+            base::NumberToString(
+                request->get_request_priority_params().priority),
+        queuing_duration);
+
     InsertInFlightRequest(request);
     request->Start(start_mode);
   }
@@ -796,26 +746,28 @@ class ResourceScheduler::Client {
     if (!url_request.url().SchemeIsHTTPOrHTTPS())
       return START_REQUEST;
 
+    if (params_for_network_quality_.max_queuing_time &&
+        tick_clock_->NowTicks() - url_request.creation_time() >=
+            params_for_network_quality_.max_queuing_time) {
+      return START_REQUEST;
+    }
+
     const net::HostPortPair& host_port_pair = request->host_port_pair();
 
     bool priority_delayable =
-        resource_scheduler_->priority_requests_delayable() ||
-        (resource_scheduler_->head_priority_requests_delayable() &&
-         !deprecated_has_html_body_);
+        params_for_network_quality_.delay_requests_on_multiplexed_connections;
+
+    url::SchemeHostPort scheme_host_port(url_request.url());
+    bool supports_priority = url_request.context()
+                                 ->http_server_properties()
+                                 ->SupportsRequestPriority(scheme_host_port);
 
     if (!priority_delayable) {
-      if (using_spdy_proxy_ && url_request.url().SchemeIs(url::kHttpScheme))
-        return ShouldStartOrYieldRequest(request);
-
-      url::SchemeHostPort scheme_host_port(url_request.url());
-
-      net::HttpServerProperties& http_server_properties =
-          *url_request.context()->http_server_properties();
       // TODO(willchan): We should really improve this algorithm as described in
       // https://crbug.com/164101. Also, theoretically we should not count a
       // request-priority capable request against the delayable requests limit.
-      if (http_server_properties.SupportsRequestPriority(scheme_host_port))
-        return ShouldStartOrYieldRequest(request);
+      if (supports_priority)
+        return START_REQUEST;
     }
 
     // Non-delayable requests.
@@ -825,16 +777,14 @@ class ResourceScheduler::Client {
     // Delayable requests.
     DCHECK_GE(in_flight_requests_.size(), in_flight_delayable_count_);
     size_t num_non_delayable_requests_weighted = static_cast<size_t>(
-        resource_scheduler_->throttle_delayable_
-            .GetParamsForNetworkQuality(network_quality_estimator_)
-            .non_delayable_weight *
+        params_for_network_quality_.non_delayable_weight *
         (in_flight_requests_.size() - in_flight_delayable_count_));
     if ((in_flight_delayable_count_ + num_non_delayable_requests_weighted >=
-         max_delayable_requests_)) {
+         params_for_network_quality_.max_delayable_requests)) {
       return DO_NOT_START_REQUEST_AND_STOP_SEARCHING;
     }
 
-    if (ShouldKeepSearching(host_port_pair)) {
+    if (ReachedMaxRequestsPerHostPerClient(host_port_pair, supports_priority)) {
       // There may be other requests for other hosts that may be allowed,
       // so keep checking.
       return DO_NOT_START_REQUEST_AND_KEEP_SEARCHING;
@@ -845,7 +795,7 @@ class ResourceScheduler::Client {
     // delayable requests is handled above here so this is deciding what to
     // do with a delayable request while we are in the layout-blocking phase
     // of loading.
-    if (!deprecated_has_html_body_ || total_layout_blocking_count_ != 0) {
+    if (total_layout_blocking_count_ != 0) {
       size_t non_delayable_requests_in_flight_count =
           in_flight_requests_.size() - in_flight_delayable_count_;
       if (non_delayable_requests_in_flight_count >
@@ -886,34 +836,6 @@ class ResourceScheduler::Client {
     num_skipped_scans_due_to_scheduled_start_ += 1;
   }
 
-  void ResumeIfYielded() {
-    bool yielded = did_scheduler_yield_;
-    started_requests_since_yielding_ = 0;
-    did_scheduler_yield_ = false;
-
-    if (yielded)
-      LoadAnyStartablePendingRequests(RequestStartTrigger::START_WAS_YIELDED);
-  }
-
-  // For a request that is ready to start, return START_REQUEST if the
-  // scheduler doesn't need to yield, else YIELD_SCHEDULER.
-  ShouldStartReqResult ShouldStartOrYieldRequest(
-      ScheduledResourceRequestImpl* request) const {
-    DCHECK_GE(started_requests_since_yielding_, 0);
-
-    // Don't yield if:
-    // 1. The yielding scheduler isn't enabled
-    // 2. The resource is high priority
-    // 3. There haven't been enough recent requests to warrant yielding.
-    if (!resource_scheduler_->yielding_scheduler_enabled() ||
-        request->url_request()->priority() >= kDelayablePriorityThreshold ||
-        started_requests_since_yielding_ <
-            resource_scheduler_->max_requests_before_yielding()) {
-      return START_REQUEST;
-    }
-    return YIELD_SCHEDULER;
-  }
-
   void LoadAnyStartablePendingRequests(RequestStartTrigger trigger) {
     // We iterate through all the pending requests, starting with the highest
     // priority one. For each entry, one of three things can happen:
@@ -950,9 +872,6 @@ class ResourceScheduler::Client {
       } else if (query_result == DO_NOT_START_REQUEST_AND_KEEP_SEARCHING) {
         ++request_iter;
         continue;
-      } else if (query_result == YIELD_SCHEDULER) {
-        did_scheduler_yield_ = true;
-        break;
       } else {
         DCHECK(query_result == DO_NOT_START_REQUEST_AND_STOP_SEARCHING);
         break;
@@ -960,13 +879,10 @@ class ResourceScheduler::Client {
     }
   }
 
-  bool deprecated_is_loaded_;
   // Tracks if the main HTML parser has reached the body which marks the end of
   // layout-blocking resources.
   // This is disabled and the is always true when kRendererSideResourceScheduler
   // is enabled.
-  bool deprecated_has_html_body_;
-  bool using_spdy_proxy_;
   RequestQueue pending_requests_;
   RequestSet in_flight_requests_;
   // The number of delayable in-flight requests.
@@ -978,49 +894,34 @@ class ResourceScheduler::Client {
   // to smarter task scheduling around reprioritization.
   int num_skipped_scans_due_to_scheduled_start_;
 
-  // The number of started requests since the last ResumeIfYielded task was
-  // run.
-  int started_requests_since_yielding_;
-
-  // If the scheduler had to yield the start of a request since the last
-  // ResumeIfYielded task was run.
-  bool did_scheduler_yield_;
-
   // Network quality estimator for network aware resource scheudling. This may
   // be null.
-  const net::NetworkQualityEstimator* const network_quality_estimator_;
+  net::NetworkQualityEstimator* network_quality_estimator_;
 
-  // The value of the maximum number of delayable requests in flight. This gets
-  // recalculated every time an |OnNavigate| event is triggered.
-  size_t max_delayable_requests_;
+  // Resource scheduling params computed for the current network quality.
+  // These are recomputed every time an |OnNavigate| event is triggered.
+  ResourceSchedulerParamsManager::ParamsForNetworkQuality
+      params_for_network_quality_;
 
   // A pointer to the resource scheduler which contains the resource scheduling
   // configuration.
   ResourceScheduler* resource_scheduler_;
 
+  // Guaranteed to be non-null.
+  const base::TickClock* tick_clock_;
+
   base::WeakPtrFactory<ResourceScheduler::Client> weak_ptr_factory_;
 };
 
-ResourceScheduler::ResourceScheduler(bool enabled)
-    : enabled_(enabled),
-      priority_requests_delayable_(
-          base::FeatureList::IsEnabled(kPrioritySupportedRequestsDelayable)),
-      head_priority_requests_delayable_(base::FeatureList::IsEnabled(
-          kHeadPrioritySupportedRequestsDelayable)),
-      yielding_scheduler_enabled_(
-          base::FeatureList::IsEnabled(kNetworkSchedulerYielding)),
-      max_requests_before_yielding_(base::GetFieldTrialParamByFeatureAsInt(
-          kNetworkSchedulerYielding,
-          kMaxRequestsBeforeYieldingParam,
-          kMaxRequestsBeforeYieldingDefault)),
-      yield_time_(base::TimeDelta::FromMilliseconds(
-          base::GetFieldTrialParamByFeatureAsInt(kNetworkSchedulerYielding,
-                                                 kYieldMsParam,
-                                                 kYieldMsDefault))),
+ResourceScheduler::ResourceScheduler(bool enabled,
+                                     const base::TickClock* tick_clock)
+    : tick_clock_(tick_clock ? tick_clock
+                             : base::DefaultTickClock::GetInstance()),
+      enabled_(enabled),
       task_runner_(base::ThreadTaskRunnerHandle::Get()) {
-  // Don't run the two experiments together.
-  if (priority_requests_delayable_ && head_priority_requests_delayable_)
-    priority_requests_delayable_ = false;
+  DCHECK(tick_clock_);
+
+  StartLongQueuedRequestsDispatchTimerIfNeeded();
 }
 
 ResourceScheduler::~ResourceScheduler() {
@@ -1052,8 +953,12 @@ ResourceScheduler::ScheduleRequest(int child_id,
     return std::move(request);
   }
 
-  Client* client = it->second;
+  Client* client = it->second.get();
   client->ScheduleRequest(*url_request, request.get());
+
+  if (!IsLongQueuedRequestsDispatchTimerRunning())
+    StartLongQueuedRequestsDispatchTimerIfNeeded();
+
   return std::move(request);
 }
 
@@ -1065,33 +970,39 @@ void ResourceScheduler::RemoveRequest(ScheduledResourceRequestImpl* request) {
   }
 
   ClientMap::iterator client_it = client_map_.find(request->client_id());
-  if (client_it == client_map_.end()) {
+  if (client_it == client_map_.end())
     return;
-  }
 
-  Client* client = client_it->second;
+  Client* client = client_it->second.get();
   client->RemoveRequest(request);
 }
 
 void ResourceScheduler::OnClientCreated(
     int child_id,
     int route_id,
-    const net::NetworkQualityEstimator* const network_quality_estimator) {
+    net::NetworkQualityEstimator* network_quality_estimator) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
   ClientId client_id = MakeClientId(child_id, route_id);
   DCHECK(!base::ContainsKey(client_map_, client_id));
 
-  Client* client = new Client(network_quality_estimator, this);
-  client_map_[client_id] = client;
+  client_map_[client_id] =
+      std::make_unique<Client>(network_quality_estimator, this, tick_clock_);
 }
 
 void ResourceScheduler::OnClientDeleted(int child_id, int route_id) {
   DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+
   ClientId client_id = MakeClientId(child_id, route_id);
   ClientMap::iterator it = client_map_.find(client_id);
-  DCHECK(it != client_map_.end());
+  // TODO(crbug.com/873959): Turns this CHECK to DCHECK once the investigation
+  // is done.
+  CHECK(it != client_map_.end());
 
-  Client* client = it->second;
+  Client* client = it->second.get();
+  // TODO(crbug.com/873959): Remove this CHECK once the investigation is done.
+  CHECK(client);
+  DCHECK(client->HasNoPendingRequests() ||
+         IsLongQueuedRequestsDispatchTimerRunning());
   // ResourceDispatcherHost cancels all requests except for cross-renderer
   // navigations, async revalidations and detachable requests after
   // OnClientDeleted() returns.
@@ -1101,76 +1012,45 @@ void ResourceScheduler::OnClientDeleted(int child_id, int route_id) {
     unowned_requests_.insert(*request_it);
   }
 
-  delete client;
   client_map_.erase(it);
 }
 
-void ResourceScheduler::DeprecatedOnLoadingStateChanged(int child_id,
-                                                        int route_id,
-                                                        bool is_loaded) {
-  Client* client = GetClient(child_id, route_id);
-  DCHECK(client);
-  client->DeprecatedOnLoadingStateChanged(is_loaded);
-}
-
-void ResourceScheduler::DeprecatedOnNavigate(int child_id, int route_id) {
-  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  ClientId client_id = MakeClientId(child_id, route_id);
-
-  ClientMap::iterator it = client_map_.find(client_id);
-  if (it == client_map_.end()) {
-    // The client was likely deleted shortly before we received this IPC.
-    return;
-  }
-
-  Client* client = it->second;
-  client->DeprecatedOnNavigate();
-}
-
-void ResourceScheduler::DeprecatedOnWillInsertBody(int child_id, int route_id) {
-  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  ClientId client_id = MakeClientId(child_id, route_id);
-
-  ClientMap::iterator it = client_map_.find(client_id);
-  if (it == client_map_.end()) {
-    // The client was likely deleted shortly before we received this IPC.
-    return;
-  }
-
-  Client* client = it->second;
-  client->DeprecatedOnWillInsertBody();
-}
-
-void ResourceScheduler::OnReceivedSpdyProxiedHttpResponse(int child_id,
-                                                          int route_id) {
-  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
-  ClientId client_id = MakeClientId(child_id, route_id);
-
-  ClientMap::iterator client_it = client_map_.find(client_id);
-  if (client_it == client_map_.end()) {
-    return;
-  }
-
-  Client* client = client_it->second;
-  client->OnReceivedSpdyProxiedHttpResponse();
-}
-
-bool ResourceScheduler::DeprecatedHasLoadingClients() const {
-  for (const auto& client : client_map_) {
-    if (!client.second->deprecated_is_loaded())
-      return true;
-  }
-  return false;
-}
-
 ResourceScheduler::Client* ResourceScheduler::GetClient(int child_id,
                                                         int route_id) {
   ClientId client_id = MakeClientId(child_id, route_id);
   ClientMap::iterator client_it = client_map_.find(client_id);
-  if (client_it == client_map_.end()) {
+  if (client_it == client_map_.end())
     return nullptr;
+  return client_it->second.get();
+}
+
+void ResourceScheduler::StartLongQueuedRequestsDispatchTimerIfNeeded() {
+  bool pending_request_found = false;
+  for (const auto& client : client_map_) {
+    if (!client.second->HasNoPendingRequests()) {
+      pending_request_found = true;
+      break;
+    }
   }
-  return client_it->second;
+
+  // If there are no pending requests, then do not start the timer. This ensures
+  // that we are not running the periodic timer when Chrome is not being
+  // actively used (e.g., it's in background).
+  if (!pending_request_found)
+    return;
+
+  long_queued_requests_dispatch_timer_.Start(
+      FROM_HERE, kLongQueuedRequestsDispatchPeriodicity, this,
+      &ResourceScheduler::OnLongQueuedRequestsDispatchTimerFired);
+}
+
+void ResourceScheduler::OnLongQueuedRequestsDispatchTimerFired() {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+
+  for (auto& client : client_map_)
+    client.second->OnLongQueuedRequestsDispatchTimerFired();
+
+  StartLongQueuedRequestsDispatchTimerIfNeeded();
 }
 
 void ResourceScheduler::ReprioritizeRequest(net::URLRequest* request,
@@ -1208,7 +1088,7 @@ void ResourceScheduler::ReprioritizeRequest(net::URLRequest* request,
     return;
   }
 
-  Client* client = client_it->second;
+  Client* client = client_it->second.get();
   client->ReprioritizeRequest(scheduled_resource_request, old_priority_params,
                               new_priority_params);
 }
@@ -1224,89 +1104,29 @@ void ResourceScheduler::ReprioritizeRequest(net::URLRequest* request,
   ReprioritizeRequest(request, new_priority, current_intra_priority);
 }
 
-ResourceScheduler::ClientId ResourceScheduler::MakeClientId(int child_id,
-                                                            int route_id) {
+ResourceScheduler::ClientId ResourceScheduler::MakeClientId(
+    int child_id,
+    int route_id) const {
   return (static_cast<ResourceScheduler::ClientId>(child_id) << 32) | route_id;
 }
 
-ResourceScheduler::ThrottleDelayable::ThrottleDelayable()
-    : params_for_network_quality_container_(
-          GetParamsForNetworkQualityContainer()) {}
-
-ResourceScheduler::ThrottleDelayable::~ThrottleDelayable() {}
-
-ResourceScheduler::ParamsForNetworkQuality
-ResourceScheduler::ThrottleDelayable::GetParamsForNetworkQuality(
-    const net::NetworkQualityEstimator* network_quality_estimator) const {
-  if (network_quality_estimator) {
-    net::EffectiveConnectionType effective_connection_type =
-        network_quality_estimator->GetEffectiveConnectionType();
-
-    for (const auto& range : params_for_network_quality_container_) {
-      if (effective_connection_type == range.effective_connection_type) {
-        return range;
-      }
-    }
-  }
-
-  return ResourceScheduler::ParamsForNetworkQuality{
-      net::EFFECTIVE_CONNECTION_TYPE_UNKNOWN,
-      kDefaultMaxNumDelayableRequestsPerClient, 0.0};
+bool ResourceScheduler::IsLongQueuedRequestsDispatchTimerRunning() const {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+  return long_queued_requests_dispatch_timer_.IsRunning();
 }
 
-ResourceScheduler::ParamsForNetworkQualityContainer
-ResourceScheduler::ThrottleDelayable::GetParamsForNetworkQualityContainer() {
-  static const char kMaxDelayableRequestsBase[] = "MaxDelayableRequests";
-  static const char kEffectiveConnectionTypeBase[] = "EffectiveConnectionType";
-  static const char kNonDelayableWeightBase[] = "NonDelayableWeight";
-
-  ParamsForNetworkQualityContainer result;
-  if (!base::FeatureList::IsEnabled(kThrottleDelayable))
-    return result;
-
-  int config_param_index = 1;
-  while (true) {
-    size_t max_delayable_requests;
-
-    if (!base::StringToSizeT(
-            base::GetFieldTrialParamValueByFeature(
-                kThrottleDelayable, kMaxDelayableRequestsBase +
-                                        base::IntToString(config_param_index)),
-            &max_delayable_requests)) {
-      DCHECK_LE(result.size(), 20u);
-      return result;
-    }
-
-    base::Optional<net::EffectiveConnectionType> effective_connection_type =
-        net::GetEffectiveConnectionTypeForName(
-            base::GetFieldTrialParamValueByFeature(
-                kThrottleDelayable, kEffectiveConnectionTypeBase +
-                                        base::IntToString(config_param_index)));
-    if (!effective_connection_type)
-      return result;
-
-    double non_delayable_weight;
-    if (!base::StringToDouble(
-            base::GetFieldTrialParamValueByFeature(
-                kThrottleDelayable, kNonDelayableWeightBase +
-                                        base::IntToString(config_param_index)),
-            &non_delayable_weight)) {
-      return result;
-    }
-
-    result.push_back({effective_connection_type.value(), max_delayable_requests,
-                      non_delayable_weight});
-    config_param_index++;
+void ResourceScheduler::SetResourceSchedulerParamsManagerForTests(
+    const ResourceSchedulerParamsManager& resource_scheduler_params_manager) {
+  DCHECK_CALLED_ON_VALID_SEQUENCE(sequence_checker_);
+  resource_scheduler_params_manager_.Reset(resource_scheduler_params_manager);
+  for (const auto& pair : client_map_) {
+    pair.second->UpdateParamsForNetworkQuality();
   }
 }
 
-bool ResourceScheduler::IsRendererSideResourceSchedulerEnabled() {
-  // We are assuming that kRendererSideResourceScheduler will be shipped when
-  // launching Network Service, so let's act as if
-  // kRendererSideResourceScheduler is enabled when kNetworkService is enabled.
-  return base::FeatureList::IsEnabled(
-             features::kRendererSideResourceScheduler) ||
-         base::FeatureList::IsEnabled(features::kNetworkService);
+void ResourceScheduler::DispatchLongQueuedRequestsForTesting() {
+  long_queued_requests_dispatch_timer_.Stop();
+  OnLongQueuedRequestsDispatchTimerFired();
 }
 
 }  // namespace network
