@@ -48,7 +48,7 @@ constexpr base::TimeDelta kHeapDumpTimeout = base::TimeDelta::FromSeconds(60);
 // event.
 class StringWrapper : public base::trace_event::ConvertableToTraceFormat {
  public:
-  explicit StringWrapper(std::string json) : json_(std::move(json)) {}
+  explicit StringWrapper(std::string&& json) : json_(std::move(json)) {}
 
   void AppendAsTraceFormat(std::string* out) const override {
     out->append(json_);
@@ -67,7 +67,8 @@ CoordinatorImpl* CoordinatorImpl::GetInstance() {
 
 CoordinatorImpl::CoordinatorImpl(service_manager::Connector* connector)
     : next_dump_id_(0),
-      client_process_timeout_(base::TimeDelta::FromSeconds(15)) {
+      client_process_timeout_(base::TimeDelta::FromSeconds(15)),
+      weak_ptr_factory_(this) {
   process_map_ = std::make_unique<ProcessMap>(connector);
   DCHECK(!g_coordinator_impl);
   g_coordinator_impl = this;
@@ -112,57 +113,83 @@ void CoordinatorImpl::RequestGlobalMemoryDump(
     MemoryDumpType dump_type,
     MemoryDumpLevelOfDetail level_of_detail,
     const std::vector<std::string>& allocator_dump_names,
-    const RequestGlobalMemoryDumpCallback& callback) {
+    RequestGlobalMemoryDumpCallback callback) {
   // This merely strips out the |dump_guid| argument.
-  auto adapter = [](const RequestGlobalMemoryDumpCallback& callback,
-                    bool success, uint64_t,
-                    mojom::GlobalMemoryDumpPtr global_memory_dump) {
-    callback.Run(success, std::move(global_memory_dump));
+  auto adapter = [](RequestGlobalMemoryDumpCallback callback, bool success,
+                    uint64_t, mojom::GlobalMemoryDumpPtr global_memory_dump) {
+    std::move(callback).Run(success, std::move(global_memory_dump));
   };
 
   QueuedRequest::Args args(dump_type, level_of_detail, allocator_dump_names,
-                           false /* add_to_trace */, base::kNullProcessId);
-  RequestGlobalMemoryDumpInternal(args, base::BindRepeating(adapter, callback));
+                           false /* add_to_trace */, base::kNullProcessId,
+                           /*memory_footprint_only=*/false);
+  RequestGlobalMemoryDumpInternal(args,
+                                  base::BindOnce(adapter, std::move(callback)));
 }
 
 void CoordinatorImpl::RequestGlobalMemoryDumpForPid(
     base::ProcessId pid,
-    const RequestGlobalMemoryDumpForPidCallback& callback) {
+    const std::vector<std::string>& allocator_dump_names,
+    RequestGlobalMemoryDumpForPidCallback callback) {
   // Error out early if process id is null to avoid confusing with global
   // dump for all processes case when pid is kNullProcessId.
   if (pid == base::kNullProcessId) {
-    callback.Run(false, nullptr);
+    std::move(callback).Run(false, nullptr);
     return;
   }
 
   // This merely strips out the |dump_guid| argument; this is not relevant
   // as we are not adding to trace.
-  auto adapter = [](const RequestGlobalMemoryDumpForPidCallback& callback,
+  auto adapter = [](RequestGlobalMemoryDumpForPidCallback callback,
                     bool success, uint64_t,
                     mojom::GlobalMemoryDumpPtr global_memory_dump) {
-    callback.Run(success, std::move(global_memory_dump));
+    std::move(callback).Run(success, std::move(global_memory_dump));
+  };
+
+  QueuedRequest::Args args(
+      base::trace_event::MemoryDumpType::SUMMARY_ONLY,
+      base::trace_event::MemoryDumpLevelOfDetail::BACKGROUND,
+      allocator_dump_names, false /* add_to_trace */, pid,
+      /*memory_footprint_only=*/false);
+  RequestGlobalMemoryDumpInternal(args,
+                                  base::BindOnce(adapter, std::move(callback)));
+}
+
+void CoordinatorImpl::RequestPrivateMemoryFootprint(
+    base::ProcessId pid,
+    RequestPrivateMemoryFootprintCallback callback) {
+  // This merely strips out the |dump_guid| argument; this is not relevant
+  // as we are not adding to trace.
+  auto adapter = [](RequestPrivateMemoryFootprintCallback callback,
+                    bool success, uint64_t,
+                    mojom::GlobalMemoryDumpPtr global_memory_dump) {
+    std::move(callback).Run(success, std::move(global_memory_dump));
   };
 
   QueuedRequest::Args args(
       base::trace_event::MemoryDumpType::SUMMARY_ONLY,
       base::trace_event::MemoryDumpLevelOfDetail::BACKGROUND, {},
-      false /* add_to_trace */, pid);
-  RequestGlobalMemoryDumpInternal(args, base::BindRepeating(adapter, callback));
+      false /* add_to_trace */, pid, /*memory_footprint_only=*/true);
+  RequestGlobalMemoryDumpInternal(args,
+                                  base::BindOnce(adapter, std::move(callback)));
 }
 
 void CoordinatorImpl::RequestGlobalMemoryDumpAndAppendToTrace(
     MemoryDumpType dump_type,
     MemoryDumpLevelOfDetail level_of_detail,
-    const RequestGlobalMemoryDumpAndAppendToTraceCallback& callback) {
+    RequestGlobalMemoryDumpAndAppendToTraceCallback callback) {
   // This merely strips out the |dump_ptr| argument.
-  auto adapter =
-      [](const RequestGlobalMemoryDumpAndAppendToTraceCallback& callback,
-         bool success, uint64_t dump_guid,
-         mojom::GlobalMemoryDumpPtr) { callback.Run(success, dump_guid); };
+  auto adapter = [](RequestGlobalMemoryDumpAndAppendToTraceCallback callback,
+                    bool success, uint64_t dump_guid,
+                    mojom::GlobalMemoryDumpPtr) {
+    std::move(callback).Run(success, dump_guid);
+  };
 
   QueuedRequest::Args args(dump_type, level_of_detail, {},
-                           true /* add_to_trace */, base::kNullProcessId);
-  RequestGlobalMemoryDumpInternal(args, base::BindRepeating(adapter, callback));
+                           true /* add_to_trace */, base::kNullProcessId,
+                           /*memory_footprint_only=*/false);
+  RequestGlobalMemoryDumpInternal(args,
+                                  base::BindOnce(adapter, std::move(callback)));
 }
 
 void CoordinatorImpl::RegisterHeapProfiler(
@@ -173,11 +200,11 @@ void CoordinatorImpl::RegisterHeapProfiler(
 
 void CoordinatorImpl::GetVmRegionsForHeapProfiler(
     const std::vector<base::ProcessId>& pids,
-    const GetVmRegionsForHeapProfilerCallback& callback) {
+    GetVmRegionsForHeapProfilerCallback callback) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   uint64_t dump_guid = ++next_dump_id_;
   std::unique_ptr<QueuedVmRegionRequest> request =
-      std::make_unique<QueuedVmRegionRequest>(dump_guid, callback);
+      std::make_unique<QueuedVmRegionRequest>(dump_guid, std::move(callback));
   in_progress_vm_region_requests_[dump_guid] = std::move(request);
 
   std::vector<QueuedRequestDispatcher::ClientInfo> clients;
@@ -189,8 +216,9 @@ void CoordinatorImpl::GetVmRegionsForHeapProfiler(
 
   QueuedVmRegionRequest* request_ptr =
       in_progress_vm_region_requests_[dump_guid].get();
-  auto os_callback = base::Bind(&CoordinatorImpl::OnOSMemoryDumpForVMRegions,
-                                base::Unretained(this), dump_guid);
+  auto os_callback =
+      base::BindRepeating(&CoordinatorImpl::OnOSMemoryDumpForVMRegions,
+                          weak_ptr_factory_.GetWeakPtr(), dump_guid);
   QueuedRequestDispatcher::SetUpAndDispatchVmRegionRequest(request_ptr, clients,
                                                            pids, os_callback);
   FinalizeVmRegionDumpIfAllManagersReplied(dump_guid);
@@ -202,8 +230,8 @@ void CoordinatorImpl::RegisterClientProcess(
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   mojom::ClientProcess* client_process = client_process_ptr.get();
   client_process_ptr.set_connection_error_handler(
-      base::Bind(&CoordinatorImpl::UnregisterClientProcess,
-                 base::Unretained(this), client_process));
+      base::BindOnce(&CoordinatorImpl::UnregisterClientProcess,
+                     weak_ptr_factory_.GetWeakPtr(), client_process));
   auto identity = GetClientIdentityForCurrentRequest();
   auto client_info = std::make_unique<ClientInfo>(
       std::move(identity), std::move(client_process_ptr), process_type);
@@ -251,7 +279,7 @@ void CoordinatorImpl::UnregisterClientProcess(
         FROM_HERE,
         base::BindOnce(
             &CoordinatorImpl::FinalizeVmRegionDumpIfAllManagersReplied,
-            base::Unretained(this), pair.second->dump_guid));
+            weak_ptr_factory_.GetWeakPtr(), pair.second->dump_guid));
   }
 
   size_t num_deleted = clients_.erase(client_process);
@@ -260,11 +288,12 @@ void CoordinatorImpl::UnregisterClientProcess(
 
 void CoordinatorImpl::RequestGlobalMemoryDumpInternal(
     const QueuedRequest::Args& args,
-    const RequestGlobalMemoryDumpInternalCallback& callback) {
+    RequestGlobalMemoryDumpInternalCallback callback) {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
 
-  UMA_HISTOGRAM_COUNTS_1000("Memory.Experimental.Debug.GlobalDumpQueueLength",
-                            queued_memory_dump_requests_.size());
+  UMA_HISTOGRAM_COUNTS_1000(
+      "Memory.Experimental.Debug.GlobalDumpQueueLength",
+      base::saturated_cast<int32_t>(queued_memory_dump_requests_.size()));
 
   bool another_dump_is_queued = !queued_memory_dump_requests_.empty();
 
@@ -272,8 +301,7 @@ void CoordinatorImpl::RequestGlobalMemoryDumpInternal(
   // another request in the queue with the same level of detail, there's no
   // point in enqueuing this request.
   if (another_dump_is_queued &&
-      (args.dump_type == MemoryDumpType::PERIODIC_INTERVAL ||
-       args.dump_type == MemoryDumpType::PEAK_MEMORY_USAGE)) {
+      args.dump_type == MemoryDumpType::PERIODIC_INTERVAL) {
     for (const auto& request : queued_memory_dump_requests_) {
       if (request.args.level_of_detail == args.level_of_detail) {
         VLOG(1) << "RequestGlobalMemoryDump("
@@ -283,14 +311,15 @@ void CoordinatorImpl::RequestGlobalMemoryDumpInternal(
                 << base::trace_event::MemoryDumpLevelOfDetailToString(
                        args.level_of_detail)
                 << ") is already in the queue";
-        callback.Run(false /* success */, 0 /* dump_guid */,
-                     nullptr /* global_memory_dump */);
+        std::move(callback).Run(false /* success */, 0 /* dump_guid */,
+                                nullptr /* global_memory_dump */);
         return;
       }
     }
   }
 
-  queued_memory_dump_requests_.emplace_back(args, ++next_dump_id_, callback);
+  queued_memory_dump_requests_.emplace_back(args, ++next_dump_id_,
+                                            std::move(callback));
 
   // If another dump is already in queued, this dump will automatically be
   // scheduled when the other dump finishes.
@@ -341,29 +370,34 @@ void CoordinatorImpl::PerformNextQueuedGlobalMemoryDump() {
   DCHECK_CALLED_ON_VALID_THREAD(thread_checker_);
   QueuedRequest* request = GetCurrentRequest();
 
+  if (request == nullptr)
+    return;
+
   std::vector<QueuedRequestDispatcher::ClientInfo> clients;
   for (const auto& kv : clients_) {
     auto client_identity = kv.second->identity;
     const base::ProcessId pid = GetProcessIdForClientIdentity(client_identity);
     if (pid == base::kNullProcessId) {
-      VLOG(1) << "Couldn't find a PID for client \"" << client_identity.name()
-              << "." << client_identity.instance() << "\"";
+      VLOG(1) << "Couldn't find a PID for client "
+              << client_identity.ToString();
       continue;
     }
     clients.emplace_back(kv.second->client.get(), pid, kv.second->process_type);
   }
 
-  auto chrome_callback = base::Bind(
-      &CoordinatorImpl::OnChromeMemoryDumpResponse, base::Unretained(this));
-  auto os_callback = base::Bind(&CoordinatorImpl::OnOSMemoryDumpResponse,
-                                base::Unretained(this), request->dump_guid);
+  auto chrome_callback =
+      base::Bind(&CoordinatorImpl::OnChromeMemoryDumpResponse,
+                 weak_ptr_factory_.GetWeakPtr());
+  auto os_callback =
+      base::Bind(&CoordinatorImpl::OnOSMemoryDumpResponse,
+                 weak_ptr_factory_.GetWeakPtr(), request->dump_guid);
   QueuedRequestDispatcher::SetUpAndDispatch(request, clients, chrome_callback,
                                             os_callback);
 
   base::SequencedTaskRunnerHandle::Get()->PostDelayedTask(
       FROM_HERE,
       base::BindOnce(&CoordinatorImpl::OnQueuedRequestTimedOut,
-                     base::Unretained(this), request->dump_guid),
+                     weak_ptr_factory_.GetWeakPtr(), request->dump_guid),
       client_process_timeout_);
 
   if (request->args.add_to_trace && heap_profiler_) {
@@ -379,13 +413,14 @@ void CoordinatorImpl::PerformNextQueuedGlobalMemoryDump() {
             .IsArgumentFilterEnabled();
     heap_profiler_->DumpProcessesForTracing(
         strip_path_from_mapped_files,
-            base::BindRepeating(&CoordinatorImpl::OnDumpProcessesForTracing,
-                           base::Unretained(this), request->dump_guid));
+        base::BindRepeating(&CoordinatorImpl::OnDumpProcessesForTracing,
+                            weak_ptr_factory_.GetWeakPtr(),
+                            request->dump_guid));
 
     base::SequencedTaskRunnerHandle::Get()->PostDelayedTask(
         FROM_HERE,
         base::BindOnce(&CoordinatorImpl::OnHeapDumpTimeOut,
-                       base::Unretained(this), request->dump_guid),
+                       weak_ptr_factory_.GetWeakPtr(), request->dump_guid),
         kHeapDumpTimeout);
   }
 
@@ -487,7 +522,7 @@ void CoordinatorImpl::FinalizeVmRegionDumpIfAllManagersReplied(
 
   QueuedRequestDispatcher::VmRegions results =
       QueuedRequestDispatcher::FinalizeVmRegionRequest(it->second.get());
-  it->second->callback.Run(std::move(results));
+  std::move(it->second->callback).Run(std::move(results));
   in_progress_vm_region_requests_.erase(it);
 }
 
@@ -517,12 +552,8 @@ void CoordinatorImpl::OnDumpProcessesForTracing(
 
     const char* char_buffer = static_cast<const char*>(mapping.get());
     std::string json(char_buffer, char_buffer + size);
-
-    const int kTraceEventNumArgs = 1;
-    const char* const kTraceEventArgNames[] = {"dumps"};
-    const unsigned char kTraceEventArgTypes[] = {TRACE_VALUE_TYPE_CONVERTABLE};
-    std::unique_ptr<base::trace_event::ConvertableToTraceFormat> wrapper(
-        new StringWrapper(std::move(json)));
+    base::trace_event::TraceArguments args(
+        "dumps", std::make_unique<StringWrapper>(std::move(json)));
 
     // Using the same id merges all of the heap dumps into a single detailed
     // dump node in the UI.
@@ -531,9 +562,7 @@ void CoordinatorImpl::OnDumpProcessesForTracing(
         base::trace_event::TraceLog::GetCategoryGroupEnabled(
             base::trace_event::MemoryDumpManager::kTraceCategory),
         "periodic_interval", trace_event_internal::kGlobalScope, dump_guid,
-        buffer_ptr->pid, kTraceEventNumArgs, kTraceEventArgNames,
-        kTraceEventArgTypes, nullptr /* arg_values */, &wrapper,
-        TRACE_EVENT_FLAG_HAS_ID);
+        buffer_ptr->pid, &args, TRACE_EVENT_FLAG_HAS_ID);
   }
 
   FinalizeGlobalMemoryDumpIfAllManagersReplied();
@@ -575,8 +604,8 @@ void CoordinatorImpl::FinalizeGlobalMemoryDumpIfAllManagersReplied() {
   if (!queued_memory_dump_requests_.empty()) {
     base::SequencedTaskRunnerHandle::Get()->PostTask(
         FROM_HERE,
-        base::Bind(&CoordinatorImpl::PerformNextQueuedGlobalMemoryDump,
-                   base::Unretained(this)));
+        base::BindOnce(&CoordinatorImpl::PerformNextQueuedGlobalMemoryDump,
+                       weak_ptr_factory_.GetWeakPtr()));
   }
 }
 
