@@ -8,6 +8,9 @@
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
 #include "base/test/trace_event_analyzer.h"
+#include "base/threading/platform_thread.h"
+#include "base/time/time.h"
+#include "base/time/time_override.h"
 #include "base/trace_event/memory_dump_request_args.h"
 #include "build/build_config.h"
 #include "mojo/public/cpp/bindings/binding.h"
@@ -80,7 +83,8 @@ class CoordinatorImplTest : public testing::Test {
   void RegisterClientProcess(mojom::ClientProcessPtr client_process,
                              base::ProcessId pid,
                              mojom::ProcessType process_type) {
-    auto identity = service_manager::Identity(std::to_string(pid));
+    service_manager::Identity identity(std::to_string(pid), base::Token{1, 1},
+                                       base::Token{}, base::Token{1, 1});
 
     ON_CALL(*coordinator_, GetClientIdentityForCurrentRequest())
         .WillByDefault(Return(identity));
@@ -94,7 +98,8 @@ class CoordinatorImplTest : public testing::Test {
 
   void RequestGlobalMemoryDump(RequestGlobalMemoryDumpCallback callback) {
     RequestGlobalMemoryDump(MemoryDumpType::SUMMARY_ONLY,
-                            MemoryDumpLevelOfDetail::BACKGROUND, {}, callback);
+                            MemoryDumpLevelOfDetail::BACKGROUND, {},
+                            std::move(callback));
   }
 
   void RequestGlobalMemoryDump(
@@ -102,27 +107,29 @@ class CoordinatorImplTest : public testing::Test {
       MemoryDumpLevelOfDetail level_of_detail,
       const std::vector<std::string>& allocator_dump_names,
       RequestGlobalMemoryDumpCallback callback) {
-    coordinator_->RequestGlobalMemoryDump(dump_type, level_of_detail,
-                                          allocator_dump_names, callback);
+    coordinator_->RequestGlobalMemoryDump(
+        dump_type, level_of_detail, allocator_dump_names, std::move(callback));
   }
 
   void RequestGlobalMemoryDumpForPid(
       base::ProcessId pid,
+      const std::vector<std::string>& allocator_dump_names,
       RequestGlobalMemoryDumpForPidCallback callback) {
-    coordinator_->RequestGlobalMemoryDumpForPid(pid, callback);
+    coordinator_->RequestGlobalMemoryDumpForPid(pid, allocator_dump_names,
+                                                std::move(callback));
   }
 
   void RequestGlobalMemoryDumpAndAppendToTrace(
       RequestGlobalMemoryDumpAndAppendToTraceCallback callback) {
     coordinator_->RequestGlobalMemoryDumpAndAppendToTrace(
         MemoryDumpType::EXPLICITLY_TRIGGERED, MemoryDumpLevelOfDetail::DETAILED,
-        callback);
+        std::move(callback));
   }
 
   void GetVmRegionsForHeapProfiler(
       const std::vector<base::ProcessId>& pids,
       GetVmRegionsForHeapProfilerCallback callback) {
-    coordinator_->GetVmRegionsForHeapProfiler(pids, callback);
+    coordinator_->GetVmRegionsForHeapProfiler(pids, std::move(callback));
   }
 
   void ReduceCoordinatorClientProcessTimeout() {
@@ -152,44 +159,51 @@ class MockClientProcess : public mojom::ClientProcess {
     test_coordinator->RegisterClientProcess(std::move(client_process), pid,
                                             process_type);
 
-    ON_CALL(*this, RequestChromeMemoryDump(_, _))
+    ON_CALL(*this, RequestChromeMemoryDumpMock(_, _))
         .WillByDefault(
             Invoke([pid](const MemoryDumpRequestArgs& args,
-                         const RequestChromeMemoryDumpCallback& callback) {
+                         RequestChromeMemoryDumpCallback& callback) {
               MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
-              auto pmd =
-                  std::make_unique<ProcessMemoryDump>(nullptr, dump_args);
+              auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
               auto* mad = pmd->CreateAllocatorDump(
                   "malloc", base::trace_event::MemoryAllocatorDumpGuid(pid));
               mad->AddScalar(MemoryAllocatorDump::kNameSize,
                              MemoryAllocatorDump::kUnitsBytes, 1024);
 
-              callback.Run(true, args.dump_guid, std::move(pmd));
+              std::move(callback).Run(true, args.dump_guid, std::move(pmd));
             }));
 
-    ON_CALL(*this, RequestOSMemoryDump(_, _, _))
-        .WillByDefault(Invoke([](bool want_mmaps,
+    ON_CALL(*this, RequestOSMemoryDumpMock(_, _, _))
+        .WillByDefault(Invoke([](mojom::MemoryMapOption,
                                  const std::vector<base::ProcessId> pids,
-                                 const RequestOSMemoryDumpCallback& callback) {
-          std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
-          callback.Run(true, std::move(results));
+                                 RequestOSMemoryDumpCallback& callback) {
+          base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          std::move(callback).Run(true, std::move(results));
         }));
   }
 
   ~MockClientProcess() override {}
 
-  MOCK_METHOD2(RequestChromeMemoryDump,
+  // TODO(crbug.com/729950): Remove non const reference here once GMock is
+  // updated to support move-only types.
+  MOCK_METHOD2(RequestChromeMemoryDumpMock,
                void(const MemoryDumpRequestArgs& args,
-                    const RequestChromeMemoryDumpCallback& callback));
-
-  MOCK_METHOD3(RequestOSMemoryDump,
-               void(bool want_mmaps,
+                    RequestChromeMemoryDumpCallback& callback));
+  MOCK_METHOD3(RequestOSMemoryDumpMock,
+               void(mojom::MemoryMapOption option,
                     const std::vector<base::ProcessId>& args,
-                    const RequestOSMemoryDumpCallback& callback));
+                    RequestOSMemoryDumpCallback& callback));
 
-  MOCK_METHOD2(EnableHeapProfiling,
-               void(base::trace_event::HeapProfilingMode mode,
-                    const EnableHeapProfilingCallback& callback));
+  void RequestChromeMemoryDump(
+      const MemoryDumpRequestArgs& args,
+      RequestChromeMemoryDumpCallback callback) override {
+    RequestChromeMemoryDumpMock(args, callback);
+  }
+  void RequestOSMemoryDump(mojom::MemoryMapOption option,
+                           const std::vector<base::ProcessId>& args,
+                           RequestOSMemoryDumpCallback callback) override {
+    RequestOSMemoryDumpMock(option, args, callback);
+  }
 
  private:
   mojo::Binding<mojom::ClientProcess> binding_;
@@ -227,12 +241,11 @@ class MockGlobalMemoryDumpAndAppendToTraceCallback {
 class MockGetVmRegionsForHeapProfilerCallback {
  public:
   MockGetVmRegionsForHeapProfilerCallback() = default;
-  MOCK_METHOD1(
-      OnCall,
-      void(const std::unordered_map<base::ProcessId,
-                                    std::vector<mojom::VmRegionPtr>>&));
+  MOCK_METHOD1(OnCall,
+               void(const base::flat_map<base::ProcessId,
+                                         std::vector<mojom::VmRegionPtr>>&));
 
-  void Run(std::unordered_map<base::ProcessId, std::vector<mojom::VmRegionPtr>>
+  void Run(base::flat_map<base::ProcessId, std::vector<mojom::VmRegionPtr>>
                results) {
     OnCall(results);
   }
@@ -263,7 +276,7 @@ mojom::RawOSMemDumpPtr FillRawOSDump(int pid) {
     raw_os_dump->memory_maps.push_back(std::move(vm_region));
   }
   return raw_os_dump;
-};
+}
 
 // Tests that the global dump is acked even in absence of clients.
 TEST_F(CoordinatorImplTest, NoClients) {
@@ -280,8 +293,8 @@ TEST_F(CoordinatorImplTest, SeveralClients) {
                                                mojom::ProcessType::BROWSER);
   NiceMock<MockClientProcess> client_process_2(this);
 
-  EXPECT_CALL(client_process_1, RequestChromeMemoryDump(_, _)).Times(1);
-  EXPECT_CALL(client_process_2, RequestChromeMemoryDump(_, _)).Times(1);
+  EXPECT_CALL(client_process_1, RequestChromeMemoryDumpMock(_, _)).Times(1);
+  EXPECT_CALL(client_process_2, RequestChromeMemoryDumpMock(_, _)).Times(1);
 
   MockGlobalMemoryDumpCallback callback;
   EXPECT_CALL(callback, OnCall(true, NotNull()))
@@ -290,20 +303,76 @@ TEST_F(CoordinatorImplTest, SeveralClients) {
   run_loop.Run();
 }
 
+// Issuing two requests will cause the second one to be queued.
+TEST_F(CoordinatorImplTest, QueuedRequest) {
+  base::RunLoop run_loop;
+
+  // Override TimeTicks::Now with a timer that has extra_time added.
+  // This variable to be static as the lambda below has to convert to a function
+  // pointer rather than a functor.
+  static base::TimeDelta extra_time;
+  base::subtle::ScopedTimeClockOverrides time_override(
+      nullptr,
+      []() {
+        return base::subtle::TimeTicksNowIgnoringOverride() + extra_time;
+      },
+      nullptr);
+
+  NiceMock<MockClientProcess> client_process_1(this, 1,
+                                               mojom::ProcessType::BROWSER);
+  NiceMock<MockClientProcess> client_process_2(this);
+
+  // Each request will invoke on both processes.
+  EXPECT_CALL(client_process_1, RequestChromeMemoryDumpMock(_, _)).Times(2);
+  EXPECT_CALL(client_process_2, RequestChromeMemoryDumpMock(_, _))
+      .Times(2)
+      .WillRepeatedly(Invoke(
+          [](const MemoryDumpRequestArgs& args,
+             MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
+            // Skip the wall clock time-ticks forward to make sure start_time
+            // is strictly increasing.
+            extra_time += base::TimeDelta::FromMilliseconds(10);
+            MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
+            auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
+            std::move(callback).Run(true, args.dump_guid, std::move(pmd));
+          }));
+
+  MockGlobalMemoryDumpCallback callback1;
+  MockGlobalMemoryDumpCallback callback2;
+
+  // Verify that the start time of subsequent dumps is monotonically
+  // increasing.
+  base::TimeTicks before = base::TimeTicks::Now();
+  base::TimeTicks first_dump_time;
+  EXPECT_CALL(callback1, OnCall(true, NotNull()))
+      .WillOnce(Invoke([&](bool success, GlobalMemoryDump* global_dump) {
+        EXPECT_LT(before, global_dump->start_time);
+        first_dump_time = global_dump->start_time;
+      }));
+  EXPECT_CALL(callback2, OnCall(true, NotNull()))
+      .WillOnce(Invoke([&](bool success, GlobalMemoryDump* global_dump) {
+        EXPECT_LT(before, global_dump->start_time);
+        EXPECT_LT(first_dump_time, global_dump->start_time);
+        run_loop.Quit();
+      }));
+  RequestGlobalMemoryDump(callback1.Get());
+  RequestGlobalMemoryDump(callback2.Get());
+  run_loop.Run();
+}
+
 TEST_F(CoordinatorImplTest, MissingChromeDump) {
   base::RunLoop run_loop;
 
   NiceMock<MockClientProcess> client_process(this, 1,
                                              mojom::ProcessType::BROWSER);
 
-  EXPECT_CALL(client_process, RequestChromeMemoryDump(_, _))
-      .WillOnce(
-          Invoke([](const MemoryDumpRequestArgs& args,
-                    const MockClientProcess::RequestChromeMemoryDumpCallback&
-                        callback) {
+  EXPECT_CALL(client_process, RequestChromeMemoryDumpMock(_, _))
+      .WillOnce(Invoke(
+          [](const MemoryDumpRequestArgs& args,
+             MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
-            auto pmd = std::make_unique<ProcessMemoryDump>(nullptr, dump_args);
-            callback.Run(true, args.dump_guid, std::move(pmd));
+            auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
+            std::move(callback).Run(true, args.dump_guid, std::move(pmd));
           }));
 
   MockGlobalMemoryDumpCallback callback;
@@ -322,12 +391,12 @@ TEST_F(CoordinatorImplTest, MissingOsDump) {
   NiceMock<MockClientProcess> client_process(this, 1,
                                              mojom::ProcessType::BROWSER);
 
-  EXPECT_CALL(client_process, RequestOSMemoryDump(_, _, _))
+  EXPECT_CALL(client_process, RequestOSMemoryDumpMock(_, _, _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
-            callback.Run(true, std::move(results));
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+            std::move(callback).Run(true, std::move(results));
           }));
 
   MockGlobalMemoryDumpCallback callback;
@@ -351,12 +420,13 @@ TEST_F(CoordinatorImplTest, TimeOutStuckChild) {
 
   // Store a reference to the callback passed to RequestChromeMemoryDump
   // to emulate "stuck" behaviour.
-  EXPECT_CALL(client_process, RequestChromeMemoryDump(_, _))
-      .WillOnce(
-          Invoke([&stuck_callback](
-                     const MemoryDumpRequestArgs&,
-                     const MockClientProcess::RequestChromeMemoryDumpCallback&
-                         callback) { stuck_callback = callback; }));
+  EXPECT_CALL(client_process, RequestChromeMemoryDumpMock(_, _))
+      .WillOnce(Invoke(
+          [&stuck_callback](
+              const MemoryDumpRequestArgs&,
+              MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
+            stuck_callback = std::move(callback);
+          }));
 
   MockGlobalMemoryDumpCallback callback;
   EXPECT_CALL(
@@ -388,44 +458,45 @@ TEST_F(CoordinatorImplTest, TimeOutStuckChildMultiProcess) {
 // platforms, they are expected to come from each individual client.
 #if defined(OS_LINUX)
   EXPECT_CALL(browser_client,
-              RequestOSMemoryDump(
+              RequestOSMemoryDumpMock(
                   _, AllOf(Contains(kBrowserPid), Contains(kRendererPid)), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[kBrowserPid] = FillRawOSDump(kBrowserPid);
             results[kRendererPid] = FillRawOSDump(kRendererPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(renderer_client, RequestOSMemoryDump(_, _, _)).Times(0);
+  EXPECT_CALL(renderer_client, RequestOSMemoryDumpMock(_, _, _)).Times(0);
 #else
-  EXPECT_CALL(browser_client, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(browser_client, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kBrowserPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(renderer_client, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(renderer_client, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kRendererPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
 #endif  // defined(OS_LINUX)
 
   // Make the browser respond correctly but pretend the renderer is "stuck"
   // by storing a callback.
-  EXPECT_CALL(renderer_client, RequestChromeMemoryDump(_, _))
-      .WillOnce(
-          Invoke([&stuck_callback](
-                     const MemoryDumpRequestArgs&,
-                     const MockClientProcess::RequestChromeMemoryDumpCallback&
-                         callback) { stuck_callback = callback; }));
+  EXPECT_CALL(renderer_client, RequestChromeMemoryDumpMock(_, _))
+      .WillOnce(Invoke(
+          [&stuck_callback](
+              const MemoryDumpRequestArgs&,
+              MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
+            stuck_callback = std::move(callback);
+          }));
 
   MockGlobalMemoryDumpCallback callback;
   EXPECT_CALL(callback, OnCall(false, _))
@@ -454,23 +525,21 @@ TEST_F(CoordinatorImplTest, ClientCrashDuringGlobalDump) {
   // dump attempt has failed.
 
   // Whichever client is called first destroys the other client.
-  ON_CALL(*client_process_1, RequestChromeMemoryDump(_, _))
-      .WillByDefault(
-          Invoke([&client_process_2](
-                     const MemoryDumpRequestArgs& args,
-                     const MockClientProcess::RequestChromeMemoryDumpCallback&
-                         callback) {
+  ON_CALL(*client_process_1, RequestChromeMemoryDumpMock(_, _))
+      .WillByDefault(Invoke(
+          [&client_process_2](
+              const MemoryDumpRequestArgs& args,
+              MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             client_process_2.reset();
-            callback.Run(true, args.dump_guid, nullptr);
+            std::move(callback).Run(true, args.dump_guid, nullptr);
           }));
-  ON_CALL(*client_process_2, RequestChromeMemoryDump(_, _))
-      .WillByDefault(
-          Invoke([&client_process_1](
-                     const MemoryDumpRequestArgs& args,
-                     const MockClientProcess::RequestChromeMemoryDumpCallback&
-                         callback) {
+  ON_CALL(*client_process_2, RequestChromeMemoryDumpMock(_, _))
+      .WillByDefault(Invoke(
+          [&client_process_1](
+              const MemoryDumpRequestArgs& args,
+              MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             client_process_1.reset();
-            callback.Run(true, args.dump_guid, nullptr);
+            std::move(callback).Run(true, args.dump_guid, nullptr);
           }));
 
   MockGlobalMemoryDumpCallback callback;
@@ -488,16 +557,15 @@ TEST_F(CoordinatorImplTest, SingleClientCrashDuringGlobalDump) {
   auto client_process = std::make_unique<NiceMock<MockClientProcess>>(
       this, 1, mojom::ProcessType::BROWSER);
 
-  ON_CALL(*client_process, RequestChromeMemoryDump(_, _))
-      .WillByDefault(
-          Invoke([&client_process](
-                     const MemoryDumpRequestArgs& args,
-                     const MockClientProcess::RequestChromeMemoryDumpCallback&
-                         callback) {
+  ON_CALL(*client_process, RequestChromeMemoryDumpMock(_, _))
+      .WillByDefault(Invoke(
+          [&client_process](
+              const MemoryDumpRequestArgs& args,
+              MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             // The dtor here will cause mojo to post an UnregisterClient call to
             // the coordinator.
             client_process.reset();
-            callback.Run(true, args.dump_guid, nullptr);
+            std::move(callback).Run(true, args.dump_guid, nullptr);
           }));
 
   MockGlobalMemoryDumpCallback callback;
@@ -513,12 +581,12 @@ TEST_F(CoordinatorImplTest, GlobalMemoryDumpStruct) {
   MockClientProcess browser_client(this, 1, mojom::ProcessType::BROWSER);
   MockClientProcess renderer_client(this, 2, mojom::ProcessType::RENDERER);
 
-  EXPECT_CALL(browser_client, RequestChromeMemoryDump(_, _))
+  EXPECT_CALL(browser_client, RequestChromeMemoryDumpMock(_, _))
       .WillOnce(Invoke([](const MemoryDumpRequestArgs& args,
-                          const MockClientProcess::
-                              RequestChromeMemoryDumpCallback& callback) {
+                          MockClientProcess::RequestChromeMemoryDumpCallback&
+                              callback) {
         MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
-        auto pmd = std::make_unique<ProcessMemoryDump>(nullptr, dump_args);
+        auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
         auto* size = MemoryAllocatorDump::kNameSize;
         auto* bytes = MemoryAllocatorDump::kUnitsBytes;
         const uint32_t kB = 1024;
@@ -551,28 +619,27 @@ TEST_F(CoordinatorImplTest, GlobalMemoryDumpStruct) {
         pmd->CreateAllocatorDump("partition_alloc/partitions/not_ignored_2")
             ->AddScalar(size, bytes, 2 * kB);
 
-        callback.Run(true, args.dump_guid, std::move(pmd));
+        std::move(callback).Run(true, args.dump_guid, std::move(pmd));
       }));
-  EXPECT_CALL(renderer_client, RequestChromeMemoryDump(_, _))
-      .WillOnce(
-          Invoke([](const MemoryDumpRequestArgs& args,
-                    const MockClientProcess::RequestChromeMemoryDumpCallback&
-                        callback) {
+  EXPECT_CALL(renderer_client, RequestChromeMemoryDumpMock(_, _))
+      .WillOnce(Invoke(
+          [](const MemoryDumpRequestArgs& args,
+             MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
-            auto pmd = std::make_unique<ProcessMemoryDump>(nullptr, dump_args);
+            auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
             auto* mad = pmd->CreateAllocatorDump(
                 "malloc", base::trace_event::MemoryAllocatorDumpGuid(2));
             mad->AddScalar(MemoryAllocatorDump::kNameSize,
                            MemoryAllocatorDump::kUnitsBytes, 1024 * 2);
-            callback.Run(true, args.dump_guid, std::move(pmd));
+            std::move(callback).Run(true, args.dump_guid, std::move(pmd));
           }));
 #if defined(OS_LINUX)
   EXPECT_CALL(browser_client,
-              RequestOSMemoryDump(_, AllOf(Contains(1), Contains(2)), _))
+              RequestOSMemoryDumpMock(_, AllOf(Contains(1), Contains(2)), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[1] = mojom::RawOSMemDump::New();
             results[1]->resident_set_kb = 1;
             results[1]->platform_private_footprint =
@@ -581,31 +648,31 @@ TEST_F(CoordinatorImplTest, GlobalMemoryDumpStruct) {
             results[2]->platform_private_footprint =
                 mojom::PlatformPrivateFootprint::New();
             results[2]->resident_set_kb = 2;
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(renderer_client, RequestOSMemoryDump(_, _, _)).Times(0);
+  EXPECT_CALL(renderer_client, RequestOSMemoryDumpMock(_, _, _)).Times(0);
 #else
-  EXPECT_CALL(browser_client, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(browser_client, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = mojom::RawOSMemDump::New();
             results[0]->platform_private_footprint =
                 mojom::PlatformPrivateFootprint::New();
             results[0]->resident_set_kb = 1;
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(renderer_client, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(renderer_client, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = mojom::RawOSMemDump::New();
             results[0]->platform_private_footprint =
                 mojom::PlatformPrivateFootprint::New();
             results[0]->resident_set_kb = 2;
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
 #endif  // defined(OS_LINUX)
 
@@ -624,24 +691,9 @@ TEST_F(CoordinatorImplTest, GlobalMemoryDumpStruct) {
             renderer_dump = std::move(dump);
           }
         }
-        // For malloc we only count the root "malloc" not children "malloc/*".
-        EXPECT_EQ(1u, browser_dump->chrome_dump->malloc_total_kb);
-
-        // For blink_gc we only count the root "blink_gc" not children
-        // "blink_gc/*".
-        EXPECT_EQ(2u, browser_dump->chrome_dump->blink_gc_total_kb);
-
-        // For v8 we count the children ("v8/*") as the root total is not given.
-        EXPECT_EQ(3u, browser_dump->chrome_dump->v8_total_kb);
-
-        // partition_alloc has partition_alloc/allocated_objects/* which is a
-        // subset of partition_alloc/partitions/* so we only count the latter.
-        EXPECT_EQ(4u, browser_dump->chrome_dump->partition_alloc_total_kb);
 
         EXPECT_EQ(browser_dump->os_dump->resident_set_kb, 1u);
         EXPECT_EQ(renderer_dump->os_dump->resident_set_kb, 2u);
-        EXPECT_EQ(browser_dump->chrome_dump->malloc_total_kb, 1u);
-        EXPECT_EQ(renderer_dump->chrome_dump->malloc_total_kb, 2u);
         run_loop.Quit();
       }));
 
@@ -667,35 +719,34 @@ TEST_F(CoordinatorImplTest, VmRegionsForHeapProfiler) {
 // On Linux, all memory dumps come from the browser client. On all other
 // platforms, they are expected to come from each individual client.
 #if defined(OS_LINUX)
-  EXPECT_CALL(
-      browser_client,
-      RequestOSMemoryDump(
-          true, AllOf(Contains(kBrowserPid), Contains(kRendererPid)), _))
+  EXPECT_CALL(browser_client,
+              RequestOSMemoryDumpMock(
+                  _, AllOf(Contains(kBrowserPid), Contains(kRendererPid)), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[kBrowserPid] = FillRawOSDump(kBrowserPid);
             results[kRendererPid] = FillRawOSDump(kRendererPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(renderer_client, RequestOSMemoryDump(_, _, _)).Times(0);
+  EXPECT_CALL(renderer_client, RequestOSMemoryDumpMock(_, _, _)).Times(0);
 #else
-  EXPECT_CALL(browser_client, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(browser_client, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kBrowserPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(renderer_client, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(renderer_client, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kRendererPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
 #endif  // defined(OS_LINUX)
 
@@ -703,8 +754,8 @@ TEST_F(CoordinatorImplTest, VmRegionsForHeapProfiler) {
   EXPECT_CALL(callback, OnCall(_))
       .WillOnce(Invoke(
           [&run_loop](
-              const std::unordered_map<
-                  base::ProcessId, std::vector<mojom::VmRegionPtr>>& results) {
+              const base::flat_map<base::ProcessId,
+                                   std::vector<mojom::VmRegionPtr>>& results) {
             ASSERT_EQ(2U, results.size());
 
             auto browser_it = results.find(kBrowserPid);
@@ -747,14 +798,13 @@ TEST_F(CoordinatorImplTest, DumpsArentAddedToTraceUnlessRequested) {
   NiceMock<MockClientProcess> client_process(this, 1,
                                              mojom::ProcessType::BROWSER);
 
-  EXPECT_CALL(client_process, RequestChromeMemoryDump(_, _))
-      .WillOnce(
-          Invoke([](const MemoryDumpRequestArgs& args,
-                    const MockClientProcess::RequestChromeMemoryDumpCallback&
-                        callback) {
+  EXPECT_CALL(client_process, RequestChromeMemoryDumpMock(_, _))
+      .WillOnce(Invoke(
+          [](const MemoryDumpRequestArgs& args,
+             MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
-            auto pmd = std::make_unique<ProcessMemoryDump>(nullptr, dump_args);
-            callback.Run(true, args.dump_guid, std::move(pmd));
+            auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
+            std::move(callback).Run(true, args.dump_guid, std::move(pmd));
           }));
 
   MockGlobalMemoryDumpCallback callback;
@@ -786,14 +836,13 @@ TEST_F(CoordinatorImplTest, DumpsAreAddedToTraceWhenRequested) {
 
   NiceMock<MockClientProcess> client_process(this, 1,
                                              mojom::ProcessType::BROWSER);
-  EXPECT_CALL(client_process, RequestChromeMemoryDump(_, _))
-      .WillOnce(
-          Invoke([](const MemoryDumpRequestArgs& args,
-                    const MockClientProcess::RequestChromeMemoryDumpCallback&
-                        callback) {
+  EXPECT_CALL(client_process, RequestChromeMemoryDumpMock(_, _))
+      .WillOnce(Invoke(
+          [](const MemoryDumpRequestArgs& args,
+             MockClientProcess::RequestChromeMemoryDumpCallback& callback) {
             MemoryDumpArgs dump_args{MemoryDumpLevelOfDetail::DETAILED};
-            auto pmd = std::make_unique<ProcessMemoryDump>(nullptr, dump_args);
-            callback.Run(true, args.dump_guid, std::move(pmd));
+            auto pmd = std::make_unique<ProcessMemoryDump>(dump_args);
+            std::move(callback).Run(true, args.dump_guid, std::move(pmd));
           }));
 
   MockGlobalMemoryDumpAndAppendToTraceCallback callback;
@@ -832,52 +881,52 @@ TEST_F(CoordinatorImplTest, DumpByPidSuccess) {
 // On Linux, all memory dumps come from the browser client. On all other
 // platforms, they are expected to come from each individual client.
 #if defined(OS_LINUX)
-  EXPECT_CALL(client_process_1, RequestOSMemoryDump(_, _, _))
+  EXPECT_CALL(client_process_1, RequestOSMemoryDumpMock(_, _, _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[kBrowserPid] = FillRawOSDump(kBrowserPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[kRendererPid] = FillRawOSDump(kRendererPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[kGpuPid] = FillRawOSDump(kGpuPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
 #else
-  EXPECT_CALL(client_process_1, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(client_process_1, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kBrowserPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(client_process_2, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(client_process_2, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kRendererPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
-  EXPECT_CALL(client_process_3, RequestOSMemoryDump(_, Contains(0), _))
+  EXPECT_CALL(client_process_3, RequestOSMemoryDumpMock(_, Contains(0), _))
       .WillOnce(Invoke(
-          [](bool want_mmaps, const std::vector<base::ProcessId>& pids,
-             const MockClientProcess::RequestOSMemoryDumpCallback& callback) {
-            std::unordered_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
+          [](mojom::MemoryMapOption, const std::vector<base::ProcessId>& pids,
+             MockClientProcess::RequestOSMemoryDumpCallback& callback) {
+            base::flat_map<base::ProcessId, mojom::RawOSMemDumpPtr> results;
             results[0] = FillRawOSDump(kGpuPid);
-            callback.Run(true, std::move(results));
+            std::move(callback).Run(true, std::move(results));
           }));
 #endif  // defined(OS_LINUX)
 
@@ -900,9 +949,9 @@ TEST_F(CoordinatorImplTest, DumpByPidSuccess) {
             run_loop.Quit();
           }));
 
-  RequestGlobalMemoryDumpForPid(kBrowserPid, callback.Get());
-  RequestGlobalMemoryDumpForPid(kRendererPid, callback.Get());
-  RequestGlobalMemoryDumpForPid(kGpuPid, callback.Get());
+  RequestGlobalMemoryDumpForPid(kBrowserPid, {}, callback.Get());
+  RequestGlobalMemoryDumpForPid(kRendererPid, {}, callback.Get());
+  RequestGlobalMemoryDumpForPid(kGpuPid, {}, callback.Get());
   run_loop.Run();
 }
 
@@ -916,7 +965,7 @@ TEST_F(CoordinatorImplTest, DumpByPidFailure) {
   EXPECT_CALL(callback, OnCall(false, nullptr))
       .WillOnce(RunClosure(run_loop.QuitClosure()));
 
-  RequestGlobalMemoryDumpForPid(2, callback.Get());
+  RequestGlobalMemoryDumpForPid(2, {}, callback.Get());
   run_loop.Run();
 }
 
