@@ -9,16 +9,24 @@
 namespace service_manager {
 namespace features {
 
+// Enables audio service sandbox.
+// (Only causes an effect when feature kAudioServiceOutOfProcess is enabled.)
+const base::Feature kAudioServiceSandbox{"AudioServiceSandbox",
+                                         base::FEATURE_DISABLED_BY_DEFAULT};
+
+// Enables network service sandbox.
+// (Only causes an effect when feature kNetworkService is enabled.)
+const base::Feature kNetworkServiceSandbox{"NetworkServiceSandbox",
+                                           base::FEATURE_DISABLED_BY_DEFAULT};
+
 #if defined(OS_WIN)
 // Emergency "off switch" for new Windows sandbox security mitigation,
 // sandbox::MITIGATION_EXTENSION_POINT_DISABLE.
 const base::Feature kWinSboxDisableExtensionPoints{
     "WinSboxDisableExtensionPoint", base::FEATURE_ENABLED_BY_DEFAULT};
 
-// Emergency "off switch" for new Windows sandbox security mitigation,
-// sandbox::MITIGATION_FORCE_MS_SIGNED_BINS.
-const base::Feature kWinSboxForceMsSigned{"WinSboxForceMsSigned",
-                                          base::FEATURE_ENABLED_BY_DEFAULT};
+// Controls whether the isolated XR service is sandboxed.
+const base::Feature kXRSandbox{"XRSandbox", base::FEATURE_DISABLED_BY_DEFAULT};
 #endif  // defined(OS_WIN)
 
 }  // namespace features
