@@ -26,13 +26,13 @@
 #include "base/files/scoped_file.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/ptr_util.h"
 #include "base/memory/singleton.h"
 #include "base/posix/eintr_wrapper.h"
 #include "base/strings/string_number_conversions.h"
-#include "base/sys_info.h"
+#include "base/system/sys_info.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
+#include "sandbox/constants.h"
 #include "sandbox/linux/services/credentials.h"
 #include "sandbox/linux/services/namespace_sandbox.h"
 #include "sandbox/linux/services/proc_util.h"
@@ -42,7 +42,7 @@
 #include "sandbox/linux/suid/client/setuid_sandbox_client.h"
 #include "sandbox/linux/syscall_broker/broker_command.h"
 #include "sandbox/linux/syscall_broker/broker_process.h"
-#include "sandbox/sandbox_features.h"
+#include "sandbox/sandbox_buildflags.h"
 #include "services/service_manager/sandbox/linux/bpf_broker_policy_linux.h"
 #include "services/service_manager/sandbox/linux/sandbox_seccomp_bpf_linux.h"
 #include "services/service_manager/sandbox/sandbox.h"
@@ -107,10 +107,15 @@ bool UpdateProcessTypeAndEnableSandbox(
   base::CommandLine::ForCurrentProcess()->InitFromArgv(exec);
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
-  command_line->AppendSwitchASCII(
-      switches::kProcessType,
-      command_line->GetSwitchValueASCII(switches::kProcessType)
-          .append("-broker"));
+  std::string new_process_type =
+      command_line->GetSwitchValueASCII(switches::kProcessType);
+  if (!new_process_type.empty()) {
+    new_process_type.append("-broker");
+  } else {
+    new_process_type = "broker";
+  }
+
+  command_line->AppendSwitchASCII(switches::kProcessType, new_process_type);
 
   if (broker_side_hook)
     CHECK(std::move(broker_side_hook).Run(options));
@@ -192,26 +197,11 @@ void SandboxLinux::PreinitializeSandbox() {
 }
 
 void SandboxLinux::EngageNamespaceSandbox(bool from_zygote) {
-  CHECK(pre_initialized_);
-  if (from_zygote) {
-    // Check being in a new PID namespace created by the namespace sandbox and
-    // being the init process.
-    CHECK(sandbox::NamespaceSandbox::InNewPidNamespace());
-    const pid_t pid = getpid();
-    CHECK_EQ(1, pid);
-  }
+  CHECK(EngageNamespaceSandboxInternal(from_zygote));
+}
 
-  CHECK(sandbox::Credentials::MoveToNewUserNS());
-
-  // Note: this requires SealSandbox() to be called later in this process to be
-  // safe, as this class is keeping a file descriptor to /proc/.
-  CHECK(sandbox::Credentials::DropFileSystemAccess(proc_fd_));
-
-  // We do not drop CAP_SYS_ADMIN because we need it to place each child process
-  // in its own PID namespace later on.
-  std::vector<sandbox::Credentials::Capability> caps;
-  caps.push_back(sandbox::Credentials::Capability::SYS_ADMIN);
-  CHECK(sandbox::Credentials::SetCapabilities(proc_fd_, caps));
+bool SandboxLinux::EngageNamespaceSandboxIfPossible() {
+  return EngageNamespaceSandboxInternal(false /* from_zygote */);
 }
 
 std::vector<int> SandboxLinux::GetFileDescriptorsToClose() {
@@ -389,12 +379,19 @@ bool SandboxLinux::InitializeSandbox(SandboxType sandbox_type,
   if (options.engage_namespace_sandbox)
     EngageNamespaceSandbox(false /* from_zygote */);
 
-  DCHECK(!HasOpenDirectories())
+  CHECK(!HasOpenDirectories())
       << "InitializeSandbox() called after unexpected directories have been "
       << "opened. This breaks the security of the setuid sandbox.";
 
   // Attempt to limit the future size of the address space of the process.
-  LimitAddressSpace(process_type, options);
+  int error = 0;
+  const bool limited_as = LimitAddressSpace(&error);
+  if (error) {
+    // Restore errno. Internally to |LimitAddressSpace|, the errno due to
+    // setrlimit may be lost.
+    errno = error;
+    PCHECK(limited_as);
+  }
 
   return StartSeccompBPF(sandbox_type, std::move(hook), options);
 }
@@ -414,69 +411,27 @@ bool SandboxLinux::seccomp_bpf_with_tsync_supported() const {
   return seccomp_bpf_with_tsync_supported_;
 }
 
-bool SandboxLinux::LimitAddressSpace(const std::string& process_type,
-                                     const Options& options) {
+bool SandboxLinux::LimitAddressSpace(int* error) {
 #if !defined(ADDRESS_SANITIZER) && !defined(MEMORY_SANITIZER) && \
     !defined(THREAD_SANITIZER) && !defined(LEAK_SANITIZER)
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
   if (SandboxTypeFromCommandLine(*command_line) == SANDBOX_TYPE_NO_SANDBOX) {
     return false;
   }
-  // Limit the address space to 4GB.
-  // This is in the hope of making some kernel exploits more complex and less
-  // reliable. It also limits sprays a little on 64 bits.
-  rlim_t address_space_limit = std::numeric_limits<uint32_t>::max();
-  rlim_t address_space_limit_max = std::numeric_limits<uint32_t>::max();
 
-  if (sizeof(rlim_t) == 8) {
-    // On 64 bits, V8 and possibly others will reserve massive memory ranges and
-    // rely on on-demand paging for allocation.  Unfortunately, even
-    // MADV_DONTNEED ranges count towards RLIMIT_AS so this is not an option.
-    // See crbug.com/169327 for a discussion.
-    // On the GPU process, irrespective of V8, we can exhaust a 4GB address
-    // space under normal usage, see crbug.com/271119.
-    // For now, increase limit to 16GB for renderer, worker, and GPU processes
-    // to accomodate.
-    if (process_type == switches::kRendererProcess ||
-        process_type == switches::kGpuProcess) {
-      address_space_limit = 1ULL << 34;
-      if (options.has_wasm_trap_handler) {
-        // WebAssembly memory objects use a large amount of address space when
-        // trap-based bounds checks are enabled. To accomodate this, we allow
-        // the address space limit to adjust dynamically up to a certain limit.
-        // The limit is currently 4TiB, which should allow enough address space
-        // for any reasonable page. See https://crbug.com/750378.
-        address_space_limit_max = 1ULL << 42;
-      } else {
-        // If we are not using trap-based bounds checks, there's no reason to
-        // allow the address space limit to grow.
-        address_space_limit_max = address_space_limit;
-      }
-    }
-  }
+  // Unfortunately, it does not appear possible to set RLIMIT_AS such that it
+  // will both (a) be high enough to support V8's and WebAssembly's address
+  // space requirements while also (b) being low enough to mitigate exploits
+  // using integer overflows that require large allocations, heap spray, or
+  // other memory-hungry attack modes.
 
-  // By default, add a limit to the VmData memory area that would prevent
-  // allocations that can't be index by an int.
-  rlim_t new_data_segment_max_size = std::numeric_limits<int>::max();
-
-  if (sizeof(rlim_t) == 8) {
-    // On 64 bits, increase the RLIMIT_DATA limit to 8GB.
-    // RLIMIT_DATA did not account for mmap()-ed memory until
-    // https://github.com/torvalds/linux/commit/84638335900f1995495838fe1bd4870c43ec1f6.
-    // When Chrome runs on devices with this patch, it will OOM very easily.
-    // See https://crbug.com/752185.
-    new_data_segment_max_size = 1ULL << 33;
-  }
-
-  bool limited_as = sandbox::ResourceLimits::LowerSoftAndHardLimits(
-      RLIMIT_AS, address_space_limit, address_space_limit_max);
-  bool limited_data =
-      sandbox::ResourceLimits::Lower(RLIMIT_DATA, new_data_segment_max_size);
+  *error = sandbox::ResourceLimits::Lower(
+      RLIMIT_DATA, static_cast<rlim_t>(sandbox::kDataSizeLimit));
 
   // Cache the resource limit before turning on the sandbox.
   base::SysInfo::AmountOfVirtualMemory();
 
-  return limited_as && limited_data;
+  return *error == 0;
 #else
   base::SysInfo::AmountOfVirtualMemory();
   return false;
@@ -532,4 +487,35 @@ void SandboxLinux::StopThreadAndEnsureNotCounted(base::Thread* thread) const {
       sandbox::ThreadHelpers::StopThreadAndWatchProcFS(proc_fd.get(), thread));
 }
 
+bool SandboxLinux::EngageNamespaceSandboxInternal(bool from_zygote) {
+  CHECK(pre_initialized_);
+  if (from_zygote) {
+    // Check being in a new PID namespace created by the namespace sandbox and
+    // being the init process.
+    CHECK(sandbox::NamespaceSandbox::InNewPidNamespace());
+    const pid_t pid = getpid();
+    CHECK_EQ(1, pid);
+  }
+
+  // After we successfully move to a new user ns, we don't allow this function
+  // to fail.
+  if (!sandbox::Credentials::MoveToNewUserNS()) {
+    return false;
+  }
+
+  // Note: this requires SealSandbox() to be called later in this process to be
+  // safe, as this class is keeping a file descriptor to /proc/.
+  CHECK(sandbox::Credentials::DropFileSystemAccess(proc_fd_));
+
+  // Now we drop all capabilities that we can. In the zygote process, we need
+  // to keep CAP_SYS_ADMIN, to place each child in its own PID namespace
+  // later on.
+  std::vector<sandbox::Credentials::Capability> caps;
+  if (from_zygote) {
+    caps.push_back(sandbox::Credentials::Capability::SYS_ADMIN);
+  }
+  CHECK(sandbox::Credentials::SetCapabilities(proc_fd_, caps));
+  return true;
+}
+
 }  // namespace service_manager
