@@ -8,20 +8,19 @@
 #include "mojo/public/cpp/bindings/binding_set.h"
 #include "services/service_manager/public/cpp/binder_registry.h"
 #include "services/service_manager/public/cpp/service.h"
+#include "services/service_manager/public/cpp/service_binding.h"
+#include "services/service_manager/public/mojom/service.mojom.h"
 #include "services/test/echo/public/mojom/echo.mojom.h"
 
 namespace echo {
 
-std::unique_ptr<service_manager::Service> CreateEchoService();
-
 class EchoService : public service_manager::Service, public mojom::Echo {
  public:
-  EchoService();
+  explicit EchoService(service_manager::mojom::ServiceRequest request);
   ~EchoService() override;
 
  private:
   // service_manager::Service:
-  void OnStart() override;
   void OnBindInterface(const service_manager::BindSourceInfo& source_info,
                        const std::string& interface_name,
                        mojo::ScopedMessagePipeHandle interface_pipe) override;
@@ -33,6 +32,7 @@ class EchoService : public service_manager::Service, public mojom::Echo {
 
   void BindEchoRequest(mojom::EchoRequest request);
 
+  service_manager::ServiceBinding service_binding_;
   service_manager::BinderRegistry registry_;
   mojo::BindingSet<mojom::Echo> bindings_;
 
