@@ -19,13 +19,12 @@
 #include "base/logging.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
-#include "base/task_scheduler/post_task.h"
-#include "base/task_scheduler/task_traits.h"
+#include "base/task/post_task.h"
+#include "base/task/task_traits.h"
 #include "base/threading/thread_task_runner_handle.h"
 #include "base/time/time.h"
-#include "base/trace_event/trace_config.h"
 #include "base/trace_event/trace_event.h"
-#include "mojo/common/data_pipe_utils.h"
+#include "mojo/public/cpp/system/data_pipe_utils.h"
 #include "services/service_manager/public/cpp/bind_source_info.h"
 #include "services/service_manager/public/cpp/service_context_ref.h"
 #include "services/tracing/agent_registry.h"
@@ -37,13 +36,7 @@ namespace {
 
 const char kMetadataTraceLabel[] = "metadata";
 
-const char kGetCategoriesClosureName[] = "GetCategoriesClosure";
 const char kRequestBufferUsageClosureName[] = "RequestBufferUsageClosure";
-const char kRequestClockSyncMarkerClosureName[] =
-    "RequestClockSyncMarkerClosure";
-const char kStartTracingClosureName[] = "StartTracingClosure";
-
-tracing::Coordinator* g_coordinator = nullptr;
 
 }  // namespace
 
@@ -65,22 +58,25 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
         stream_is_empty_(true),
         json_field_name_written_(false) {}
 
-  // Destroyed on |background_task_runner_|.
+  // Destroyed on |backend_task_runner_|.
   ~TraceStreamer() = default;
 
-  // Called from |background_task_runner_|.
+  // Called from |backend_task_runner_|.
   void CreateAndSendRecorder(
       const std::string& label,
       mojom::TraceDataType type,
       base::WeakPtr<AgentRegistry::AgentEntry> agent_entry) {
     mojom::RecorderPtr ptr;
-    auto recorder = std::make_unique<Recorder>(
-        MakeRequest(&ptr), type,
-        base::BindRepeating(&Coordinator::TraceStreamer::OnRecorderDataChange,
-                            AsWeakPtr(), label));
-    recorders_[label].insert(std::move(recorder));
-    DCHECK(type != mojom::TraceDataType::STRING ||
-           recorders_[label].size() == 1);
+    mojom::RecorderRequest request = MakeRequest(&ptr);
+    if (processed_labels_.count(label) == 0) {
+      auto recorder = std::make_unique<Recorder>(
+          std::move(request), type,
+          base::BindRepeating(&Coordinator::TraceStreamer::OnRecorderDataChange,
+                              AsWeakPtr(), label));
+      recorders_[label].insert(std::move(recorder));
+      DCHECK(type != mojom::TraceDataType::STRING ||
+             recorders_[label].size() == 1);
+    }
 
     // Tracing agent proxies are bound on the main thread and should be called
     // from the main thread.
@@ -89,10 +85,6 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
                                   agent_entry, std::move(ptr)));
   }
 
-  // Called from |background_task_runner_| to close the recorder proxy on the
-  // correct task runner.
-  void CloseRecorder(mojom::RecorderPtr recorder) {}
-
   // Called from |main_task_runner_| either after flushing is complete or at
   // shutdown. We either will not write to the stream afterwards or do not care
   // what happens to what we try to write.
@@ -108,16 +100,24 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
   }
 
  private:
-  // Called from |background_task_runner_|.
+  // Handles synchronize writes to |stream_|, if the stream is not already
+  // closed.
+  void WriteToStream(const std::string& data) {
+    if (stream_.is_valid())
+      mojo::BlockingCopyFromString(data, stream_);
+  }
+
+  // Called from |backend_task_runner_|.
   void OnRecorderDataChange(const std::string& label) {
     // Bail out if we are in the middle of writing events for another label to
     // the stream, since we do not want to interleave chunks for different
     // fields. For example, we do not want to mix |traceEvent| chunks with
-    // |battor| chunks.
+    // |systrace| chunks.
     //
-    // If we receive a |battor| chunk from an agent while writing |traceEvent|
-    // chunks to the stream, we wait until all agents that send |traceEvent|
-    // chunks are done, and then, we start writing |battor| chunks.
+    // If we receive a |systemTraceEvents| chunk from an agent while writing
+    // |traceEvent| chunks to the stream, we wait until all agents that send
+    // |traceEvent| chunks are done, and then, we start writing
+    // |systemTraceEvents| chunks.
     if (!streaming_label_.empty() && streaming_label_ != label)
       return;
 
@@ -125,7 +125,10 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
       // We are not waiting for data from any particular label now. So, we look
       // at the recorders that have some data available and select the next
       // label to stream.
-      streaming_label_.clear();
+      if (!streaming_label_.empty()) {
+        processed_labels_.insert(streaming_label_);
+        streaming_label_.clear();
+      }
       bool all_finished = true;
       for (const auto& key_value : recorders_) {
         for (const auto& recorder : key_value.second) {
@@ -145,7 +148,7 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
         if (all_finished) {
           StreamMetadata();
           if (!stream_is_empty_ && agent_label_.empty()) {
-            mojo::common::BlockingCopyFromString("}", stream_);
+            WriteToStream("}");
             stream_is_empty_ = false;
           }
           // Recorder connections should be closed on their binding thread.
@@ -158,7 +161,7 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
     }
   }
 
-  // Called from |background_task_runner_|.
+  // Called from |backend_task_runner_|.
   bool StreamEventsForCurrentLabel() {
     bool waiting_for_agents = false;
     mojom::TraceDataType data_type =
@@ -193,12 +196,11 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
         std::string escaped;
         base::EscapeJSONString(recorder->data(), false /* put_in_quotes */,
                                &escaped);
-        mojo::common::BlockingCopyFromString(prefix + escaped, stream_);
+        WriteToStream(prefix + escaped);
       } else {
         if (prefix.empty() && !stream_is_empty_)
           prefix = ",";
-        mojo::common::BlockingCopyFromString(prefix + recorder->data(),
-                                             stream_);
+        WriteToStream(prefix + recorder->data());
       }
       stream_is_empty_ = false;
       recorder->clear_data();
@@ -207,13 +209,13 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
       if (json_field_name_written_) {
         switch (data_type) {
           case mojom::TraceDataType::ARRAY:
-            mojo::common::BlockingCopyFromString("]", stream_);
+            WriteToStream("]");
             break;
           case mojom::TraceDataType::OBJECT:
-            mojo::common::BlockingCopyFromString("}", stream_);
+            WriteToStream("}");
             break;
           case mojom::TraceDataType::STRING:
-            mojo::common::BlockingCopyFromString("\"", stream_);
+            WriteToStream("\"");
             break;
           default:
             NOTREACHED();
@@ -224,7 +226,7 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
     return waiting_for_agents;
   }
 
-  // Called from |background_task_runner_|.
+  // Called from |backend_task_runner_|.
   void StreamMetadata() {
     if (!agent_label_.empty())
       return;
@@ -239,9 +241,8 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
     if (!metadata_->empty() &&
         base::JSONWriter::Write(*metadata_, &metadataJSON)) {
       std::string prefix = stream_is_empty_ ? "{\"" : ",\"";
-      mojo::common::BlockingCopyFromString(
-          prefix + std::string(kMetadataTraceLabel) + "\":" + metadataJSON,
-          stream_);
+      WriteToStream(prefix + std::string(kMetadataTraceLabel) +
+                    "\":" + metadataJSON);
       stream_is_empty_ = false;
     }
   }
@@ -254,6 +255,7 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
   base::WeakPtr<Coordinator> coordinator_;
 
   std::map<std::string, std::set<std::unique_ptr<Recorder>>> recorders_;
+  std::set<std::string> processed_labels_;
 
   // If |streaming_label_| is not empty, it shows the label for which we are
   // writing chunks to the output stream.
@@ -265,246 +267,178 @@ class Coordinator::TraceStreamer : public base::SupportsWeakPtr<TraceStreamer> {
   DISALLOW_COPY_AND_ASSIGN(TraceStreamer);
 };
 
-// static
-Coordinator* Coordinator::GetInstance() {
-  DCHECK(g_coordinator);
-  return g_coordinator;
-}
-
-Coordinator::Coordinator(
-    service_manager::ServiceContextRefFactory* service_ref_factory)
-    : binding_(this),
-      task_runner_(base::ThreadTaskRunnerHandle::Get()),
-      agent_registry_(AgentRegistry::GetInstance()),
-      service_ref_(service_ref_factory->CreateRef()),
+Coordinator::Coordinator(AgentRegistry* agent_registry,
+                         const base::RepeatingClosure& on_disconnect_callback)
+    : on_disconnect_callback_(std::move(on_disconnect_callback)),
+      binding_(this),
+      task_runner_(base::SequencedTaskRunnerHandle::Get()),
+      // USER_VISIBLE because the task posted from StopAndFlushInternal() is
+      // required to stop tracing from the UI.
+      // TODO(fdoray): Once we have support for dynamic priorities
+      // (https://crbug.com/889029), use BEST_EFFORT initially and increase the
+      // priority only when blocking the tracing UI.
+      backend_task_runner_(base::CreateSequencedTaskRunnerWithTraits(
+          {base::TaskPriority::USER_VISIBLE, base::MayBlock(),
+           base::WithBaseSyncPrimitives()})),
+      agent_registry_(agent_registry),
       weak_ptr_factory_(this) {
-  DCHECK(!g_coordinator);
   DCHECK(agent_registry_);
-  g_coordinator = this;
-  constexpr base::TaskTraits traits = {base::MayBlock(),
-                                       base::WithBaseSyncPrimitives(),
-                                       base::TaskPriority::BACKGROUND};
-  background_task_runner_ = base::CreateSequencedTaskRunnerWithTraits(traits);
 }
 
 Coordinator::~Coordinator() {
+  Reset();
+}
+
+bool Coordinator::IsConnected() {
+  return !!binding_;
+}
+
+void Coordinator::Reset() {
   if (!stop_and_flush_callback_.is_null()) {
     base::ResetAndReturn(&stop_and_flush_callback_)
-        .Run(std::make_unique<base::DictionaryValue>());
+        .Run(base::Value(base::Value::Type::DICTIONARY));
   }
-  if (!start_tracing_callback_.is_null())
-    base::ResetAndReturn(&start_tracing_callback_).Run(false);
   if (!request_buffer_usage_callback_.is_null())
     base::ResetAndReturn(&request_buffer_usage_callback_).Run(false, 0, 0);
-  if (!get_categories_callback_.is_null())
-    base::ResetAndReturn(&get_categories_callback_).Run(false, "");
 
   if (trace_streamer_) {
     // We are in the middle of flushing trace data. We need to
     // 1- Close the stream so that the TraceStreamer does not block on writing
     //    to it.
-    // 2- Delete the TraceStreamer on the background task runner; it owns
-    //    recorders that should be destructed on the background task runner
-    //    because they are bound on the background task runner.
+    // 2- Delete the TraceStreamer on the backend task runner; it owns recorders
+    //    that should be destructed on the backend task runner because they are
+    //    bound on the backend task runner.
     trace_streamer_->CloseStream();
-    background_task_runner_->DeleteSoon(FROM_HERE, trace_streamer_.release());
+    backend_task_runner_->DeleteSoon(FROM_HERE, trace_streamer_.release());
   }
-
-  g_coordinator = nullptr;
 }
 
+void Coordinator::OnClientConnectionError() {
+  Reset();
+  binding_.Close();
+  on_disconnect_callback_.Run();
+}
 void Coordinator::BindCoordinatorRequest(
     mojom::CoordinatorRequest request,
     const service_manager::BindSourceInfo& source_info) {
   binding_.Bind(std::move(request));
+  binding_.set_connection_error_handler(base::BindRepeating(
+      &Coordinator::OnClientConnectionError, base::Unretained(this)));
 }
 
-void Coordinator::StartTracing(const std::string& config,
-                               const StartTracingCallback& callback) {
-  if (is_tracing_) {
-    // Cannot change the config while tracing is enabled.
-    callback.Run(config == config_);
+void Coordinator::StartTracing(const std::string& config) {
+  if ((is_tracing_ && config == config_)) {
     return;
   }
 
   is_tracing_ = true;
   config_ = config;
-  agent_registry_->SetAgentInitializationCallback(base::BindRepeating(
-      &Coordinator::SendStartTracingToAgent, weak_ptr_factory_.GetWeakPtr()));
-  if (!agent_registry_->HasDisconnectClosure(&kStartTracingClosureName)) {
-    callback.Run(true);
-    return;
-  }
-  start_tracing_callback_ = callback;
+  parsed_config_ = base::trace_event::TraceConfig(config);
+  agent_registry_->SetAgentInitializationCallback(
+      base::BindRepeating(&Coordinator::SendStartTracingToAgent,
+                          weak_ptr_factory_.GetWeakPtr()),
+      false /* call_on_new_agents_only */);
 }
 
 void Coordinator::SendStartTracingToAgent(
     AgentRegistry::AgentEntry* agent_entry) {
-  DCHECK(!agent_entry->is_tracing());
-  agent_entry->AddDisconnectClosure(
-      &kStartTracingClosureName,
-      base::BindOnce(&Coordinator::OnTracingStarted,
-                     weak_ptr_factory_.GetWeakPtr(),
-                     base::Unretained(agent_entry), false));
-  agent_entry->agent()->StartTracing(
-      config_, base::TimeTicks::Now(),
-      base::BindRepeating(&Coordinator::OnTracingStarted,
-                          weak_ptr_factory_.GetWeakPtr(),
-                          base::Unretained(agent_entry)));
-}
-
-void Coordinator::OnTracingStarted(AgentRegistry::AgentEntry* agent_entry,
-                                   bool success) {
-  agent_entry->set_is_tracing(success);
-  bool removed =
-      agent_entry->RemoveDisconnectClosure(&kStartTracingClosureName);
-  DCHECK(removed);
-
-  if (!agent_registry_->HasDisconnectClosure(&kStartTracingClosureName) &&
-      !start_tracing_callback_.is_null()) {
-    base::ResetAndReturn(&start_tracing_callback_).Run(true);
-  }
+  if (!parsed_config_.process_filter_config().IsEnabled(agent_entry->pid()))
+    return;
+  agent_entry->agent()->StartTracing(config_, TRACE_TIME_TICKS_NOW());
 }
 
 void Coordinator::StopAndFlush(mojo::ScopedDataPipeProducerHandle stream,
-                               const StopAndFlushCallback& callback) {
-  StopAndFlushAgent(std::move(stream), "", callback);
+                               StopAndFlushCallback callback) {
+  StopAndFlushAgent(std::move(stream), "", std::move(callback));
 }
 
 void Coordinator::StopAndFlushAgent(mojo::ScopedDataPipeProducerHandle stream,
                                     const std::string& agent_label,
-                                    const StopAndFlushCallback& callback) {
+                                    StopAndFlushCallback callback) {
   if (!is_tracing_) {
     stream.reset();
-    callback.Run(std::make_unique<base::DictionaryValue>());
+    std::move(callback).Run(base::Value(base::Value::Type::DICTIONARY));
     return;
   }
   DCHECK(!trace_streamer_);
   DCHECK(stream.is_valid());
   is_tracing_ = false;
 
-  // Do not send |StartTracing| to agents that connect from now on.
-  agent_registry_->RemoveAgentInitializationCallback();
   trace_streamer_.reset(new Coordinator::TraceStreamer(
       std::move(stream), agent_label, task_runner_,
       weak_ptr_factory_.GetWeakPtr()));
-  stop_and_flush_callback_ = callback;
+  stop_and_flush_callback_ = std::move(callback);
   StopAndFlushInternal();
 }
 
 void Coordinator::StopAndFlushInternal() {
-  if (agent_registry_->HasDisconnectClosure(&kStartTracingClosureName)) {
-    // We received a |StopAndFlush| command before receiving |StartTracing| acks
-    // from all agents. Let's retry after a delay.
-    task_runner_->PostDelayedTask(
-        FROM_HERE,
-        base::BindRepeating(&Coordinator::StopAndFlushInternal,
-                            weak_ptr_factory_.GetWeakPtr()),
-        base::TimeDelta::FromMilliseconds(
-            mojom::kStopTracingRetryTimeMilliseconds));
-    return;
-  }
-
-  agent_registry_->ForAllAgents([this](AgentRegistry::AgentEntry* agent_entry) {
-    if (!agent_entry->is_tracing() ||
-        !agent_entry->supports_explicit_clock_sync()) {
-      return;
-    }
-    const std::string sync_id = base::GenerateGUID();
-    agent_entry->AddDisconnectClosure(
-        &kRequestClockSyncMarkerClosureName,
-        base::BindOnce(&Coordinator::OnRequestClockSyncMarkerResponse,
-                       weak_ptr_factory_.GetWeakPtr(),
-                       base::Unretained(agent_entry), sync_id,
-                       base::TimeTicks(), base::TimeTicks()));
-    agent_entry->agent()->RequestClockSyncMarker(
-        sync_id,
-        base::BindRepeating(&Coordinator::OnRequestClockSyncMarkerResponse,
-                            weak_ptr_factory_.GetWeakPtr(),
-                            base::Unretained(agent_entry), sync_id));
-  });
-  if (!agent_registry_->HasDisconnectClosure(
-          &kRequestClockSyncMarkerClosureName)) {
-    StopAndFlushAfterClockSync();
-  }
-}
-
-void Coordinator::OnRequestClockSyncMarkerResponse(
-    AgentRegistry::AgentEntry* agent_entry,
-    const std::string& sync_id,
-    base::TimeTicks issue_ts,
-    base::TimeTicks issue_end_ts) {
-  bool removed =
-      agent_entry->RemoveDisconnectClosure(&kRequestClockSyncMarkerClosureName);
-  DCHECK(removed);
-
-  // TODO(charliea): Change this function so that it can accept a boolean
-  // success indicator instead of having to rely on sentinel issue_ts and
-  // issue_end_ts values to signal failure.
-  if (!(issue_ts == base::TimeTicks() || issue_end_ts == base::TimeTicks()))
-    TRACE_EVENT_CLOCK_SYNC_ISSUER(sync_id, issue_ts, issue_end_ts);
-
-  if (!agent_registry_->HasDisconnectClosure(
-          &kRequestClockSyncMarkerClosureName)) {
-    StopAndFlushAfterClockSync();
-  }
-}
-
-void Coordinator::StopAndFlushAfterClockSync() {
-  bool has_tracing_agents = false;
-  agent_registry_->ForAllAgents(
-      [this, &has_tracing_agents](AgentRegistry::AgentEntry* agent_entry) {
-        if (!agent_entry->is_tracing())
-          return;
-        has_tracing_agents = true;
-        background_task_runner_->PostTask(
-            FROM_HERE,
-            base::BindOnce(&Coordinator::TraceStreamer::CreateAndSendRecorder,
-                           trace_streamer_->AsWeakPtr(), agent_entry->label(),
-                           agent_entry->type(), agent_entry->AsWeakPtr()));
-      });
-  if (!has_tracing_agents)
+  size_t num_initialized_agents =
+      agent_registry_->SetAgentInitializationCallback(
+          base::BindRepeating(&Coordinator::SendStopTracingToAgent,
+                              weak_ptr_factory_.GetWeakPtr()),
+          false /* call_on_new_agents_only */);
+  if (num_initialized_agents == 0)
     OnFlushDone();
 }
 
+void Coordinator::SendStopTracingToAgent(
+    AgentRegistry::AgentEntry* agent_entry) {
+  backend_task_runner_->PostTask(
+      FROM_HERE,
+      base::BindOnce(&Coordinator::TraceStreamer::CreateAndSendRecorder,
+                     trace_streamer_->AsWeakPtr(), agent_entry->label(),
+                     agent_entry->type(), agent_entry->AsWeakPtr()));
+}
+
 void Coordinator::SendRecorder(
     base::WeakPtr<AgentRegistry::AgentEntry> agent_entry,
     mojom::RecorderPtr recorder) {
   if (agent_entry) {
     agent_entry->agent()->StopAndFlush(std::move(recorder));
   } else {
-    // Recorders are created and closed on |background_task_runner_|.
-    background_task_runner_->PostTask(
+    // Recorders are created and closed on |backend_task_runner_|.
+    backend_task_runner_->PostTask(
         FROM_HERE,
-        base::BindOnce(&Coordinator::TraceStreamer::CloseRecorder,
-                       trace_streamer_->AsWeakPtr(), std::move(recorder)));
+        base::BindOnce([](mojom::RecorderPtr ptr) {}, std::move(recorder)));
   }
 }
 
 void Coordinator::OnFlushDone() {
-  base::ResetAndReturn(&stop_and_flush_callback_)
-      .Run(trace_streamer_->GetMetadata());
-  background_task_runner_->DeleteSoon(FROM_HERE, trace_streamer_.release());
-  agent_registry_->ForAllAgents([this](AgentRegistry::AgentEntry* agent_entry) {
-    agent_entry->set_is_tracing(false);
-  });
+  std::move(stop_and_flush_callback_)
+      .Run(std::move(*trace_streamer_->GetMetadata()));
+  backend_task_runner_->DeleteSoon(FROM_HERE, trace_streamer_.release());
   is_tracing_ = false;
+  agent_registry_->SetAgentInitializationCallback(
+      base::BindRepeating(&Coordinator::SendStopTracingWithNoOpRecorderToAgent,
+                          weak_ptr_factory_.GetWeakPtr()),
+      true /* call_on_new_agents_only */);
 }
 
-void Coordinator::IsTracing(const IsTracingCallback& callback) {
-  callback.Run(is_tracing_);
+void Coordinator::SendStopTracingWithNoOpRecorderToAgent(
+    AgentRegistry::AgentEntry* agent_entry) {
+  mojom::RecorderPtr ptr;
+  // We need to create a message pipe and bind |ptr| to one end of it before
+  // sending |ptr| to the agent; otherwise, the agent will fail as soon as it
+  // tries to invoke a method on |ptr|. We do not have to bind the
+  // implementation end of the message pipe since we are going to ignore the
+  // messages.
+  MakeRequest(&ptr);
+  agent_entry->agent()->StopAndFlush(std::move(ptr));
 }
 
-void Coordinator::RequestBufferUsage(
-    const RequestBufferUsageCallback& callback) {
+void Coordinator::IsTracing(IsTracingCallback callback) {
+  std::move(callback).Run(is_tracing_);
+}
+
+void Coordinator::RequestBufferUsage(RequestBufferUsageCallback callback) {
   if (!request_buffer_usage_callback_.is_null()) {
-    callback.Run(false, 0, 0);
+    std::move(callback).Run(false, 0, 0);
     return;
   }
 
   maximum_trace_buffer_usage_ = 0;
   approximate_event_count_ = 0;
-  request_buffer_usage_callback_ = callback;
+
   agent_registry_->ForAllAgents([this](AgentRegistry::AgentEntry* agent_entry) {
     agent_entry->AddDisconnectClosure(
         &kRequestBufferUsageClosureName,
@@ -516,6 +450,12 @@ void Coordinator::RequestBufferUsage(
         &Coordinator::OnRequestBufferStatusResponse,
         weak_ptr_factory_.GetWeakPtr(), base::Unretained(agent_entry)));
   });
+
+  if (!agent_registry_->HasDisconnectClosure(&kRequestBufferUsageClosureName)) {
+    std::move(callback).Run(true, 0.0f, 0);
+    return;
+  }
+  request_buffer_usage_callback_ = std::move(callback);
 }
 
 void Coordinator::OnRequestBufferStatusResponse(
@@ -535,52 +475,9 @@ void Coordinator::OnRequestBufferStatusResponse(
   }
 
   if (!agent_registry_->HasDisconnectClosure(&kRequestBufferUsageClosureName)) {
-    base::ResetAndReturn(&request_buffer_usage_callback_)
+    std::move(request_buffer_usage_callback_)
         .Run(true, maximum_trace_buffer_usage_, approximate_event_count_);
   }
 }
 
-void Coordinator::GetCategories(const GetCategoriesCallback& callback) {
-  if (is_tracing_) {
-    callback.Run(false, "");
-  }
-
-  DCHECK(get_categories_callback_.is_null());
-  is_tracing_ = true;
-  category_set_.clear();
-  get_categories_callback_ = callback;
-  agent_registry_->ForAllAgents([this](AgentRegistry::AgentEntry* agent_entry) {
-    agent_entry->AddDisconnectClosure(
-        &kGetCategoriesClosureName,
-        base::BindOnce(&Coordinator::OnGetCategoriesResponse,
-                       weak_ptr_factory_.GetWeakPtr(),
-                       base::Unretained(agent_entry), ""));
-    agent_entry->agent()->GetCategories(base::BindRepeating(
-        &Coordinator::OnGetCategoriesResponse, weak_ptr_factory_.GetWeakPtr(),
-        base::Unretained(agent_entry)));
-  });
-}
-
-void Coordinator::OnGetCategoriesResponse(
-    AgentRegistry::AgentEntry* agent_entry,
-    const std::string& categories) {
-  bool removed =
-      agent_entry->RemoveDisconnectClosure(&kGetCategoriesClosureName);
-  DCHECK(removed);
-
-  std::vector<std::string> split = base::SplitString(
-      categories, ",", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
-  for (const auto& category : split) {
-    category_set_.insert(category);
-  }
-
-  if (!agent_registry_->HasDisconnectClosure(&kGetCategoriesClosureName)) {
-    std::vector<std::string> category_vector(category_set_.begin(),
-                                             category_set_.end());
-    base::ResetAndReturn(&get_categories_callback_)
-        .Run(true, base::JoinString(category_vector, ","));
-    is_tracing_ = false;
-  }
-}
-
 }  // namespace tracing
