@@ -9,12 +9,25 @@
 #include "src/contexts.h"
 #include "src/debug/debug-interface.h"
 #include "src/detachable-vector.h"
-#include "src/factory.h"
+#include "src/heap/factory.h"
 #include "src/isolate.h"
+#include "src/objects.h"
+#include "src/objects/bigint.h"
 #include "src/objects/js-collection.h"
+#include "src/objects/js-generator.h"
+#include "src/objects/js-promise.h"
+#include "src/objects/js-proxy.h"
+#include "src/objects/module.h"
+#include "src/objects/shared-function-info.h"
+
+#include "src/objects/templates.h"
 
 namespace v8 {
 
+namespace internal {
+class JSArrayBufferView;
+}  // namespace internal
+
 // Constants used in the implementation of the API.  The most natural thing
 // would usually be to place these with the classes that use them, but
 // we want to keep them out of v8.h because it is an externally
@@ -27,24 +40,19 @@ class Consts {
   };
 };
 
-template <typename T> inline T ToCData(v8::internal::Object* obj) {
-  STATIC_ASSERT(sizeof(T) == sizeof(v8::internal::Address));
-  if (obj == v8::internal::Smi::kZero) return nullptr;
-  return reinterpret_cast<T>(
-      reinterpret_cast<intptr_t>(
-          v8::internal::Foreign::cast(obj)->foreign_address()));
-}
+template <typename T>
+inline T ToCData(v8::internal::Object obj);
 
+template <>
+inline v8::internal::Address ToCData(v8::internal::Object obj);
 
 template <typename T>
 inline v8::internal::Handle<v8::internal::Object> FromCData(
-    v8::internal::Isolate* isolate, T obj) {
-  STATIC_ASSERT(sizeof(T) == sizeof(v8::internal::Address));
-  if (obj == nullptr) return handle(v8::internal::Smi::kZero, isolate);
-  return isolate->factory()->NewForeign(
-      reinterpret_cast<v8::internal::Address>(reinterpret_cast<intptr_t>(obj)));
-}
+    v8::internal::Isolate* isolate, T obj);
 
+template <>
+inline v8::internal::Handle<v8::internal::Object> FromCData(
+    v8::internal::Isolate* isolate, v8::internal::Address obj);
 
 class ApiFunction {
  public:
@@ -58,15 +66,22 @@ class ApiFunction {
 
 class RegisteredExtension {
  public:
-  explicit RegisteredExtension(Extension* extension);
-  static void Register(RegisteredExtension* that);
+  static void Register(Extension*);
+  static void Register(std::unique_ptr<Extension>);
   static void UnregisterAll();
-  Extension* extension() { return extension_; }
-  RegisteredExtension* next() { return next_; }
+  Extension* extension() const {
+    return legacy_unowned_extension_ ? legacy_unowned_extension_
+                                     : extension_.get();
+  }
+  RegisteredExtension* next() const { return next_; }
   static RegisteredExtension* first_extension() { return first_extension_; }
  private:
-  Extension* extension_;
-  RegisteredExtension* next_;
+  explicit RegisteredExtension(Extension*);
+  explicit RegisteredExtension(std::unique_ptr<Extension>);
+  // TODO(clemensh): Remove this after the 7.4 branch.
+  Extension* legacy_unowned_extension_ = nullptr;
+  std::unique_ptr<Extension> extension_;
+  RegisteredExtension* next_ = nullptr;
   static RegisteredExtension* first_extension_;
 };
 
@@ -100,6 +115,7 @@ class RegisteredExtension {
   V(String, String)                            \
   V(Symbol, Symbol)                            \
   V(Script, JSFunction)                        \
+  V(UnboundModuleScript, SharedFunctionInfo)   \
   V(UnboundScript, SharedFunctionInfo)         \
   V(Module, Module)                            \
   V(Function, JSReceiver)                      \
@@ -111,9 +127,11 @@ class RegisteredExtension {
   V(Proxy, JSProxy)                            \
   V(debug::GeneratorObject, JSGeneratorObject) \
   V(debug::Script, Script)                     \
+  V(debug::WeakMap, JSWeakMap)                 \
   V(Promise, JSPromise)                        \
   V(Primitive, Object)                         \
   V(PrimitiveArray, FixedArray)                \
+  V(BigInt, BigInt)                            \
   V(ScriptOrModule, Script)
 
 class Utils {
@@ -124,7 +142,8 @@ class Utils {
     if (!condition) Utils::ReportApiFailure(location, message);
     return condition;
   }
-  static void ReportOOMFailure(const char* location, bool is_heap_oom);
+  static void ReportOOMFailure(v8::internal::Isolate* isolate,
+                               const char* location, bool is_heap_oom);
 
   static inline Local<Context> ToLocal(
       v8::internal::Handle<v8::internal::Context> obj);
@@ -223,27 +242,22 @@ class Utils {
   static inline Local<ScriptOrModule> ScriptOrModuleToLocal(
       v8::internal::Handle<v8::internal::Script> obj);
 
-#define DECLARE_OPEN_HANDLE(From, To) \
-  static inline v8::internal::Handle<v8::internal::To> \
-      OpenHandle(const From* that, bool allow_empty_handle = false);
+#define DECLARE_OPEN_HANDLE(From, To)                              \
+  static inline v8::internal::Handle<v8::internal::To> OpenHandle( \
+      const From* that, bool allow_empty_handle = false);
 
-OPEN_HANDLE_LIST(DECLARE_OPEN_HANDLE)
+  OPEN_HANDLE_LIST(DECLARE_OPEN_HANDLE)
 
 #undef DECLARE_OPEN_HANDLE
 
-  template<class From, class To>
-  static inline Local<To> Convert(v8::internal::Handle<From> obj) {
-    DCHECK(obj.is_null() ||
-           (obj->IsSmi() ||
-            !obj->IsTheHole(i::HeapObject::cast(*obj)->GetIsolate())));
-    return Local<To>(reinterpret_cast<To*>(obj.location()));
-  }
+  template <class From, class To>
+  static inline Local<To> Convert(v8::internal::Handle<From> obj);
 
   template <class T>
   static inline v8::internal::Handle<v8::internal::Object> OpenPersistent(
       const v8::Persistent<T>& persistent) {
     return v8::internal::Handle<v8::internal::Object>(
-        reinterpret_cast<v8::internal::Object**>(persistent.val_));
+        reinterpret_cast<v8::internal::Address*>(persistent.val_));
   }
 
   template <class T>
@@ -257,11 +271,15 @@ OPEN_HANDLE_LIST(DECLARE_OPEN_HANDLE)
     return OpenHandle(*handle);
   }
 
+  static inline CompiledWasmModule Convert(
+      std::shared_ptr<i::wasm::NativeModule> native_module) {
+    return CompiledWasmModule{std::move(native_module)};
+  }
+
  private:
   static void ReportApiFailure(const char* location, const char* message);
 };
 
-
 template <class T>
 inline T* ToApi(v8::internal::Handle<v8::internal::Object> obj) {
   return reinterpret_cast<T*>(obj.location());
@@ -285,87 +303,6 @@ inline bool ToLocal(v8::internal::MaybeHandle<v8::internal::Object> maybe,
   return false;
 }
 
-
-// Implementations of ToLocal
-
-#define MAKE_TO_LOCAL(Name, From, To)                                       \
-  Local<v8::To> Utils::Name(v8::internal::Handle<v8::internal::From> obj) { \
-    return Convert<v8::internal::From, v8::To>(obj);  \
-  }
-
-
-#define MAKE_TO_LOCAL_TYPED_ARRAY(Type, typeName, TYPE, ctype, size)  \
-  Local<v8::Type##Array> Utils::ToLocal##Type##Array(                 \
-      v8::internal::Handle<v8::internal::JSTypedArray> obj) {         \
-    DCHECK(obj->type() == v8::internal::kExternal##Type##Array);      \
-    return Convert<v8::internal::JSTypedArray, v8::Type##Array>(obj); \
-  }
-
-
-MAKE_TO_LOCAL(ToLocal, Context, Context)
-MAKE_TO_LOCAL(ToLocal, Object, Value)
-MAKE_TO_LOCAL(ToLocal, Module, Module)
-MAKE_TO_LOCAL(ToLocal, Name, Name)
-MAKE_TO_LOCAL(ToLocal, String, String)
-MAKE_TO_LOCAL(ToLocal, Symbol, Symbol)
-MAKE_TO_LOCAL(ToLocal, JSRegExp, RegExp)
-MAKE_TO_LOCAL(ToLocal, JSReceiver, Object)
-MAKE_TO_LOCAL(ToLocal, JSObject, Object)
-MAKE_TO_LOCAL(ToLocal, JSFunction, Function)
-MAKE_TO_LOCAL(ToLocal, JSArray, Array)
-MAKE_TO_LOCAL(ToLocal, JSMap, Map)
-MAKE_TO_LOCAL(ToLocal, JSSet, Set)
-MAKE_TO_LOCAL(ToLocal, JSProxy, Proxy)
-MAKE_TO_LOCAL(ToLocal, JSArrayBuffer, ArrayBuffer)
-MAKE_TO_LOCAL(ToLocal, JSArrayBufferView, ArrayBufferView)
-MAKE_TO_LOCAL(ToLocal, JSDataView, DataView)
-MAKE_TO_LOCAL(ToLocal, JSTypedArray, TypedArray)
-MAKE_TO_LOCAL(ToLocalShared, JSArrayBuffer, SharedArrayBuffer)
-
-TYPED_ARRAYS(MAKE_TO_LOCAL_TYPED_ARRAY)
-
-MAKE_TO_LOCAL(ToLocal, FunctionTemplateInfo, FunctionTemplate)
-MAKE_TO_LOCAL(ToLocal, ObjectTemplateInfo, ObjectTemplate)
-MAKE_TO_LOCAL(SignatureToLocal, FunctionTemplateInfo, Signature)
-MAKE_TO_LOCAL(AccessorSignatureToLocal, FunctionTemplateInfo, AccessorSignature)
-MAKE_TO_LOCAL(MessageToLocal, Object, Message)
-MAKE_TO_LOCAL(PromiseToLocal, JSObject, Promise)
-MAKE_TO_LOCAL(StackTraceToLocal, FixedArray, StackTrace)
-MAKE_TO_LOCAL(StackFrameToLocal, StackFrameInfo, StackFrame)
-MAKE_TO_LOCAL(NumberToLocal, Object, Number)
-MAKE_TO_LOCAL(IntegerToLocal, Object, Integer)
-MAKE_TO_LOCAL(Uint32ToLocal, Object, Uint32)
-MAKE_TO_LOCAL(ToLocal, BigInt, BigInt);
-MAKE_TO_LOCAL(ExternalToLocal, JSObject, External)
-MAKE_TO_LOCAL(CallableToLocal, JSReceiver, Function)
-MAKE_TO_LOCAL(ToLocalPrimitive, Object, Primitive)
-MAKE_TO_LOCAL(ToLocal, FixedArray, PrimitiveArray)
-MAKE_TO_LOCAL(ScriptOrModuleToLocal, Script, ScriptOrModule)
-
-#undef MAKE_TO_LOCAL_TYPED_ARRAY
-#undef MAKE_TO_LOCAL
-
-
-// Implementations of OpenHandle
-
-#define MAKE_OPEN_HANDLE(From, To)                                             \
-  v8::internal::Handle<v8::internal::To> Utils::OpenHandle(                    \
-      const v8::From* that, bool allow_empty_handle) {                         \
-    DCHECK(allow_empty_handle || that != nullptr);                             \
-    DCHECK(that == nullptr ||                                                  \
-           (*reinterpret_cast<v8::internal::Object* const*>(that))->Is##To()); \
-    return v8::internal::Handle<v8::internal::To>(                             \
-        reinterpret_cast<v8::internal::To**>(const_cast<v8::From*>(that)));    \
-  }
-
-OPEN_HANDLE_LIST(MAKE_OPEN_HANDLE)
-
-#undef MAKE_OPEN_HANDLE
-#undef OPEN_HANDLE_LIST
-
-extern Isolate* IsolateNewImpl(internal::Isolate* isolate,
-                               const Isolate::CreateParams& params);
-
 namespace internal {
 
 class V8_EXPORT_PRIVATE DeferredHandles {
@@ -373,7 +310,7 @@ class V8_EXPORT_PRIVATE DeferredHandles {
   ~DeferredHandles();
 
  private:
-  DeferredHandles(Object** first_block_limit, Isolate* isolate)
+  DeferredHandles(Address* first_block_limit, Isolate* isolate)
       : next_(nullptr),
         previous_(nullptr),
         first_block_limit_(first_block_limit),
@@ -383,10 +320,10 @@ class V8_EXPORT_PRIVATE DeferredHandles {
 
   void Iterate(RootVisitor* v);
 
-  std::vector<Object**> blocks_;
+  std::vector<Address*> blocks_;
   DeferredHandles* next_;
   DeferredHandles* previous_;
-  Object** first_block_limit_;
+  Address* first_block_limit_;
   Isolate* isolate_;
 
   friend class HandleScopeImplementer;
@@ -405,19 +342,26 @@ class V8_EXPORT_PRIVATE DeferredHandles {
 // data.
 class HandleScopeImplementer {
  public:
+  class EnteredContextRewindScope {
+   public:
+    explicit EnteredContextRewindScope(HandleScopeImplementer* hsi)
+        : hsi_(hsi), saved_entered_context_count_(hsi->EnteredContextCount()) {}
+
+    ~EnteredContextRewindScope() {
+      DCHECK_LE(saved_entered_context_count_, hsi_->EnteredContextCount());
+      while (saved_entered_context_count_ < hsi_->EnteredContextCount())
+        hsi_->LeaveContext();
+    }
+
+   private:
+    HandleScopeImplementer* hsi_;
+    size_t saved_entered_context_count_;
+  };
+
   explicit HandleScopeImplementer(Isolate* isolate)
       : isolate_(isolate),
-        microtask_context_(nullptr),
         spare_(nullptr),
         call_depth_(0),
-        microtasks_depth_(0),
-        microtasks_suppressions_(0),
-        entered_contexts_count_(0),
-        entered_context_count_during_microtasks_(0),
-#ifdef DEBUG
-        debug_microtasks_depth_(0),
-#endif
-        microtasks_policy_(v8::MicrotasksPolicy::kAuto),
         last_handle_before_deferred_block_(nullptr) {
   }
 
@@ -435,75 +379,48 @@ class HandleScopeImplementer {
   void Iterate(v8::internal::RootVisitor* v);
   static char* Iterate(v8::internal::RootVisitor* v, char* data);
 
-  inline internal::Object** GetSpareOrNewBlock();
-  inline void DeleteExtensions(internal::Object** prev_limit);
+  inline internal::Address* GetSpareOrNewBlock();
+  inline void DeleteExtensions(internal::Address* prev_limit);
 
   // Call depth represents nested v8 api calls.
   inline void IncrementCallDepth() {call_depth_++;}
   inline void DecrementCallDepth() {call_depth_--;}
   inline bool CallDepthIsZero() { return call_depth_ == 0; }
 
-  // Microtasks scope depth represents nested scopes controlling microtasks
-  // invocation, which happens when depth reaches zero.
-  inline void IncrementMicrotasksScopeDepth() {microtasks_depth_++;}
-  inline void DecrementMicrotasksScopeDepth() {microtasks_depth_--;}
-  inline int GetMicrotasksScopeDepth() { return microtasks_depth_; }
-
-  // Possibly nested microtasks suppression scopes prevent microtasks
-  // from running.
-  inline void IncrementMicrotasksSuppressions() {microtasks_suppressions_++;}
-  inline void DecrementMicrotasksSuppressions() {microtasks_suppressions_--;}
-  inline bool HasMicrotasksSuppressions() { return !!microtasks_suppressions_; }
-
-#ifdef DEBUG
-  // In debug we check that calls not intended to invoke microtasks are
-  // still correctly wrapped with microtask scopes.
-  inline void IncrementDebugMicrotasksScopeDepth() {debug_microtasks_depth_++;}
-  inline void DecrementDebugMicrotasksScopeDepth() {debug_microtasks_depth_--;}
-  inline bool DebugMicrotasksScopeDepthIsZero() {
-    return debug_microtasks_depth_ == 0;
-  }
-#endif
-
-  inline void set_microtasks_policy(v8::MicrotasksPolicy policy);
-  inline v8::MicrotasksPolicy microtasks_policy() const;
-
-  inline void EnterContext(Handle<Context> context);
+  inline void EnterContext(Context context);
   inline void LeaveContext();
-  inline bool LastEnteredContextWas(Handle<Context> context);
+  inline bool LastEnteredContextWas(Context context);
+  inline size_t EnteredContextCount() const { return entered_contexts_.size(); }
+
+  inline void EnterMicrotaskContext(Context context);
 
   // Returns the last entered context or an empty handle if no
   // contexts have been entered.
   inline Handle<Context> LastEnteredContext();
+  inline Handle<Context> LastEnteredOrMicrotaskContext();
 
-  inline void EnterMicrotaskContext(Handle<Context> context);
-  inline void LeaveMicrotaskContext();
-  inline Handle<Context> MicrotaskContext();
-  inline bool MicrotaskContextIsLastEnteredContext() const {
-    return microtask_context_ &&
-           entered_context_count_during_microtasks_ == entered_contexts_.size();
-  }
-
-  inline void SaveContext(Context* context);
-  inline Context* RestoreContext();
+  inline void SaveContext(Context context);
+  inline Context RestoreContext();
   inline bool HasSavedContexts();
 
-  inline DetachableVector<Object**>* blocks() { return &blocks_; }
+  inline DetachableVector<Address*>* blocks() { return &blocks_; }
   Isolate* isolate() const { return isolate_; }
 
-  void ReturnBlock(Object** block) {
+  void ReturnBlock(Address* block) {
     DCHECK_NOT_NULL(block);
     if (spare_ != nullptr) DeleteArray(spare_);
     spare_ = block;
   }
 
+  static const size_t kEnteredContextsOffset;
+  static const size_t kIsMicrotaskContextOffset;
+
  private:
   void ResetAfterArchive() {
     blocks_.detach();
     entered_contexts_.detach();
+    is_microtask_context_.detach();
     saved_contexts_.detach();
-    microtask_context_ = nullptr;
-    entered_context_count_during_microtasks_ = 0;
     spare_ = nullptr;
     last_handle_before_deferred_block_ = nullptr;
     call_depth_ = 0;
@@ -512,11 +429,12 @@ class HandleScopeImplementer {
   void Free() {
     DCHECK(blocks_.empty());
     DCHECK(entered_contexts_.empty());
+    DCHECK(is_microtask_context_.empty());
     DCHECK(saved_contexts_.empty());
-    DCHECK(!microtask_context_);
 
     blocks_.free();
     entered_contexts_.free();
+    is_microtask_context_.free();
     saved_contexts_.free();
     if (spare_ != nullptr) {
       DeleteArray(spare_);
@@ -526,26 +444,25 @@ class HandleScopeImplementer {
   }
 
   void BeginDeferredScope();
-  DeferredHandles* Detach(Object** prev_limit);
+  DeferredHandles* Detach(Address* prev_limit);
 
   Isolate* isolate_;
-  DetachableVector<Object**> blocks_;
+  DetachableVector<Address*> blocks_;
+
   // Used as a stack to keep track of entered contexts.
-  DetachableVector<Context*> entered_contexts_;
+  // If |i|th item of |entered_contexts_| is added by EnterMicrotaskContext,
+  // `is_microtask_context_[i]` is 1.
+  // TODO(tzik): Remove |is_microtask_context_| after the deprecated
+  // v8::Isolate::GetEnteredContext() is removed.
+  DetachableVector<Context> entered_contexts_;
+  DetachableVector<int8_t> is_microtask_context_;
+
   // Used as a stack to keep track of saved contexts.
-  DetachableVector<Context*> saved_contexts_;
-  Context* microtask_context_;
-  Object** spare_;
+  DetachableVector<Context> saved_contexts_;
+  Address* spare_;
   int call_depth_;
-  int microtasks_depth_;
-  int microtasks_suppressions_;
-  size_t entered_contexts_count_;
-  size_t entered_context_count_during_microtasks_;
-#ifdef DEBUG
-  int debug_microtasks_depth_;
-#endif
-  v8::MicrotasksPolicy microtasks_policy_;
-  Object** last_handle_before_deferred_block_;
+
+  Address* last_handle_before_deferred_block_;
   // This is only used for threading support.
   HandleScopeData handle_scope_data_;
 
@@ -560,42 +477,14 @@ class HandleScopeImplementer {
   DISALLOW_COPY_AND_ASSIGN(HandleScopeImplementer);
 };
 
-class HandleScopeImplementerOffsets {
- public:
-  enum Offsets {
-    kMicrotaskContext = offsetof(HandleScopeImplementer, microtask_context_),
-    kEnteredContexts = offsetof(HandleScopeImplementer, entered_contexts_),
-    kEnteredContextsCount =
-        offsetof(HandleScopeImplementer, entered_contexts_count_),
-    kEnteredContextCountDuringMicrotasks = offsetof(
-        HandleScopeImplementer, entered_context_count_during_microtasks_)
-  };
-
- private:
-  DISALLOW_IMPLICIT_CONSTRUCTORS(HandleScopeImplementerOffsets);
-};
-
 const int kHandleBlockSize = v8::internal::KB - 2;  // fit in one page
 
-
-void HandleScopeImplementer::set_microtasks_policy(
-    v8::MicrotasksPolicy policy) {
-  microtasks_policy_ = policy;
-}
-
-
-v8::MicrotasksPolicy HandleScopeImplementer::microtasks_policy() const {
-  return microtasks_policy_;
-}
-
-
-void HandleScopeImplementer::SaveContext(Context* context) {
+void HandleScopeImplementer::SaveContext(Context context) {
   saved_contexts_.push_back(context);
 }
 
-
-Context* HandleScopeImplementer::RestoreContext() {
-  Context* last_context = saved_contexts_.back();
+Context HandleScopeImplementer::RestoreContext() {
+  Context last_context = saved_contexts_.back();
   saved_contexts_.pop_back();
   return last_context;
 }
@@ -605,61 +494,50 @@ bool HandleScopeImplementer::HasSavedContexts() {
   return !saved_contexts_.empty();
 }
 
-
-void HandleScopeImplementer::EnterContext(Handle<Context> context) {
-  entered_contexts_.push_back(*context);
-  entered_contexts_count_ = entered_contexts_.size();
+void HandleScopeImplementer::EnterContext(Context context) {
+  DCHECK_EQ(entered_contexts_.size(), is_microtask_context_.size());
+  entered_contexts_.push_back(context);
+  is_microtask_context_.push_back(0);
 }
 
 void HandleScopeImplementer::LeaveContext() {
+  DCHECK(!entered_contexts_.empty());
+  DCHECK_EQ(entered_contexts_.size(), is_microtask_context_.size());
   entered_contexts_.pop_back();
-  entered_contexts_count_ = entered_contexts_.size();
+  is_microtask_context_.pop_back();
 }
 
-bool HandleScopeImplementer::LastEnteredContextWas(Handle<Context> context) {
-  return !entered_contexts_.empty() && entered_contexts_.back() == *context;
+bool HandleScopeImplementer::LastEnteredContextWas(Context context) {
+  return !entered_contexts_.empty() && entered_contexts_.back() == context;
 }
 
-
-Handle<Context> HandleScopeImplementer::LastEnteredContext() {
-  if (entered_contexts_.empty()) return Handle<Context>::null();
-  return Handle<Context>(entered_contexts_.back());
-}
-
-void HandleScopeImplementer::EnterMicrotaskContext(Handle<Context> context) {
-  DCHECK(!microtask_context_);
-  microtask_context_ = *context;
-  entered_context_count_during_microtasks_ = entered_contexts_.size();
-}
-
-void HandleScopeImplementer::LeaveMicrotaskContext() {
-  DCHECK(microtask_context_);
-  microtask_context_ = nullptr;
-  entered_context_count_during_microtasks_ = 0;
-}
-
-Handle<Context> HandleScopeImplementer::MicrotaskContext() {
-  if (microtask_context_) return Handle<Context>(microtask_context_);
-  return Handle<Context>::null();
+void HandleScopeImplementer::EnterMicrotaskContext(Context context) {
+  DCHECK_EQ(entered_contexts_.size(), is_microtask_context_.size());
+  entered_contexts_.push_back(context);
+  is_microtask_context_.push_back(1);
 }
 
 // If there's a spare block, use it for growing the current scope.
-internal::Object** HandleScopeImplementer::GetSpareOrNewBlock() {
-  internal::Object** block =
+internal::Address* HandleScopeImplementer::GetSpareOrNewBlock() {
+  internal::Address* block =
       (spare_ != nullptr) ? spare_
-                          : NewArray<internal::Object*>(kHandleBlockSize);
+                          : NewArray<internal::Address>(kHandleBlockSize);
   spare_ = nullptr;
   return block;
 }
 
-
-void HandleScopeImplementer::DeleteExtensions(internal::Object** prev_limit) {
+void HandleScopeImplementer::DeleteExtensions(internal::Address* prev_limit) {
   while (!blocks_.empty()) {
-    internal::Object** block_start = blocks_.back();
-    internal::Object** block_limit = block_start + kHandleBlockSize;
+    internal::Address* block_start = blocks_.back();
+    internal::Address* block_limit = block_start + kHandleBlockSize;
 
     // SealHandleScope may make the prev_limit to point inside the block.
-    if (block_start <= prev_limit && prev_limit <= block_limit) {
+    // Cast possibly-unrelated pointers to plain Addres before comparing them
+    // to avoid undefined behavior.
+    if (reinterpret_cast<Address>(block_start) <=
+            reinterpret_cast<Address>(prev_limit) &&
+        reinterpret_cast<Address>(prev_limit) <=
+            reinterpret_cast<Address>(block_limit)) {
 #ifdef ENABLE_HANDLE_ZAPPING
       internal::HandleScope::ZapRange(prev_limit, block_limit);
 #endif
